@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28207 $
+ * $Revision: 30842 $
  * $Author: rjongbloed $
- * $Date: 2012-08-16 19:09:54 -0500 (Thu, 16 Aug 2012) $
+ * $Date: 2013-11-06 01:11:39 +0000 (Wed, 06 Nov 2013) $
  */
 
 #ifndef PTLIB_ASNER_H
@@ -37,6 +37,10 @@
 
 #if P_ASN
 
+#if !PTRACING && !defined(PASN_NOPRINTON)
+  #define PASN_NOPRINTON 1
+#endif
+
 // provide options to omit vertain encodings, if needed
 #define     P_INCLUDE_PER
 #define     P_INCLUDE_BER
@@ -413,9 +417,12 @@ class PASN_BitString : public PASN_ConstrainedObject
     PASN_BitString(const PASN_BitString & other);
     PASN_BitString & operator=(const PASN_BitString & other);
 
+    void SetData(const PBYTEArray & bytes) { SetData(bytes.GetSize()*8, bytes); }
+    void SetData(const BYTE * buf, PINDEX size) { SetData(size*8, buf, size); }
     void SetData(unsigned nBits, const PBYTEArray & bytes);
     void SetData(unsigned nBits, const BYTE * buf, PINDEX size = 0);
 
+    const PBYTEArray & GetData() const { return bitData; }
     const BYTE * GetDataPointer() const { return bitData; }
 
     unsigned GetSize() const { return totalBits; }
@@ -475,6 +482,7 @@ class PASN_OctetString : public PASN_ConstrainedObject
     void SetValue(const PBYTEArray & arr) { operator=(arr); }
     void SetValue(const BYTE * data, PINDEX len);
     const PBYTEArray & GetValue() const { return value; }
+    PBYTEArray & GetWritableValue() { return value; }
     operator const PBYTEArray &() const { return value; }
     operator const BYTE *() const { return value; }
     PString AsString() const;
