@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29414 $
+ * $Revision: 33810 $
  * $Author: rjongbloed $
- * $Date: 2013-04-02 15:39:30 -0500 (Tue, 02 Apr 2013) $
+ * $Date: 2015-08-03 16:06:47 +0100 (Mon, 03 Aug 2015) $
  */
 
 
@@ -36,6 +36,39 @@
 #pragma interface
 #endif
 
+#if P_CYPHER
+
+/** RFC4013 "SASLprep" compatible string.
+    For security systems, often some identifiers, usersname, passwords etc
+    must not contain extraneous characters. This class represents a string
+    that will obey (most) of those rules.
+  */
+class PSASLString : public PString
+{
+    PCLASSINFO(PSASLString, PString);
+  public:
+    PSASLString() { }
+    PSASLString(const char * str)    { AppendValidated(str); }
+    PSASLString(const PString & str) { AppendValidated(str); }
+    PSASLString(const PSASLString & str) : PString(str) { }
+
+    PSASLString & operator=(char ch)                 { MakeEmpty(); AppendValidated(ch);  return *this; }
+    PSASLString & operator=(wchar_t ch)              { MakeEmpty(); AppendValidated(ch);  return *this; }
+    PSASLString & operator=(const char * str)        { MakeEmpty(); AppendValidated(str); return *this; }
+    PSASLString & operator=(const PString & str)     { MakeEmpty(); AppendValidated(str); return *this; }
+    PSASLString & operator=(const PSASLString & str) { PString::operator=(str);           return *this; }
+
+    PSASLString & operator+=(char ch)                 { AppendValidated(ch);     return *this; }
+    PSASLString & operator+=(wchar_t ch)              { AppendValidated(ch);     return *this; }
+    PSASLString & operator+=(const char * str)        { AppendValidated(str);    return *this; }
+    PSASLString & operator+=(const PString & str)     { AppendValidated(str);    return *this; }
+    PSASLString & operator+=(const PSASLString & str) { PString::operator=(str); return *this; }
+
+  protected:
+    void AppendValidated(const char * str);
+    void AppendValidated(wchar_t ch);
+};
+
 
 /** This class is used to encode/decode data using the MIME standard base64
    encoding mechanism as defined in RFC1521.
@@ -78,10 +111,12 @@ class PBase64 : public PObject
     PBase64();
 
     void StartEncoding(
-      bool useCRLFs = true  ///< Use CR, LF pairs in end of line characters.
+      bool useCRLFs = true,  ///< Use CR, LF pairs in end of line characters.
+      PINDEX width = 76      ///< Line widths if usCRLF true
     );
     void StartEncoding(
-      const char * endOfLine  ///< String to use for end of line.
+      const char * endOfLine,  ///< String to use for end of line.
+      PINDEX width = 76        ///< Line widths if endOfLine non empty
     );
     // Begin a base 64 encoding operation, initialising the object instance.
 
@@ -117,21 +152,25 @@ class PBase64 : public PObject
 
 
     static PString Encode(
-      const PString & str,          ///< String to be encoded to Base64
-      const char * endOfLine = "\n" ///< String to use for end of line.
+      const PString & str,            ///< String to be encoded to Base64
+      const char * endOfLine = "\n",  ///< String to use for end of line.
+      PINDEX width = 76               ///< Line widths if endOfLine non empty
     );
     static PString Encode(
-      const char * cstr,            ///< C String to be encoded to Base64
-      const char * endOfLine = "\n" ///< String to use for end of line.
+      const char * cstr,              ///< C String to be encoded to Base64
+      const char * endOfLine = "\n",  ///< String to use for end of line.
+      PINDEX width = 76               ///< Line widths if endOfLine non empty
     );
     static PString Encode(
-      const PBYTEArray & data,      ///< Data block to be encoded to Base64
-      const char * endOfLine = "\n" ///< String to use for end of line.
+      const PBYTEArray & data,        ///< Data block to be encoded to Base64
+      const char * endOfLine = "\n",  ///< String to use for end of line.
+      PINDEX width = 76               ///< Line widths if endOfLine non empty
     );
     static PString Encode(
-      const void * dataBlock,       ///< Pointer to data to be encoded to Base64
-      PINDEX length,                ///< Length of the data block.
-      const char * endOfLine = "\n" ///< String to use for end of line.
+      const void * dataBlock,         ///< Pointer to data to be encoded to Base64
+      PINDEX length,                  ///< Length of the data block.
+      const char * endOfLine = "\n",  ///< String to use for end of line.
+      PINDEX width = 76               ///< Line widths if endOfLine non empty
     );
     // Encode the data in memory to Base 64 data returnin the string.
 
@@ -204,8 +243,9 @@ class PBase64 : public PObject
     PString encodedString;
     BYTE    saveTriple[3];
     PINDEX  saveCount;
-    PINDEX  nextLine;
     PString endOfLine;
+    PINDEX  maxLineLength;
+    PINDEX  currentLineLength;
 
     bool       perfectDecode;
     PINDEX     quadPosition;
@@ -281,6 +321,9 @@ class PMessageDigest : public PObject
 class PHMAC : public PObject
 {
   public:
+    enum { KeyLength = 20 };
+    enum { BlockSize = 64 };
+
     typedef PMessageDigest::Result Result;
     virtual void Hash(const BYTE * data, PINDEX len, Result & result) = 0;
 
@@ -293,8 +336,8 @@ class PHMAC : public PObject
     virtual void Process(const PString & str, Result & result);
 
   protected:
-    virtual int GetL() const = 0;
-    virtual int GetB() const { return 64; }
+    virtual PINDEX GetL() const { return KeyLength; };
+    virtual PINDEX GetB() const { return BlockSize; }
     virtual void Initialise(const BYTE * key, PINDEX len);
     virtual void InternalProcess(const BYTE * data, PINDEX len, PHMAC::Result & result);
 
@@ -311,7 +354,6 @@ class PHMACTemplate : public PHMAC
 
     virtual void Hash(const BYTE * data, PINDEX len, Result & result)
     { hash_class::Encode(data, len, result); }
-    virtual int GetL() const { return 20; };
 };
 
 /** MD5 Message Digest.
@@ -324,6 +366,8 @@ class PMessageDigest5 : public PMessageDigest
   PCLASSINFO(PMessageDigest5, PMessageDigest)
 
   public:
+    enum { DigestLength = 16 };
+
     /// Create a new message digestor
     PMessageDigest5();
 
@@ -444,6 +488,8 @@ class PMessageDigestSHA1 : public PMessageDigest
   PCLASSINFO(PMessageDigestSHA1, PMessageDigest)
 
   public:
+    enum { DigestLength = 20 };
+
     /// Create a new message digestor
     PMessageDigestSHA1();
     ~PMessageDigestSHA1();
@@ -853,6 +899,8 @@ class PSecureConfig : public PConfig
 
 #endif // P_CONFIG_FILE
 
+#endif // P_CYPHER
+
 #endif // PTLIB_CYPHER_H
 
 
