@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28997 $
+ * $Revision: 33998 $
  * $Author: rjongbloed $
- * $Date: 2013-01-25 20:37:28 -0600 (Fri, 25 Jan 2013) $
+ * $Date: 2015-09-09 22:26:50 +0100 (Wed, 09 Sep 2015) $
  */
 
 #ifndef PTLIB_HTTP_H
@@ -179,6 +179,9 @@ class PHTTP : public PInternetProtocol
       BadResponse = 1,             ///<   1 = response is malformed
       ContentProcessorError,       ///< Content Processer returned error
       UnknownTransferEncoding,     ///< Unknown "Transfer-Encoding" extension
+      TransportConnectError,       ///< Could not connect transport
+      TransportWriteError,         ///< Could not write transport
+      TransportReadError,          ///< Could not read transport
       Continue = 100,              ///< 100 - Continue
       SwitchingProtocols,          ///< 101 - upgrade allowed
       RequestOK = 200,             ///< 200 - request has succeeded
@@ -236,6 +239,12 @@ class PHTTP : public PInternetProtocol
     static const PCaselessString & MIMEVersionTag();
     static const PCaselessString & ConnectionTag();
     static const PCaselessString & KeepAliveTag();
+    static const PCaselessString & UpgradeTag();
+    static const PCaselessString & WebSocketTag();
+    static const PCaselessString & WebSocketKeyTag();
+    static const PCaselessString & WebSocketAcceptTag();
+    static const PCaselessString & WebSocketProtocolTag();
+    static const PCaselessString & WebSocketVersionTag();
     static const PCaselessString & TransferEncodingTag();
     static const PCaselessString & ChunkedTag();
     static const PCaselessString & ProxyConnectionTag();
@@ -270,6 +279,18 @@ class PHTTP : public PInternetProtocol
 };
 
 
+class PHTTPContentProcessor
+{
+public:
+  PHTTPContentProcessor(bool reader) : m_reader(reader) { }
+  virtual ~PHTTPContentProcessor() { }
+  virtual void * GetBuffer(PINDEX & size) = 0;
+  virtual bool Process(const void * /*data*/, PINDEX /*length*/) { return true; }
+  virtual void Reset() { }
+protected:
+  bool m_reader;
+};
+
 
 class PHTTPClientAuthentication : public PObject
 {
@@ -280,7 +301,8 @@ class PHTTPClientAuthentication : public PObject
         virtual ~AuthObject() { }
         virtual PMIMEInfo & GetMIME() = 0;
         virtual PString GetURI() = 0;
-        virtual PString GetEntityBody() = 0;
+        virtual PString GetEntityBody() { return PString::Empty(); }
+        virtual PHTTPContentProcessor * GetContentProcessor() { return NULL; }
         virtual PString GetMethod() = 0;
     };
 
@@ -331,17 +353,17 @@ class PHTTPClientAuthenticator : public PHTTPClientAuthentication::AuthObject
       const PString & cmdName, 
       const PString & uri, 
       PMIMEInfo & mime, 
-      const PString & body
+      PHTTPContentProcessor & processor
     );
     virtual PMIMEInfo & GetMIME();
     virtual PString GetURI();
-    virtual PString GetEntityBody();
+    virtual PHTTPContentProcessor * GetContentProcessor();
     virtual PString GetMethod();
   protected:
     PString m_method;
     PString m_uri;
     PMIMEInfo & m_mime;
-    PString m_body;
+    PHTTPContentProcessor & m_contentProcessor;
 };
 
 ///////////////////////////////////////////////////////////////////////
@@ -413,7 +435,7 @@ class PHTTPClientDigestAuthentication : public PHTTPClientAuthentication
     bool    qopAuthInt;
     bool    stale;
     PString cnonce;
-    mutable PAtomicInteger nonceCount;
+    mutable atomic<uint32_t> nonceCount;
 };
 
 
@@ -449,9 +471,18 @@ class PHTTPClient : public PHTTP
     PHTTPClient(
       const PString & userAgentName = PString::Empty()
     );
+    ~PHTTPClient();
 
 
   // New functions for class.
+    /// Connect at transport level to remote, based on URL
+    bool ConnectURL(
+      const PURL & url
+    );
+
+    /// Call back to process the body of the HTTP command
+    typedef PHTTPContentProcessor ContentProcessor;
+
     /** Send a command and wait for the response header (including MIME fields).
        Note that a body may still be on its way even if lasResponseCode is not
        200!
@@ -467,10 +498,17 @@ class PHTTPClient : public PHTTP
       PMIMEInfo & replyMime
     );
     int ExecuteCommand(
-      const PString & cmdName,
+      Commands cmd,
       const PURL & url,
       PMIMEInfo & outMIME,
-      const PString & dataBody,
+      const PBYTEArray & dataBody,
+      PMIMEInfo & replyMime
+    );
+    int ExecuteCommand(
+      Commands cmd,
+      const PURL & url,
+      PMIMEInfo & outMIME,
+      ContentProcessor & processor,
       PMIMEInfo & replyMime
     );
 
@@ -479,13 +517,7 @@ class PHTTPClient : public PHTTP
       Commands cmd,
       const PString & url,
       PMIMEInfo & outMIME,
-      const PString & dataBody
-    );
-    bool WriteCommand(
-      const PString & cmdName,
-      const PString & url,
-      PMIMEInfo & outMIME,
-      const PString & dataBody
+      ContentProcessor & processor
     );
 
     /// Read a response from the server
@@ -493,14 +525,6 @@ class PHTTPClient : public PHTTP
       PMIMEInfo & replyMIME
     );
 
-    /// Call back to process the body of the HTTP command
-    struct ContentProcessor
-    {
-      virtual ~ContentProcessor() { }
-      virtual void * GetBuffer(PINDEX & size) = 0;
-      virtual bool Process(const void * data, PINDEX length) = 0;
-    };
-
     /// Read the body of the HTTP command
     bool ReadContentBody(
       PMIMEInfo & replyMIME,        ///< Reply MIME from server
@@ -648,10 +672,11 @@ class PHTTPClient : public PHTTP
        @return
        true if document is being transferred.
      */
-    bool PutTextDocument(
-      const PURL & url,         ///< Universal Resource Locator for document.
-      const PString & document,       ///< Body read
-      const PString & contentType = PMIMEInfo::TextPlain() ///< Content-Type header to use
+    bool PutDocument(
+      const PURL & url,           ///< Universal Resource Locator for document.
+      const PString & document,  ///< Body to write
+      const PString & contentType = PMIMEInfo::TextPlain(), ///< Content-Type header to use
+      const PMIMEInfo & mime = PMIMEInfo()   ///< Extra MIME fields to be sent
     );
 
     /** Put the document specified by the URL.
@@ -660,11 +685,28 @@ class PHTTPClient : public PHTTP
        true if document is being transferred.
      */
     bool PutDocument(
-      const PURL & url,         ///< Universal Resource Locator for document.
-      PMIMEInfo & outMIME,      ///< MIME info in request
-      PMIMEInfo & replyMIME     ///< MIME info in response
+      const PURL & url,             ///< Universal Resource Locator for document.
+      const PBYTEArray & document,  ///< Body to write
+      const PString & contentType,  ///< Content-Type header to use
+      const PMIMEInfo & mime = PMIMEInfo()   ///< Extra MIME fields to be sent
     );
 
+    /** Put the document specified by the URL.
+
+       @return
+       true if document is being transferred.
+     */
+    bool PutDocument(
+      const PURL & url,             ///< Universal Resource Locator for document.
+      const PFilePath & document,   ///< Body to write
+      const PString & contentType = PString::Empty(), ///< Content-Type header to use
+      const PMIMEInfo & mime = PMIMEInfo()   ///< Extra MIME fields to be sent
+    );
+
+    // For backward compatibility
+    bool PutTextDocument(const PURL & url, const PString & document, const PString & contentType = PMIMEInfo::TextPlain())
+    { return PutDocument(url, document, contentType); }
+
     /** Delete the document specified by the URL.
 
        @return
@@ -697,9 +739,11 @@ class PHTTPClient : public PHTTP
     /// Get persistent connection mode
     bool GetPersistent() const { return m_persist; }
 
-  protected:
-    bool AssureConnect(const PURL & url, PMIMEInfo & outMIME);
+#if PTRACING
+    static PINDEX MaxTraceContentSize;
+#endif
 
+  protected:
     PString m_userAgentName;
     bool    m_persist;
     PString m_userName;
@@ -713,6 +757,136 @@ class PHTTPClient : public PHTTP
 };
 
 
+//////////////////////////////////////////////////////////////////////////////
+// PWebSocket
+
+#if P_SSL
+
+/** This channel reads from a channel (usually socket) that is under the RFC6455
+    framing rules.
+    
+    Note the WebSocket handshake is assumed to have already occurred.
+*/
+
+class PWebSocket : public PIndirectChannel
+{
+    PCLASSINFO(PWebSocket, PIndirectChannel)
+  public:
+    /// Create a new WebSocket channel.
+    PWebSocket();
+
+  // Overrides from PChannel
+    /** Low level read from the channel.
+
+        @return
+        true indicates that at least one character was read from the channel.
+        false means no bytes were read due to timeout or some other I/O error.
+    */
+    virtual PBoolean Read(
+      void * buf,   ///< Pointer to a block of memory to receive the read bytes.
+      PINDEX len    ///< Maximum number of bytes to read into the buffer.
+    );
+
+    /** Low level write to the channel.
+        This function will always write a single WebSocket "frame".
+
+        If m_fragmenting is false, a single write will constitute a single
+        WebSocket "message" as well. If true, then this will be a fragment of
+        the message. The user should call SetFragmenting(false) before sending
+        the last part of the message.
+
+        @return
+        true if at least len bytes were written to the channel.
+    */
+    virtual PBoolean Write(
+      const void * buf, ///< Pointer to a block of memory to write.
+      PINDEX len        ///< Number of bytes to write.
+    );
+
+
+    /** Connect to the WebSocket.
+        This performs the HTTP handshake for the WebSocket establishment.
+      */
+    bool Connect(
+      const PStringArray & protocols,    ///< WebSocket sub-protocol to use.
+      PString * selectedProtocol = NULL  ///< Selected protocol by server
+    );
+
+
+    /// Read a complete WebSocket message
+    virtual bool ReadMessage(
+      PBYTEArray & msg
+    );
+
+    /// Indicate the last Read() completed the WebSocket message.
+    bool IsMessageComplete() const { return m_fragmentedRead && m_remainingPayload == 0; }
+
+    /** Indicate Write() calls are fragments of a large or indeterminate
+        message. The user should call SetFragmenting(false) before sending
+        the last part of the message.
+
+        Default is true, so every Write() call is a single message.
+      */
+    void SetWriteFragmentation(
+      bool frag
+    ) { m_fragmentingWrite = frag; }
+
+    /// Set writing binary data.
+    void SetBinaryMode(
+      bool bin = true
+    ) { m_binaryWrite = bin; }
+
+    /// Set writing text data.
+    void SetTextMode(
+      bool txt = true
+    ) { m_binaryWrite = !txt; }
+
+
+  protected:
+    enum OpCodes
+    {
+      Continuation = 0x0,
+      TextFrame = 0x1,
+      BinaryFrame = 0x2,
+      ConnectionClose = 0x8,
+      Ping = 0x9,
+      Pong = 0xA
+    };
+
+    virtual bool ReadHeader(
+      OpCodes  & opCode,
+      bool     & fragment,
+      uint64_t & payloadLength,
+      int64_t  & masking
+    );
+
+    virtual bool WriteHeader(
+      OpCodes  opCode,
+      bool     fragment,
+      uint64_t payloadLength,
+      int64_t  masking
+    );
+
+    bool WriteMasked(
+      const uint32_t * data,
+      PINDEX len,
+      uint32_t mask
+    );
+
+    bool     m_client;
+    bool     m_fragmentingWrite;
+    bool     m_binaryWrite;
+
+    uint64_t m_remainingPayload;
+    int64_t  m_currentMask;
+    bool     m_fragmentedRead;
+
+    bool     m_recursiveRead;
+};
+
+#endif // P_SSL
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PHTTPConnectionInfo
 
@@ -772,6 +946,9 @@ class PHTTPConnectionInfo : public PObject
 
     PString GetEntityBody() const   { return entityBody; }
 
+    bool IsWebSocket() const { return m_isWebSocket; }
+    void ClearWebSocket() { m_isWebSocket = false; }
+
   protected:
     PBoolean Initialise(PHTTPServer & server, PString & args);
     bool DecodeMultipartFormInfo() { return mimeInfo.DecodeMultiPartList(m_multipartFormInfo, entityBody); }
@@ -783,6 +960,7 @@ class PHTTPConnectionInfo : public PObject
     bool            isPersistent;
     bool            wasPersistent;
     bool            isProxyConnection;
+    bool            m_isWebSocket;
     int             majorVersion;
     int             minorVersion;
     PString         entityBody;        // original entity body (POST only)
@@ -880,14 +1058,10 @@ class PHTTPServer : public PHTTP
        If there is no ContentLength field in the response, this value must
        be false for correct operation.
      */
-    virtual PBoolean OnGET(
-      const PURL & url,                    ///< Universal Resource Locator for document.
-      const PMIMEInfo & info,              ///< Extra MIME information in command.
+    virtual bool OnGET(
       const PHTTPConnectionInfo & conInfo  ///< HTTP connection information
     );
 
-
-
     /** Handle a HEAD command from a client.
 
        The default implemetation looks up the URL in the name space declared by
@@ -899,9 +1073,7 @@ class PHTTPServer : public PHTTP
        If there is no ContentLength field in the response, this value must
        be false for correct operation.
      */
-    virtual PBoolean OnHEAD(
-      const PURL & url,                   ///< Universal Resource Locator for document.
-      const PMIMEInfo & info,             ///< Extra MIME information in command.
+    virtual bool OnHEAD(
       const PHTTPConnectionInfo & conInfo ///< HTTP connection information
     );
 
@@ -916,10 +1088,7 @@ class PHTTPServer : public PHTTP
        If there is no ContentLength field in the response, this value must
        be false for correct operation.
      */
-    virtual PBoolean OnPOST(
-      const PURL & url,                   ///< Universal Resource Locator for document.
-      const PMIMEInfo & info,             ///< Extra MIME information in command.
-      const PStringToString & data,       ///< Variables provided in the POST data.
+    virtual bool OnPOST(
       const PHTTPConnectionInfo & conInfo ///< HTTP connection information
     );
 
@@ -1020,6 +1189,44 @@ class PHTTPServer : public PHTTP
       PHTTPConnectionInfo & connectInfo
     );
 
+    /**Called when a request indicates a swtch to WebSocket protocol.
+       This will complete the WebSocket handshake if m_webSocketNotifiers
+       contains and entry for an offered protocol. If none, a 404 is
+       returned and the socket remains in HTTP mode.
+
+       After the start up handshake has completed, if there is a notifier and
+       it is non-null, then the notifier is called. The notifier, may then call
+       connectInfo.ClearWebSocket() if it wishes to return to HTTP mode. That
+       is, ProcessCommand() will return true and the normal persistence rules
+       apply. If ClearWebSocket() is not called then ProcessCommand() will
+       return false.
+
+       Note, that the latter case does not mean that the underlying socket is
+       necessarily closed, it could subsequently be disconnected from the
+       PHTTPServer objects and passed to PWebSocket for processing.
+
+       @return true if want to persist with HTTP connection.
+      */
+    virtual bool OnWebSocket(
+      PHTTPConnectionInfo & connectInfo
+    );
+
+    typedef PNotifierTemplate<PHTTPConnectionInfo> WebSocketNotifier;
+
+    /** Set the handler for WebSocket sub-protocol.
+        A NULL notifier may be set which indiates that the protocol is
+        supported, but that ProcessCommand() is to simply return false as
+        the caller will handle the channel from then on.
+    */
+    void SetWebSocketNotifier(
+      const PString & protocol,
+      const WebSocketNotifier & notifier
+    );
+
+    /// Remove a WebSocket notifier
+    void ClearWebSocketNotifier(
+      const PString & protocol
+    );
 
   protected:
     void Construct();
@@ -1028,6 +1235,12 @@ class PHTTPServer : public PHTTP
     PHTTPConnectionInfo connectInfo;
     unsigned            transactionCount;
     PTimeInterval       nextTimeout;
+
+    std::map<PString, WebSocketNotifier> m_webSocketNotifiers;
+
+    P_REMOVE_VIRTUAL(PBoolean,OnGET(const PURL&,const PMIMEInfo&, const PHTTPConnectionInfo&),false);
+    P_REMOVE_VIRTUAL(PBoolean,OnHEAD(const PURL&,const PMIMEInfo&,const PHTTPConnectionInfo&),false);
+    P_REMOVE_VIRTUAL(PBoolean,OnPOST(const PURL&,const PMIMEInfo&,const PStringToString&,const PHTTPConnectionInfo&),false);
 };
 
 
@@ -1182,6 +1395,14 @@ class PHTTPSimpleAuth : public PHTTPAuthority
      */
     virtual PBoolean IsActive() const;
 
+    /** Get the realm or name space for the user authorisation name and
+       password as required by the basic authorisation system of HTTP/1.0.
+
+       @return
+       String for the authorisation realm name.
+     */
+    const PString & GetLocalRealm() const { return realm; }
+
     /** Get the user name allocated to this simple authorisation.
 
        @return
@@ -1366,9 +1587,29 @@ class PHTTPResource : public PObject
      */
     DWORD GetHitCount() const { return hitCount; }
 
+    /// Clear the hit count for the resource.
     void ClearHitCount() { hitCount = 0; }
-    // Clear the hit count for the resource.
 
+    /**Called when a request indicates a swtch to WebSocket protocol.
+       This will handle a WebScoket protocol change.
+
+       After the start up handshake has completed, if there is a notifier and
+       it is non-null, then the notifier is called. The notifier, may then call
+       connectInfo.ClearWebSocket() if it wishes to return to HTTP mode. That
+       is, ProcessCommand() will return true and the normal persistence rules
+       apply. If ClearWebSocket() is not called then ProcessCommand() will
+       return false.
+
+       Note, that the latter case does not mean that the underlying socket is
+       necessarily closed, it could subsequently be disconnected from the
+       PHTTPServer objects and passed to PWebSocket for processing.
+
+       @return true if want to persist with HTTP connection.
+      */
+    virtual bool OnWebSocket(
+      PHTTPServer & server,       ///< HTTP server that received the request
+      PHTTPConnectionInfo & connectInfo   ///< HTTP connection information
+    );
 
     /** Handle the GET command passed from the HTTP socket.
 
@@ -1381,6 +1622,10 @@ class PHTTPResource : public PObject
        If there is no ContentLength field in the response, this value must
        be false for correct operation.
      */
+    virtual bool OnGET(
+      PHTTPServer & server,       ///< HTTP server that received the request
+      const PHTTPConnectionInfo & conInfo  ///< HTTP connection information
+    );
     virtual PBoolean OnGET(
       PHTTPServer & server,       ///< HTTP server that received the request
       const PURL & url,           ///< Universal Resource Locator for document.
@@ -1415,6 +1660,10 @@ class PHTTPResource : public PObject
        If there is no ContentLength field in the response, this value must
        be false for correct operation.
      */
+    virtual bool OnHEAD(
+      PHTTPServer & server,       ///< HTTP server that received the request
+      const PHTTPConnectionInfo & conInfo ///< HTTP connection information
+    );
     virtual PBoolean OnHEAD(
       PHTTPServer & server,       ///< HTTP server that received the request
       const PURL & url,           ///< Universal Resource Locator for document.
@@ -1433,6 +1682,10 @@ class PHTTPResource : public PObject
        If there is no ContentLength field in the response, this value must
        be false for correct operation.
      */
+    virtual bool OnPOST(
+      PHTTPServer & server,       ///< HTTP server that received the request
+      const PHTTPConnectionInfo & conInfo ///< HTTP connection information
+    );
     virtual PBoolean OnPOST(
       PHTTPServer & server,         ///< HTTP server that received the request
       const PURL & url,             ///< Universal Resource Locator for document.
@@ -1497,7 +1750,7 @@ class PHTTPResource : public PObject
      */
     virtual PBoolean LoadHeaders(
       PHTTPRequest & request    ///<  Information on this request.
-    ) = 0;
+    );
 
     /**Send the data associated with a command.
 
