@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27824 $
+ * $Revision: 31996 $
  * $Author: rjongbloed $
- * $Date: 2012-06-13 17:58:20 -0500 (Wed, 13 Jun 2012) $
+ * $Date: 2014-05-28 09:32:58 +0100 (Wed, 28 May 2014) $
  */
 
 #ifndef PTLIB_HTTPFORM_H
@@ -173,8 +173,9 @@ class PHTTPField : public PObject
 
     /** Get the value of the PConfig to the sub-field. If the field is not
        composite then it always sets the value as for the non-indexed version.
+       @return true if no configuration set and a default should be used.
      */
-    virtual void LoadFromConfig(
+    virtual bool LoadFromConfig(
       PConfig & cfg   // Configuration for value transfer.
     );
 
@@ -275,7 +276,7 @@ class PHTTPCompositeField : public PHTTPField
       const PString & newValue   // New value for the field.
     );
 
-    virtual void LoadFromConfig(
+    virtual bool LoadFromConfig(
       PConfig & cfg   // Configuration for value transfer.
     );
     virtual void SaveToConfig(
@@ -357,7 +358,7 @@ class PHTTPFieldArray : public PHTTPCompositeField
       PHTML & html    // HTML to receive the field info.
     ) const;
 
-    virtual void LoadFromConfig(
+    virtual bool LoadFromConfig(
       PConfig & cfg   // Configuration for value transfer.
     );
     virtual void SaveToConfig(
@@ -373,7 +374,8 @@ class PHTTPFieldArray : public PHTTPCompositeField
     void SetSize(PINDEX newSize);
 
     PStringArray GetStrings(
-      PConfig & cfg   ///< Config file to get strings from
+      PConfig & cfg,   ///< Config file to get strings from
+      const PStringArray & defaultValues
     );
 
     void SetStrings(
@@ -528,7 +530,7 @@ class PHTTPIntegerField : public PHTTPField
       const PString & newVal
     );
 
-    virtual void LoadFromConfig(
+    virtual bool LoadFromConfig(
       PConfig & cfg   ///< Configuration for value transfer.
     );
     virtual void SaveToConfig(
@@ -580,7 +582,7 @@ class PHTTPBooleanField : public PHTTPField
       const PString & newVal
     );
 
-    virtual void LoadFromConfig(
+    virtual bool LoadFromConfig(
       PConfig & cfg   ///< Configuration for value transfer.
     );
     virtual void SaveToConfig(
@@ -733,6 +735,8 @@ class PHTTPSelectField : public PHTTPField
     );
 
   protected:
+    void Construct(PINDEX initVal, bool enumeration);
+
     PStringArray m_values;
     bool         m_enumeration;
     PINDEX       m_initialValue;
@@ -885,7 +889,7 @@ class PHTTPConfig : public PHTTPForm
 
     /** Load all of the values for the resource from the configuration.
      */
-    void LoadFromConfig();
+    bool LoadFromConfig();
 
     /** Get the configuration file section that the page will alter.
 
@@ -917,6 +921,73 @@ class PHTTPConfig : public PHTTPForm
       PHTTPField * valFld   ///< Field for the value of the key yto be added.
     );
 
+    /**Add a boolean field.
+       Returns current value from PConfig, using defaultValue for first time.
+      */
+    bool AddBooleanField(
+      const char * name,
+      bool defaultValue = false,
+      const char * help = NULL
+    );
+
+    /**Add an integer field.
+       Returns current value from PConfig, using defaultValue for first time.
+      */
+    int AddIntegerField(
+      const char * name,
+      int low, int high,
+      int defaultValue = 0,
+      const char * units = NULL,
+      const char * help = NULL
+    );
+
+    /**Add a string field.
+       Returns current value from PConfig, using defaultValue for first time.
+      */
+    PString AddStringField(
+      const char * name,
+      PINDEX maxLength,
+      const char * defaultValue = NULL,
+      const char * help = NULL,
+      int rows = 0, // 0=auto, 1=InputText, >1=TextArea
+      int columns = 0
+    );
+
+    /**Add a string array field.
+       Returns current value from PConfig, using defaultValue for first time.
+      */
+    PStringArray AddStringArrayField(
+      const char * name,
+      bool sorted,
+      PINDEX maxLength,
+      const PStringArray & defaultValues,
+      const char * help = NULL,
+      int rows = 0, // 0=auto, 1=InputText, >1=TextArea
+      int columns = 0
+    );
+
+    /**Add a string field.
+       Returns current value from PConfig, using defaultValue for first time.
+      */
+    PString AddSelectField(
+      const char * name,
+      const PStringArray & valueArray,
+      const char * defaultValue = NULL,
+      const char * help = NULL,
+      bool enumeration = false
+    );
+
+    /**Add a string array field.
+       Returns current value from PConfig, using defaultValue for first time.
+      */
+    PStringArray AddSelectArrayField(
+      const char * name,
+      bool sorted,
+      const PStringArray & defaultValues,
+      const PStringArray & possibleValues,
+      const char * help = NULL,
+      bool enumeration = false
+    );
 
   protected:
     PString section;
