@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 28501 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-10-05 15:07:55 +0100 (Fri, 05 Oct 2012) $
  */
 
 #ifndef PTLIB_MODEM_H
@@ -39,6 +39,7 @@
 
 #include <ptlib/serchan.h>
 
+#if P_SERIAL
 
 /** A class representing a modem attached to a serial port. This adds the usual
    modem operations to the basic serial port.
@@ -48,14 +49,14 @@
    operations are the exchange of strings in "chat" script.
    
    The following defaults are used for command strings:
-       initialise         <CODE>ATZ\r\w2sOK\w100m</CODE>
-       deinitialise       <CODE>\d2s+++\d2sATH0\r</CODE>
+       initialise         <CODE>ATZ\\r\\w2sOK\\w100m</CODE>
+       deinitialise       <CODE>\\d2s+++\\d2sATH0\\r</CODE>
        pre-dial           <CODE>ATDT</CODE>
-       post-dial          <CODE>\r</CODE>
+       post-dial          <CODE>\\r</CODE>
        busy reply         <CODE>BUSY</CODE>
        no carrier reply   <CODE>NO CARRIER</CODE>
        connect reply      <CODE>CONNECT</CODE>
-       hang up            <CODE>\d2s+++\d2sATH0\r</CODE>
+       hang up            <CODE>\\d2s+++\\d2sATH0\\r</CODE>
 
  */
 class PModem : public PSerialChannel
@@ -103,7 +104,7 @@ class PModem : public PSerialChannel
        parameters.
        
        @return
-       PTrue if the modem serial port was successfully opened.
+       true if the modem serial port was successfully opened.
      */
     virtual PBoolean Open(
       const PString & port,   ///< Serial port name to open.
@@ -121,7 +122,7 @@ class PModem : public PSerialChannel
        correct configuration file section is already set.
 
        @return
-       PTrue if the modem serial port was successfully opened.
+       true if the modem serial port was successfully opened.
      */
     virtual PBoolean Open(
       PConfig & cfg   ///< Configuration file to read parameters from.
@@ -140,7 +141,7 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is an implied <CODE>\s</CODE> before the string.
+       Note there is an implied <CODE>\\s</CODE> before the string.
      */
     void SetInitString(
       const PString & str   ///< New initialisation command string.
@@ -159,7 +160,7 @@ class PModem : public PSerialChannel
     /** The modem is in a state that allows the initialise to start.
     
        @return
-       PTrue if the <A>Initialise()</A> function may proceeed.
+       true if the <A>Initialise()</A> function may proceeed.
      */
     PBoolean CanInitialise() const;
 
@@ -169,7 +170,7 @@ class PModem : public PSerialChannel
        sent with all replies met.
 
        @return
-       PTrue if command string sent successfully and the objects state has
+       true if command string sent successfully and the objects state has
        changed.
      */
     PBoolean Initialise();
@@ -179,7 +180,7 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is an implied <CODE>\s</CODE> before the string.
+       Note there is an implied <CODE>\\s</CODE> before the string.
      */
     void SetDeinitString(
       const PString & str   ///< New de-initialisation command string.
@@ -198,7 +199,7 @@ class PModem : public PSerialChannel
     /** The modem is in a state that allows the de-initialise to start.
     
        @return
-       PTrue if the <A>Deinitialise()</A> function may proceeed.
+       true if the <A>Deinitialise()</A> function may proceeed.
      */
     PBoolean CanDeinitialise() const;
 
@@ -208,7 +209,7 @@ class PModem : public PSerialChannel
        sent with all replies met.
 
        @return
-       PTrue if command string sent successfully and the objects state has
+       true if command string sent successfully and the objects state has
        changed.
      */
     PBoolean Deinitialise();
@@ -218,7 +219,7 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is an implied <CODE>\s</CODE> before the string.
+       Note there is an implied <CODE>\\s</CODE> before the string.
      */
     void SetPreDialString(
       const PString & str   ///< New pre-dial command string.
@@ -239,7 +240,7 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is <EM>not</EM> an implied <CODE>\s</CODE> before the
+       Note there is <EM>not</EM> an implied <CODE>\\s</CODE> before the
        string, unlike the pre-dial string.
      */
     void SetPostDialString(
@@ -261,8 +262,8 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is an implied <CODE>\w120s</CODE> before the string. Also
-       the <CODE>\s</CODE> and <CODE>\d</CODE> commands do not operate and
+       Note there is an implied <CODE>\\w120s</CODE> before the string. Also
+       the <CODE>\\s</CODE> and <CODE>\\d</CODE> commands do not operate and
        will simply terminate the string match.
      */
     void SetBusyString(
@@ -284,8 +285,8 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is an implied <CODE>\w120s</CODE> before the string. Also
-       the <CODE>\s</CODE> and <CODE>\d</CODE> commands do not operate and
+       Note there is an implied <CODE>\\w120s</CODE> before the string. Also
+       the <CODE>\\s</CODE> and <CODE>\\d</CODE> commands do not operate and
        will simply terminate the string match.
      */
     void SetNoCarrierString(
@@ -307,8 +308,8 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is an implied <CODE>\w120s</CODE> before the string. Also
-       the <CODE>\s</CODE> and <CODE>\d</CODE> commands do not operate and
+       Note there is an implied <CODE>\\w120s</CODE> before the string. Also
+       the <CODE>\\s</CODE> and <CODE>\\d</CODE> commands do not operate and
        will simply terminate the string match.
      */
     void SetConnectString(
@@ -328,7 +329,7 @@ class PModem : public PSerialChannel
     /** The modem is in a state that allows the dial to start.
     
        @return
-       PTrue if the <A>Dial()</A> function may proceeed.
+       true if the <A>Dial()</A> function may proceeed.
      */
     PBoolean CanDial() const;
 
@@ -338,11 +339,11 @@ class PModem : public PSerialChannel
        sent with all replies met.
 
        The string sent to the modem is the concatenation of the pre-dial
-       string, a <CODE>\s</CODE>, the <CODE>number</CODE> parameter and the
+       string, a <CODE>\\s</CODE>, the <CODE>number</CODE> parameter and the
        post-dial string.
 
        @return
-       PTrue if command string sent successfully and the objects state has
+       true if command string sent successfully and the objects state has
        changed.
      */
     PBoolean Dial(const PString & number);
@@ -352,7 +353,7 @@ class PModem : public PSerialChannel
        See the <A>PChannel::SendCommandString()</A> function for more
        information on the format of the command string.
 
-       Note there is an implied <CODE>\s</CODE> before the string.
+       Note there is an implied <CODE>\\s</CODE> before the string.
      */
     void SetHangUpString(
       const PString & str   ///< New hang up command string.
@@ -371,7 +372,7 @@ class PModem : public PSerialChannel
     /** The modem is in a state that allows the hang up to start.
     
        @return
-       PTrue if the <A>HangUp()</A> function may proceeed.
+       true if the <A>HangUp()</A> function may proceeed.
      */
     PBoolean CanHangUp() const;
 
@@ -381,7 +382,7 @@ class PModem : public PSerialChannel
        sent with all replies met.
 
        @return
-       PTrue if command string sent successfully and the objects state has
+       true if command string sent successfully and the objects state has
        changed.
      */
     PBoolean HangUp();
@@ -389,7 +390,7 @@ class PModem : public PSerialChannel
     /** The modem is in a state that allows the user command to start.
     
        @return
-       PTrue if the <A>SendUser()</A> function may proceeed.
+       true if the <A>SendUser()</A> function may proceeed.
      */
     PBoolean CanSendUser() const;
 
@@ -399,7 +400,7 @@ class PModem : public PSerialChannel
        sent with all replies met.
 
        @return
-       PTrue if command string sent successfully.
+       true if command string sent successfully.
      */
     PBoolean SendUser(
       const PString & str   ///< User command string to send.
@@ -409,12 +410,12 @@ class PModem : public PSerialChannel
     // Abort the current meta-string command operation eg dial, hang up etc.
 
     /** The modem is in a state that allows the user application to read from
-       the channel. Reading while this is PTrue can interfere with the operation
+       the channel. Reading while this is true can interfere with the operation
        of the meta-string processing. This function is only usefull when
        multi-threading is used.
 
        @return
-       PTrue if <A>Read()</A> operations are "safe".
+       true if <A>Read()</A> operations are "safe".
      */
     PBoolean CanRead() const;
 
@@ -458,6 +459,8 @@ class PModem : public PSerialChannel
 };
 
 
+#endif // P_SERIAL
+
 #endif // PTLIB_MODEM_H
 
 
