@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22671 $
+ * $Revision: 30765 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 22:37:23 -0500 (Tue, 19 May 2009) $
+ * $Date: 2013-10-21 02:26:29 +0100 (Mon, 21 Oct 2013) $
  */
 
 #ifndef PTLIB_PFFVDEV_H
@@ -69,7 +69,7 @@ class PVideoInputDevice_FFMPEG : public PVideoInputDevice
       */
     PBoolean Open(
       const PString & deviceName,   /// Device name to open
-      PBoolean startImmediate = PTrue    /// Immediately start device
+      PBoolean startImmediate = true    /// Immediately start device
     );
 
     /**Determine of the device is currently open.
@@ -141,18 +141,6 @@ class PVideoInputDevice_FFMPEG : public PVideoInputDevice
       VideoFormat videoFormat   /// New video format
     );
 
-    /**Get the number of video channels available on the device.
-
-       Default behaviour returns 1.
-    */
-    virtual int GetNumChannels() ;
-
-    /**Set the video channel to be used on the device. 
-    */
-    virtual PBoolean SetChannel(
-         int channelNumber  /// New channel number for device.
-    );
-    
     /**Set the colour format to be used.
 
        Default behaviour sets the value of the colourFormat variable and then
@@ -174,7 +162,7 @@ class PVideoInputDevice_FFMPEG : public PVideoInputDevice
     /**Get the minimum & maximum size of a frame on the device.
 
        Default behaviour returns the value 1 to UINT_MAX for both and returns
-       PFalse.
+       false.
     */
     virtual PBoolean GetFrameSizeLimits(
       unsigned & minWidth,   /// Variable to receive minimum width
@@ -195,10 +183,6 @@ class PVideoInputDevice_FFMPEG : public PVideoInputDevice
 
     void ClearMapping() { return ; }
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats()
-      { return PTrue; }
 
  protected:
    unsigned       m_ffmpegFrameWidth;
