@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25539 $
- * $Author: willamowius $
- * $Date: 2011-04-08 07:26:56 -0500 (Fri, 08 Apr 2011) $
+ * $Revision: 30779 $
+ * $Author: rjongbloed $
+ * $Date: 2013-10-29 07:10:28 +0000 (Tue, 29 Oct 2013) $
  */
 
 #ifndef PTLIB_PLDAP_H
@@ -515,8 +515,8 @@ class PLDAPSchema : public PObject
     typedef std::list<Attribute> attributeList;
 
     static PLDAPSchema * CreateSchema(const PString & schemaname, PPluginManager * pluginMgr = NULL);
-    static PStringList GetSchemaNames(PPluginManager * pluginMgr = NULL);
-    static PStringList GetSchemaFriendlyNames(const PString & schema, PPluginManager * pluginMgr = NULL);
+    static PStringArray GetSchemaNames(PPluginManager * pluginMgr = NULL);
+    static PStringArray GetSchemaFriendlyNames(PPluginManager * pluginMgr = NULL);
 
     void OnReceivedAttribute(const PString & attribute, const PString & value);
 
@@ -552,16 +552,10 @@ class PLDAPSchema : public PObject
 };
 
 
-template <class className> class LDAPPluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject *    CreateInstance(int /*userData*/) const { return new className; }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const { return className::SchemaName(); } 
-};
+PCREATE_PLUGIN_SERVICE(PLDAPSchema);
+
+#define LDAP_Schema(name) PCREATE_PLUGIN(name, PLDAPSchema)
 
-#define LDAP_Schema(name)    \
-  static LDAPPluginServiceDescriptor<name##_schema> name##_schema_descriptor; \
-  PCREATE_PLUGIN(name##_schema, PLDAPSchema, &name##_schema_descriptor)
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
