@@ -26,15 +26,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29076 $
+ * $Revision: 34826 $
  * $Author: rjongbloed $
- * $Date: 2013-02-12 02:30:20 -0600 (Tue, 12 Feb 2013) $
+ * $Date: 2016-05-12 09:50:51 +0100 (Thu, 12 May 2016) $
  */
 
 #ifndef PTLIB_PNAT_H
 #define PTLIB_PNAT_H
 
-#include <ptbuildopts.h>
+#include <ptlib.h>
 
 #if P_NAT
 
@@ -58,17 +58,16 @@ class PNatMethod  : public PObject
     PCLASSINFO(PNatMethod,PObject);
 
   public:
-    enum {
-      NatPortStart = 49152,
-      NatPortEnd   = 65535
-    };
-
     enum Component {
       eComponent_RTP  = 1,
       eComponent_RTCP = 2,
       eComponent_Unknown = 255,
     };
 
+    /** NAT types.
+        Note the values in this enum are determined by various protocols and
+        should not be changed.
+     */
     P_DECLARE_STREAMABLE_ENUM(NatTypes,
       UnknownNat,         ///< NAT was not determined
       OpenNat,            ///< No NAT was detected
@@ -119,24 +118,26 @@ class PNatMethod  : public PObject
                                Only an external host that receives a packet
                                from an internal host can send a packet back.
                             */
-      PartiallyBlocked,   /**< Partially blocked.
+      BlockedNat,         /**< Completely blocked.
+                               Packets cannot pass through the router on one
+                               or the other direction.
+                            */
+      PartiallyBlocked    /**< Partially blocked.
                                A pathological condition where some packets get
                                through and some do not causing confusion to the
                                STUN protocol in particular. Usually indicates a
                                badly configured firewall rules.
                             */
-      BlockedNat          /**< Completely blocked.
-                               Packets cannot pass through the router on one
-                               or the other direction.
-                            */
     );
 
   /**@name Construction */
   //@{
     /** Default Contructor
     */
-    PNatMethod();
+  protected:
+    PNatMethod(unsigned priority);
 
+  public:
     /** Deconstructor
     */
     ~PNatMethod();
@@ -148,6 +149,9 @@ class PNatMethod  : public PObject
     virtual void PrintOn(
       ostream & strm
     ) const;
+    virtual Comparison Compare(
+      const PObject & obj
+    ) const;
   //@}
 
 
@@ -168,7 +172,24 @@ class PNatMethod  : public PObject
 
     /** Get the NAT traversal method Name
     */
-    virtual PString GetName() const = 0;
+    virtual PCaselessString GetMethodName() const = 0;
+
+    /** Get the NAT traversal method Name
+    */
+    virtual PString GetFriendlyName() const;
+
+    /**Activate/DeActivate the NAT Method on a call by call basis
+     Default does nothing
+      */
+    virtual void Activate(bool active);
+
+    /**Returns whether the Nat Method is ready and available in
+       assisting in NAT Traversal on the specified interface.
+    */
+    virtual bool IsAvailable(
+      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
+      PObject * context = NULL
+    );
 
     /**Get the current server address name.
       */
@@ -184,13 +205,15 @@ class PNatMethod  : public PObject
       PIPSocketAddressAndPort & externalAddressAndPort 
     ) const;
 
-    __inline NatTypes GetNatType(
+    NatTypes GetNatType(
       const PTimeInterval & maxAge
-    ) { return InternalGetNatType(false, maxAge); }
+    );
 
     __inline NatTypes GetNatType(
       bool force = false    ///< Force a new check
-    ) { return InternalGetNatType(force, PMaxTimeInterval); }
+    ) { return GetNatType(force ? PTimeInterval(0) : PMaxTimeInterval); }
+
+    __inline NatTypes GetNatType() const { return m_natType; }
 
     /**Determine via the STUN protocol the NAT type for the router.
        As for GetNatType() but returns an English string for the type.
@@ -199,12 +222,14 @@ class PNatMethod  : public PObject
       bool force = false    ///< Force a new check
     ) { return GetNatTypeString(GetNatType(force)); }
 
+    PString GetNatTypeName() const { return GetNatTypeString(GetNatType()); }
+
     /**Set the current server address name.
        Defaults to be "address:port" string form.
       */
     virtual bool SetServer(const PString & server);
 
-    /**Set the authentication credientials.
+    /**Set the authentication credentials.
       */
     virtual void SetCredentials(
       const PString & username, 
@@ -212,12 +237,17 @@ class PNatMethod  : public PObject
       const PString & realm
     );
 
+    static const PTimeInterval & GetDefaultMaxAge();
+
     /** Get the acquired External IP Address.
     */
-    virtual bool GetExternalAddress(
+    bool GetExternalAddress(
       PIPSocket::Address & externalAddress, ///< External address of router
-      const PTimeInterval & maxAge = 1000   ///< Maximum age for caching
+      const PTimeInterval & maxAge = GetDefaultMaxAge() ///< Maximum age for caching
     );
+    bool GetExternalAddress(
+      PIPSocket::Address & externalAddress  ///< External address of router
+    ) const;
 
     /**Return the interface NAT router is using.
       */
@@ -251,13 +281,9 @@ class PNatMethod  : public PObject
     virtual PBoolean CreateSocket(
       PUDPSocket * & socket,
       const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
-      WORD localPort = 0
-    ) { return CreateSocket(eComponent_Unknown, socket, binding, localPort); }
-    virtual bool CreateSocket(
-      Component component,
-      PUDPSocket * & socket,
-      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
-      WORD localPort = 0
+      WORD localPort = 0,
+      PObject * context = NULL,
+      Component component = eComponent_Unknown
     );
 
     /**Create a socket pair.
@@ -274,75 +300,10 @@ class PNatMethod  : public PObject
        false.
       */
     virtual bool CreateSocketPair(
-      PUDPSocket * & socket1,
-      PUDPSocket * & socket2,
-      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny()
-    );
-
-    /**Create a socket pair.
-       The NAT traversal protocol is used to create a pair of sockets with
-       adjacent port numbers for which the external IP address and port
-       numbers are known. PUDPSocket descendants are returned which will, in
-       response to GetLocalAddress() return the externally visible IP and port
-       rather than the local machines IP and socket.
-
-       The will create new socket pointers. It is up to the caller to make
-       sure the sockets are deleted to avoid memory leaks.
-
-       The socket pointers are set to NULL if the function fails and returns
-       false.
-      */
-    virtual bool CreateSocketPair(
-      PUDPSocket * & socket1,
-      PUDPSocket * & socket2,
-      const PIPSocket::Address & binding,
-      void * userData
-    );
-
-    /**Create a socket pair asynchronously.
-       This is similar to CreateSocketPair but returns immediately, initiating
-       the creating of a socket pair in the background. Use GetSocketPairAsync
-       at some later time to get the resultant sockets.
-
-       Default behaviour does nothing.
-      */
-    virtual bool CreateSocketPairAsync(
-      const PString & token
-    );
-
-    /**Get a socket pair asynchronously created.
-       This function will get the socket pair initated by the
-       CreateSocketPairAsync function. If the sockets are not ready yet it
-       will block until they are.
-
-       Default behaviour calls CreateSocketPair synchronously
-      */
-    virtual bool GetSocketPairAsync(
-      const PString & token,
       PUDPSocket * & socket1,
       PUDPSocket * & socket2,
       const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
-      void * userData = NULL
-    );
-
-    /**Returns whether the Nat Method is ready and available in
-    assisting in NAT Traversal on the specified interface.
-    */
-    virtual bool IsAvailable(
-      const PIPSocket::Address & binding
-    );
-
-    /**Activate/DeActivate the NAT Method on a call by call basis
-     Default does nothing
-      */
-    virtual void Activate(bool active);
-
-    /**Set Alternate Candidate (ICE) or probe (H.460.24A) addresses.
-       Default does nothing.
-      */
-    virtual void SetAlternateAddresses(
-      const PStringArray & addresses,   ///< List of probe/candidates
-      void * userData = NULL            ///< User data to link to NAT handler.
+      PObject * context = NULL
     );
 
     P_DECLARE_ENUM(RTPSupportTypes,
@@ -377,96 +338,94 @@ class PNatMethod  : public PObject
     );
   //@}
 
-    struct PortInfo {
-      PortInfo(WORD port = 0)
-        : basePort(port)
-        , maxPort(port)
-        , currentPort(port)
-      {
-      }
-
-      void SetPorts(
-        WORD start,
-        WORD end
-      );
-
-      WORD GetNext(
-        unsigned increment
-      );
-
-      /** RandomPortPair
-	      This function returns a random port pair base number in the specified range for
-	      the creation of the RTP port pairs (this used to avoid issues with multiple
-	      NATed devices opening the same local ports and experiencing issues with
-	      the behaviour of the NAT device Refer: draft-jennings-behave-test-results-04 sect 3
-      */
-      WORD GetRandomPair();
-
-      PMutex mutex;
-      WORD   basePort;
-      WORD   maxPort;
-      WORD   currentPort;
-    } ;
+    bool     IsActive()    const { return m_active; }
+    unsigned GetPriority() const { return m_priority; }
 
   protected:
-    virtual NatTypes InternalGetNatType(bool forced, const PTimeInterval & maxAge) = 0;
+    virtual PNATUDPSocket * InternalCreateSocket(Component component, PObject * context) = 0;
+    virtual void InternalUpdate() = 0;
 
-    PortInfo singlePortInfo, pairedPortInfo;
+    bool m_active;
+
+    PIPSocket::PortRange m_singlePortRange;
+    PIPSocket::PortRange m_pairedPortRange;
+
+    PNatMethod::NatTypes    m_natType;
+    PIPSocketAddressAndPort m_externalAddress;
+    PTime                   m_updateTime;
+    PMutex                  m_mutex;
+
+  private:
+    unsigned m_priority;
+
+  P_REMOVE_VIRTUAL(NatTypes,InternalGetNatType(bool,const PTimeInterval &),UnknownNat);
+
+  friend class PNatMethods;
 };
 
+
 /////////////////////////////////////////////////////////////
 
-PLIST(PNatList, PNatMethod);
-
-////////////////////////////////////////////////////////////////////////////////
-
 class PNatCandidate : public PObject
 {
   PCLASSINFO(PNatCandidate, PObject);
   public:
-    enum {
-      eType_Unknown,
-      eType_Host,
-      eType_ServerReflexive,
-      eType_PeerReflexive,
-      eType_Relay
-    };
+    P_DECLARE_ENUM(Types,
+      HostType,
+      ServerReflexiveType,
+      PeerReflexiveType,
+      RelayType
+    );
 
-    PNatCandidate();
-    PNatCandidate(int type, PNatMethod::Component component);
+    PNatCandidate(
+      Types type = EndTypes,
+      PNatMethod::Component component = PNatMethod::eComponent_Unknown,
+      const char * foundation = NULL
+    );
 
-    virtual PString AsString() const;
+    virtual Comparison Compare(const PObject & other) const;
+    virtual void PrintOn(ostream & strm) const;
 
-    PIPSocketAddressAndPort m_baseAddress;
-    PIPSocketAddressAndPort m_transport;
-    int                     m_type;
+    void CalculatePriority();
+
+    Types                   m_type;
     PNatMethod::Component   m_component;
+    unsigned                m_priority;
+    PString                 m_foundation;             // ICE support
+    PCaselessString         m_protocol;               // Almost invariably "udp"
+    PIPSocketAddressAndPort m_baseTransportAddress;   // Address of physical host
+    PIPSocketAddressAndPort m_localTransportAddress;  // Address presented to remote system
 };
 
 
+typedef PList<PNatCandidate> PNatCandidateList;
+
+
+//////////////////////////////////////////////////////////////////////////
+
 /**UDP socket that has been created by a NAT method.
   */
-
 class PNATUDPSocket : public PUDPSocket
 {
   PCLASSINFO(PNATUDPSocket, PUDPSocket);
 
   public:
     PNATUDPSocket(
-      PNatMethod::Component component = PNatMethod::eComponent_Unknown
+      PNatMethod::Component component
     );
 
-    virtual PNatCandidate GetCandidateInfo();
+    virtual PString GetName() const;
+    virtual const char * GetNatName() const { return "NAT"; }
 
-    virtual PString GetBaseAddress();
-    virtual bool GetBaseAddress(PIPSocketAddressAndPort & addrAndPort);
+    virtual void GetCandidateInfo(PNatCandidate & candidate);
+
+    // Get phsyical, non-translated, IP/port for socket.
+    PString GetBaseAddress() const;
+    bool GetBaseAddress(PIPSocketAddressAndPort & addrAndPort) const;
 
     PNatMethod::Component GetComponent() const
     { return m_component; }
 
-    void SetComponent(PNatMethod::Component component)
-    { m_component = component; }
-
   protected:
     virtual bool InternalGetBaseAddress(PIPSocketAddressAndPort & addrAndPort);
 
@@ -499,86 +458,106 @@ class PNatMethod_Fixed  : public PNatMethod
 {
   PCLASSINFO(PNatMethod_Fixed, PNatMethod);
   public:
-    PNatMethod_Fixed();
+    enum { DefaultPriority = 50 };
+    PNatMethod_Fixed(unsigned priority = DefaultPriority);
 
-    static PString GetNatMethodName();
-    virtual PString GetName() const;
+    static const char * MethodName();
+    virtual PCaselessString GetMethodName() const;
 
     virtual PString GetServer() const;
     virtual bool SetServer(const PString & str);
-    virtual bool GetExternalAddress(PIPSocket::Address & addr ,const PTimeInterval &);
     virtual bool GetInterfaceAddress(PIPSocket::Address & addr) const;
     virtual bool Open(const PIPSocket::Address & addr);
-    virtual bool IsAvailable(const PIPSocket::Address &);
+    virtual bool IsAvailable(const PIPSocket::Address & binding, PObject * context);
 
+    class Socket : public PNATUDPSocket
+    {
+        PCLASSINFO(Socket, PNATUDPSocket);
+      public:
+        Socket(
+          PNatMethod::Component component,
+          const PIPSocket::Address & externalAddress
+        );
+      private:
+        bool InternalGetLocalAddress(PIPSocketAddressAndPort & addr);
+        PIPSocket::Address m_externalAddress;
+    };
   protected:
-    virtual NatTypes InternalGetNatType(bool forced, const PTimeInterval & maxAge);
+    virtual PNATUDPSocket * InternalCreateSocket(Component component, PObject * context);
+    virtual void InternalUpdate();
 
-    NatTypes           m_type;
     PIPSocket::Address m_interfaceAddress;
-    PIPSocket::Address m_externalAddress;
 };
 
 /////////////////////////////////////////////////////////////
 
 /** PNatStrategy
-  The main container for all
-  NAT traversal Strategies. 
+  The main container for all NAT traversal Strategies. 
 */
-
-class PNatStrategy : public PObject
+class PNatMethods : public PSortedList<PNatMethod>
 {
-  PCLASSINFO(PNatStrategy,PObject);
+    typedef PSortedList<PNatMethod> BaseClass;
+    PCLASSINFO(PNatMethods, BaseClass);
+  public :
+    /**@name Construction */
+    //@{
+    /** Default Contructor
+      */
+    PNatMethods(
+      bool loadFromFactory = false,
+      PPluginManager * pluginMgr = NULL
+    );
+    //@}
 
-public :
+    /**@name Method Handling */
+    //@{
+    /** Load all known NAT methods.
+      */
+    void LoadAll(PPluginManager * pluginMgr = NULL);
 
-  /**@name Construction */
-  //@{
-  /** Default Contructor
-  */
-  PNatStrategy();
+    /** This function retrieves the first available NAT Traversal Method.
+        If no available NAT Method is found then NULL is returned. 
+      */
+    virtual PNatMethod * GetMethod(
+      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
+      PObject * context = NULL
+    );
 
-  /** Deconstructor
-  */
-  ~PNatStrategy();
-  //@}
+    /** This function retrieves the NAT Traversal Method with the given name. 
+        If it is not found then NULL is returned. 
+      */
+    virtual PNatMethod * GetMethodByName(
+      const PString & name
+    );
 
-  /**@name Method Handling */
-  //@{
-  /** AddMethod
-    This function is used to add the required NAT
-    Traversal Method. The Order of Loading is important
-    The first added has the highest priority.
-  */
-  void AddMethod(PNatMethod * method);
+    /** This function removes a NAT method from the list.
+      */
+    virtual bool RemoveMethod(
+      const PString & name
+    );
 
-  /** GetMethod
-    This function retrieves the first available NAT
-    Traversal Method. If no available NAT Method is found
-    then NULL is returned. 
-  */
-  PNatMethod * GetMethod(const PIPSocket::Address & address = PIPSocket::GetDefaultIpAny());
+    /** Set the priority of the method. 
+      */
+    virtual bool SetMethodPriority(
+      const PString & name,
+      unsigned priority
+    );
 
-  /** GetMethodByName
-    This function retrieves the NAT Traversal Method with the given name. 
-    If it is not found then NULL is returned. 
-  */
-  PNatMethod * GetMethodByName(const PString & name);
+    /** Determine if the address is local or external.
+      */
+    virtual bool IsLocalAddress(
+      const PIPSocket::Address & ip
+    ) const;
 
-  /** RemoveMethod
-    This function removes a NAT method from the NATlist matching the supplied method name
-   */
-  bool RemoveMethod(const PString & meth);
+    /** Set the port ranges to be used on local machine.
+        Note that the ports used on the NAT router may not be the same unless
+        some form of port forwarding is present.
 
-    /**Set the port ranges to be used on local machine.
-       Note that the ports used on the NAT router may not be the same unless
-       some form of port forwarding is present.
+        If the port base is zero then standard operating system port allocation
+        method is used.
 
-       If the port base is zero then standard operating system port allocation
-       method is used.
-
-       If the max port is zero then it will be automatically set to the port
-       base + 99.
+        If the max port is zero then it will be automatically set to the port
+        base + 99.
       */
     void SetPortRanges(
       WORD portBase,          ///< Single socket port number base
@@ -586,49 +565,40 @@ public :
       WORD portPairBase = 0,  ///< Socket pair port number base
       WORD portPairMax = 0    ///< Socket pair port number max
     );
+    //@}
 
-    /** Get Loaded NAT Method List
-     */
-    PNatList & GetNATList() {  return natlist; };
 
-	PNatMethod * LoadNatMethod(const PString & name);
-
-    static PStringArray GetRegisteredList();
-
-  //@}
-
-private:
-  PNatList natlist;
-  PPluginManager * pluginMgr;
+    PNatMethods & GetNATList() { return *this; }  // For backward compatibility
 };
 
+
+typedef PNatMethods PNatList;     // For backward compatibility
+typedef PNatMethods PNatStrategy; // For backward compatibility
+
+
+
 ////////////////////////////////////////////////////////
 //
 // declare macros and structures needed for NAT plugins
 //
 
-template <class SVC> class PNatMethodServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject *    CreateInstance(int /*userData*/) const { return new SVC; }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const { return SVC::GetNatMethodName(); }
-    virtual bool  ValidateDeviceName(const PString & deviceName, int /*userData*/) const { return SVC::GetNatMethodName() *= deviceName; }
-};
+PCREATE_PLUGIN_SERVICE(PNatMethod);
 
-#define PCREATE_NAT_PLUGIN(name) \
-  static PNatMethodServiceDescriptor<PNatMethod_##name> PNatMethod_##name##_descriptor; \
-  PCREATE_PLUGIN(name, PNatMethod, &PNatMethod_##name##_descriptor) \
+#define PCREATE_NAT_PLUGIN(name, friendlyName) \
+    PCREATE_PLUGIN(name, PNatMethod, PNatMethod_##name, PPlugin_PNatMethod, \
+      virtual const char * GetFriendlyName() const { return friendlyName; } \
+    )
 
 
 #define P_NAT_PARAM(...) ,__VA_ARGS__
 
 
 #if P_STUN
-PPLUGIN_STATIC_LOAD(STUN, PNatMethod);
+  PPLUGIN_STATIC_LOAD(STUN, PNatMethod);
 #endif
 
 #if P_TURN
-PPLUGIN_STATIC_LOAD(TURN, PNatMethod);
+  PPLUGIN_STATIC_LOAD(TURN, PNatMethod);
 #endif
 
 #else  // P_NAT
