@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28819 $
+ * $Revision: 33984 $
  * $Author: rjongbloed $
- * $Date: 2013-01-09 00:17:37 -0600 (Wed, 09 Jan 2013) $
+ * $Date: 2015-09-07 11:59:59 +0100 (Mon, 07 Sep 2015) $
  */
 
 #ifndef PTLIB_PSOCKBUN_H
@@ -148,19 +148,9 @@ class PInterfaceMonitor : public PProcessStartup
       InterfaceChange(const InterfaceEntry & entry, bool added)
         : InterfaceEntry(entry)
         , m_added(added)
-#if P_NAT
-        , m_natMethod(NULL)
-#endif
         { }
 
       const bool m_added;
-
-#if P_NAT
-      InterfaceChange(const PNatMethod * natMethod, bool added)
-        : m_added(added), m_natMethod(natMethod) { }
-
-      const PNatMethod * const m_natMethod;
-#endif
     };
 
     /// Type for disposition notifiers
@@ -192,10 +182,6 @@ class PInterfaceMonitor : public PProcessStartup
     void SetInterfaceFilter(PInterfaceFilter * filter);
     bool HasInterfaceFilter() const { return m_interfaceFilter != NULL; }
 
-#if P_NAT
-    void OnRemoveNatMethod(const PNatMethod * natMethod);
-#endif
-
   protected:
     virtual void OnShutdown();
 
@@ -206,14 +192,14 @@ class PInterfaceMonitor : public PProcessStartup
 
     typedef std::multimap<unsigned, Notifier> Notifiers;
     Notifiers m_notifiers;
-    PMutex    m_notifiersMutex;
+    PDECLARE_MUTEX(m_notifiersMutex);
 
     PIPSocket::InterfaceTable m_interfaces;
-    PMutex                    m_interfacesMutex;
+   PDECLARE_MUTEX(m_interfacesMutex);
 
     bool           m_runMonitorThread;
     PTimeInterval  m_refreshInterval;
-    PMutex         m_threadMutex;
+    PDECLARE_MUTEX(m_threadMutex);
     PThread      * m_updateThread;
 
     PInterfaceFilter * m_interfaceFilter;
@@ -236,7 +222,7 @@ class PMonitoredSockets : public PSafeObject
   protected:
     PMonitoredSockets(
       bool reuseAddr    ///< Flag for sharing socket/exclusve use
-      P_NAT_PARAM(PNatMethod * natMethod)  ///< NAT method to use to create sockets.
+      P_NAT_PARAM(PNatMethods * natMethods)  ///< NAT method to use to create sockets.
     );
 
   public:
@@ -253,13 +239,13 @@ class PMonitoredSockets : public PSafeObject
     ) = 0;
 
     /// Indicate if the socket(s) are open and ready for reads/writes.
-    PBoolean IsOpen() const { return opened; }
+    PBoolean IsOpen() const { return m_opened; }
 
     /// Close all socket(s)
     virtual PBoolean Close() = 0;
 
     /// Return the local port number being used by the socket(s)
-    WORD GetPort() const { return localPort; }
+    WORD GetPort() const { return m_localPort; }
 
     /// Get the local address for the given interface.
     virtual PBoolean GetAddress(
@@ -331,17 +317,6 @@ class PMonitoredSockets : public PSafeObject
       InterfaceEntry & info   ///< Information on the interface
     ) const;
 
-#if P_NAT
-    /// Set the NAT method, eg STUN client pointer
-    void SetNatMethod(
-      PNatMethod * method
-    ) { natMethod = method; }
-
-    // Get the current NAT method, eg STUN client pointer
-    PNatMethod * GetNatMethod() const { return natMethod; }
-
-#endif
-
     /** Create a new monitored socket instance based on the interface
         descriptor. This will create a multiple or single socket derived class
         of PMonitoredSockets depending on the iface parameter.
@@ -361,16 +336,10 @@ class PMonitoredSockets : public PSafeObject
     static PMonitoredSockets * Create(
       const PString & iface,              ///< Interface name to create socket for
       bool reuseAddr = false              ///< Re-use or exclusive port number
-      P_NAT_PARAM(PNatMethod * natMethod = NULL)     ///< NAT method
+      P_NAT_PARAM(PNatMethods * natMethods = NULL)     ///< NAT method
     );
 
   protected:
-#if P_NAT
-    virtual void OnRemoveNatMethod(
-      const PNatMethod * natMethod
-    );
-#endif
-
     struct SocketInfo {
       SocketInfo()
         : socket(NULL)
@@ -402,14 +371,14 @@ class PMonitoredSockets : public PSafeObject
       BundleParams & param
     );
 
-    WORD          localPort;
-    bool          reuseAddress;
+    WORD          m_localPort;
+    bool          m_reuseAddress;
 #if P_NAT
-    PNatMethod  * natMethod;
+    PNatMethods * m_natMethods;
 #endif
 
-    bool          opened;
-    PUDPSocket    interfaceAddedSignal;
+    bool          m_opened;
+    PUDPSocket    m_interfaceAddedSignal;
 };
 
 typedef PSafePtr<PMonitoredSockets> PMonitoredSocketsPtr;
@@ -472,12 +441,19 @@ class PMonitoredSocketChannel : public PChannel
       WORD & port,                  ///< Port listening on
       bool usingNAT                 ///< Require NAT address/port
     );
+    bool GetLocal(
+      PIPSocket::AddressAndPort & ap, ///< IP address and port of local interface
+      bool usingNAT                 ///< Require NAT address/port
+    );
 
     /// Set the remote address/port for all Write() functions
     void SetRemote(
       const PIPSocket::Address & address, ///< Remote IP address
       WORD port                           ///< Remote port number
-    );
+    ) { m_remoteAP.SetAddress(address, port); }
+    void SetRemote(
+      const PIPSocket::AddressAndPort & ap ///< Remote IP address and port
+    ) { m_remoteAP = ap; }
 
     /// Set the remote address/port for all Write() functions
     void SetRemote(
@@ -488,7 +464,10 @@ class PMonitoredSocketChannel : public PChannel
     void GetRemote(
       PIPSocket::Address & addr,  ///< Remote IP address
       WORD & port                 ///< Remote port number
-    ) const { addr = remoteAddress; port = remotePort; }
+    ) const { addr = m_remoteAP.GetAddress(); port = m_remoteAP.GetPort(); }
+    void GetRemote(
+      PIPSocketAddressAndPort & ap  ///< Remote IP address and port
+    ) const { ap = m_remoteAP; }
 
     /** Set flag for receiving UDP data from any remote address. If the flag
         is false then data received from anything other than the configured
@@ -496,36 +475,37 @@ class PMonitoredSocketChannel : public PChannel
       */
     void SetPromiscuous(
       bool flag   ///< New flag
-    ) { promiscuousReads = flag; }
+    ) { m_promiscuousReads = flag; }
 
     /// Get flag for receiving UDP data from any remote address
-    bool GetPromiscuous() { return promiscuousReads; }
+    bool GetPromiscuous() { return m_promiscuousReads; }
 
     /// Get the IP address and port of the last received UDP data.
     void GetLastReceived(
       PIPSocket::Address & addr,  ///< Remote IP address
       WORD & port                 ///< Remote port number
-    ) const { addr = lastReceivedAddress; port = lastReceivedPort; }
+    ) const { addr = m_lastReceivedAP.GetAddress(); port = m_lastReceivedAP.GetPort(); }
+    void GetLastReceived(
+      PIPSocketAddressAndPort & ap  ///< Remote IP address and port
+    ) const { ap = m_lastReceivedAP; }
 
     /// Get the interface the last received UDP data was recieved on.
-    PString GetLastReceivedInterface() const { return lastReceivedInterface; }
+    PString GetLastReceivedInterface() const { return m_lastReceivedInterface; }
 
     /// Get the monitored socket bundle being used by this channel.
-    const PMonitoredSocketsPtr & GetMonitoredSockets() const { return socketBundle; }
+    const PMonitoredSocketsPtr & GetMonitoredSockets() const { return m_socketBundle; }
   //@}
 
   protected:
-    PMonitoredSocketsPtr socketBundle;
-    bool                 sharedBundle;
-    PString              currentInterface;
-    bool                 promiscuousReads;
-    PIPSocket::Address   remoteAddress;
-    bool                 closing;
-    WORD                 remotePort;
-    PIPSocket::Address   lastReceivedAddress;
-    WORD                 lastReceivedPort;
-    PString              lastReceivedInterface;
-    PMutex               mutex;
+    PMonitoredSocketsPtr    m_socketBundle;
+    bool                    m_sharedBundle;
+    PString                 m_currentInterface;
+    bool                    m_promiscuousReads;
+    bool                    m_closing;
+    PIPSocketAddressAndPort m_remoteAP;
+    PIPSocketAddressAndPort m_lastReceivedAP;
+    PString                 m_lastReceivedInterface;
+    PDECLARE_MUTEX(m_mutex);
 };
 
 
@@ -542,7 +522,7 @@ class PMonitoredSocketBundle : public PMonitoredSockets
       const PString & fixedInterface, ///< Interface name for bundle, "" is all interfaces
       unsigned ipVersion,             ///< Version of IP to listen
       bool reuseAddr                  ///< Flag for sharing socket/exclusve use
-      P_NAT_PARAM(PNatMethod * natMethod = NULL)  ///< NAT method to use to create sockets.
+      P_NAT_PARAM(PNatMethods * natMethods = NULL)  ///< NAT method to use to create sockets.
     );
     ~PMonitoredSocketBundle();
 
@@ -600,6 +580,7 @@ class PMonitoredSocketBundle : public PMonitoredSockets
 
   protected:
     PDECLARE_InterfaceNotifier(PMonitoredSocketBundle, OnInterfaceChange);
+    PInterfaceMonitor::Notifier m_onInterfaceChange;
 
     typedef std::map<std::string, SocketInfo> SocketInfoMap_T;
 
@@ -625,7 +606,7 @@ class PSingleMonitoredSocket : public PMonitoredSockets
     PSingleMonitoredSocket(
       const PString & theInterface, ///< Interface to bind to
       bool reuseAddr    ///< Flag for sharing socket/exclusve use
-      P_NAT_PARAM(PNatMethod * natMethod = NULL)  ///< NET method to use to create sockets.
+      P_NAT_PARAM(PNatMethods * natMethods = NULL)  ///< NET method to use to create sockets.
     );
     ~PSingleMonitoredSocket();
 
@@ -682,6 +663,7 @@ class PSingleMonitoredSocket : public PMonitoredSockets
 
   protected:
     PDECLARE_InterfaceNotifier(PSingleMonitoredSocket, OnInterfaceChange);
+    PInterfaceMonitor::Notifier m_onInterfaceChange;
 
     bool IsInterface(const PString & iface) const;
 
