@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28941 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 33980 $
+ * $Author: rjongbloed $
+ * $Date: 2015-09-05 09:57:55 +0100 (Sat, 05 Sep 2015) $
  */
 
 #ifndef PTLIB_PSSL_H
@@ -43,9 +43,12 @@ struct ssl_ctx_st;
 struct x509_st;
 struct X509_name_st;
 struct evp_pkey_st;
+struct evp_cipher_ctx_st;
 struct dh_st;
 struct aes_key_st;
 struct SHAstate_st;
+struct bio_st;
+
 
 enum PSSLFileTypes {
   PSSLFileTypePEM,
@@ -54,6 +57,10 @@ enum PSSLFileTypes {
 };
 
 
+typedef PNotifierTemplate<bool> PSSLPasswordNotifier;
+#define PDECLARE_SSLPasswordNotifier(cls, fn) PDECLARE_NOTIFIER2(PString, cls, fn, bool)
+
+
 /**Private key for SSL.
    This class embodies a common environment for all private keys used by the
    PSSLContext and PSSLChannel classes.
@@ -167,7 +174,8 @@ class PSSLPrivateKey : public PObject
       */
     PBoolean Load(
       const PFilePath & keyFile,  ///< Private key file
-      PSSLFileTypes fileType = PSSLFileTypeDEFAULT  ///< Type of file to read
+      PSSLFileTypes fileType = PSSLFileTypeDEFAULT,  ///< Type of file to read
+      const PSSLPasswordNotifier & notifier = PSSLPasswordNotifier()  ///< Call back for asking of password
     );
 
     /**Save private key to file.
@@ -360,9 +368,58 @@ class PSSLCertificate : public PObject
 };
 
 
-/**Diffie-Hellman parameters for SSL.
-   This class embodies a set of Diffie Helman parameters as used by
-   PSSLContext and PSSLChannel classes.
+/** Create a "fingerprint" for SSL certificate.
+  */
+class PSSLCertificateFingerprint : public PObject
+{
+    PCLASSINFO(PSSLCertificateFingerprint, PObject);
+  public:
+    P_DECLARE_ENUM(HashType,
+      HashMd5,
+      HashSha1,
+      HashSha256,
+      HashSha512
+    );
+
+    /// Create empty fingerprint.
+    PSSLCertificateFingerprint();
+
+    /** Create fingerprint by parsing string in format: "type fingerprint"
+     */
+    PSSLCertificateFingerprint(
+      const PString& inStr    ///< Fingerprint string.
+    );
+
+    /** Create fingerprint with specified type from certificate.
+     */
+    PSSLCertificateFingerprint(
+      HashType type,                      ///< Hasing algorithm to use
+      const PSSLCertificate& certificate  ///< Certificate to fingerprint.
+    );
+
+    Comparison Compare(const PObject & other) const;
+
+    bool IsValid() const;
+
+    bool MatchForCertificate(
+      const PSSLCertificate& cert
+    ) const;
+
+    PString AsString() const;
+    bool FromString(const PString & str);
+
+    HashType GetHash() const { return m_hashAlogorithm; }
+    void SetHash(HashType hash) { m_hashAlogorithm = hash; }
+
+  private:
+    HashType m_hashAlogorithm;
+    PString  m_fingerprint;
+};
+
+
+/**Diffie-Hellman key exchange conteext.
+   This class embodies a set of Diffie Helman key exchange parameters and
+   context.
   */
 class PSSLDiffieHellman : public PObject
 {
@@ -383,12 +440,24 @@ class PSSLDiffieHellman : public PObject
     );
 
     /**Create a set of Diffie-Hellman parameters.
+       If \p pubKey is NULL, it is automatically generated.
+
+       Note, all three buffers (if not NULL) must point to numBits/8 bytes.
       */
     PSSLDiffieHellman(
-      const BYTE * pData, ///< P data
-      PINDEX pSize,       ///< Size of P data
-      const BYTE * gData, ///< G data
-      PINDEX gSize        ///< Size of G data
+      PINDEX numBits,            ///< Number of bits
+      const BYTE * pData,        ///< Modulus data (numBits/8 bytes)
+      const BYTE * gData,        ///< Generator data (numBits/8 bytes)
+      const BYTE * pubKey = NULL ///< Public key data (numBits/8 bytes)
+    );
+
+    /**Create a set of Diffie-Hellman parameters.
+       If \p pubKey is empty, it is automatically generated.
+      */
+    PSSLDiffieHellman(
+      const PBYTEArray & pData,   ///< Modulus data
+      const PBYTEArray & gData,   ///< Generator data
+      const PBYTEArray & pubKey = PBYTEArray() ///< Public key data
     );
 
     /**Create a copy of the Diffie-Hellman parameters.
@@ -407,9 +476,13 @@ class PSSLDiffieHellman : public PObject
       */
     ~PSSLDiffieHellman();
 
+    /**Return true if is a valid Diffie-Hellman context.
+     */
+    bool IsValid() const { return m_dh != NULL; }
+
     /**Get internal OpenSSL DH structure.
       */
-    operator dh_st *() const { return dh; }
+    operator dh_st *() const { return m_dh; }
 
     /**Load Diffie-Hellman parameters from file.
        The type of the file can be specified explicitly, or if
@@ -421,8 +494,37 @@ class PSSLDiffieHellman : public PObject
       PSSLFileTypes fileType = PSSLFileTypeDEFAULT  ///< Type of file to read
     );
 
+    /**Get number of bits being used.
+      */
+    PINDEX GetNumBits() const;
+
+    /**Get the P value
+      */
+    PBYTEArray GetModulus() const;
+
+    /**Get the G value
+      */
+    PBYTEArray GetGenerator() const;
+
+    /**Get the "half-key" value
+      */
+    PBYTEArray GetHalfKey() const;
+
+    /**Compute the session key, geven other half-key
+      */
+    bool ComputeSessionKey(const PBYTEArray & otherHalf);
+
+    /**Get the session key value
+      */
+    const PBYTEArray & GetSessionKey() const { return m_sessionKey; }
+
   protected:
-    dh_st * dh;
+    bool Construct(const BYTE * pData, PINDEX pSize,
+                   const BYTE * gData, PINDEX gSize,
+                   const BYTE * kData, PINDEX kSize);
+
+    dh_st    * m_dh;
+    PBYTEArray m_sessionKey;
 };
 
 
@@ -448,6 +550,104 @@ class PAESContext : public PObject
 #endif // P_SSL_AES
 
 
+/// Encryption/decryption context
+class PSSLCipherContext : public PObject
+{
+    PCLASSINFO(PSSLCipherContext, PObject);
+  public:
+    PSSLCipherContext(
+      bool encrypt
+    );
+    
+    ~PSSLCipherContext();
+
+    /**Get internal OpenSSL cipher context structure.
+      */
+    operator evp_cipher_ctx_st *() const { return m_context; }
+
+    /// Indicate we are encrypting data
+    bool IsEncrypt() const;
+
+    /**Get selected algorithm
+      */
+    PString GetAlgorithm() const;
+
+    /** Set encryption/decryption algorithm.
+        The \p 
+      */
+    bool SetAlgorithm(
+      const PString & name  ///< Name or OID for the algorithm.
+    );
+
+    /** Set encryption/decryption key.
+      */
+    bool SetKey(const PBYTEArray & key) { return SetKey(key, key.GetSize()); }
+    bool SetKey(const BYTE * keyPtr, PINDEX keyLen);
+
+    /** Set encryption/decryption initial vector.
+      */
+    bool SetIV(const PBYTEArray & iv) { return SetIV(iv, iv.GetSize()); }
+    bool SetIV(const BYTE * ivPtr, PINDEX ivLen);
+
+    enum PadMode {
+      NoPadding,
+      PadPKCS,
+      PadLoosePKCS,
+      PadCipherStealing
+    };
+
+    /**Set padding mode.
+       If NoPadding, then buffers supplied to Process() must be exact multiple
+       of the block size.
+      */
+    bool SetPadding(PadMode pad);
+
+    /**Get padding mode.
+       If NoPadding, then buffers supplied to Process() must be exact multiple
+       of the block size.
+      */
+    PadMode GetPadding() const { return m_padMode; }
+
+    /** Encrypt/Decrypt a block of data.
+      */
+    bool Process(
+      const PBYTEArray & in,  ///< Data to be encrypted
+      PBYTEArray & out        ///< Encrypted data
+    );
+    bool Process(
+      const BYTE * inPtr,     ///< Data to be encrypted
+      PINDEX inLen,           ///< Length of data to be encrypted
+      BYTE * outPtr,          ///< Encrypted data
+      PINDEX & outLen,        ///< Max output space on input, then actual output data size
+      bool partial = false    ///< Partial data, more to come
+    );
+
+    /** Get the cipher key length
+      */
+    PINDEX GetKeyLength() const;
+
+    /** Get the cipher initial vector length
+      */
+    PINDEX GetIVLength() const;
+
+    /** Get the cipher block size
+      */
+    PINDEX GetBlockSize() const;
+
+    /** Calculate the rounded up size for encrypted data
+      */
+    PINDEX GetBlockedDataSize(PINDEX size) const;
+
+  protected:
+    PadMode             m_padMode;
+    evp_cipher_ctx_st * m_context;
+
+  private:
+    PSSLCipherContext(const PSSLCipherContext &) { }
+    void operator=(const PSSLCipherContext &) { }
+};
+
+
 /// SHA1 digest scheme
 class PSHA1Context : public PObject
 {
@@ -469,6 +669,10 @@ class PSHA1Context : public PObject
 
   protected:
     SHAstate_st * m_context;
+
+  private:
+    PSHA1Context(const PSHA1Context &) { }
+    void operator=(const PSHA1Context &) { }
 };
 
 
@@ -477,13 +681,23 @@ class PSHA1Context : public PObject
    using the PSSLChannel class. It includes such things as the version of SSL
    and certificates, CA's etc.
   */
-class PSSLContext {
+class PSSLContext : public PObject
+{
+    PCLASSINFO(PSSLContext, PObject);
   public:
-    enum Method {
+    P_DECLARE_TRACED_ENUM(Method,
       SSLv23,
       SSLv3,
-      TLSv1
-    };
+      TLSv1,
+      TLSv1_1,
+      TLSv1_2,
+      DTLSv1,
+      DTLSv1_2,
+      DTLSv1_2_v1_0
+    );
+
+    static const Method HighestTLS = TLSv1_2;
+    static const Method HighestDTLS = DTLSv1_2_v1_0;
 
     /**Create a new context for SSL channels.
        An optional session ID may be provided in the context. This is used
@@ -583,9 +797,28 @@ class PSSLContext {
       bool create = false           ///< If certificate/provateKey are file paths and do not exist, then create.
     );
 
+    /// Set the notifier for when SSL needs to get a password to unlock a private key.
+    void SetPasswordNotifier(
+      const PSSLPasswordNotifier & notifier   ///< Notifier to be called
+    );
+
+    /// Set TLS extension
+    bool SetExtension(
+      const char * extension
+    );
+
+    Method GetMethod() const { return m_method; }
+
   protected:
-    void Construct(Method method, const void * sessionId, PINDEX idSize);
+    void Construct(const void * sessionId, PINDEX idSize);
+
+    Method       m_method;
     ssl_ctx_st * m_context;
+    PSSLPasswordNotifier m_passwordNotifier;
+
+  private:
+    PSSLContext(const PSSLContext &) { }
+    void operator=(const PSSLContext &) { }
 };
 
 
@@ -616,7 +849,7 @@ class PSSLChannel : public PIndirectChannel
     virtual PBoolean Close();
     virtual PBoolean Shutdown(ShutdownValue) { return true; }
     virtual PString GetErrorText(ErrorGroup group = NumErrorGroups) const;
-    virtual PBoolean ConvertOSError(P_INT_PTR error, ErrorGroup group = LastGeneralError);
+    virtual PBoolean ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group = LastGeneralError);
 
     // New functions
     /**Accept a new inbound connection (server).
@@ -685,10 +918,29 @@ class PSSLChannel : public PIndirectChannel
 
     typedef PSSLContext::VerifyMode VerifyMode;
 
+    struct VerifyInfo
+    {
+      VerifyInfo(bool ok, const PSSLCertificate & cert) : m_ok(ok), m_peerCertificate(cert) { }
+      bool m_ok;
+      PSSLCertificate m_peerCertificate;
+    };
+    typedef PNotifierTemplate<VerifyInfo &> VerifyNotifier;
+    #define PDECLARE_SSLVerifyNotifier(cls, fn) PDECLARE_NOTIFIER2(PSSLChannel, cls, fn, PSSLChannel::VerifyInfo &)
+    #define PCREATE_SSLVerifyNotifier(fn) PCREATE_NOTIFIER2(fn, PSSLChannel::VerifyInfo &)
+
     /**Set certificate verification mode for connection.
       */
     void SetVerifyMode(
-      VerifyMode mode
+      VerifyMode mode,
+      const VerifyNotifier & notifier = VerifyNotifier()
+    );
+
+    /** Call back for certificate verification.
+        Default calls m_verifyNotifier if not NULL.
+      */
+    virtual bool OnVerify(
+      bool ok,
+      const PSSLCertificate & peerCertificate
     );
 
     /**Get the peer certificate, if there is one.
@@ -704,28 +956,87 @@ class PSSLChannel : public PIndirectChannel
 
     PSSLContext * GetContext() const { return m_context; }
 
-    virtual PBoolean RawSSLRead(void * buf, PINDEX & len);
+    /**Get the internal SSL context structure.
+      */
+    operator ssl_st *() const { return m_ssl; }
+
 
   protected:
     void Construct(PSSLContext * ctx, PBoolean autoDel);
-
-    /**This callback is executed when the Open() function is called with
-       open channels. It may be used by descendent channels to do any
-       handshaking required by the protocol that channel embodies.
-
-       The default behaviour "connects" the channel to the OpenSSL library.
-
-       @return
-       Returns true if the protocol handshaking is successful.
-     */
-    virtual PBoolean OnOpen();
+    virtual bool InternalAccept();
+    virtual bool InternalConnect();
 
   protected:
-    PSSLContext * m_context;
-    bool          m_autoDeleteContext;
-    ssl_st      * m_ssl;
+    static int  BioRead(bio_st * bio, char * buf, int len);
+    static int  BioWrite(bio_st * bio, const char * buf, int len);
+    static long BioControl(bio_st * bio, int cmd, long num, void * ptr);
+    static int  BioClose(bio_st * bio);
+
+    virtual int  BioRead(char * buf, int len);
+    virtual int  BioWrite(const char * buf, int len);
+    virtual long BioControl(int cmd, long num, void * ptr);
+    virtual int  BioClose();
+
+    PSSLContext  * m_context;
+    bool           m_autoDeleteContext;
+    ssl_st       * m_ssl;
+    bio_st       * m_bio;
+    VerifyNotifier m_verifyNotifier;
+
+    P_REMOVE_VIRTUAL(PBoolean,RawSSLRead(void *, PINDEX &),false);
 };
 
+
+/**This class will start a secure SSL based channel.
+*/
+class PSSLChannelDTLS : public PSSLChannel
+{
+    PCLASSINFO(PSSLChannelDTLS, PSSLChannel)
+  public:
+    /**Create a new channel given the context.
+       If no context is given a default one is created.
+    */
+    PSSLChannelDTLS(
+      PSSLContext * context = NULL,   ///< Context for SSL channel
+      bool autoDeleteContext = false  ///< Flag for context to be automatically deleted.
+    );
+    PSSLChannelDTLS(
+      PSSLContext & context           ///< Context for SSL channel
+    );
+
+    /**Close and clear the SSL channel.
+    */
+    ~PSSLChannelDTLS();
+
+    /** Set the MTU for DTLS handshake.
+        Note, should be done before calling ExecuteHandshake().
+      */
+    bool SetMTU(
+      unsigned bytes
+    );
+
+    /** Perform negotiation handshake.
+      */
+    bool ExecuteHandshake();
+
+    /// Indicate we are operating as a server
+    bool IsServer() const;
+
+    /// Get the selected profile name.
+    PCaselessString GetSelectedProfile() const;
+
+    /// Get the key material after exchange completed.
+    PBYTEArray GetKeyMaterial(
+      PINDEX materialSize,
+      const char * name
+    ) const;
+
+  protected:
+    virtual bool InternalAccept();
+    virtual bool InternalConnect();
+};
+
+
 #endif // PTLIB_PSSL_H
 
 
