@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28623 $
+ * $Revision: 34057 $
  * $Author: rjongbloed $
- * $Date: 2012-12-02 22:09:17 -0600 (Sun, 02 Dec 2012) $
+ * $Date: 2015-09-18 17:46:05 +0100 (Fri, 18 Sep 2015) $
  */
 
 #ifndef PTLIB_PSTUN_H
@@ -35,12 +35,14 @@
 #pragma interface
 #endif
 
-#include <ptbuildopts.h>
+#include <ptlib.h>
 
 #if P_STUN
 
 #include <ptclib/pnat.h>
 #include <ptlib/sockets.h>
+#include <ptclib/cypher.h>
+
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -91,10 +93,14 @@ class PSTUN {
       PSTUNMessage & message
     );
 
+    virtual bool ValidateMessageIntegrity(
+      const PSTUNMessage & message
+    );
+
     virtual void SetCredentials(
       const PString & username, 
       const PString & password, 
-      const PString & realm
+      const PString & realm       /// If empty the using short term credentials
     );
 
     /**Get the timeout for responses from STUN server.
@@ -117,13 +123,11 @@ class PSTUN {
       PINDEX retries    ///< Number of retries
     ) { m_pollRetries = retries; }
 
-    PNatMethod::NatTypes    m_natType;
     PINDEX                  m_pollRetries;
-    PTime                   m_timeAddressObtained;
-    PString                 m_userName;
-    PString                 m_realm;
-    PString                 m_nonce;
-    PBYTEArray              m_credentialsHash;
+    PSASLString             m_userName;
+    PSASLString             m_realm;
+    PSASLString             m_nonce;
+    PBYTEArray              m_password; // text for short term, MD5 for long term
     PIPSocket::Address      m_interface;
     PIPSocketAddressAndPort m_serverAddress;
     PTimeInterval           replyTimeout;
@@ -142,7 +146,7 @@ struct PSTUNAttribute
     SOURCE_ADDRESS      = 0x0004,   // RFC 3489
     CHANGED_ADDRESS     = 0x0005,   // RFC 3489
     USERNAME            = 0x0006,   // RFC 3489 & RFC 5389
-    PASSWORD            = 0x0007,   // RFC 3489
+    PASSWORD            = 0x0007,   // RFC 3489 (deprecated)
     MESSAGE_INTEGRITY   = 0x0008,   // RFC 3489 & RFC 5389
     ERROR_CODE          = 0x0009,   // RFC 3489 & RFC 5389
     UNKNOWN_ATTRIBUTES  = 0x000a,   // RFC 3489 & RFC 5389  
@@ -166,9 +170,16 @@ struct PSTUNAttribute
     // TIMER-VAL        =  0x0021,
     RESERVATION_TOKEN   =  0x0022,  // RFC 5766
 
+    PRIORITY            = 0x0024,   // RFC 5245 (ICE)
+    USE_CANDIDATE       = 0x0025,   // RFC 5245 (ICE)
+    ICE_CONTROLLED      = 0x8029,   // RFC 5245 (ICE)
+    ICE_CONTROLLING     = 0x802A,   // RFC 5245 (ICE)
+
     PADDING             = 0x0026,   // RFC 5389 (added in RFC 5780)
     RESPONSE_PORT       = 0x0027,   // RFC 5389 (added in RFC 5780)
 
+    FINGERPRINT         = 0x8028,   // RFC 5389
+
     ALTERNATE_SERVER    = 0x8023,   // RFC 5389
 
     RESPONSE_ORIGIN     = 0x802b,   // RFC 5389 (added in RFC 5780)
@@ -177,7 +188,13 @@ struct PSTUNAttribute
   
   PUInt16b type;
   PUInt16b length;
-  
+
+  PSTUNAttribute(Types newType = ERROR_CODE, size_t len = 0)
+    : type((WORD)newType)
+    , length((WORD)len)
+  {
+  }
+
   PSTUNAttribute * GetNext() const;
 };
 
@@ -192,12 +209,12 @@ class PSTUNAddressAttribute : public PSTUNAttribute
     BYTE     ip[4];
 
   public:
-    void InitAddrAttr(Types newType)
+    PSTUNAddressAttribute(Types newType, const PIPSocketAddressAndPort & addrAndPort)
+      : PSTUNAttribute(newType, SizeofAddressAttribute)
+      , pad(0)
+      , family(1)
     {
-      pad    = 0;
-      family = 1;
-      type   = (WORD)newType;
-      length = SizeofAddressAttribute;
+      SetIPAndPort(addrAndPort);
     }
 
     WORD GetPort() const;
@@ -207,10 +224,7 @@ class PSTUNAddressAttribute : public PSTUNAttribute
 
   protected:
     enum { SizeofAddressAttribute = sizeof(BYTE)+sizeof(BYTE)+sizeof(WORD)+4 };
-    bool IsValidAddrAttr(Types checkType) const
-    {
-      return type == checkType && length == SizeofAddressAttribute;
-    }
+    bool IsValidAddrAttr(Types checkType) const { return type == checkType && length == SizeofAddressAttribute; }
 };
 
 
@@ -220,17 +234,16 @@ class PSTUNStringAttribute : public PSTUNAttribute
     char m_string[763];   // not actually 763, but this will do
 
     PSTUNStringAttribute(Types newType, const PString & str)
-    { InitStringAttr(newType, str); }
-
-    PString GetString() const { return PString(m_string, length); }
-
-    void InitStringAttr(Types newType, const PString & str)
+      : PSTUNAttribute(newType, str.GetLength())
     {
-      type   = (WORD)newType;
-      length = (WORD)str.GetLength();
       memcpy(m_string, (const char *)str, length);
     }
 
+    PString GetString() const
+    {
+      return PString(m_string, length);
+    }
+
     bool IsValidStringAttr(Types checkType) const
     {
       return (type == checkType) && (length == strlen(m_string));
@@ -243,22 +256,14 @@ class PSTUNChangeRequest : public PSTUNAttribute
   public:
     BYTE flags[4];
     
-    PSTUNChangeRequest() { }
-
-    PSTUNChangeRequest(bool changeIP, bool changePort)
+    PSTUNChangeRequest(bool changeIP = false, bool changePort = false)
+      : PSTUNAttribute(CHANGE_REQUEST, sizeof(flags))
     {
-      Initialise();
+      memset(flags, 0, sizeof(flags));
       SetChangeIP(changeIP);
       SetChangePort(changePort);
     }
 
-    void Initialise()
-    {
-      type = CHANGE_REQUEST;
-      length = sizeof(flags);
-      memset(flags, 0, sizeof(flags));
-    }
-
     bool IsValid() const { return type == CHANGE_REQUEST && length == sizeof(flags); }
     
     bool GetChangeIP() const { return (flags[3]&4) != 0; }
@@ -268,31 +273,53 @@ class PSTUNChangeRequest : public PSTUNAttribute
     void SetChangePort(bool on) { if (on) flags[3] |= 2; else flags[3] &= ~2; }
 };
 
+
 class PSTUNMessageIntegrity : public PSTUNAttribute
 {
   public:
-    BYTE hmac[20];
+    BYTE m_hmac[PHMAC::KeyLength];
 
-    PSTUNMessageIntegrity(const BYTE * data = NULL)
-    { Initialise(data); }
-    
-    void Initialise(const BYTE * data = NULL)
+    PSTUNMessageIntegrity(const BYTE * hmac = NULL)
+      : PSTUNAttribute(MESSAGE_INTEGRITY, sizeof(m_hmac))
     {
-      type = MESSAGE_INTEGRITY;
-      length = sizeof(hmac);
-      if (data == NULL)
-        memset(hmac, 0, sizeof(hmac));
+      if (hmac == NULL)
+        memset(m_hmac, 0, sizeof(m_hmac));
       else
-        memcpy(hmac, data, sizeof(hmac));
+        memcpy(m_hmac, hmac, sizeof(m_hmac));
     }
-    bool IsValid() const { return type == MESSAGE_INTEGRITY && length == sizeof(hmac); }
+
+    bool IsValid() const { return type == MESSAGE_INTEGRITY && length == sizeof(m_hmac); }
 };
 
+
+class PSTUNFingerprint : public PSTUNAttribute
+{
+  public:
+    PUInt32b m_crc;
+
+    PSTUNFingerprint(DWORD crc = 0)
+      : PSTUNAttribute(FINGERPRINT, sizeof(m_crc))
+      , m_crc(crc)
+    {
+    }
+
+    bool IsValid() const { return type == FINGERPRINT && length == sizeof(m_crc); }
+};
+
+
 class PSTUNErrorCode : public PSTUNAttribute
 {
   public:
     PSTUNErrorCode()
-    { Initialise(); }
+    {
+      Initialise();
+    }
+
+    PSTUNErrorCode(int code, const PString & reason = PString::Empty())
+    {
+      Initialise();
+      SetErrorCode(code, reason);
+    }
 
     BYTE m_zero1;
     BYTE m_zero2;
@@ -338,10 +365,14 @@ class PSTUNUDPSocket : public PNATUDPSocket
 {
   PCLASSINFO(PSTUNUDPSocket, PNATUDPSocket);
   public:
-    PSTUNUDPSocket();
+    PSTUNUDPSocket(
+      PNatMethod::Component component
+    );
+
+    virtual const char * GetNatName() const;
 
     bool OpenSTUN(PSTUNClient & client);
-    PNatCandidate GetCandidateInfo();
+    virtual void GetCandidateInfo(PNatCandidate & candidate);
 
     bool BaseWriteTo(const void * buf, PINDEX len, const PIPSocketAddressAndPort & ap)
     { Slice slice((void *)buf, len); return PUDPSocket::InternalWriteTo(&slice, 1, ap); }
@@ -357,7 +388,6 @@ class PSTUNUDPSocket : public PNATUDPSocket
     PIPSocketAddressAndPort m_baseAddressAndPort;
 
     bool InternalGetLocalAddress(PIPSocketAddressAndPort & addr);
-    bool InternalGetBaseAddress(PIPSocketAddressAndPort & addr);
 
   private:
     PNatMethod::NatTypes m_natType;
@@ -369,6 +399,8 @@ class PSTUNMessage : public PBYTEArray
 {
   public:
     enum MsgType {
+      InvalidMessage,
+
       BindingRequest        = 0x0001,
       BindingResponse       = 0x0101,
       BindingError          = 0x0111,
@@ -377,57 +409,95 @@ class PSTUNMessage : public PBYTEArray
       SharedSecretResponse  = 0x0102,
       SharedSecretError     = 0x0112,
 
-      Allocate              = 0x0003,  // RFC 5766
-      AllocateResponse      = 0x0103,  // RFC 5766
-      AllocateError         = 0x0113,  // RFC 5766
+      // RFC 5766
+      Allocate              = 0x0003,  
+      AllocateResponse      = 0x0103,
+      AllocateError         = 0x0113,
 
-      Refresh               = 0x0004,  // RFC 5766
-      Send                  = 0x0006,  // RFC 5766
-      Data                  = 0x0007,  // RFC 5766
-      CreatePermission      = 0x0008,  // RFC 5766
-      ChannelBind           = 0x0009,  // RFC 5766
+      Refresh               = 0x0004,
+      RefreshResponse       = 0x0104,
+      RefreshError          = 0x0114,
 
+      Send                  = 0x0016,
+      Data                  = 0x0017,
+
+      CreatePermission      = 0x0008,
+      CreatePermResponse    = 0x0108,
+      CreatePermError       = 0x0118,
+
+      ChannelBind           = 0x0009,
+      ChannelBindResponse   = 0x0109,
+      ChannelBindError      = 0x0119,
+
+      // RFC6062
+      Connect               = 0x000a,
+      ConnectResponse       = 0x010a,
+      ConnectError          = 0x011a,
+
+      ConnectionBind        = 0x000b,
+      ConnectionBindResponse= 0x010b,
+      ConnectionBindError   = 0x011b,
+
+      ConnectionAttempt     = 0x001c,
     };
     
     PSTUNMessage();
     PSTUNMessage(MsgType newType, const BYTE * id = NULL);
+    PSTUNMessage(const BYTE * data, PINDEX size, const PIPSocketAddressAndPort & srcAddr);
 
-    void SetType(MsgType newType, const BYTE * id = NULL);
-    MsgType GetType() const;
+#if PTRACING
+    void PrintOn(ostream & strm) const;
+#endif
 
-    const BYTE * GetTransactionID() const;
+    bool IsValid() const;
+    bool IsValidFor(const PSTUNMessage & request) const;
+
+    PBYTEArray GetTransactionID() const;
+
+    bool IsRFC5389() const;
 
     const PSTUNMessageHeader * operator->() const { return (const PSTUNMessageHeader *)theArray; }
 
-    PSTUNAttribute * GetFirstAttribute() const;
+    MsgType GetType() const;
+    void SetType(MsgType newType, const BYTE * id = NULL);
+    void SetErrorType(int code, const BYTE * id, const char * reason = NULL);
 
-    bool Validate();
-    bool Validate(const PSTUNMessage & request);
+    bool IsRequest()         const { return (GetType() & 0x0110) == 0x0000; }
+    bool IsIndication()      const { return (GetType() & 0x0110) == 0x0010; }
+    bool IsSuccessResponse() const { return (GetType() & 0x0110) == 0x0100; }
+    bool IsErrorResponse()   const { return (GetType() & 0x0110) == 0x0110; }
 
     PSTUNAttribute * AddAttribute(const PSTUNAttribute & attribute);
     PSTUNAttribute * SetAttribute(const PSTUNAttribute & attribute);
     PSTUNAttribute * FindAttribute(PSTUNAttribute::Types type) const;
 
-    template <class Type> Type * FindAttributeOfType(PSTUNAttribute::Types type) const
+    template <class Type> Type * FindAttributeAs(PSTUNAttribute::Types type) const
     { return static_cast<Type *>(FindAttribute(type)); }
 
+    PString FindAttributeString(PSTUNAttribute::Types type, const char * dflt = NULL) const;
+
     bool Read(PUDPSocket & socket);
     bool Write(PUDPSocket & socket) const;
+    bool Write(PUDPSocket & socket, const PIPSocketAddressAndPort & ap) const;
     bool Poll(PUDPSocket & socket, const PSTUNMessage & request, PINDEX pollRetries);
 
-    bool IsRFC5389() const { return m_isRFC5389; }
-
     const PIPSocketAddressAndPort GetSourceAddressAndPort() const { return m_sourceAddressAndPort; }
 
-    void InsertMessageIntegrity(BYTE * credentialsHash, PINDEX credentialsHashLen);
-    void InsertMessageIntegrity(BYTE * credentialsHash, PINDEX credentialsHashLen, PSTUNMessageIntegrity * mi);
+    void AddMessageIntegrity(const PBYTEArray & credentialsHash) { AddMessageIntegrity(credentialsHash, credentialsHash.GetSize()); }
+    void AddMessageIntegrity(const BYTE * credentialsHashPtr, PINDEX credentialsHashLen, PSTUNMessageIntegrity * mi = NULL);
 
-    bool CheckMessageIntegrity(BYTE * credentialsHash, PINDEX credentialsHashLen);
-    void CalculateMessageIntegrity(BYTE * credentialsHash, PINDEX credentialsHashLen, PSTUNMessageIntegrity * mi, BYTE * hmac);
+    bool CheckMessageIntegrity(const PBYTEArray & credentialsHash) const { return CheckMessageIntegrity(credentialsHash, credentialsHash.GetSize()); }
+    bool CheckMessageIntegrity(const BYTE * credentialsHashPtr, PINDEX credentialsHashLen) const;
+
+    void AddFingerprint(PSTUNFingerprint * fp = NULL);
+    bool CheckFingerprint(bool required) const;
 
   protected:
+    PSTUNAttribute * GetFirstAttribute() const;
+    void CalculateMessageIntegrity(const BYTE * credentialsHash, PINDEX credentialsHashLen, PSTUNMessageIntegrity * mi, BYTE * hmac) const;
+    DWORD CalculateFingerprint(PSTUNFingerprint * fp) const;
+
     PIPSocketAddressAndPort m_sourceAddressAndPort;
-    bool m_isRFC5389;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -436,15 +506,16 @@ class PSTUNMessage : public PBYTEArray
   */
 class PSTUNClient : public PNatMethod, public PSTUN
 {
-  PCLASSINFO(PSTUNClient, PNatMethod);
+    PCLASSINFO(PSTUNClient, PNatMethod);
   public:
-    PSTUNClient();
+    enum { DefaultPriority = 40 };
+    PSTUNClient(unsigned priority = DefaultPriority);
     ~PSTUNClient();
 
     /**Get the NAT Method Name
      */
-    static PString GetNatMethodName();
-    virtual PString GetName() const;
+    static const char * MethodName();
+    virtual PCaselessString GetMethodName() const;
 
     /**Set the STUN server to use.
        The server string may be of the form host:port. If :port is absent
@@ -456,6 +527,14 @@ class PSTUNClient : public PNatMethod, public PSTUN
       const PString & server
     );
 
+    /**Set the authentication credentials.
+      */
+    virtual void SetCredentials(
+      const PString & username, 
+      const PString & password, 
+      const PString & realm
+    );
+
     /**Get the current server address name.
        Defaults to be "address:port" string form.
       */
@@ -465,11 +544,6 @@ class PSTUNClient : public PNatMethod, public PSTUN
       PIPSocketAddressAndPort & serverAddressAndPort 
     ) const;
 
-    virtual bool GetExternalAddress(
-      PIPSocket::Address & externalAddress, ///< External address of router
-      const PTimeInterval & maxAge = 1000   ///< Maximum age for caching
-    );
-
     virtual bool GetInterfaceAddress(
       PIPSocket::Address & internalAddress
     ) const;
@@ -478,17 +552,13 @@ class PSTUNClient : public PNatMethod, public PSTUN
       const PIPSocket::Address & ifaceAddr
     );
 
-    bool IsAvailable(
-      const PIPSocket::Address & binding
+    virtual bool IsAvailable(
+      const PIPSocket::Address & binding,
+      PObject * userData
     );
 
     virtual void Close();
 
-    // new functions
-    NatTypes FindNatType(
-      const PIPSocket::Address & binding
-    );
-
     /**Create a single socket.
        The STUN protocol is used to create a socket for which the external IP
        address and port numbers are known. A PUDPSocket descendant is returned
@@ -501,11 +571,12 @@ class PSTUNClient : public PNatMethod, public PSTUN
        The socket pointer is set to NULL if the function fails and returns
        false.
       */
-    bool CreateSocket(
-      Component component,
+    virtual bool CreateSocket(
       PUDPSocket * & socket,
-      const PIPSocket::Address & = PIPSocket::GetDefaultIpAny(), 
-      WORD port = 0
+      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
+      WORD localPort = 0,
+      PObject * context = NULL,
+      Component component = eComponent_Unknown
     );
 
     /**Create a socket pair.
@@ -524,20 +595,20 @@ class PSTUNClient : public PNatMethod, public PSTUN
     virtual bool CreateSocketPair(
       PUDPSocket * & socket1,
       PUDPSocket * & socket2,
-      const PIPSocket::Address & = PIPSocket::GetDefaultIpAny()
+      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
+      PObject * context = NULL
     );
 
-    bool InternalOpenSocket(Component component, const PIPSocket::Address & binding, PSTUNUDPSocket & socket, PortInfo & portInfo);
-
-  protected:    
-    virtual NatTypes InternalGetNatType(bool forced, const PTimeInterval & maxAge);
+  protected:
+    virtual PNATUDPSocket * InternalCreateSocket(Component component, PObject * context);
+    virtual void InternalUpdate();
+    bool InternalSetServer(const PIPSocketAddressAndPort & addr);
 
     PSTUNUDPSocket * m_socket;
-    PMutex m_mutex;
 
   private:
-    PIPSocketAddressAndPort m_externalAddress;
-    PINDEX                  numSocketsForPairing;
+    PString m_serverName;
+    PINDEX  m_numSocketsForPairing;
 };
 
 
@@ -613,12 +684,16 @@ class PTURNUDPSocket : public PSTUNUDPSocket, public PSTUN
 
     friend class PTURNClient;
 
-    PTURNUDPSocket();
+    PTURNUDPSocket(
+      PNatMethod::Component component
+    );
     ~PTURNUDPSocket();
 
+    virtual const char * GetNatName() const;
+
     virtual bool Close();
 
-    virtual PNatCandidate GetCandidateInfo();
+    virtual void GetCandidateInfo(PNatCandidate & candidate);
 
     int OpenTURN(PTURNClient & client);
 
@@ -651,40 +726,33 @@ class PTURNUDPSocket : public PSTUNUDPSocket, public PSTUN
 
 class PTURNClient : public PSTUNClient
 {
-  PCLASSINFO(PTURNClient, PSTUNClient);
+    PCLASSINFO(PTURNClient, PSTUNClient);
   public:
 
     friend class PTURNUDPSocket;
 
     /**Get the NAT Method Name
      */
-    static PString GetNatMethodName();
-    virtual PString GetName() const;
+    static const char * MethodName();
+    virtual PCaselessString GetMethodName() const;
 
-    PTURNClient();
+    enum { DefaultPriority = 30 };
+    PTURNClient(unsigned priority = DefaultPriority);
 
     // overrides from PNatMethod
-    virtual bool Open(
-      const PIPSocket::Address & iface
-    );
-
-    virtual void SetCredentials(
-      const PString & username, 
-      const PString & password, 
-      const PString & realm
-    );
-
     bool CreateSocket(
-      Component component,
       PUDPSocket * & socket,
-      const PIPSocket::Address & = PIPSocket::GetDefaultIpAny(), 
-      WORD port = 0
+      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
+      WORD localPort = 0,
+      PObject * context = NULL,
+      Component component = eComponent_Unknown
     );
 
     bool CreateSocketPair(
       PUDPSocket * & socket1,
       PUDPSocket * & socket2,
-      const PIPSocket::Address & binding
+      const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),
+      PObject * context = NULL
     );
 
     /**Return an indication if the current STUN type supports RTP
@@ -695,9 +763,9 @@ class PTURNClient : public PSTUNClient
     );
 
   protected:
+    virtual PNATUDPSocket * InternalCreateSocket(Component component, PObject * context);
     // New functions
     virtual bool RefreshAllocation(DWORD lifetime = 600);
-    PString m_password;
 };
 
 
