@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28207 $
+ * $Revision: 29535 $
  * $Author: rjongbloed $
- * $Date: 2012-08-16 19:09:54 -0500 (Thu, 16 Aug 2012) $
+ * $Date: 2013-04-19 09:51:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef PTLIB_PTTS_H
@@ -35,7 +35,7 @@
 #pragma interface
 #endif
 
-#include <ptbuildopts.h>
+#include <ptlib.h>
 
 #if P_TTS
 
