@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28684 $
+ * $Revision: 33833 $
  * $Author: rjongbloed $
- * $Date: 2012-12-17 23:41:26 -0600 (Mon, 17 Dec 2012) $
+ * $Date: 2015-08-05 20:13:34 +0100 (Wed, 05 Aug 2015) $
  */
 
 #ifndef PTLIB_PVIDFILE_H
@@ -46,28 +46,15 @@
 #include <ptlib/videoio.h>
 
 
-/**
- * Abstract class for a file containing a sequence of video frames
- */
-class PVideoFile : public PVideoFrameInfo
+/**Abstract class for a file containing a sequence of video frames.
+  */
+class PVideoFile : public PFile
 {
-  PCLASSINFO(PVideoFile, PVideoFrameInfo);
+  PCLASSINFO(PVideoFile, PFile);
   protected:
     PVideoFile();
 
   public:
-    virtual PBoolean Open(
-      const PFilePath & name,    // Name of file to open.
-      PFile::OpenMode mode = PFile::ReadWrite, // Mode in which to open the file.
-      PFile::OpenOptions opts = PFile::ModeDefault     // <code>OpenOptions</code> enum# for open operation.
-    );
-
-    virtual PBoolean IsOpen() const { return m_file.IsOpen(); }
-    virtual PBoolean Close() { return m_file.Close(); }
-
-    virtual PBoolean WriteFrame(const void * frame);
-    virtual PBoolean ReadFrame(void * frame);
-
     virtual off_t GetLength() const;
     virtual PBoolean SetLength(
       off_t len   // New length of file in frames.
@@ -79,6 +66,9 @@ class PVideoFile : public PVideoFrameInfo
       PFile::FilePositionOrigin origin = PFile::Start  ///< Origin for position change.
     );
 
+    virtual PBoolean WriteFrame(const void * frame);
+    virtual PBoolean ReadFrame(void * frame);
+
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
       unsigned height   ///< New height of frame
@@ -87,49 +77,104 @@ class PVideoFile : public PVideoFrameInfo
       unsigned rate  ///< Frames  per second
     );
 
-    const PFilePath & GetFilePath() const { return m_file.GetFilePath(); }
-    PChannel::Errors GetErrorCode(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const { return m_file.GetErrorCode(group); }
-    int GetErrorNumber(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const { return m_file.GetErrorNumber(group); }
-    PString GetErrorText(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const { return m_file.GetErrorText(group); }
     PINDEX GetFrameBytes() const { return m_frameBytes; }
 
+    bool SetFrameSizeFromFilename(const PString & fn);
+    bool SetFPSFromFilename(const PString & fn);
+
+    operator const PVideoFrameInfo &() const { return m_videoInfo; }
+    bool GetFrameSize(unsigned & w, unsigned & h) const { return m_videoInfo.GetFrameSize(w, h); }
+    unsigned GetFrameWidth() const { return m_videoInfo.GetFrameWidth(); }
+    unsigned GetFrameHeight() const { return m_videoInfo.GetFrameHeight(); }
+    unsigned GetFrameRate() const { return m_videoInfo.GetFrameRate(); }
+    const PString & GetColourFormat() const { return m_videoInfo.GetColourFormat(); }
 
   protected:
     bool   m_fixedFrameSize;
     bool   m_fixedFrameRate;
     PINDEX m_frameBytes;
     off_t  m_headerOffset;
-    PFile  m_file;
+    off_t  m_frameHeaderLen;
+    PVideoFrameInfo m_videoInfo;
 };
 
-/**
- * A file containing a sequence of raw YUV files in planar 4:2:0 format
- * Example files can be found at http://media.xiph.org/video/derf/
- */
+typedef PFactory<PVideoFile, PFilePathString> PVideoFileFactory;
 
+
+/**A file containing a sequence of raw YUV files in planar 4:2:0 format.
+   Example files can be found at http://media.xiph.org/video/derf/
+  */
 class PYUVFile : public PVideoFile
 {
-  PCLASSINFO(PYUVFile, PVideoFile);
+    PCLASSINFO(PYUVFile, PVideoFile);
   public:
     PYUVFile();
 
-    virtual PBoolean Open(
-      const PFilePath & name,    // Name of file to open.
-      PFile::OpenMode mode = PFile::ReadWrite, // Mode in which to open the file.
-      PFile::OpenOptions opts = PFile::ModeDefault     // <code>OpenOptions</code> enum# for open operation.
-    );
-
     virtual PBoolean WriteFrame(const void * frame);
     virtual PBoolean ReadFrame(void * frame);
 
   protected:
+    virtual bool InternalOpen(OpenMode mode, OpenOptions opts, PFileInfo::Permissions permissions);
+
     bool m_y4mMode;
 };
 
 PFACTORY_LOAD(PYUVFile);
 
-#endif
-#endif
+
+/**A file containing a single image, which is repeatedly output.
+  */
+class PBMPFile : public PVideoFile
+{
+    PCLASSINFO(PBMPFile, PVideoFile);
+  public:
+    PBMPFile();
+
+    virtual PBoolean WriteFrame(const void * frame);
+    virtual PBoolean ReadFrame(void * frame);
+
+  protected:
+    virtual bool InternalOpen(OpenMode mode, OpenOptions opts, PFileInfo::Permissions permissions);
+
+    PBYTEArray m_imageData;
+};
+
+PFACTORY_LOAD(PBMPFile);
+
+
+#if P_JPEG_DECODER
+
+/**A file containing a JPEG image, which is repeatedly output.
+  */
+class PJPEGFile : public PVideoFile
+{
+  PCLASSINFO(PJPEGFile, PVideoFile);
+  public:
+    PJPEGFile();
+    ~PJPEGFile();
+
+    virtual bool Close();
+    virtual PBoolean IsOpen() const;
+    virtual off_t GetLength() const;
+    virtual off_t GetPosition() const;
+    virtual bool SetPosition(off_t pos, PFile::FilePositionOrigin origin);
+
+    virtual PBoolean WriteFrame(const void * frame);
+    virtual PBoolean ReadFrame(void * frame);
+
+  protected:
+    virtual bool InternalOpen(OpenMode mode, OpenOptions opts, PFileInfo::Permissions permissions);
+
+    PBYTEArray m_pixelData;
+};
+
+PFACTORY_LOAD(PJPEGFile);
+
+#endif  // P_JPEG_DECODER
+
+#endif  // P_VIDFILE
+
+#endif  // P_VIDEO
 
 #endif // PTLIB_PVIDFILE_H
 
