@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 33309 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2015-02-02 09:44:57 +0000 (Mon, 02 Feb 2015) $
  */
 
 #ifndef PTLIB_RANDOM_H
@@ -67,54 +67,62 @@ class PRandom
        get the same sequence of values on each run. Useful for debugging.
       */
     PRandom(
-      DWORD seed    ///< New seed value, must not be zero
+      uint32_t seed    ///< New seed value, must not be zero
     );
 
     /**Set the seed for the random number generator.
       */
     void SetSeed(
-      DWORD seed    ///< New seed value, must not be zero
+      uint32_t seed    ///< New seed value, must not be zero
     );
 
     /**Get the next psuedo-random number in sequence.
        This generates one pseudorandom unsigned integer (32bit) which is
        uniformly distributed among 0 to 2^32-1 for each call.
       */
-    unsigned Generate();
+    uint32_t Generate();
 
     /**Get the next psuedo-random number in sequence.
        This generates one pseudorandom unsigned integer from 0 to maximum.
-       Uses the Generate() function and scales accordingly.
+       Uses the Generate() function and scales accordingly. Is inclusive of
+       maximum, i.e. [0..maximum].
       */
-    unsigned Generate(unsigned maximum);
+    uint32_t Generate(uint32_t maximum);
 
     /**Get the next psuedo-random number in sequence.
        This generates one pseudorandom unsigned integer from minimum to maximum.
-       Uses the Generate() function and scales and shifts accordingly.
+       Uses the Generate() function and scales and shifts accordingly. Is
+       inclusive of endpoints, i.e. [minimum..maximum].
       */
-    unsigned Generate(unsigned minimum, unsigned maximum);
+    uint32_t Generate(uint32_t minimum, uint32_t maximum);
 
     /**Get the next psuedo-random number in sequence.
        This generates one pseudorandom unsigned integer which is
        uniformly distributed among 0 to maximum for each call. Uses
        Generate()
       */
-    inline operator unsigned() { return Generate(); }
+    inline operator uint32_t() { return Generate(); }
 
 
     /**Get the next psuedo-random number in sequence.
        This utilises a single system wide thread safe PRandom variable. All
        threads etc will share the same psuedo-random sequence.
       */
-    static unsigned Number();
+    static uint32_t Number();
 
     /** Get a random number between zero and maximum
     */
-    static unsigned Number(unsigned maximum);
+    static uint32_t Number(unsigned maximum);
 
     /** Get a random number between minimum and maximum
     */
-    static unsigned Number(unsigned minimum, unsigned maximum);
+    static uint32_t Number(unsigned minimum, unsigned maximum);
+
+    /** Get a random set of bits.
+    */
+    static PBYTEArray Octets(PINDEX size);
+    static void Octets(PBYTEArray & octets, PINDEX size = 0);
+    static void Octets(BYTE *  octets, PINDEX size);
 
   protected:
     enum {
@@ -122,12 +130,12 @@ class PRandom
       RandSize = 1<<RandBits
     };
 
-    DWORD randcnt;
-    DWORD randrsl[RandSize];
-    DWORD randmem[RandSize];
-    DWORD randa;
-    DWORD randb;
-    DWORD randc;
+    uint32_t randcnt;
+    uint32_t randrsl[RandSize];
+    uint32_t randmem[RandSize];
+    uint32_t randa;
+    uint32_t randb;
+    uint32_t randc;
 };
 
 
