@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29866 $
+ * $Revision: 34413 $
  * $Author: rjongbloed $
- * $Date: 2013-06-02 21:53:30 -0500 (Sun, 02 Jun 2013) $
+ * $Date: 2015-12-11 07:16:49 +0000 (Fri, 11 Dec 2015) $
  */
 
 
@@ -180,11 +180,13 @@ class PThreadPoolBase : public PObject
         std::string m_group;
     };
 
-    ~PThreadPoolBase();
+    ~PThreadPoolBase()
+    {
+      Shutdown();
+    }
 
-    virtual WorkerThreadBase * CreateWorkerThread() = 0;
-    virtual WorkerThreadBase * AllocateWorker();
-    virtual WorkerThreadBase * NewWorker();
+    void Shutdown();
+    virtual void ReclaimWorkers();
 
     unsigned GetMaxWorkers() const { return m_maxWorkerCount; }
 
@@ -206,16 +208,19 @@ class PThreadPoolBase : public PObject
       PThread::Priority priority
     );
 
-    virtual bool CheckWorker(WorkerThreadBase * worker);
-    void StopWorker(WorkerThreadBase * worker);
-    PMutex m_listMutex;
+    virtual WorkerThreadBase * AllocateWorker();
+    virtual WorkerThreadBase * CreateWorkerThread() = 0;
+    virtual void StopWorker(WorkerThreadBase * worker);
 
     typedef std::vector<WorkerThreadBase *> WorkerList_t;
     WorkerList_t m_workers;
+    PMutex       m_mutex;
 
     unsigned          m_maxWorkerCount;
     unsigned          m_maxWorkUnitCount;
+#if PTRACING
     unsigned          m_highWaterMark; // For logging
+#endif
     PString           m_threadName;
     PThread::Priority m_priority;
 };
@@ -266,15 +271,15 @@ class PThreadPool : public PThreadPoolBase
     class InternalWork : public InternalWorkBase
     {
       public:
-        InternalWork(WorkerThread * worker, Work_T * work, const char * group)
+        InternalWork(Work_T * work, const char * group)
           : InternalWorkBase(group)
-          , m_worker(worker)
           , m_work(work)
+          , m_worker(NULL)
         { 
         }
 
-        WorkerThread * m_worker;
         Work_T * m_work;
+        WorkerThread * m_worker;
     };
 
     //
@@ -290,6 +295,7 @@ class PThreadPool : public PThreadPoolBase
     struct GroupInfo {
       unsigned m_count;
       WorkerThread * m_worker;
+      GroupInfo(WorkerThread * worker) : m_count(1), m_worker(worker) { }
     };
 
 
@@ -305,52 +311,40 @@ class PThreadPool : public PThreadPoolBase
     //
     bool AddWork(Work_T * work, const char * group = NULL)
     {
-      PWaitAndSignal m(m_listMutex);
+      // create internal work structure
+      InternalWork internalWork(work, group);
 
-      // allocate by group if specified
-      // else allocate to least busy
-      WorkerThread * worker;
-      if ((group == NULL) || (strlen(group) == 0)) {
-        worker = (WorkerThread *)AllocateWorker();
+      typename GroupInfoMap_t::iterator iterGroup = m_groupInfoMap.end();
+
+      PWaitAndSignal m(m_mutex);
+
+      // allocate by group if specified, else allocate to least busy
+      if (internalWork.m_group.empty() || (iterGroup = m_groupInfoMap.find(group)) == m_groupInfoMap.end()) {
+        internalWork.m_worker = (WorkerThread *)AllocateWorker();
+
+        // if cannot allocate worker, return
+        if (internalWork.m_worker == NULL) 
+          return false;
+
+        // add group ID to map
+        if (!internalWork.m_group.empty()) {
+          m_groupInfoMap.insert(make_pair(internalWork.m_group, GroupInfo(internalWork.m_worker)));
+          PTRACE(6, "ThreadPool", "Setting worker thread \"" << *internalWork.m_worker << "\""
+                             " with group Id \"" << internalWork.m_group << '"');
+        }
       }
       else {
-
-        // find the worker thread with the matching group ID
-        // if no matching Id, then create a new thread
-        typename GroupInfoMap_t::iterator g = m_groupInfoMap.find(group);
-        if (g == m_groupInfoMap.end()) 
-          worker = (WorkerThread *)AllocateWorker();
-        else {
-          worker = g->second.m_worker;
-          PTRACE(4, "ThreadPool\tAllocated worker thread by group Id " << group);
-        }
+        internalWork.m_worker = iterGroup->second.m_worker;
+        ++iterGroup->second.m_count;
+        PTRACE(6, "ThreadPool", "Using existing worker thread \"" << *internalWork.m_worker << "\""
+                           " with group Id \"" << internalWork.m_group << "\", count=" << iterGroup->second.m_count);
       }
 
-      // if cannot allocate worker, return
-      if (worker == NULL) 
-        return false;
-
-      // create internal work structure
-      InternalWork internalWork(worker, work, group);
-
       // add work to external to internal map
-      m_externalToInternalWorkMap.insert(typename ExternalToInternalWorkMap_T::value_type(work, internalWork));
+      m_externalToInternalWorkMap.insert(make_pair(work, internalWork));
 
-      // add group ID to map
-      if (!internalWork.m_group.empty()) {
-        typename GroupInfoMap_t::iterator r = m_groupInfoMap.find(internalWork.m_group);
-        if (r != m_groupInfoMap.end())
-          ++r->second.m_count;
-        else {
-          GroupInfo info;
-          info.m_count  = 1;
-          info.m_worker = worker;
-          m_groupInfoMap.insert(typename GroupInfoMap_t::value_type(internalWork.m_group, info));
-        }
-      }
-      
       // give the work to the worker
-      worker->AddWork(work);
+      internalWork.m_worker->AddWork(work);
     
       return true;
     }
@@ -358,33 +352,29 @@ class PThreadPool : public PThreadPoolBase
     //
     //  remove a unit of work from a worker thread
     //
-    bool RemoveWork(Work_T * work, bool removeFromWorker = true)
+    bool RemoveWork(Work_T * work)
     {
-      PWaitAndSignal m(m_listMutex);
+      PWaitAndSignal m(m_mutex);
 
       // find worker with work unit to remove
       typename ExternalToInternalWorkMap_T::iterator iterWork = m_externalToInternalWorkMap.find(work);
-      if (iterWork == m_externalToInternalWorkMap.end())
+      if (!PAssert(iterWork != m_externalToInternalWorkMap.end(), "Missing work!"))
         return false;
 
       InternalWork & internalWork = iterWork->second;
 
-      // tell worker to stop processing work
-      if (removeFromWorker)
-        internalWork.m_worker->RemoveWork(work);
-
       // update group information
       if (!internalWork.m_group.empty()) {
         typename GroupInfoMap_t::iterator iterGroup = m_groupInfoMap.find(internalWork.m_group);
-        PAssert(iterGroup != m_groupInfoMap.end(), "Attempt to find thread from unknown work group");
-        if (iterGroup != m_groupInfoMap.end()) {
-          if (--iterGroup->second.m_count == 0)
-            m_groupInfoMap.erase(iterGroup);
+        if (PAssert(iterGroup != m_groupInfoMap.end(), "Unknown work group") && --iterGroup->second.m_count == 0) {
+          PTRACE(6, "ThreadPool", "Removing worker thread \"" << *internalWork.m_worker << "\""
+                                  " from group Id \"" << internalWork.m_group << '"');
+          m_groupInfoMap.erase(iterGroup);
         }
       }
 
-      // see if workers need reorganising
-      CheckWorker(internalWork.m_worker);
+      // tell worker to stop processing work
+      internalWork.m_worker->RemoveWork(work);
 
       // remove element from work unit map
       m_externalToInternalWorkMap.erase(iterWork);
@@ -399,17 +389,38 @@ class PThreadPool : public PThreadPoolBase
 template <class Work_T>
 class PQueuedThreadPool : public PThreadPool<Work_T>
 {
+  protected:
+    PTimeInterval m_workerIncreaseLatency;
+    unsigned      m_workerIncreaseLimit;
+
   public:
     //
     //  constructor
     //
     PQueuedThreadPool(
-      unsigned maxWorkers = 10,
+      unsigned maxWorkers = std::max(PThread::GetNumProcessors(), 10U),
       unsigned maxWorkUnits = 0,
       const char * threadName = NULL,
-      PThread::Priority priority = PThread::NormalPriority
+      PThread::Priority priority = PThread::NormalPriority,
+      const PTimeInterval & workerIncreaseLatency = PMaxTimeInterval,
+      unsigned workerIncreaseLimit = 0
     ) : PThreadPool<Work_T>(maxWorkers, maxWorkUnits, threadName, priority)
-    { }
+      , m_workerIncreaseLatency(workerIncreaseLatency)
+      , m_workerIncreaseLimit(workerIncreaseLimit)
+    {
+        PTRACE(4, NULL, "ThreadPool", "Thread pool created:"
+                                      " maxWorkers=" << maxWorkers << ","
+                                      " maxWorkUnits=" << maxWorkUnits << ","
+                                      " threadName=" << this->m_threadName << ","
+                                      " priority=" << priority << ","
+                                      " workerIncreaseLatency=" << workerIncreaseLatency << ","
+                                      " workerIncreaseLimit=" << workerIncreaseLimit);
+    }
+
+    const PTimeInterval & GetWorkerIncreaseLatency() const { return m_workerIncreaseLatency; }
+    void SetWorkerIncreaseLatency(const PTimeInterval & time) { m_workerIncreaseLatency = time; }
+    unsigned GetWorkerIncreaseLimit() const { return m_workerIncreaseLimit; }
+    void SetWorkerIncreaseLimit(unsigned limit) { m_workerIncreaseLimit = limit; }
 
     class QueuedWorkerThread : public PThreadPool<Work_T>::WorkerThread
     {
@@ -418,66 +429,80 @@ class PQueuedThreadPool : public PThreadPool<Work_T>
                            PThread::Priority priority = PThread::NormalPriority,
                            const char * threadName = NULL)
           : PThreadPool<Work_T>::WorkerThread(pool, priority, threadName)
-          , m_available(0, INT_MAX)
+          , m_working(false)
         {
         }
 
-        ~QueuedWorkerThread()
-        { }
-
         void AddWork(Work_T * work)
         {
-          m_mutex.Wait();
-          m_queue.push(work);
-          m_available.Signal();
-          m_mutex.Signal();
+          if (PAssertNULL(work) != NULL)
+            this->m_queue.Enqueue(QueuedWork(work));
         }
 
-        void RemoveWork(Work_T * )
+        void RemoveWork(Work_T * work)
         {
-          m_mutex.Wait();
-          Work_T * work = m_queue.front();
-          m_queue.pop();
-          m_mutex.Signal();
           delete work;
         }
 
         unsigned GetWorkSize() const
         {
-          return (unsigned)m_queue.size();
+          return (unsigned)this->m_queue.size()+this->m_working;
         }
 
         void Main()
         {
-          for (;;) {
-            m_available.Wait();
-            if (PThreadPool<Work_T>::WorkerThread::m_shutdown)
-              break;
+          QueuedWork item;
+          while (this->m_queue.Dequeue(item)) {
+            this->m_working = true;
 
-            m_mutex.Wait();
-            Work_T * work = m_queue.empty() ? NULL : m_queue.front();
-            m_mutex.Signal();
+            PQueuedThreadPool & pool = dynamic_cast<PQueuedThreadPool &>(this->m_pool);
+            PTimeInterval latency = item.m_time.GetElapsed();
 
-            if (work != NULL) {
-              work->Work();
-              PThreadPool<Work_T>::WorkerThread::m_pool.RemoveWork(work);
-            }
+            item.m_work->Work();
+
+            if (!pool.RemoveWork(item.m_work))
+              this->RemoveWork(item.m_work);
+
+            this->m_working = false;
+
+            if (latency > pool.m_workerIncreaseLatency)
+              pool.OnMaxWaitTime(latency);
           }
         }
 
         void Shutdown()
         {
-          PThreadPool<Work_T>::WorkerThread::m_shutdown = true;
-          m_available.Signal();
+          this->m_shutdown = true;
+          this->m_queue.Close(true);
         }
 
       protected:
-        typedef std::queue<Work_T *> Queue;
-        Queue      m_queue;
-        PMutex     m_mutex;
-        PSemaphore m_available;
+        struct QueuedWork
+        {
+          QueuedWork() : m_time(0), m_work(NULL) { }
+          explicit QueuedWork(Work_T * work) : m_work(work) { }
+          PTime    m_time;
+          Work_T * m_work;
+        };
+        PSyncQueue<QueuedWork> m_queue;
+        bool                   m_working;
     };
 
+    virtual void OnMaxWaitTime(const PTimeInterval & PTRACE_PARAM(latency))
+    {
+      unsigned newMaxWorkers = std::min((this->m_maxWorkerCount*11+9)/10, m_workerIncreaseLimit);
+      if (newMaxWorkers != this->m_maxWorkerCount) {
+        PTRACE(2, NULL, "ThreadPool", "Thread pool latency excessive"
+               " (" << latency << "s > " << this->m_workerIncreaseLatency << "s),"
+               " increasing maximum threads from " << this->m_maxWorkerCount << " to " << newMaxWorkers);
+        this->m_maxWorkerCount = newMaxWorkers;
+      }
+      else {
+        PTRACE(2, NULL, "ThreadPool", "Thread pool latency excessive"
+               " (" << latency << "s > " << this->m_workerIncreaseLatency << "s),"
+               " cannot increase threads past " << m_workerIncreaseLimit);
+      }
+    }
 
     virtual PThreadPoolBase::WorkerThreadBase * CreateWorkerThread()
     {
@@ -514,14 +539,15 @@ typedef PQueuedThreadPool<PSafeWork> PSafeThreadPool;
 
 /// A PSafeWork thread pool item where call back has no arguments.
 template <class PtrClass, typename FuncRet = void>
-class PSafeWorkNoArg : public PSafeWork {
+class PSafeWorkNoArg : public PSafeWork
+{
+  PCLASSINFO_ALIGNED(PSafeWorkNoArg, PSafeWork, 16);
+
   public:
     typedef FuncRet (PtrClass::*Function)();
 
   protected:
-#pragma pack(16)
-    Function m_function;
-#pragma pack()
+    P_ALIGN_FIELD(Function,m_function,16);
 
   public:
     PSafeWorkNoArg(
@@ -544,15 +570,16 @@ template <
   typename Arg1Type,
   typename FuncRet = void
 >
-class PSafeWorkArg1 : public PSafeWork {
+class PSafeWorkArg1 : public PSafeWork
+{
+  PCLASSINFO_ALIGNED(PSafeWorkArg1, PSafeWork, 16);
+
   public:
     typedef FuncRet (PtrClass::*Function)(Arg1Type arg1);
 
   protected:
-#pragma pack(16)
-    Function m_function;
+    P_ALIGN_FIELD(Function,m_function,16);
     Arg1Type m_arg1;
-#pragma pack()
 
   public:
     PSafeWorkArg1(
@@ -578,16 +605,17 @@ template <
   typename Arg2Type,
   typename FuncRet = void
 >
-class PSafeWorkArg2 : public PSafeWork {
+class PSafeWorkArg2 : public PSafeWork
+{
+  PCLASSINFO_ALIGNED(PSafeWorkArg2, PSafeWork, 16);
+
   public:
     typedef FuncRet (PtrClass::*Function)(Arg1Type arg1, Arg2Type arg2);
 
   protected:
-#pragma pack(16)
-    Function m_function;
+    P_ALIGN_FIELD(Function,m_function,16);
     Arg1Type m_arg1;
     Arg2Type m_arg2;
-#pragma pack()
 
   public:
     PSafeWorkArg2(
@@ -616,17 +644,18 @@ template <
   typename Arg3Type,
   typename FuncRet = void
 >
-class PSafeWorkArg3 : public PSafeWork {
+class PSafeWorkArg3 : public PSafeWork
+{
+  PCLASSINFO_ALIGNED(PSafeWorkArg3, PSafeWork, 16);
+
   public:
     typedef FuncRet (PtrClass::*Function)(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3);
 
   protected:
-#pragma pack(16)
-    Function m_function;
+    P_ALIGN_FIELD(Function,m_function,16);
     Arg1Type m_arg1;
     Arg2Type m_arg2;
     Arg3Type m_arg3;
-#pragma pack()
 
   public:
     PSafeWorkArg3(
