@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29661 $
+ * $Revision: 33749 $
  * $Author: rjongbloed $
- * $Date: 2013-05-08 00:02:01 -0500 (Wed, 08 May 2013) $
+ * $Date: 2015-07-26 14:47:31 +0100 (Sun, 26 Jul 2015) $
  */
 
 #ifndef PTLIB_PURL_H
@@ -226,6 +226,9 @@ class PURL : public PObject
     );
 
 
+    /// Extract scheme as per RFC2396
+    static PCaselessString ExtractScheme(const char * str);
+
     /// Get the scheme field of the URL.
     const PCaselessString & GetScheme() const { return scheme; }
 
@@ -298,7 +301,10 @@ class PURL : public PObject
 
     /// Set the parameter (;) field(s) of the URL as a string dictionary.
     /// Note the values will be translated using TranslateString
-    void SetParamVars(const PStringToString & paramVars);
+    void SetParamVars(
+      const PStringToString & paramVars,
+      bool merge = false
+    );
 
     /// Set the parameter (;) field of the URL as a string dictionary.
     /// Note the values will be translated using TranslateString
@@ -501,12 +507,12 @@ class PURLLegacyScheme : public PURLScheme
 
   */
 #define PURL_LEGACY_SCHEME(schemeName, \
-                           hasUsername,           /* URL scheme has a ysername */ \
+                           hasUsername,           /* URL scheme has a username */ \
                            hasPassword,           /* URL scheme has a password */ \
                            hasHostPort,           /* URL scheme has a host:port */ \
                            defaultToUserIfNoAt,   /* URL scheme is username if no @, otherwise host:port */ \
                            defaultHostToLocal,    /* URL scheme defaults to PIPSocket::GetHostName() if not present */ \
-                           hasQuery,              /* URL scheme has a query secton */ \
+                           hasQuery,              /* URL scheme has a query section */ \
                            hasParameters,         /* URL scheme has a parameter section */ \
                            hasFragments,          /* URL scheme has a fragment section */ \
                            hasPath,               /* URL scheme has a path */ \
@@ -529,7 +535,7 @@ class PURLLegacyScheme : public PURLScheme
                            defaultPort) \
         { } \
   }; \
-  static PURLSchemeFactory::Worker<PURLLegacyScheme_##schemeName> schemeName##Factory(#schemeName, true); \
+  PFACTORY_CREATE(PURLSchemeFactory, PURLLegacyScheme_##schemeName, #schemeName, true)
 
 
 
