@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29430 $
+ * $Revision: 34645 $
  * $Author: rjongbloed $
- * $Date: 2013-04-03 21:27:43 -0500 (Wed, 03 Apr 2013) $
+ * $Date: 2016-02-24 17:58:08 +0000 (Wed, 24 Feb 2016) $
  */
 
 #ifndef PTLIB_VXML_H
@@ -44,6 +44,7 @@
 #include <ptclib/pwavfile.h>
 #include <ptclib/ptts.h>
 #include <ptclib/url.h>
+#include <ptclib/script.h>
 
 #include <queue>
 
@@ -81,18 +82,21 @@ class PVXMLGrammar : public PObject
 
     GrammarState GetState() const { return m_state; }
 
+    void SetSessionTimeout();
     void SetTimeout(const PTimeInterval & timeout);
 
   protected:
     PDECLARE_NOTIFIER(PTimer, PVXMLGrammar, OnTimeout);
 
+	virtual bool IsFilled() { return false; }
+
     PVXMLSession & m_session;
     PXMLElement  & m_field;
     PString        m_value;
     GrammarState   m_state;
     PTimeInterval  m_timeout;
     PTimer         m_timer;
-    PMutex         m_mutex;
+    PDECLARE_MUTEX(m_mutex);
 };
 
 
@@ -124,6 +128,8 @@ class PVXMLDigitsGrammar : public PVXMLGrammar
 
     virtual void OnUserInput(const char ch);
 
+	virtual bool IsFilled();
+
   protected:
     PINDEX  m_minDigits;
     PINDEX  m_maxDigits;
@@ -293,13 +299,15 @@ class PVXMLSession : public PIndirectChannel
     virtual bool ProcessEvents();
     virtual bool ProcessGrammar();
     virtual bool NextNode(bool processChildren);
+    void ClearBargeIn();
+    void FlushInput();
 
     void SayAs(const PString & className, const PString & text);
     void SayAs(const PString & className, const PString & text, const PString & voice);
 
     PURL NormaliseResourceName(const PString & src);
 
-    PMutex           m_sessionMutex;
+    PDECLARE_MUTEX(m_sessionMutex);
 
     PURL             m_rootURL;
     PXML             m_xml;
@@ -322,9 +330,12 @@ class PVXMLSession : public PIndirectChannel
 
     PStringToString  m_variables;
     PString          m_variableScope;
+#if P_SCRIPTS
+    PScriptLanguage *m_scriptContext;
+#endif
 
     std::queue<char> m_userInputQueue;
-    PMutex           m_userInputMutex;
+    PDECLARE_MUTEX(m_userInputMutex);
 
     enum {
       NotRecording,
@@ -340,7 +351,8 @@ class PVXMLSession : public PIndirectChannel
       NotTransfering,
       TransferInProgress,
       TransferFailed,
-      TransferSuccessful
+      TransferSuccessful,
+      TransferCompleted
     }     m_transferStatus;
     PTime m_transferStartTime;
 };
@@ -600,8 +612,8 @@ class PVXMLChannel : public PDelayChannel
     PString mediaFormat;
     PString wavFilePrefix;
 
-    PMutex   m_channelWriteMutex;
-    PMutex   m_channelReadMutex;
+    PDECLARE_MUTEX(m_channelWriteMutex);
+    PDECLARE_MUTEX(m_channelReadMutex);
     bool     m_closed;
     bool     m_paused;
     PINDEX   m_totalData;
