@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 28497 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2012-10-05 13:38:38 +0100 (Fri, 05 Oct 2012) $
  */
 
 #ifndef PTLIB_XMPP_ROSTER_H
@@ -73,19 +73,19 @@ namespace XMPP
       const PStringSet&   GetGroups() const     { return m_Groups; }
       const PresenceInfo& GetPresence() const   { return m_Presence; }
 
-      virtual void  SetJID(const JID& jid, PBoolean dirty = PTrue)
+      virtual void  SetJID(const JID& jid, PBoolean dirty = true)
                                                 { m_JID = jid; if (dirty) SetDirty(); }
-      virtual void  SetType(ItemType type, PBoolean dirty = PTrue)
+      virtual void  SetType(ItemType type, PBoolean dirty = true)
                                                 { m_Type = type; if (dirty) SetDirty(); }
-      virtual void  SetName(const PString& name, PBoolean dirty = PTrue) 
+      virtual void  SetName(const PString& name, PBoolean dirty = true) 
                                                 { m_Name = name; if (dirty) SetDirty(); }
 
-      virtual void  AddGroup(const PString& group, PBoolean dirty = PTrue);
-      virtual void  RemoveGroup(const PString& group, PBoolean dirty = PTrue);
+      virtual void  AddGroup(const PString& group, PBoolean dirty = true);
+      virtual void  RemoveGroup(const PString& group, PBoolean dirty = true);
 
       virtual void  SetPresence(const Presence& p);
 
-      void SetDirty(PBoolean b = PTrue) { m_IsDirty = b; }
+      void SetDirty(PBoolean b = true) { m_IsDirty = b; }
 
       /** This operator will set the dirty flag
        */
@@ -105,7 +105,7 @@ namespace XMPP
       // a presence stanza if kept.
       PDictionary<PString, Presence> m_Presence;
 
-      PBoolean        m_IsDirty; // item modified locally, server needs to be updated
+      bool        m_IsDirty; // item modified locally, server needs to be updated
     };
     PLIST(ItemList, Item);
 
@@ -117,13 +117,13 @@ namespace XMPP
 
     virtual Item * FindItem(const PString& jid);
 
-    virtual PBoolean SetItem(Item * item, PBoolean localOnly = PFalse);
-    virtual PBoolean RemoveItem(const PString& jid, PBoolean localOnly = PFalse);
-    virtual PBoolean RemoveItem(Item * item, PBoolean localOnly = PFalse);
+    virtual PBoolean SetItem(Item * item, PBoolean localOnly = false);
+    virtual PBoolean RemoveItem(const PString& jid, PBoolean localOnly = false);
+    virtual PBoolean RemoveItem(Item * item, PBoolean localOnly = false);
 
     virtual void  Attach(XMPP::C2S::StreamHandler * handler);
     virtual void  Detach();
-    virtual void  Refresh(PBoolean sendPresence = PTrue);
+    virtual void  Refresh(PBoolean sendPresence = true);
 
     virtual PNotifierList& ItemChangedHandlers()    { return m_ItemChangedHandlers; }
     virtual PNotifierList& RosterChangedHandlers()  { return m_RosterChangedHandlers; }
