@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28339 $
+ * $Revision: 33502 $
  * $Author: rjongbloed $
- * $Date: 2012-09-11 02:43:32 -0500 (Tue, 11 Sep 2012) $
+ * $Date: 2015-04-14 07:15:18 +0100 (Tue, 14 Apr 2015) $
  */
 
 #ifndef PTLIB_ARRAY_H
@@ -205,6 +205,10 @@ class PAbstractArray : public PContainer
     void * GetPointer(
       PINDEX minSize = 1  ///< Minimum size the array must be.
     );
+    const void * GetPointer() const { return theArray; }
+
+    /// Retrun the length in bytes for the array
+    virtual PINDEX GetLength() const;
 
     /**Concatenate one array to the end of this array.
        This function will allocate a new array large enough for the existing
@@ -332,7 +336,7 @@ template <class T> class PBaseArray : public PAbstractArray
       PINDEX index  ///< Position on the array to get value from.
     ) const {
       PASSERTINDEX(index);
-      return index < GetSize() ? ((T *)theArray)[index] : (T)0;
+      return index < GetSize() ? (reinterpret_cast<T *>(theArray))[index] : T();
     }
 
     /**Attach a pointer to a static block to the base array type. The pointer
@@ -448,7 +452,7 @@ template <class T> class PBaseArray : public PAbstractArray
       stream << GetAt(index);
     }
 
-    PBaseArray(PContainerReference & reference) : PAbstractArray(reference, sizeof(T)) { }
+    PBaseArray(PContainerReference & reference_) : PAbstractArray(reference_, sizeof(T)) { }
 };
 
 /**Declare a dynamic array base type.
@@ -477,8 +481,8 @@ template <class T> class PBaseArray : public PAbstractArray
   PDECLARE_CLASS(cls, PBaseArray<T>) \
     cls(PINDEX initialSize = 0) \
       : PBaseArray<T>(initialSize) { } \
-    cls(PContainerReference & reference) \
-      : PBaseArray<T>(reference) { } \
+    cls(PContainerReference & reference_) \
+      : PBaseArray<T>(reference_) { } \
     cls(T const * buffer, PINDEX length, PBoolean dynamic = true) \
       : PBaseArray<T>(buffer, length, dynamic) { } \
     virtual PObject * Clone() const \
@@ -571,8 +575,8 @@ class PCharArray : public PBaseArray<char>
       PBoolean dynamic = true ///< Buffer is copied and dynamically allocated.
     ) : ParentClass(buffer, length, dynamic) { }
 
-    PCharArray(PContainerReference & reference)
-      : ParentClass(reference) { }
+    PCharArray(PContainerReference & reference_)
+      : ParentClass(reference_) { }
   //@}
 
   /**@name Overrides from class PObject */
@@ -624,8 +628,8 @@ class PBYTEArray : public PBaseArray<BYTE>
       PBoolean dynamic = true ///< Buffer is copied and dynamically allocated.
     ) : ParentClass(buffer, length, dynamic) { }
 
-    PBYTEArray(PContainerReference & reference)
-      : ParentClass(reference) { }
+    PBYTEArray(PContainerReference & reference_)
+      : ParentClass(reference_) { }
   //@}
 
   /**@name Overrides from class PObject */
@@ -1143,27 +1147,6 @@ class PBitArray : public PBYTEArray
 };
 
 
-template <class ObjClass>
-class PPtrVector : public std::vector<ObjClass *>
-{
-  public:
-    typedef std::vector<ObjClass *> Parent;
-    virtual ~PPtrVector()
-    {
-      Clear();
-    }
-
-    void Clear()
-    {
-      while (Parent::size() > 0) {
-        delete Parent::front();
-        Parent::erase(Parent::begin());
-      }
-    }
-};
-
-
-
 #endif // PTLIB_ARRAY_H
 
 
