@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29415 $
+ * $Revision: 33966 $
  * $Author: rjongbloed $
- * $Date: 2013-04-02 16:41:13 -0500 (Tue, 02 Apr 2013) $
+ * $Date: 2015-08-30 11:57:15 +0100 (Sun, 30 Aug 2015) $
  */
 
 #ifndef PTLIB_CHANNEL_H
@@ -63,7 +63,7 @@ class PChannelStreamBuffer : public streambuf {
     virtual int_type overflow(int_type = EOF);
     virtual int_type underflow();
     virtual int sync();
-    virtual pos_type seekoff(off_type, ios_base::seekdir, ios_base::openmode = ios_base::in | ios_base::out);
+    virtual pos_type seekoff(std::streamoff, ios_base::seekdir, ios_base::openmode = ios_base::in | ios_base::out);
     virtual pos_type seekpos(pos_type, ios_base::openmode = ios_base::in | ios_base::out);
 
     PBoolean SetBufferSize(
@@ -104,7 +104,8 @@ class PChannelStreamBuffer : public streambuf {
    resource. The act of opening a channel is a key event that may fail. In this
    case the channel remains closed and error values are set.
  */
-class PChannel : public PObject, public P_IOSTREAM {
+class PChannel : public PObject, public std::iostream
+{
   PCLASSINFO(PChannel, PObject);
 
   /**@name Construction */
@@ -172,7 +173,14 @@ class PChannel : public PObject, public P_IOSTREAM {
        @return
        standard OS descriptor integer.
      */
-    P_INT_PTR GetHandle() const;
+    virtual P_INT_PTR GetHandle() const;
+
+    /** Re-open the device using the stdio library. This closes the PChannel
+
+       @return
+       stdio FILE * handle for the opened device.
+     */
+    FILE * FDOpen(const char * mode);
 
     /** Get the base channel of channel indirection using PIndirectChannel.
        This function returns the eventual base channel for reading of a series
@@ -269,18 +277,6 @@ class PChannel : public PObject, public P_IOSTREAM {
       Errors lastError,   ///< Error code to translate.
       int osError = 0     ///< OS error number to translate.
     );
-
-    /** Convert an operating system error into platform independent error.
-       This will set the lastError and osError member variables for access by
-       GetErrorCode() and GetErrorNumber().
-       
-       @return true if there was no error.
-     */
-    static PBoolean ConvertOSError(
-      P_INT_PTR libcReturnValue,
-      Errors & lastError,
-      int & osError
-    );
   //@}
 
   /**@name Reading functions */
@@ -335,12 +331,16 @@ class PChannel : public PObject, public P_IOSTREAM {
        @return
        the number of bytes read.
      */
-    virtual PINDEX GetLastReadCount() const;
+    PINDEX GetLastReadCount() const;
 
-    /** Read a single 8 bit byte from the channel. If one was not available
+    /** Read a single character from the channel. If one was not available
        within the read timeout period, or an I/O error occurred, then the
        function gives with a -1 return value.
 
+       Note, normally the character read will be an 8 bit value, but for some
+       derived classes, e.g. PConsoleChannel, this may be a code for special
+       input.
+
        @return
        byte read or -1 if no character could be read.
      */
@@ -470,14 +470,6 @@ class PChannel : public PObject, public P_IOSTREAM {
     #define PDECLARE_AsyncNotifier(cls, fn) PDECLARE_NOTIFIER2(PChannel, cls, fn, PChannel::AsyncContext &)
     #define PCREATE_AsyncNotifier(fn) PCREATE_NOTIFIER2(fn, PChannel::AsyncContext &)
 
-#ifdef _MSC_VER
-  #pragma pack(push)
-  #if P_64BIT
-    #pragma pack(8)
-  #else
-    #pragma pack(4)
-  #endif
-#endif
     /** Parameters for asynchronous I/O operation.
       */
 #ifdef _WIN32
@@ -510,13 +502,12 @@ class PChannel : public PObject, public P_IOSTREAM {
         // Internal stuff
         PChannel * m_channel;
         typedef void (PChannel::*CompletionFunction)(AsyncContext &);
-        CompletionFunction m_onComplete;
+        P_ALIGN_FIELD(CompletionFunction,m_onComplete,16);
         bool Initialise(PChannel * channel, CompletionFunction onComplete);
         void OnIOComplete(PINDEX length, int errorNumber);
+
+        PNEW_AND_DELETE_FUNCTIONS(16)
     };
-#ifdef _MSC_VER
-  #pragma pack(pop)
-#endif
 
     /** Begin an asynchronous read from channel. The read timeout is used as in
        other read operations, in this case calling the OnReadComplete()
@@ -718,10 +709,13 @@ class PChannel : public PObject, public P_IOSTREAM {
       PINDEX start
     );
 
+    bool CheckNotOpen();
+    virtual int os_errno() const;
 
     // Member variables
     /// The operating system file handle return by standard open() function.
     P_INT_PTR os_handle;
+
     /// The platform independant error code.
     Errors lastErrorCode[NumErrorGroups+1];
     /// The operating system error number (eg as returned by errno).
@@ -770,6 +764,41 @@ class PNullChannel : public PChannel
 };
 
 
+class P_fd_set
+{
+public:
+  P_fd_set();
+  P_fd_set(intptr_t fd);
+  ~P_fd_set()
+  {
+    free(set);
+  }
+
+  P_fd_set & operator=(intptr_t fd);
+  P_fd_set & operator+=(intptr_t fd);
+  P_fd_set & operator-=(intptr_t fd);
+
+  void Zero();
+
+  PBoolean IsPresent(intptr_t fd) const;
+
+  operator fd_set*() const
+  {
+    return set;
+  }
+
+protected:
+  void Construct();
+
+  intptr_t max_fd;
+  fd_set * set;
+
+private:
+  P_fd_set(const P_fd_set &) { }
+  void operator=(const P_fd_set &) { }
+};
+
+
 #endif // PTLIB_CHANNEL_H
 
 
