@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27066 $
+ * $Revision: 32816 $
  * $Author: rjongbloed $
- * $Date: 2012-02-28 02:23:49 -0600 (Tue, 28 Feb 2012) $
+ * $Date: 2014-09-25 04:59:23 +0100 (Thu, 25 Sep 2014) $
  */
 
 #ifndef PTLIB_CONTAIN_H
@@ -40,7 +40,7 @@
 
 
 #include <ptlib/object.h>
-#include <ptlib/critsec.h>
+#include <ptlib/atomic.h>
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -67,7 +67,7 @@ class PContainerReference
     }
 
     PINDEX         size;          // Size of what the container contains
-    PAtomicInteger count;         // reference count to the container content - guaranteed to be atomic
+    atomic<uint32_t> count;         // reference count to the container content - guaranteed to be atomic
     bool           deleteObjects; // Used by PCollection but put here for efficiency
     bool           constObject;   // Indicates object is constant/static, copy on write.
 
@@ -150,6 +150,7 @@ class PContainer : public PObject
        @return number of objects in container.
      */
     virtual PINDEX GetSize() const;
+    __inline size_t size() const { return GetSize(); }
 
     /**Set the new current size of the container.
        The exact behavious of this is determined by the descendent class. For
@@ -184,6 +185,7 @@ class PContainer : public PObject
        @return true if <code>GetSize()</code> returns zero.
      */
     virtual PBoolean IsEmpty() const;
+    __inline bool empty() const { return IsEmpty(); }
 
     /**Determine if container is unique reference.
        Determine if this instance is the one and only reference to the
@@ -489,6 +491,7 @@ class PCollection : public PContainer
     virtual PBoolean Remove(
       const PObject * obj   ///< Existing object to remove from the collection.
     ) = 0;
+    __inline void remove(const PObject * obj) { Remove(obj); }
 
     /**Remove the object at the specified ordinal index from the collection.
        If the AllowDeleteObjects option is set then the object is also deleted.
@@ -509,6 +512,7 @@ class PCollection : public PContainer
        <code>(GetSize()-1)</code> toward the first at index zero.
      */
     virtual void RemoveAll();
+    __inline void clear()  { RemoveAll(); }
 
     /**Set the object at the specified ordinal position to the new value. This
        will overwrite the existing entry. If the AllowDeleteObjects option is
