@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29567 $
+ * $Revision: 31645 $
  * $Author: rjongbloed $
- * $Date: 2013-04-24 22:31:57 -0500 (Wed, 24 Apr 2013) $
+ * $Date: 2014-04-01 01:10:17 +0100 (Tue, 01 Apr 2014) $
  */
 
 
@@ -158,29 +158,43 @@ typedef PKey<PINDEX> POrdinalKey;
 // Member variables
 struct PHashTableElement
 {
-    PObject * key;
-    PObject * data;
-    PHashTableElement * next;
-    PHashTableElement * prev;
-    PINDEX bucket;
+    PObject           * m_key;
+    PObject           * m_data;
+    PHashTableElement * m_next;
+    PHashTableElement * m_prev;
+    PINDEX              m_bucket;
 
     PDECLARE_POOL_ALLOCATOR();
 };
 
-class PHashTableInfo : public PBaseArray<PHashTableElement *>
+struct PHashTableList
 {
-    typedef PBaseArray<PHashTableElement *> ParentClass;
+  PHashTableList() : m_head(NULL), m_tail(NULL) { }
+  PHashTableElement * m_head;
+  PHashTableElement * m_tail;
+#if PTRACING
+  PINDEX              m_size;
+#endif
+};
+__inline std::ostream & operator<<(std::ostream & strm, const PHashTableList & hash) { return strm << (void *)hash.m_head; }
+
+class PHashTable;
+
+
+class PHashTableInfo : public PBaseArray<PHashTableList>
+{
+    typedef PBaseArray<PHashTableList> ParentClass;
     PCLASSINFO(PCharArray, ParentClass);
   public:
     PHashTableInfo(PINDEX initialSize = 0)
       : ParentClass(initialSize) { }
-    PHashTableInfo(PHashTableElement * const * buffer, PINDEX length, PBoolean dynamic = true)
+    PHashTableInfo(PHashTableList const * buffer, PINDEX length, PBoolean dynamic = true)
       : ParentClass(buffer, length, dynamic) { }
     virtual PObject * Clone() const { return PNEW PHashTableInfo(*this, GetSize()); }
     virtual ~PHashTableInfo() { Destruct(); }
     virtual void DestroyContents();
 
-    PINDEX AppendElement(PObject * key, PObject * data);
+    void AppendElement(PObject * key, PObject * data PTRACE_PARAM(, PHashTable * owner));
     PObject * RemoveElement(const PObject & key);
     PHashTableElement * GetElementAt(PINDEX index);
     PHashTableElement * GetElementAt(const PObject & key);
@@ -190,7 +204,6 @@ class PHashTableInfo : public PBaseArray<PHashTableElement *>
 
     PBoolean deleteKeys;
 
-  typedef PHashTableElement Element;
   friend class PHashTable;
   friend class PAbstractSet;
 };
@@ -304,9 +317,6 @@ class PHashTable : public PCollection
     ) const;
   //@}
 
-    // The type below cannot be nested as DevStudio 2005 AUTOEXP.DAT doesn't like it
-    typedef PHashTableElement Element;
-    typedef PHashTableInfo Table;
     PHashTableInfo * hashTable;
 };
 
@@ -636,7 +646,7 @@ template <class T> class PSet : public PAbstractSet
         void Next() { this->element = PAssertNULL(this->table)->NextElement(this->element); }
         void Prev() { this->element = PAssertNULL(this->table)->PrevElement(this->element); }
 
-        T * Ptr() const { return dynamic_cast<T *>(PAssertNULL(this->element)->key); }
+        T * Ptr() const { return dynamic_cast<T *>(PAssertNULL(this->element)->m_key); }
 
       public:
         bool operator==(const iterator_base & it) const { return this->element == it.element; }
@@ -1146,8 +1156,8 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
         {
           this->m_element = element;
           if (element != NULL) {
-            this->m_internal_first  = dynamic_cast<K *>(element->key);
-            this->m_internal_second = dynamic_cast<D *>(element->data);
+            this->m_internal_first  = dynamic_cast<K *>(element->m_key);
+            this->m_internal_second = dynamic_cast<D *>(element->m_data);
           }
           else {
             this->m_internal_first = NULL;
@@ -1223,8 +1233,8 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
     const_iterator end()   const { return const_iterator(); }
     const_iterator find(const K & k) const { return const_iterator(this, k); }
 
-    void erase(const       iterator & it) { this->AbstractSetAt(*it.m_element->key, NULL); }
-    void erase(const const_iterator & it) { this->AbstractSetAt(*it.m_element->key, NULL); }
+    void erase(const       iterator & it) { this->AbstractSetAt(*it.m_element->m_key, NULL); }
+    void erase(const const_iterator & it) { this->AbstractSetAt(*it.m_element->m_key, NULL); }
   //@}
 
   protected:
