@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27039 $
+ * $Revision: 33618 $
  * $Author: rjongbloed $
- * $Date: 2012-02-22 01:19:46 -0600 (Wed, 22 Feb 2012) $
+ * $Date: 2015-06-05 17:55:49 +0100 (Fri, 05 Jun 2015) $
  */
 
 
@@ -91,15 +91,16 @@ class PFile : public PChannel
           \arg \c WriteOnly <code>Create | Truncate</code>
           \arg \c ReadWrite <code>Create</code>
      */
-    P_DECLARE_BITWISE_ENUM_EX(OpenOptions, 6,
-        (MustExist,       ///< File open fails if file does not exist.
+    P_DECLARE_BITWISE_ENUM_EX(OpenOptions, 7,
+        (NoOptions,       ///< No options selected
+         MustExist,       ///< File open fails if file does not exist.
          Create,          ///< File is created if it does not exist.
          Truncate,        ///< File is set to zero length if it already exists.
          Exclusive,       ///< File open fails if file already exists.
          Temporary,       ///< File is temporary and is to be deleted when closed.
          DenySharedRead,  ///< File may not be read by another process.
          DenySharedWrite  ///< File may not be written by another process.
-        ), 
+        ),
         ModeDefault = -1  ///< File options depend on the OpenMode parameter.
     );
 
@@ -203,7 +204,7 @@ class PFile : public PChannel
        @return
        true if the file exists.
      */
-    static PBoolean Exists(
+    static bool Exists(
       const PFilePath & name  ///< Name of file to see if exists.
     );
 
@@ -214,7 +215,7 @@ class PFile : public PChannel
        @return
        true if the file exists.
      */
-    PBoolean Exists() const;
+    bool Exists() const;
 
     /**Check for file access modes.
        Determine if the file specified may be opened in the specified mode. This would
@@ -225,7 +226,7 @@ class PFile : public PChannel
        @return
        true if a file open would succeed.
      */
-    static PBoolean Access(
+    static bool Access(
       const PFilePath & name, ///< Name of file to have its access checked.
       OpenMode mode         ///< Mode in which the file open would be done.
     );
@@ -240,10 +241,40 @@ class PFile : public PChannel
        @return
        true if a file open would succeed.
      */
-    PBoolean Access(
+    bool Access(
       OpenMode mode         ///< Mode in which the file open would be done.
     );
 
+    /**Set access & modification times for file.
+       If accessTime or modTime is invalid then current time is used for that
+       time. If the modTime is not present it is set to the same as the accessTime.
+
+       @return
+       true if file times changed.
+     */
+    static bool Touch(
+      const PFilePath & name,       ///< Name of file to have its access time changed.
+      const PTime & accessTime = 0  ///< Time to set access time to.
+    );
+    static bool Touch(
+      const PFilePath & name,   ///< Name of file to have its access time changed.
+      const PTime & accessTime, ///< Time to set access time to.
+      const PTime & modTime     ///< Time to set modification time to.
+    );
+
+    /**Set access & modification times for file.
+
+       @return
+       true if file times changed.
+     */
+    bool Touch(
+      const PTime & accessTime = 0 ///< Time to set access time to.
+    );
+    bool Touch(
+      const PTime & accessTime,    ///< Time to set access time to.
+      const PTime & modTime        ///< Time to set modification time to.
+    );
+
     /**Delete the specified file. If <code>force</code> is false and the file
        is protected against being deleted then the function fails. If
        <code>force</code> is true then the protection is ignored. What
@@ -256,13 +287,13 @@ class PFile : public PChannel
        @return
        true if the file was deleted.
      */
-    static PBoolean Remove(
-      const PFilePath & name,   // Name of file to delete.
-      PBoolean force = false      // Force deletion even if file is protected.
+    static bool Remove(
+      const PFilePath & name,   ///< Name of file to delete.
+      bool force = false        ///< Force deletion even if file is protected.
     );
-    static PBoolean Remove(
-      const PString & name,   // Name of file to delete.
-      PBoolean force = false      // Force deletion even if file is protected.
+    static bool Remove(
+      const PString & name,   ///< Name of file to delete.
+      bool force = false      ///< Force deletion even if file is protected.
     );
 
     /**Delete the current file. If <code>force</code> is false and the file
@@ -277,8 +308,8 @@ class PFile : public PChannel
        @return
        true if the file was deleted.
      */
-    PBoolean Remove(
-      PBoolean force = false      // Force deletion even if file is protected.
+    bool Remove(
+      bool force = false      ///< Force deletion even if file is protected.
     );
 
     /**Change the specified files name. This does not move the file in the
@@ -296,11 +327,10 @@ class PFile : public PChannel
        @return
        true if the file was renamed.
      */
-    static PBoolean Rename(
+    static bool Rename(
       const PFilePath & oldname,  ///< Old name of the file.
       const PString & newname,    ///< New name for the file.
-      PBoolean force = false
-        ///< Delete file if a destination exists with the same name.
+      bool force = false          ///< Delete file if a destination exists with the same name.
     );
 
     /**Change the current files name.
@@ -319,10 +349,9 @@ class PFile : public PChannel
        @return
        true if the file was renamed.
      */
-    PBoolean Rename(
+    bool Rename(
       const PString & newname,  ///< New name for the file.
-      PBoolean force = false
-        ///< Delete file if a destination exists with the same name.
+      bool force = false        ///< Delete file if a destination exists with the same name.
     );
 
     /**Make a copy of the specified file.
@@ -330,11 +359,11 @@ class PFile : public PChannel
        @return
        true if the file was renamed.
      */
-    static PBoolean Copy(
+    static bool Copy(
       const PFilePath & oldname,  ///< Old name of the file.
       const PFilePath & newname,  ///< New name for the file.
-      PBoolean force = false
-        ///< Delete file if a destination exists with the same name.
+      bool force = false,         ///< Delete file if a destination exists with the same name.
+      bool recurse = false        ///< Recursively create all intermediate sub-directories
     );
 
     /**Make a copy of the current file.
@@ -342,10 +371,10 @@ class PFile : public PChannel
        @return
        true if the file was renamed.
      */
-    PBoolean Copy(
+    bool Copy(
       const PFilePath & newname,  ///< New name for the file.
-      PBoolean force = false
-        ///< Delete file if a destination exists with the same name.
+      bool force = false,         ///< Delete file if a destination exists with the same name.
+      bool recurse = false        ///< Recursively create all intermediate sub-directories
     );
 
     /**Move the specified file. This will move the file from one position in
@@ -357,11 +386,11 @@ class PFile : public PChannel
        @return
        true if the file was moved.
      */
-    static PBoolean Move(
+    static bool Move(
       const PFilePath & oldname,  ///< Old path and name of the file.
       const PFilePath & newname,  ///< New path and name for the file.
-      PBoolean force = false
-        ///< Delete file if a destination exists with the same name.
+      bool force = false,         ///< Delete file if a destination exists with the same name.
+      bool recurse = false        ///< Recursively create all intermediate sub-directories
     );
 
     /**Move the current file. This will move the file from one position in
@@ -373,10 +402,10 @@ class PFile : public PChannel
        @return
        true if the file was moved.
      */
-    PBoolean Move(
+    bool Move(
       const PFilePath & newname,  ///< New path and name for the file.
-      PBoolean force = false
-        ///< Delete file if a destination exists with the same name.
+      bool force = false,         ///< Delete file if a destination exists with the same name.
+      bool recurse = false        ///< Recursively create all intermediate sub-directories
     );
   //@}
 
@@ -409,27 +438,51 @@ class PFile : public PChannel
        @return
        true if the file was successfully opened.
      */
-    virtual PBoolean Open(
-      OpenMode mode = ReadWrite,  // Mode in which to open the file.
-      OpenOptions opts = ModeDefault      // Options for open operation.
+    bool Open(
+      OpenMode mode = ReadWrite,        ///< Mode in which to open the file.
+      OpenOptions opts = ModeDefault    ///< Options for open operation.
+    );
+
+    /**Open the specified file name in the specified mode and with
+       the specified options and permissions. If the file object already has an open file then
+       it is closed.
+       
+       @return
+       true if the file was successfully opened.
+     */
+    bool Open(
+      OpenMode mode,                      ///< Mode in which to open the file.
+      OpenOptions opts,                   ///< <code>OpenOptions</code> enum# for open operation.
+      PFileInfo::Permissions permissions  ///< Permission for file if created
     );
 
     /**Open the specified file name in the specified mode and with
        the specified options. If the file object already has an open file then
        it is closed.
        
-       Note: if <code>mode</code> is StandardInput, StandardOutput or StandardError,
-       then the <code>name</code> parameter is ignored.
-
        @return
        true if the file was successfully opened.
      */
-    virtual PBoolean Open(
-      const PFilePath & name,    // Name of file to open.
-      OpenMode mode = ReadWrite, // Mode in which to open the file.
-      OpenOptions opts = ModeDefault     // <code>OpenOptions</code> enum# for open operation.
+    bool Open(
+      const PFilePath & name,         ///< Name of file to open.
+      OpenMode mode = ReadWrite,      ///< Mode in which to open the file.
+      OpenOptions opts = ModeDefault  ///< <code>OpenOptions</code> enum# for open operation.
     );
-      
+
+    /**Open the specified file name in the specified mode and with
+       the specified options and permissions. If the file object already has an open file then
+       it is closed.
+       
+       @return
+       true if the file was successfully opened.
+     */
+    bool Open(
+      const PFilePath & name,             ///< Name of file to open.
+      OpenMode mode,                      ///< Mode in which to open the file.
+      OpenOptions opts,                   ///< <code>OpenOptions</code> enum# for open operation.
+      PFileInfo::Permissions permissions  ///< Permission for file if created
+    );
+
     /**Get the current size of the file.
 
        @return
@@ -449,12 +502,9 @@ class PFile : public PChannel
 
     /// Options for the origin in setting the file position.
     enum FilePositionOrigin {
-      /// Set position relative to start of file.
-      Start = SEEK_SET,   
-      /// Set position relative to current file position.
-      Current = SEEK_CUR, 
-      /// Set position relative to end of file.
-      End = SEEK_END      
+      Start = SEEK_SET,    ///< Set position relative to start of file.
+      Current = SEEK_CUR,  ///< Set position relative to current file position.
+      End = SEEK_END       ///< Set position relative to end of file.
     };
 
     /**Set the current active position in the file for the next read or write
@@ -486,17 +536,16 @@ class PFile : public PChannel
        @return
        true if at end of file.
      */
-    PBoolean IsEndOfFile() const;
+    bool IsEndOfFile() const;
       
     /**Get information (eg protection, timestamps) on the specified file.
 
        @return
        true if the file info was retrieved.
      */
-    static PBoolean GetInfo(
-      const PFilePath & name,  // Name of file to get the information on.
-      PFileInfo & info
-      // <code>PFileInfo</code> structure to receive the information.
+    static bool GetInfo(
+      const PFilePath & name,  ///< Name of file to get the information on.
+      PFileInfo & info         ///< <code>PFileInfo</code> structure to receive the information.
     );
 
     /**Get information (eg protection, timestamps) on the current file.
@@ -504,9 +553,8 @@ class PFile : public PChannel
        @return
        true if the file info was retrieved.
      */
-    PBoolean GetInfo(
-      PFileInfo & info
-      // <code>PFileInfo</code> structure to receive the information.
+    bool GetInfo(
+      PFileInfo & info ///< <code>PFileInfo</code> structure to receive the information.
     );
 
     /**Set permissions on the specified file.
@@ -514,25 +562,27 @@ class PFile : public PChannel
        @return
        true if the file was renamed.
      */
-    static PBoolean SetPermissions(
-      const PFilePath & name,   // Name of file to change the permission of.
-      int permissions           // New permissions mask for the file.
+    static bool SetPermissions(
+      const PFilePath & name,             ///< Name of file to change the permission of.
+      PFileInfo::Permissions permissions  ///< New permissions mask for the file.
     );
     /**Set permissions on the current file.
 
        @return
        true if the file was renamed.
      */
-    PBoolean SetPermissions(
-      int permissions           // New permissions mask for the file.
+    bool SetPermissions(
+      PFileInfo::Permissions permissions  ///< New permissions mask for the file.
     );
   //@}
 
   protected:
+    virtual bool InternalOpen(OpenMode mode, OpenOptions opts, PFileInfo::Permissions permissions);
+
     // Member variables
 
-    PFilePath path;         ///< The fully qualified path name for the file.
-    PBoolean removeOnClose; ///< File is to be removed when closed.
+    PFilePath m_path;         ///< The fully qualified path name for the file.
+    bool      m_removeOnClose; ///< File is to be removed when closed.
 
 
 // Include platform dependent part of class
