@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28062 $
+ * $Revision: 31101 $
  * $Author: rjongbloed $
- * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
+ * $Date: 2013-12-18 01:36:42 +0000 (Wed, 18 Dec 2013) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -37,11 +37,6 @@
   public:
     ~PConsoleChannel();
 
-    // Overrides from class PChannel
-    virtual PString GetName() const;
-      // Return the name of the channel.
-
-
     virtual PBoolean Read(void * buf, PINDEX len);
       // Low level read from the channel. This function will block until the
       // requested number of characters were read.
@@ -50,10 +45,9 @@
       // Low level write to the channel. This function will block until the
       // requested number of characters were written.
 
-    virtual PBoolean Close();
-      // Close the channel.
-
   protected:
+    bool InternalSetConsoleMode(DWORD bit, bool on);
     PWin32Handle m_hConsole;
+    DWORD m_lastMouseState;
 
 // End Of File ///////////////////////////////////////////////////////////////
