@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23677 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2009-10-16 07:00:31 +0100 (Fri, 16 Oct 2009) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -36,7 +36,7 @@
 
   protected:
 #if defined(_WINDOWS) || defined(_WIN32)
-    HINSTANCE _hDLL;
+    HINSTANCE m_hDLL;
 #endif
 
 // End Of File ///////////////////////////////////////////////////////////////
