@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 31371 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2014-02-06 02:34:49 +0000 (Thu, 06 Feb 2014) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -37,7 +37,7 @@
   protected:
     HANDLE hFindFile;
     WIN32_FIND_DATA fileinfo;
-    PBoolean Filtered();
+    bool InternalEntryCheck();
 
   public:
     static PString CreateFullPath(const PString & path, PBoolean isDirectory);
