@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28062 $
+ * $Revision: 29288 $
  * $Author: rjongbloed $
- * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
+ * $Date: 2013-03-21 08:05:43 +0000 (Thu, 21 Mar 2013) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -36,12 +36,9 @@
 
   public:
     HANDLE GetHandle() const { return m_handle; }
-    unsigned GetInitialVal() const  { return m_initialVal; }
-    unsigned GetMaxCountVal() const { return m_maxCountVal; }
 
   protected:
-    unsigned     m_maxCountVal;
-    unsigned     m_initialVal;
     PWin32Handle m_handle;
 
+
 // End Of File ///////////////////////////////////////////////////////////////
