@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28941 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 32281 $
+ * $Author: rjongbloed $
+ * $Date: 2014-07-02 05:39:38 +0100 (Wed, 02 Jul 2014) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -45,8 +45,6 @@
   protected:
     virtual HANDLE GetAsyncReadHandle() const;
     virtual HANDLE GetAsyncWriteHandle() const;
-    PBoolean ConvertOSError(P_INT_PTR status, ErrorGroup group = LastGeneralError);
-    static PBoolean ConvertOSError(P_INT_PTR error, Errors & lastError, int & osError);
 
 
 // End Of File ///////////////////////////////////////////////////////////////
