@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28941 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 31816 $
+ * $Author: rjongbloed $
+ * $Date: 2014-04-30 06:55:39 +0100 (Wed, 30 Apr 2014) $
  */
 
 
@@ -130,22 +130,12 @@ PARRAY(PWaveBufferArray, PWaveBuffer);
 class PSoundChannelWin32: public PSoundChannel
 {
  public:
+    static const char * GetDriverName();
     PSoundChannelWin32();
-    void Construct();
-    PSoundChannelWin32(const PString &device,
-                     PSoundChannel::Directions dir,
-                     unsigned numChannels,
-                     unsigned sampleRate,
-                     unsigned bitsPerSample);
     ~PSoundChannelWin32();
+    static PString GetDefaultDevice(Directions dir);
     static PStringArray GetDeviceNames(PSoundChannel::Directions = Player);
-    PBoolean Open(
-      const PString & device,
-      Directions dir,
-      unsigned numChannels,
-      unsigned sampleRate,
-      unsigned bitsPerSample
-    );
+    bool Open(const Params & params);
     PBoolean Setup();
     PBoolean Close();
     PBoolean IsOpen() const;
@@ -160,7 +150,6 @@ class PSoundChannelWin32: public PSoundChannel
     PBoolean SetBuffers(PINDEX size, PINDEX count);
     PBoolean GetBuffers(PINDEX & size, PINDEX & count);
     PBoolean PlaySound(const PSound & sound, PBoolean wait);
-    PBoolean PlayFile(const PFilePath & filename, PBoolean wait);
     PBoolean HasPlayCompleted();
     PBoolean WaitForPlayCompletion();
     PBoolean RecordSound(PSound & sound);
@@ -192,7 +181,6 @@ class PSoundChannelWin32: public PSoundChannel
 
   protected:
     PString      deviceName;
-    Directions   direction;
     HWAVEIN      hWaveIn;
     HWAVEOUT     hWaveOut;
     HMIXER       hMixer;
@@ -201,6 +189,7 @@ class PSoundChannelWin32: public PSoundChannel
     HANDLE       hEventDone;
     PWaveFormat  waveFormat;
     bool         opened;
+    bool         m_reopened;
 
     PWaveBufferArray buffers;
     PINDEX           bufferIndex;
@@ -210,6 +199,7 @@ class PSoundChannelWin32: public PSoundChannel
   private:
     PBoolean OpenDevice(P_INT_PTR id);
     PBoolean GetDeviceID(const PString & device, Directions dir, unsigned& id);
+    int WaitEvent(ErrorGroup group);
 };
 
 
