@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29306 $
+ * $Revision: 31385 $
  * $Author: rjongbloed $
- * $Date: 2013-03-24 23:14:19 -0500 (Sun, 24 Mar 2013) $
+ * $Date: 2014-02-06 04:49:00 +0000 (Thu, 06 Feb 2014) $
  */
 
 
@@ -53,6 +53,9 @@
 
     virtual PBoolean IsServiceProcess() const;
 
+    void SetDescription(const PString & description);
+    const PString & GetDescription() const;
+
   protected:
     bool m_debugHidden; /// Flag to indicate service is run in simulation mode without showing the control window 
 
@@ -122,5 +125,7 @@
     HWND                  m_controlWindow;
     HWND                  m_debugWindow;
 
+    PString               m_description;
+
 
 // End Of File ///////////////////////////////////////////////////////////////
