@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27635 $
+ * $Revision: 34426 $
  * $Author: rjongbloed $
- * $Date: 2012-05-15 01:14:08 -0500 (Tue, 15 May 2012) $
+ * $Date: 2015-12-13 16:37:41 +0000 (Sun, 13 Dec 2015) $
  */
 
 #ifndef PTLIB_FACTORY_H
@@ -163,7 +163,14 @@ class PFactoryTemplate : public PFactoryBase
     typedef ParamType                      Param_T;
     typedef KeyType                        Key_T;
     typedef std::vector<Key_T>             KeyList_T;
-    typedef std::map<Key_T, WorkerBase*>   WorkerMap_T;
+
+    struct WorkerWrap
+    {
+      WorkerWrap(WorkerBase * worker, bool autoDelete) : m_worker(worker), m_autoDelete(autoDelete) { }
+      WorkerBase * m_worker;
+      bool         m_autoDelete;
+    };
+    typedef std::map<Key_T, WorkerWrap>    WorkerMap_T;
     typedef typename WorkerMap_T::iterator WorkerIter_T;
 
     class WorkerBase
@@ -221,12 +228,14 @@ class PFactoryTemplate : public PFactoryBase
 
         Types        m_type;
         Abstract_T * m_singletonInstance;
+
+      friend class PFactoryTemplate;
     };
 
     virtual void DestroySingletons()
     {
       for (WorkerIter_T it = m_workers.begin(); it != m_workers.end(); ++it)
-        it->second->DestroySingleton();
+        it->second.m_worker->DestroySingleton();
     }
 
   protected:
@@ -236,44 +245,58 @@ class PFactoryTemplate : public PFactoryBase
     ~PFactoryTemplate()
     {
       DestroySingletons();
+      InternalUnregisterAll();
     }
 
-    bool InternalRegister(const Key_T & key, WorkerBase * worker)
+    bool InternalRegister(const Key_T & key, WorkerBase * worker, bool autoDeleteWorker)
     {
       PWaitAndSignal mutex(m_mutex);
-      if (m_workers.find(key) != m_workers.end())
-        return false;
+      typename WorkerMap_T::iterator it = m_workers.find(key);
+      if (it != m_workers.end()) {
+        return it->second.m_worker == worker;
+      }
 
       PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
-      m_workers[key] = PAssertNULL(worker);
+      m_workers.insert(make_pair(key, WorkerWrap(PAssertNULL(worker), autoDeleteWorker)));
       return true;
     }
 
     bool InternalRegister(const Key_T & key, Abstract_T * instance, bool autoDeleteInstance)
     {
       PWaitAndSignal mutex(m_mutex);
-      if (m_workers.find(key) != m_workers.end())
-        return false;
+      typename WorkerMap_T::iterator it = m_workers.find(key);
+      if (it != m_workers.end())
+        return it->second.m_worker->m_singletonInstance == instance;
 
       PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
-      m_workers[key] = PNEW WorkerBase(instance, autoDeleteInstance);
+      m_workers.insert(make_pair(key, WorkerWrap(PNEW WorkerBase(instance, autoDeleteInstance), true)));
       return true;
     }
 
     PBoolean InternalRegisterAs(const Key_T & newKey, const Key_T & oldKey)
     {
       PWaitAndSignal mutex(m_mutex);
-      if (m_workers.find(oldKey) == m_workers.end())
+      typename WorkerMap_T::iterator itOld = m_workers.find(oldKey);
+      if (itOld == m_workers.end())
         return false;
 
-      m_workers[newKey] = m_workers[oldKey];
+      typename WorkerMap_T::iterator itNew = m_workers.find(newKey);
+      if (itNew != m_workers.end())
+        return itNew->second.m_worker == itOld->second.m_worker;
+
+      m_workers.insert(make_pair(newKey, WorkerWrap(itOld->second.m_worker, false)));
       return true;
     }
 
     void InternalUnregister(const Key_T & key)
     {
       m_mutex.Wait();
-      m_workers.erase(key);
+      typename WorkerMap_T::iterator it = m_workers.find(key);
+      if (it != m_workers.end()) {
+        if (it->second.m_autoDelete)
+          delete it->second.m_worker;
+        m_workers.erase(it);
+      }
       m_mutex.Signal();
     }
 
@@ -282,7 +305,7 @@ class PFactoryTemplate : public PFactoryBase
       m_mutex.Wait();
       for (WorkerIter_T it = m_workers.begin(); it != m_workers.end(); ++it) {
         if (it->second == instance) {
-          m_workers.erase(it);
+          InternalUnregister(it->first);
           break;
         }
       }
@@ -292,6 +315,10 @@ class PFactoryTemplate : public PFactoryBase
     void InternalUnregisterAll()
     {
       m_mutex.Wait();
+      for (typename WorkerMap_T::iterator it = m_workers.begin(); it != m_workers.end(); ++it) {
+        if (it->second.m_autoDelete)
+          delete it->second.m_worker;
+      }
       m_workers.clear();
       m_mutex.Signal();
     }
@@ -306,14 +333,14 @@ class PFactoryTemplate : public PFactoryBase
     {
       PWaitAndSignal mutex(m_mutex);
       WorkerIter_T entry = m_workers.find(key);
-      return entry == m_workers.end() ? NULL : entry->second->CreateInstance(param);
+      return entry == m_workers.end() ? NULL : entry->second.m_worker->CreateInstance(param);
     }
 
     void InternalDestroy(const Key_T & key, Abstract_T * instance)
     {
       PWaitAndSignal mutex(m_mutex);
       WorkerIter_T entry = m_workers.find(key);
-      if (entry != m_workers.end() && !entry->second->IsSingleton())
+      if (entry != m_workers.end() && !entry->second.m_worker->IsSingleton())
         delete instance;
     }
 
@@ -321,7 +348,7 @@ class PFactoryTemplate : public PFactoryBase
     {
       PWaitAndSignal mutex(m_mutex);
       WorkerIter_T entry = m_workers.find(key);
-      return entry != m_workers.end() && entry->second->IsSingleton();
+      return entry != m_workers.end() && entry->second.m_worker->IsSingleton();
     }
 
     KeyList_T InternalGetKeyList()
@@ -350,7 +377,7 @@ typedef std::string PDefaultPFactoryKey;
 
 #define PFACTORY_STATICS(cls) \
   static cls & GetFactory()                                            { return PFactoryBase::GetFactoryAs<cls>(); } \
-  static bool Register(const Key_T & k, WorkerBase_T * w)              { return GetFactory().InternalRegister(k, w); } \
+  static bool Register(const Key_T & k, WorkerBase_T *w, bool a=false) { return GetFactory().InternalRegister(k, w, a); } \
   static bool Register(const Key_T & k, Abstract_T * i, bool a = true) { return GetFactory().InternalRegister(k, i, a); } \
   static bool RegisterAs(const Key_T & synonym, const Key_T & original){ return GetFactory().InternalRegisterAs(synonym, original); } \
   static void Unregister(const Key_T & k)                              {        GetFactory().InternalUnregister(k); } \
@@ -380,13 +407,23 @@ class PFactory : public PFactoryTemplate<AbstractClass, const KeyType &, KeyType
     template <class ConcreteClass>
     class Worker : protected WorkerBase_T
     {
+      private:
+        Key_T m_key;
       public:
         Worker(const Key_T & key, bool singleton = false)
           : WorkerBase_T(singleton)
+          , m_key(key)
         {
-          Register(key, this);
+          PAssert(Register(key, this), "Factory Worker already registered");
         }
 
+        ~Worker()
+        {
+          Unregister(m_key);
+        }
+
+        const Key_T & GetKey() const { return m_key; }
+
       protected:
         virtual Abstract_T * Create(Param_T) const
         {
@@ -431,13 +468,23 @@ class PParamFactory : public PFactoryTemplate<AbstractClass, ParamType, KeyType>
     template <class ConcreteClass>
     class Worker : protected WorkerBase_T
     {
+      private:
+        Key_T m_key;
       public:
         Worker(const Key_T & key, bool singleton = false)
           : WorkerBase_T(singleton)
+          , m_key(key)
         {
-          Register(key, this);
+          PAssert(Register(key, this), "Factory Worker already registered");
         }
 
+        ~Worker()
+        {
+          Unregister(m_key);
+        }
+
+        const Key_T & GetKey() const { return m_key; }
+
       protected:
         virtual Abstract_T * Create(Param_T param) const
         {
@@ -476,10 +523,15 @@ class PParamFactory : public PFactoryTemplate<AbstractClass, ParamType, KeyType>
   */
 #define PFACTORY_CREATE(factory, ConcreteClass, ...) \
   namespace PFactoryLoader { \
-    int ConcreteClass##_link() { return 0; } \
+    int ConcreteClass##_link(int const *) { return 0; } \
     factory::Worker<ConcreteClass> ConcreteClass##_instance(__VA_ARGS__); \
   }
 
+#define PFACTORY_SYNONYM(factory, ConcreteClass, name, key) \
+  namespace PFactoryLoader { \
+    bool ConcreteClass##name##_synonym = factory::RegisterAs(key, ConcreteClass##_instance.GetKey()); \
+  }
+
 #define PFACTORY_CREATE_SINGLETON(factory, ConcreteClass) \
         PFACTORY_CREATE(factory, ConcreteClass, typeid(ConcreteClass).name(), true)
 
@@ -496,8 +548,8 @@ class PParamFactory : public PFactoryTemplate<AbstractClass, ParamType, KeyType>
  */
 #define PFACTORY_LOAD(ConcreteType) \
   namespace PFactoryLoader { \
-    extern int ConcreteType##_link(); \
-    int const ConcreteType##_loader = ConcreteType##_link(); \
+    extern int ConcreteType##_link(int const *); \
+    static int const ConcreteType##_loader = ConcreteType##_link(&ConcreteType##_loader); \
   }
 
 
