@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29834 $
+ * $Revision: 34199 $
  * $Author: rjongbloed $
- * $Date: 2013-05-30 01:12:29 -0500 (Thu, 30 May 2013) $
+ * $Date: 2015-10-18 15:10:29 +0100 (Sun, 18 Oct 2015) $
  */
 
 #ifndef PTLIB_PROCESS_H
@@ -43,8 +43,6 @@
 #include <ptlib/thread.h>
 #include <ptlib/pfactory.h>
 
-#include <queue>
-#include <set>
 
 /**Create a process.
    This macro is used to create the components necessary for a user PWLib
@@ -102,111 +100,6 @@ extern "C" {\
       virtual void Main(); \
   };
 
-
-class PTimerList : public PObject
-/* This class defines a list of <code>PTimer</code> objects. It is primarily used
-   internally by the library and the user should never create an instance of
-   it. The <code>PProcess</code> instance for the application maintains an instance
-   of all of the timers created so that it may decrements them at regular
-   intervals.
- */
-{
-  PCLASSINFO(PTimerList, PObject);
-
-  public:
-    // Create a new timer list
-    PTimerList();
-
-    /* Decrement all the created timers and dispatch to their callback
-       functions if they have expired. The <code>PTimer::Tick()</code> function
-       value is used to determine the time elapsed since the last call to
-       Process().
-
-       The return value is the number of milliseconds until the next timer
-       needs to be despatched. The function need not be called again for this
-       amount of time, though it can (and usually is).
-       
-       @return
-       maximum time interval before function should be called again.
-     */
-    PTimeInterval Process();
-
-    PTimer::IDType GetNewTimerId() const { return ++timerId; }
-
-    class RequestType {
-      public:
-        enum Action {
-          Stop,
-          Start,
-          Pause
-        } m_action;
-
-        RequestType(Action act, PTimer * t)
-          : m_action(act)
-          , m_timer(t)
-          , m_id(t->GetTimerId())
-          , m_absoluteTime(t->GetAbsoluteTime())
-          , m_serialNumber(t->GetNextSerialNumber())
-          , m_sync(NULL)
-        { }
-
-        PTimer *                    m_timer;
-        PTimer::IDType              m_id;
-        PInt64                      m_absoluteTime;
-        PAtomicInteger::IntegerType m_serialNumber;
-        PSyncPoint *                m_sync;
-    };
-
-    void QueueRequest(RequestType::Action action, PTimer * timer, bool isSync = true);
-
-    void ProcessTimerQueue();
-
-  private:
-    // queue of timer action requests
-    PMutex m_queueMutex;
-    typedef std::queue<RequestType> RequestQueueType;
-    RequestQueueType m_requestQueue;
-
-    //  counter to keep track of timer IDs
-    mutable PAtomicInteger timerId; 
-
-    // map used to store active timer information
-    struct ActiveTimerInfo {
-      ActiveTimerInfo(PTimer * t, PAtomicInteger::IntegerType serialNumber) 
-        : m_timer(t), m_serialNumber(serialNumber) { }
-      PTimer * m_timer;
-      PAtomicInteger::IntegerType m_serialNumber;
-    };
-    typedef std::map<PTimer::IDType, ActiveTimerInfo> ActiveTimerInfoMap;
-    ActiveTimerInfoMap m_activeTimers;
-
-    // set used to store timer expiry times, in order
-    struct TimerExpiryInfo {
-      TimerExpiryInfo(PTimer::IDType id, PInt64 expireTime, PAtomicInteger::IntegerType serialNumber)
-        : m_timerId(id), m_expireTime(expireTime), m_serialNumber(serialNumber) { }
-      PTimer::IDType m_timerId;
-      PInt64 m_expireTime;
-      PAtomicInteger::IntegerType m_serialNumber;
-    };
-
-	  struct TimerExpiryInfo_compare
-		  : public binary_function<TimerExpiryInfo, TimerExpiryInfo, bool>
-	  {	
-	    bool operator()(const TimerExpiryInfo & _Left, const TimerExpiryInfo & _Right) const
-		  {	return (_Left.m_expireTime < _Right.m_expireTime); }
-	  };
-
-    typedef std::multiset<TimerExpiryInfo, TimerExpiryInfo_compare> TimerExpiryInfoList;
-    TimerExpiryInfoList m_expiryList;
-
-    // The last system timer tick value that was used to process timers.
-    PTimeInterval m_lastSample;
-
-    // thread that handles the timer stuff
-    PThread * m_timerThread;
-};
-
-
 ///////////////////////////////////////////////////////////////////////////////
 // PProcess
 
@@ -245,10 +138,10 @@ class PProcess : public PThread
     PProcess(
       const char * manuf = "",         ///< Name of manufacturer
       const char * name = "",          ///< Name of product
-      WORD majorVersion = 1,           ///< Major version number of the product
-      WORD minorVersion = 0,           ///< Minor version number of the product
+      unsigned majorVersion = 1,       ///< Major version number of the product
+      unsigned minorVersion = 0,       ///< Minor version number of the product
       CodeStatus status = ReleaseCode, ///< Development status of the product
-      WORD buildNumber = 1,            ///< Build number of the product
+      unsigned buildNumber = 1,        ///< Build number of the product
       bool library = false,            ///< PProcess is a library rather than an application
       bool suppressStartup = false     ///< Do not execute Startup()
     );
@@ -272,22 +165,6 @@ class PProcess : public PThread
     /**Terminate the process. Usually only used in abnormal abort situation.
      */
     virtual void Terminate();
-
-    /** Get the name of the thread. Thread names are a optional debugging aid.
-
-       @return
-       current thread name.
-     */
-    virtual PString GetThreadName() const;
-
-    /** Change the name of the thread. Thread names are a optional debugging aid.
-
-       @return
-       current thread name.
-     */
-    virtual void SetThreadName(
-      const PString & name        ///< New name for the thread.
-    );
   //@}
 
   /**@name Process information functions */
@@ -436,10 +313,51 @@ class PProcess : public PThread
      */
     static PProcessIdentifier GetCurrentProcessID();
 
+    /**Indicate if this is the second, or more, instance of this executable.
+      */
+    bool IsMultipleInstance() const;
+
     /**Return the time at which the program was started 
     */
     PTime GetStartTime() const;
 
+    struct MemoryUsage {
+      MemoryUsage()
+        : m_virtual(0)
+        , m_resident(0)
+        , m_max(0)
+        , m_current(0)
+        , m_blocks(0)
+        { }
+
+      size_t m_virtual;
+      size_t m_resident;
+      size_t m_max;
+      size_t m_current;
+      size_t m_blocks;
+    };
+
+    /**Get process memory suage.
+      */
+    void GetMemoryUsage(
+      MemoryUsage & usage
+    );
+
+    PPROFILE_EXCLUDE(
+    /** Get the process execution times.
+     */
+    bool GetProcessTimes(
+      Times & times   ///< Times for thread execution.
+    ) const
+    );
+
+    PPROFILE_EXCLUDE(
+    /** Get the system execution times.
+     */
+    static bool GetSystemTimes(
+      Times & times   ///< Times for thread execution.
+    ));
+
     /**Get the effective user name of the owner of the process, eg "root" etc.
        This is a platform dependent string only provided by platforms that are
        multi-user. Note that some value may be returned as a "simulated" user.
@@ -551,8 +469,9 @@ class PProcess : public PThread
     /**Set the default file or set of directories to search for use in PConfig.
        To find the .ini file for use in the default PConfig() instance, this
        explicit filename is used, or if it is a set of directories separated
-       by either ':' or ';' characters, then the application base name postfixed
-       with ".ini" is searched for through those directories.
+       by either ':' or ';' characters, depending on platform, then the
+       application base name postfixed with ".ini" is searched for through
+       those directories.
 
        The search is actually done when the GetConfigurationFile() is called,
        this function only sets the internal variable.
@@ -629,14 +548,6 @@ class PProcess : public PThread
     static PString GetLibVersion();
   //@}
 
-    /**Get the list of timers handled by the application. This is an internal
-       function and should not need to be called by the user.
-       
-       @return
-       list of timers.
-     */
-    PTimerList * GetTimerList();
-
     /**Internal initialisation function called directly from
        <code>InternalMain()</code>. The user should never call this function.
      */
@@ -720,10 +631,10 @@ class PProcess : public PThread
     PString manufacturer;             // Application manufacturer name.
     PString productName;              // Application executable base name from argv[0]
 
-    WORD       majorVersion;          // Major version number of the product
-    WORD       minorVersion;          // Minor version number of the product
+    unsigned   majorVersion;          // Major version number of the product
+    unsigned   minorVersion;          // Minor version number of the product
     CodeStatus status;                // Development status of the product
-    WORD       buildNumber;           // Build number of the product
+    unsigned   buildNumber;           // Build number of the product
 
     PFilePath    executableFile;      // Application executable file from argv[0] (not open)
     PStringArray configurationPaths;  // Explicit file or set of directories to find default PConfig
@@ -733,7 +644,7 @@ class PProcess : public PThread
     PTime programStartTime;           // time at which process was intantiated, i.e. started
 
     bool   m_shuttingDown;
-    PMutex m_threadMutex;
+    PCriticalSection m_threadMutex;
 
     typedef std::map<PThreadIdentifier, PThread *> ThreadMap;
     ThreadMap m_activeThreads;
@@ -741,16 +652,18 @@ class PProcess : public PThread
     void InternalThreadEnded(PThread * thread);
     
     typedef PList<PThread> ThreadList;
-    ThreadList m_autoDeleteThreads;
-    void InternalSetAutoDeleteThread(PThread * thread);
-    void InternalCleanAutoDeleteThreads();
+    ThreadList m_externalThreads;
+    PSyncQueue<PThread*> m_autoDeleteThreads;
 
-    PAtomicBoolean m_keepingHouse;
+    atomic<bool>   m_keepingHouse;
     PThread      * m_houseKeeper; // Thread for doing timers, thread clean up etc.
     PSyncPoint     m_signalHouseKeeper;
     void HouseKeeping();
 
-    PTimerList m_timers;
+#if P_TIMERS
+    PTimer::List * m_timerList;
+    friend PTimer::List * PTimer::TimerList();
+#endif
 
     PProcessIdentifier m_processID;
 
@@ -778,13 +691,14 @@ class PProcess : public PThread
     /** Create a new process instance.
      */
     PLibraryProcess(
-      const char * manuf = "",         ///< Name of manufacturer
-      const char * name = "",          ///< Name of product
-      WORD majorVersionNum = 1,           ///< Major version number of the product
-      WORD minorVersionNum = 0,           ///< Minor version number of the product
+      const char * manuf = "",             ///< Name of manufacturer
+      const char * name = "",              ///< Name of product
+      unsigned majorVersionNum = 1,        ///< Major version number of the product
+      unsigned minorVersionNum = 0,        ///< Minor version number of the product
       CodeStatus statusCode = ReleaseCode, ///< Development status of the product
-      WORD buildNum = 1             ///< Build number of the product
-    ) : PProcess(manuf, name, majorVersionNum, minorVersionNum, statusCode, buildNum, true) { }
+      unsigned buildNum = 1,               ///< Build number of the product
+      bool suppressStartup = false         ///< Do not execute Startup()
+    ) : PProcess(manuf, name, majorVersionNum, minorVersionNum, statusCode, buildNum, true, suppressStartup) { }
   //@}
 
     ///< Dummy Main() as libraries do not have one.
