@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26284 $
+ * $Revision: 34473 $
  * $Author: rjongbloed $
- * $Date: 2011-08-08 02:55:05 -0500 (Mon, 08 Aug 2011) $
+ * $Date: 2015-12-31 10:31:33 +0000 (Thu, 31 Dec 2015) $
  */
 
 #ifndef PTLIB_TIME_H
@@ -79,8 +79,8 @@ class PTime : public PObject
      */
     PTime(
       time_t tsecs,     ///< Time in seconds since 00:00:00 1/1/70 UTC
-      long usecs = 0    ///< microseconds part of time.
-    ) { theTime = tsecs; microseconds = usecs; }
+      int64_t usecs = 0    ///< microseconds part of time.
+    );
 
     /**Create a time object instance.
        This initialises the time to the specified time, parsed from the
@@ -114,6 +114,9 @@ class PTime : public PObject
       int year,             ///< Year from 1970 to 2038
       int tz = Local        ///< local time or UTC
     );
+
+    PTime(const PTime & other);
+    PTime & operator=(const PTime & other);
   //@}
 
   /**@name Overrides from class PObject */
@@ -136,8 +139,11 @@ class PTime : public PObject
       const PObject & obj   ///< Other time to compare against.
     ) const;
 
-    /**Output the time to the stream. This uses the <code>AsString()</code> function
-       with the <code>ShortDateTime</code> parameter.
+    /**Output the time to the stream.
+       This uses the <code>AsString()</code> function with the default
+       <code>RFC1123</code> parameter.
+       The strm.precision(), if negative may be used to change this to a value
+       from the TimeFormat enum.
      */
     virtual void PrintOn(
       ostream & strm    ///< Stream to output the time to.
@@ -174,6 +180,27 @@ class PTime : public PObject
       */
     PBoolean IsValid() const;
 
+    /**Set the the objects time with the current time in the current time zone.
+      */
+    void SetCurrentTime();
+
+    /**Get the Network Time Protocol value.
+       The upper 32 bits are seconds since 1st January 1900 and the lower 32
+       bits are 2^32'th of a second, or 0.23ns.
+
+       @return
+       NTP format time.
+     */
+    PUInt64 GetNTP() const;
+
+    /**Set the Network Time Protocol value.
+       The upper 32 bits are seconds since 1st January 1900 and the lower 32
+       bits are 2^32'th of a second, or 0.23ns.
+     */
+    void SetNTP(
+      PUInt64 ntp
+    );
+
     /**Get the total microseconds since the epoch. The epoch is the 1st
        January 1970.
 
@@ -182,15 +209,17 @@ class PTime : public PObject
      */
     PInt64 GetTimestamp() const;
 
-    /**Set the the objects time with the current time in the current time zone.
-      */
-    void SetCurrentTime();
-
     /**Set the time in seconds and microseconds.
       */
     void SetTimestamp(
       time_t seconds,
-      long usecs = 0
+      int64_t usecs = 0
+    );
+
+    /**Add the time in microseconds.
+      */
+    void AddTimestamp(
+      int64_t usecs
     );
 
     /**Get the total seconds since the epoch. The epoch is the 1st
@@ -206,7 +235,7 @@ class PTime : public PObject
        @return
        integer in range 0..999999.
      */
-    long GetMicrosecond() const;
+    unsigned GetMicrosecond() const;
 
     /**Get the second of the time.
 
@@ -238,7 +267,8 @@ class PTime : public PObject
 
     /// Month codes.
     enum Months {
-      January = 1,
+      InvalidMonth,
+      January,
       February,
       March,
       April,
@@ -268,6 +298,7 @@ class PTime : public PObject
 
     /// Days of the week.
     enum Weekdays {
+      InvalidWeekday = -1,
       Sunday,
       Monday,
       Tuesday,
@@ -291,19 +322,26 @@ class PTime : public PObject
      */
     int GetDayOfYear() const;
 
+    /**Retrun time elapsed from "now".
+
+       @return
+       PTimerInterval for elapsed time.
+     */
+    PTimeInterval GetElapsed() const;
+
     /**Determine if the time is in the past or in the future.
 
        @return
        true if time is before the current real time.
      */
-    PBoolean IsPast() const;
+    bool IsPast() const;
 
     /**Determine if the time is in the past or in the future.
 
        @return
        true if time is after the current real time.
      */
-    PBoolean IsFuture() const;
+    bool IsFuture() const;
   //@}
 
   /**@name Time Zone configuration functions */
@@ -313,7 +351,7 @@ class PTime : public PObject
        @return
        true if daylight savings time is active.
      */
-    static PBoolean IsDaylightSavings();
+    static bool IsDaylightSavings();
 
     /// Flag for time zone adjustment on daylight savings.
     enum TimeZoneType {
@@ -397,32 +435,21 @@ class PTime : public PObject
   //@{
     /// Standard time formats for string representations of a time and date.
     enum TimeFormat {
-      /// Internet standard format. (eg. Wed, 09 Feb 2011 11:25:58 +01:00)
-      RFC1123,
-      /// Another Internet standard format. (eg. 2011-02-09T11:14:41ZZ)
-      RFC3339,
-      /// Short form ISO standard format. (eg. 20110209T111108Z)
-      ShortISO8601,
-      /// Long form ISO standard format. (eg. 2011-02-09 T 11:13:06 Z)
-      LongISO8601,
-      /// Date with weekday, full month names and time with seconds.
-      LongDateTime,
-      /// Date with weekday, full month names and no time.
-      LongDate,
-      /// Time with seconds.
-      LongTime,
-      /// Date with abbreviated month names and time without seconds.
-      MediumDateTime,
-      /// Date with abbreviated month names and no time.
-      MediumDate,
-      /// Date with numeric month name and time without seconds.
-      ShortDateTime,
-      /// Date with numeric month and no time.
-      ShortDate,
-      /// Time without seconds.
-      ShortTime,
-      /// Epoch format (e.g. 1234476388.123456)
-      EpochTime,
+      RFC1123,        ///< Internet standard format. (eg. Wed, 09 Feb 2011 11:25:58 +01:00)
+      RFC3339,        ///< Another Internet standard format. (eg. 2011-02-09T11:14:41ZZ)
+      ShortISO8601,   ///< Short form ISO standard format. (eg. 20110209T111108Z)
+      LongISO8601,    ///< Long form ISO standard format. (eg. 2011-02-09 T 11:13:06 Z)
+      LongDateTime,   ///< Date with weekday, full month names and time with seconds.
+      LongDate,       ///< Date with weekday, full month names and no time.
+      LongTime,       ///< Time with seconds.
+      MediumDateTime, ///< Date with abbreviated month names and time without seconds.
+      MediumDate,     ///< Date with abbreviated month names and no time.
+      ShortDateTime,  ///< Date with numeric month name and time without seconds.
+      ShortDate,      ///< Date with numeric month and no time.
+      ShortTime,      ///< Time without seconds.
+      EpochTime,      ///< Epoch format (e.g. 1234476388.123456)
+      LoggingFormat,  ///< Format used by logging. (eg. 2011/02/09 11:13:06.312)
+      TodayFormat,    ///< If +/- 12 hours, only prints hh:mm:ss.uuu, otherwise LoggingFormat
       NumTimeStrings
     };
 
@@ -514,7 +541,7 @@ class PTime : public PObject
        @return
        true is 12 hour, false if 24 hour.
      */
-    static PBoolean GetTimeAMPM();
+    static bool GetTimeAMPM();
 
     /**Get the internationalised time AM string.
     
@@ -588,11 +615,11 @@ class PTime : public PObject
     */
 
   protected:
-    // Member variables
-    /// Number of seconds since 1 January 1970.
-    time_t theTime;
-    long   microseconds;
+    bool InternalLocalTime(struct tm & t) const;
 
+    // Member variables
+    atomic<int64_t> m_microSecondsSinceEpoch;
+    static int64_t const Micro = 1000000;
 
 // Include platform dependent part of class
 #ifdef _WIN32
@@ -603,6 +630,45 @@ class PTime : public PObject
 };
 
 
+class P_timeval
+{
+public:
+  P_timeval(long secs = 0, long usecs = 0);
+
+  P_timeval(const PTimeInterval & time)
+  {
+    operator=(time);
+  }
+
+  P_timeval(const PTime & time)
+  {
+    operator=(time);
+  }
+
+  P_timeval & operator=(const PTimeInterval & time);
+  P_timeval & operator=(const PTime & time);
+
+  operator timeval*()
+  {
+    return m_infinite ? NULL : &m_timeval;
+  }
+
+  timeval * operator->()
+  {
+    return &m_timeval;
+  }
+
+  timeval & operator*()
+  {
+    return m_timeval;
+  }
+
+private:
+  struct timeval m_timeval;
+  bool           m_infinite;
+};
+
+
 #endif // PTLIB_TIME_H
 
 
