@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28930 $
+ * $Revision: 33214 $
  * $Author: rjongbloed $
- * $Date: 2013-01-18 02:37:32 -0600 (Fri, 18 Jan 2013) $
+ * $Date: 2015-01-08 06:44:36 +0000 (Thu, 08 Jan 2015) $
  */
 
 #ifndef PTLIB_SOCKET_H
@@ -485,6 +485,8 @@ class PSocket : public PChannel
   //@}
 
   protected:
+    virtual PBoolean ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group = LastGeneralError);
+
     /*This function calls os_socket() with the correct parameters for the
        socket protocol type.
      */
@@ -524,6 +526,8 @@ class PSocket : public PChannel
       socklen_t * size
     );
 
+    virtual int os_errno() const;
+
 
   // Member variables
     /// Port to be used by the socket when opening the channel.
@@ -538,84 +542,6 @@ class PSocket : public PChannel
 };
 
 
-// Utility classes
-
-class P_fd_set {
-  public:
-    P_fd_set();
-    P_fd_set(SOCKET fd);
-    ~P_fd_set()
-      {
-        free(set);
-      }
-
-    P_fd_set & operator=(SOCKET fd);
-    P_fd_set & operator+=(SOCKET fd);
-    P_fd_set & operator-=(SOCKET fd);
-
-    void Zero();
-
-    PBoolean IsPresent(SOCKET fd) const
-      {
-        return FD_ISSET(fd, set);
-      }
-
-    operator fd_set*() const
-      {
-        return set;
-      }
-
-  protected:
-    void Construct();
-
-    SOCKET max_fd;
-    fd_set * set;
-
-  private:
-#ifdef __MINGW32__
-  // avoid compile error when this declaration is private
-  // this could be a gcc bug
-  public:
-    P_fd_set(const P_fd_set &) {}
-  private:
-#else
-    P_fd_set(const P_fd_set &) {}
-#endif
-    void operator=(const P_fd_set &) {}
-};
-
-
-class P_timeval {
-  public:
-    P_timeval();
-    P_timeval(const PTimeInterval & time)
-      {
-        operator=(time);
-      }
-
-    P_timeval & operator=(const PTimeInterval & time);
-
-    operator timeval*()
-      {
-        return infinite ? NULL : &tval;
-      }
-
-    timeval * operator->()
-      {
-        return &tval;
-      }
-
-    timeval & operator*()
-      {
-        return tval;
-      }
-
-  private:
-    struct timeval tval;
-    PBoolean infinite;
-};
-
-
 #endif // PTLIB_SOCKET_H
 
 
