@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29306 $
+ * $Revision: 33214 $
  * $Author: rjongbloed $
- * $Date: 2013-03-24 23:14:19 -0500 (Sun, 24 Mar 2013) $
+ * $Date: 2015-01-08 06:44:36 +0000 (Thu, 08 Jan 2015) $
  */
 
 #ifndef PTLIB_SERVICEPROCESS_H
@@ -56,12 +56,12 @@ class PServiceProcess : public PProcess
     /** Create a new service process.
      */
     PServiceProcess(
-      const char * manuf,   ///< Name of manufacturer
-      const char * name,    ///< Name of product
-      WORD majorVersion,    ///< Major version number of the product
-      WORD minorVersion,    ///< Minor version number of the product
-      CodeStatus status,    ///< Development status of the product
-      WORD buildNumber      ///< Build number of the product
+      const char * manuf,     ///< Name of manufacturer
+      const char * name,      ///< Name of product
+      unsigned majorVersion,  ///< Major version number of the product
+      unsigned minorVersion,  ///< Minor version number of the product
+      CodeStatus status,      ///< Development status of the product
+      unsigned buildNumber    ///< Build number of the product
     );
   //@}
 
@@ -120,7 +120,7 @@ class PServiceProcess : public PProcess
      */
     static PServiceProcess & Current();
 
-
+#if P_SYSTEMLOG
     /** Set the level at which errors are logged. Only messages higher than or
        equal to the specified level will be logged.
     
@@ -132,7 +132,7 @@ class PServiceProcess : public PProcess
      */
     void SetLogLevel(
       PSystemLog::Level level  ///< New log level
-    ) { PSystemLog::GetTarget().SetThresholdLevel(level); }
+    );
 
     /** Get the current level for logging.
 
@@ -140,6 +140,7 @@ class PServiceProcess : public PProcess
        Log level.
      */
     PSystemLog::Level GetLogLevel() const { return PSystemLog::GetTarget().GetThresholdLevel(); }
+#endif
   //@}
 
 
