@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28062 $
+ * $Revision: 34117 $
  * $Author: rjongbloed $
- * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
+ * $Date: 2015-10-01 20:32:31 +0100 (Thu, 01 Oct 2015) $
  */
 
 #ifndef PTLIB_SYNCTHRD_H
@@ -39,8 +39,10 @@
 #endif
 
 #include <ptlib/mutex.h>
+#include <ptlib/semaphor.h>
 #include <ptlib/syncpoint.h>
 #include <map>
+#include <queue>
 
 
 /** This class defines a thread synchronisation object.
@@ -254,7 +256,10 @@ class PReadWriteMutex : public PObject
   public:
   /**@name Construction */
   //@{
-    PReadWriteMutex();
+    explicit PReadWriteMutex(
+        const char * name = NULL,  ///< Arbitrary name, or filename of mutex variable declaration
+        unsigned line = 0          ///< Line number, if non zero, name is assumed to be a filename
+    );
     ~PReadWriteMutex();
   //@}
 
@@ -301,6 +306,8 @@ class PReadWriteMutex : public PObject
     void EndWrite();
   //@}
 
+    virtual void PrintOn(ostream &strm) const;
+
   protected:
     PSemaphore  m_readerSemaphore;
     PTimedMutex m_readerMutex;
@@ -316,11 +323,13 @@ class PReadWriteMutex : public PObject
       unsigned m_readerCount;
       unsigned m_writerCount;
       bool     m_waiting;
+      PUniqueThreadIdentifier m_uniqueId;
 
       Nest()
         : m_readerCount(0)
         , m_writerCount(0)
         , m_waiting(false)
+        , m_uniqueId(PThread::GetCurrentUniqueIdentifier())
       { }
     };
     typedef std::map<PThreadIdentifier, Nest> NestMap;
@@ -333,8 +342,17 @@ class PReadWriteMutex : public PObject
     void InternalStartRead(Nest & nest);
     void InternalEndRead(Nest & nest);
     void InternalWait(Nest & nest, PSync & sync) const;
+
+    const char * m_fileOrName;
+    unsigned     m_line;
+
+  friend class PSafeObject;
 };
 
+/// Declare a PReadWriteMutex with compiled file/line for deadlock debugging
+#define PDECLARE_READ_WRITE_MUTEX(var) struct PReadWriteMutex_##var : PReadWriteMutex { PReadWriteMutex_##var() : PReadWriteMutex(__FILE__,__LINE__) { } } var
+#define PDECLARE_READ_WRITE_MUTEX2(var, name) struct PReadWriteMutex_##var : PReadWriteMutex { PReadWriteMutex_##var() : PReadWriteMutex(#name) { } } var
+
 
 /**This class starts a read operation for the PReadWriteMutex on construction
    and automatically ends the read operation on destruction.
@@ -412,6 +430,150 @@ class PWriteWaitAndSignal {
 };
 
 
+/** A synchronous queue of objects.
+    This implements a queue of objects between threads. The dequeue action will
+    always block until an object is placed into the queue.
+  */
+template <class T> class PSyncQueue : public PObject, public std::queue<T>
+{
+    PCLASSINFO(PSyncQueue, PObject);
+  public:
+    typedef std::queue<T> BaseQueue;
+
+    enum State
+    {
+      e_Open,
+      e_Blocked,
+      e_Closed
+    };
+
+    /// Construct synchronous queue
+    PSyncQueue()
+      : m_state(e_Open)
+      , m_available(0, INT_MAX)
+    {
+    }
+
+    /// Destroy synchronous queue
+    ~PSyncQueue()
+    {
+      Close(true);
+    }
+
+    /// Enqueue an object to the synchronous queue.
+    bool Enqueue(const T & obj)
+    {
+      PWaitAndSignal lock(m_mutex);
+      if (m_state == e_Closed)
+        return false;
+      BaseQueue::push(obj);
+      m_available.Signal();
+      return true;
+    }
+
+    /** Dequeue an object from the synchronous queue.
+        @return false if a timeout occurs, or the queue was flushed.
+      */
+    bool Dequeue(T & value, const PTimeInterval & timeout = PMaxTimeInterval)
+    {
+      bool dequeued = false;
+
+      m_mutex.Wait();
+
+      switch (m_state) {
+        case e_Blocked :
+          PAssertAlways("Multiple threads in PSyncQueue::pop()");
+          break;
+
+        case e_Open :
+          m_state = e_Blocked;
+
+          {
+            m_mutex.Signal();
+            bool available = m_available.Wait(timeout);
+            m_mutex.Wait();
+
+            if (available && !BaseQueue::empty()) {
+              value = BaseQueue::front();
+              BaseQueue::pop();
+              dequeued = true;
+            }
+          }
+
+          if (m_state != e_Closed) {
+            m_state = e_Open;
+            break;
+          }
+          // Do closed case
+
+        case e_Closed :
+          m_closed.Signal();
+      }
+
+      m_mutex.Signal();
+
+      return dequeued;
+    }
+
+    /** Close the queue and break block in Dequeue() function.
+        This may optionally wait for Dequeue() to exit before returning.
+      */
+    void Close(bool wait)
+    {
+      m_mutex.Wait();
+
+      bool blocked = m_state == e_Blocked;
+      m_state = e_Closed;
+
+      while (!BaseQueue::empty())
+        BaseQueue::pop();
+
+      m_available.Signal();
+      m_mutex.Signal();
+
+      if (blocked && wait)
+        m_closed.Wait();
+    }
+
+    // Indicate queue is in use.
+    bool IsOpen() const
+    {
+        PWaitAndSignal mutex(m_mutex);
+        return m_state != e_Closed;
+    }
+
+    /// Restart the queue after it has been closed.
+    void Restart()
+    {
+      m_mutex.Wait();
+      if (m_state == e_Closed) {
+        while (!BaseQueue::empty())
+          BaseQueue::pop();
+        while (m_available.Wait(0))
+          ;
+        m_state = e_Open;
+      }
+      m_mutex.Signal();
+    }
+
+
+    __inline const PMutex & GetMutex() const { return m_mutex; }
+    __inline       PMutex & GetMutex()       { return m_mutex; }
+
+  protected:
+    State          m_state;
+    PSemaphore     m_available;
+    PMutex         m_mutex;
+    PSyncPoint     m_closed;
+
+  private:
+    __inline PSyncQueue(const PSyncQueue & other) : PObject(other) { }
+    __inline void operator=(const PSyncQueue &) { }
+    __inline void push(T * obj) { BaseQueue::push(obj); }
+    __inline void pop() { BaseQueue::pop(); }
+};
+
+
 #endif // PTLIB_SYNCTHRD_H
 
 
