@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29970 $
+ * $Revision: 34413 $
  * $Author: rjongbloed $
- * $Date: 2013-06-16 20:14:51 -0500 (Sun, 16 Jun 2013) $
+ * $Date: 2015-12-11 07:16:49 +0000 (Fri, 11 Dec 2015) $
  */
 
 #ifndef PTLIB_THREAD_H
@@ -43,6 +43,7 @@
 #endif
 
 #include <ptlib/mutex.h>
+#include <ptlib/notifier.h>
 #include <set>
 
 
@@ -73,7 +74,7 @@ class PThread : public PObject
   /**@name Construction */
   //@{
     /// Codes for thread priorities.
-    enum Priority {
+    P_DECLARE_TRACED_ENUM(Priority,
       LowestPriority,  ///< Will only run if all other threads are blocked.
 
       LowPriority,     ///< Runs approximately half as often as normal.
@@ -82,10 +83,8 @@ class PThread : public PObject
 
       HighPriority,    ///< Runs approximately twice as often as normal.
 
-      HighestPriority, ///< Is only thread that will run, unless blocked.
-
-      NumPriorities
-    };
+      HighestPriority  ///< Is only thread that will run, unless blocked.
+    );
 
     /// Codes for thread autodelete flag
     enum AutoDeleteFlag {
@@ -272,6 +271,23 @@ class PThread : public PObject
      */
     virtual PString GetThreadName() const;
 
+    /** Get the name of the thread. Thread names are a optional debugging aid.
+
+       @return
+       current thread name.
+     */
+    static PString GetThreadName(PThreadIdentifier id);
+
+    /**Get the current threads name.
+      */
+    static PString GetCurrentThreadName() { return GetThreadName(GetCurrentThreadId()); }
+
+    /**Convert to thread identifers as a string.
+       For platforms where unique thread id is different from the "normal" thread id,
+       then the output will append it to the thread id string, e.g. "0x12345678 (54321)".
+      */
+    static PString GetIdentifiersAsString(PThreadIdentifier tid, PUniqueThreadIdentifier uid);
+
     /** Change the name of the thread. Thread names are a optional debugging aid.
 
        @return
@@ -284,28 +300,85 @@ class PThread : public PObject
 
   /**@name Miscellaneous */
   //@{
-    /** Get operating system specific thread identifier for this thread.
-      * Note that the return value from these functions is only valid
-      * if called by the owning thread. Calling this function for another
-      * thread that may be terminating is a very bad idea.
+    PPROFILE_EXCLUDE(
+      /** Get operating system specific thread identifier for this thread.
       */
-    virtual PThreadIdentifier GetThreadId() const { return m_threadId; }
-    static PThreadIdentifier GetCurrentThreadId();
+    virtual PThreadIdentifier GetThreadId() const;
+    );
+
+    /** Get operating system specific thread identifier for the current thread.
+      */
+    PPROFILE_EXCLUDE(
+      static PThreadIdentifier GetCurrentThreadId()
+    );
+
+    PPROFILE_EXCLUDE(
+    /** This returns a unique number for the thread.
+        For most platforms this is identical to GetThreadId(), but others,
+        e.g. Linux, it is different and GetThreadId() return values get
+        re-used during the run of the process.
+      */
+    PUniqueThreadIdentifier GetUniqueIdentifier() const
+    );
+
+    PPROFILE_EXCLUDE(
+    /** This returns a unique number for the currentthread.
+      */
+      static PUniqueThreadIdentifier GetCurrentUniqueIdentifier()
+    );
 
     /// Times for execution of the thread.
     struct Times
     {
-      PTimeInterval m_real;     ///< Total real time since thread start in milliseconds.
-      PTimeInterval m_kernel;   ///< Total kernel CPU time in milliseconds.
-      PTimeInterval m_user;     ///< Total user CPU time in milliseconds.
-      friend ostream & operator<<(ostream & strm, const Times & times);
+      PPROFILE_EXCLUDE(
+        Times()
+      );
+      PPROFILE_EXCLUDE(
+        friend ostream & operator<<(ostream & strm, const Times & times)
+      );
+      PPROFILE_EXCLUDE(
+        float AsPercentage() const
+      );
+      PPROFILE_EXCLUDE(
+        Times operator-(const Times & rhs) const
+      );
+      PPROFILE_EXCLUDE(
+        Times & operator-=(const Times & rhs)
+      );
+
+      PString                 m_name;     ///< Name of thread
+      PThreadIdentifier       m_threadId; ///< Operating system thread ID
+      PUniqueThreadIdentifier m_uniqueId; ///< Unique thread identifier
+      PTimeInterval           m_real;     ///< Total real time since thread start in milliseconds.
+      PTimeInterval           m_kernel;   ///< Total kernel CPU time in milliseconds.
+      PTimeInterval           m_user;     ///< Total user CPU time in milliseconds.
     };
 
+    PPROFILE_EXCLUDE(
     /** Get the thread execution times.
      */
     bool GetTimes(
       Times & times   ///< Times for thread execution.
-    );
+    ));
+
+    PPROFILE_EXCLUDE(
+    /** Get the thread execution times.
+     */
+    static bool GetTimes(
+      PThreadIdentifier id, ///< Thread identifier to get times for
+      Times & times         ///< Times for thread execution.
+    ));
+
+    PPROFILE_EXCLUDE(
+    /** Get the thread execution times for all threads.
+     */
+    static void GetTimes(
+      std::list<Times> & times         ///< Times for thread execution.
+    ));
+
+    /**Get number of processors, or processor cores, this machine has available.
+      */
+    static unsigned GetNumProcessors();
 
     /** User override function for the main execution routine of the thread. A
        descendent class must provide the code that will be executed in the
@@ -371,7 +444,7 @@ class PThread : public PObject
         void ThreadDestroyed(PThread * thread) const;
         typedef std::map<PThread *, void *> StorageMap;
         mutable StorageMap m_storage;
-        PMutex m_mutex;
+        PCriticalSection m_mutex;
       friend class PThread;
     };
     friend class LocalStorageBase;
@@ -398,7 +471,7 @@ class PThread : public PObject
     PINDEX m_originalStackSize;
 
     PString m_threadName; // Give the thread a name for debugging purposes.
-    PMutex  m_threadNameMutex;
+    PCriticalSection m_threadNameMutex;
 
     PThreadIdentifier m_threadId;
 
@@ -505,8 +578,13 @@ class PThread1Arg : public PThread
   public:
     typedef void (*FnType)(Arg1Type arg1);
 
-    PThread1Arg(Arg1Type arg1, FnType function, bool autoDel = false)
-      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread1Arg(
+      Arg1Type arg1,
+      FnType function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, priority, name)
       , m_function(function)
       , m_arg1(arg1)
     {
@@ -542,8 +620,14 @@ class PThread2Arg : public PThread
     PCLASSINFO(PThread2Arg, PThread);
   public:
     typedef void (*FnType)(Arg1Type arg1, Arg2Type arg2); 
-    PThread2Arg(Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDel = false)
-      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread2Arg(
+      Arg1Type arg1,
+      Arg2Type arg2,
+      FnType function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, priority, name)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
@@ -580,8 +664,15 @@ class PThread3Arg : public PThread
   PCLASSINFO(PThread3Arg, PThread);
   public:
     typedef void (*FnType)(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3); 
-    PThread3Arg(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDel = false)
-      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread3Arg(
+      Arg1Type arg1,
+      Arg2Type arg2,
+      Arg3Type arg3,
+      FnType function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, priority, name)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
@@ -599,7 +690,7 @@ class PThread3Arg : public PThread
     FnType   m_function;
     Arg1Type m_arg1;
     Arg2Type m_arg2;
-    Arg2Type m_arg3;
+    Arg3Type m_arg3;
 };
 
 /*
@@ -621,7 +712,7 @@ class PThread3Arg : public PThread
 template <typename ObjType>
 class PThreadObj : public PThread
 {
-    PCLASSINFO(PThreadObj, PThread);
+    PCLASSINFO_ALIGNED(PThreadObj, PThread, 16);
   public:
     typedef void (ObjType::*ObjTypeFn)(); 
 
@@ -648,7 +739,7 @@ class PThreadObj : public PThread
 
   protected:
     ObjType & m_object;
-    ObjTypeFn P_ALIGN_FIELD(m_function, 8);
+    P_ALIGN_FIELD(ObjTypeFn,m_function,16);
 };
 
 
@@ -671,7 +762,7 @@ class PThreadObj : public PThread
 template <class ObjType, typename Arg1Type>
 class PThreadObj1Arg : public PThread
 {
-    PCLASSINFO(PThreadObj1Arg, PThread);
+    PCLASSINFO_ALIGNED(PThreadObj1Arg, PThread, 16);
   public:
     typedef void (ObjType::*ObjTypeFn)(Arg1Type); 
 
@@ -700,14 +791,14 @@ class PThreadObj1Arg : public PThread
 
   protected:
     ObjType & m_object;
-    ObjTypeFn P_ALIGN_FIELD(m_function, 8);
+    P_ALIGN_FIELD(ObjTypeFn,m_function,16);
     Arg1Type  m_arg1;
 };
 
 template <class ObjType, typename Arg1Type, typename Arg2Type>
 class PThreadObj2Arg : public PThread
 {
-    PCLASSINFO(PThreadObj2Arg, PThread);
+    PCLASSINFO_ALIGNED(PThreadObj2Arg, PThread, 16);
   public:
     typedef void (ObjType::*ObjTypeFn)(Arg1Type, Arg2Type);
 
@@ -738,7 +829,7 @@ class PThreadObj2Arg : public PThread
 
   protected:
     ObjType & m_object;
-    ObjTypeFn P_ALIGN_FIELD(m_function, 8);
+    P_ALIGN_FIELD(ObjTypeFn,m_function,16);
     Arg1Type  m_arg1;
     Arg2Type  m_arg2;
 };
