@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29407 $
+ * $Revision: 34242 $
  * $Author: rjongbloed $
- * $Date: 2013-04-01 05:10:26 -0500 (Mon, 01 Apr 2013) $
+ * $Date: 2015-10-26 12:01:52 +0000 (Mon, 26 Oct 2015) $
  */
 
 #ifndef PTLIB_TIMER_H
@@ -38,10 +38,18 @@
 #pragma interface
 #endif
 
+#include <ptlib_config.h>
+
+#if P_TIMERS
 
 #include <ptlib/notifier.h>
+#include <ptlib/id_generator.h>
 #include <ptclib/threadpool.h>
 
+#include <queue>
+#include <set>
+#include <map>
+
 
 // To avoid ambiguous operator error we need a one for every integer variant
 #define PTIMER_OPERATORS(cls) \
@@ -110,28 +118,6 @@ class PSimpleTimer : public PTimeInterval
 
   /**@name Control functions */
   //@{
-    /** Set the value of the time interval. The time interval, in milliseconds,
-       is the sum of all of the parameters. For example all of the following
-       are equivalent:
-<pre><code>
-              SetInterval(120000)
-              SetInterval(60000, 60)
-              SetInterval(60000, 0, 1)
-              SetInterval(0, 60, 1)
-              SetInterval(0, 0, 2)
-</code></pre>
-
-       The timer will be expired the specified time interval after "now" in
-       real time.
-     */
-    virtual void SetInterval(
-      PInt64 milliseconds = 0,  ///< Number of milliseconds for interval.
-      long seconds = 0,         ///< Number of seconds for interval.
-      long minutes = 0,         ///< Number of minutes for interval.
-      long hours = 0,           ///< Number of hours for interval.
-      int days = 0              ///< Number of days for interval.
-    );
-
     /**Stop the timer.
       */
     void Stop();
@@ -159,6 +145,7 @@ class PSimpleTimer : public PTimeInterval
   //@}
 
   protected:
+    virtual void InternalSet(int64_t t);
     PTimeInterval m_startTick;
 };
 
@@ -172,31 +159,27 @@ class PSimpleTimer : public PTimeInterval
    may either override the virtual function or set a callback as desired.
    
    A list of active timers is maintained by the applications <code>PProcess</code> 
-   instance and the timeout functions are executed in the context of a single
-   thread of execution. There are many consequences of this: only one timeout
-   function can be executed at a time and thus a user should not execute a
-   lot of code in the timeout call-back functions or it will dealy the timely
-   execution of other timers call-back functions.
+   instance and the timeout functions are executed in the context of a separate
+   thread of execution.
 
    Also timers are not very accurate in sub-second delays, even though you can
    set the timer in milliseconds, its accuracy is only to -0/+250 ms. Even
-   more (potentially MUCH more) if there are delays in the user call-back
-   functions.
+   more (potentially MUCH more) if there are large delays in the user call-back
+   functions and many of them exhausting the thread pool.
 
-   Another trap is you cannot destroy a timer in its own call-back. There is
-   code to cause an assert if you try but it is very easy to accidentally do
-   this when you delete an object that contains an onject that contains the
-   timer!
+   When you subclass PTimer you MUST call Stop() in destructor. This guarantees
+   that object not destroyed in OnTimeout execution.
 
-   Finally static timers cause race conditions on start up and termination and
-   should be avoided.
+   Finally, while the timers "system" threads are fully thread safe, the
+   PTimeInterval on which it is based is not. So care is required for user
+   threads on platforms where a 64 bit integer access is not an atomic
+   operation.
  */
 class PTimer : public PTimeInterval
 {
-  PCLASSINFO(PTimer, PTimeInterval);
+  PCLASSINFO_WITH_CLONE(PTimer, PTimeInterval);
 
   public:
-    typedef unsigned IDType;
 
   /**@name Construction */
   //@{
@@ -227,37 +210,27 @@ class PTimer : public PTimeInterval
      */
     PTimer & operator=(
       const PTimer & timer          ///< New time interval for timer.
-    ) { SetInterval(timer.GetMilliSeconds()); return *this; }
+    );
 
     PTIMER_OPERATORS(PTimer);
 
-    /** Destroy the timer object, removing it from the applications timer list
-       if it was running.
+    /** Destroy the timer object, stops timer if it is running.
      */
     virtual ~PTimer();
   //@}
 
+  /**@name Overrides from class PObject */
+  //@{
+    /** Output the time interval to the I/O stream. This outputs the number of
+       milliseconds as a signed decimal integer number.
+     */
+    virtual void PrintOn(
+      ostream & strm    ///< I/O stream to output the time interval.
+    ) const;
+  //@}
+
   /**@name Control functions */
   //@{
-    /** Set the value of the time interval. The time interval, in milliseconds,
-       is the sum of all of the parameters. For example all of the following
-       are equivalent:
-<pre><code>
-              SetInterval(120000)
-              SetInterval(60000, 60)
-              SetInterval(60000, 0, 1)
-              SetInterval(0, 60, 1)
-              SetInterval(0, 0, 2)
-</code></pre>
-     */
-    virtual void SetInterval(
-      PInt64 milliseconds = 0,  ///< Number of milliseconds for interval.
-      long seconds = 0,         ///< Number of seconds for interval.
-      long minutes = 0,         ///< Number of minutes for interval.
-      long hours = 0,           ///< Number of hours for interval.
-      int days = 0              ///< Number of days for interval.
-    );
-
     /** Start a timer in continous cycle mode. Whenever the timer runs out it
        is automatically reset to the time specified. Thus, it calls the
        notification function every time interval.
@@ -270,16 +243,13 @@ class PTimer : public PTimeInterval
        and is reset back to the original timer value. Thus when the timer
        is restarted it begins again from the beginning.
 
-       The wait flag indicates that the function should wait for the timeout
-       callback to complete before returning. That way external logic can be
-       assured there is no race condition. However, under some circumstances
-       this can cause a deadlock if the timeout function tries to acquire a
-       mutex the calling thread already has, so an aysnchronouse version is
-       provided. It is then the responsibility of the caller to handle the
-       race condition with the timeout function.
-     */
+       While OnTimeout() is running this method will wait until OnTimeout()
+       will finish.
+       
+       The wait flag is deprecated and not used in this implementation.
+       */
     void Stop(
-      bool wait = true  
+      bool wait = true
     );
 
     /** Determine if the timer is currently running. This really is only useful
@@ -290,32 +260,13 @@ class PTimer : public PTimeInterval
      */
     PBoolean IsRunning() const;
 
-    /** Pause a running timer. This differs from the <code>Stop()</code> function in
-       that the timer may be resumed at the point that it left off. That is
-       time is "frozen" while the timer is paused.
-     */
-    void Pause();
-
-    /** Restart a paused timer continuing at the time it was paused. The time
-       left at the moment the timer was paused is the time until the next
-       call to the notification function.
-     */
-    void Resume();
-
-    /** Determine if the timer is currently paused.
-
-       @return
-       true if timer paused.
-     */
-    PBoolean IsPaused() const;
-
     /** Restart a timer continuing from the time it was initially.
      */
     void Reset();
 
     /** Get the time this timer was set to initially.
      */
-    const PTimeInterval & GetResetTime() const;
+    PTimeInterval GetResetTime() const;
   //@}
 
   /**@name Notification functions */
@@ -346,12 +297,14 @@ class PTimer : public PTimeInterval
        This is called by the <code>OnTimeout()</code> function.
      */
     void SetNotifier(
-      const PNotifier & func  // New notifier function for the timer.
+      const PNotifier & func,  ///< New notifier function for the timer.
+      const PString & threadName = PString::Empty()  ///< Thread name to use in pool when func executed
     );
   //@}
 
   /**@name Global real time functions */
   //@{
+    PPROFILE_EXCLUDE(
     /** Get the number of milliseconds since some arbtrary point in time. This
        is a platform dependent function that yields a real time counter.
        
@@ -364,7 +317,8 @@ class PTimer : public PTimeInterval
        @return
        millisecond counter.
      */
-    static PTimeInterval Tick();
+    static PTimeInterval Tick()
+    );
 
     /** Get the smallest number of milliseconds that the timer can be set to.
        All actual timing events will be rounded up to the next value. This is
@@ -377,59 +331,73 @@ class PTimer : public PTimeInterval
     static unsigned Resolution();
   //@}
 
-  /**@name Member access */
-  //@{
-    /**Return number of milliseconds left in timer.
-      */
-    PInt64 GetMilliSeconds() const;
 
-    /**Return absolute time timer will expire.
-      */
-    PInt64 GetAbsoluteTime() const { return m_absoluteTime; }
-  //@}
+    /* This class defines a list of <code>PTimer</code> objects. It is primarily used
+       internally by the library and the user should never create an instance of
+       it. The <code>PProcess</code> instance for the application maintains an instance
+       of all of the timers created so that it may decrements them at regular
+       intervals.
+     */
+    class List
+    {
+      public:
+        // Create a new timer list
+        List();
+
+        /* Decrement all the created timers and dispatch to their callback
+           functions if they have expired. The <code>PTimer::Tick()</code> function
+           value is used to determine the time elapsed since the last call to
+           Process().
+
+           The return value is the number of milliseconds until the next timer
+           needs to be dispatched. The function need not be called again for this
+           amount of time, though it can (and usually is).
+       
+           @return
+           maximum time interval before function should be called again.
+         */
+        PTimeInterval Process();
+
+      private:
+        bool OnTimeout(PIdGenerator::Handle handle);
+
+        struct Timeout
+        {
+          PIdGenerator::Handle m_handle;
+          Timeout(PIdGenerator::Handle handle) : m_handle(handle) { }
+          virtual ~Timeout() { }
+          virtual void Work();
+        };
+        PQueuedThreadPool<Timeout> m_threadPool;
+
+        typedef std::map<PIdGenerator::Handle, PTimer *> TimerMap;
+        TimerMap m_timers;
+        PCriticalSection m_timersMutex;
+#if PTRACING
+        size_t m_highWaterMark;
+#endif
+
+      friend class PTimer;
+    };
+
+    static List * TimerList();
 
-    // Internal functions.
-    IDType GetTimerId() const { return m_timerId; }
-    PAtomicInteger::IntegerType GetNextSerialNumber() { return ++m_serialNumber; }
 
   private:
-    void Construct();
-
-    /* Start or restart the timer from the <code>resetTime</code> variable.
-       This is an internal function.
-     */
-    void StartRunning(
-      PBoolean once   // Flag for one shot or continuous.
-    );
-
-    /* Process the timer decrementing it by the delta amount and calling the
-       <code>OnTimeout()</code> when zero. This is used internally by the
-       <code>PTimerList::Process()</code> function.
-     */
-    void Process(
-      PInt64 now             // time consider as "now"
-    );
+    virtual int64_t InternalGet() const;
+    virtual void InternalSet(int64_t t);
+    void InternalStart(bool once, int64_t resetTime);
 
     // Member variables
+    PNotifier            m_callback;     // Callback function for expired timers.
+    PString              m_threadName;
+    bool                 m_oneshot;      // Timer operates once then stops.
+    PIdGenerator::Handle m_handle;
+    bool                 m_running;
+    PTimeInterval        m_absoluteTime;
+    mutable PTimedMutex  m_callbackMutex;
 
-    // Callback function for expired timers.
-    PNotifier m_callback;
-
-    // The time to reset a timer to when RunContinuous() is called.
-    PTimeInterval m_resetTime;
-
-    // Timer operates once then stops.
-    PBoolean m_oneshot;
-
-    // Timer state.
-    enum TimerState { Stopped, Running, Paused, InTimeout } m_state;
-
-    friend class PTimerList;              // needed for Process
-    class PTimerList * m_timerList;  
-
-    IDType m_timerId;
-    PAtomicInteger m_serialNumber;
-    PInt64 m_absoluteTime;
+    friend class Emitter;
 
 // Include platform dependent part of class
 #ifdef _WIN32
@@ -441,7 +409,7 @@ class PTimer : public PTimeInterval
 
 
 /**Template abstract class for a PTimer that queues a work item to a thread pool.
-   THis allows for load balancing of timer actions and, in complex muti-threading
+   This allows for load balancing of timer actions and, in complex muti-threading
    sytems, it is very easy for a deadlock to occur with the usual PNotifier
    method of executing a PTimer call back. This tries to largely avoid the
    issue by the housekeeper thread queuing a PThreadPool "work item" for later
@@ -600,6 +568,16 @@ class PPoolTimerArg3: public PPoolTimer<Base_T, Pool_T>
 };
 
 
+#else
+
+namespace PTimer
+{
+  PTimeInterval Tick();
+  unsigned Resolution();
+};
+
+#endif // P_TIMERS
+
 #endif // PTLIB_TIMER_H
 
 
