@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29415 $
+ * $Revision: 33764 $
  * $Author: rjongbloed $
- * $Date: 2013-04-02 16:41:13 -0500 (Tue, 02 Apr 2013) $
+ * $Date: 2015-07-27 09:42:17 +0100 (Mon, 27 Jul 2015) $
  */
 
 #ifndef PTLIB_UDPSOCKET_H
@@ -129,35 +129,17 @@ class PUDPSocket : public PIPDatagramSocket
       PIPSocketAddressAndPort & addressAndPort    ///< IP address and port to send packets.
     ) const;
     PString GetLastReceiveAddress() const;
-
-    /** CallBack to check if the detected address of the connectionless Read()
-        is an alternate address. Use this to switch the target to send and
-        receive the connectionless read/write.
-     */
-    virtual PBoolean IsAlternateAddress(
-      const Address & address,    ///< Detected IP Address.
-      WORD port                    ///< Detected Port.
-    );
-
-    /** PseudoRead
-        This indicates to the upper system that reading on this socket
-        will be a pseudo read when means there is data available that
-        did not orginate from the physical socket but was programmically
-        injected.
-     */
-    virtual PBoolean DoPseudoRead(int & selectStatus);
   //@}
 
     // Normally, one would expect these to be protected, but they are just so darn
     // useful that it's just easier if they are public
+    virtual bool InternalReadFrom(Slice * slices, size_t sliceCount, PIPSocketAddressAndPort & ipAndPort);
     virtual void InternalSetSendAddress(const PIPSocketAddressAndPort & addr);
     virtual void InternalGetSendAddress(PIPSocketAddressAndPort & addr) const;
     virtual void InternalSetLastReceiveAddress(const PIPSocketAddressAndPort & addr);
     virtual void InternalGetLastReceiveAddress(PIPSocketAddressAndPort & addr) const;
 
   protected:
-    virtual PBoolean ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group = LastGeneralError);
-
     // Open an IPv4 socket (for backward compatibility)
     virtual PBoolean OpenSocket();
 
@@ -178,11 +160,8 @@ class PUDPSocket : public PIPDatagramSocket
 #endif
 
     private:
-      Address m_sendAddress;
-      WORD    m_sendPort;
-
-      Address m_lastReceiveAddress;
-      WORD    m_lastReceivePort;
+      AddressAndPort m_sendAddressAndPort;
+      AddressAndPort m_lastReceiveAddressAndPort;
 };
 
 
