@@ -26,17 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 30795 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2013-10-31 03:19:37 +0000 (Thu, 31 Oct 2013) $
  */
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
-  public:
-    PString GetName() const;
-    PBoolean Close();
 
 // End Of File ////////////////////////////////////////////////////////////////
