@@ -26,19 +26,21 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 32477 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2014-08-09 07:39:19 +0100 (Sat, 09 Aug 2014) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
 // PConfiguration
 
   public:
-    PConfig(int, const PString & name);
     ~PConfig();
 
   protected:
-    PXConfig * config;
+    class Cached;
+    Cached * m_config;
+
+  friend class PConfigCache;
 
 // End Of File ////////////////////////////////////////////////////////////////
