@@ -26,15 +26,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28062 $
+ * $Revision: 29254 $
  * $Author: rjongbloed $
- * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
+ * $Date: 2013-03-13 03:11:33 +0000 (Wed, 13 Mar 2013) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
 // PSyncPoint
 
-#if defined(P_PTHREADS) || defined(__BEOS__) || defined(P_MAC_MPTHREADS)
+#if defined(P_PTHREADS) || defined(__BEOS__)
   public:
     virtual ~PSyncPoint();
   private:
