@@ -26,16 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28100 $
+ * $Revision: 34174 $
  * $Author: rjongbloed $
- * $Date: 2012-07-26 04:25:31 -0500 (Thu, 26 Jul 2012) $
+ * $Date: 2015-10-10 11:25:01 +0100 (Sat, 10 Oct 2015) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
 // PThread
 
-#define PTHREAD_ID_FMT "0x%lx"
-
   public:
     int PXBlockOnChildTerminate(int pid, const PTimeInterval & timeout);
 
@@ -53,6 +51,7 @@
 #ifndef P_HAS_SEMAPHORES
     void PXSetWaitingSemaphore(PSemaphore * sem);
 #endif
+    static bool PX_kill(PThreadIdentifier id, int sig);
 
   protected:
     void PX_StartThread();
@@ -99,28 +98,6 @@
   public:
     int unblockPipe[2];
 
-#elif defined(P_MAC_MPTHREADS)
-  public:
-    void PXSetWaitingSemaphore(PSemaphore * sem);
-    static long PX_ThreadStart(void *);
-    static void PX_ThreadEnd(void *);
-    MPTaskID    PX_GetThreadId() const;
-
-  protected:
-    void PX_NewThread(PBoolean startSuspended);
-
-    PINDEX     PX_origStackSize;
-    int        PX_suspendCount;
-    PSemaphore *suspend_semaphore;
-    long       PX_signature;
-    enum { kMPThreadSig = 'THRD', kMPDeadSig = 'DEAD'};
-
-    MPTaskID   PX_threadId;
-    MPSemaphoreID PX_suspendMutex;
-
-    int unblockPipe[2];
-    friend class PSocket;
-
 #elif defined(VX_TASKS)
   public:
     SEM_ID syncPoint;
