@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 29331 $
+ * $Revision: 33855 $
  * $Author: rjongbloed $
- * $Date: 2013-03-26 20:22:40 -0500 (Tue, 26 Mar 2013) $
+ * $Date: 2015-08-08 18:52:41 +0100 (Sat, 08 Aug 2015) $
  */
 
 
@@ -35,7 +35,7 @@
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
-#include <ptbuildopts.h>
+#include <ptlib.h>
 
 #if P_VIDEO
 
@@ -278,79 +278,63 @@ class PVideoControlInfo : public PObject
     P_DECLARE_STREAMABLE_ENUM(Types,
       Pan,
       Tilt,
-      Zoom
+      Zoom,
+      Focus
     );
 
-    Types m_type;
-    long  m_min;
-    long  m_max;
-    long  m_step;
-    long  m_default;
-    long  m_flags;
-    long  m_current;
-
-    PVideoControlInfo()
-      : m_type(EndTypes)
-      , m_min(0)
-      , m_max(0)
-      , m_step(0)
-      , m_default(0)
-      , m_flags(0)
-      , m_current(0)
+    PVideoControlInfo(
+      Types type = EndTypes,
+      int minimum = 0,
+      int maximum = 100,
+      int step = 1,
+      int reset = 0
+    ) : m_type(type)
+      , m_minimum(minimum)
+      , m_maximum(maximum)
+      , m_step(step)
+      , m_reset(reset)
+      , m_current(reset)
     { }
 
-    bool IsValid() const { return m_max > 0 && m_min < m_max; }
+    virtual void PrintOn(ostream & strm) const;
+
+    bool IsValid() const   { return m_type != EndTypes; }
+
+    Types GetType() const  { return m_type; }
+    int GetStep()    const { return m_step; }
+    int Reset()            { return m_current = m_reset; }
+    int GetCurrent() const { return m_current; }
+    int SetCurrent(int current);
+
+  protected:
+    Types m_type;
+    int   m_minimum;
+    int   m_maximum;
+    int   m_step;
+    int   m_reset;
+    int   m_current;
 };
 
 
-/**This class defines a video Input device control (Camera controls PTZ)
-*/
-
-class PVideoInputControl : public PObject
-{
-    PCLASSINFO(PVideoInputControl, PObject);
-
-public:
-  ~PVideoInputControl();
-
-  virtual PBoolean Pan (long value, bool absolute = false);
-  virtual PBoolean Tilt(long value, bool absolute = false);
-  virtual PBoolean Zoom(long value, bool absolute = false);
-
-  virtual long GetPan()  const { return m_control[PVideoControlInfo::Pan ].m_current; }
-  virtual long GetTilt() const { return m_control[PVideoControlInfo::Tilt].m_current; }
-  virtual long GetZoom() const { return m_control[PVideoControlInfo::Zoom].m_current; }
-
-  virtual void Reset();
-
-  bool IsValid(PVideoControlInfo::Types type) const { return m_control[type].IsValid(); }
-  const PVideoControlInfo & GetControl(PVideoControlInfo::Types type) const { return m_control[type]; }
-
-protected:
-  PVideoControlInfo m_control[PVideoControlInfo::NumTypes];
-  PMutex            m_mutex;
-
-};
-
 /**This class defines a video Input device Interactions (Remote Inputs/Controls)
 */
 class PVideoInteractionInfo : public PObject
 {
-  PCLASSINFO(PVideoInteractionInfo, PObject);
+    PCLASSINFO(PVideoInteractionInfo, PObject);
+  public:
+    P_DECLARE_ENUM(Type,
+      InteractKey,       /// Register remote KeyPresses
+      InteractMouse,     /// Register remote Mouse Movement Clicks
+      InteractNavigate,  /// Register remote Navigation commands
+      InteractRTSP,      /// Register remote RTSP (Real Time Streaming Protocol) Inputs
+      InteractOther      /// Register remote application specific Inputs
+    );
 
- public:
+    PVideoInteractionInfo();
+    virtual void PrintOn(ostream & strm) const;
+    static PString AsString(const Type & type);
 
-   typedef enum {
-        InteractKey,    /// Register remote KeyPresses
-        InteractMouse,    /// Register remote Mouse Movement Clicks
-        InteractNavigate,  /// Register remote Navigation commands
-    InteractRTSP,    /// Register remote RTSP (Real Time Streaming Protocol) Inputs
-    InteractOther    /// Register remote application specific Inputs
-     } InputInteractType;
-
-   static PString AsString(const InputInteractType & type);
-
-  InputInteractType type;
+    Type m_type;
 };
 
 
@@ -397,13 +381,12 @@ class PVideoDevice : public PVideoFrameInfo
      */
     virtual ~PVideoDevice();
 
-    enum VideoFormat {
+    P_DECLARE_STREAMABLE_ENUM(VideoFormat,
       PAL,
       NTSC,
       SECAM,
-      Auto,
-      NumVideoFormats
-    };
+      Auto
+    );
 
     /**Get the device name of the open device.
       */
@@ -442,6 +425,19 @@ class PVideoDevice : public PVideoFrameInfo
       ResizeMode  resizeMode;
       bool        flip;
       Attributes  m_attributes;
+
+      template<class PVideoXxxDevice>
+      bool Validate(PVideoDevice::OpenArgs & validated) const
+      {
+        // Check that the input device is legal
+        PVideoXxxDevice * pDevice = PVideoXxxDevice::CreateDeviceByName(deviceName, driverName, pluginMgr);
+        if (pDevice == NULL)
+          return false;
+
+        delete pDevice;
+        validated = *this;
+        return true;
+      }
     };
 
     /**Open the device given the device name.
@@ -475,10 +471,6 @@ class PVideoDevice : public PVideoFrameInfo
     virtual PBoolean Stop() = 0;
 
 
-#if PTRACING
-    friend ostream & operator<<(ostream &, VideoFormat);
-#endif
-
     /**Set the video format to be used.
 
        Default behaviour sets the value of the videoFormat variable and then
@@ -500,6 +492,10 @@ class PVideoDevice : public PVideoFrameInfo
     */
     virtual int GetNumChannels();
 
+    /**Get the names of video channels available on the device.
+    */
+    virtual PStringArray GetChannelNames();
+
     /**Set the video channel to be used on the device.
        The channel number is an integer from 0 to GetNumChannels()-1. The
        special value of -1 will find the first working channel number.
@@ -679,8 +675,9 @@ class PVideoDevice : public PVideoFrameInfo
     
   protected:
     PINDEX GetMaxFrameBytesConverted(PINDEX rawFrameBytes) const;
+    PString GetDeviceNameFromOpenArgs(const OpenArgs & args) const;
 
-    PString      deviceName;
+    PCaselessString deviceName;
     int          lastError;
     VideoFormat  videoFormat;
     int          channelNumber;
@@ -763,15 +760,20 @@ class PVideoOutputDevice : public PVideoDevice
     static PVideoOutputDevice *CreateOpenedDevice(
       const PString & driverName,         ///< Name of driver
       const PString & deviceName,         ///< Name of device
-      PBoolean startImmediate = true,         ///< Immediately start display
+      bool startImmediate = true,         ///< Immediately start display
       PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
     );
+    static PVideoOutputDevice *CreateOpenedDevice(
+      const PString & deviceName,         ///< Name of device
+      bool startImmediate = true,         ///< Immediately start display
+      PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
+    ) { return CreateOpenedDevice(PString::Empty(), deviceName, startImmediate, pluginMgr); }
 
     /**Create an opened video output device that corresponds to the specified arguments.
     */
     static PVideoOutputDevice *CreateOpenedDevice(
       const OpenArgs & args,              ///< Parameters to set on opened device
-      PBoolean startImmediate = true          ///< Immediately start display
+      bool startImmediate = true          ///< Immediately start display
     );
 
     /**Close the device.
@@ -1025,32 +1027,49 @@ class PVideoInputDevice : public PVideoDevice
     static PVideoInputDevice *CreateOpenedDevice(
       const PString & driverName,         ///< Name of driver
       const PString & deviceName,         ///< Name of device
-      PBoolean startImmediate = true,         ///< Immediately start grabbing
+      bool startImmediate = true,         ///< Immediately start grabbing
       PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
     );
+    static PVideoInputDevice *CreateOpenedDevice(
+      const PString & deviceName,         ///< Name of device
+      bool startImmediate = true,         ///< Immediately start grabbing
+      PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
+    ) { return CreateOpenedDevice(PString::Empty(), deviceName, startImmediate, pluginMgr); }
 
     /**Create an opened video output device that corresponds to the specified arguments.
     */
     static PVideoInputDevice *CreateOpenedDevice(
       const OpenArgs & args,              ///< Parameters to set on opened device
-      PBoolean startImmediate = true          ///< Immediately start display
+      bool startImmediate = true          ///< Immediately start display
     );
 
-  typedef struct {
-     std::list<PVideoFrameInfo> framesizes;
-     std::list<PVideoControlInfo> controls;
-     std::list<PVideoInteractionInfo> interactions;
-  } Capabilities;
+    class Capabilities : public PObject {
+      PCLASSINFO(Capabilities, PObject);
+    public:
+      Capabilities();
+      virtual void PrintOn(ostream & strm) const;
+
+      unsigned                         m_channels;
+      bool                             m_brightness;
+      bool                             m_contrast;
+      bool                             m_saturation;
+      bool                             m_hue;
+      bool                             m_gamma;
+      bool                             m_exposure;
+      std::list<PVideoFrameInfo>       framesizes;
+      std::list<PVideoControlInfo>     controls;
+      std::list<PVideoInteractionInfo> interactions;
+    };
 
     /**Retrieve a list of Device Capabilities
       */
     virtual bool GetDeviceCapabilities(
       Capabilities * capabilities          ///< List of supported capabilities
-    ) const { return GetDeviceCapabilities(GetDeviceName(), capabilities); }
+      ) const;
 
     /**Retrieve a list of Device Capabilities for particular device
       */
-    static PBoolean GetDeviceCapabilities(
+    static bool GetDeviceCapabilities(
       const PString & deviceName,           ///< Name of device
       Capabilities * capabilities,          ///< List of supported capabilities
       PPluginManager * pluginMgr = NULL     ///< Plug in manager, use default if NULL
@@ -1058,18 +1077,13 @@ class PVideoInputDevice : public PVideoDevice
 
     /**Retrieve a list of Device Capabilities for a particular driver
       */
-    static PBoolean GetDeviceCapabilities(
+    static bool GetDeviceCapabilities(
       const PString & deviceName,           ///< Name of device
       const PString & driverName,           ///< Device Driver
       Capabilities * caps,                  ///< List of supported capabilities
       PPluginManager * pluginMgr = NULL     ///< Plug in manager, use default if NULL
     );
 
-    /**Get the devices video Input controls
-        By Default return NULL;
-      */
-    virtual PVideoInputControl * GetVideoInputControls();
-
     /**Open the device given the device name.
       */
     virtual PBoolean Open(
@@ -1155,6 +1169,20 @@ class PVideoInputDevice : public PVideoDevice
     */
     virtual int GetCaptureMode() const;
 
+    enum ControlMode {
+      AutomaticControl,
+      AbsoluteControl,
+      RelativeControl,
+      ResetControl
+    };
+
+    virtual bool SetControl(PVideoControlInfo::Types type, int value, ControlMode mode);
+    virtual const PVideoControlInfo & GetControlInfo(PVideoControlInfo::Types type) const { return m_controlInfo[type]; }
+
+  protected:
+    PVideoControlInfo m_controlInfo[PVideoControlInfo::NumTypes];
+
+  private:
     P_REMOVE_VIRTUAL(PBoolean, GetFrameData(BYTE *, PINDEX *, unsigned &), false);
     P_REMOVE_VIRTUAL(PBoolean, GetFrameDataNoDelay(BYTE *, PINDEX *, unsigned &), false);
 };
@@ -1165,22 +1193,30 @@ class PVideoInputDevice : public PVideoDevice
 // declare macros and structures needed for video input plugins
 //
 
-template <class className> class PVideoInputPluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject *    CreateInstance(int /*userData*/) const { return new className; }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const { return className::GetInputDeviceNames(); }
-    virtual bool         GetDeviceCapabilities(const PString & deviceName, void * caps) const
-      { return className::GetDeviceCapabilities(deviceName, (PVideoInputDevice::Capabilities *)caps); }
-};
+PCREATE_PLUGIN_DEVICE(PVideoInputDevice);
 
-#define PCREATE_VIDINPUT_PLUGIN(name) \
-  static PVideoInputPluginServiceDescriptor<PVideoInputDevice_##name> PVideoInputDevice_##name##_descriptor; \
-  PCREATE_PLUGIN(name, PVideoInputDevice, &PVideoInputDevice_##name##_descriptor)
+#define PCREATE_VIDINPUT_PLUGIN_EX(name, extra) \
+    PCREATE_PLUGIN(name, PVideoInputDevice, PVideoInputDevice_##name, PPlugin_PVideoInputDevice, \
+      virtual PStringArray GetDeviceNames(P_INT_PTR /*userData*/) const { return PVideoInputDevice_##name::GetInputDeviceNames(); } \
+      virtual bool GetDeviceCapabilities(const PString & deviceName, void * caps) const { return PVideoInputDevice_##name::GetDeviceCapabilities(deviceName, (PVideoInputDevice::Capabilities *)caps); } \
+      extra)
+
+#define PCREATE_VIDINPUT_PLUGIN(name) PCREATE_VIDINPUT_PLUGIN_EX(name, )
+
+
+#define P_FAKE_VIDEO_DRIVER         "FakeVideo"
+#define P_FAKE_VIDEO_PREFIX         "Fake/"
+#define P_FAKE_VIDEO_MOVING_BLOCKS  P_FAKE_VIDEO_PREFIX"MovingBlocks"
+#define P_FAKE_VIDEO_MOVING_LINE    P_FAKE_VIDEO_PREFIX"MovingLine"
+#define P_FAKE_VIDEO_BOUNCING_BOXES P_FAKE_VIDEO_PREFIX"BouncingBoxes"
+#define P_FAKE_VIDEO_SOLID_COLOUR   P_FAKE_VIDEO_PREFIX"SolidColour"
+#define P_FAKE_VIDEO_TEXT           P_FAKE_VIDEO_PREFIX"Text"
+#define P_FAKE_VIDEO_NTSC           P_FAKE_VIDEO_PREFIX"NTSCTest"
 
 PPLUGIN_STATIC_LOAD(FakeVideo, PVideoInputDevice);
 
 #ifdef P_APPSHARE
+  #define P_APPLICATION_VIDEO_DRIVER "Application"
   PPLUGIN_STATIC_LOAD(Application, PVideoInputDevice);
 #endif
 
@@ -1189,40 +1225,60 @@ PPLUGIN_STATIC_LOAD(FakeVideo, PVideoInputDevice);
 #endif
 
 #if P_VIDFILE
-  PPLUGIN_STATIC_LOAD(YUVFile, PVideoInputDevice);
+  #define P_VIDEO_FILE_DRIVER "VideoFile"
+  PPLUGIN_STATIC_LOAD(VideoFile, PVideoInputDevice);
+#endif
+
+#ifdef WIN32
+  #define P_VIDEO_FOR_WINDOWS_DRIVER "VideoForWindows"
+  PPLUGIN_STATIC_LOAD(VideoForWindows, PVideoInputDevice);
 #endif
 
 #ifdef P_DIRECTSHOW
+  #define P_DIRECT_SHOW_DRIVER "DirectShow"
   PPLUGIN_STATIC_LOAD(DirectShow, PVideoInputDevice);
 #endif
 
+#if defined(P_MACOSX)
+  PPLUGIN_STATIC_LOAD(Mac, PVideoInputDevice);
+#endif
+
 
 ////////////////////////////////////////////////////////
 //
 // declare macros and structures needed for video output plugins
 //
 
-template <class className> class PVideoOutputPluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject *    CreateInstance(int /*userData*/) const { return new className; }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const { return className::GetOutputDeviceNames(); }
-};
+PCREATE_PLUGIN_DEVICE(PVideoOutputDevice);
+
+#define PCREATE_VIDOUTPUT_PLUGIN_EX(name, extra) \
+    PCREATE_PLUGIN(name, PVideoOutputDevice, PVideoOutputDevice_##name, PPlugin_PVideoOutputDevice, \
+      virtual PStringArray GetDeviceNames(P_INT_PTR /*userData*/) const { return PVideoOutputDevice_##name::GetOutputDeviceNames(); } \
+    extra)
+
+#define PCREATE_VIDOUTPUT_PLUGIN(name) PCREATE_VIDOUTPUT_PLUGIN_EX(name,)
+
+#define P_NULL_VIDEO_DRIVER "NULLOutput"
+#define P_NULL_VIDEO_DEVICE "Null Video Out"
+PPLUGIN_STATIC_LOAD(NULLOutput, PVideoOutputDevice);
 
-#define PCREATE_VIDOUTPUT_PLUGIN(name) \
-  static PVideoOutputPluginServiceDescriptor<PVideoOutputDevice_##name> PVideoOutputDevice_##name##_descriptor; \
-  PCREATE_PLUGIN(name, PVideoOutputDevice, &PVideoOutputDevice_##name##_descriptor)
 
 #if P_VFW_CAPTURE
 #if _WIN32
+  #define P_MSWIN_VIDEO_DRIVER "Window"
+  #define P_MSWIN_VIDEO_PREFIX "MSWIN"
+  #define P_MSWIN_VIDEO_DEVICE(x,y,width,height) P_MSWIN_VIDEO_PREFIX " X=" x " Y=" y " WIDTH=" width " HEIGHT=" height
   PPLUGIN_STATIC_LOAD(Window, PVideoOutputDevice);
 #endif
 #endif
 
 #if P_SDL
-  #if P_MACOSX
+  #if defined(P_MACOSX) || defined(P_IOS)
     #include <SDL_main.h>
   #endif
+  #define P_SDL_VIDEO_DRIVER "SDL"
+  #define P_SDL_VIDEO_PREFIX "SDL"
+  #define P_SDL_VIDEO_DEVICE(x,y,width,height) P_SDL_VIDEO_PREFIX " X=" x " Y=" y " WIDTH=" width " HEIGHT=" height
   PPLUGIN_STATIC_LOAD(SDL, PVideoOutputDevice);
 #endif
 
