@@ -18,15 +18,14 @@
  * Classes to support streaming video input from IEEE 1394 cameras.
  * Detailed explanation can be found at src/ptlib/unix/video4dc1394.cxx
  *
- * $Revision: 20385 $
+ * $Revision: 24177 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2010-04-05 12:52:04 +0100 (Mon, 05 Apr 2010) $
  */
 
 
-#ifndef _PVIDEOIO1394DC
-
-#define _PVIDEOIO1394DC
+#ifndef PTLIB_VIDEOIO1394DC_H
+#define PTLIB_VIDEOIO1394DC_H
 
 #ifdef __GNUC__
 #pragma interface
@@ -54,7 +53,7 @@ class PVideoInput1394DcDevice : public PVideoInputDevice
       */
     PBoolean Open(
       const PString & deviceName,   /// Device name to open
-      PBoolean startImmediate = PTrue    /// Immediately start device
+      PBoolean startImmediate = true    /// Immediately start device
     );
 
     /**Determine of the device is currently open.
@@ -222,7 +221,8 @@ class PVideoInput1394DcDevice : public PVideoInputDevice
     unsigned     desiredFrameHeight;
 };
 
-#endif
+
+#endif // PTLIB_VIDEOIO1394DC_H
 
 
 // End Of File ///////////////////////////////////////////////////////////////
