@@ -23,17 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29745 $
+ * $Revision: 31310 $
  * $Author: rjongbloed $
- * $Date: 2013-05-19 22:52:54 -0500 (Sun, 19 May 2013) $
+ * $Date: 2014-01-29 10:27:09 +0000 (Wed, 29 Jan 2014) $
  */
 
 #ifndef PTLIB_WXSTRING_H
 #define PTLIB_WXSTRING_H
 
-#ifdef P_USE_PRAGMA
-#pragma interface
-#endif
 
 /**This class defines a class to bridge WX Widgets strings to PTLib strings.
  */
@@ -84,6 +81,7 @@ class PwxString : public wxString
 #if wxUSE_UNICODE
     inline PString p_str() const { return ToUTF8().data(); }
     inline operator PString() const { return ToUTF8().data(); }
+    inline operator PCaselessString() const { return ToUTF8().data(); }
     inline operator PFilePath() const { return ToUTF8().data(); }
 #if defined(PTLIB_PURL_H) && defined(P_URL)
     inline operator PURL() const { return ToUTF8().data(); }
@@ -96,6 +94,7 @@ class PwxString : public wxString
 #else
     inline PString p_str() const { return c_str(); }
     inline operator PString() const { return c_str(); }
+    inline operator PCaselessString() const { return c_str(); }
     inline operator PFilePath() const { return c_str(); }
 #if defined(PTLIB_PURL_H) && defined(P_URL)
     inline operator PURL() const { return c_str(); }
