@@ -27,9 +27,9 @@
  *
  * Contributor(s): /
  *
- * $Revision: 28822 $
- * $Author: ededu $
- * $Date: 2013-01-09 04:46:11 -0600 (Wed, 09 Jan 2013) $
+ * $Revision: 30697 $
+ * $Author: rjongbloed $
+ * $Date: 2013-10-14 04:16:35 +0100 (Mon, 14 Oct 2013) $
  */
 
 #pragma implementation "sound_alsa.h"
@@ -47,42 +47,16 @@ PMutex dictionaryMutex;
 ///////////////////////////////////////////////////////////////////////////////
 
 PSoundChannelALSA::PSoundChannelALSA()
+  : mNumChannels(1)
+  , mSampleRate(8000)
+  , mBitsPerSample(16)
+  , isInitialised(false)
+  , pcm_handle(NULL)
+  , card_nr(-1)
+  , m_bufferSize(320)
+  , m_bufferCount(2)
+  , frameBytes(2)
 {
-  card_nr = 0;
-  pcm_handle = NULL;
-}
-
-
-PSoundChannelALSA::PSoundChannelALSA(const PString & device,
-                                          Directions dir,
-                                            unsigned numChannels,
-                                            unsigned sampleRate,
-                                            unsigned bitsPerSample)
-{
-  card_nr = 0;
-  pcm_handle = NULL;
-  Open(device, dir, numChannels, sampleRate, bitsPerSample);
-}
-
-
-void PSoundChannelALSA::Construct()
-{
-//  enum _snd_pcm_format val;
-//
-//#if PBYTE_ORDER == PLITTLE_ENDIAN
-//  val = (mBitsPerSample == 16) ? SND_PCM_FORMAT_S16_LE : SND_PCM_FORMAT_U8;
-//#else
-//  val = (mBitsPerSample == 16) ? SND_PCM_FORMAT_S16_BE : SND_PCM_FORMAT_U8;
-//#endif
-//
-  frameBytes = 2;
-  m_bufferSize = 320; // 20 ms worth of 8kHz data
-  m_bufferCount = 2;  // double buffering
-
-  card_nr = 0;
-  pcm_handle = NULL;
-  os_handle = -1;
-  isInitialised = false;
 }
 
 
@@ -172,6 +146,7 @@ PStringArray PSoundChannelALSA::GetDeviceNames(Directions dir)
       devices += it->first;
   }
 
+  PTRACE2(5, NULL, "ALSA\t" << dir << " devices: " << setfill(',') << devices);
   return devices;
 }
 
@@ -185,59 +160,54 @@ PString PSoundChannelALSA::GetDefaultDevice(Directions dir)
 }
 
 
-PBoolean PSoundChannelALSA::Open(const PString & devName,
-                                      Directions dir,
-                                        unsigned numChannels,
-                                        unsigned sampleRate,
-                                        unsigned bitsPerSample)
+bool PSoundChannelALSA::Open(const Params & params)
 {
   Close();
 
-  direction = dir;
-  mNumChannels = numChannels;
-  mSampleRate = sampleRate;
-  mBitsPerSample = bitsPerSample;
-
-  Construct();
-
   PWaitAndSignal m(device_mutex);
 
+  activeDirection = params.m_direction;
+  mNumChannels = params.m_channels;
+  mSampleRate = params.m_sampleRate;
+  mBitsPerSample = params.m_bitsPerSample;
+
+  device = params.m_device;
+
   PString real_device_name;
-  if (devName == "Default") {
+  if (params.m_device == "Default") {
     real_device_name = "default";
     card_nr = -2;
   }
   else {
-    PStringToOrdinal & devices = dir == Recorder ? capture_devices : playback_devices;
+    PStringToOrdinal & devices = activeDirection == Recorder ? capture_devices : playback_devices;
     if (devices.IsEmpty())
-      UpdateDictionary(dir);
+      UpdateDictionary(activeDirection);
 
-    POrdinalKey * index = devices.GetAt(devName);
+    POrdinalKey * index = devices.GetAt(params.m_device);
     if (index == NULL) {
       PTRACE(1, "ALSA\tDevice not found");
       return false;
     }
 
-    real_device_name = "plughw:" + PString(*index);
     card_nr = *index;
+    real_device_name.sprintf("plughw:%i", card_nr);
   }
 
   /* Open in NONBLOCK mode */
   if (snd_pcm_open(&pcm_handle,
                    real_device_name,
-                   dir == Recorder ? SND_PCM_STREAM_CAPTURE : SND_PCM_STREAM_PLAYBACK,
+                   activeDirection == Recorder ? SND_PCM_STREAM_CAPTURE : SND_PCM_STREAM_PLAYBACK,
                    SND_PCM_NONBLOCK) < 0) {
-    PTRACE(1, "ALSA\tOpen Failed");
+    PTRACE(1, "ALSA\tOpen failed for \"" << params.m_device << "\", card=" << card_nr);
     return false;
   }
 
   snd_pcm_nonblock(pcm_handle, 0);
 
-  /* save internal parameters */
-  device = real_device_name;
+  if (!SetHardwareParams())
+    return false;
 
-  Setup();
-  PTRACE(3, "ALSA\tDevice " << device << " Opened");
+  PTRACE(3, "ALSA\tDevice \"" << device << "\", card=" << card_nr << " opened");
   os_handle=1;
 
   return true;
@@ -245,8 +215,8 @@ PBoolean PSoundChannelALSA::Open(const PString & devName,
 
 bool PSoundChannelALSA::SetHardwareParams()
 {
-  PTRACE(4,"ALSA\tSetHardwareParams " << ((direction == Player) ? "Player" : "Recorder") << " channels=" << mNumChannels
-	   << " sample rate=" << mSampleRate);
+  if (isInitialised)
+    return true;
 
   if (!pcm_handle)
     return SetErrorValues(NotOpen, EBADF);
@@ -259,73 +229,79 @@ bool PSoundChannelALSA::SetHardwareParams()
 #endif
 
   frameBytes = (mNumChannels * (snd_pcm_format_width(sndFormat) / 8));
-
   if (frameBytes == 0)
     frameBytes = 2;
+
+  PTRACE(4,"ALSA\tSetHardwareParams " << ((activeDirection == Player) ? "Player" : "Recorder")
+         << " channels=" << mNumChannels
+         << " sample rate=" << mSampleRate
+         << " buffer size=" << m_bufferSize
+         << " buffer count=" << m_bufferCount
+         << " sndFormat=" << sndFormat
+         << " frameBytes=" << frameBytes);
+
   int err;
 
   // Finally set the hardware parameters
+  snd_pcm_hw_params_t *hw_params = NULL;
+  snd_pcm_hw_params_alloca(&hw_params);
+
+  if ((err = snd_pcm_hw_params_any(pcm_handle, hw_params)) < 0) {
+    PTRACE(1, "ALSA\tCannot initialize hardware parameter structure: " << snd_strerror(err));
+    return false;
+  }
+
+
+  if ((err = snd_pcm_hw_params_set_access(pcm_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
+    PTRACE(1, "ALSA\tCannot set access type: " <<  snd_strerror(err));
+    return false;
+  }
+
+  if ((err = snd_pcm_hw_params_set_format(pcm_handle, hw_params, sndFormat)) < 0) {
+    PTRACE(1, "ALSA\tCannot set sample format: " << snd_strerror(err));
+    return false;
+  }
+
+
+  if ((err = snd_pcm_hw_params_set_channels(pcm_handle, hw_params, mNumChannels)) < 0) {
+    PTRACE(1, "ALSA\tCannot set channel count: " << snd_strerror(err));
+    return false;
+  }
+
+  if ((err = snd_pcm_hw_params_set_rate_near(pcm_handle, hw_params, &mSampleRate, NULL)) < 0) {
+    PTRACE(1, "ALSA\tCannot set sample rate: " << snd_strerror(err));
+    return false;
+  }
+
+  int dir = 0;
+  int totalBufferSize = m_bufferSize*m_bufferCount;
+  snd_pcm_uframes_t desiredPeriodSize = m_bufferSize/frameBytes;
+
+  /* use of get function (ie. snd_pcm_hw_params_get_period_size) and the check as done before was, in my opinion, was pretty unuseful
+      because actually the set function (ie.snd_pcm_hw_params_set_period_size_near) returns the real set value
+      in the argument passed (ie. desiredPeriodSize) */
+  if ((err = snd_pcm_hw_params_set_period_size_near(pcm_handle, hw_params, &desiredPeriodSize, &dir)) < 0) {
+      PTRACE(1, "ALSA\tCannot set period size: " << snd_strerror(err));
+  }
+  else {
+      PTRACE(5, "ALSA\tSuccessfully set period size to " << desiredPeriodSize);
+  }
+
+  /* i experimented (3 different sound cards) that is better to rounds value to the nearest integer to avoid buffer underrun/overrun */
+  unsigned desiredPeriods = (unsigned)(((float)totalBufferSize / (float)(desiredPeriodSize*frameBytes))+0.5);
+  if (desiredPeriods < 2)
+    desiredPeriods = 2;
+
+  if ((err = (int) snd_pcm_hw_params_set_periods_near(pcm_handle, hw_params, &desiredPeriods, &dir)) < 0) {
+    PTRACE(1, "ALSA\tCannot set periods to: " << snd_strerror(err));
+  }
+  else {
+    PTRACE(5, "ALSA\tSuccessfully set periods to " << desiredPeriods);
+  }
+
   for (unsigned retry = 0; retry < 100; ++retry) {
-    snd_pcm_hw_params_t *hw_params = NULL;
-    snd_pcm_hw_params_alloca(&hw_params);
-
-    if ((err = snd_pcm_hw_params_any(pcm_handle, hw_params)) < 0) {
-      PTRACE(1, "ALSA\tCannot initialize hardware parameter structure: " << snd_strerror(err));
-      return false;
-    }
-
-
-    if ((err = snd_pcm_hw_params_set_access(pcm_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
-      PTRACE(1, "ALSA\tCannot set access type: " <<  snd_strerror(err));
-      return false;
-    }
-
-
-    if ((err = snd_pcm_hw_params_set_format(pcm_handle, hw_params, sndFormat)) < 0) {
-      PTRACE(1, "ALSA\tCannot set sample format: " << snd_strerror(err));
-      return false;
-    }
-
-
-    if ((err = snd_pcm_hw_params_set_channels(pcm_handle, hw_params, mNumChannels)) < 0) {
-      PTRACE(1, "ALSA\tCannot set channel count: " << snd_strerror(err));
-      return false;
-    }
-
-    if ((err = snd_pcm_hw_params_set_rate_near(pcm_handle, hw_params, &mSampleRate, NULL)) < 0) {
-      PTRACE(1, "ALSA\tCannot set sample rate: " << snd_strerror(err));
-      return false;
-    }
-
-    int dir = 0;
-    int totalBufferSize = m_bufferSize*m_bufferCount;
-    snd_pcm_uframes_t desiredPeriodSize = m_bufferSize/frameBytes;
-
-    /* use of get function (ie. snd_pcm_hw_params_get_period_size) and the check as done before was, in my opinion, was pretty unuseful
-       because actually the set function (ie.snd_pcm_hw_params_set_period_size_near) returns the real set value
-       in the argument passed (ie. desiredPeriodSize) */
-
-    if ((err = snd_pcm_hw_params_set_period_size_near(pcm_handle, hw_params, &desiredPeriodSize, &dir)) < 0) {
-       PTRACE(1, "ALSA\tCannot set period size: " << snd_strerror(err));
-    }
-    else {
-       PTRACE(4, "ALSA\tSuccessfully set period size to " << desiredPeriodSize);
-    }
-
-    /* i experimented (3 different sound cards) that is better to rounds value to the nearest integer to avoid buffer underrun/overrun */
-    unsigned desiredPeriods = (unsigned)(((float)totalBufferSize / (float)(desiredPeriodSize*frameBytes))+0.5);
-
-    if (desiredPeriods < 2) desiredPeriods = 2;
-
-    if ((err = (int) snd_pcm_hw_params_set_periods_near(pcm_handle, hw_params, &desiredPeriods, &dir)) < 0) {
-      PTRACE(1, "ALSA\tCannot set periods to: " << snd_strerror(err));
-    }
-    else {
-      PTRACE(4, "ALSA\tSuccessfully set periods to " << desiredPeriods);
-    }
-
     if ((err = snd_pcm_hw_params(pcm_handle, hw_params)) >= 0) {
-      PTRACE(4, "ALSA\tparameters set ok");
+      PTRACE(4, "ALSA\thardware parameters set");
       isInitialised = true;
       return true;
     }
@@ -341,21 +317,6 @@ bool PSoundChannelALSA::SetHardwareParams()
   return false;
 }
 
-PBoolean PSoundChannelALSA::Setup()
-{
-  if (pcm_handle == NULL) {
-    PTRACE(6, "ALSA\tSkipping setup of " << device << " as not open");
-    return false;
-  }
-
-  if (isInitialised) {
-    PTRACE(6, "ALSA\tSkipping setup of " << device << " as instance already initialised");
-    return true;
-  }
-
-  return SetHardwareParams();
-}
-
 
 PBoolean PSoundChannelALSA::Close()
 {
@@ -365,11 +326,12 @@ PBoolean PSoundChannelALSA::Close()
   if (!pcm_handle)
     return false;
 
-  PTRACE(3, "ALSA\tClosing " << device);
+  PTRACE(3, "ALSA\tClosing \"" << device << "\", card=" << card_nr);
   snd_pcm_close(pcm_handle);
   pcm_handle = NULL;
   os_handle = -1;
   isInitialised = false;
+  card_nr = 0;
 
   return true;
 }
@@ -381,7 +343,7 @@ PBoolean PSoundChannelALSA::Write(const void *buf, PINDEX len)
 
   PWaitAndSignal m(device_mutex);
 
-  if ((!isInitialised && !Setup()) || !len || !pcm_handle)
+  if (!SetHardwareParams())
     return false;
 
   int pos = 0, max_try = 0;
@@ -432,7 +394,7 @@ PBoolean PSoundChannelALSA::Read(void * buf, PINDEX len)
 
   PWaitAndSignal m(device_mutex);
 
-  if ((!isInitialised && !Setup()) || !len || !pcm_handle)
+  if (!SetHardwareParams())
     return false;
 
   memset((char *) buf, 0, len);
@@ -518,7 +480,7 @@ unsigned PSoundChannelALSA::GetSampleSize() const
 PBoolean PSoundChannelALSA::SetBuffers(PINDEX size, PINDEX count)
 {
   PTRACE(4,"ALSA\tSetBuffers direction=" <<
-	         ((direction == Player) ? "Player" : "Recorder") << " size=" << size << " count=" << count);
+	         ((activeDirection == Player) ? "Player" : "Recorder") << " size=" << size << " count=" << count);
 
   m_bufferSize = size;
   m_bufferCount = count;
@@ -536,63 +498,6 @@ PBoolean PSoundChannelALSA::GetBuffers(PINDEX & size, PINDEX & count)
 }
 
 
-PBoolean PSoundChannelALSA::PlaySound(const PSound & sound, PBoolean wait)
-{
-  if (!pcm_handle)
-    return SetErrorValues(NotOpen, EBADF);
-
-  if (!Write((const BYTE *)sound, sound.GetSize()))
-    return false;
-
-  if (wait)
-    return WaitForPlayCompletion();
-
-  return true;
-}
-
-
-PBoolean PSoundChannelALSA::PlayFile(const PFilePath & filename, PBoolean wait)
-{
-#if P_WAVFILE
-  BYTE buffer [512];
-  PTRACE(1, "ALSA\tPlayFile " << filename);
-
-  if (!pcm_handle)
-    return SetErrorValues(NotOpen, EBADF);
-
-  /* use PWAVFile instead of PFile -> skips wav header bytes */
-
-  PWAVFile file(filename, PFile::ReadOnly, PFile::MustExist, PWAVFile::fmt_NotKnown);
-  snd_pcm_prepare(pcm_handle);
-
-  if (!file.IsOpen())
-    return false;
-
-  for (;;) {
-    if (!file.Read(buffer, 512))
-      break;
-
-    PINDEX len = file.GetLastReadCount();
-
-    if (len == 0)
-      break;
-
-    if (!Write(buffer, len))
-      break;
-  }
-
-  file.Close();
-
-  if (wait)
-    return WaitForPlayCompletion();
-
-  return true;
-#else
-  return false;
-#endif // P_WAVFILE
-}
-
-
 PBoolean PSoundChannelALSA::HasPlayCompleted()
 {
   if (!pcm_handle)
@@ -613,18 +518,6 @@ PBoolean PSoundChannelALSA::WaitForPlayCompletion()
 }
 
 
-PBoolean PSoundChannelALSA::RecordSound(PSound & sound)
-{
-  return false;
-}
-
-
-PBoolean PSoundChannelALSA::RecordFile(const PFilePath & filename)
-{
-  return false;
-}
-
-
 PBoolean PSoundChannelALSA::StartRecording()
 {
   return false;
@@ -662,7 +555,7 @@ PBoolean PSoundChannelALSA::Abort()
   if (!pcm_handle)
     return false;
 
-  PTRACE(4, "ALSA\tAborting " << device);
+  PTRACE(4, "ALSA\tAborting \"" << device << "\", card=" << card_nr);
   if ((r = snd_pcm_drain(pcm_handle)) < 0) {
     PTRACE(1, "ALSA\tCannot abort" << snd_strerror(r));
     return false;
@@ -746,10 +639,10 @@ PBoolean PSoundChannelALSA::Volume(PBoolean set, unsigned set_vol, unsigned &get
   }
 
   do {
-    snd_mixer_selem_id_set_name(sid, (direction == Player)?play_mix_name[i]:rec_mix_name[i]);
+    snd_mixer_selem_id_set_name(sid, (activeDirection == Player)?play_mix_name[i]:rec_mix_name[i]);
     elem = snd_mixer_find_selem(handle, sid);
     i++;
-  } while (!elem && ((direction == Player && play_mix_name[i] != NULL) || (direction == Recorder && rec_mix_name[i] != NULL)));
+  } while (!elem && ((activeDirection == Player && play_mix_name[i] != NULL) || (activeDirection == Recorder && rec_mix_name[i] != NULL)));
 
   if (!elem) {
     PTRACE(1, "ALSA\tUnable to find simple control.");
@@ -758,7 +651,7 @@ PBoolean PSoundChannelALSA::Volume(PBoolean set, unsigned set_vol, unsigned &get
   }
 
   if (set) {
-    if (direction == Player) {
+    if (activeDirection == Player) {
       snd_mixer_selem_get_playback_volume_range(elem, &pmin, &pmax);
       vol = (set_vol * (pmax?pmax:31)) / 100;
       snd_mixer_selem_set_playback_volume_all(elem, vol);
@@ -771,7 +664,7 @@ PBoolean PSoundChannelALSA::Volume(PBoolean set, unsigned set_vol, unsigned &get
     PTRACE(4, "ALSA\tSet volume to " << vol);
   }
   else {
-    if (direction == Player) {
+    if (activeDirection == Player) {
       snd_mixer_selem_get_playback_volume_range(elem, &pmin, &pmax);
       snd_mixer_selem_get_playback_volume(elem, SND_MIXER_SCHN_FRONT_LEFT, &vol);
     }
