@@ -8,25 +8,11 @@
 class PSoundChannelALSA : public PSoundChannel {
  public:
   PSoundChannelALSA();
-  void Construct();
-  PSoundChannelALSA(
-    const PString &device,
-    PSoundChannel::Directions dir,
-    unsigned numChannels,
-    unsigned sampleRate,
-    unsigned bitsPerSample
-  );
   ~PSoundChannelALSA();
   static PStringArray GetDeviceNames(PSoundChannel::Directions);
   static PString GetDefaultDevice(PSoundChannel::Directions);
-  PBoolean Open(
-    const PString & device,
-    Directions dir,
-    unsigned numChannels,
-    unsigned sampleRate,
-    unsigned bitsPerSample
-  );
-  PBoolean Setup();
+  bool Open(const Params & params);
+  PString GetName() const { return device; }
   PBoolean Close();
   PBoolean Write(const void * buf, PINDEX len);
   PBoolean Read(void * buf, PINDEX len);
@@ -36,12 +22,8 @@ class PSoundChannelALSA : public PSoundChannel {
   unsigned GetSampleSize() const;
   PBoolean SetBuffers(PINDEX size, PINDEX count);
   PBoolean GetBuffers(PINDEX & size, PINDEX & count);
-  PBoolean PlaySound(const PSound & sound, PBoolean wait);
-  PBoolean PlayFile(const PFilePath & filename, PBoolean wait);
   PBoolean HasPlayCompleted();
   PBoolean WaitForPlayCompletion();
-  PBoolean RecordSound(PSound & sound);
-  PBoolean RecordFile(const PFilePath & filename);
   PBoolean StartRecording();
   PBoolean IsRecordBufferFull();
   PBoolean AreAllRecordBuffersFull();
@@ -57,7 +39,6 @@ class PSoundChannelALSA : public PSoundChannel {
   bool SetHardwareParams();
   PBoolean Volume(PBoolean, unsigned, unsigned &);
 
-  PSoundChannel::Directions direction;
   PString device;
   unsigned mNumChannels;
   unsigned mSampleRate;
