@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 30696 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2013-10-14 04:09:25 +0100 (Mon, 14 Oct 2013) $
  */
 
 #pragma implementation "sound.h"
@@ -45,23 +45,9 @@ PCREATE_SOUND_PLUGIN(ESD, PSoundChannelESD);
 ///////////////////////////////////////////////////////////////////////////////
 
 PSoundChannelESD::PSoundChannelESD()
-{
-  Construct();
-}
-
-
-PSoundChannelESD::PSoundChannelESD(const PString & device,
-                             Directions dir,
-                             unsigned numChannels,
-                             unsigned sampleRate,
-                             unsigned bitsPerSample)
-{
-  Construct();
-  Open(device, dir, numChannels, sampleRate, bitsPerSample);
-}
-
-
-void PSoundChannelESD::Construct()
+  : mNumChannels(1)
+  , mSampleRate(8000)
+  , mBitsPerSample(16)
 {
 }
 
@@ -72,13 +58,10 @@ PSoundChannelESD::~PSoundChannelESD()
 }
 
 
-PStringArray PSoundChannelESD::GetDeviceNames(Directions /*dir*/)
+PStringArray PSoundChannelESD::GetDeviceNames(Directions dir)
 {
-  PStringArray array;
-
-  array[0] = "ESound";
-
-  return array;
+  PTRACE2(5, NULL, "ESound\t" << dir << " devices: ESound");
+  return GetDefaultDevice(dir);
 }
 
 
@@ -87,67 +70,64 @@ PString PSoundChannelESD::GetDefaultDevice(Directions /*dir*/)
   return "ESound";
 }
 
-PBoolean PSoundChannelESD::Open(const PString & device,
-                         Directions dir,
-                         unsigned numChannels,
-                         unsigned sampleRate,
-                         unsigned bitsPerSample)
+bool PSoundChannelESD::Open(const Params & params)
 {
-  int bits, channels, rate, mode, func;
-  esd_format_t format;
+  int mode;
+  esd_format_t format = ESD_STREAM;
   char *host = NULL, *name = NULL;
 
   Close();
 
+  mSampleRate = params.m_sampleRate;
+
   // make sure we have proper bits per sample
-  switch (bitsPerSample) {
+  switch (params.m_bitsPerSample) {
   case 16:
-    bits = ESD_BITS16;
+    format |= ESD_BITS16;
     break;
   case 8:
-    bits = ESD_BITS8;
+    format |= ESD_BITS8;
     break;
   default:
     return (false);
   }
+  mBitsPerSample = params.m_bitsPerSample;
 
   // make sure we have proper number of channels
-  switch (numChannels) {
+  switch (params.m_channels) {
   case 2:
-    channels = ESD_STEREO;
+    format |= ESD_STEREO;
     break;
   case 1:
-    channels = ESD_MONO;
+    format |= ESD_MONO;
     break;
   default:
     return (false);
   }
-
-  rate = sampleRate;
-  mode = ESD_STREAM;
+  mNumChannels = params.m_channels;
 
   // a separate stream for Player and Recorder
-  switch (dir) {
+  switch (params.m_direction) {
   case Recorder:
-    func = ESD_RECORD;
+    format |= ESD_RECORD;
     break;
   case Player:
-    func = ESD_PLAY;
+    format |= ESD_PLAY;
     break;
   default:
     return (false);
   }
+  activeDirection = params.m_direction;
 
-  format = bits | channels | mode | func;
-  if (dir == Recorder) 
-    os_handle = esd_record_stream_fallback( format, rate, host, name );
+  if (params.m_direction == Recorder) 
+    os_handle = esd_record_stream_fallback( format, mSampleRate, host, name );
   else
-    os_handle = esd_play_stream_fallback( format, rate, host, name );
+    os_handle = esd_play_stream_fallback( format, mSampleRate, host, name );
 
   if ( os_handle <= 0 ) 
     return (false);
 
-  return SetFormat(numChannels, sampleRate, bitsPerSample);
+  return SetFormat(params.m_channels, params.m_sampleRate, params.m_bitsPerSample);
 }
 
 PBoolean PSoundChannelESD::SetVolume(unsigned newVal)
@@ -216,7 +196,7 @@ PBoolean PSoundChannelESD::Write(const void * buf, PINDEX len)
     // Sends data to esd.
     rval = ::write(os_handle, buf, len);
     if (rval > 0) {
-#if defined(P_MACOSX)
+#if 0 //defined(P_MACOSX)
       // Mac OS X's esd has a big input buffer so we need to simulate
       // writing data out at the correct rate.
       writeDelay.Delay(len/16);
@@ -229,26 +209,6 @@ PBoolean PSoundChannelESD::Write(const void * buf, PINDEX len)
 }
 
 
-PBoolean PSoundChannelESD::PlaySound(const PSound & sound, PBoolean wait)
-{
-  Abort();
-
-  if (!Write((const BYTE *)sound, sound.GetSize()))
-    return false;
-
-  if (wait)
-    return WaitForPlayCompletion();
-
-  return true;
-}
-
-
-PBoolean PSoundChannelESD::PlayFile(const PFilePath & filename, PBoolean wait)
-{
-  return true;
-}
-
-
 PBoolean PSoundChannelESD::HasPlayCompleted()
 {
   return false;
@@ -277,18 +237,6 @@ PBoolean PSoundChannelESD::Read(void * buf, PINDEX len)
 }
 
 
-PBoolean PSoundChannelESD::RecordSound(PSound & sound)
-{
-  return true;
-}
-
-
-PBoolean PSoundChannelESD::RecordFile(const PFilePath & filename)
-{
-  return true;
-}
-
-
 PBoolean PSoundChannelESD::StartRecording()
 {
   return (true);
