@@ -26,15 +26,22 @@
  *
  * Contributor(s): Loopback feature: Philip Edelbrock <phil@netroedge.com>.
  *
- * $Revision: 28484 $
+ * $Revision: 33254 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2015-01-14 03:53:59 +0000 (Wed, 14 Jan 2015) $
  */
 
 #pragma implementation "sound_oss.h"
 
 #include "sound_oss.h"
 
+#include <sys/ioctl.h>
+#if defined(P_SOLARIS)
+  #include <sys/filio.h>
+#endif
+
+
+
 PCREATE_SOUND_PLUGIN(OSS, PSoundChannelOSS);
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -62,24 +69,6 @@ SoundHandleEntry::SoundHandleEntry()
 
 PSoundChannelOSS::PSoundChannelOSS()
 {
-  PSoundChannelOSS::Construct();
-}
-
-
-PSoundChannelOSS::PSoundChannelOSS(const PString & device,
-                                        Directions dir,
-                                          unsigned numChannels,
-                                          unsigned sampleRate,
-                                          unsigned bitsPerSample)
-{
-  Construct();
-  Open(device, dir, numChannels, sampleRate, bitsPerSample);
-}
-
-
-void PSoundChannelOSS::Construct()
-{
-  os_handle = -1;
 }
 
 
@@ -207,7 +196,7 @@ static void CollectSoundDevices(PDirectory devdir, POrdinalToString & dsp, POrdi
 }
 
 
-PStringArray PSoundChannelOSS::GetDeviceNames(Directions /*dir*/)
+PStringArray PSoundChannelOSS::GetDeviceNames(Directions dir)
 {
   // First locate sound cards. On Linux with devfs and on the other platforms
   // (eg FreeBSD), we search for filenames with dspN or mixerN.
@@ -268,6 +257,7 @@ PStringArray PSoundChannelOSS::GetDeviceNames(Directions /*dir*/)
     }
   }
 
+  PTRACE2(5, NULL, "OSS\t" << dir << " devices: " << setfill(',') << devices);
   return devices;
 }
 
@@ -282,11 +272,8 @@ PString PSoundChannelOSS::GetDefaultDevice(Directions dir)
   return devicenames[0];
 }
 
-PBoolean PSoundChannelOSS::Open(const PString & _device,
-                              Directions _dir,
-                                unsigned _numChannels,
-                                unsigned _sampleRate,
-                                unsigned _bitsPerSample)
+
+bool PSoundChannelOSS::Open(const Params & params)
 {
 
   Close();
@@ -296,12 +283,12 @@ PBoolean PSoundChannelOSS::Open(const PString & _device,
   PWaitAndSignal mutex(dictMutex);
 
   // make the direction value 1 or 2
-  int dir = _dir + 1;
+  int dir = params.m_direction + 1;
 
   // if this device is in the dictionary
-  if (handleDict().Contains(_device)) {
+  if (handleDict().Contains(params.m_device)) {
 
-    SoundHandleEntry & entry = handleDict()[_device];
+    SoundHandleEntry & entry = handleDict()[params.m_device];
 
     // see if the sound channel is already open in this direction
     if ((entry.direction & dir) != 0) {
@@ -317,7 +304,7 @@ PBoolean PSoundChannelOSS::Open(const PString & _device,
     // this is the first time this device has been used
     // open the device in read/write mode always
     // open the device in non-blocking mode to avoid hang if already open
-    os_handle = ::open((const char *)_device, O_RDWR | O_NONBLOCK);
+    os_handle = ::open((const char *)params.m_device, O_RDWR | O_NONBLOCK);
 
     if ((os_handle < 0) && (errno != EWOULDBLOCK)) 
       return ConvertOSError(os_handle);
@@ -328,23 +315,23 @@ PBoolean PSoundChannelOSS::Open(const PString & _device,
 
     // add the device to the dictionary
     SoundHandleEntry * entry = PNEW SoundHandleEntry;
-    handleDict().SetAt(_device, entry); 
+    handleDict().SetAt(params.m_device, entry); 
 
     // save the information into the dictionary entry
     entry->handle        = os_handle;
-    entry->direction     = dir;
-    entry->numChannels   = mNumChannels     = _numChannels;
-    entry->sampleRate    = actualSampleRate = mSampleRate    = _sampleRate;
-    entry->bitsPerSample = mBitsPerSample   = _bitsPerSample;
+    entry->direction     = params.m_direction;
+    entry->numChannels   = mNumChannels     = params.m_channels;
+    entry->sampleRate    = actualSampleRate = mSampleRate    = params.m_sampleRate;
+    entry->bitsPerSample = mBitsPerSample   = params.m_bitsPerSample;
     entry->isInitialised = false;
     entry->fragmentValue = 0x7fff0008;
     entry->resampleRate  = 0;
   }
    
   // save the direction and device
-  direction     = _dir;
-  device        = _device;
-  isInitialised = false;
+  activeDirection = params.m_direction;
+  device          = params.m_device;
+  isInitialised   = false;
 
   return true;
 }
@@ -471,7 +458,7 @@ PBoolean PSoundChannelOSS::Close()
   PAssert((entry = handleDict().GetAt(device)) != NULL, "Unknown sound device \"" + device + "\" found");
 
   // modify the directions bit mask in the dictionary
-  entry->direction ^= (direction+1);
+  entry->direction ^= (activeDirection+1);
 
   // if this is the last usage of this entry, then remove it
   if (entry->direction == 0) {
@@ -744,52 +731,6 @@ PBoolean PSoundChannelOSS::GetBuffers(PINDEX & size, PINDEX & count)
 }
 
 
-PBoolean PSoundChannelOSS::PlaySound(const PSound & sound, PBoolean wait)
-{
-  if (os_handle < 0)
-    return SetErrorValues(NotOpen, EBADF);
-
-  Abort();
-
-  if (!Write((const BYTE *)sound, sound.GetSize()))
-    return false;
-
-  if (wait)
-    return WaitForPlayCompletion();
-
-  return true;
-}
-
-
-PBoolean PSoundChannelOSS::PlayFile(const PFilePath & filename, PBoolean wait)
-{
-  if (os_handle < 0)
-    return SetErrorValues(NotOpen, EBADF);
-
-  PFile file(filename, PFile::ReadOnly);
-  if (!file.IsOpen())
-    return false;
-
-  for (;;) {
-    BYTE buffer[256];
-    if (!file.Read(buffer, 256))
-      break;
-    PINDEX len = file.GetLastReadCount();
-    if (len == 0)
-      break;
-    if (!Write(buffer, len))
-      break;
-  }
-
-  file.Close();
-
-  if (wait)
-    return WaitForPlayCompletion();
-
-  return true;
-}
-
-
 PBoolean PSoundChannelOSS::HasPlayCompleted()
 {
   if (os_handle < 0)
@@ -812,24 +753,6 @@ PBoolean PSoundChannelOSS::WaitForPlayCompletion()
 }
 
 
-PBoolean PSoundChannelOSS::RecordSound(PSound & sound)
-{
-  if (os_handle < 0)
-    return SetErrorValues(NotOpen, EBADF);
-
-  return false;
-}
-
-
-PBoolean PSoundChannelOSS::RecordFile(const PFilePath & filename)
-{
-  if (os_handle < 0)
-    return SetErrorValues(NotOpen, EBADF);
-
-  return false;
-}
-
-
 PBoolean PSoundChannelOSS::StartRecording()
 {
   if (os_handle < 0)
@@ -838,9 +761,16 @@ PBoolean PSoundChannelOSS::StartRecording()
   if (os_handle == 0)
     return true;
 
+#if P_HAS_POLL
+  struct pollfd pfd;
+  pfd.fd = os_handle;
+  pfd.events = POLLIN;
+  return ConvertOSError(::poll(&pfd, 1, -1));
+#else
   P_fd_set fds(os_handle);
   P_timeval instant;
   return ConvertOSError(::select(1, fds, NULL, NULL, instant));
+#endif
 }
 
 
@@ -899,7 +829,7 @@ PBoolean PSoundChannelOSS::SetVolume(unsigned newVal)
 
   int rc, deviceVol = (newVal << 8) | newVal;
 
-  if (direction  == Player) 
+  if (activeDirection == Player) 
     rc = ::ioctl(os_handle, MIXER_WRITE(SOUND_MIXER_VOLUME), &deviceVol);
    else 
     rc = ::ioctl(os_handle, MIXER_WRITE(SOUND_MIXER_MIC), &deviceVol);
@@ -918,7 +848,7 @@ PBoolean  PSoundChannelOSS::GetVolume(unsigned &devVol)
     return false;
   
   int vol, rc;
-  if (direction == Player)
+  if (activeDirection == Player)
     rc = ::ioctl(os_handle, MIXER_READ(SOUND_MIXER_VOLUME), &vol);
   else
     rc = ::ioctl(os_handle, MIXER_READ(SOUND_MIXER_MIC), &vol);
