@@ -26,27 +26,24 @@
  *
  * Contributor(s): Derek Smithies - from a modification of oss module.
  *
- * $Revision: 28819 $
+ * $Revision: 30696 $
  * $Author: rjongbloed $
- * $Date: 2013-01-09 00:17:37 -0600 (Wed, 09 Jan 2013) $
+ * $Date: 2013-10-14 04:09:25 +0100 (Mon, 14 Oct 2013) $
  */
    
 
 #pragma implementation "sound_pulse.h"
 
+#include "sound_pulse.h"
+
 #include <pulse/context.h>
 #include <pulse/error.h>
 #include <pulse/introspect.h>
 #include <pulse/thread-mainloop.h>
 #include <pulse/volume.h>
-//#include <pulse/gccmacro.h>
 #include <pulse/sample.h>
-#include <ptlib.h>
+
 #include <ptclib/random.h>
-#include <ptclib/pwavfile.h>
-
-#include "sound_pulse.h"
-
 
 
 PCREATE_SOUND_PLUGIN(Pulse, PSoundChannelPulse);
@@ -86,7 +83,6 @@ public:
   static void signal() {
     pa_threaded_mainloop_signal(paloop,0);
   }
-
 };
 
 static PulseContext pamain;
@@ -124,33 +120,10 @@ public:
 PSoundChannelPulse::PSoundChannelPulse()
 {
   PTRACE(6, "Pulse\tConstructor for no args");
-  PSoundChannelPulse::Construct();
-  setenv ("PULSE_PROP_media.role", "phone", true);
-}
-
-
-PSoundChannelPulse::PSoundChannelPulse(const PString & device,
-				       Directions dir,
-				       unsigned numChannels,
-				       unsigned sampleRate,
-				       unsigned bitsPerSample)
-{
-  PTRACE(6, "Pulse\tConstructor with many args\n");
-
-  PAssert((bitsPerSample == 16), PInvalidParameter);
-  Construct();
-  ss.rate = sampleRate;
-  ss.channels = numChannels;
-  Open(device, dir, numChannels, sampleRate, bitsPerSample);
-}
-
-
-void PSoundChannelPulse::Construct()
-{
-  PTRACE(6, "Pulse\tConstruct ");
   os_handle = -1;
   s = NULL;
   ss.format =  PA_SAMPLE_S16LE;
+  setenv ("PULSE_PROP_media.role", "phone", true);
 }
 
 
@@ -184,7 +157,6 @@ static void source_info_cb(pa_context *c, const pa_source_info *i, int eol, void
 
 PStringArray PSoundChannelPulse::GetDeviceNames(Directions dir)
 {
-  PTRACE2(6, NULL, "Pulse\tReport devicenames as \"PulseAudio\"");
   PulseLock lock;
   PStringArray devices;
   devices.AppendString("PulseAudio"); // Default device
@@ -197,17 +169,16 @@ PStringArray PSoundChannelPulse::GetDeviceNames(Directions dir)
       pa_context_get_source_info_list(context,source_info_cb,&devices);
   }
   lock.waitFor(operation);
+
+  PTRACE2(5, NULL, "Pulse\t" << dir << " devices: " << setfill(',') << devices);
   return devices;
 }
 
 
 PString PSoundChannelPulse::GetDefaultDevice(Directions dir)
 {
-  PTRACE2(6, NULL, "Pulse\t report default device as \"PulseAudio\"");
-  PStringArray devicenames;
-  devicenames = PSoundChannelPulse::GetDeviceNames(dir);
-
-  return devicenames[0];
+  PStringArray devices = PSoundChannelPulse::GetDeviceNames(dir);
+  return devices[0];
 }
 
 static void stream_notify_cb(pa_stream *s, void *userdata) {
@@ -218,20 +189,15 @@ static void stream_write_cb(pa_stream* s,size_t nbytes,void *userdata) {
   PulseContext::signal();
 }
 
-PBoolean PSoundChannelPulse::Open(const PString & _device,
-				  Directions _dir,
-				  unsigned _numChannels,
-				  unsigned _sampleRate,
-				  unsigned _bitsPerSample)
+bool PSoundChannelPulse::Open(const Params & params)
 {
   PWaitAndSignal m(deviceMutex);
-  PTRACE(6, "Pulse\t Open on device name of " << _device);
+  PTRACE(6, "Pulse\t Open on device name of " << params.m_device);
   Close();
-  direction = _dir;
-  mNumChannels = _numChannels;
-  mSampleRate = _sampleRate;
-  mBitsPerSample = _bitsPerSample;
-  Construct();
+  activeDirection = params.m_direction;
+  mNumChannels = params.m_channels;
+  mSampleRate = params.m_sampleRate;
+  mBitsPerSample = params.m_bitsPerSample;
 
   PulseLock lock;
   char *app = getenv ("PULSE_PROP_application.name");
@@ -240,21 +206,18 @@ PBoolean PSoundChannelPulse::Open(const PString & _device,
     appName << app;
   else
     appName << "PTLib plugin ";
-  if (_dir == Player) 
-    streamName << ::hex << PRandom::Number();
-  else
-    streamName << ::hex << PRandom::Number();
+  streamName << ::hex << PRandom::Number();
 
-  ss.rate = _sampleRate;
-  ss.channels = _numChannels;
+  ss.rate = params.m_sampleRate;
+  ss.channels = params.m_channels;
   ss.format =  PA_SAMPLE_S16LE;  
 
   const char* dev;
-  if (_device=="PulseAudio") {
+  if (params.m_device=="PulseAudio") {
     /* Default device */
     dev=NULL;
   } else {
-    dev=_device;
+    dev=params.m_device;
   }
   s=pa_stream_new(context,appName.GetPointer(),&ss,NULL);
   pa_stream_set_state_callback(s,stream_notify_cb,NULL);
@@ -267,7 +230,7 @@ PBoolean PSoundChannelPulse::Open(const PString & _device,
     return false;
   }
 
-  if (_dir == Player) {
+  if (activeDirection == Player) {
     int err=pa_stream_connect_playback(s,dev,NULL,PA_STREAM_NOFLAGS,NULL,NULL);
     if (err) {
       PTRACE(2, ": pa_connect_playback() failed: " << pa_strerror(err));
@@ -450,53 +413,6 @@ PBoolean PSoundChannelPulse::GetBuffers(PINDEX & size, PINDEX & count)
 }
 
 
-PBoolean PSoundChannelPulse::PlaySound(const PSound & sound, PBoolean wait)
-{
-
-  return false;
-}
-
-
-PBoolean PSoundChannelPulse::PlayFile(const PFilePath & filename, PBoolean wait)
-{
-#if P_WAVFILE
-  BYTE buffer [512];
-  PTRACE(1, "PULSE\tPlayFile " << filename);
-
-  if (!os_handle)
-    return SetErrorValues(NotOpen, EBADF);
-
-  /* use PWAVFile instead of PFile -> skips wav header bytes */
-  PWAVFile file(filename, PFile::ReadOnly);
-
-  if (!file.IsOpen())
-    return false;
-
-  for (;;) {
-    if (!file.Read(buffer, 512))
-      break;
-
-    PINDEX len = file.GetLastReadCount();
-
-    if (len == 0)
-      break;
-
-    if (!Write(buffer, len))
-      break;
-  }
-
-  file.Close();
-
-  if (wait)
-   return WaitForPlayCompletion();
-
-  return true;
-#else
-  return false;
-#endif
-}
-
-
 PBoolean PSoundChannelPulse::HasPlayCompleted()
 {
   return true;
@@ -509,18 +425,6 @@ PBoolean PSoundChannelPulse::WaitForPlayCompletion()
 }
 
 
-PBoolean PSoundChannelPulse::RecordSound(PSound & sound)
-{
-  return false;
-}
-
-
-PBoolean PSoundChannelPulse::RecordFile(const PFilePath & filename)
-{
-  return false;
-}
-
-
 PBoolean PSoundChannelPulse::StartRecording()
 {
   return false;
@@ -571,14 +475,14 @@ PBoolean PSoundChannelPulse::SetVolume(unsigned newVal)
     int dev=pa_stream_get_device_index(s);
     pa_operation* operation;
     pa_cvolume volume;
-    if (direction==Player) {
+    if (activeDirection==Player) {
       operation=pa_context_get_sink_info_by_index(context,dev,sink_volume_cb,&volume);
     } else {
       operation=pa_context_get_source_info_by_index(context,dev,source_volume_cb,&volume);
     }
     if (!lock.waitFor(operation)) return false;
     pa_cvolume_scale(&volume,newVal*PA_VOLUME_NORM/100);
-    if (direction==Player) {
+    if (activeDirection==Player) {
       pa_context_set_sink_volume_by_index(context,dev,&volume,NULL,NULL);
     } else {
       pa_context_set_source_volume_by_index(context,dev,&volume,NULL,NULL);
@@ -594,7 +498,7 @@ PBoolean  PSoundChannelPulse::GetVolume(unsigned &devVol)
     int dev=pa_stream_get_device_index(s);
     pa_operation* operation;
     pa_cvolume volume;
-    if (direction==Player) {
+    if (activeDirection==Player) {
       operation=pa_context_get_sink_info_by_index(context,dev,sink_volume_cb,&volume);
     } else {
       operation=pa_context_get_source_info_by_index(context,dev,source_volume_cb,&volume);
