@@ -33,20 +33,10 @@ class PSoundChannelPulse: public PSoundChannel
 {
  public:
     PSoundChannelPulse();
-    void Construct();
-    PSoundChannelPulse(const PString &device,
-                     PSoundChannel::Directions dir,
-                     unsigned numChannels,
-                     unsigned sampleRate,
-		     unsigned bitsPerSample);
     ~PSoundChannelPulse();
     static PStringArray GetDeviceNames(PSoundChannel::Directions = Player);
     static PString GetDefaultDevice(PSoundChannel::Directions);
-    PBoolean Open(const PString & _device,
-              Directions _dir,
-              unsigned _numChannels,
-              unsigned _sampleRate,
-              unsigned _bitsPerSample);
+    bool Open(const Params & params);
     PBoolean Setup();
     PBoolean Close();
     PBoolean IsOpen() const;
@@ -60,12 +50,8 @@ class PSoundChannelPulse: public PSoundChannel
     unsigned GetSampleSize() const;
     PBoolean SetBuffers(PINDEX size, PINDEX count);
     PBoolean GetBuffers(PINDEX & size, PINDEX & count);
-    PBoolean PlaySound(const PSound & sound, PBoolean wait);
-    PBoolean PlayFile(const PFilePath & filename, PBoolean wait);
     PBoolean HasPlayCompleted();
     PBoolean WaitForPlayCompletion();
-    PBoolean RecordSound(PSound & sound);
-    PBoolean RecordFile(const PFilePath & filename);
     PBoolean StartRecording();
     PBoolean IsRecordBufferFull();
     PBoolean AreAllRecordBuffersFull();
@@ -80,7 +66,6 @@ class PSoundChannelPulse: public PSoundChannel
     unsigned mSampleRate;
     unsigned mBitsPerSample;
     unsigned actualSampleRate;
-    Directions direction;
     PString device;
     PBoolean isInitialised;
     unsigned resampleRate;
