@@ -30,14 +30,15 @@
  * Contributor(s): Guilhem Tardy (gtardy@salyens.com)
  *  Nicola Orru' <nigu@itadinanta.it>
  *
- * $Revision: 29349 $
- * $Author: rjongbloed $
- * $Date: 2013-03-27 16:18:26 -0500 (Wed, 27 Mar 2013) $
+ * $Revision: 32417 $
+ * $Author: ededu $
+ * $Date: 2014-08-04 10:14:13 +0100 (Mon, 04 Aug 2014) $
  */
 
 #pragma implementation "vidinput_v4l2.h"
 
 #include "vidinput_v4l2.h"
+#include <sys/ioctl.h>
 #include <sys/utsname.h>
 
 PCREATE_VIDINPUT_PLUGIN(V4L2);
@@ -144,7 +145,7 @@ V4L2Names::Update()
   }
   if (procvideo) {
     PTRACE(2,"PV4L2Plugin\tdetected device metadata at "<<*procvideo);
-    if (((kernelVersion==K2_6 && procvideo->Open(PFileInfo::SubDirectory)) ||
+    if (((kernelVersion==K2_6 && procvideo->Open(PFileInfo::SubDirectory|PFileInfo::SymbolicLink)) ||
         (procvideo->Open(PFileInfo::RegularFile)))) {
       do {
         entry = procvideo->GetEntryName();
@@ -611,6 +612,7 @@ PBoolean PVideoInputDevice_V4L2::SetColourFormat(const PString & newFormat)
   }
 
   if(videoFormat.fmt.pix.pixelformat == colourFormatTab[colourFormatIndex].code){
+    frameBytes = videoFormat.fmt.pix.sizeimage;
     PTRACE(3,"V4L2\tcolour format already set.");
     return true;
   }
@@ -662,7 +664,8 @@ PBoolean PVideoInputDevice_V4L2::SetColourFormat(const PString & newFormat)
 
     frameBytes = videoFormat.fmt.pix.sizeimage;
 
-    PTRACE(6,"V4L2\tset colour format \"" << newFormat << "\" set for " << deviceName << ", fd=" << videoFd);
+    PTRACE(4,"V4L2\tset colour format \"" << newFormat << "\" set for " << deviceName
+           << ", size=" << frameBytes << ", fd=" << videoFd);
 
     if (resume) {
       if (false == Start()) {
@@ -696,7 +699,7 @@ PBoolean PVideoInputDevice_V4L2::SetFrameRate(unsigned rate)
     Stop();
 
   // set the stream parameters
-  if (!DoIOCTL(VIDIOC_S_PARM, &videoStreamParm, true))  {
+  if (!DoIOCTL(VIDIOC_S_PARM, &videoStreamParm, sizeof(videoStreamParm), true))  {
     PTRACE(1,"V4L2\tS_PARM failed : "<< ::strerror(errno));
     frameRate = originalFrameRate;
     return false;
@@ -970,8 +973,10 @@ PBoolean PVideoInputDevice_V4L2::GetFrameDataNoDelay(BYTE * buffer, PINDEX * byt
   if(buf.bytesused){
     // If converting on the fly do it from frame store to output buffer,
     // otherwise do straight copy.
-    if (converter != NULL)
-      converter->Convert(videoBuffer[buf.index], buffer, buf.bytesused, bytesReturned);
+    if (converter != NULL) {
+      converter->SetSrcFrameBytes(buf.bytesused);
+      converter->Convert(videoBuffer[buf.index], buffer, bytesReturned);
+    }
     else {
       memcpy(buffer, videoBuffer[buf.index], buf.bytesused);
       if (bytesReturned != NULL)
@@ -1075,11 +1080,6 @@ PBoolean PVideoInputDevice_V4L2::VerifyHardwareFrameSize(unsigned & width, unsig
     PTRACE(1,"V4L2\tG_PARM failed (preserving frame rate may not work) : " << ::strerror(errno));
   }
 
-  if(videoFormat.fmt.pix.width == width && videoFormat.fmt.pix.height == height){
-    PTRACE(3,"V4L2\tFrame size already set.");
-    return true;
-  }
-
   videoFormat.fmt.pix.width = width;
   videoFormat.fmt.pix.height = height;
 
@@ -1091,7 +1091,7 @@ PBoolean PVideoInputDevice_V4L2::VerifyHardwareFrameSize(unsigned & width, unsig
     }
 
     PTRACE(4, "V4L2\tTry setting resolution: " << videoFormat.fmt.pix.width << "x" << videoFormat.fmt.pix.height);
-    if(!DoIOCTL(VIDIOC_S_FMT, &videoFormat, true)){
+    if(!DoIOCTL(VIDIOC_S_FMT, &videoFormat, sizeof(videoFormat), true)){
       PTRACE(1,"V4L2\tS_FMT failed: " << ::strerror(errno));
       return false;
     }
@@ -1107,8 +1107,8 @@ PBoolean PVideoInputDevice_V4L2::VerifyHardwareFrameSize(unsigned & width, unsig
       return false;
     } else {
       frameBytes = videoFormat.fmt.pix.sizeimage;
-      PTRACE(8, "V4L2\tG_FMT returned resolution: "<< videoFormat.fmt.pix.width << "x" <<
-              videoFormat.fmt.pix.height);
+      PTRACE(5, "V4L2\tG_FMT returned resolution: "<< videoFormat.fmt.pix.width << "x" <<
+              videoFormat.fmt.pix.height << ", size=" << frameBytes);
       width = videoFormat.fmt.pix.width;
       height = videoFormat.fmt.pix.height;
     }
@@ -1133,19 +1133,31 @@ PBoolean PVideoInputDevice_V4L2::VerifyHardwareFrameSize(unsigned & width, unsig
   return true;
 }
 
-PBoolean PVideoInputDevice_V4L2::DoIOCTL(unsigned long int r, void * s, PBoolean retryOnBusy)
+PBoolean PVideoInputDevice_V4L2::DoIOCTL(unsigned long int r, void * s, int structSize, PBoolean retryOnBusy)
 {
-  if (v4l2_ioctl(videoFd, r, s) >= 0)
-    return true;
+  PBoolean retval = false;
+  void *structCopy = NULL;
+
+  if(!(structCopy = malloc(structSize)))
+    goto end;
+  memcpy(structCopy, s, structSize);
+  
+  if ((retval = (v4l2_ioctl(videoFd, r, s) >= 0)))
+    goto end;
 
   if (errno != EBUSY || !retryOnBusy)
-    return false;
+    goto end;
 
-  PTRACE(3,"V4L2\tRetrying IOCTL ("<< r << ')');
+  PTRACE(3,"V4L2\tReopening device and retrying IOCTL ("<< r << ')');
   Close();
   Open(userFriendlyDevName, true);
 
-  return v4l2_ioctl(videoFd, r, s) >= 0;
+  memcpy(s, structCopy, structSize);
+  retval = (v4l2_ioctl(videoFd, r, s) >= 0);
+
+end:
+  free(structCopy);
+  return retval;
 }
 
 
@@ -1183,6 +1195,8 @@ bool PVideoInputDevice_V4L2::GetAttributes(Attributes & attrib)
   GetControlCommon(V4L2_CID_SATURATION, &attrib.m_saturation);
   GetControlCommon(V4L2_CID_CONTRAST, &attrib.m_contrast);
   GetControlCommon(V4L2_CID_HUE, &attrib.m_hue);
+  GetControlCommon(V4L2_CID_GAMMA, &attrib.m_gamma);
+  GetControlCommon(V4L2_CID_EXPOSURE, &attrib.m_exposure);
 
   return true;
 }
@@ -1225,7 +1239,9 @@ bool PVideoInputDevice_V4L2::SetAttributes(const Attributes & attrib)
   return SetControlCommon(V4L2_CID_BRIGHTNESS, attrib.m_brightness) &&
          SetControlCommon(V4L2_CID_SATURATION, attrib.m_saturation) &&
          SetControlCommon(V4L2_CID_CONTRAST, attrib.m_contrast) &&
-         SetControlCommon(V4L2_CID_HUE, attrib.m_hue);
+         SetControlCommon(V4L2_CID_HUE, attrib.m_hue) &&
+         SetControlCommon(V4L2_CID_GAMMA, attrib.m_gamma) &&
+         SetControlCommon(V4L2_CID_EXPOSURE, attrib.m_exposure);
 }
 
 PBoolean PVideoInputDevice_V4L2::QueueAllBuffers()
