@@ -23,9 +23,9 @@
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *                 Nicola Orru' <nigu@itadinanta.it>
  *
- * $Revision: 28954 $
+ * $Revision: 30753 $
  * $Author: rjongbloed $
- * $Date: 2013-01-21 18:00:44 -0600 (Mon, 21 Jan 2013) $
+ * $Date: 2013-10-18 02:47:06 +0100 (Fri, 18 Oct 2013) $
  */
 #ifndef _PVIDEOIOV4L2
 #define _PVIDEOIOV4L2
@@ -124,7 +124,7 @@ private:
   PBoolean StartStreaming();
   void StopStreaming();
 
-  PBoolean DoIOCTL(unsigned long int r, void * s, PBoolean retryOnBusy=false);
+  PBoolean DoIOCTL(unsigned long int r, void * s, int structSize, PBoolean retryOnBusy=false);
 
   PBoolean EnumFrameFormats(Capabilities & capabilities) const;
   PBoolean EnumControls(Capabilities & capabilities) const;
