@@ -7,9 +7,9 @@
  *
  * Copyright 2002 Equivalence
  *
- * $Revision: 28484 $
+ * $Revision: 29460 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2013-04-08 09:17:02 +0100 (Mon, 08 Apr 2013) $
  */
 
 #include <ptlib.h>
@@ -140,7 +140,7 @@ void Vxmltest::Main()
   cout << "Media finished" << endl;
 }
 
-void Vxmltest::InputThread(PThread &, INT)
+void Vxmltest::InputThread(PThread &, P_INT_PTR)
 {
   PConsoleChannel console(PConsoleChannel::StandardInput);
 
