@@ -25,9 +25,9 @@
  *
  * $Log: asner.cxx,v $
  * Revision 1.93  2005/11/30 12:47:41  csoutheren
- * $Revision: 28484 $
+ * $Revision: 31894 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2014-05-14 02:55:43 +0100 (Wed, 14 May 2014) $
  */
 
 #include <ptlib.h>
@@ -71,15 +71,15 @@ inline PBoolean CheckByteOffset(PINDEX offset, PINDEX upper = MaximumStringSize)
   return (0 <= offset && offset <= upper);
 }
 
-static unsigned FindNameByValue(const PASN_Names *names, unsigned namesCount, unsigned value)
+static PINDEX FindNameByValue(const PASN_Names *names, PINDEX namesCount, unsigned value)
 {
   if (names != NULL) {
-    for (unsigned i = 0;i < namesCount;i++) {
+    for (PINDEX i = 0; i < namesCount;i++) {
       if (names[i].value == value)
         return i;
     }
   }
-  return UINT_MAX;
+  return P_MAX_INDEX;
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -496,7 +496,7 @@ PObject * PASN_Enumeration::Clone() const
 void PASN_Enumeration::PrintOn(ostream & strm) const
 {
   unsigned idx = FindNameByValue(names, namesCount, value);
-  if (idx != UINT_MAX)
+  if (idx != P_MAX_INDEX)
     strm << names[idx].name;
   else
     strm << '<' << value << '>';
@@ -986,13 +986,14 @@ void PASN_BitString::PrintOn(ostream & strm) const
          << hex << setfill('0') << resetiosflags(ios::floatfield) << setiosflags(ios::fixed)
          << setw(16) << setprecision(indent) << bitData
          << dec << setfill(' ') << resetiosflags(ios::floatfield)
-         << setw(indent-1) << "}";
+         << '\n' << setw(indent-1) << "}";
   else if (totalBits > 32)
-    strm << "Hex:"
+    strm << "Hex: "
          << hex << setfill('0') << resetiosflags(ios::floatfield) << setiosflags(ios::fixed)
          << setprecision(2) << setw(16) << bitData
          << dec << setfill(' ') << resetiosflags(ios::floatfield);
   else {
+    strm << "Bits: ";
     BYTE mask = 0x80;
     PINDEX offset = 0;
     for (unsigned i = 0; i < totalBits; i++) {
