@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28709 $
+ * $Revision: 32634 $
  * $Author: rjongbloed $
- * $Date: 2012-12-20 16:17:55 -0600 (Thu, 20 Dec 2012) $
+ * $Date: 2014-08-30 06:26:37 +0100 (Sat, 30 Aug 2014) $
  */
 
 #include <ptlib.h>
@@ -56,7 +56,7 @@ PFTPClient::~PFTPClient()
 
 PBoolean PFTPClient::Close()
 {
-  if (!IsOpen())
+  if (CheckNotOpen())
     return false;
   PBoolean ok = ExecuteCommand(QUIT)/100 == 2;
   return PFTP::Close() && ok;
