@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28747 $
+ * $Revision: 29936 $
  * $Author: rjongbloed $
- * $Date: 2013-01-02 18:29:08 -0600 (Wed, 02 Jan 2013) $
+ * $Date: 2013-06-12 13:46:35 +0100 (Wed, 12 Jun 2013) $
  */
 
 #include <ptlib.h>
@@ -668,8 +668,8 @@ void PFTPServer::SendToClient(const PFilePath & filename)
         if (!file.IsOpen())
           WriteResponse(450, filename + ": cannot open file");
         else {
-          PString fileSize(PString::Unsigned, file.GetLength());
-          WriteResponse(150, PString("Opening ASCII data connection for " + filename.GetFileName() + "(" + fileSize + " bytes)"));
+          WriteResponse(150, PSTRSTRM("Opening ASCII data connection for "
+                        << filename.GetFileName() << '(' << file.GetLength() << " bytes)"));
           PString line;
           PBoolean ok = true;
           while (ok && file.ReadLine(line)) {
@@ -685,8 +685,8 @@ void PFTPServer::SendToClient(const PFilePath & filename)
         if (!file.IsOpen())
           WriteResponse(450, filename + ": cannot open file");
         else {
-          PString fileSize(PString::Unsigned, file.GetLength());
-          WriteResponse(150, PString("Opening BINARY data connection for " + filename.GetFileName() + "(" + fileSize + " bytes)"));
+          WriteResponse(150, PSTRSTRM("Opening BINARY data connection for "
+                        << filename.GetFileName() << '(' << file.GetLength() << " bytes)"));
           BYTE buffer[2048];
           PBoolean ok = true;
           while (ok && file.Read(buffer, 2048)) {
