@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 34747 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2016-04-02 10:22:45 +0100 (Sat, 02 Apr 2016) $
  */
 
 #ifdef __GNUC__
@@ -35,6 +35,8 @@
 #include <ptlib.h>
 #include <ptclib/html.h>
 
+#include <math.h>
+
 
 //////////////////////////////////////////////////////////////////////////////
 // PHTML
@@ -155,6 +157,15 @@ PString PHTML::Escape(const char * str)
 }
 
 
+const PString & PHTML::GetNonBreakSpace() { static const PConstString s("&nbsp;"); return s; }
+
+void PHTML::NonBreakSpace::Output(ostream & strm) const
+{
+  for (unsigned i = 0; i < m_count; ++i)
+    strm << GetNonBreakSpace();
+}
+
+
 void PHTML::Element::Output(PHTML & html) const
 {
   PAssert(reqElement == NumElementsInSet || html.Is(reqElement),
@@ -580,71 +591,94 @@ void PHTML::DefinitionItem::Output(PHTML & html) const
 }
 
 
-PHTML::TableStart::TableStart(const char * attr)
-  : Element("TABLE", attr, InTable, InBody, BothCRLF)
+static PConstString const AttStr[] =
 {
-  borderFlag = false;
-}
+  "nowrap",
+  "border=1",
+  "border=2",
+  "cellpadding=0",
+  "cellpadding=1",
+  "cellpadding=2",
+  "cellpadding=4",
+  "cellpadding=8",
+  "cellspacing=0",
+  "cellspacing=1",
+  "cellspacing=2",
+  "cellspacing=4",
+  "cellspacing=8",
+  "align=left",
+  "align=center",
+  "align=right",
+  "align=justify",
+  "valign=baseline",
+  "valign=bottom",
+  "valign=middle",
+  "valign=top"
+};
 
-PHTML::TableStart::TableStart(BorderCodes border, const char * attr)
-  : Element("TABLE", attr, InTable, InBody, BothCRLF)
+#define P_DEF_HTML_TABLE_CTOR(cls, name, elmt, req, opt) \
+  PHTML::cls::cls(const char * attr) \
+    : TableElement(name, attr, elmt, req, opt) { } \
+  PHTML::cls::cls(TableAttr attr1, const char * attr) \
+    : TableElement(name, AttStr[attr1]&attr, elmt, req, opt) { } \
+  PHTML::cls::cls(TableAttr attr1, TableAttr attr2, const char * attr) \
+    : TableElement(name, AttStr[attr1]&AttStr[attr2]&attr, elmt, req, opt) { } \
+  PHTML::cls::cls(TableAttr attr1, TableAttr attr2, TableAttr attr3, const char * attr) \
+    : TableElement(name, AttStr[attr1]&AttStr[attr2]&AttStr[attr3]&attr, elmt, req, opt) { } \
+  PHTML::cls::cls(TableAttr attr1, TableAttr attr2, TableAttr attr3, TableAttr attr4, const char * attr) \
+    : TableElement(name, AttStr[attr1]&AttStr[attr2]&AttStr[attr3]&AttStr[attr4]&attr, elmt, req, opt) { } \
+  PHTML::cls::cls(TableAttr attr1, TableAttr attr2, TableAttr attr3, TableAttr attr4, TableAttr attr5, const char * attr) \
+    : TableElement(name, AttStr[attr1]&AttStr[attr2]&AttStr[attr3]&AttStr[attr4]&AttStr[attr5]&attr, elmt, req, opt) { } \
+
+
+P_DEF_HTML_TABLE_CTOR(TableStart, "TABLE", InTable, InBody, BothCRLF)
+P_DEF_HTML_TABLE_CTOR(TableRow, "TR", NumElementsInSet, InTable, OpenCRLF)
+P_DEF_HTML_TABLE_CTOR(TableHeader, "TH", NumElementsInSet, InTable, CloseCRLF)
+P_DEF_HTML_TABLE_CTOR(TableData, "TD", NumElementsInSet, InTable, NoCRLF)
+
+
+void PHTML::TableElement::Output(PHTML & html) const
 {
-  borderFlag = border == Border;
+  // All these are reset by a table data/row boundary
+  for (int i = InNote; i < InList; ++i)
+    html.Clr((ElementInSet)i);
+  Element::Output(html);
 }
 
 void PHTML::TableStart::Output(PHTML & html) const
 {
   if (html.tableNestLevel > 0)
     html.Clr(InTable);
-  Element::Output(html);
+  TableElement::Output(html);
 }
 
 void PHTML::TableStart::AddAttr(PHTML & html) const
 {
-  if (borderFlag)
-    html << " BORDER";
   html.tableNestLevel++;
 }
 
 
 PHTML::TableEnd::TableEnd()
-  : Element("TABLE", "", InTable, InBody, BothCRLF)
+  : TableElement("TABLE", "", InTable, InBody, BothCRLF)
 {
 }
 
 void PHTML::TableEnd::Output(PHTML & html) const
 {
   PAssert(html.tableNestLevel > 0, "Table nesting error");
-  Element::Output(html);
+  TableElement::Output(html);
   html.tableNestLevel--;
   if (html.tableNestLevel > 0)
     html.Set(InTable);
 }
 
 
-PHTML::TableRow::TableRow(const char * attr)
-  : Element("TR", attr, NumElementsInSet, InTable, OpenCRLF)
-{
-}
-
-
-PHTML::TableHeader::TableHeader(const char * attr)
-  : Element("TH", attr, NumElementsInSet, InTable, CloseCRLF)
-{
-}
-
-
-PHTML::TableData::TableData(const char * attr)
-  : Element("TD", attr, NumElementsInSet, InTable, NoCRLF)
-{
-}
-
-
 PHTML::Form::Form(const char * method,
                   const char * action,
                   const char * mimeType,
-                  const char * script)
-  : Element("FORM", NULL, InForm, InBody, BothCRLF)
+                  const char * script,
+                  const char * attr)
+  : Element("FORM", attr, InForm, InBody, BothCRLF)
 {
   methodString = method;
   actionString = action;
@@ -657,9 +691,9 @@ void PHTML::Form::AddAttr(PHTML & html) const
   if (methodString != NULL)
     html << " METHOD=" << methodString;
   if (actionString != NULL)
-    html << " ACTION=\"" << Escaped(actionString) << '"';
+    html << " ACTION=\"" << actionString << '"';
   if (mimeTypeString != NULL)
-    html << " ENCTYPE=\"" << Escaped(mimeTypeString) << '"';
+    html << " ENCTYPE=\"" << mimeTypeString << '"';
   if (scriptString != NULL)
     html << " SCRIPT=\"" << Escaped(scriptString) << '"';
 }
@@ -1040,11 +1074,11 @@ void PHTML::InputNumber::Construct(int min, int max, int value)
 void PHTML::InputNumber::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
-  PINDEX max = PMAX(-minValue, maxValue);
+  int range = PMAX(-minValue, maxValue);
   PINDEX width = 3;
-  while (max > 10) {
+  while (range > 10) {
     width++;
-    max /= 10;
+    range /= 10;
   }
   html << " SIZE=" << width
        << " MIN=" << minValue
@@ -1053,6 +1087,36 @@ void PHTML::InputNumber::AddAttr(PHTML & html) const
 }
 
 
+PHTML::InputReal::InputReal(const char * fname,
+                            double minimum, double maximum, double value,
+                            int decs,
+                            DisableCodes disabled,
+                            const char * attr)
+  : InputField("number", fname, disabled, attr)
+{
+  PAssert(minimum <= maximum, PInvalidParameter);
+  minValue = minimum;
+  maxValue = maximum;
+  if (value < minimum)
+    initValue = minimum;
+  else if (value > maximum)
+    initValue = maximum;
+  else
+    initValue = value;
+  decimals = decs;
+}
+
+void PHTML::InputReal::AddAttr(PHTML & html) const
+{
+  InputField::AddAttr(html);
+  html << std::fixed << std::setprecision(decimals)
+       << " MIN=" << minValue
+       << " MAX=" << maxValue
+       << " VALUE=" << initValue
+       << " STEP=" << pow(10, -decimals);
+}
+
+
 PHTML::InputRange::InputRange(const char * fname,
                               int min, int max, int value,
                               DisableCodes disabled,
