@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28998 $
+ * $Revision: 33998 $
  * $Author: rjongbloed $
- * $Date: 2013-01-25 21:11:10 -0600 (Fri, 25 Jan 2013) $
+ * $Date: 2015-09-09 22:26:50 +0100 (Wed, 09 Sep 2015) $
  */
 
 #include <ptlib.h>
@@ -42,10 +42,15 @@
 
 #include <ctype.h>
 
+
+#define PTraceModule() "HTTP"
+
+
 ////////////////////////////////////////////////////////////////////////////////////
 
-static PHTTPClientAuthenticationFactory::Worker<PHTTPClientBasicAuthentication> httpClient_basicAuthenticator("basic");
-static PHTTPClientAuthenticationFactory::Worker<PHTTPClientDigestAuthentication> httpClient_md5Authenticator("digest");
+PFACTORY_CREATE(PHTTPClientAuthenticationFactory, PHTTPClientBasicAuthentication, "basic");
+PFACTORY_CREATE(PHTTPClientAuthenticationFactory, PHTTPClientDigestAuthentication, "digest");
+
 static const char * const AlgorithmNames[PHTTPClientDigestAuthentication::NumAlgorithms] = {
   "MD5"
 };
@@ -55,7 +60,161 @@ static const char * const AlgorithmNames[PHTTPClientDigestAuthentication::NumAlg
 
 static __inline bool IsOK(int response) { return (response/100) == 2; }
 
-static PINDEX MaxTraceContentSize = 1000;
+#if PTRACING
+PINDEX PHTTPClient::MaxTraceContentSize = 1000;
+#endif
+
+
+//////////////////////////////////////////////////////////////////////////////
+
+struct PHTTPClient_DummyProcessor : public PHTTPContentProcessor
+{
+  BYTE m_body[4096];
+
+  PHTTPClient_DummyProcessor(bool reader)
+    : PHTTPContentProcessor(reader)
+  {
+  }
+
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    if (m_reader) {
+      size = sizeof(m_body);
+      return m_body;
+    }
+
+    size = 0;
+    return NULL;
+  }
+};
+
+
+struct PHTTPClient_StringReader : public PHTTPContentProcessor
+{
+  PString & m_body;
+
+  PHTTPClient_StringReader(PString & body)
+    : PHTTPContentProcessor(true)
+    , m_body(body)
+  {
+  }
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    PINDEX oldLength = m_body.GetLength();
+    char * ptr = m_body.GetPointerAndSetLength(oldLength + size);
+    return ptr != NULL ? ptr + oldLength : NULL;
+  }
+};
+
+
+struct PHTTPClient_StringWriter : public PHTTPContentProcessor
+{
+  PString m_body;
+  bool    m_written;
+
+  PHTTPClient_StringWriter(const PString & body)
+    : PHTTPContentProcessor(false)
+    , m_body(body)
+    , m_written(false)
+  {
+  }
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    if (m_written)
+      return NULL;
+
+    m_written = true;
+    size = m_body.GetLength();
+    return m_body.PCharArray::GetPointer();
+  }
+
+  virtual void Reset()
+  {
+    m_written = false;
+  }
+};
+
+
+struct PHTTPClient_BinaryReader : public PHTTPContentProcessor
+{
+  PBYTEArray & m_body;
+
+  PHTTPClient_BinaryReader(PBYTEArray & body)
+    : PHTTPContentProcessor(true)
+    , m_body(body)
+  {
+  }
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    PINDEX oldSize = m_body.GetSize();
+    BYTE * ptr = m_body.GetPointer(oldSize+size);
+    return ptr != NULL ? ptr+oldSize : NULL;
+  }
+};
+
+
+struct PHTTPClient_BinaryWriter : public PHTTPContentProcessor
+{
+  PBYTEArray m_body;
+  bool       m_written;
+
+  PHTTPClient_BinaryWriter(const PBYTEArray & body)
+    : PHTTPContentProcessor(false)
+    , m_body(body)
+    , m_written(false)
+  {
+  }
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    if (m_written)
+      return NULL;
+
+    m_written = true;
+    size = m_body.GetLength();
+    return m_body.GetPointer();
+  }
+
+  virtual void Reset()
+  {
+    m_written = false;
+  }
+};
+
+
+struct PHTTPClient_FileWriter : public PHTTPContentProcessor
+{
+  PFile   m_file;
+  uint8_t m_buffer[4096];
+
+  PHTTPClient_FileWriter(const PFilePath & path)
+    : PHTTPContentProcessor(false)
+    , m_file(path, PFile::ReadOnly)
+  {
+  }
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    if (!m_file.IsOpen())
+      return NULL;
+
+    off_t left = m_file.GetLength() - m_file.GetPosition();
+    size = left > (off_t)sizeof(m_buffer) ? (PINDEX)sizeof(m_buffer) : (PINDEX)left;
+    if (m_file.Read(m_buffer, size))
+      return m_buffer;
+
+    return NULL;
+  }
+
+  virtual void Reset()
+  {
+    m_file.Open();
+  }
+};
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -69,20 +228,38 @@ PHTTPClient::PHTTPClient(const PString & userAgent)
 }
 
 
+PHTTPClient::~PHTTPClient()
+{
+  delete m_authentication;
+}
+
+
 int PHTTPClient::ExecuteCommand(Commands cmd,
                                 const PURL & url,
                                 PMIMEInfo & outMIME,
                                 const PString & dataBody,
                                 PMIMEInfo & replyMime)
 {
-  return ExecuteCommand(commandNames[cmd], url, outMIME, dataBody, replyMime);
+  PHTTPClient_StringWriter processor(dataBody);
+  return ExecuteCommand(cmd, url, outMIME, processor, replyMime);
 }
 
 
-int PHTTPClient::ExecuteCommand(const PString & cmdName,
+int PHTTPClient::ExecuteCommand(Commands cmd,
                                 const PURL & url,
                                 PMIMEInfo & outMIME,
-                                const PString & dataBody,
+                                const PBYTEArray & dataBody,
+                                PMIMEInfo & replyMIME)
+{
+  PHTTPClient_BinaryWriter processor(dataBody);
+  return ExecuteCommand(cmd, url, outMIME, processor, replyMIME);
+}
+
+
+int PHTTPClient::ExecuteCommand(Commands cmd,
+                                const PURL & url,
+                                PMIMEInfo & outMIME,
+                                ContentProcessor & processor,
                                 PMIMEInfo & replyMIME)
 {
   if (!outMIME.Contains(DateTag()))
@@ -97,12 +274,21 @@ int PHTTPClient::ExecuteCommand(const PString & cmdName,
   bool needAuthentication = true;
   PURL adjustableURL = url;
   for (int retry = 3; retry > 0; --retry) {
-    if (!AssureConnect(adjustableURL, outMIME))
+    if (!ConnectURL(adjustableURL))
       break;
 
-    if (!WriteCommand(cmdName, url.AsString(PURL::RelativeOnly), outMIME, dataBody)) {
-      lastResponseCode = -1;
+    // Have connection, so fill in the required MIME fields
+    if (!outMIME.Contains(HostTag)) {
+      if (url.GetHostName().IsEmpty())
+        outMIME.SetAt(HostTag, "localhost");
+      else
+        outMIME.SetAt(HostTag, url.GetHostPort());
+    }
+
+    if (!WriteCommand(cmd, url.AsString(PURL::RelativeOnly), outMIME, processor)) {
+      lastResponseCode = TransportWriteError;
       lastResponseInfo = GetErrorText(LastWriteError);
+      lastResponseInfo.sprintf(" (errno=%i)", GetErrorNumber(LastWriteError));
       break;
     }
 
@@ -153,12 +339,12 @@ int PHTTPClient::ExecuteCommand(const PString & cmdName,
       if (!m_persist)
         break;
 
-      // ... we close the channel and allow AssureConnect() to reopen it.
+      // ... we close the channel and allow ConnectURL() to reopen it.
       Close();
     }
   }
 
-  PTRACE_IF(3, !IsOK(lastResponseCode), "HTTP", "Error " << lastResponseCode << ' ' << lastResponseInfo);
+  PTRACE_IF(3, !IsOK(lastResponseCode), "Error " << lastResponseCode << ' ' << lastResponseInfo);
   return lastResponseCode;
 }
 
@@ -166,33 +352,28 @@ int PHTTPClient::ExecuteCommand(const PString & cmdName,
 bool PHTTPClient::WriteCommand(Commands cmd,
                         const PString & url,
                             PMIMEInfo & outMIME,
-                        const PString & dataBody)
+                     ContentProcessor & processor)
 {
-  return WriteCommand(commandNames[cmd], url, outMIME, dataBody);
-}
-
-
-bool PHTTPClient::WriteCommand(const PString & cmdName,
-                               const PString & url,
-                                   PMIMEInfo & outMIME,
-                               const PString & dataBody)
-{
-  ostream & this_stream = *this;
-  PINDEX len = dataBody.GetLength();
-  if (!outMIME.Contains(ContentLengthTag()))
-    outMIME.SetInteger(ContentLengthTag(), len);
-
-  if (m_authentication != NULL) {
-    PHTTPClientAuthenticator auth(cmdName, url, outMIME, dataBody);
-    m_authentication->Authorise(auth);
+  if (!outMIME.Contains(ContentLengthTag())) {
+    processor.Reset();
+    PINDEX total = 0;
+    PINDEX len;
+    while (processor.GetBuffer(len) != NULL)
+      total += len;
+    outMIME.SetInteger(ContentLengthTag(), total);
   }
 
-  PString cmd(cmdName.IsEmpty() ? "GET" : cmdName);
+  PString cmdName = commandNames[cmd];
+
+  if (m_authentication != NULL) {
+    PHTTPClientAuthenticator auth(cmdName, url, outMIME, processor);
+    m_authentication->Authorise(auth);
+  }
 
 #if PTRACING
   if (PTrace::CanTrace(3)) {
     ostream & strm = PTRACE_BEGIN(3);
-    strm << "HTTP\tSending ";
+    strm << "Sending ";
     if (PTrace::CanTrace(4))
       strm << '\n';
     strm << cmdName << ' ';
@@ -200,23 +381,22 @@ bool PHTTPClient::WriteCommand(const PString & cmdName,
       strm << '/';
     else
       strm << url;
-    if (PTrace::CanTrace(4)) {
-      strm << '\n' << outMIME;
-      if (!dataBody.IsEmpty()) {
-        PINDEX amt = PTrace::CanTrace(5) ? 10000 : 100;
-        strm << dataBody.Left(amt);
-        if (len > amt)
-          strm << "\n....";
-      }
-    }
     strm << PTrace::End;
-  } 
+  }
 #endif
 
-  this_stream << cmd << ' ' << (url.IsEmpty() ? "/" :  (const char*) url) << " HTTP/1.1\r\n"
-              << setfill('\r') << outMIME;
+  *this << cmdName << ' ' << (url.IsEmpty() ? "/" : (const char*)url) << " HTTP/1.1\r\n"
+        << setfill('\r') << outMIME;
 
-  return Write((const char *)dataBody, len);
+  processor.Reset();
+  const void * data;
+  PINDEX len = 0;
+  while ((data = processor.GetBuffer(len)) != NULL) {
+    if (!Write(data, len))
+      return false;
+  }
+
+  return true;
 }
 
 
@@ -229,7 +409,7 @@ bool PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
     if (http.Find("HTTP/") == P_MAX_INDEX) {
       lastResponseCode = PHTTP::RequestOK;
       lastResponseInfo = "HTTP/0.9";
-      PTRACE(3, "HTTP\tRead HTTP/0.9 OK");
+      PTRACE(3, "Read HTTP/0.9 OK");
       return true;
     }
 
@@ -243,16 +423,18 @@ bool PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 
       PString body;
       if (lastResponseCode >= 300) {
+#if PTRACING
         if ((int)replyMIME.GetInteger(ContentLengthTag(), INT_MAX) <= MaxTraceContentSize)
           ReadContentBody(replyMIME, body);
         else
+#endif
           ReadContentBody(replyMIME); // Waste body
       }
 
 #if PTRACING
       if (PTrace::CanTrace(3)) {
         ostream & strm = PTRACE_BEGIN(3);
-        strm << "HTTP\tResponse ";
+        strm << "Response ";
         if (PTrace::CanTrace(4))
           strm << '\n';
         strm << lastResponseCode << ' ' << lastResponseInfo;
@@ -273,9 +455,11 @@ bool PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
     }
   }
  
-  lastResponseCode = -1;
-  if (GetErrorCode(LastReadError) != NoError)
+  lastResponseCode = TransportReadError;
+  if (GetErrorCode(LastReadError) != NoError) {
     lastResponseInfo = GetErrorText(LastReadError);
+    lastResponseInfo.sprintf(" (errno=%i)", GetErrorNumber(LastReadError));
+  }
   else {
     lastResponseInfo = "Premature shutdown";
     SetErrorValues(ProtocolFailure, 0, LastReadError);
@@ -285,85 +469,23 @@ bool PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 }
 
 
-struct PHTTPClient_DummyProcessor : public PHTTPClient::ContentProcessor
-{
-  BYTE m_body[4096];
-
-  virtual void * GetBuffer(PINDEX & size)
-  {
-    size = sizeof(m_body);
-    return m_body;
-  }
-
-  virtual bool Process(const void *, PINDEX)
-  {
-    return true;
-  }
-};
-
-
 bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME)
 {
-  PHTTPClient_DummyProcessor processor;
+  PHTTPClient_DummyProcessor processor(true);
   return ReadContentBody(replyMIME, processor);
 }
 
 
-struct PHTTPClient_StringProcessor : public PHTTPClient::ContentProcessor
-{
-  PString & m_body;
-
-  PHTTPClient_StringProcessor(PString & body)
-    : m_body(body)
-  {
-  }
-
-  virtual void * GetBuffer(PINDEX & size)
-  {
-    PINDEX oldLength = m_body.GetLength();
-    char * ptr = m_body.GetPointerAndSetLength(oldLength+size);
-    return ptr != NULL ? ptr+oldLength : NULL;
-  }
-
-  virtual bool Process(const void *, PINDEX)
-  {
-    return true;
-  }
-};
-
-
 bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
 {
-  PHTTPClient_StringProcessor processor(body);
+  PHTTPClient_StringReader processor(body);
   return ReadContentBody(replyMIME, processor);
 }
 
 
-struct PHTTPClient_BinaryProcessor : public PHTTPClient::ContentProcessor
-{
-  PBYTEArray & m_body;
-
-  PHTTPClient_BinaryProcessor(PBYTEArray & body)
-    : m_body(body)
-  {
-  }
-
-  virtual void * GetBuffer(PINDEX & size)
-  {
-    PINDEX oldSize = m_body.GetSize();
-    BYTE * ptr = m_body.GetPointer(oldSize+size);
-    return ptr != NULL ? ptr+oldSize : NULL;
-  }
-
-  virtual bool Process(const void *, PINDEX)
-  {
-    return true;
-  }
-};
-
 bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
 {
-  PHTTPClient_BinaryProcessor processor(body);
+  PHTTPClient_BinaryReader processor(body);
   return ReadContentBody(replyMIME, processor);
 }
 
@@ -399,7 +521,7 @@ bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, ContentProcessor & proc
     }
 
     if (!(encoding.IsEmpty())) {
-      lastResponseCode = -1;
+      lastResponseCode = UnknownTransferEncoding;
       lastResponseInfo = "Unknown Transfer-Encoding extension";
       return false;
     }
@@ -483,7 +605,7 @@ static bool CheckContentType(const PMIMEInfo & replyMIME, const PString & requir
       actualContentType.NumCompare(requiredContentType, requiredContentType.Find(';')) == PObject::EqualTo)
     return true;
 
-  PTRACE(2, "HTTP\tIncorrect Content-Type for document: expecting "
+  PTRACE(2, "Incorrect Content-Type for document: expecting "
          << requiredContentType << ", got " << actualContentType);
   return false;
 }
@@ -503,11 +625,11 @@ bool PHTTPClient::GetTextDocument(const PURL & url,
   }
 
   if (!ReadContentBody(replyMIME, document)) {
-    PTRACE(2, "HTTP\tRead of body failed");
+    PTRACE(2, "Read of body failed");
     return false;
   }
 
-  PTRACE_IF(4, !document.IsEmpty(), "HTTP\tReceived body:\n"
+  PTRACE_IF(4, !document.IsEmpty(), "Received body:\n"
             << document.Left(MaxTraceContentSize) << (document.GetLength() > MaxTraceContentSize ? "\n...." : ""));
   return true;
 }
@@ -527,11 +649,11 @@ bool PHTTPClient::GetBinaryDocument(const PURL & url,
   }
 
   if (!ReadContentBody(replyMIME, document)) {
-    PTRACE(2, "HTTP\tRead of body failed");
+    PTRACE(2, "Read of body failed");
     return false;
   }
 
-  PTRACE_IF(4, !document.IsEmpty(), "HTTP\tReceived " << document.GetSize() << " byte body\n");
+  PTRACE_IF(4, !document.IsEmpty(), "Received " << document.GetSize() << " byte body\n");
   return true;
 }
 
@@ -610,21 +732,28 @@ bool PHTTPClient::PostData(const PURL & url,
 }
 
 
-bool PHTTPClient::PutTextDocument(const PURL & url,
-                                  const PString & document,
-                                  const PString & contentType)
+bool PHTTPClient::PutDocument(const PURL & url, const PString & data, const PString & contentType, const PMIMEInfo & mime)
 {
-  PMIMEInfo outMIME, replyMIME;
+  PMIMEInfo outMIME(mime), replyMIME;
   outMIME.SetAt(ContentTypeTag(), contentType);
-  return IsOK(ExecuteCommand(PUT, url, outMIME, document, replyMIME));
+  return IsOK(ExecuteCommand(PUT, url, outMIME, data, replyMIME));
 }
 
 
-bool PHTTPClient::PutDocument(const PURL & url,
-                              PMIMEInfo & outMIME,
-                              PMIMEInfo & replyMIME)
+bool PHTTPClient::PutDocument(const PURL & url, const PBYTEArray & data, const PString & contentType, const PMIMEInfo & mime)
 {
-  return IsOK(ExecuteCommand(PUT, url, outMIME, PString::Empty(), replyMIME));
+  PMIMEInfo outMIME(mime), replyMIME;
+  outMIME.SetAt(ContentTypeTag(), contentType);
+  return IsOK(ExecuteCommand(PUT, url, outMIME, data, replyMIME));
+}
+
+
+bool PHTTPClient::PutDocument(const PURL & url, const PFilePath & path, const PString & contentType, const PMIMEInfo & mime)
+{
+  PMIMEInfo outMIME(mime), replyMIME;
+  outMIME.SetAt(ContentTypeTag(), contentType.IsEmpty() ? PMIMEInfo::GetContentType(path.GetType()) : contentType);
+  PHTTPClient_FileWriter processor(path);
+  return IsOK(ExecuteCommand(PUT, url, outMIME, processor, replyMIME));
 }
 
 
@@ -635,72 +764,76 @@ bool PHTTPClient::DeleteDocument(const PURL & url)
 }
 
 
-bool PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
+bool PHTTPClient::ConnectURL(const PURL & url)
 {
+  if (IsOpen())
+    return true;
+
   PString host = url.GetHostName();
 
   // Is not open or other end shut down, restablish connection
-  if (!IsOpen()) {
-    if (host.IsEmpty()) {
-      lastResponseCode = BadRequest;
-      lastResponseInfo = "No host specified";
-      return SetErrorValues(ProtocolFailure, 0, LastReadError);
-    }
+  if (host.IsEmpty()) {
+    lastResponseCode = BadRequest;
+    lastResponseInfo = "No host specified";
+    return SetErrorValues(ProtocolFailure, 0, LastReadError);
+  }
 
 #if P_SSL
-    if (url.GetScheme() == "https") {
+  if (url.GetScheme() == "https") {
+    PSSLChannel * ssl = NULL;
+    PSSLContext::Method method = PSSLContext::HighestTLS;
+    for (;;) {
       PTCPSocket * tcp = new PTCPSocket(url.GetPort());
       tcp->SetReadTimeout(readTimeout);
       if (!tcp->Connect(host)) {
-        lastResponseCode = -2;
+        lastResponseCode = TransportConnectError;
         lastResponseInfo = tcp->GetErrorText();
+        lastResponseInfo.sprintf(" (errno=%i)", tcp->GetErrorNumber());
         delete tcp;
         return false;
       }
 
-      PSSLContext * context = new PSSLContext;
+      PSSLContext * context = new PSSLContext(method);
       if (!context->SetCredentials(m_authority, m_certificate, m_privateKey)) {
-        lastResponseCode = -2;
+        lastResponseCode = TransportConnectError;
         lastResponseInfo = "Could not set certificates";
         delete context;
+        delete tcp;
         return false;
       }
 
-      PSSLChannel * ssl = new PSSLChannel(context);
-      if (!ssl->Connect(tcp)) {
-        lastResponseCode = -2;
+      ssl = new PSSLChannel(context, true);
+      if (ssl->Connect(tcp))
+        break;
+
+      if ((unsigned)ssl->GetErrorNumber() != 0x9408f10b || method <= PSSLContext::BeginMethod) {
+        lastResponseCode = TransportConnectError;
         lastResponseInfo = ssl->GetErrorText();
         delete ssl;
         return false;
       }
 
-      if (!Open(ssl)) {
-        lastResponseCode = -2;
-        lastResponseInfo = GetErrorText();
-        return false;
-      }
+      delete ssl;
+      --method;
     }
-    else
-#endif
 
-    if (!Connect(host, url.GetPort())) {
-      lastResponseCode = -2;
+    if (!Open(ssl)) {
+      lastResponseCode = TransportConnectError;
       lastResponseInfo = GetErrorText();
       return false;
     }
   }
+  else
+#endif
 
-  // Have connection, so fill in the required MIME fields
-  if (!outMIME.Contains(HostTag)) {
-    if (!host)
-      outMIME.SetAt(HostTag, host);
-    else {
-      PIPSocket * sock = GetSocket();
-      if (sock != NULL)
-        outMIME.SetAt(HostTag, sock->GetHostName());
-    }
+  if (!Connect(host, url.GetPort())) {
+    lastResponseCode = TransportConnectError;
+    lastResponseInfo = GetErrorText();
+    lastResponseInfo.sprintf(" (errno=%i)", GetErrorNumber());
+    return false;
   }
 
+  PTRACE(5, "Connected to " << host);
   return true;
 }
 
@@ -832,6 +965,7 @@ PHTTPClientDigestAuthentication::PHTTPClientDigestAuthentication()
   , qopAuth(false)
   , qopAuthInt(false)
   , stale(false)
+  , nonceCount(0)
 {
 }
 
@@ -848,7 +982,7 @@ PHTTPClientDigestAuthentication & PHTTPClientDigestAuthentication::operator =(co
   qopAuthInt = auth.qopAuthInt;
   stale      = auth.stale;
   cnonce     = auth.cnonce;
-  nonceCount.SetValue(auth.nonceCount);
+  nonceCount = (uint32_t)auth.nonceCount;
 
   return *this;
 }
@@ -885,10 +1019,10 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
 
   qopAuth = qopAuthInt = false;
   cnonce.MakeEmpty();
-  nonceCount.SetValue(1);
+  nonceCount = 1;
 
   if (auth.Find("digest") == P_MAX_INDEX) {
-    PTRACE(1, "HTTP\tDigest auth does not contian digest keyword");
+    PTRACE(1, "Digest auth does not contian digest keyword");
     return false;
   }
 
@@ -898,7 +1032,7 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
     while (str != AlgorithmNames[algorithm]) {
       algorithm = (Algorithm)(algorithm+1);
       if (algorithm >= PHTTPClientDigestAuthentication::NumAlgorithms) {
-        PTRACE(1, "HTTP\tUnknown digest algorithm " << str);
+        PTRACE(1, "Unknown digest algorithm " << str);
         return false;
       }
     }
@@ -906,24 +1040,24 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
 
   authRealm = GetAuthParam(auth, "realm");
   if (authRealm.IsEmpty()) {
-    PTRACE(1, "HTTP\tNo realm in authentication");
+    PTRACE(1, "No realm in authentication");
     return false;
   }
 
   nonce = GetAuthParam(auth, "nonce");
   if (nonce.IsEmpty()) {
-    PTRACE(1, "HTTP\tNo nonce in authentication");
+    PTRACE(1, "No nonce in authentication");
     return false;
   }
 
   opaque = GetAuthParam(auth, "opaque");
   if (!opaque.IsEmpty()) {
-    PTRACE(2, "HTTP\tAuthentication contains opaque data");
+    PTRACE(2, "Authentication contains opaque data");
   }
 
   PString qopStr = GetAuthParam(auth, "qop");
   if (!qopStr.IsEmpty()) {
-    PTRACE(3, "HTTP\tAuthentication contains qop-options " << qopStr);
+    PTRACE(3, "Authentication contains qop-options " << qopStr);
     PStringList options = qopStr.Tokenise(',', true);
     qopAuth    = options.GetStringsIndex("auth") != P_MAX_INDEX;
     qopAuthInt = options.GetStringsIndex("auth-int") != P_MAX_INDEX;
@@ -931,7 +1065,7 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
   }
 
   PCaselessString staleStr = GetAuthParam(auth, "stale");
-  PTRACE_IF(3, !staleStr.IsEmpty(), "HTTP\tAuthentication contains stale flag \"" << staleStr << '"');
+  PTRACE_IF(3, !staleStr.IsEmpty(), "Authentication contains stale flag \"" << staleStr << '"');
   stale = staleStr.Find("true") != P_MAX_INDEX;
 
   isProxy = proxy;
@@ -941,7 +1075,7 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
 
 PBoolean PHTTPClientDigestAuthentication::Authorise(AuthObject & authObject) const
 {
-  PTRACE(3, "HTTP\tAdding authentication information");
+  PTRACE(3, "Adding authentication information");
 
   PMessageDigest5 digestor;
   PMessageDigest5::Code a1, a2, entityBodyCode, response;
@@ -961,7 +1095,16 @@ PBoolean PHTTPClientDigestAuthentication::Authorise(AuthObject & authObject) con
 
   if (qopAuthInt) {
     digestor.Start();
-    digestor.Process(authObject.GetEntityBody());
+    PHTTPContentProcessor * processor = authObject.GetContentProcessor();
+    if (processor == NULL)
+      digestor.Process(authObject.GetEntityBody());
+    else {
+      processor->Reset();
+      void * data;
+      PINDEX len;
+      while ((data = processor->GetBuffer(len)) != NULL)
+        digestor.Process(data, len);
+    }
     digestor.Complete(entityBodyCode);
   }
 
@@ -1063,11 +1206,11 @@ PHTTPClientAuthentication * PHTTPClientAuthentication::ParseAuthenticationRequir
 
 ////////////////////////////////////////////////////////////////////////////////////
 
-PHTTPClientAuthenticator::PHTTPClientAuthenticator(const PString & method, const PString & uri, PMIMEInfo & mime, const PString & body)
+PHTTPClientAuthenticator::PHTTPClientAuthenticator(const PString & method, const PString & uri, PMIMEInfo & mime, PHTTPContentProcessor & processor)
   : m_method(method)
   , m_uri(uri)
   , m_mime(mime)
-  , m_body(body)
+  , m_contentProcessor(processor)
 {
 }
 
@@ -1081,9 +1224,9 @@ PString PHTTPClientAuthenticator::GetURI()
   return m_uri;
 }
 
-PString PHTTPClientAuthenticator::GetEntityBody()
+PHTTPContentProcessor * PHTTPClientAuthenticator::GetContentProcessor()
 {
-  return m_body;
+  return &m_contentProcessor;
 }
 
 PString PHTTPClientAuthenticator::GetMethod()
@@ -1127,7 +1270,7 @@ class PURL_HttpLoader : public PURLLoader
       PCaselessString actualContentType = replyMIME(PHTTP::ContentTypeTag());
       if (!params.m_requiredContentType.IsEmpty() && !actualContentType.IsEmpty() &&
             actualContentType.NumCompare(params.m_requiredContentType, params.m_requiredContentType.Find(';')) != EqualTo) {
-        PTRACE(2, "HTTP\tIncorrect Content-Type for document: expecting " << params.m_requiredContentType << ", got " << actualContentType);
+        PTRACE(2, "Incorrect Content-Type for document: expecting " << params.m_requiredContentType << ", got " << actualContentType);
         return false;
       }
 
@@ -1137,7 +1280,7 @@ class PURL_HttpLoader : public PURLLoader
 
 PFACTORY_CREATE(PURLLoaderFactory, PURL_HttpLoader, "http", true);
 #if P_SSL
-static PURLLoaderFactory::Worker<PURL_HttpLoader> httpsLoader("https", true);
+PFACTORY_SYNONYM(PURLLoaderFactory, PURL_HttpLoader, https, "https");
 #endif
 
 #endif // P_HTTP
