@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28989 $
+ * $Revision: 34360 $
  * $Author: rjongbloed $
- * $Date: 2013-01-25 00:26:28 -0600 (Fri, 25 Jan 2013) $
+ * $Date: 2015-11-27 09:12:38 +0000 (Fri, 27 Nov 2015) $
  */
 
 #ifdef __GNUC__
@@ -362,16 +362,23 @@ static int SplitConfigKey(const PString & fullName,
 }
 
 
-void PHTTPField::LoadFromConfig(PConfig & cfg)
+bool PHTTPField::LoadFromConfig(PConfig & cfg)
 {
   PString section, key;
   switch (SplitConfigKey(fullName, section, key)) {
     case 1 :
-      SetValue(cfg.GetString(key, GetValue(true)));
+      if (!cfg.HasKey(key))
+        return true;
+      SetValue(cfg.GetString(key));
       break;
+
     case 2 :
-      SetValue(cfg.GetString(section, key, GetValue(true)));
+      if (!cfg.HasKey(section, key))
+        return true;
+      SetValue(cfg.GetString(section, key, PString::Empty()));
   }
+
+  return false;
 }
 
 
@@ -549,11 +556,18 @@ void PHTTPCompositeField::SetValue(const PString &)
 }
 
 
-void PHTTPCompositeField::LoadFromConfig(PConfig & cfg)
+bool PHTTPCompositeField::LoadFromConfig(PConfig & cfg)
 {
+  bool useDefault = true;
+
   SetName(fullName);
-  for (PINDEX i = 0; i < GetSize(); i++)
-    fields[i].LoadFromConfig(cfg);
+
+  for (PINDEX i = 0; i < GetSize(); i++) {
+    if (!fields[i].LoadFromConfig(cfg))
+      useDefault = false;
+  }
+
+  return useDefault;
 }
 
 
@@ -852,19 +866,28 @@ static int SplitArraySizeKey(const PString & fullName,
 }
 
 
-void PHTTPFieldArray::LoadFromConfig(PConfig & cfg)
+bool PHTTPFieldArray::LoadFromConfig(PConfig & cfg)
 {
   if (canAddElements) {
     PString section, key;
     switch (SplitArraySizeKey(fullName, section, key)) {
       case 1 :
-        SetSize(cfg.GetInteger(key, GetSize()));
+        if (!cfg.HasKey(key))
+          return true;
+
+        SetSize(cfg.GetInteger(key));
         break;
+
       case 2 :
-        SetSize(cfg.GetInteger(section, key, GetSize()));
+        if (!cfg.HasKey(section, key))
+          return true;
+
+        SetSize(cfg.GetInteger(section, key));
     }
   }
+
   PHTTPCompositeField::LoadFromConfig(cfg);
+  return false;
 }
 
 
@@ -984,9 +1007,10 @@ void PHTTPFieldArray::AddBlankField()
 }
 
 
-PStringArray PHTTPFieldArray::GetStrings(PConfig & cfg)
+PStringArray PHTTPFieldArray::GetStrings(PConfig & cfg, const PStringArray & defaultValues)
 {
-  LoadFromConfig(cfg);
+  if (LoadFromConfig(cfg))
+    SetStrings(cfg, defaultValues);
 
   PStringArray values(GetSize());
 
@@ -1045,7 +1069,7 @@ PHTTPStringField::PHTTPStringField(const char * name,
 
 PHTTPField * PHTTPStringField::NewField() const
 {
-  return new PHTTPStringField(baseName, title, maxLength, initialValue, help);
+  return new PHTTPStringField(baseName, title, maxLength, initialValue, help, rows, columns);
 }
 
 
@@ -1255,16 +1279,21 @@ PString PHTTPIntegerField::GetValue(PBoolean dflt) const
 }
 
 
-void PHTTPIntegerField::LoadFromConfig(PConfig & cfg)
+bool PHTTPIntegerField::LoadFromConfig(PConfig & cfg)
 {
   PString section, key;
   switch (SplitConfigKey(fullName, section, key)) {
     case 1 :
+      if (!cfg.HasKey(key))
+        return true;
       value = cfg.GetInteger(key, initialValue);
       break;
     case 2 :
+      if (!cfg.HasKey(section, key))
+        return true;
       value = cfg.GetInteger(section, key, initialValue);
   }
+  return false;
 }
 
 
@@ -1393,16 +1422,21 @@ PString PHTTPBooleanField::GetValue(PBoolean dflt) const
 }
 
 
-void PHTTPBooleanField::LoadFromConfig(PConfig & cfg)
+bool PHTTPBooleanField::LoadFromConfig(PConfig & cfg)
 {
   PString section, key;
   switch (SplitConfigKey(fullName, section, key)) {
     case 1 :
+      if (!cfg.HasKey(key))
+        return true;
       value = cfg.GetBoolean(key, initialValue);
       break;
     case 2 :
+      if (!cfg.HasKey(section, key))
+        return true;
       value = cfg.GetBoolean(section, key, initialValue);
   }
+  return false;
 }
 
 
@@ -1563,10 +1597,16 @@ PString PHTTPRadioField::GetHTMLInput(const PString & input) const
   else
     inval = baseName;
 
-  if (inval != value)
+  bool checked = input.Find("CHECKED>") != P_MAX_INDEX;
+  bool sameValue = inval == value;
+  if (checked == sameValue)
     return input;
 
-  return "<input checked" + input.Mid(6);
+  PINDEX len = input.GetLength();
+  if (sameValue)
+    return input.Left(len-1) + "CHECKED>";
+
+  return input.Left(len-8) + '>';
 }
 
 
@@ -1581,7 +1621,10 @@ PString PHTTPRadioField::GetValue(PBoolean dflt) const
 
 void PHTTPRadioField::SetValue(const PString & newVal)
 {
-  value = newVal;
+  PINDEX idx;
+  if ((idx = values.GetValuesIndex(newVal)) != P_MAX_INDEX ||
+      (idx = titles.GetValuesIndex(newVal)) != P_MAX_INDEX)
+    value = values[idx];
 }
 
 
@@ -1595,10 +1638,8 @@ PHTTPSelectField::PHTTPSelectField(const char * name,
                                    bool enumeration)
   : PHTTPField(name, NULL, help)
   , m_values(valueArray)
-  , m_enumeration(enumeration)
-  , m_initialValue(initVal)
-  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
+  Construct(initVal, enumeration);
 }
 
 
@@ -1610,10 +1651,8 @@ PHTTPSelectField::PHTTPSelectField(const char * name,
                                    bool enumeration)
   : PHTTPField(name, NULL, help)
   , m_values(count, valueStrings)
-  , m_enumeration(enumeration)
-  , m_initialValue(initVal)
-  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
+  Construct(initVal, enumeration);
 }
 
 
@@ -1625,10 +1664,8 @@ PHTTPSelectField::PHTTPSelectField(const char * name,
                                    bool enumeration)
   : PHTTPField(name, title, help)
   , m_values(valueArray)
-  , m_enumeration(enumeration)
-  , m_initialValue(initVal)
-  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
+  Construct(initVal, enumeration);
 }
 
 
@@ -1641,39 +1678,56 @@ PHTTPSelectField::PHTTPSelectField(const char * name,
                                    bool enumeration)
   : PHTTPField(name, title, help)
   , m_values(count, valueStrings)
-  , m_enumeration(enumeration)
-  , m_initialValue(initVal)
-  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
+  Construct(initVal, enumeration);
+}
+
+
+void PHTTPSelectField::Construct(PINDEX initVal, bool enumeration)
+{
+  m_initialValue = initVal;
+  m_enumeration = enumeration;
+
+  if (enumeration)
+    m_value.sprintf("%u", initVal);
+  else if (initVal < m_values.GetSize())
+    m_value = m_values[initVal];
 }
 
 
 PHTTPField * PHTTPSelectField::NewField() const
 {
-  return new PHTTPSelectField(baseName, title, m_values, m_initialValue, help);
+  return new PHTTPSelectField(baseName, title, m_values, m_initialValue, help, m_enumeration);
 }
 
 
 void PHTTPSelectField::GetHTMLTag(PHTML & html) const
 {
   html << PHTML::Select(fullName);
-  for (PINDEX i = 0; i < m_values.GetSize(); i++)
-    html << PHTML::Option(m_values[i] == m_value ? PHTML::Selected : PHTML::NotSelected,
-                          m_enumeration ? psprintf("value=\"%u\"", i) : PString::Empty())
-         << PHTML::Escaped(m_values[i]);
+  PINDEX selection = m_value.AsUnsigned();
+  for (PINDEX i = 0; i < m_values.GetSize(); i++) {
+    if (m_enumeration)
+      html << PHTML::Option(i == selection ? PHTML::Selected : PHTML::NotSelected, psprintf("VALUE=\"%u\"", i));
+    else
+      html << PHTML::Option(m_values[i] == m_value ? PHTML::Selected : PHTML::NotSelected);
+    html << PHTML::Escaped(m_values[i]);
+  }
   html << PHTML::Select();
 }
 
 
 PString PHTTPSelectField::GetValue(PBoolean dflt) const
 {
-  if (dflt)
-    if (m_initialValue < m_values.GetSize())
-      return m_values[m_initialValue];
-    else
-      return PString::Empty();
-  else
+  if (!dflt)
     return m_value;
+
+  if (m_enumeration)
+    return m_initialValue;
+
+  if (m_initialValue < m_values.GetSize())
+    return m_values[m_initialValue];
+
+  return PString::Empty();
 }
 
 
@@ -1980,7 +2034,7 @@ PBoolean PHTTPForm::Post(PHTTPRequest & request,
       msg << PHTML::Title() << "Accepted New Configuration" << PHTML::Body()
           << PHTML::Heading(1) << "Accepted New Configuration" << PHTML::Heading(1)
           << PHTML::HotLink(request.url.AsString()) << "Reload page" << PHTML::HotLink()
-          << "&nbsp;&nbsp;&nbsp;&nbsp;"
+          << PHTML::NonBreakSpace(4)
           << PHTML::HotLink("/") << "Home page" << PHTML::HotLink();
     }
     else {
@@ -2001,7 +2055,7 @@ PBoolean PHTTPForm::Post(PHTTPRequest & request,
           << errors
           << PHTML::Paragraph()
           << PHTML::HotLink(request.url.AsString()) << "Reload page" << PHTML::HotLink()
-          << "&nbsp;&nbsp;&nbsp;&nbsp;"
+          << PHTML::NonBreakSpace(4)
           << PHTML::HotLink("/") << "Home page" << PHTML::HotLink();
     }
     else {
@@ -2081,10 +2135,10 @@ void PHTTPConfig::Construct()
 }
 
 
-void PHTTPConfig::LoadFromConfig()
+bool PHTTPConfig::LoadFromConfig()
 {
   PConfig cfg(section);
-  fields.LoadFromConfig(cfg);
+  return fields.LoadFromConfig(cfg);
 }
 
 
@@ -2207,6 +2261,78 @@ void PHTTPConfig::AddNewKeyFields(PHTTPField * keyFld,
 }
 
 
+bool PHTTPConfig::AddBooleanField(const char * name, bool defaultValue, const char * help)
+{
+  PConfig cfg(section);
+  bool currentValue = cfg.GetBoolean(name, defaultValue);
+  Add(new PHTTPBooleanField(name, currentValue, help));
+  return currentValue;
+}
+
+
+int PHTTPConfig::AddIntegerField(const char * name, int low, int high, int defaultValue, const char * units, const char * help)
+{
+  PConfig cfg(section);
+  int currentValue = cfg.GetInteger(name, defaultValue);
+  Add(new PHTTPIntegerField(name, low, high, currentValue, units, help));
+  return currentValue;
+}
+
+
+PString PHTTPConfig::AddStringField(const char * name, PINDEX maxLength, const char * defaultValue, const char * help, int rows, int columns)
+{
+  PConfig cfg(section);
+  PString currentValue = cfg.GetString(name, defaultValue);
+  Add(new PHTTPStringField(name, maxLength, currentValue, help, rows, columns));
+  return currentValue;
+}
+
+
+PStringArray PHTTPConfig::AddStringArrayField(const char * name,
+                                              bool sorted,
+                                              PINDEX maxLength,
+                                              const PStringArray & defaultValues,
+                                              const char * help,
+                                              int rows,
+                                              int columns)
+{
+  PConfig cfg(section);
+  PHTTPFieldArray* fieldArray = new PHTTPFieldArray(new PHTTPStringField(name, maxLength, "", help, rows, columns), sorted);
+  Add(fieldArray);
+  return fieldArray->GetStrings(cfg, defaultValues);
+}
+
+
+PString PHTTPConfig::AddSelectField(const char * name,
+                                    const PStringArray & valueArray,
+                                    const char * defaultValue,
+                                    const char * help,
+                                    bool enumeration)
+{
+  PConfig cfg(section);
+  PString currentValue = cfg.GetString(name, defaultValue);
+  PINDEX selection = valueArray.GetValuesIndex(currentValue);
+  if (selection == P_MAX_INDEX)
+    selection = 0;
+  Add(new PHTTPSelectField(name, valueArray, selection, help, enumeration));
+  return currentValue;
+}
+
+
+PStringArray PHTTPConfig::AddSelectArrayField(const char * name,
+                                              bool sorted,
+                                              const PStringArray & defaultValues,
+                                              const PStringArray & possibleValues,
+                                              const char * help,
+                                              bool enumeration)
+{
+  PConfig cfg(section);
+  PHTTPFieldArray* fieldArray = new PHTTPFieldArray(new PHTTPSelectField(name, possibleValues, 0, help, enumeration), sorted);
+  Add(fieldArray);
+  return fieldArray->GetStrings(cfg, defaultValues);
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PHTTPConfigSectionList
 
