@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26962 $
+ * $Revision: 34174 $
  * $Author: rjongbloed $
- * $Date: 2012-02-08 20:55:46 -0600 (Wed, 08 Feb 2012) $
+ * $Date: 2015-10-10 11:25:01 +0100 (Sat, 10 Oct 2015) $
  */
 
 #ifdef __GNUC__
@@ -330,9 +330,10 @@ PBoolean PInternetProtocol::ReadLine(PString & line, PBoolean allowContinuation)
 
   SetReadTimeout(oldTimeout);
 
-  if (count < line.GetSize()-1)
-    line[count] = '\0';
-  line.MakeMinimumSize();
+  if (count > 0)
+    line.MakeMinimumSize(count);
+  else
+    line.MakeEmpty();
 
   return gotEndOfLine;
 }
@@ -590,7 +591,8 @@ void PMIMEInfo::PrintOn(ostream &strm) const
 
 ostream & PMIMEInfo::PrintContents(ostream &strm) const
 {
-  PBoolean output_cr = strm.fill() == '\r';
+  bool output_cr = strm.fill() == '\r';
+  std::streamsize indent = strm.width();
   strm.fill(' ');
   for (const_iterator it = begin(); it != end(); ++it) {
     PString name = it->first + ": ";
@@ -598,6 +600,8 @@ ostream & PMIMEInfo::PrintContents(ostream &strm) const
     if (value.FindOneOf("\r\n") != P_MAX_INDEX) {
       PStringArray vals = value.Lines();
       for (PINDEX j = 0; j < vals.GetSize(); j++) {
+        if (indent > 0)
+          strm << setw(indent) << " ";
         strm << name << vals[j];
         if (output_cr)
           strm << '\r';
@@ -605,6 +609,8 @@ ostream & PMIMEInfo::PrintContents(ostream &strm) const
       }
     }
     else {
+      if (indent > 0)
+        strm << setw(indent) << " ";
       strm << name << value;
       if (output_cr)
         strm << '\r';
@@ -621,10 +627,13 @@ void PMIMEInfo::ReadFrom(istream &strm)
 
   PString line;
   PString lastLine;
+  bool incomplete = true;
   while (strm.good()) {
     strm >> line;
-    if (line.IsEmpty())
+    if (line.IsEmpty()) {
+      incomplete = false;
       break;
+    }
     if (line[0] == ' ' || line[0] == '\t') // RFC 2822 section 2.2.2 & 2.2.3
       lastLine += line;
     else {
@@ -635,6 +644,9 @@ void PMIMEInfo::ReadFrom(istream &strm)
 
   if (!lastLine.IsEmpty())
     AddMIME(lastLine);
+
+  if (incomplete)
+    strm.setstate(ios::failbit);
 }
 
 
@@ -866,6 +878,7 @@ const PCaselessString & PMIMEInfo::ContentDescriptionTag()      { static const P
 const PCaselessString & PMIMEInfo::ContentIdTag()               { static const PConstCaselessString s("Content-ID");                return s; }
 
 const PCaselessString & PMIMEInfo::TextPlain()                  { static const PConstCaselessString s("text/plain");                return s; }
+const PCaselessString & PMIMEInfo::TextHTML()                   { static const PConstCaselessString s("text/html");                 return s; }
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -965,13 +978,17 @@ bool PMultiPartList::Decode(const PString & entityBody, const PStringToString &
     PCaselessString encoding = info->m_mime.GetString(PMIMEInfo::ContentTransferEncodingTag);
 
     // save the entity body, being careful of binary files
+#if P_CYPHER
     if (encoding == "base64")
       PBase64::Decode(PString(partPtr, partLen), info->m_binaryBody);
-#ifdef P_HAS_WCHAR
-    else if (typeInfo("charset") *= "UCS-2")
-      info->m_textBody = PString((const wchar_t *)partPtr, partLen/2);
+    else
 #endif
-    else if (encoding == "7bit" || encoding == "8bit" || (typeInfo("charset") *= "UTF-8") || memchr(partPtr, 0, partLen) == NULL)
+#ifdef P_HAS_WCHAR
+    if (typeInfo("charset") *= "UCS-2")
+      info->m_textBody = PString((const wchar_t *)partPtr, partLen/2);
+    else
+#endif
+    if (encoding == "7bit" || encoding == "8bit" || (typeInfo("charset") *= "UTF-8") || memchr(partPtr, 0, partLen) == NULL)
       info->m_textBody = PString(partPtr, partLen);
     else
       info->m_binaryBody = PBYTEArray((const BYTE *)partPtr, partLen);
