@@ -24,9 +24,9 @@
  * Contributor(s): Craig Southeren
  *                 Robert Jongbloed
  *
- * $Revision: 28224 $
+ * $Revision: 34644 $
  * $Author: rjongbloed $
- * $Date: 2012-08-21 03:09:25 -0500 (Tue, 21 Aug 2012) $
+ * $Date: 2016-02-24 17:19:11 +0000 (Wed, 24 Feb 2016) $
  */
 
 #ifdef __GNUC__
@@ -35,23 +35,29 @@
 
 #include <ptlib.h>
 
-#include <ptbuildopts.h>
-
 #if P_LUA
 
 #include <ptclib/lua.h>
-#include <lua.hpp>
+
+extern "C" {
+  #include <lua.h>
+  #include <lualib.h>
+  #include <lauxlib.h>
+}
 
 
 #ifdef _MSC_VER
   #pragma comment(lib, P_LUA_LIBRARY)
-  #pragma message("Lua scripting support enabled")
 #endif
 
+#if LUA_VERSION_NUM > 501 && !defined(lua_objlen)
+  #define lua_objlen lua_rawlen
+#endif
 
 #define PTraceModule() "Lua"
 
-PFACTORY_CREATE(PFactory<PScriptLanguage>, PLua, "Lua", false);
+static PConstString const LuaName("Lua");
+PFACTORY_CREATE(PFactory<PScriptLanguage>, PLua, LuaName, false);
 
 #define new PNEW
 
@@ -104,6 +110,18 @@ PLua::~PLua()
 }
 
 
+PString PLua::LanguageName()
+{
+  return LuaName;
+}
+
+
+PString PLua::GetLanguageName() const
+{
+  return LuaName;
+}
+
+
 bool PLua::LoadFile(const PFilePath & filename)
 {
   PWaitAndSignal mutex(m_mutex);
@@ -607,7 +625,7 @@ bool PLua::OnLuaError(int code, const PString & str, int pop)
     if (luaError.IsEmpty())
       OnError(code, psprintf("Error code %i", code));
     else
-      OnError(code, str);
+      OnError(code, luaError);
   }
 
   if (pop > 0)
@@ -784,7 +802,7 @@ void PLua::ParamVector::Pop(void * data)
   lua_State * lua = static_cast<lua_State *>(data);
 
   resize(lua_gettop(lua));
-  PTRACE(6, NULL, NULL, "ParamVector::Pop stack=" << size());
+  PTRACE(6, PTraceModule(), "ParamVector::Pop stack=" << size());
 
   for (reverse_iterator it = rbegin(); it != rend(); ++it) {
     switch (lua_type(lua, -1)) {
@@ -808,12 +826,4 @@ void PLua::ParamVector::Pop(void * data)
     lua_pop(lua, 1);
   }
 }
-
-
-#else // P_LUA
-
-  #ifdef _MSC_VER
-    #pragma message("Lua scripting support DISABLED")
-  #endif
-
 #endif // P_LUA
