@@ -23,9 +23,9 @@
  *
  * Copyright 2003 Equivalence Pty. Ltd.
  *
- * $Revision: 28931 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 06:31:11 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 32504 $
+ * $Author: ededu $
+ * $Date: 2014-08-12 09:02:59 +0100 (Tue, 12 Aug 2014) $
  */
 
 #ifdef __GNUC__
@@ -45,7 +45,6 @@
 
 #ifdef _MSC_VER
   #pragma comment(lib, "DnsAPI.Lib")
-  #pragma message("DNS support enabled")
 #endif
 
 
@@ -558,11 +557,20 @@ PBoolean PDNS::LookupSRV(
     return false;
   }
 
+  PString addr = domain;
+  PINDEX colon = addr.FindLast(':');
+  if (colon != P_MAX_INDEX && (addr.Find('[') == P_MAX_INDEX || addr.Find(']') < colon))
+    addr.Delete(colon, P_MAX_INDEX);
+
+  if (PIPSocket::Address(addr).IsValid()) {
+    PTRACE(4, "DNS\tSRV lookup failed - domain is IP address");
+    return false;
+  }
+
   PString srvLookupStr = service;
-  if (srvLookupStr.Right(1) != ".")
-    srvLookupStr += ".";
-  srvLookupStr += domain;
-  
+  if (srvLookupStr[srvLookupStr.GetLength()-1] != '.')
+    srvLookupStr += '.';
+  srvLookupStr += addr;
   PTRACE(4,"DNS\tSRV Lookup \"" << srvLookupStr << '"');
   return LookupSRV(srvLookupStr, defaultPort, addrList);
 }
@@ -573,17 +581,16 @@ PBoolean PDNS::LookupSRV(
               PIPSocketAddressAndPortVector & addrList
 )
 {
-
   PDNS::SRVRecordList srvRecords;
-  PBoolean found = PDNS::GetRecords(srvLookupStr, srvRecords);
-  if (found) {
+  bool found = false;
+  if (PDNS::GetRecords(srvLookupStr, srvRecords)) {
     PTRACE(5,"DNS\tSRV Record found \"" << srvLookupStr << '"');
     PDNS::SRVRecord * recPtr = srvRecords.GetFirst();
     while (recPtr != NULL) {
       PIPSocketAddressAndPort addrAndPort;
       addrAndPort.SetAddress(recPtr->hostAddress, recPtr->port > 0 ? recPtr->port : defaultPort);
       addrList.push_back(addrAndPort);
-
+      found = true;
       recPtr = srvRecords.GetNext();
     }
   } 
@@ -759,17 +766,4 @@ PDNS::PDnsRecords::~PDnsRecords()
   if (m_records != NULL)
     DnsRecordListFree(m_records, DnsFreeRecordList);
 }
-
-
-/////////////////////////////////////////////////////////////////
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("DNS resolver support DISABLED")
-  #endif
-
 #endif // P_DNS_RESOLVER
-
-
-// End Of File ///////////////////////////////////////////////////////////////
