@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 30779 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2013-10-29 07:10:28 +0000 (Tue, 29 Oct 2013) $
  */
 
 #ifdef __GNUC__
@@ -40,6 +40,8 @@
 #if P_VIDEO
 #if P_FFVDEV
 
+#define P_FORCE_STATIC_PLUGIN 1
+
 #include <ptlib/vconvert.h>
 #include <ptclib/pffvdev.h>
 #include <ptlib/pfactory.h>
@@ -65,42 +67,38 @@ static const char * ffmpegExe = "ffmpeg.exe";
 static const char * ffmpegExe = "ffmpeg";
 #endif
 
-class PVideoInputDevice_FFMPEG_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject * CreateInstance(int /*userData*/) const
-    {
-      return new PVideoInputDevice_FFMPEG;
-    }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const
-    {
-       return PVideoInputDevice_FFMPEG::GetInputDeviceNames();
-    }
-    virtual bool ValidateDeviceName(const PString & deviceName, int /*userData*/) const
-    {
-      PCaselessString adjustedDevice = deviceName;
 
-      PINDEX r = 0;
-      while (ffmpegExtensions[r] != NULL) {
-        PString ext = ffmpegExtensions[r];
-        PINDEX extLen = ext.GetLength();
-        PINDEX length = adjustedDevice.GetLength();
-        if (length > (2+extLen) && adjustedDevice.NumCompare(PString(".") + ext + "*", 2+extLen, length-(2+extLen)) == PObject::EqualTo)
-          adjustedDevice.Delete(length-1, 1);
-        else if (length < (2+extLen) || adjustedDevice.NumCompare(PString(".") + ext, 1+extLen, length-(1+extLen)) != PObject::EqualTo) {
-          ++r;
-          continue;
-        }
-        if (PFile::Access(adjustedDevice, PFile::ReadOnly)) 
-          return true;
-        PTRACE(1, "FFVDev\tUnable to access file '" << adjustedDevice << "' for use as a video input device");
-        return false;
+PCREATE_VIDINPUT_PLUGIN_EX(FFMPEG,
+
+  virtual const char * GetFriendlyName() const
+  {
+    return "File Video source using FFMPEG";
+  }
+
+  virtual bool ValidateDeviceName(const PString & deviceName, P_INT_PTR /*userData*/) const
+  {
+    PCaselessString adjustedDevice = deviceName;
+
+    PINDEX r = 0;
+    while (ffmpegExtensions[r] != NULL) {
+      PString ext = ffmpegExtensions[r];
+      PINDEX extLen = ext.GetLength();
+      PINDEX length = adjustedDevice.GetLength();
+      if (length > (2+extLen) && adjustedDevice.NumCompare(PString(".") + ext + "*", 2+extLen, length-(2+extLen)) == PObject::EqualTo)
+        adjustedDevice.Delete(length-1, 1);
+      else if (length < (2+extLen) || adjustedDevice.NumCompare(PString(".") + ext, 1+extLen, length-(1+extLen)) != PObject::EqualTo) {
+        ++r;
+        continue;
       }
+      if (PFile::Access(adjustedDevice, PFile::ReadOnly)) 
+        return true;
+      PTRACE(1, "FFVDev\tUnable to access file '" << adjustedDevice << "' for use as a video input device");
       return false;
     }
-} PVideoInputDevice_FFMPEG_descriptor;
+    return false;
+  }
+);
 
-PCREATE_PLUGIN(FFMPEG, PVideoInputDevice, &PVideoInputDevice_FFMPEG_descriptor);
 
 
 PVideoInputDevice_FFMPEG::PVideoInputDevice_FFMPEG()
@@ -236,17 +234,6 @@ PBoolean PVideoInputDevice_FFMPEG::SetVideoFormat(VideoFormat newFormat)
 }
 
 
-int PVideoInputDevice_FFMPEG::GetNumChannels() 
-{
-  return 1;  
-}
-
-
-PBoolean PVideoInputDevice_FFMPEG::SetChannel(int newChannel)
-{
-  return PVideoDevice::SetChannel(newChannel);
-}
-
 PBoolean PVideoInputDevice_FFMPEG::SetColourFormat(const PString & newFormat)
 {
   if (!(newFormat *= "YUV420P"))
