@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
- * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Revision: 32504 $
+ * $Author: ededu $
+ * $Date: 2014-08-12 09:02:59 +0100 (Tue, 12 Aug 2014) $
  */
 
 #ifdef __GNUC__
@@ -48,7 +48,6 @@
 
 #if defined(_MSC_VER)
   #pragma comment(lib, P_LDAP_LIBRARY)
-  #pragma message("LDAP support enabled")
 #endif
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -915,11 +914,6 @@ void PLDAPStructBase::EndConstructor()
 
 static const char PLDAPSchemaPluginBaseClass[] = "PLDAPSchema";
 
-template <> PLDAPSchema * PDevicePluginFactory<PLDAPSchema>::Worker::Create(const PDefaultPFactoryKey & type) const
-{
-  return PLDAPSchema::CreateSchema(type);
-}
-
 PLDAPSchema::PLDAPSchema()
 {
 }
@@ -934,28 +928,19 @@ void PLDAPSchema::LoadSchema()
 	AttributeList(attributelist); 
 }
 
-PLDAPSchema * PLDAPSchema::CreateSchema(const PString & schemaname, PPluginManager * pluginMgr)
+PLDAPSchema * PLDAPSchema::CreateSchema(const PString & schemaName, PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return (PLDAPSchema *)pluginMgr->CreatePluginsDeviceByName(schemaname, PLDAPSchemaPluginBaseClass);
+  return PPluginManager::CreatePluginAs<PLDAPSchema>(pluginMgr, schemaName, PLDAPSchemaPluginBaseClass);
 }
 
-PStringList PLDAPSchema::GetSchemaNames(PPluginManager * pluginMgr)
+PStringArray PLDAPSchema::GetSchemaNames(PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return pluginMgr->GetPluginsProviding(PLDAPSchemaPluginBaseClass);
+  return PPluginManager::GetPluginsProviding(pluginMgr, PLDAPSchemaPluginBaseClass, false);
 }
 
-PStringList PLDAPSchema::GetSchemaFriendlyNames(const PString & schema, PPluginManager * pluginMgr)
+PStringArray PLDAPSchema::GetSchemaFriendlyNames(PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return pluginMgr->GetPluginsDeviceNames(schema, PLDAPSchemaPluginBaseClass);
+  return PPluginManager::GetPluginsProviding(pluginMgr, PLDAPSchemaPluginBaseClass, true);
 }
 
 void PLDAPSchema::OnReceivedAttribute(const PString & attribute, const PString & value)
@@ -1059,16 +1044,4 @@ void PLDAPSchema::OnSendSchema(PArray<PLDAPSession::ModAttrib> & attrib, PLDAPSe
         attrib.Append(new PLDAPSession::BinaryModAttrib(s->first,s->second,op));
 	}
 }
-
-
-#else
-
-  #if defined(_MSC_VER)
-    #pragma message("LDAP support DISABLED")
-  #endif
-
 #endif // P_LDAP
-
-
-// End of file ////////////////////////////////////////////////////////////////
-
