@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 32816 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2014-09-25 04:59:23 +0100 (Thu, 25 Sep 2014) $
  */
 
 #ifdef __GNUC__
@@ -52,7 +52,6 @@ extern "C" {
 
 #ifdef _MSC_VER
   #pragma comment(lib, P_SASL_LIBRARY)
-  #pragma message("SASL support enabled")
 #endif
 
 ///////////////////////////////////////////////////////
@@ -174,7 +173,7 @@ static void psasl_Initialise()
   sasl_client_init(cbs);
 }
 
-static PAtomicInteger psasl_UsageCount(0);
+static atomic<uint32_t> psasl_UsageCount(0);
 PString PSASLClient::s_Realm;
 PString PSASLClient::s_Path;
 
@@ -336,15 +335,5 @@ PBoolean PSASLClient::End()
 
     return false;
 }
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("SASL support DISABLED")
-  #endif
-
 #endif // P_SASL
 
-// End of File ///////////////////////////////////////////////////////////////
-
