@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29695 $
+ * $Revision: 34827 $
  * $Author: rjongbloed $
- * $Date: 2013-05-13 02:57:28 -0500 (Mon, 13 May 2013) $
+ * $Date: 2016-05-12 09:51:21 +0100 (Thu, 12 May 2016) $
  */
 
 //////////////////////////////////////////////////
@@ -98,8 +98,7 @@ void PInterfaceMonitor::Start()
 
     if (m_runMonitorThread) {
       m_changedDetector = PIPSocket::CreateRouteTableDetector();
-      m_updateThread = new PThreadObj<PInterfaceMonitor>(*this, &PInterfaceMonitor::UpdateThreadMain);
-      m_updateThread->SetThreadName("Network Interface Monitor");
+      m_updateThread = new PThreadObj<PInterfaceMonitor>(*this, &PInterfaceMonitor::UpdateThreadMain, false, "Network Interface Monitor");
     }
   }
 }
@@ -208,13 +207,15 @@ void PInterfaceMonitor::RefreshInterfaceList()
   // look for interfaces to add that are in new list that are not in the old list
   for (i = 0; i < newInterfaces.GetSize(); ++i) {
     PIPSocket::InterfaceEntry & newEntry = newInterfaces[i];
-    if (!newEntry.GetAddress().IsLoopback() && !IsInterfaceInList(newEntry, oldInterfaces))
+    PIPSocket::Address addr = newEntry.GetAddress();
+    if (addr.IsValid() && !addr.IsLoopback() && !IsInterfaceInList(newEntry, oldInterfaces))
       interfacesToAdd.Append(&newEntry);
   }
   // look for interfaces to remove that are in old list that are not in the new list
   for (i = 0; i < oldInterfaces.GetSize(); ++i) {
     PIPSocket::InterfaceEntry & oldEntry = oldInterfaces[i];
-    if (!oldEntry.GetAddress().IsLoopback() && !IsInterfaceInList(oldEntry, newInterfaces))
+    PIPSocket::Address addr = oldEntry.GetAddress();
+    if (addr.IsValid() && !addr.IsLoopback() && !IsInterfaceInList(oldEntry, newInterfaces))
       interfacesToRemove.Append(&oldEntry);
   }
 
@@ -302,7 +303,7 @@ PStringArray PInterfaceMonitor::GetInterfaces(bool includeLoopBack,
 
   for (PINDEX i = 0; i < ifaces.GetSize(); ++i) {
     PIPSocket::InterfaceEntry & entry = ifaces[i];
-    if (includeLoopBack || !entry.GetAddress().IsLoopback())
+    if (entry.GetAddress().IsValid() && (includeLoopBack || !entry.GetAddress().IsLoopback()))
       names[count++] = MakeInterfaceDescription(entry);
   }
 
@@ -415,29 +416,16 @@ void PInterfaceMonitor::OnInterfacesChanged(const PIPSocket::InterfaceTable & ad
 }
 
 
-#if P_NAT
-
-void PInterfaceMonitor::OnRemoveNatMethod(const PNatMethod * natMethod)
-{
-  PWaitAndSignal guard(m_notifiersMutex);
-
-  for (Notifiers::iterator it = m_notifiers.begin(); it != m_notifiers.end(); ++it)
-    it->second(*this, InterfaceChange(natMethod, false));
-}
-
-#endif
-
-
 //////////////////////////////////////////////////
 
-PMonitoredSockets::PMonitoredSockets(bool reuseAddr P_NAT_PARAM(PNatMethod * nat))
-  : localPort(0)
-  , reuseAddress(reuseAddr)
+PMonitoredSockets::PMonitoredSockets(bool reuseAddr P_NAT_PARAM(PNatMethods * nat))
+  : m_localPort(0)
+  , m_reuseAddress(reuseAddr)
 #if P_NAT
-  , natMethod(nat)
+  , m_natMethods(nat)
 #endif
-  , opened(false)
-  , interfaceAddedSignal(localPort, PIPSocket::GetDefaultIpAddressFamily())
+  , m_opened(false)
+  , m_interfaceAddedSignal(m_localPort, PIPSocket::GetDefaultIpAddressFamily())
 {
 }
 
@@ -460,23 +448,29 @@ bool PMonitoredSockets::CreateSocket(SocketInfo & info, const PIPSocket::Address
   info.socket = NULL;
   
 #if P_NAT
-  if (natMethod != NULL && natMethod->IsAvailable(binding)) {
-    PIPSocket::Address address;
-    WORD port;
-    natMethod->GetServerAddress(address, port);
-    if (PInterfaceMonitor::GetInstance().IsValidBindingForDestination(binding, address)) {
-      if (natMethod->CreateSocket(info.socket, binding, localPort)) {
-        info.socket->PUDPSocket::GetLocalAddress(address, port);
-        PTRACE(4, "Created bundled UDP socket via " << natMethod->GetName()
-               << ", internal=" << address << ':' << port << ", external=" << info.socket->GetLocalAddress());
-        return true;
+  if (m_natMethods != NULL) {
+    PNatMethod * natMethod = m_natMethods->GetMethod(binding, this);
+    if (natMethod) {
+      PIPAddressAndPort ap;
+      natMethod->GetServerAddress(ap);
+      if (PInterfaceMonitor::GetInstance().IsValidBindingForDestination(binding, ap.GetAddress())) {
+        if (natMethod->CreateSocket(info.socket, binding, m_localPort)) {
+          PNATUDPSocket * natSocket = dynamic_cast<PNATUDPSocket*>(info.socket);
+          if (natSocket != NULL)
+            natSocket->GetBaseAddress(ap);
+          else
+            ap.SetAddress(0);
+          PTRACE(4, "Created bundled UDP socket via " << natMethod->GetMethodName()
+                 << ", internal=" << ap << ", external=" << info.socket->GetLocalAddress());
+          return true;
+        }
       }
     }
   }
 #endif
 
-  info.socket = new PUDPSocket(localPort, (int) (binding.GetVersion() == 6 ? AF_INET6 : AF_INET));
-  if (info.socket->Listen(binding, 0, localPort, reuseAddress?PIPSocket::CanReuseAddress:PIPSocket::AddressIsExclusive)) {
+  info.socket = new PUDPSocket(m_localPort, (int) (binding.GetVersion() == 6 ? AF_INET6 : AF_INET));
+  if (info.socket->Listen(binding, 0, m_localPort, m_reuseAddress?PIPSocket::CanReuseAddress:PIPSocket::AddressIsExclusive)) {
     PTRACE(4, "Created bundled UDP socket " << binding << ':' << info.socket->GetPort());
     int sz = 0;
     if (info.socket->GetOption(SO_RCVBUF, sz) && sz < UDP_BUFFER_SIZE) {
@@ -485,11 +479,12 @@ bool PMonitoredSockets::CreateSocket(SocketInfo & info, const PIPSocket::Address
       }
     }
 
+    info.socket->SetReadTimeout(0);
     return true;
   }
 
   PTRACE(1, "Could not listen on "
-         << binding << ':' << localPort
+         << binding << ':' << m_localPort
          << " - " << info.socket->GetErrorText());
   delete info.socket;
   info.socket = NULL;
@@ -575,7 +570,7 @@ void PMonitoredSockets::ReadFromSocketList(PSocket::SelectList & readers,
 
   param.m_errorCode = PSocket::Select(readers, param.m_timeout);
 
-  if (!LockReadWrite() || !opened) {
+  if (!LockReadWrite() || !m_opened) {
     param.m_errorCode = PChannel::NotOpen;  // Closed, break out
     return;
   }
@@ -584,14 +579,18 @@ void PMonitoredSockets::ReadFromSocketList(PSocket::SelectList & readers,
     case PChannel::NoError :
       break;
 
-    case PChannel::NotOpen : // Interface went down
-      if (!interfaceAddedSignal.IsOpen()) {
-        interfaceAddedSignal.Listen(); // Reset if this was used to break Select() block
+    case PChannel::NotOpen :
+    case PChannel::Interrupted :
+      // Interface went down (socket was closed)
+      if (!m_interfaceAddedSignal.IsOpen()) {
+        m_interfaceAddedSignal.Listen(); // Reset if this was used to break Select() block
         param.m_errorCode = PChannel::Interrupted;
         PTRACE(4, "Interfaces changed");
         return;
       }
-      // Do next case
+      if (readers.IsEmpty())
+        return;
+      break;
 
     default :
       return;
@@ -639,7 +638,7 @@ void PMonitoredSockets::SocketInfo::Read(PMonitoredSockets & bundle, BundleParam
   // Assume is already locked
 
   if (inUse) {
-    PTRACE(2, &bundle, NULL, "Cannot read from multiple threads.");
+    PTRACE(2, &bundle, PTraceModule(), "Cannot read from multiple threads.");
     param.m_errorCode = PChannel::DeviceInUse;
     return;
   }
@@ -654,7 +653,7 @@ void PMonitoredSockets::SocketInfo::Read(PMonitoredSockets & bundle, BundleParam
       sockets += *socket;
       inUse = true;
     }
-    sockets += bundle.interfaceAddedSignal;
+    sockets += bundle.m_interfaceAddedSignal;
 
     PUDPSocket * socket;
     bundle.ReadFromSocketList(sockets, socket, param);
@@ -664,111 +663,97 @@ void PMonitoredSockets::SocketInfo::Read(PMonitoredSockets & bundle, BundleParam
 }
 
 
-PMonitoredSockets * PMonitoredSockets::Create(const PString & iface, bool reuseAddr P_NAT_PARAM(PNatMethod * natMethod))
+PMonitoredSockets * PMonitoredSockets::Create(const PString & iface, bool reuseAddr P_NAT_PARAM(PNatMethods * natMethods))
 {
   if (iface.IsEmpty() || iface == "*")
-    return new PMonitoredSocketBundle(PString::Empty(), 0, reuseAddr P_NAT_PARAM(natMethod));
+    return new PMonitoredSocketBundle(PString::Empty(), 0, reuseAddr P_NAT_PARAM(natMethods));
 
   PINDEX percent = iface.Find('%');
 
   if (percent == 0 || (percent == 1 && iface[0] == '*'))
-    return new PMonitoredSocketBundle(iface.Mid(percent+1), 0, reuseAddr P_NAT_PARAM(natMethod));
+    return new PMonitoredSocketBundle(iface.Mid(percent+1), 0, reuseAddr P_NAT_PARAM(natMethods));
 
-  PIPSocket::Address ip(iface);
-  if (!ip.IsValid())
+  PIPSocket::Address ip;
+  if (!PIPSocket::GetHostAddress(iface, ip))
     return NULL;
-  
+
   if (ip.IsAny())
     return new PMonitoredSocketBundle(percent != P_MAX_INDEX ? iface.Mid(percent+1) : PString::Empty(),
-                                      ip.GetVersion(), reuseAddr P_NAT_PARAM(natMethod));
+                                      ip.GetVersion(), reuseAddr P_NAT_PARAM(natMethods));
 
-  return new PSingleMonitoredSocket(ip.AsString(true), reuseAddr P_NAT_PARAM(natMethod));
+  return new PSingleMonitoredSocket(ip.AsString(true), reuseAddr P_NAT_PARAM(natMethods));
 }
 
 
-#if P_NAT
-
-void PMonitoredSockets::OnRemoveNatMethod(const PNatMethod * nat)
-{
-  if (natMethod == nat)
-    natMethod = NULL;
-}
-
-#endif
-
-
 //////////////////////////////////////////////////
 
 PMonitoredSocketChannel::PMonitoredSocketChannel(const PMonitoredSocketsPtr & sock, bool shared)
-  : socketBundle(sock)
-  , sharedBundle(shared)
-  , promiscuousReads(false)
-  , closing(false)
-  , remotePort(0)
-  , lastReceivedAddress(PIPSocket::GetDefaultIpAny())
-  , lastReceivedPort(0)
+  : m_socketBundle(sock)
+  , m_sharedBundle(shared)
+  , m_promiscuousReads(false)
+  , m_closing(false)
+  , m_lastReceivedAP(PIPSocket::GetDefaultIpAny())
 {
 }
 
 
 PBoolean PMonitoredSocketChannel::IsOpen() const
 {
-  return !closing && socketBundle != NULL && socketBundle->IsOpen();
+  return !m_closing && m_socketBundle != NULL && m_socketBundle->IsOpen();
 }
 
 
 PBoolean PMonitoredSocketChannel::Close()
 {
-  closing = true;
-  return sharedBundle || socketBundle == NULL || socketBundle->Close();
+  m_closing = true;
+  return m_sharedBundle || m_socketBundle == NULL || m_socketBundle->Close();
 }
 
 
 PBoolean PMonitoredSocketChannel::Read(void * buffer, PINDEX length)
 {
-  if (!IsOpen())
+  if (CheckNotOpen())
     return false;
 
   do {
-    lastReceivedInterface = GetInterface();
+    m_lastReceivedInterface = GetInterface();
     PMonitoredSockets::BundleParams param;
     param.m_buffer = buffer;
     param.m_length = length;
     param.m_timeout = readTimeout;
-    socketBundle->ReadFromBundle(param);
-    lastReceivedAddress = param.m_addr;
-    lastReceivedPort = param.m_port;
-    lastReceivedInterface = param.m_iface;
+    m_socketBundle->ReadFromBundle(param);
+    m_lastReceivedAP.SetAddress(param.m_addr, param.m_port);
+    m_lastReceivedInterface = param.m_iface;
     lastReadCount = param.m_lastCount;
     if (!SetErrorValues(param.m_errorCode, param.m_errorNumber, LastReadError))
       return false;
 
-    if (promiscuousReads)
+    if (m_promiscuousReads)
       return true;
 
-    if (remoteAddress.IsAny())
-      remoteAddress = lastReceivedAddress;
-    if (remotePort == 0)
-      remotePort = lastReceivedPort;
+    if (m_remoteAP.GetAddress().IsAny())
+      m_remoteAP.SetAddress(m_lastReceivedAP.GetAddress());
+    if (m_remoteAP.GetPort() == 0)
+      m_remoteAP.SetPort(m_lastReceivedAP.GetPort());
 
-  } while (remoteAddress != lastReceivedAddress || remotePort != lastReceivedPort);
+  } while (m_remoteAP != m_lastReceivedAP);
   return true;
 }
 
 
 PBoolean PMonitoredSocketChannel::Write(const void * buffer, PINDEX length)
 {
-  if (!IsOpen())
+  if (CheckNotOpen())
     return false;
 
   PMonitoredSockets::BundleParams param;
   param.m_buffer = (void *)buffer;
   param.m_length = length;
-  param.m_addr = remoteAddress;
-  param.m_port = remotePort;
+  param.m_addr = m_remoteAP.GetAddress();
+  param.m_port = m_remoteAP.GetPort();
   param.m_iface = GetInterface();
   param.m_timeout = readTimeout;
-  socketBundle->WriteToBundle(param);
+  m_socketBundle->WriteToBundle(param);
   lastWriteCount = param.m_lastCount;
   return SetErrorValues(param.m_errorCode, param.m_errorNumber, LastWriteError);
 }
@@ -776,18 +761,18 @@ PBoolean PMonitoredSocketChannel::Write(const void * buffer, PINDEX length)
 
 void PMonitoredSocketChannel::SetInterface(const PString & iface)
 {
-  mutex.Wait();
+  m_mutex.Wait();
 
   PIPSocket::InterfaceEntry info;
-  if (socketBundle != NULL && socketBundle->GetInterfaceInfo(iface, info))
-    currentInterface = MakeInterfaceDescription(info);
+  if (m_socketBundle != NULL && m_socketBundle->GetInterfaceInfo(iface, info))
+    m_currentInterface = MakeInterfaceDescription(info);
   else
-    currentInterface = iface;
+    m_currentInterface = iface;
 
-  if (lastReceivedInterface.IsEmpty())
-    lastReceivedInterface = currentInterface;
+  if (m_lastReceivedInterface.IsEmpty())
+    m_lastReceivedInterface = m_currentInterface;
 
-  mutex.Signal();
+  m_mutex.Signal();
 }
 
 
@@ -795,15 +780,15 @@ PString PMonitoredSocketChannel::GetInterface()
 {
   PString iface;
 
-  mutex.Wait();
+  m_mutex.Wait();
 
-  if (currentInterface.Find('%') == P_MAX_INDEX)
-    SetInterface(currentInterface);
+  if (m_currentInterface.Find('%') == P_MAX_INDEX)
+    SetInterface(m_currentInterface);
 
-  iface = currentInterface;
+  iface = m_currentInterface;
   iface.MakeUnique();
 
-  mutex.Signal();
+  m_mutex.Signal();
 
   return iface;
 }
@@ -811,26 +796,26 @@ PString PMonitoredSocketChannel::GetInterface()
 
 bool PMonitoredSocketChannel::GetLocal(PIPSocket::Address & address, WORD & port, bool usingNAT)
 {
-  return socketBundle->GetAddress(GetInterface(), address, port, usingNAT);
+  return m_socketBundle->GetAddress(GetInterface(), address, port, usingNAT);
 }
 
 
-void PMonitoredSocketChannel::SetRemote(const PIPSocket::Address & address, WORD port)
+bool PMonitoredSocketChannel::GetLocal(PIPSocket::AddressAndPort & ap, bool usingNAT)
 {
-  remoteAddress = address;
-  remotePort = port;
+  PIPAddress ip;
+  WORD port;
+  if (!m_socketBundle->GetAddress(GetInterface(), ip, port, usingNAT))
+    return false;
+
+  ap.SetAddress(ip);
+  ap.SetPort(port);
+  return true;
 }
 
 
 void PMonitoredSocketChannel::SetRemote(const PString & hostAndPort)
 {
-  PINDEX colon = hostAndPort.Find(':');
-  if (colon == P_MAX_INDEX)
-    remoteAddress = hostAndPort;
-  else {
-    remoteAddress = hostAndPort.Left(colon);
-    remotePort = PIPSocket::GetPortByService("udp", hostAndPort.Mid(colon+1));
-  }
+  m_remoteAP.Parse(hostAndPort, 0, ':', "udp");
 }
 
 
@@ -839,12 +824,13 @@ void PMonitoredSocketChannel::SetRemote(const PString & hostAndPort)
 PMonitoredSocketBundle::PMonitoredSocketBundle(const PString & fixedInterface,
                                                unsigned ipVersion,
                                                bool reuseAddr
-                                               P_NAT_PARAM(PNatMethod * natMethod))
-  : PMonitoredSockets(reuseAddr P_NAT_PARAM(natMethod))
+                                               P_NAT_PARAM(PNatMethods * natMethods))
+  : PMonitoredSockets(reuseAddr P_NAT_PARAM(natMethods))
+  , m_onInterfaceChange(PCREATE_InterfaceNotifier(OnInterfaceChange))
   , m_fixedInterface(fixedInterface)
   , m_ipVersion(ipVersion)
 {
-  PInterfaceMonitor::GetInstance().AddNotifier(PCREATE_InterfaceNotifier(OnInterfaceChange));
+  PInterfaceMonitor::GetInstance().AddNotifier(m_onInterfaceChange);
 
   PTRACE(4, "Created socket bundle for "
          << (fixedInterface.IsEmpty() ? "all" : "fixed")
@@ -857,7 +843,7 @@ PMonitoredSocketBundle::~PMonitoredSocketBundle()
 {
   Close();
 
-  PInterfaceMonitor::GetInstance().RemoveNotifier(PCREATE_InterfaceNotifier(OnInterfaceChange));
+  PInterfaceMonitor::GetInstance().RemoveNotifier(m_onInterfaceChange);
 }
 
 
@@ -876,12 +862,12 @@ PBoolean PMonitoredSocketBundle::Open(WORD port)
 {
   PSafeLockReadWrite guard(*this);
 
-  if (IsOpen() && localPort != 0  && localPort == port)
+  if (IsOpen() && m_localPort != 0  && m_localPort == port)
     return true;
 
-  opened = true;
+  m_opened = true;
 
-  localPort = port;
+  m_localPort = port;
 
   // Close and re-open all sockets
   while (!m_socketInfoMap.empty())
@@ -891,7 +877,7 @@ PBoolean PMonitoredSocketBundle::Open(WORD port)
   for (PINDEX i = 0; i < interfaces.GetSize(); ++i)
     OpenSocket(interfaces[i]);
 
-  return true;
+  return !m_socketInfoMap.empty(); // any of the sockets are opened
 }
 
 
@@ -900,11 +886,11 @@ PBoolean PMonitoredSocketBundle::Close()
   if (!LockReadWrite())
     return false;
 
-  opened = false;
+  m_opened = false;
 
   while (!m_socketInfoMap.empty())
     CloseSocket(m_socketInfoMap.begin());
-  interfaceAddedSignal.Close(); // Fail safe break out of Select()
+  m_interfaceAddedSignal.Close(); // Fail safe break out of Select()
 
   UnlockReadWrite();
 
@@ -927,7 +913,7 @@ PBoolean PMonitoredSocketBundle::GetAddress(const PString & iface,
   }
 
   address = PIPSocket::Address::GetAny(m_ipVersion);
-  port = localPort;
+  port = m_localPort;
   return false;
 }
 
@@ -955,10 +941,10 @@ void PMonitoredSocketBundle::OpenSocket(const PString & iface)
 
   SocketInfo info;
   if (CreateSocket(info, binding)) {
-    if (localPort == 0) {
+    if (m_localPort == 0) {
       PIPSocketAddressAndPort addrAndPort;
       info.socket->PUDPSocket::InternalGetLocalAddress(addrAndPort);
-      localPort = addrAndPort.GetPort();
+      m_localPort = addrAndPort.GetPort();
     }
     m_socketInfoMap[iface] = info;
   }
@@ -1005,7 +991,7 @@ void PMonitoredSocketBundle::WriteToBundle(BundleParams & param)
 
 void PMonitoredSocketBundle::ReadFromBundle(BundleParams & param)
 {
-  if (!opened || !LockReadWrite()) {
+  if (!m_opened || !LockReadWrite()) {
     param.m_errorCode = PChannel::NotOpen;
     return;
   }
@@ -1027,7 +1013,7 @@ void PMonitoredSocketBundle::ReadFromBundle(BundleParams & param)
           iter->second.inUse = true;
         }
       }
-      readers += interfaceAddedSignal;
+      readers += m_interfaceAddedSignal;
 
       PUDPSocket * socket;
       ReadFromSocketList(readers, socket, param);
@@ -1054,20 +1040,17 @@ void PMonitoredSocketBundle::ReadFromBundle(BundleParams & param)
 
 void PMonitoredSocketBundle::OnInterfaceChange(PInterfaceMonitor &, PInterfaceMonitor::InterfaceChange entry)
 {
-  if (!opened || !LockReadWrite())
+  if (!m_opened || !LockReadWrite())
     return;
 
   if (entry.m_added) {
     OpenSocket(MakeInterfaceDescription(entry));
     PTRACE(3, "UDP socket bundle has added interface " << entry);
-    interfaceAddedSignal.Close();
+    m_interfaceAddedSignal.Close();
   }
   else {
     CloseSocket(m_socketInfoMap.find(MakeInterfaceDescription(entry)));
     PTRACE(3, "UDP socket bundle has removed interface " << entry);
-#if P_NAT
-    OnRemoveNatMethod(entry.m_natMethod);
-#endif
   }
 
   UnlockReadWrite();
@@ -1076,11 +1059,12 @@ void PMonitoredSocketBundle::OnInterfaceChange(PInterfaceMonitor &, PInterfaceMo
 
 //////////////////////////////////////////////////
 
-PSingleMonitoredSocket::PSingleMonitoredSocket(const PString & theInterface, bool reuseAddr P_NAT_PARAM(PNatMethod * natMethod))
-  : PMonitoredSockets(reuseAddr P_NAT_PARAM(natMethod))
+PSingleMonitoredSocket::PSingleMonitoredSocket(const PString & theInterface, bool reuseAddr P_NAT_PARAM(PNatMethods * natMethods))
+  : PMonitoredSockets(reuseAddr P_NAT_PARAM(natMethods))
+  , m_onInterfaceChange(PCREATE_InterfaceNotifier(OnInterfaceChange))
   , m_interface(theInterface)
 {
-  PInterfaceMonitor::GetInstance().AddNotifier(PCREATE_InterfaceNotifier(OnInterfaceChange));
+  PInterfaceMonitor::GetInstance().AddNotifier(m_onInterfaceChange);
 
   PTRACE(4, "Created monitored socket for interface " << theInterface);
 }
@@ -1090,7 +1074,7 @@ PSingleMonitoredSocket::~PSingleMonitoredSocket()
 {
   Close();
 
-  PInterfaceMonitor::GetInstance().RemoveNotifier(PCREATE_InterfaceNotifier(OnInterfaceChange));
+  PInterfaceMonitor::GetInstance().RemoveNotifier(m_onInterfaceChange);
 }
 
 
@@ -1109,14 +1093,14 @@ PBoolean PSingleMonitoredSocket::Open(WORD port)
 {
   PSafeLockReadWrite guard(*this);
 
-  if (opened && localPort == port && m_info.socket != NULL && m_info.socket->IsOpen())
+  if (m_opened && m_localPort == port && m_info.socket != NULL && m_info.socket->IsOpen())
     return true;
 
   Close();
 
-  opened = true;
+  m_opened = true;
 
-  localPort = port;
+  m_localPort = port;
 
   if (!m_entry.GetAddress().IsValid() && !GetInterfaceInfo(m_interface, m_entry)) {
     PTRACE(3, "Not creating socket as interface \"" << m_entry.GetName() << "\" is  not up.");
@@ -1126,7 +1110,7 @@ PBoolean PSingleMonitoredSocket::Open(WORD port)
   if (!CreateSocket(m_info, m_entry.GetAddress()))
     return false;
     
-  localPort = m_info.socket->PUDPSocket::GetPort();
+  m_localPort = m_info.socket->PUDPSocket::GetPort();
   return true;
 }
 
@@ -1135,11 +1119,11 @@ PBoolean PSingleMonitoredSocket::Close()
 {
   PSafeLockReadWrite guard(*this);
 
-  if (!opened)
+  if (!m_opened)
     return true;
 
-  opened = false;
-  interfaceAddedSignal.Close(); // Fail safe break out of Select()
+  m_opened = false;
+  m_interfaceAddedSignal.Close(); // Fail safe break out of Select()
   return DestroySocket(m_info);
 }
 
@@ -1168,7 +1152,7 @@ void PSingleMonitoredSocket::WriteToBundle(BundleParams & param)
 
 void PSingleMonitoredSocket::ReadFromBundle(BundleParams & param)
 {
-  if (!opened || !LockReadWrite()) {
+  if (!m_opened || !LockReadWrite()) {
     param.m_errorCode = PChannel::NotOpen;
     return;
   }
@@ -1187,7 +1171,7 @@ void PSingleMonitoredSocket::ReadFromBundle(BundleParams & param)
 void PSingleMonitoredSocket::OnInterfaceChange(PInterfaceMonitor &, PInterfaceMonitor::InterfaceChange entry)
 {
   PSafeLockReadWrite guard(*this);
-  if (!guard.IsLocked() || !opened)
+  if (!guard.IsLocked() || !m_opened)
     return;
 
   if (entry.m_added) {
@@ -1198,10 +1182,10 @@ void PSingleMonitoredSocket::OnInterfaceChange(PInterfaceMonitor &, PInterfaceMo
 
     if ((!addr.IsValid() || entry.GetAddress() == addr) && entry.GetName().NumCompare(name) == EqualTo) {
       m_entry = entry;
-      if (!Open(localPort))
+      if (!Open(m_localPort))
         m_entry = InterfaceEntry();
       else {
-        interfaceAddedSignal.Close();
+        m_interfaceAddedSignal.Close();
         PTRACE(3, "Bound UDP socket UP event on interface " << m_entry);
       }
     }
@@ -1213,9 +1197,6 @@ void PSingleMonitoredSocket::OnInterfaceChange(PInterfaceMonitor &, PInterfaceMo
     PTRACE(3, "Bound UDP socket DOWN event on interface " << m_entry);
     m_entry = InterfaceEntry();
     DestroySocket(m_info);
-#if P_NAT
-    OnRemoveNatMethod(entry.m_natMethod);
-#endif
   }
 }
 
