@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28665 $
+ * $Revision: 33231 $
  * $Author: rjongbloed $
- * $Date: 2012-12-12 00:08:21 -0600 (Wed, 12 Dec 2012) $
+ * $Date: 2015-01-10 02:44:45 +0000 (Sat, 10 Jan 2015) $
  */
 
 #ifdef __GNUC__
@@ -40,63 +40,35 @@
 #if P_VIDEO
 #if P_VIDFILE
 
-#include <ptlib/vconvert.h>
+#define P_FORCE_STATIC_PLUGIN 1
+
 #include <ptclib/pvfiledev.h>
 #include <ptlib/pfactory.h>
-#include <ptlib/pluginmgr.h>
-#include <ptlib/videoio.h>
-
-
-static const char DefaultYUVFileName[] = "*.yuv";
 
 
 #define new PNEW
 
 
 ///////////////////////////////////////////////////////////////////////////////
-// PVideoInputDevice_YUVFile
+// PVideoInputDevice_VideoFile
 
-class PVideoInputDevice_YUVFile_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    typedef PFactory<PVideoFile> FileTypeFactory_T;
+PCREATE_VIDINPUT_PLUGIN_EX(VideoFile,
 
-    virtual PObject * CreateInstance(int /*userData*/) const
-    {
-      return new PVideoInputDevice_YUVFile;
-    }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const
-    {
-      return PVideoInputDevice_YUVFile::GetInputDeviceNames();
-    }
-    virtual bool ValidateDeviceName(const PString & deviceName, int /*userData*/) const
-    {
-      PCaselessString adjustedDevice = deviceName;
+  virtual const char * GetFriendlyName() const
+  {
+    return "Raw YUV File Video Input";
+  }
 
-      FileTypeFactory_T::KeyList_T keyList = FileTypeFactory_T::GetKeyList();
-      FileTypeFactory_T::KeyList_T::iterator r;
-      for (r = keyList.begin(); r != keyList.end(); ++r) {
-        PString ext = *r;
-        PINDEX extLen = ext.GetLength();
-        PINDEX length = adjustedDevice.GetLength();
-        if (length > (2+extLen) && adjustedDevice.NumCompare(PString(".") + ext + "*", 2+extLen, length-(2+extLen)) == PObject::EqualTo)
-          adjustedDevice.Delete(length-1, 1);
-        else if (length < (2+extLen) || adjustedDevice.NumCompare(PString(".") + ext, 1+extLen, length-(1+extLen)) != PObject::EqualTo)
-          continue;
-        if (PFile::Access(adjustedDevice, PFile::ReadOnly)) 
-          return true;
-        PTRACE(1, "Unable to access file '" << adjustedDevice << "' for use as a video input device");
-        return false;
-      }
-      return false;
-    }
-} PVideoInputDevice_YUVFile_descriptor;
-
-PCREATE_PLUGIN(YUVFile, PVideoInputDevice, &PVideoInputDevice_YUVFile_descriptor);
+  virtual bool ValidateDeviceName(const PString & deviceName, P_INT_PTR /*userData*/) const
+  {
+    PVideoFileFactory::KeyList_T keyList = PVideoFileFactory::GetKeyList();
+    return std::find(keyList.begin(), keyList.end(), PFilePath(deviceName).GetType()) != keyList.end();
+  }
+);
 
 
 
-PVideoInputDevice_YUVFile::PVideoInputDevice_YUVFile()
+PVideoInputDevice_VideoFile::PVideoInputDevice_VideoFile()
   : m_file(NULL)
   , m_pacing(500)
   , m_frameRateAdjust(0)
@@ -106,62 +78,72 @@ PVideoInputDevice_YUVFile::PVideoInputDevice_YUVFile()
 }
 
 
-PVideoInputDevice_YUVFile::~PVideoInputDevice_YUVFile()
+PVideoInputDevice_VideoFile::~PVideoInputDevice_VideoFile()
 {
   Close();
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::Open(const PString & devName, PBoolean /*startImmediate*/)
+PBoolean PVideoInputDevice_VideoFile::Open(const PString & devName, PBoolean /*startImmediate*/)
 {
   Close();
 
-  PFilePath fileName;
-  if (devName != DefaultYUVFileName) {
-    fileName = devName;
-    PINDEX lastCharPos = fileName.GetLength()-1;
-    if (fileName[lastCharPos] == '*') {
-      fileName.Delete(lastCharPos, 1);
-      SetChannel(Channel_PlayAndRepeat);
-    }
-  }
+  if (devName.IsEmpty())
+    return false;
+
+  PFilePath filePath;
+
+  PINDEX pos = devName.GetLength()-1;
+  if (devName[pos] != '*')
+    filePath = devName;
   else {
-    PDirectory dir;
+    filePath = devName.Left(pos);
+    SetChannel(Channel_PlayAndRepeat);
+  }
+
+  if (filePath.Find('*') != P_MAX_INDEX) {
+    bool noFilesOfType = true;
+    PDirectory dir = filePath.GetDirectory();
+    PTRACE(1, "VidFileDev", "Searching directory \"" << dir << '"');
     if (dir.Open(PFileInfo::RegularFile|PFileInfo::SymbolicLink)) {
       do {
-        if (dir.GetEntryName().Right(4) == (DefaultYUVFileName+1)) {
-          fileName = dir.GetEntryName();
+        PFilePath dirFile = dir + dir.GetEntryName();
+        if (dirFile.GetType() == filePath.GetType()) {
+          filePath = dirFile;
+          noFilesOfType = false;
           break;
         }
       } while (dir.Next());
     }
-    if (fileName.IsEmpty()) {
-      PTRACE(1, "VidFileDev\tCannot find any file using " << dir << DefaultYUVFileName << " as video input device");
+    if (noFilesOfType) {
+      PTRACE(1, "VidFileDev\tCannot find any file using " << PDirectory()  << " as source for video input device");
       return false;
     }
   }
 
-  m_file = PFactory<PVideoFile>::CreateInstance("yuv");
-  if (m_file == NULL || !m_file->Open(fileName, PFile::ReadOnly, PFile::MustExist)) {
-    PTRACE(1, "VidFileDev\tCannot open file " << fileName << " as video input device");
+  m_file = PVideoFileFactory::CreateInstance(filePath.GetType());
+  if (m_file == NULL || !m_file->Open(filePath, PFile::ReadOnly, PFile::MustExist)) {
+    PTRACE(1, "VidFileDev\tCannot open file \"" << filePath << "\" as video input device");
     return false;
   }
 
-  *static_cast<PVideoFrameInfo *>(this) = *static_cast<PVideoFrameInfo *>(m_file);
+  PTRACE(3, "VidFileDev", "Opening file " << filePath);
+
+  *static_cast<PVideoFrameInfo *>(this) = *m_file;
 
   deviceName = m_file->GetFilePath();
   m_opened = true;
-  return true;    
+  return true;
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::IsOpen() 
+PBoolean PVideoInputDevice_VideoFile::IsOpen() 
 {
   return m_opened;
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::Close()
+PBoolean PVideoInputDevice_VideoFile::Close()
 {
   m_opened = false;
 
@@ -176,56 +158,66 @@ PBoolean PVideoInputDevice_YUVFile::Close()
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::Start()
+PBoolean PVideoInputDevice_VideoFile::Start()
 {
   return true;
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::Stop()
+PBoolean PVideoInputDevice_VideoFile::Stop()
 {
   return true;
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::IsCapturing()
+PBoolean PVideoInputDevice_VideoFile::IsCapturing()
 {
   return IsOpen();
 }
 
 
-PStringArray PVideoInputDevice_YUVFile::GetInputDeviceNames()
+PStringArray PVideoInputDevice_VideoFile::GetInputDeviceNames()
 {
+  PStringArray names;
 
-  return PString(DefaultYUVFileName);
+  PVideoFileFactory::KeyList_T keyList = PVideoFileFactory::GetKeyList();
+  for (PVideoFileFactory::KeyList_T::iterator it = keyList.begin(); it != keyList.end(); ++it)
+    names.AppendString("*" + *it);
+
+  return names;
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::SetVideoFormat(VideoFormat newFormat)
+PBoolean PVideoInputDevice_VideoFile::SetVideoFormat(VideoFormat newFormat)
 {
   return PVideoDevice::SetVideoFormat(newFormat);
 }
 
 
-int PVideoInputDevice_YUVFile::GetNumChannels() 
+int PVideoInputDevice_VideoFile::GetNumChannels() 
 {
   return ChannelCount;  
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::SetChannel(int newChannel)
+PStringArray PVideoInputDevice_VideoFile::GetChannelNames()
 {
-  return newChannel < 0 || PVideoDevice::SetChannel(newChannel);
+  PStringArray names(ChannelCount);
+  names[0] = "Once, then close";
+  names[1] = "Repeat";
+  names[2] = "Once, then still";
+  names[3] = "Once, then black";
+  return names;
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::SetColourFormat(const PString & newFormat)
+PBoolean PVideoInputDevice_VideoFile::SetColourFormat(const PString & newFormat)
 {
   return (colourFormat *= newFormat);
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::SetFrameRate(unsigned rate)
+PBoolean PVideoInputDevice_VideoFile::SetFrameRate(unsigned rate)
 {
   // Set file, if it will change, if not convert in GetFrameData
   if (m_file != NULL)
@@ -235,7 +227,7 @@ PBoolean PVideoInputDevice_YUVFile::SetFrameRate(unsigned rate)
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::GetFrameSizeLimits(unsigned & minWidth,
+PBoolean PVideoInputDevice_VideoFile::GetFrameSizeLimits(unsigned & minWidth,
                                            unsigned & minHeight,
                                            unsigned & maxWidth,
                                            unsigned & maxHeight) 
@@ -255,7 +247,7 @@ PBoolean PVideoInputDevice_YUVFile::GetFrameSizeLimits(unsigned & minWidth,
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::SetFrameSize(unsigned width, unsigned height)
+PBoolean PVideoInputDevice_VideoFile::SetFrameSize(unsigned width, unsigned height)
 {
   if (m_file == NULL) {
     PTRACE(2, "VidFileDev\tCannot set frame size, no file opened.");
@@ -266,13 +258,13 @@ PBoolean PVideoInputDevice_YUVFile::SetFrameSize(unsigned width, unsigned height
 }
 
 
-PINDEX PVideoInputDevice_YUVFile::GetMaxFrameBytes()
+PINDEX PVideoInputDevice_VideoFile::GetMaxFrameBytes()
 {
   return GetMaxFrameBytesConverted(m_file->GetFrameBytes());
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::GetFrameData(BYTE * buffer, PINDEX * bytesReturned)
+PBoolean PVideoInputDevice_VideoFile::GetFrameData(BYTE * buffer, PINDEX * bytesReturned)
 {
   m_pacing.Delay(1000/frameRate);
 
@@ -308,7 +300,7 @@ PBoolean PVideoInputDevice_YUVFile::GetFrameData(BYTE * buffer, PINDEX * bytesRe
 }
 
 
-PBoolean PVideoInputDevice_YUVFile::GetFrameDataNoDelay(BYTE * frame, PINDEX * bytesReturned)
+PBoolean PVideoInputDevice_VideoFile::GetFrameDataNoDelay(BYTE * frame, PINDEX * bytesReturned)
 {
   if (!m_opened || PAssertNULL(m_file) == NULL) {
     PTRACE(5, "VidFileDev\tAbort GetFrameDataNoDelay, closed.");
@@ -374,42 +366,37 @@ PBoolean PVideoInputDevice_YUVFile::GetFrameDataNoDelay(BYTE * frame, PINDEX * b
 
 
 ///////////////////////////////////////////////////////////////////////////////
-// PVideoOutputDevice_YUVFile
+// PVideoOutputDevice_VideoFile
 
-class PVideoOutputDevice_YUVFile_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject * CreateInstance(int /*userData*/) const
-    {
-        return new PVideoOutputDevice_YUVFile;
-    }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const
-    {
-        return PVideoOutputDevice_YUVFile::GetOutputDeviceNames();
-    }
-    virtual bool ValidateDeviceName(const PString & deviceName, int /*userData*/) const
-    {
-      return (deviceName.Right(4) *= ".yuv") && (!PFile::Exists(deviceName) || PFile::Access(deviceName, PFile::WriteOnly));
-    }
-} PVideoOutputDevice_YUVFile_descriptor;
+PCREATE_VIDOUTPUT_PLUGIN_EX(VideoFile,
 
-PCREATE_PLUGIN(YUVFile, PVideoOutputDevice, &PVideoOutputDevice_YUVFile_descriptor);
+  virtual const char * GetFriendlyName() const
+  {
+    return "Raw YUV File Video Output";
+  }
+
+  virtual bool ValidateDeviceName(const PString & deviceName, P_INT_PTR /*userData*/) const
+  {
+    return (deviceName.Right(4) *= ".yuv") && (!PFile::Exists(deviceName) || PFile::Access(deviceName, PFile::WriteOnly));
+  }
+);
 
 
-PVideoOutputDevice_YUVFile::PVideoOutputDevice_YUVFile()
+PVideoOutputDevice_VideoFile::PVideoOutputDevice_VideoFile()
   : m_file(NULL)
   , m_opened(false)
 {
 }
 
 
-PVideoOutputDevice_YUVFile::~PVideoOutputDevice_YUVFile()
+PVideoOutputDevice_VideoFile::~PVideoOutputDevice_VideoFile()
 {
   Close();
 }
 
+static const char DefaultYUVFileName[] = "*.yuv";
 
-PBoolean PVideoOutputDevice_YUVFile::Open(const PString & devName, PBoolean /*startImmediate*/)
+PBoolean PVideoOutputDevice_VideoFile::Open(const PString & devName, PBoolean /*startImmediate*/)
 {
   PFilePath fileName;
   if (devName != DefaultYUVFileName)
@@ -422,9 +409,9 @@ PBoolean PVideoOutputDevice_YUVFile::Open(const PString & devName, PBoolean /*st
     } while (PFile::Exists(fileName));
   }
 
-  m_file = PFactory<PVideoFile>::CreateInstance("yuv");
+  m_file = PVideoFileFactory::CreateInstance(fileName.GetType());
   if (m_file == NULL || !m_file->Open(fileName, PFile::WriteOnly, PFile::Create|PFile::Truncate)) {
-    PTRACE(1, "YUVFile\tCannot create file " << fileName << " as video output device");
+    PTRACE(1, "VideoFile\tCannot create file " << fileName << " as video output device");
     return false;
   }
 
@@ -433,7 +420,7 @@ PBoolean PVideoOutputDevice_YUVFile::Open(const PString & devName, PBoolean /*st
   return true;
 }
 
-PBoolean PVideoOutputDevice_YUVFile::Close()
+PBoolean PVideoOutputDevice_VideoFile::Close()
 {
   m_opened = false;
 
@@ -447,41 +434,47 @@ PBoolean PVideoOutputDevice_YUVFile::Close()
   return ok;
 }
 
-PBoolean PVideoOutputDevice_YUVFile::Start()
+PBoolean PVideoOutputDevice_VideoFile::Start()
 {
   return m_file != NULL && m_file->SetFrameSize(frameHeight, frameWidth);
 }
 
-PBoolean PVideoOutputDevice_YUVFile::Stop()
+PBoolean PVideoOutputDevice_VideoFile::Stop()
 {
   return true;
 }
 
-PBoolean PVideoOutputDevice_YUVFile::IsOpen()
+PBoolean PVideoOutputDevice_VideoFile::IsOpen()
 {
   return m_opened;
 }
 
 
-PStringArray PVideoOutputDevice_YUVFile::GetOutputDeviceNames()
+PStringArray PVideoOutputDevice_VideoFile::GetOutputDeviceNames()
 {
-  return PString(DefaultYUVFileName);
+  PStringArray names;
+
+  PVideoFileFactory::KeyList_T keyList = PVideoFileFactory::GetKeyList();
+  for (PVideoFileFactory::KeyList_T::iterator it = keyList.begin(); it != keyList.end(); ++it)
+    names.AppendString("*" + *it);
+
+  return names;
 }
 
 
-PBoolean PVideoOutputDevice_YUVFile::SetColourFormat(const PString & newFormat)
+PBoolean PVideoOutputDevice_VideoFile::SetColourFormat(const PString & newFormat)
 {
   return (newFormat *= "YUV420P") && PVideoDevice::SetColourFormat(newFormat);
 }
 
 
-PINDEX PVideoOutputDevice_YUVFile::GetMaxFrameBytes()
+PINDEX PVideoOutputDevice_VideoFile::GetMaxFrameBytes()
 {
   return GetMaxFrameBytesConverted(CalculateFrameBytes(frameWidth, frameHeight, colourFormat));
 }
 
 
-PBoolean PVideoOutputDevice_YUVFile::SetFrameData(unsigned x, unsigned y,
+PBoolean PVideoOutputDevice_VideoFile::SetFrameData(unsigned x, unsigned y,
                                               unsigned width, unsigned height,
                                               const BYTE * data,
                                               PBoolean /*endFrame*/)
@@ -492,7 +485,7 @@ PBoolean PVideoOutputDevice_YUVFile::SetFrameData(unsigned x, unsigned y,
   }
 
   if (x != 0 || y != 0 || width != frameWidth || height != frameHeight) {
-    PTRACE(1, "YUVFile\tOutput device only supports full frame writes");
+    PTRACE(1, "VideoFile\tOutput device only supports full frame writes");
     return false;
   }
 
