@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 28497 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2012-10-05 13:38:38 +0100 (Fri, 05 Oct 2012) $
  */
 
 // This depends on the expat XML library by Jim Clark
@@ -145,7 +145,7 @@ void PXMLRPCServerResource::OnXMLRPCRequest(const PString & body, PString & repl
     reply = FormatFault(PXMLRPC::MethodNameIsEmpty, "methodName is empty");
     return;
   }
-  PString method = ((PXMLData *)methodName->GetElement(0))->GetString();
+  PString method = methodName->GetData();
 
   // extract params
   PTRACE(3, "XMLRPC\tReceived XMLRPC request for method " << method);
