@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 24312 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2010-05-03 01:42:41 +0100 (Mon, 03 May 2010) $
  */
 
 #ifdef __GNUC__
@@ -74,40 +74,31 @@ void PTelnetSocket::Construct()
   SetTheirOption(StatusOption);
   SetTheirOption(TimingMark);
   SetTheirOption(EchoOption);
-
-#ifdef _DEBUG
-  debug = PTrue;
-#else
-  debug = PFalse;
-#endif
 }
 
 
-#define PTelnetError if (debug) PError << "PTelnetSocket: "
-#define PDebugError if (debug) PError
-
 PBoolean PTelnetSocket::Connect(const PString & host)
 {
-  PTelnetError << "Connect" << endl;
+  PTRACE(3, "Telnet\tConnecting to " << host);
 
   if (!PTCPSocket::Connect(host))
-    return PFalse;
+    return false;
 
   SendDo(SuppressGoAhead);
   SendDo(StatusOption);
   SendWill(TerminalSpeed);
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PTelnetSocket::Accept(PSocket & sock)
 {
   if (!PTCPSocket::Accept(sock))
-    return PFalse;
+    return false;
 
   SendDo(SuppressGoAhead);
   SendWill(StatusOption);
-  return PTrue;
+  return true;
 }
 
 
@@ -122,12 +113,12 @@ PBoolean PTelnetSocket::Write(void const * buffer, PINDEX length)
             !(length > 1 && next[1] == '\n') && !IsOurOption(TransmitBinary)) {
       // send the characters
       if (!PTCPSocket::Write(base, (next - base) + 1))
-        return PFalse;
+        return false;
       count += lastWriteCount;
 
       char null = '\0';
       if (!PTCPSocket::Write(&null, 1))
-        return PFalse;
+        return false;
       count += lastWriteCount;
 
       base = next+1;
@@ -136,7 +127,7 @@ PBoolean PTelnetSocket::Write(void const * buffer, PINDEX length)
     if (*next == IAC) {
       // send the characters
       if (!PTCPSocket::Write(base, (next - base) + 1))
-        return PFalse;
+        return false;
       count += lastWriteCount;
       base = next;
     }
@@ -147,12 +138,18 @@ PBoolean PTelnetSocket::Write(void const * buffer, PINDEX length)
 
   if (next > base) {
     if (!PTCPSocket::Write(base, next - base))
-      return PFalse;
+      return false;
     count += lastWriteCount;
   }
 
   lastWriteCount = count;
-  return PTrue;
+  return true;
+}
+
+
+bool PTelnetSocket::SetLocalEcho(bool localEcho)
+{
+  return localEcho ? SendWont(EchoOption) : SendWill(EchoOption);
 }
 
 
@@ -178,19 +175,19 @@ PBoolean PTelnetSocket::SendCommand(Command cmd, int opt)
       if (opt) {
         // Send the command
         if (!PTCPSocket::Write(buffer, 2))
-          return PFalse;
+          return false;
         // Send a TimingMark for output flush.
         buffer[1] = TimingMark;
         if (!PTCPSocket::Write(buffer, 2))
-          return PFalse;
+          return false;
         // Send a DataMark for synchronisation.
         if (cmd != AbortOutput) {
           buffer[1] = DataMark;
           if (!PTCPSocket::Write(buffer, 2))
-            return PFalse;
+            return false;
           // Send the datamark character as the only out of band data byte.
           if (!WriteOutOfBand(&buffer[1], 1))
-            return PFalse;
+            return false;
         }
         // Then flush any waiting input data.
         PTimeInterval oldTimeout = readTimeout;
@@ -205,259 +202,256 @@ PBoolean PTelnetSocket::SendCommand(Command cmd, int opt)
       return PTCPSocket::Write(buffer, 2);
   }
 
-  return PTrue;
+  return true;
 }
 
 
-static PString GetTELNETOptionName(PINDEX code)
-{
-  static const char * const name[] = {
-    "TransmitBinary",
-    "EchoOption",
-    "ReconnectOption",
-    "SuppressGoAhead",
-    "MessageSizeOption",
-    "StatusOption",
-    "TimingMark",
-    "RCTEOption",
-    "OutputLineWidth",
-    "OutputPageSize",
-    "CRDisposition",
-    "HorizontalTabsStops",
-    "HorizTabDisposition",
-    "FormFeedDisposition",
-    "VerticalTabStops",
-    "VertTabDisposition",
-    "LineFeedDisposition",
-    "ExtendedASCII",
-    "ForceLogout",
-    "ByteMacroOption",
-    "DataEntryTerminal",
-    "SupDupProtocol",
-    "SupDupOutput",
-    "SendLocation",
-    "TerminalType",
-    "EndOfRecordOption",
-    "TACACSUID",
-    "OutputMark",
-    "TerminalLocation",
-    "Use3270RegimeOption",
-    "UseX3PADOption",
-    "WindowSize",
-    "TerminalSpeed",
-    "FlowControl",
-    "LineMode",
-    "XDisplayLocation",
-    "EnvironmentOption",
-    "AuthenticateOption",
-    "EncriptionOption"
+#if PTRACING
+  static PString GetTELNETOptionName(PINDEX code)
+  {
+    static const char * const name[] = {
+      "TransmitBinary",
+      "EchoOption",
+      "ReconnectOption",
+      "SuppressGoAhead",
+      "MessageSizeOption",
+      "StatusOption",
+      "TimingMark",
+      "RCTEOption",
+      "OutputLineWidth",
+      "OutputPageSize",
+      "CRDisposition",
+      "HorizontalTabsStops",
+      "HorizTabDisposition",
+      "FormFeedDisposition",
+      "VerticalTabStops",
+      "VertTabDisposition",
+      "LineFeedDisposition",
+      "ExtendedASCII",
+      "ForceLogout",
+      "ByteMacroOption",
+      "DataEntryTerminal",
+      "SupDupProtocol",
+      "SupDupOutput",
+      "SendLocation",
+      "TerminalType",
+      "EndOfRecordOption",
+      "TACACSUID",
+      "OutputMark",
+      "TerminalLocation",
+      "Use3270RegimeOption",
+      "UseX3PADOption",
+      "WindowSize",
+      "TerminalSpeed",
+      "FlowControl",
+      "LineMode",
+      "XDisplayLocation",
+      "EnvironmentOption",
+      "AuthenticateOption",
+      "EncriptionOption"
+    };
+
+    if (code < PARRAYSIZE(name))
+      return name[code];
+    if (code == PTelnetSocket::ExtendedOptionsList)
+      return "ExtendedOptionsList";
+    return PString(PString::Printf, "Option #%u", code);
+  }
+
+  struct PTelnetTrace
+  {
+    ostream & m_strm;
+    PTelnetTrace(const char * file, int line) : m_strm(PTrace::Begin(3, file, line)) { }
+    ~PTelnetTrace() { m_strm << PTrace::End; }
   };
-
-  if (code < PARRAYSIZE(name))
-    return name[code];
-  if (code == PTelnetSocket::ExtendedOptionsList)
-    return "ExtendedOptionsList";
-  return PString(PString::Printf, "Option #%u", code);
-}
-
-
-PBoolean PTelnetSocket::StartSend(const char * which, BYTE code)
-{
-  PTelnetError << which << ' ' << GetTELNETOptionName(code) << ' ';
-  if (IsOpen())
-    return PTrue;
-
-  PDebugError << "not open yet." << endl;
-  return SetErrorValues(NotOpen, EBADF);
-}
-
+  #define SEND_OP_START(which, code) \
+     PTelnetTrace traceOutput(__FILE__, __LINE__); \
+     traceOutput.m_strm << which << ' ' << GetTELNETOptionName(code) << ' '; \
+     if (IsOpen()) ; else { traceOutput.m_strm << "not open yet."; return SetErrorValues(NotOpen, EBADF); }
+  #define ON_OP_START(which, code) \
+     PTelnetTrace traceOutput(__FILE__, __LINE__); \
+     traceOutput.m_strm << which << ' ' << GetTELNETOptionName(code) << ' '
+  #define TELNET_TRACE(info) traceOutput.m_strm << info
+#else
+  #define SEND_OP_START(which, code) if ((IsOpen() || SetErrorValues(NotOpen, EBADF))) return false
+  #define ON_OP_START(which, code)
+  #define TELNET_TRACE(info)
+#endif
 
 PBoolean PTelnetSocket::SendDo(BYTE code)
 {
-  if (!StartSend("SendDo", code))
-    return PFalse;
+  SEND_OP_START("SendDo", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.theirState) {
     case OptionInfo::IsNo :
-      PDebugError << "initiated.";
+      TELNET_TRACE("initiated.");
       SendCommand(DO, code);
       opt.theirState = OptionInfo::WantYes;
       break;
 
     case OptionInfo::IsYes :
-      PDebugError << "already enabled." << endl;
-      return PFalse;
+      TELNET_TRACE("already enabled.");
+      return false;
 
     case OptionInfo::WantNo :
-      PDebugError << "queued.";
+      TELNET_TRACE("queued.");
       opt.theirState = OptionInfo::WantNoQueued;
       break;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "already queued." << endl;
+      TELNET_TRACE("already queued.");
       opt.theirState = OptionInfo::IsNo;
-      return PFalse;
+      return false;
 
     case OptionInfo::WantYes :
-      PDebugError << "already negotiating." << endl;
+      TELNET_TRACE("already negotiating.");
       opt.theirState = OptionInfo::IsNo;
-      return PFalse;
+      return false;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "dequeued.";
+      TELNET_TRACE("dequeued.");
       opt.theirState = OptionInfo::WantYes;
       break;
   }
 
-  PDebugError << endl;
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PTelnetSocket::SendDont(BYTE code)
 {
-  if (!StartSend("SendDont", code))
-    return PFalse;
+  SEND_OP_START("SendDont", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.theirState) {
     case OptionInfo::IsNo :
-      PDebugError << "already disabled." << endl;
-      return PFalse;
+      TELNET_TRACE("already disabled.");
+      return false;
 
     case OptionInfo::IsYes :
-      PDebugError << "initiated.";
+      TELNET_TRACE("initiated.");
       SendCommand(DONT, code);
       opt.theirState = OptionInfo::WantNo;
       break;
 
     case OptionInfo::WantNo :
-      PDebugError << "already negotiating." << endl;
+      TELNET_TRACE("already negotiating.");
       opt.theirState = OptionInfo::IsNo;
-      return PFalse;
+      return false;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "dequeued.";
+      TELNET_TRACE("dequeued.");
       opt.theirState = OptionInfo::WantNo;
       break;
 
     case OptionInfo::WantYes :
-      PDebugError << "queued.";
+      TELNET_TRACE("queued.");
       opt.theirState = OptionInfo::WantYesQueued;
       break;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "already queued." << endl;
+      TELNET_TRACE("already queued.");
       opt.theirState = OptionInfo::IsYes;
-      return PFalse;
+      return false;
   }
 
-  PDebugError << endl;
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PTelnetSocket::SendWill(BYTE code)
 {
-  if (!StartSend("SendWill", code))
-    return PFalse;
-
-  if (!IsOpen())
-    return PFalse;
+  SEND_OP_START("SendWill", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.ourState) {
     case OptionInfo::IsNo :
-      PDebugError << "initiated.";
+      TELNET_TRACE("initiated.");
       SendCommand(WILL, code);
       opt.ourState = OptionInfo::WantYes;
       break;
 
     case OptionInfo::IsYes :
-      PDebugError << "already enabled." << endl;
-      return PFalse;
+      TELNET_TRACE("already enabled.");
+      return false;
 
     case OptionInfo::WantNo :
-      PDebugError << "queued.";
+      TELNET_TRACE("queued.");
       opt.ourState = OptionInfo::WantNoQueued;
       break;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "already queued." << endl;
+      TELNET_TRACE("already queued.");
       opt.ourState = OptionInfo::IsNo;
-      return PFalse;
+      return false;
 
     case OptionInfo::WantYes :
-      PDebugError << "already negotiating." << endl;
+      TELNET_TRACE("already negotiating.");
       opt.ourState = OptionInfo::IsNo;
-      return PFalse;
+      return false;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "dequeued.";
+      TELNET_TRACE("dequeued.");
       opt.ourState = OptionInfo::WantYes;
       break;
   }
 
-  PDebugError << endl;
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PTelnetSocket::SendWont(BYTE code)
 {
-  if (!StartSend("SendWont", code))
-    return PFalse;
+  SEND_OP_START("SendWont", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.ourState) {
     case OptionInfo::IsNo :
-      PDebugError << "already disabled." << endl;
-      return PFalse;
+      TELNET_TRACE("already disabled.");
+      return false;
 
     case OptionInfo::IsYes :
-      PDebugError << "initiated.";
+      TELNET_TRACE("initiated.");
       SendCommand(WONT, code);
       opt.ourState = OptionInfo::WantNo;
       break;
 
     case OptionInfo::WantNo :
-      PDebugError << "already negotiating." << endl;
+      TELNET_TRACE("already negotiating.");
       opt.ourState = OptionInfo::IsNo;
-      return PFalse;
+      return false;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "dequeued.";
+      TELNET_TRACE("dequeued.");
       opt.ourState = OptionInfo::WantNo;
       break;
 
     case OptionInfo::WantYes :
-      PDebugError << "queued.";
+      TELNET_TRACE("queued.");
       opt.ourState = OptionInfo::WantYesQueued;
       break;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "already queued." << endl;
+      TELNET_TRACE("already queued.");
       opt.ourState = OptionInfo::IsYes;
-      return PFalse;
+      return false;
   }
 
-  PDebugError << endl;
-  return PTrue;
+  return true;
 }
 
 
-PBoolean PTelnetSocket::SendSubOption(BYTE code,
-                                    const BYTE * info, PINDEX len, int subCode)
+PBoolean PTelnetSocket::SendSubOption(BYTE code, const BYTE * info, PINDEX len, int subCode)
 {
-  if (!StartSend("SendSubOption", code))
-    return PFalse;
-
-  PDebugError << "with " << len << " bytes." << endl;
+  {
+    SEND_OP_START("SendSubOption", code);
+    TELNET_TRACE("with " << len << " bytes.");
+  }
 
   PBYTEArray buffer(len + 6);
   buffer[0] = IAC;
@@ -494,7 +488,7 @@ void PTelnetSocket::SetWindowSize(WORD width, WORD height)
     buffer[1] = (BYTE)width;
     buffer[2] = (BYTE)(height >> 8);
     buffer[3] = (BYTE)height;
-    SendSubOption(WindowSize, buffer, 4);
+    SendSubOption(WindowSize, buffer, sizeof(buffer));
   }
   else {
     SetOurOption(WindowSize);
@@ -572,7 +566,7 @@ PBoolean PTelnetSocket::Read(void * data, PINDEX bytesToRead)
               /* We may have missed an urgent notification, so make sure we
                  flush whatever is in the buffer currently.
                */
-              PTelnetError << "received DataMark" << endl;
+              PTRACE(3, "Telnet\tReceived DataMark");
               if (synchronising > 0)
                 synchronising--;
               break;
@@ -642,7 +636,7 @@ PBoolean PTelnetSocket::Read(void * data, PINDEX bytesToRead)
           break;
 
         default :
-          PTelnetError << "illegal state: " << (int)state << endl;
+          PTRACE(2, "Telnet\tIllegal state: " << (int)state);
           state = StateNormal;
       }
       if (synchronising > 0) {
@@ -652,68 +646,67 @@ PBoolean PTelnetSocket::Read(void * data, PINDEX bytesToRead)
     }
   }
   lastReadCount = bytesToRead;
-  return PTrue;
+  return true;
 }
 
 
 void PTelnetSocket::OnDo(BYTE code)
 {
-  PTelnetError << "OnDo " << GetTELNETOptionName(code) << ' ';
+  {
+    ON_OP_START("OnDo", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.ourState) {
     case OptionInfo::IsNo :
       if (opt.weCan) {
-        PDebugError << "WILL.";
+          TELNET_TRACE("WILL.");
         SendCommand(WILL, code);
         opt.ourState = OptionInfo::IsYes;
       }
       else {
-        PDebugError << "WONT.";
+          TELNET_TRACE("WONT.");
         SendCommand(WONT, code);
       }
       break;
 
     case OptionInfo::IsYes :
-      PDebugError << "ignored.";
+        TELNET_TRACE("ignored.");
       break;
 
     case OptionInfo::WantNo :
-      PDebugError << "is answer to WONT.";
+        TELNET_TRACE("is answer to WONT.");
       opt.ourState = OptionInfo::IsNo;
       break;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "impossible answer.";
+        TELNET_TRACE("impossible answer.");
       opt.ourState = OptionInfo::IsYes;
       break;
 
     case OptionInfo::WantYes :
-      PDebugError << "accepted.";
+        TELNET_TRACE("accepted.");
       opt.ourState = OptionInfo::IsYes;
       break;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "refused.";
+        TELNET_TRACE("refused.");
       opt.ourState = OptionInfo::WantNo;
       SendCommand(WONT, code);
       break;
   }
-
-  PDebugError << endl;
+  }
 
   if (IsOurOption(code)) {
     switch (code) {
       case TerminalSpeed : {
           static BYTE defSpeed[] = "38400,38400";
-          SendSubOption(TerminalSpeed,defSpeed,sizeof(defSpeed)-1,SubOptionIs);
+          SendSubOption(TerminalSpeed, defSpeed, sizeof(defSpeed)-1, SubOptionIs);
         }
         break;
 
       case TerminalType :
-        SendSubOption(TerminalType,
-                          terminalType, terminalType.GetLength(), SubOptionIs);
+        SendSubOption(TerminalType, terminalType, terminalType.GetLength(), SubOptionIs);
         break;
 
       case WindowSize :
@@ -726,156 +719,155 @@ void PTelnetSocket::OnDo(BYTE code)
 
 void PTelnetSocket::OnDont(BYTE code)
 {
-  PTelnetError << "OnDont " << GetTELNETOptionName(code) << ' ';
+  ON_OP_START("OnDont", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.ourState) {
     case OptionInfo::IsNo :
-      PDebugError << "ignored.";
+      TELNET_TRACE("ignored.");
       break;
 
     case OptionInfo::IsYes :
-      PDebugError << "WONT.";
+      TELNET_TRACE("WONT.");
       opt.ourState = OptionInfo::IsNo;
       SendCommand(WONT, code);
       break;
 
     case OptionInfo::WantNo :
-      PDebugError << "disabled.";
+      TELNET_TRACE("disabled.");
       opt.ourState = OptionInfo::IsNo;
       break;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "accepting.";
+      TELNET_TRACE("accepting.");
       opt.ourState = OptionInfo::WantYes;
       SendCommand(DO, code);
       break;
 
     case OptionInfo::WantYes :
-      PDebugError << "queued disable.";
+      TELNET_TRACE("queued disable.");
       opt.ourState = OptionInfo::IsNo;
       break;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "refused.";
+      TELNET_TRACE("refused.");
       opt.ourState = OptionInfo::IsNo;
       break;
   }
-
-  PDebugError << endl;
 }
 
 
 void PTelnetSocket::OnWill(BYTE code)
 {
-  PTelnetError << "OnWill " << GetTELNETOptionName(code) << ' ';
+  ON_OP_START("OnWill", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.theirState) {
     case OptionInfo::IsNo :
       if (opt.theyShould) {
-        PDebugError << "DO.";
+        TELNET_TRACE("DO.");
         SendCommand(DO, code);
         opt.theirState = OptionInfo::IsYes;
       }
       else {
-        PDebugError << "DONT.";
+        TELNET_TRACE("DONT.");
         SendCommand(DONT, code);
       }
       break;
 
     case OptionInfo::IsYes :
-      PDebugError << "ignored.";
+      TELNET_TRACE("ignored.");
       break;
 
     case OptionInfo::WantNo :
-      PDebugError << "is answer to DONT.";
+      TELNET_TRACE("is answer to DONT.");
       opt.theirState = OptionInfo::IsNo;
       break;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "impossible answer.";
+      TELNET_TRACE("impossible answer.");
       opt.theirState = OptionInfo::IsYes;
       break;
 
     case OptionInfo::WantYes :
-      PDebugError << "accepted.";
+      TELNET_TRACE("accepted.");
       opt.theirState = OptionInfo::IsYes;
       break;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "refused.";
+      TELNET_TRACE("refused.");
       opt.theirState = OptionInfo::WantNo;
       SendCommand(DONT, code);
       break;
   }
-
-  PDebugError << endl;
 }
 
 
 void PTelnetSocket::OnWont(BYTE code)
 {
-  PTelnetError << "OnWont " << GetTELNETOptionName(code) << ' ';
+  ON_OP_START("OnWont", code);
 
   OptionInfo & opt = option[code];
 
   switch (opt.theirState) {
     case OptionInfo::IsNo :
-      PDebugError << "ignored.";
+      TELNET_TRACE("ignored.");
       break;
 
     case OptionInfo::IsYes :
-      PDebugError << "DONT.";
+      TELNET_TRACE("DONT.");
       opt.theirState = OptionInfo::IsNo;
       SendCommand(DONT, code);
       break;
 
     case OptionInfo::WantNo :
-      PDebugError << "disabled.";
+      TELNET_TRACE("disabled.");
       opt.theirState = OptionInfo::IsNo;
       break;
 
     case OptionInfo::WantNoQueued :
-      PDebugError << "accepting.";
+      TELNET_TRACE("accepting.");
       opt.theirState = OptionInfo::WantYes;
       SendCommand(DO, code);
       break;
 
     case OptionInfo::WantYes :
-      PDebugError << "refused.";
+      TELNET_TRACE("refused.");
       opt.theirState = OptionInfo::IsNo;
       break;
 
     case OptionInfo::WantYesQueued :
-      PDebugError << "queued refusal.";
+      TELNET_TRACE("queued refusal.");
       opt.theirState = OptionInfo::IsNo;
       break;
   }
-
-  PDebugError << endl;
 }
 
 
-void PTelnetSocket::OnSubOption(BYTE code, const BYTE * info, PINDEX len)
+void PTelnetSocket::OnSubOption(BYTE code, const BYTE * info, PINDEX PTRACE_PARAM(len))
 {
-  PTelnetError << "OnSubOption " << GetTELNETOptionName(code)
-               << " of " << len << " bytes." << endl;
+  ON_OP_START("OnSubOption", code);
+
   switch (code) {
     case TerminalType :
-      if (*info == SubOptionSend)
-        SendSubOption(TerminalType,
-                          terminalType, terminalType.GetLength(), SubOptionIs);
+      if (*info == SubOptionSend) {
+        TELNET_TRACE("TerminalType");
+        SendSubOption(TerminalType, terminalType, terminalType.GetLength(), SubOptionIs);
+      }
       break;
 
     case TerminalSpeed :
       if (*info == SubOptionSend) {
+        TELNET_TRACE("TerminalSpeed");
         static BYTE defSpeed[] = "38400,38400";
-        SendSubOption(TerminalSpeed,defSpeed,sizeof(defSpeed)-1,SubOptionIs);
+        SendSubOption(TerminalSpeed, defSpeed, sizeof(defSpeed)-1, SubOptionIs);
       }
       break;
+
+    default :
+      TELNET_TRACE(" of " << len << " bytes.");
   }
 }
 
@@ -883,14 +875,15 @@ void PTelnetSocket::OnSubOption(BYTE code, const BYTE * info, PINDEX len)
 PBoolean PTelnetSocket::OnCommand(BYTE code)
 {
   if (code == NOP)
-    return PTrue;
-  PTelnetError << "unknown command " << (int)code << endl;
-  return PTrue;
+    return true;
+  PTRACE(2, "Telnet\tunknown command " << (int)code);
+  return true;
 }
 
-void PTelnetSocket::OnOutOfBand(const void *, PINDEX length)
+
+void PTelnetSocket::OnOutOfBand(const void *, PINDEX PTRACE_PARAM(length))
 {
-  PTelnetError << "out of band data received of length " << length << endl;
+  PTRACE(3, "Telnet\tout of band data received of length " << length);
   synchronising++;
 }
 
