@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28090 $
+ * $Revision: 34426 $
  * $Author: rjongbloed $
- * $Date: 2012-07-22 23:27:41 -0500 (Sun, 22 Jul 2012) $
+ * $Date: 2015-12-13 16:37:41 +0000 (Sun, 13 Dec 2015) $
  */
 
 #ifdef __GNUC__
@@ -48,26 +48,34 @@ PThreadPoolBase::PThreadPoolBase(unsigned int maxWorkerCount,
                                  PThread::Priority priority)
   : m_maxWorkerCount(maxWorkerCount)
   , m_maxWorkUnitCount(maxWorkUnitCount)
+#if PTRACING
   , m_highWaterMark(0)
+#endif
   , m_threadName(threadName != NULL ? threadName : "Pool")
   , m_priority(priority)
 {
 }
 
-PThreadPoolBase::~PThreadPoolBase()
+
+void PThreadPoolBase::Shutdown()
 {
+  PTRACE(3, "PTLib", "Shutting down thread pool \"" << m_threadName << '"');
+
   while (!m_workers.empty()) {
-    m_listMutex.Wait();
+    m_mutex.Wait();
     WorkerThreadBase * worker = m_workers.front();
     m_workers.erase(m_workers.begin());
-    m_listMutex.Signal();
+    m_mutex.Signal();
 
     StopWorker(worker);
   }
 }
 
+
 PThreadPoolBase::WorkerThreadBase * PThreadPoolBase::AllocateWorker()
 {
+  // Assumes m_mutex already locked
+
   // find the worker thread with the minimum number of work units
   // shortcut the search if we find an empty one
   WorkerList_t::iterator minWorker = m_workers.end();
@@ -97,73 +105,64 @@ PThreadPoolBase::WorkerThreadBase * PThreadPoolBase::AllocateWorker()
   else if ((m_workers.size() > 0) && (m_workers.size() == m_maxWorkerCount))
     return *minWorker;
 
-  return NewWorker();
-}
-
-PThreadPoolBase::WorkerThreadBase * PThreadPoolBase::NewWorker()
-{
   // create a new worker thread
   WorkerThreadBase * worker = CreateWorkerThread();
 
-  m_listMutex.Wait();
-  m_workers.push_back(worker);
+  m_workers.push_back(PAssertNULL(worker));
 
+#if PTRACING
   if (m_workers.size() > m_highWaterMark) {
     m_highWaterMark = m_workers.size();
-    PTRACE(4, "PTLib\tThread pool high water mark: " << m_highWaterMark);
+    PTRACE(2, "PTLib", "Created new pool thread \"" << worker->GetThreadName() << "\", high water mark=" << m_highWaterMark);
   }
-
-  m_listMutex.Signal();
+  else
+    PTRACE(4, "PTLib", "Created new pool thread \"" << worker->GetThreadName() << '"');
+#endif
 
   worker->Resume();
   return worker;
 }
 
 
-bool PThreadPoolBase::CheckWorker(WorkerThreadBase * worker)
+void PThreadPoolBase::ReclaimWorkers()
 {
-  {
-    PWaitAndSignal mutex(m_listMutex);
+  WorkerThreadBase * stoppedWorker;
+  do {
+    stoppedWorker = NULL;
 
-    // find worker in list
-    WorkerList_t::iterator iter;
-    for (iter = m_workers.begin(); iter != m_workers.end(); ++iter) {
-      if (*iter == worker)
-        break;
-    }
-    PAssert(iter != m_workers.end(), "cannot find thread pool worker");
-
-    // if the worker thread has work, leave it alone
-    if (worker->GetWorkSize() > 0) 
-      return true;
+    m_mutex.Wait();
 
     // don't shut down the last thread, so we don't have the overhead of starting it up again
-    // don't try and kill ourselves - just leave the thread for someone else to use
-    if ((m_workers.size() == 1) || (worker == PThread::Current()))
-      return true;
+    if (m_workers.size() > 1) {
+      for (WorkerList_t::iterator iter = m_workers.begin(); iter != m_workers.end(); ++iter) {
+        WorkerThreadBase * worker = *iter;
 
-    // remove the thread from the list or workers
-    m_workers.erase(iter);
+        // if the worker thread has work, leave it alone
+        // don't try and kill ourselves - just leave the thread for someone else to use
+        if (worker->GetWorkSize() == 0 && worker != PThread::Current()) {
+          stoppedWorker = worker;
+          m_workers.erase(iter);
+          break;
+        }
+      }
+    }
 
-    // shutdown the thread
-    worker->Shutdown();
-  }
+    m_mutex.Signal();
 
-  StopWorker(worker);
+    // Outside of mutex to assure deadlock freedom
+    StopWorker(stoppedWorker);
+  } while (stoppedWorker != NULL);
 
-  return true;
 }
 
 
 void PThreadPoolBase::StopWorker(WorkerThreadBase * worker)
 {
+  if (worker == NULL)
+    return;
+
+  PTRACE(4, "PTLib", "Shutting down pool thread " << worker);
   worker->Shutdown();
-
-  // the worker is now finished
-  if (!worker->WaitForTermination(10000)) {
-    PTRACE(4, "ThreadPool\tWorker did not terminate promptly");
-  }
-
-  PTRACE(4, "ThreadPool\tDestroying pool thread");
+  PAssert(worker->WaitForTermination(10000), "Worker did not terminate promptly");
   delete worker;
 }
