@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29844 $
+ * $Revision: 34360 $
  * $Author: rjongbloed $
- * $Date: 2013-05-30 22:00:59 -0500 (Thu, 30 May 2013) $
+ * $Date: 2015-11-27 09:12:38 +0000 (Fri, 27 Nov 2015) $
  */
 
 #ifdef __GNUC__
@@ -104,6 +104,8 @@ PURL_LEGACY_SCHEME(sip,       true,  true,  true,  false,  false,  true,  true,
 PURL_LEGACY_SCHEME(sips,      true,  true,  true,  false,  false,  true,  true,  false, false, false, DEFAULT_SIPS_PORT)
 PURL_LEGACY_SCHEME(fax,       false, false, false, true,   false,  false, true,  false, false, false, 0)
 PURL_LEGACY_SCHEME(msrp,      false, false, true,  false,  false,  true,  true,  false, true,  false, DEFAULT_MSRP_PORT)
+PURL_LEGACY_SCHEME(ws,        false, false, true,  false,  true,   true,  false, false, true,  false, DEFAULT_HTTP_PORT)
+PURL_LEGACY_SCHEME(wss,       false, false, true,  false,  true,   true,  false, false, true,  false, DEFAULT_HTTPS_PORT)
 
 #define DEFAULT_SCHEME "http"
 #define FILE_SCHEME    "file"
@@ -244,13 +246,18 @@ PString PURL::TranslateString(const PString & str, TranslationType type)
 
     case QuotedParameterTranslation :
       safeChars += "[]/:@&=+$,|";
-      return str.FindSpan(safeChars) != P_MAX_INDEX ? str.ToLiteral() : str;
+      // If already starts and ends with quotes, assume it already formatted correctly.
+      if ((str.GetLength() >= 2 && str[0] == '"' && str[str.GetLength()-1] == '"') ||
+           str.FindSpan(safeChars) == P_MAX_INDEX)
+        return str;
+
+      return str.ToLiteral();
 
     default :
       break;    // Section 3.4, no reserved characters may be used
   }
-  PINDEX pos = (PINDEX)-1;
-  while ((pos = xlat.FindSpan(safeChars, pos+1)) != P_MAX_INDEX) {
+  
+  for (PINDEX pos = 0; (pos = xlat.FindSpan(safeChars, pos)) != P_MAX_INDEX; ++pos) {
     char buf[10];
     sprintf(buf, "%%%02X", (BYTE)xlat[pos]);
     xlat.Splice(buf, pos, 1);
@@ -268,13 +275,11 @@ PString PURL::UntranslateString(const PString & str, TranslationType type)
   PINDEX pos;
   if (type == PURL::QueryTranslation) {
     /* Even though RFC2396 never mentions this, RFC1630 does. */
-    pos = (PINDEX)-1;
-    while ((pos = xlat.Find('+', pos+1)) != P_MAX_INDEX)
+    for (pos = 0; (pos = xlat.Find('+', pos)) != P_MAX_INDEX; ++pos)
       xlat[pos] = ' ';
   }
 
-  pos = (PINDEX)-1;
-  while ((pos = xlat.Find('%', pos+1)) != P_MAX_INDEX) {
+  for (pos = 0; (pos = xlat.Find('%', pos)) != P_MAX_INDEX; ++pos) {
     int digit1 = xlat[pos+1];
     int digit2 = xlat[pos+2];
     if (isxdigit(digit1) && isxdigit(digit2)) {
@@ -304,10 +309,13 @@ void PURL::SplitVars(const PString & str, PStringToString & vars, char sep1, cha
     PCaselessString key, data;
 
     PINDEX sep2pos = str.Find(sep2, sep1prev);
-    if (sep2pos > sep1next)
-      key = str(sep1prev, sep1next-1);
+    if (sep2pos > sep1next) {
+      if (sep1next > 0)
+        key = str(sep1prev, sep1next-1);
+    }
     else {
-      key = str(sep1prev, sep2pos-1);
+      if (sep2pos > 0)
+        key = str(sep1prev, sep2pos-1);
       if (type != QuotedParameterTranslation)
         data = str(sep2pos+1, sep1next-1);
       else {
@@ -370,18 +378,59 @@ void PURL::OutputVars(ostream & strm,
     }
 
     PString key  = TranslateString(it->first,  type);
-    PString data = TranslateString(it->second, type);
 
-    if (key.IsEmpty())
-      strm << data;
-    else if (data.IsEmpty())
-      strm << key;
-    else
-      strm << key << sep2 << data;
+    if (type < ParameterTranslation) {
+        PString data = TranslateString(it->second, type);
+      if (key.IsEmpty())
+        strm << data;
+      else
+        strm << key << sep2 << data;
+    }
+    else {
+      PStringArray values = it->second.Lines();
+      if (values.IsEmpty())
+        strm << key;
+      else {
+        for (PINDEX i = 0; i < values.GetSize(); ++i) {
+          PString data = TranslateString(values[i], type);
+
+          if (i > 0)
+            strm << sep1;
+          if (key.IsEmpty())
+            strm << data;
+          else
+            strm << key << sep2 << data;
+        }
+      }
+    }
   }
 }
 
 
+PCaselessString PURL::ExtractScheme(const char * cstr)
+{
+  // Character set as per RFC2396
+  //    scheme        = alpha *( alpha | digit | "+" | "-" | "." )
+  if (cstr == NULL)
+    return PString::Empty();
+
+  while (((*cstr & 0x80) == 0x00) && isspace(*cstr))
+    ++cstr;
+
+  if (!isalpha(*cstr))
+    return PString::Empty();
+
+  const char * ptr = cstr+1;
+  while (isalnum(*ptr) || *ptr == '+' || *ptr == '-' || *ptr == '.')
+    ++ptr;
+
+  if (*ptr != ':')
+    return PString::Empty();
+
+  return PString(cstr, ptr-cstr);
+}
+
+
 PBoolean PURL::InternalParse(const char * cstr, const char * defaultScheme)
 {
   scheme.MakeEmpty();
@@ -397,6 +446,10 @@ PBoolean PURL::InternalParse(const char * cstr, const char * defaultScheme)
   queryVars.RemoveAll();
   m_contents.MakeEmpty();
 
+  // get information which tells us how to parse URL for this
+  // particular scheme
+  schemeInfo = NULL;
+
   if (cstr == NULL)
     return false;
 
@@ -408,25 +461,12 @@ PBoolean PURL::InternalParse(const char * cstr, const char * defaultScheme)
   if (*cstr == '\0')
     return false;
 
-  // get information which tells us how to parse URL for this
-  // particular scheme
-  schemeInfo = NULL;
-
-  // Character set as per RFC2396
-  //    scheme        = alpha *( alpha | digit | "+" | "-" | "." )
-  if (isalpha(cstr[0])) {
-    PINDEX pos = 1;
-    while (isalnum(cstr[pos]) || cstr[pos] == '+' || cstr[pos] == '-' || cstr[pos] == '.')
-      ++pos;
-
-    // Determine if the URL has an explicit scheme
-    if (cstr[pos] == ':') {
-      scheme = PString(cstr, pos);
-      // get the scheme information
-      schemeInfo = PURLSchemeFactory::CreateInstance(std::string(cstr, pos));
-      if (schemeInfo != NULL)
-        cstr += pos+1;
-    }
+  scheme = ExtractScheme(cstr);
+  if (!scheme.IsEmpty()) {
+    // get the scheme information
+    schemeInfo = PURLSchemeFactory::CreateInstance(scheme);
+    if (schemeInfo != NULL)
+      cstr = strchr(cstr, ':')+1;
   }
 
   // if we could not match a scheme, then use the specified default scheme
@@ -858,9 +898,14 @@ void PURL::SetParameters(const PString & parameters)
 }
 
 
-void PURL::SetParamVars(const PStringToString & p)
+void PURL::SetParamVars(const PStringToString & p, bool merge)
 {
-  paramVars = p;
+  if (merge)
+    paramVars.Merge(p, PStringToString::e_MergeIgnore);
+  else {
+    paramVars = p;
+    paramVars.MakeUnique();
+  }
   Recalculate();
 }
 
@@ -1051,7 +1096,7 @@ class PURL_CalltoScheme : public PURLScheme
     }
 };
 
-static PURLSchemeFactory::Worker<PURL_CalltoScheme> calltoScheme("callto", true);
+PFACTORY_CREATE(PURLSchemeFactory, PURL_CalltoScheme, "callto", true);
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1107,7 +1152,7 @@ class PURL_TelScheme : public PURLScheme
     }
 };
 
-static PURLSchemeFactory::Worker<PURL_TelScheme> telScheme("tel", true);
+PFACTORY_CREATE(PURLSchemeFactory,PURL_TelScheme, "tel", true);
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1176,7 +1221,7 @@ class PURL_DataScheme : public PURLScheme
     }
 };
 
-static PURLSchemeFactory::Worker<PURL_DataScheme> dataScheme("data", true);
+PFACTORY_CREATE(PURLSchemeFactory, PURL_DataScheme, "data", true);
 
 
 ///////////////////////////////////////////////////////////////////////////////
