@@ -23,9 +23,9 @@
  *
  * Contributor(s): Robert Jongbloed
  *
- * $Revision: 27867 $
+ * $Revision: 33214 $
  * $Author: rjongbloed $
- * $Date: 2012-06-19 19:55:11 -0500 (Tue, 19 Jun 2012) $
+ * $Date: 2015-01-08 06:44:36 +0000 (Thu, 08 Jan 2015) $
  */
 
 #ifdef __GNUC__
@@ -35,6 +35,8 @@
 #include <ptlib.h>
 #include <ptclib/vartype.h>
 
+#if P_VARTYPE
+
 #include <limits>
 
 
@@ -141,6 +143,146 @@ bool PVarType::SetType(BasicType type, PINDEX option)
 }
 
 
+PVarType & PVarType::operator = (bool value)
+{
+  if (SetType(VarBoolean)) {
+    m_.boolean = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(char value)
+{
+  if (SetType(VarChar)) {
+    m_.character = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(int16_t value)
+{
+  if (SetType(VarInt16)) {
+    m_.int16 = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(int32_t value)
+{
+  if (SetType(VarInt32)) {
+    m_.int32 = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(int64_t value)
+{
+  if (SetType(VarInt64)) {
+    m_.int64 = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(uint8_t value)
+{
+  if (SetType(VarUInt8)) {
+    m_.uint8 = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(uint16_t value)
+{
+  if (SetType(VarUInt16)) {
+    m_.uint16 = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(uint32_t value)
+{
+  if (SetType(VarUInt32)) {
+    m_.uint32 = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(uint64_t value)
+{
+  if (SetType(VarUInt64)) {
+    m_.uint64 = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(float value)
+{
+  if (SetType(VarFloatSingle)) {
+    m_.floatSingle = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(double value)
+{
+  if (SetType(VarFloatDouble)) {
+    m_.floatDouble = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(long double value)
+{
+  if (SetType(VarFloatExtended)) {
+    m_.floatExtended = value;
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(const PGloballyUniqueID & value)
+{
+  if (SetType(VarGUID)) {
+    memcpy(m_.guid, value, value.GetSize());
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
+PVarType & PVarType::operator=(const PTime & value)
+{
+  if (SetType(VarTime)) {
+    m_.time.seconds = value.GetTimeInSeconds();
+    OnValueChanged();
+  }
+  return *this;
+}
+
+
 PVarType & PVarType::SetValue(const PString & value)
 {
   switch (m_type) {
@@ -687,53 +829,55 @@ PString PVarType::AsString() const
 {
   const_cast<PVarType *>(this)->OnGetValue();
 
+  PStringStream strm;
+
   switch (m_type) {
     case VarNULL :
       break;
     case VarBoolean :
-      return PString(m_.boolean ? "true" : "false");
+      strm << (m_.boolean ? "true" : "false"); break;
     case VarChar :
-      return PString(m_.character);
+      strm << m_.character; break;
     case VarInt8 :
-      return PString((int)m_.int8);
+      strm << m_.int8; break;
     case VarInt16 :
-      return PString(m_.int16);
+      strm << m_.int16; break;
     case VarInt32 :
-      return PString(m_.int32);
+      strm << m_.int32; break;
     case VarInt64 :
-      return PString(m_.int64);
+      strm << m_.int64; break;
     case VarUInt8 :
-      return PString((unsigned)m_.uint8);
+      strm << m_.uint8; break;
     case VarUInt16 :
-      return PString(m_.uint16);
+      strm << m_.uint16; break;
     case VarUInt32 :
-      return PString(m_.uint32);
+      strm << m_.uint32; break;
     case VarUInt64 :
-      return PString(m_.uint64);
+      strm << m_.uint64; break;
     case VarFloatSingle :
-      { PStringStream strm; strm << m_.floatSingle; return strm; }
+      strm << m_.floatSingle; break;
     case VarFloatDouble :
-      { PStringStream strm; strm << m_.floatDouble; return strm; }
+      strm << m_.floatDouble; break;
     case VarFloatExtended :
-      { PStringStream strm; strm << m_.floatExtended; return strm; }
+      strm << m_.floatExtended; break;
     case VarTime :
-      return PTime(m_.time.seconds).AsString(m_.time.format);
+      strm << PTime(m_.time.seconds).AsString(m_.time.format); break;
     case VarGUID :
-      return PGloballyUniqueID(m_.guid, sizeof(m_.guid)).AsString();
+      strm << PGloballyUniqueID(m_.guid, sizeof(m_.guid)); break;
     case VarStaticString :
-      return m_.staticString;
+      strm << m_.staticString; break;
     case VarFixedString :
     case VarDynamicString :
-      return PString(m_.dynamic.data);
+      strm << m_.dynamic.data; break;
     case VarStaticBinary :
-      return PString(m_.staticBinary.data, m_.staticBinary.size);
+      strm << PString(m_.staticBinary.data, m_.staticBinary.size); break;
     case VarDynamicBinary :
-      return PString(m_.dynamic.data, m_.dynamic.size);
+      strm << PString(m_.dynamic.data, m_.dynamic.size); break;
     default :
       PAssertAlways("Invalid PVarType");
   }
 
-  return PString::Empty();
+  return strm;
 }
 
 
@@ -835,3 +979,5 @@ void PVarType::OnGetValue()
 void PVarType::OnValueChanged()
 {
 }
+
+#endif // P_VARTYPE
