@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28372 $
- * $Author: rjongbloed $
- * $Date: 2012-09-17 22:58:15 -0500 (Mon, 17 Sep 2012) $
+ * $Revision: 32504 $
+ * $Author: ededu $
+ * $Date: 2014-08-12 09:02:59 +0100 (Tue, 12 Aug 2014) $
  */
 
 #ifdef __GNUC__
@@ -33,44 +33,39 @@
 #endif
 
 #include <ptlib.h>
-#include <ptlib/vconvert.h>
-#include <ptlib/pluginmgr.h>
-#include <ptclib/vsdl.h>
-
-#define new PNEW
 
 #if P_SDL
 
+#define P_FORCE_STATIC_PLUGIN 1
+
+#include <ptclib/vsdl.h>
+#include <ptlib/vconvert.h>
+
+#define new PNEW
+#define PTraceModule() "SDL"
+
+
 extern "C" {
   #include <SDL.h>
 };
 
 #ifdef _MSC_VER
   #pragma comment(lib, P_SDL_LIBRARY)
-  #pragma message("SDL video support enabled")
 #endif
 
 
-class PVideoOutputDevice_SDL_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject *    CreateInstance(int /*userData*/) const
-    {
-      return new PVideoOutputDevice_SDL;
-    }
+PCREATE_VIDOUTPUT_PLUGIN_EX(SDL,
 
-    virtual PStringArray GetDeviceNames(int /*userData*/) const
-    {
-      return PString("SDL");
-    }
+  virtual const char * GetFriendlyName() const
+  {
+    return "Simple DirectMedia Layer Video Output";
+  }
 
-    virtual bool         ValidateDeviceName(const PString & deviceName, int /*userData*/) const
-    {
-      return deviceName.NumCompare("SDL") == PObject::EqualTo;
-    }
-} PVideoOutputDevice_SDL_descriptor;
-
-PCREATE_PLUGIN(SDL, PVideoOutputDevice, &PVideoOutputDevice_SDL_descriptor);
+  virtual bool ValidateDeviceName(const PString & deviceName, P_INT_PTR /*userData*/) const
+  {
+    return deviceName.NumCompare(GetServiceName()) == PObject::EqualTo;
+  }
+);
 
 
 ///////////////////////////////////////////////////////////////////////
@@ -120,12 +115,12 @@ class PSDL_Window : public PMutex
     virtual void MainLoop()
     {
 #if PTRACING
-      PTRACE(4, "SDL\tStart of event thread");
+      PTRACE(4, "Start of event thread");
 
       SDL_version v1;
       SDL_VERSION(&v1);
       const SDL_version * v2 = SDL_Linked_Version();
-      PTRACE(3, "SDL\tCompiled version: "
+      PTRACE(3, "Compiled version: "
              << (unsigned)v1.major << '.' << (unsigned)v1.minor << '.' << (unsigned)v1.patch
              << "  Run-Time version: "
              << (unsigned)v2->major << '.' << (unsigned)v2->minor << '.' << (unsigned)v2->patch);
@@ -133,7 +128,7 @@ class PSDL_Window : public PMutex
 
       // initialise the SDL library
       if (::SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE) < 0) {
-        PTRACE(1, "SDL\tCouldn't initialize SDL: " << ::SDL_GetError());
+        PTRACE(1, "Couldn't initialize SDL: " << ::SDL_GetError());
         return;
       }
 
@@ -150,7 +145,7 @@ class PSDL_Window : public PMutex
       m_surface = NULL;
       m_thread = NULL;
 
-      PTRACE(4, "SDL\tEnd of event thread");
+      PTRACE(4, "End of event thread");
     }
 
 
@@ -158,7 +153,7 @@ class PSDL_Window : public PMutex
     {
       SDL_Event sdlEvent;
       if (!::SDL_WaitEvent(&sdlEvent)) {
-        PTRACE(1, "SDL\tError getting event: " << ::SDL_GetError());
+        PTRACE(1, "Error getting event: " << ::SDL_GetError());
         return false;
       }
 
@@ -185,12 +180,12 @@ class PSDL_Window : public PMutex
               break;
 
             default :
-              PTRACE(5, "SDL\tUnhandled user event " << sdlEvent.user.code);
+              PTRACE(5, "Unhandled user event " << sdlEvent.user.code);
           }
           break;
 
         case SDL_QUIT :
-          PTRACE(3, "SDL\tUser closed window");
+          PTRACE(3, "User closed window");
           for (DeviceList::iterator it = m_devices.begin(); it != m_devices.end(); ++it)
             (*it)->FreeOverlay();
 
@@ -198,12 +193,16 @@ class PSDL_Window : public PMutex
           return false;
 
         case SDL_VIDEORESIZE :
-          PTRACE(4, "SDL\tResize window to " << sdlEvent.resize.w << " x " << sdlEvent.resize.h);
+          PTRACE(4, "Resize window to " << sdlEvent.resize.w << " x " << sdlEvent.resize.h);
           AdjustOverlays();
           break;
 
+        case SDL_ACTIVEEVENT :
+          PTRACE(4, "Window became active");
+          break;
+
         default :
-          PTRACE(5, "SDL\tUnhandled event " << (unsigned)sdlEvent.type);
+          PTRACE(5, "Unhandled event " << (unsigned)sdlEvent.type);
       }
 
       return true;
@@ -230,7 +229,7 @@ class PSDL_Window : public PMutex
         m_surface = ::SDL_SetVideoMode(device->GetFrameWidth(),
                                        device->GetFrameHeight(),
                                        0, SDL_SWSURFACE /* | SDL_RESIZABLE */);
-        PTRACE_IF(1, m_surface == NULL, "SDL\tCouldn't create SDL surface: " << ::SDL_GetError());
+        PTRACE_IF(1, m_surface == NULL, "Couldn't create SDL surface: " << ::SDL_GetError());
       }
 
       AdjustOverlays();
@@ -254,7 +253,7 @@ class PSDL_Window : public PMutex
 
     void AdjustOverlays()
     {
-      if (m_surface == NULL)
+      if (m_surface == NULL || m_devices.empty())
         return;
 
       PString title;
@@ -295,7 +294,7 @@ class PSDL_Window : public PMutex
       ::SDL_WM_SetCaption(title, NULL);
 
       if (::SDL_SetVideoMode(fullWidth, fullHeight, 0, SDL_SWSURFACE /* | SDL_RESIZABLE */) != m_surface) {
-        PTRACE(1, "SDL\tCouldn't resize surface: " << ::SDL_GetError());
+        PTRACE(1, "Couldn't resize surface: " << ::SDL_GetError());
       }
 
       for (DeviceList::iterator it = m_devices.begin(); it != m_devices.end(); ++it)
@@ -312,18 +311,26 @@ PVideoOutputDevice_SDL::PVideoOutputDevice_SDL()
   , m_y(0)
 {
   colourFormat = "YUV420P";
+  PTRACE(5, "Constructed.");
 }
 
 
 PVideoOutputDevice_SDL::~PVideoOutputDevice_SDL()
 { 
   Close();
+  PTRACE(5, "Destroyed.");
+}
+
+
+PStringArray PVideoOutputDevice_SDL::GetOutputDeviceNames()
+{
+  return PPlugin_PVideoOutputDevice_SDL::ServiceName();
 }
 
 
 PStringArray PVideoOutputDevice_SDL::GetDeviceNames() const
 {
-  return PString("SDL");
+  return GetOutputDeviceNames();
 }
 
 
@@ -335,7 +342,6 @@ PBoolean PVideoOutputDevice_SDL::Open(const PString & name, PBoolean /*startImme
 
   PSDL_Window::GetInstance().Run();
   PostEvent(PSDL_Window::e_AddDevice, true);
-
   return IsOpen();
 }
 
@@ -350,9 +356,8 @@ PBoolean PVideoOutputDevice_SDL::Close()
 {
   if (!IsOpen())
     return false;
-
+  
   PostEvent(PSDL_Window::e_RemoveDevice, true);
-
   return true;
 }
 
@@ -459,7 +464,7 @@ void PVideoOutputDevice_SDL::CreateOverlay(struct SDL_Surface * surface)
 
   m_overlay = ::SDL_CreateYUVOverlay(frameWidth, frameHeight, SDL_IYUV_OVERLAY, surface);
   if (m_overlay == NULL) {
-    PTRACE(1, "VSDL\tCouldn't create SDL overlay: " << ::SDL_GetError());
+    PTRACE(1, "Couldn't create SDL overlay: " << ::SDL_GetError());
     return;
   }
 
@@ -489,23 +494,13 @@ void PVideoOutputDevice_SDL::PostEvent(unsigned code, bool wait)
   sdlEvent.user.data1 = this;
   sdlEvent.user.data2 = NULL;
   if (::SDL_PushEvent(&sdlEvent) < 0) {
-    PTRACE(1, "SDL\tCouldn't post user event " << (unsigned)sdlEvent.user.code << ": " << ::SDL_GetError());
-  }
-  else {
-    PTRACE(5, "SDL\tPosted user event " << (unsigned)sdlEvent.user.code);
-    if (wait)
-      m_operationComplete.Wait();
+    PTRACE(1, "Couldn't post user event " << (unsigned)sdlEvent.user.code << ": " << ::SDL_GetError());
+    return;
   }
+
+  PTRACE(5, "Posted user event " << (unsigned)sdlEvent.user.code);
+  if (wait)
+    PAssert(m_operationComplete.Wait(10000),
+            PSTRSTRM("Couldn't process user event " << (unsigned)sdlEvent.user.code));
 }
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("SDL video support DISABLED")
-  #endif
-
 #endif // P_SDL
-
-
-// End of file ////////////////////////////////////////////////////////////////
