@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29432 $
+ * $Revision: 34696 $
  * $Author: rjongbloed $
- * $Date: 2013-04-03 22:29:08 -0500 (Wed, 03 Apr 2013) $
+ * $Date: 2016-03-19 15:09:41 +0000 (Sat, 19 Mar 2016) $
  */
 
 #ifdef __GNUC__
@@ -42,9 +42,6 @@
 #include <ptclib/http.h>
 
 
-static const bool DefaultBargeIn = true;
-
-
 class PVXMLChannelPCM : public PVXMLChannel
 {
   PCLASSINFO(PVXMLChannelPCM, PVXMLChannel);
@@ -90,14 +87,14 @@ class PVXMLChannelG729 : public PVXMLChannel
 };
 
 
-static PVXMLNodeFactory::Worker<PVXMLNodeHandler> BlockNodeHandler("Block", true);
+PFACTORY_CREATE(PVXMLNodeFactory, PVXMLNodeHandler, "Block", true);
 
 #define TRAVERSE_NODE(name) \
   class PVXMLTraverse##name : public PVXMLNodeHandler { \
     virtual bool Start(PVXMLSession & session, PXMLElement & element) const \
     { return session.Traverse##name(element); } \
   }; \
-  static PVXMLNodeFactory::Worker<PVXMLTraverse##name> name##NodeHandler(#name, true)
+  PFACTORY_CREATE(PVXMLNodeFactory, PVXMLTraverse##name, #name, true)
 
 TRAVERSE_NODE(Audio);
 TRAVERSE_NODE(Break);
@@ -120,7 +117,7 @@ TRAVERSE_NODE(Disconnect);
     virtual bool Finish(PVXMLSession & session, PXMLElement & element) const \
     { return session.Traversed##name(element); } \
   }; \
-  static PVXMLNodeFactory::Worker<PVXMLTraverse##name> name##NodeHandler(#name, true)
+  PFACTORY_CREATE(PVXMLNodeFactory, PVXMLTraverse##name, #name, true)
 
 TRAVERSE_NODE2(Menu);
 TRAVERSE_NODE2(Form);
@@ -142,11 +139,11 @@ class PVXMLTraverseEvent : public PVXMLNodeHandler
     return true;
   }
 };
-static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> FilledNodeHandler("Filled", true);
-static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> NoInputNodeHandler("NoInput", true);
-static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> NoMatchNodeHandler("NoMatch", true);
-static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> ErrorNodeHandler("Error", true);
-static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> CatchNodeHandler("Catch", true);
+PFACTORY_CREATE(PVXMLNodeFactory, PVXMLTraverseEvent, "Filled", true);
+PFACTORY_SYNONYM(PVXMLNodeFactory, PVXMLTraverseEvent, NoInput, "NoInput");
+PFACTORY_SYNONYM(PVXMLNodeFactory, PVXMLTraverseEvent, NoMatch, "NoMatch");
+PFACTORY_SYNONYM(PVXMLNodeFactory, PVXMLTraverseEvent, Error, "Error");
+PFACTORY_SYNONYM(PVXMLNodeFactory, PVXMLTraverseEvent, Catch, "Catch");
 
 #if PTRACING
 class PVXMLTraverseLog : public PVXMLNodeHandler {
@@ -159,7 +156,7 @@ class PVXMLTraverseLog : public PVXMLNodeHandler {
     return true;
   }
 };
-static PVXMLNodeFactory::Worker<PVXMLTraverseLog> LogNodeHandler("Log", true);
+PFACTORY_CREATE(PVXMLNodeFactory, PVXMLTraverseLog, "Log", true);
 #endif
 
 
@@ -170,6 +167,10 @@ static PVXMLNodeFactory::Worker<PVXMLTraverseLog> LogNodeHandler("Log", true);
 #define MEDIUM_BREAK_MSECS  2500
 #define LARGE_BREAK_MSECS   5000
 
+static PConstString ApplicationScope("application");
+static PConstString DialogScope("dialog");
+static PConstString PropertyScope("property");
+
 
 //////////////////////////////////////////////////////////
 
@@ -318,7 +319,7 @@ void PVXMLPlayableFile::OnStop()
 }
 
 
-PFactory<PVXMLPlayable>::Worker<PVXMLPlayableFile> vxmlPlayableFilenameFactory("File");
+PFACTORY_CREATE(PFactory<PVXMLPlayable>, PVXMLPlayableFile, "File");
 
 
 ///////////////////////////////////////////////////////////////
@@ -388,7 +389,7 @@ void PVXMLPlayableFileList::OnStop()
   }
 }
 
-PFactory<PVXMLPlayable>::Worker<PVXMLPlayableFileList> vxmlPlayableFilenameListFactory("FileList");
+PFACTORY_CREATE(PFactory<PVXMLPlayable>, PVXMLPlayableFileList, "FileList");
 
 
 ///////////////////////////////////////////////////////////////
@@ -444,7 +445,7 @@ void PVXMLPlayableCommand::OnStop()
   PVXMLPlayable::OnStop();
 }
 
-PFactory<PVXMLPlayable>::Worker<PVXMLPlayableCommand> vxmlPlayableCommandFactory("Command");
+PFACTORY_CREATE(PFactory<PVXMLPlayable>, PVXMLPlayableCommand, "Command");
 
 #endif
 
@@ -483,7 +484,7 @@ bool PVXMLPlayableData::OnRepeat()
   return PAssert(memfile != NULL, PLogicError) && PAssertOS(memfile->SetPosition(0));
 }
 
-PFactory<PVXMLPlayable>::Worker<PVXMLPlayableData> vxmlPlayableDataFactory("PCM Data");
+PFACTORY_CREATE(PFactory<PVXMLPlayable>, PVXMLPlayableData, "PCM Data");
 
 
 ///////////////////////////////////////////////////////////////
@@ -506,7 +507,7 @@ PBoolean PVXMLPlayableTone::Open(PVXMLChannel & chan, const PString & toneSpec,
   return PVXMLPlayable::Open(chan, toneSpec, delay, repeat, autoDelete);
 }
 
-PFactory<PVXMLPlayable>::Worker<PVXMLPlayableTone> vxmlPlayableToneFactory("Tone");
+PFACTORY_CREATE(PFactory<PVXMLPlayable>, PVXMLPlayableTone, "Tone");
 
 #endif // P_DTMF
 
@@ -543,7 +544,7 @@ bool PVXMLPlayableURL::OnStart()
   return m_vxmlChannel->SetReadChannel(client, false);
 }
 
-PFactory<PVXMLPlayable>::Worker<PVXMLPlayableURL> vxmlPlayableURLFactory("URL");
+PFACTORY_CREATE(PFactory<PVXMLPlayable>, PVXMLPlayableURL, "URL");
 
 
 ///////////////////////////////////////////////////////////////
@@ -753,7 +754,7 @@ PVXMLSession::PVXMLSession(PTextToSpeech * tts, PBoolean autoDelete)
   , m_currentNode(NULL)
   , m_xmlChanged(false)
   , m_speakNodeData(true)
-  , m_bargeIn(DefaultBargeIn)
+  , m_bargeIn(true)
   , m_bargingIn(false)
   , m_grammar(NULL)
   , m_defaultMenuDTMF('N') /// Disabled
@@ -762,7 +763,19 @@ PVXMLSession::PVXMLSession(PTextToSpeech * tts, PBoolean autoDelete)
   , m_transferStatus(NotTransfering)
   , m_transferStartTime(0)
 {
+#if P_SCRIPTS
+  m_scriptContext = PScriptLanguage::Create("Java");
+  if (m_scriptContext == NULL)
+    m_scriptContext = PScriptLanguage::Create("Lua"); // Back up
+  if (m_scriptContext != NULL) {
+    m_scriptContext->CreateComposite(ApplicationScope);
+    m_scriptContext->CreateComposite(DialogScope);
+    m_scriptContext->CreateComposite(PropertyScope);
+  }
+#endif
+
   SetVar("property.timeout" , "10s");
+  SetVar("property.bargein", "true");
 }
 
 
@@ -772,6 +785,10 @@ PVXMLSession::~PVXMLSession()
 
   if (m_autoDeleteTextToSpeech)
     delete m_textToSpeech;
+
+#if P_SCRIPTS
+  delete m_scriptContext;
+#endif
 }
 
 
@@ -893,7 +910,14 @@ bool PVXMLSession::InternalLoadVXML(const PString & xmlText, const PString & fir
     PWaitAndSignal mutex(m_sessionMutex);
 
     m_xmlChanged = true;
-    LoadGrammar(NULL);
+    m_speakNodeData = true;
+    m_bargeIn = true;
+    m_bargingIn = false;
+    m_recordingStatus = NotRecording;
+    m_transferStatus = NotTransfering;
+    m_currentNode = NULL;
+
+    FlushInput();
 
     // parse the XML
     m_xml.RemoveAll();
@@ -908,7 +932,7 @@ bool PVXMLSession::InternalLoadVXML(const PString & xmlText, const PString & fir
       return false;
     }
 
-    m_variableScope = m_variableScope.IsEmpty() ? "application" : "document";
+    m_variableScope = m_variableScope.IsEmpty() ? ApplicationScope : "document";
 
     PURL pathURL = m_rootURL;
     pathURL.ChangePath(PString::Empty()); // Remove last element of root URL
@@ -965,6 +989,8 @@ PURL PVXMLSession::NormaliseResourceName(const PString & src)
 
 bool PVXMLSession::SetCurrentForm(const PString & searchId, bool fullURI)
 {
+  ClearBargeIn();
+
   PString id = searchId;
 
   if (fullURI) {
@@ -1105,24 +1131,17 @@ void PVXMLSession::VXMLExecute(PThread &, P_INT_PTR)
 
   while (!m_abortVXML) {
     // process current node in the VXML script
-    if (ProcessNode()) {
-      /* wait for something to happen, usually output of some audio. But under
-         some circumstances we want to abort the script, but we  have to make
-         sure the script has been run to the end so submit actions etc. can be
-         performed. Record and audio and other user interaction commands can
-         be skipped, so we don't wait for them */
-      do {
-        while (ProcessEvents())
-          ;
-      } while (NextNode(true));
-    }
-    else {
-      // Wait till node finishes
+    bool processChildren = ProcessNode();
+
+    /* wait for something to happen, usually output of some audio. But under
+        some circumstances we want to abort the script, but we  have to make
+        sure the script has been run to the end so submit actions etc. can be
+        performed. Record and audio and other user interaction commands can
+        be skipped, so we don't wait for them */
+    do {
       while (ProcessEvents())
         ;
-
-      NextNode(false);
-    }
+    } while (NextNode(processChildren));
 
     // Determine if we should quit
     if (m_currentNode != NULL)
@@ -1208,6 +1227,7 @@ bool PVXMLSession::ProcessEvents()
   }
   else if (m_grammar != NULL && m_grammar->GetState() == PVXMLGrammar::Started) {
     PTRACE(4, "VXML\tAwaiting input, awaiting event");
+    PlaySilence(500);
   }
   else if (m_transferStatus == TransferInProgress) {
     PTRACE(4, "VXML\tTransfer in progress, awaiting event");
@@ -1273,9 +1293,14 @@ bool PVXMLSession::NextNode(bool processChildren)
     PVXMLNodeHandler * handler = PVXMLNodeFactory::CreateInstance(nodeType);
     if (handler != NULL) {
       if (!handler->Finish(*this, *element)) {
-        PTRACE(4, "VXML\t"
-               << (element != m_currentNode ? "Exception handling for" : "Continue processing")
-               << " VoiceXML element: <" << nodeType << '>');
+        if (m_currentNode != NULL) {
+          PTRACE_IF(4, (element = dynamic_cast<PXMLElement *>(m_currentNode)) != NULL,
+                    "VXML\tException handling <" << element->GetName() << "> for VoiceXML element: <" << nodeType << '>');
+          return false;
+        }
+
+        PTRACE(4, "VXML\tContinue processing VoiceXML element: <" << nodeType << '>');
+        m_currentNode = element;
         return true;
       }
       PTRACE(4, "VXML\tProcessed VoiceXML element: <" << nodeType << '>');
@@ -1290,6 +1315,22 @@ bool PVXMLSession::NextNode(bool processChildren)
 }
 
 
+void PVXMLSession::ClearBargeIn()
+{
+  PTRACE_IF(4, m_bargingIn, "VXML\tEnding barge in");
+  m_bargingIn = false;
+}
+
+
+void PVXMLSession::FlushInput()
+{
+  m_userInputMutex.Wait();
+  while (!m_userInputQueue.empty())
+    m_userInputQueue.pop();
+  m_userInputMutex.Signal();
+}
+
+
 bool PVXMLSession::ProcessGrammar()
 {
   if (m_grammar == NULL) {
@@ -1297,6 +1338,8 @@ bool PVXMLSession::ProcessGrammar()
     return true;
   }
 
+  m_grammar->SetSessionTimeout();
+
   switch (m_grammar->GetState()) {
     case PVXMLGrammar::Idle :
       m_grammar->Start();
@@ -1306,8 +1349,7 @@ bool PVXMLSession::ProcessGrammar()
       return false;
 
     default :
-      PTRACE_IF(4, m_bargingIn, "VXML\tEnding barge in");
-      m_bargingIn = false;
+      ClearBargeIn();
 
       PVXMLGrammar * grammar = m_grammar;
       m_grammar = NULL;
@@ -1329,9 +1371,6 @@ bool PVXMLSession::ProcessNode()
   if (m_currentNode == NULL)
     return false;
 
-  if (m_bargingIn)
-    return false;
-
   m_xmlChanged = false;
 
   PXMLData * nodeData = dynamic_cast<PXMLData *>(m_currentNode);
@@ -1445,7 +1484,8 @@ PBoolean PVXMLSession::TraversedRecord(PXMLElement & element)
     SetVar(m_recordingName + "$.size", '0');
   }
 
-  m_recordStopOnDTMF = element.HasAttribute("dtmfterm")|| !(element.GetAttribute("dtmfterm") *= "false");
+  // Disable stop on DTMF if attribute explicitly false, default is true
+  m_recordStopOnDTMF = !(element.GetAttribute("dtmfterm") *= "false");
 
   PFile::Remove(destination);
 
@@ -1475,6 +1515,16 @@ PString PVXMLSession::GetXMLError() const
 
 PString PVXMLSession::EvaluateExpr(const PString & expr)
 {
+#if P_SCRIPTS
+  if (m_scriptContext != NULL) {
+    static PConstString const EvalVarName("PTLibEvaluateExpressionResult");
+    if (m_scriptContext->Run(PSTRSTRM(EvalVarName<<'='<<expr)))
+      return m_scriptContext->GetString(EvalVarName);
+    PTRACE(2, "VXML\tCould not evaluate expression \"" << expr << "\" with script language " << m_scriptContext->GetLanguageName());
+    return PString::Empty();
+  }
+#endif
+
   // Should be full ECMAScript but ...
   // We only support expressions of the form 'literal'+variable or all digits
 
@@ -1512,10 +1562,23 @@ PString PVXMLSession::EvaluateExpr(const PString & expr)
 
 PCaselessString PVXMLSession::GetVar(const PString & varName) const
 {
+  // Check for literal
+  if (varName[0] == '\'' || varName[0] == '"') {
+    PINDEX end = varName.GetLength() - 1;
+    if (varName[end] == '\'' || varName[end] == '"')
+      --end;
+    return varName(1, end);
+  }
+
   PString fullVarName = varName;
   if (varName.Find('.') == P_MAX_INDEX)
     fullVarName = m_variableScope+'.'+varName;
 
+#if P_SCRIPTS
+  if (m_scriptContext != NULL)
+    return m_scriptContext->GetString(fullVarName);
+#endif
+
   return m_variables(fullVarName);
 }
 
@@ -1526,36 +1589,44 @@ void PVXMLSession::SetVar(const PString & varName, const PString & value)
   if (varName.Find('.') == P_MAX_INDEX)
     fullVarName = m_variableScope+'.'+varName;
 
+#if P_SCRIPTS
+  if (m_scriptContext != NULL)
+    m_scriptContext->SetString(fullVarName, value);
+#endif
+
   m_variables.SetAt(fullVarName, value);
 }
 
 
 PBoolean PVXMLSession::PlayFile(const PString & fn, PINDEX repeat, PINDEX delay, PBoolean autoDelete)
 {
-  return IsOpen() && GetVXMLChannel()->QueueFile(fn, repeat, delay, autoDelete);
+  return IsOpen() && !m_bargingIn && GetVXMLChannel()->QueueFile(fn, repeat, delay, autoDelete);
 }
 
 
 PBoolean PVXMLSession::PlayCommand(const PString & cmd, PINDEX repeat, PINDEX delay)
 {
-  return IsOpen() && GetVXMLChannel()->QueueCommand(cmd, repeat, delay);
+  return IsOpen() && !m_bargingIn && GetVXMLChannel()->QueueCommand(cmd, repeat, delay);
 }
 
 
 PBoolean PVXMLSession::PlayData(const PBYTEArray & data, PINDEX repeat, PINDEX delay)
 {
-  return IsOpen() && GetVXMLChannel()->QueueData(data, repeat, delay);
+  return IsOpen() && !m_bargingIn && GetVXMLChannel()->QueueData(data, repeat, delay);
 }
 
 
 PBoolean PVXMLSession::PlayTone(const PString & toneSpec, PINDEX repeat, PINDEX delay)
 {
-  return IsOpen() && GetVXMLChannel()->QueuePlayable("Tone", toneSpec, repeat, delay, true);
+  return IsOpen() && !m_bargingIn && GetVXMLChannel()->QueuePlayable("Tone", toneSpec, repeat, delay, true);
 }
 
 
 PBoolean PVXMLSession::PlayElement(PXMLElement & element)
 {
+  if (m_bargingIn)
+    return false;
+
   PString str = element.GetAttribute("src").Trim();
   if (str.IsEmpty()) {
     str = EvaluateExpr(element.GetAttribute("expr"));
@@ -1586,7 +1657,7 @@ PBoolean PVXMLSession::PlayElement(PXMLElement & element)
   if (!safe) {
     PFilePath filename;
     if (GetCache().Get("url", url.AsString(), fileType, filename))
-      return PlayFile(url.AsFilePath());
+      return PlayFile(filename);
   }
 
   PBYTEArray data;
@@ -1623,7 +1694,7 @@ PBoolean PVXMLSession::PlaySilence(const PTimeInterval & timeout)
 PBoolean PVXMLSession::PlaySilence(PINDEX msecs)
 {
   PBYTEArray nothing;
-  return IsOpen() && GetVXMLChannel()->QueueData(nothing, 1, msecs);
+  return IsOpen() && !m_bargingIn && GetVXMLChannel()->QueueData(nothing, 1, msecs);
 }
 
 
@@ -1635,7 +1706,7 @@ PBoolean PVXMLSession::PlayStop()
 
 PBoolean PVXMLSession::PlayResource(const PURL & url, PINDEX repeat, PINDEX delay)
 {
-  return IsOpen() && GetVXMLChannel()->QueueResource(url, repeat, delay);
+  return IsOpen() && !m_bargingIn && GetVXMLChannel()->QueueResource(url, repeat, delay);
 }
 
 
@@ -1645,6 +1716,7 @@ PBoolean PVXMLSession::LoadGrammar(PVXMLGrammar * grammar)
 
   delete m_grammar;
   m_grammar = grammar;
+  ClearBargeIn();
 
   PTRACE_IF(2, grammar != NULL, "VXML\tGrammar set to " << *grammar);
   return true;
@@ -1656,7 +1728,7 @@ PBoolean PVXMLSession::PlayText(const PString & textToPlay,
                                      PINDEX repeat,
                                      PINDEX delay)
 {
-  if (!IsOpen() || textToPlay.IsEmpty())
+  if (!IsOpen() || textToPlay.IsEmpty() || m_bargingIn)
     return false;
 
   PTRACE(5, "VXML\tConverting \"" << textToPlay << "\" to speech");
@@ -1960,25 +2032,13 @@ PBoolean PVXMLSession::TraverseIf(PXMLElement & element)
     return false;
   }
 
-  // Find var name
-  PString varname = condition.Left(location);
-
-  // Find value, skip '=' signs
-  PString cond_value = condition.Mid(location + 3);
-
-  // check if var value equals value from condition and if not skip child elements
-  PCaselessString value = GetVar(varname);
-  if (value == cond_value) {
-    PTRACE(3, "VXML\tCondition matched \"" << condition << '"');
-  }
-  else {
-    PTRACE(3, "VXMLSess\t\tCondition \"" << condition << "\"did not match, " << varname << " == " << value);
-    if (element.HasSubObjects())
-      // Step to last child element (really last element is NULL?)
-      m_currentNode = element.GetElement(element.GetSize() - 1);
+  if (GetVar(condition.Left(location).Trim()) == GetVar(condition.Mid(location + 2).Trim())) {
+    PTRACE(4, "VXML\tCondition matched \"" << condition << '"');
+    return true;
   }
 
-  return true;
+  PTRACE(4, "VXML\t\tCondition \"" << condition << "\"did not match");
+  return false;
 }
 
 
@@ -2046,8 +2106,9 @@ PBoolean PVXMLSession::TraverseSubmit(PXMLElement & element)
     }
 
     PMIMEInfo replyMIME;
-    if (client.GetDocument(url, replyMIME) && client.ReadContentBody(replyMIME))
-      return true;
+	PString body;
+    if (client.GetTextDocument(url, body))
+      return InternalLoadVXML(body, PString::Empty());
 
     PTRACE(1, "VXML\t<submit> GET " << url << " failed with "
            << client.GetLastResponseCode() << ' ' << client.GetLastResponseInfo());
@@ -2142,40 +2203,53 @@ PBoolean PVXMLSession::TraverseTransfer(PXMLElement &)
 
 PBoolean PVXMLSession::TraversedTransfer(PXMLElement & element)
 {
-  if (m_transferStatus == NotTransfering) {
-    TransferType type = BridgedTransfer;
-    if (element.GetAttribute("bridge") *= "false")
-      type = BlindTransfer;
-    else {
-      PCaselessString typeStr = element.GetAttribute("type");
-      if (typeStr == "blind")
+  const char * eventName = "error";
+
+  switch (m_transferStatus) {
+    case TransferCompleted :
+      return true;
+
+    case NotTransfering :
+    {
+      TransferType type = BridgedTransfer;
+      if (element.GetAttribute("bridge") *= "false")
         type = BlindTransfer;
-      else if (typeStr == "consultation")
-        type = ConsultationTransfer;
+      else {
+        PCaselessString typeStr = element.GetAttribute("type");
+        if (typeStr == "blind")
+          type = BlindTransfer;
+        else if (typeStr == "consultation")
+          type = ConsultationTransfer;
+      }
+
+      m_transferStartTime.SetCurrentTime();
+
+      bool started = false;
+      if (element.HasAttribute("dest"))
+        started = OnTransfer(element.GetAttribute("dest"), type);
+      else if (element.HasAttribute("destexpr"))
+        started = OnTransfer(EvaluateExpr(element.GetAttribute("destexpr")), type);
+
+      if (started) {
+        m_transferStatus = TransferInProgress;
+        return false;
+      }
+      break;
     }
 
-    m_transferStartTime.SetCurrentTime();
+    case TransferSuccessful :
+      eventName = "filled";
+      // Do default case
 
-    bool started = false;
-    if (element.HasAttribute("dest"))
-      started = OnTransfer(element.GetAttribute("dest"), type);
-    else if (element.HasAttribute("destexpr"))
-      started = OnTransfer(EvaluateExpr(element.GetAttribute("destexpr")), type);
-
-    if (started) {
-      m_transferStatus = TransferInProgress;
-      return false;
-    }
-
-    m_transferStatus = TransferFailed;
-  }
-  else {
-    PString name = element.GetAttribute("name");
-    if (!name.IsEmpty())
-      SetVar(name + "$.duration", PString(PString::Unsigned, (PTime() - m_transferStartTime).GetSeconds()));
+    default :
+      PString name = element.GetAttribute("name");
+      if (!name.IsEmpty())
+        SetVar(name + "$.duration", PString(PString::Unsigned, (PTime() - m_transferStartTime).GetSeconds()));
   }
 
-  return GoToEventHandler(element, m_transferStatus == TransferSuccessful ? "filled" : "error");
+  m_transferStatus = TransferCompleted;
+
+  return GoToEventHandler(element, eventName);
 }
 
 
@@ -2234,14 +2308,14 @@ PBoolean PVXMLSession::TraverseDisconnect(PXMLElement &)
 
 PBoolean PVXMLSession::TraverseForm(PXMLElement &)
 {
-  m_variableScope = "dialog";
+  m_variableScope = DialogScope;
   return true;
 }
 
 
 PBoolean PVXMLSession::TraversedForm(PXMLElement &)
 {
-  m_variableScope = "application";
+  m_variableScope = ApplicationScope;
   return true;
 }
 
@@ -2256,14 +2330,18 @@ PBoolean PVXMLSession::TraversePrompt(PXMLElement & element)
   if (m_grammar != NULL)
     m_grammar->SetTimeout(StringToTime(element.GetAttribute("timeout")));
 
-  m_bargeIn = !(element.GetAttribute("bargein") *= "false"); // Defaults to true
+  if ((element.GetAttribute("bargein") *= "false") || GetVar("property.bargein") == "false") {
+    m_bargeIn = false;
+    ClearBargeIn();
+    FlushInput();
+  }
   return true;
 }
 
 
 PBoolean PVXMLSession::TraversedPrompt(PXMLElement &)
 {
-  m_bargeIn = DefaultBargeIn;
+  m_bargeIn = GetVar("property.bargein") != "false";
   return true;
 }
 
@@ -2307,9 +2385,15 @@ PVXMLGrammar::PVXMLGrammar(PVXMLSession & session, PXMLElement & field)
   : m_session(session)
   , m_field(field)
   , m_state(Idle)
-  , m_timeout(PVXMLSession::StringToTime(session.GetVar("property.timeout"), 10000))
 {
   m_timer.SetNotifier(PCREATE_NOTIFIER(OnTimeout));
+  SetSessionTimeout();
+}
+
+
+void PVXMLGrammar::SetSessionTimeout()
+{
+  SetTimeout(PVXMLSession::StringToTime(m_session.GetVar("property.timeout"), 10000));
 }
 
 
@@ -2333,11 +2417,11 @@ void PVXMLGrammar::Start()
 
 void PVXMLGrammar::OnTimeout(PTimer &, P_INT_PTR)
 {
-  PTRACE(3, "VXML\tTimeout for grammar " << *this);
+	PTRACE(3, "VXML\tTimeout for grammar " << *this );
   m_mutex.Wait();
 
   if (m_state == Started) {
-    m_state = NoInput;
+    m_state = IsFilled() ? Filled : NoInput;
     m_session.Trigger();
   }
 
@@ -2433,24 +2517,38 @@ void PVXMLDigitsGrammar::OnUserInput(const char ch)
 {
   PWaitAndSignal mutex(m_mutex);
 
-  // Ignore any keys until we are running
+  if (m_state == Idle)
+    Start();
+
+  // Ignore any keys unless we are running
   if (m_state != Started)
     return;
 
-  PINDEX len = m_value.GetLength();
-
   // is this char the terminator?
   if (m_terminators.Find(ch) != P_MAX_INDEX) {
-    m_state = (len >= m_minDigits && len <= m_maxDigits) ? Filled : NoMatch;
+    m_state = IsFilled() ? Filled : NoMatch;
     return;
   }
 
   // Otherwise add to the grammar and check to see if we're done
+  PINDEX len = m_value.GetLength();
+
   m_value += ch;
   if (++len >= m_maxDigits)
     m_state = PVXMLGrammar::Filled;   // the grammar is filled!
 }
 
+bool PVXMLDigitsGrammar::IsFilled()
+{
+	PINDEX len = m_value.GetLength();
+	bool filled = len >= m_minDigits && len <= m_maxDigits;
+
+	PTRACE(4, "VXML\t Grammar " << *this << 
+		(filled ? " has been FILLED" : " has NOT yet been filled" ) 
+		<< ". Collected value=" << m_value << ", length: " << len << ", while min=" << m_minDigits << " max=" << m_maxDigits);
+
+	return filled;
+}
 
 //////////////////////////////////////////////////////////////////
 
@@ -2535,38 +2633,32 @@ PString PVXMLChannel::AdjustWavFilename(const PString & ofn)
 PWAVFile * PVXMLChannel::CreateWAVFile(const PFilePath & fn, PBoolean recording)
 {
   PWAVFile * wav = new PWAVFile;
-  if (!wav->SetFormat(mediaFormat)) {
-    PTRACE(1, "VXML\tWAV file format " << mediaFormat << " not known");
-    delete wav;
-    return NULL;
-  }
-
-  wav->SetAutoconvert();
-  if (!wav->Open(fn,
-                 recording ? PFile::WriteOnly : PFile::ReadOnly,
-                 PFile::ModeDefault))
-    PTRACE(2, "VXML\tCould not open WAV file " << wav->GetName());
-
-  else if (recording) {
+  if (recording) {
     wav->SetChannels(1);
-    wav->SetSampleRate(8000);
     wav->SetSampleSize(16);
-    return wav;
+    wav->SetSampleRate(GetSampleFrequency());
+    if (!wav->SetFormat(mediaFormat))
+      PTRACE(2, "VXML\tUnsupported codec " << mediaFormat);
+    else if (!wav->Open(fn, PFile::WriteOnly))
+      PTRACE(2, "VXML\tCould not create WAV file \"" << wav->GetName() << "\" - " << wav->GetErrorText());
+    else if (!wav->SetAutoconvert())
+      PTRACE(2, "VXML\tWAV file cannot convert to " << mediaFormat);
+    else
+      return wav;
   }
-
-  else if (!wav->IsValid())
-    PTRACE(2, "VXML\tWAV file header invalid for " << wav->GetName());
-
-  else if (wav->GetSampleRate() != GetSampleFrequency())
-    PTRACE(2, "VXML\tWAV file has unsupported sample frequency " << wav->GetSampleRate());
-
-  else if (wav->GetChannels() != 1)
-    PTRACE(2, "VXML\tWAV file has unsupported channel count " << wav->GetChannels());
-
   else {
-    wav->SetAutoconvert();   /// enable autoconvert
-    PTRACE(3, "VXML\tOpened WAV file " << wav->GetName());
-    return wav;
+    if (!wav->Open(fn, PFile::ReadOnly))
+      PTRACE(2, "VXML\tCould not open WAV file \"" << wav->GetName() << "\" - " << wav->GetErrorText());
+    else if (wav->GetFormatString() != mediaFormat && !wav->SetAutoconvert())
+      PTRACE(2, "VXML\tWAV file cannot convert from " << wav->GetFormatString());
+    else if (wav->GetChannels() != 1)
+      PTRACE(2, "VXML\tWAV file has unsupported channel count " << wav->GetChannels());
+    else if (wav->GetSampleSize() != 16)
+      PTRACE(2, "VXML\tWAV file has unsupported sample size " << wav->GetSampleSize());
+    else if (wav->GetSampleRate() != GetSampleFrequency())
+      PTRACE(2, "VXML\tWAV file has unsupported sample rate " << wav->GetSampleRate());
+    else
+      return wav;
   }
 
   delete wav;
@@ -2673,14 +2765,15 @@ PBoolean PVXMLChannel::Read(void * buffer, PINDEX amount)
       m_currentPlayItem->OnStop();
       delete m_currentPlayItem;
       m_currentPlayItem = NULL;
-      m_vxmlSession->Trigger();
     }
 
     for (;;) {
       // check the queue for the next action, if none, send silence
       m_currentPlayItem = m_playQueue.Dequeue();
-      if (m_currentPlayItem == NULL)
+      if (m_currentPlayItem == NULL) {
+        m_vxmlSession->Trigger(); // notify about the end of queue
         goto double_break;
+      }
 
       // start the new item
       if (m_currentPlayItem->OnStart())
@@ -2806,7 +2899,7 @@ void PVXMLChannel::FlushQueue()
 
 ///////////////////////////////////////////////////////////////
 
-PFactory<PVXMLChannel>::Worker<PVXMLChannelPCM> pcmVXMLChannelFactory(VXML_PCM16);
+PFACTORY_CREATE(PFactory<PVXMLChannel>, PVXMLChannelPCM, VXML_PCM16);
 
 PVXMLChannelPCM::PVXMLChannelPCM()
   : PVXMLChannel(10, 160)
@@ -2878,7 +2971,7 @@ void PVXMLChannelPCM::GetBeepData(PBYTEArray & data, unsigned ms)
 
 ///////////////////////////////////////////////////////////////
 
-PFactory<PVXMLChannel>::Worker<PVXMLChannelG7231> g7231VXMLChannelFactory(VXML_G7231);
+PFACTORY_CREATE(PFactory<PVXMLChannel>, PVXMLChannelG7231, VXML_G7231);
 
 PVXMLChannelG7231::PVXMLChannelG7231()
   : PVXMLChannel(30, 0)
@@ -2935,7 +3028,7 @@ PBoolean PVXMLChannelG7231::IsSilenceFrame(const void * buf, PINDEX len) const
 
 ///////////////////////////////////////////////////////////////
 
-PFactory<PVXMLChannel>::Worker<PVXMLChannelG729> g729VXMLChannelFactory(VXML_G729);
+PFACTORY_CREATE(PFactory<PVXMLChannel>, PVXMLChannelG729, VXML_G729);
 
 PVXMLChannelG729::PVXMLChannelG729()
   : PVXMLChannel(10, 0)
@@ -2993,7 +3086,7 @@ class TextToSpeech_Sample : public PTextToSpeech
   protected:
     //PTextToSpeech * defaultEngine;
 
-    PMutex mutex;
+    PDECLARE_MUTEX(mutex);
     PBoolean opened;
     PBoolean usingFile;
     PString text;
@@ -3355,7 +3448,7 @@ PBoolean TextToSpeech_Sample::SpeakFile(const PString & text)
   return true;
 }
 
-PFactory<PTextToSpeech>::Worker<TextToSpeech_Sample> sampleTTSFactory("sampler", false);
+PFACTORY_CREATE(PFactory<PTextToSpeech>, TextToSpeech_Sample, "sampler", false);
 
 
 #endif   // P_VXML
