@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 29998 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2013-06-19 09:18:59 +0100 (Wed, 19 Jun 2013) $
  */
 
 #define _CONFIG_CXX
@@ -145,12 +145,6 @@ PXConfigDictionary * configDict;
 
 //////////////////////////////////////////////////////
 
-void PProcess::CreateConfigFilesDictionary()
-{
-  configFiles = new PXConfigDictionary(0);
-}
-
-
 PXConfigWriteThread::PXConfigWriteThread(PSyncPointAck & s)
   : PThread(10000, AutoDeleteThread),
     stop(s)
