@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
- * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Revision: 32504 $
+ * $Author: ededu $
+ * $Date: 2014-08-12 09:02:59 +0100 (Tue, 12 Aug 2014) $
  */
 
 #ifdef __GNUC__
@@ -812,16 +812,8 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
   }
 
 #else
-
-#pragma message("Platform requires implemetation of GetRouteTable()")
   return false;
-
 #endif
 }
 
-
 #include "../common/pethsock.cxx"
-
-
-///////////////////////////////////////////////////////////////////////////////
-
