@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29563 $
+ * $Revision: 33826 $
  * $Author: rjongbloed $
- * $Date: 2013-04-24 22:14:34 -0500 (Wed, 24 Apr 2013) $
+ * $Date: 2015-08-04 20:16:34 +0100 (Tue, 04 Aug 2015) $
  */
 
 #include <ptlib.h>
@@ -412,8 +412,10 @@ PObject * PAbstractList::RemoveAt(PINDEX index)
 
 void PAbstractList::InsertElement(PListElement * element, PObject * obj)
 {
-  if (element == NULL)
+  if (element == NULL) {
     Append(obj);
+    return;
+  }
 
   Element * newElement = new Element(obj);
   if (element->prev != NULL)
@@ -1115,66 +1117,86 @@ void PAbstractSortedList::DeleteSubTrees(PSortedListElement * node, PBoolean del
 void PHashTableInfo::DestroyContents()
 {
   for (PINDEX i = 0; i < GetSize(); i++) {
-    Element * list = GetAt(i);
-    if (list != NULL) {
-      Element * elmt = list;
-      do {
-        Element * nextElmt = elmt->next;
-        if (elmt->data != NULL && reference->deleteObjects)
-          delete elmt->data;
-        if (deleteKeys)
-          delete elmt->key;
-        delete elmt;
-        elmt = nextElmt;
-      } while (elmt != list);
+    PHashTableElement * elmt = GetAt(i).m_head;
+    while (elmt != NULL) {
+      PHashTableElement * nextElmt = elmt->m_next;
+      if (elmt->m_data != NULL && reference->deleteObjects)
+        delete elmt->m_data;
+      if (deleteKeys)
+        delete elmt->m_key;
+      delete elmt;
+      elmt = nextElmt;
     }
   }
   PAbstractArray::DestroyContents();
 }
 
 
-PINDEX PHashTableInfo::AppendElement(PObject * key, PObject * data)
+void PHashTableInfo::AppendElement(PObject * key, PObject * data PTRACE_PARAM(, PHashTable * owner))
 {
   PINDEX bucket = PAssertNULL(key)->HashFunction();
-  Element * list = GetAt(bucket);
-  Element * element = new Element;
+  if (bucket >= GetSize())
+    SetSize(bucket+1);
+
+  PHashTableList & list = operator[](bucket);
+  PHashTableElement * element = new PHashTableElement;
   PAssert(element != NULL, POutOfMemory);
-  element->key = key;
-  element->data = data;
-  element->bucket = bucket;
-  if (list == NULL) {
-    element->next = element->prev = element;
-    SetAt(bucket, element);
-  }
-  else if (list == list->prev) {
-    list->next = list->prev = element;
-    element->next = element->prev = list;
+  element->m_key = key;
+  element->m_data = data;
+  element->m_bucket = bucket;
+  element->m_next = NULL;
+
+  if (list.m_head == NULL) {
+    element->m_prev = NULL;
+    list.m_head = list.m_tail = element;
   }
   else {
-    element->next = list;
-    element->prev = list->prev;
-    list->prev->next = element;
-    list->prev = element;
+    element->m_prev = list.m_tail;
+    list.m_tail->m_next = element;
+    list.m_tail =  element;
   }
-  return bucket;
+
+#if PTRACING
+  ++list.m_size;
+  PINDEX totalSize = owner->GetSize();
+  PTRACE_IF(1, list.m_size > 20 && list.m_size > totalSize/2, owner, "PTLib",
+            "Poor hash function used, more than 50% of " << totalSize <<
+            " items in same bucket for class " << owner->GetClass());
+#endif
 }
 
 
 PObject * PHashTableInfo::RemoveElement(const PObject & key)
 {
   PObject * obj = NULL;
-  Element * element = GetElementAt(key);
+  PHashTableElement * element = GetElementAt(key);
   if (element != NULL) {
-    if (element == element->prev)
-      SetAt(element->bucket, NULL);
-    else {
-      element->prev->next = element->next;
-      element->next->prev = element->prev;
-      SetAt(element->bucket, element->next);
+    PHashTableList & list = operator[](element->m_bucket);
+    if (element == list.m_head) {
+      if (element == list.m_tail)
+        list.m_head = list.m_tail = NULL;
+      else {
+        list.m_head = list.m_head->m_next;
+        list.m_head->m_prev = NULL;
+      }
     }
-    obj = element->data;
+    else {
+      if (element == list.m_tail) {
+        list.m_tail = list.m_tail->m_prev;
+        list.m_tail->m_next = NULL;
+      }
+      else {
+        element->m_prev->m_next = element->m_next;
+        element->m_next->m_prev = element->m_prev;
+      }
+    }
+#if PTRACING
+    --list.m_size;
+#endif
+
+    obj = element->m_data;
     if (deleteKeys)
-      delete element->key;
+      delete element->m_key;
     delete element;
   }
   return obj;
@@ -1185,20 +1207,20 @@ PHashTableElement * PHashTableInfo::GetElementAt(PINDEX index)
 {
   PHashTableElement * element;
   PINDEX bucket = 0;
-  while ((element = GetAt(bucket)) == NULL) {
+  while ((element = GetAt(bucket).m_head) == NULL) {
     if (bucket >= GetSize())
       return NULL;
     bucket++;
   }
 
   for (PINDEX i = 0; i < index; ++i) {
-    if (element->next != operator[](bucket))
-      element = element->next;
+    if (element->m_next != NULL)
+      element = element->m_next;
     else {
       do {
         if (++bucket >= GetSize())
           return NULL;
-      } while ((element = operator[](bucket)) == NULL);
+      } while ((element = GetAt(bucket).m_head) == NULL);
     }
   }
 
@@ -1208,14 +1230,11 @@ PHashTableElement * PHashTableInfo::GetElementAt(PINDEX index)
 
 PHashTableElement * PHashTableInfo::GetElementAt(const PObject & key)
 {
-  Element * list = GetAt(key.HashFunction());
-  if (list != NULL) {
-    Element * element = list;
-    do {
-      if (*element->key == key) 
-        return element;
-      element = element->next;
-    } while (element != list);
+  PHashTableElement * element = GetAt(key.HashFunction()).m_head;
+  while (element != NULL) {
+    if (*element->m_key == key) 
+      return element;
+    element = element->m_next;
   }
   return NULL;
 }
@@ -1225,16 +1244,13 @@ PINDEX PHashTableInfo::GetElementsIndex(const PObject * obj, PBoolean byValue, P
 {
   PINDEX index = 0;
   for (PINDEX i = 0; i < GetSize(); i++) {
-    Element * list = operator[](i);
-    if (list != NULL) {
-      Element * element = list;
-      do {
-        PObject * keydata = keys ? element->key : element->data;
-        if (byValue ? (*keydata == *obj) : (keydata == obj))
-          return index;
-        element = element->next;
-        index++;
-      } while (element != list);
+    PHashTableElement * element = GetAt(i).m_head;
+    while (element != NULL) {
+      PObject * keydata = keys ? element->m_key : element->m_data;
+      if (byValue ? (*keydata == *obj) : (keydata == obj))
+        return index;
+      element = element->m_next;
+      index++;
     }
   }
   return P_MAX_INDEX;
@@ -1243,14 +1259,17 @@ PINDEX PHashTableInfo::GetElementsIndex(const PObject * obj, PBoolean byValue, P
 
 PHashTableElement * PHashTableInfo::NextElement(PHashTableElement * element) const
 {
-  PINDEX bucket = element->bucket;
-  if (element->next != operator[](bucket))
-    element = element->next;
+  if (element == NULL)
+    return NULL;
+
+  PINDEX bucket = element->m_bucket;
+  if (element->m_next != NULL)
+    element = element->m_next;
   else {
     do {
       if (++bucket >= GetSize())
         return NULL;
-    } while ((element = operator[](bucket)) == NULL);
+    } while ((element = GetAt(bucket).m_head) == NULL);
   }
   return element;
 }
@@ -1258,15 +1277,17 @@ PHashTableElement * PHashTableInfo::NextElement(PHashTableElement * element) con
 
 PHashTableElement * PHashTableInfo::PrevElement(PHashTableElement * element) const
 {
-  PINDEX bucket = element->bucket;
-  if (element != operator[](bucket))
-    element = element->prev;
+  if (element == NULL)
+    return NULL;
+
+  PINDEX bucket = element->m_bucket;
+  if (element->m_prev != NULL)
+    element = element->m_prev;
   else {
     do {
       if (bucket-- == 0)
         return NULL;
-    } while ((element = operator[](bucket)) == NULL);
-    element = element->prev;
+    } while ((element = GetAt(bucket).m_tail) == NULL);
   }
   return element;
 }
@@ -1275,7 +1296,7 @@ PHashTableElement * PHashTableInfo::PrevElement(PHashTableElement * element) con
 ///////////////////////////////////////////////////////////////////////////////
 
 PHashTable::PHashTable()
-  : hashTable(new PHashTable::Table)
+  : hashTable(new PHashTableInfo)
 {
   PAssert(hashTable != NULL, POutOfMemory);
 }
@@ -1300,18 +1321,18 @@ void PHashTable::CopyContents(const PHashTable & hash)
 void PHashTable::CloneContents(const PHashTable * hash)
 {
   PINDEX sz = PAssertNULL(hash)->GetSize();
-  PHashTable::Table * original = PAssertNULL(hash->hashTable);
+  PHashTableInfo * original = PAssertNULL(hash->hashTable);
 
-  hashTable = new PHashTable::Table(original->GetSize());
+  hashTable = new PHashTableInfo(original->GetSize());
   PAssert(hashTable != NULL, POutOfMemory);
   hashTable->deleteKeys = original->deleteKeys;
 
   for (PINDEX i = 0; i < sz; i++) {
-    Element * lastElement = original->GetElementAt(i);
-    PObject * data = lastElement->data;
+    PHashTableElement * lastElement = original->GetElementAt(i);
+    PObject * data = lastElement->m_data;
     if (data != NULL)
       data = data->Clone();
-    hashTable->AppendElement(lastElement->key->Clone(), data);
+    hashTable->AppendElement(lastElement->m_key->Clone(), data PTRACE_PARAM(, this));
   }
 }
 
@@ -1332,17 +1353,17 @@ PBoolean PHashTable::SetSize(PINDEX)
 
 PObject & PHashTable::AbstractGetDataAt(PINDEX index) const
 {
-  Element * lastElement = hashTable->GetElementAt(index);
+  PHashTableElement * lastElement = hashTable->GetElementAt(index);
   PAssert(lastElement != NULL, PInvalidArrayIndex);
-  return *lastElement->data;
+  return *lastElement->m_data;
 }
 
 
 const PObject & PHashTable::AbstractGetKeyAt(PINDEX index) const
 {
-  Element * lastElement = hashTable->GetElementAt(index);
+  PHashTableElement * lastElement = hashTable->GetElementAt(index);
   PAssert(lastElement != NULL, PInvalidArrayIndex);
-  return *lastElement->key;
+  return *lastElement->m_key;
 }
 
 
@@ -1373,8 +1394,9 @@ PINDEX PAbstractSet::Append(PObject * obj)
     return P_MAX_INDEX;
   }
 
-  reference->size++;
-  return hashTable->AppendElement(obj, NULL);
+  ++reference->size;
+  hashTable->AppendElement(obj, NULL PTRACE_PARAM(, this));
+  return reference->size;
 }
 
 
@@ -1407,11 +1429,11 @@ PBoolean PAbstractSet::Remove(const PObject * obj)
 
 PObject * PAbstractSet::RemoveAt(PINDEX index)
 {
-  Element * lastElement = hashTable->GetElementAt(index);
+  PHashTableElement * lastElement = hashTable->GetElementAt(index);
   if (lastElement == NULL)
     return NULL;
 
-  PObject * obj = lastElement->key;
+  PObject * obj = lastElement->m_key;
   hashTable->deleteKeys = hashTable->reference->deleteObjects = reference->deleteObjects;
   hashTable->RemoveElement(*obj);
   reference->size--;
@@ -1537,8 +1559,8 @@ PBoolean PAbstractDictionary::SetAt(PINDEX index, PObject * val)
 
 PObject * PAbstractDictionary::GetAt(PINDEX index) const
 {
-  Element * element = hashTable->GetElementAt(index);
-  return PAssert(element != NULL, PInvalidArrayIndex) ? element->data : NULL;
+  PHashTableElement * element = hashTable->GetElementAt(index);
+  return PAssert(element != NULL, PInvalidArrayIndex) ? element->m_data : NULL;
 }
  
  
@@ -1561,15 +1583,15 @@ PObject * PAbstractDictionary::AbstractSetAt(const PObject & key, PObject * obj)
     }
   }
   else {
-    Element * element = hashTable->GetElementAt(key);
+    PHashTableElement * element = hashTable->GetElementAt(key);
     if (element == NULL) {
-      hashTable->AppendElement(key.Clone(), obj);
-      reference->size++;
+      ++reference->size;
+      hashTable->AppendElement(key.Clone(), obj PTRACE_PARAM(, this));
     }
-    else if (element->data != obj) {
+    else if (element->m_data != obj) {
       if (reference->deleteObjects) 
-    	delete element->data;
-      element->data = obj;
+    	delete element->m_data;
+      element->m_data = obj;
     }
   }
 
@@ -1579,15 +1601,15 @@ PObject * PAbstractDictionary::AbstractSetAt(const PObject & key, PObject * obj)
 
 PObject * PAbstractDictionary::AbstractGetAt(const PObject & key) const
 {
-  Element * element = hashTable->GetElementAt(key);
-  return element != NULL ? element->data : (PObject *)NULL;
+  PHashTableElement * element = hashTable->GetElementAt(key);
+  return element != NULL ? element->m_data : (PObject *)NULL;
 }
 
 
 PObject & PAbstractDictionary::GetRefAt(const PObject & key) const
 {
-  Element * element = hashTable->GetElementAt(key);
-  return *PAssertNULL(element)->data;
+  PHashTableElement * element = hashTable->GetElementAt(key);
+  return *PAssertNULL(element)->m_data;
 }
 
 
@@ -1595,8 +1617,8 @@ void PAbstractDictionary::AbstractGetKeys(PArrayObjects & keys) const
 {
   keys.SetSize(GetSize());
   PINDEX index = 0;
-  for (Element * element = hashTable->GetElementAt((PINDEX)0); element != NULL; element = hashTable->NextElement(element))
-    keys.SetAt(index++, element->key->Clone());
+  for (PHashTableElement * element = hashTable->GetElementAt((PINDEX)0); element != NULL; element = hashTable->NextElement(element))
+    keys.SetAt(index++, element->m_key->Clone());
 }
 
 
