@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29735 $
+ * $Revision: 34492 $
  * $Author: rjongbloed $
- * $Date: 2013-05-17 03:20:09 -0500 (Fri, 17 May 2013) $
+ * $Date: 2016-01-10 16:03:47 +0000 (Sun, 10 Jan 2016) $
  */
 
 #include <ptlib.h>
@@ -36,6 +36,7 @@
 
 #include <ostream>
 #include <limits>
+#include <math.h>
 
 #ifdef __NUCLEUS_PLUS__
 extern "C" int vsprintf(char *, const char *, va_list);
@@ -140,6 +141,7 @@ void PContainer::Destruct()
 void PContainer::DestroyReference()
 {
   delete reference;
+  reference = NULL;
 }
 
 
@@ -280,7 +282,7 @@ void PAbstractArray::PrintOn(ostream & strm) const
   char separator = strm.fill();
   int width = (int)strm.width();
   for (PINDEX  i = 0; i < GetSize(); i++) {
-    if (i > 0 && separator != '\0')
+    if (i > 0 && separator != '\0' && !isdigit(separator))
       strm << separator;
     strm.width(width);
     PrintElementOn(strm, i);
@@ -426,6 +428,12 @@ void * PAbstractArray::GetPointer(PINDEX minSize)
 }
 
 
+PINDEX PAbstractArray::GetLength() const
+{
+  return elementSize*GetSize();
+}
+
+
 PBoolean PAbstractArray::Concatenate(const PAbstractArray & array)
 {
   if (!allocatedDynamically || array.elementSize != elementSize)
@@ -491,33 +499,37 @@ void PCharArray::ReadFrom(istream &strm)
 
 void PBYTEArray::PrintOn(ostream & strm) const
 {
-  PINDEX line_width = (int)strm.width();
+  int line_width = (int)strm.width();
   if (line_width == 0)
     line_width = 16;
   strm.width(0);
 
-  PINDEX indent = (int)strm.precision();
+  int indent = (int)strm.precision();
 
-  PINDEX val_width = ((strm.flags()&ios::basefield) == ios::hex) ? 2 : 3;
+  int val_width = ((strm.flags()&ios::basefield) == ios::hex) ? 2 : 3;
+
+  ios::fmtflags oldFlags = strm.flags();
+  if (strm.fill() == '0')
+    strm.setf(ios::right, ios::adjustfield);
 
   PINDEX i = 0;
   while (i < GetSize()) {
     if (i > 0)
       strm << '\n';
-    PINDEX j;
+    int j;
     for (j = 0; j < indent; j++)
       strm << ' ';
     for (j = 0; j < line_width; j++) {
-      if (j == line_width/2)
+      if (indent >= 0 && j == line_width/2)
         strm << ' ';
       if (i+j < GetSize())
         strm << setw(val_width) << (theArray[i+j]&0xff);
       else {
-        PINDEX k;
-        for (k = 0; k < val_width; k++)
+        for (int k = 0; k < val_width; k++)
           strm << ' ';
       }
-      strm << ' ';
+      if (indent >= 0)
+        strm << ' ';
     }
     if ((strm.flags()&ios::floatfield) != ios::fixed) {
       strm << "  ";
@@ -533,6 +545,8 @@ void PBYTEArray::PrintOn(ostream & strm) const
     }
     i += line_width;
   }
+
+  strm.flags(oldFlags);
 }
 
 
@@ -905,22 +919,27 @@ PString::PString(unsigned long n)
 }
 
 
-PString::PString(PInt64 n)
-  : PCharArray(sizeof(PInt64)*3+2)
-  , m_length(p_signed2string<PInt64, PUInt64>(n, 10, theArray))
+#ifdef HAVE_LONG_LONG_INT
+PString::PString(long long n)
+  : PCharArray(sizeof(long long)*3+2)
+  , m_length(p_signed2string<signed long long, unsigned long long>(n, 10, theArray))
 {
 }
+#endif
 
 
-PString::PString(PUInt64 n)
-  : PCharArray(sizeof(PUInt64)*3+1)
-  , m_length(p_unsigned2string<PUInt64>(n, 10, theArray))
+#ifdef HAVE_UNSIGNED_LONG_LONG_INT
+PString::PString(unsigned long long n)
+  : PCharArray(sizeof(unsigned long long)*3+1)
+  , m_length(p_unsigned2string<unsigned long long>(n, 10, theArray))
 {
 }
+#endif
 
 
-static const char siTable[] = { 'f', 'p', 'n', 'u', 'm', '\0', 'k', 'M', 'G', 'T', 'E' };
-static const size_t siZero = sizeof(siTable)/2;
+static const char siTable[][2] = { "f", "p", "n", "u", "m", "", "k", "M", "G", "T", "E" };
+static const size_t siCount = sizeof(siTable)/2;
+static const size_t siZero = siCount/2;
 
 static PINDEX InternalConvertScaleSI(PInt64 value, unsigned param, char * theArray)
 {
@@ -939,7 +958,7 @@ static PINDEX InternalConvertScaleSI(PInt64 value, unsigned param, char * theArr
 
   PINDEX length = 0;
   PInt64 multiplier = 1;
-  for (size_t i = siZero+1; i < sizeof(siTable); ++i) {
+  for (size_t i = siZero+1; i < siCount; ++i) {
     multiplier *= 1000;
     if (absValue < multiplier*1000) {
       length = p_signed2string<PInt64, PUInt64>(value/multiplier, 10, theArray);
@@ -951,7 +970,7 @@ static PINDEX InternalConvertScaleSI(PInt64 value, unsigned param, char * theArr
           theArray[length++] = (absValue/multiplier)%10 + '0';
         } while (--param > 0 && absValue%multiplier != 0);
       }
-      theArray[length++] = siTable[i];
+      theArray[length++] = siTable[i][0];
       break;
     }
   }
@@ -997,11 +1016,16 @@ PSTRING_CONV_CTOR(         int,   int,    unsigned int);
 PSTRING_CONV_CTOR(unsigned int,   int,    unsigned int);
 PSTRING_CONV_CTOR(         long,  long,   unsigned long);
 PSTRING_CONV_CTOR(unsigned long,  long,   unsigned long);
-PSTRING_CONV_CTOR(       PInt64,  PInt64, PUInt64);
-PSTRING_CONV_CTOR(      PUInt64,  PInt64, PUInt64);
+#ifdef HAVE_LONG_LONG_INT
+PSTRING_CONV_CTOR(         long long, long long, unsigned long long);
+#endif
+#ifdef HAVE_UNSIGNED_LONG_LONG_INT
+PSTRING_CONV_CTOR(unsigned long long, long long, unsigned long long);
+#endif
 
 
 PString::PString(ConversionType type, double value, unsigned places)
+  : m_length(0)
 {
   switch (type) {
     case Decimal :
@@ -1013,19 +1037,30 @@ PString::PString(ConversionType type, double value, unsigned places)
       break;
 
     case ScaleSI :
-    {
-      // Scale it according to SI multipliers
-      double multiplier = 1e-18;
-      for (size_t i = 0; i < sizeof(siTable); ++i) {
-        multiplier *= 1000;
-        if (( value > multiplier &&  value < multiplier*1000) ||
-            (-value > multiplier && -value < multiplier*1000)) {
-          sprintf("%0.*f%c", (int)places, value/multiplier, siTable[i]);
-          break;
+      if (value == 0)
+        sprintf("%0.*f", (int)places, value);
+      else {
+        // Scale it according to SI multipliers
+        double multiplier = 1e-15;
+        double absValue = fabs(value);
+        size_t i;
+        for (i = 0; i < siCount-1; ++i) {
+          double nextMultiplier = multiplier * 1000;
+          if (absValue < nextMultiplier)
+            break;
+          multiplier = nextMultiplier;
         }
+        value /= multiplier;
+        // Want places to be significant figures
+        if (places >= 3 && value >= 100)
+          places -= 3;
+        else if (places >= 2 && value >= 10)
+          places -= 2;
+        else if (places >= 1)
+          --places;
+        sprintf("%0.*f%s", (int)places, value, siTable[i]);
       }
       break;
-    }
 
     default :
       PAssertAlways(PInvalidParameter);
@@ -1082,20 +1117,24 @@ PString & PString::operator=(unsigned long n)
 }
 
 
-PString & PString::operator=(PInt64 n)
+#ifdef HAVE_LONG_LONG_INT
+PString & PString::operator=(long long n)
 {
-  SetMinSize(sizeof(PInt64)*3+1);
-  m_length = p_signed2string<PInt64, PUInt64>(n, 10, theArray);
+  SetMinSize(sizeof(long long)*3+1);
+  m_length = p_signed2string<signed long long, unsigned long long>(n, 10, theArray);
   return *this;
 }
+#endif
 
 
-PString & PString::operator=(PUInt64 n)
+#ifdef HAVE_UNSIGNED_LONG_LONG_INT
+PString & PString::operator=(unsigned long long n)
 {
-  SetMinSize(sizeof(PUInt64)*3+1);
-  m_length = p_unsigned2string<PUInt64>(n, 10, theArray);
+  SetMinSize(sizeof(unsigned long long)*3+1);
+  m_length = p_unsigned2string<unsigned long long>(n, 10, theArray);
   return *this;
 }
+#endif
 
 
 void PString::AssignContents(const PContainer & cont)
@@ -1159,14 +1198,26 @@ PObject::Comparison PString::Compare(const PObject & obj) const
 
 PINDEX PString::HashFunction() const
 {
-  // Hash function from "Data Structures and Algorithm Analysis in C++" by
-  // Mark Allen Weiss, with limit of only executing over first 8 characters to
-  // increase speed when dealing with large strings.
+  /* Hash function from "Data Structures and Algorithm Analysis in C++" by
+     Mark Allen Weiss, with limit of only executing over at most the first and
+     last characters to increase speed when dealing with large strings. */
 
+  switch (GetLength()) { // Use virtual function so PStringStream recalculates length
+    case 0:
+      return 0;
+    case 1:
+      return tolower(theArray[0] & 0xff) % 127;
+  }
+
+  static const PINDEX MaxCount = 18; // Make sure big enough to cover whole PGloballyUniqueID::AsString()
+  PINDEX count = std::min(m_length / 2, MaxCount);
   PINDEX hash = 0;
-  for (PINDEX i = 0; i < 8 && theArray[i] != 0; i++)
+  PINDEX i;
+  for (i = 0; i < count; i++)
     hash = (hash << 5) ^ tolower(theArray[i] & 0xff) ^ hash;
-  return PABSINDEX(hash)%127;
+  for (i = m_length - count - 1; i < m_length; i++)
+    hash = (hash << 5) ^ tolower(theArray[i] & 0xff) ^ hash;
+  return PABSINDEX(hash) % 127;
 }
 
 
@@ -1184,7 +1235,7 @@ PBoolean PString::SetSize(PINDEX newSize)
   if (!InternalSetSize(newSize, !IsUnique()))
     return false;
 
-  if (m_length >= newSize) {
+  if (GetLength() >= newSize) {
     m_length = newSize-1;
     theArray[m_length] = '\0';
   }
@@ -1329,16 +1380,16 @@ PString & PString::operator&=(char ch)
 }
 
 
-void PString::Delete(PINDEX start, PINDEX len)
+PString & PString::Delete(PINDEX start, PINDEX len)
 {
   if (start < 0 || len < 0)
-    return;
+    return *this;
 
   MakeUnique();
 
-  register PINDEX slen = GetLength();
+  PINDEX slen = GetLength();
   if (start > slen)
-    return;
+    return *this;
 
   if (len >= slen - start) {
     theArray[start] = '\0';
@@ -1351,6 +1402,7 @@ void PString::Delete(PINDEX start, PINDEX len)
 
   if (GetSize() > m_length*2)
     PAssert(MakeMinimumSize(m_length), POutOfMemory);
+  return *this;
 }
 
 
@@ -1359,7 +1411,7 @@ PString PString::operator()(PINDEX start, PINDEX end) const
   if (end < 0 || start < 0 || end < start)
     return Empty();
 
-  register PINDEX len = GetLength();
+  PINDEX len = GetLength();
   if (start > len)
     return Empty();
 
@@ -1493,7 +1545,7 @@ PINDEX PString::Find(char ch, PINDEX offset) const
   if (offset < 0)
     return P_MAX_INDEX;
 
-  register PINDEX len = GetLength();
+  PINDEX len = GetLength();
   while (offset < len) {
     if (InternalCompare(offset, ch) == EqualTo)
       return offset;
@@ -1684,12 +1736,10 @@ PBoolean PString::MatchesRegEx(const PRegularExpression & regex) const
   return (pos == 0) && (len == GetLength());
 }
 
-void PString::Replace(const PString & target,
-                      const PString & subs,
-                      PBoolean all, PINDEX offset)
+PString & PString::Replace(const PString & target, const PString & subs, PBoolean all, PINDEX offset)
 {
   if (offset < 0)
-    return;
+    return *this;
     
   MakeUnique();
 
@@ -1698,54 +1748,69 @@ void PString::Replace(const PString & target,
   do {
     PINDEX pos = Find(target, offset);
     if (pos == P_MAX_INDEX)
-      return;
+      return *this;
     Splice(subs, pos, tlen);
     offset = pos + slen;
   } while (all);
+
+  return *this;
 }
 
 
-void PString::Splice(const char * cstr, PINDEX pos, PINDEX len)
+PString & PString::Splice(const char * cstr, PINDEX pos, PINDEX len)
 {
   if (len < 0 || pos < 0)
-    return;
+    return *this;
 
-  register PINDEX slen = GetLength();
+  PINDEX slen = GetLength();
   if (pos >= slen)
-    operator+=(cstr);
-  else {
-    MakeUnique();
-    if (len > slen-pos)
-      len = slen-pos;
-    PINDEX clen = cstr != NULL ? strlen(cstr) : 0;
-    PINDEX newlen = slen-len+clen;
-    if (clen > len)
-      SetMinSize(newlen+1);
-    if (pos+len < slen)
-      memmove(theArray+pos+clen, theArray+pos+len, slen-pos-len+1);
-    if (clen > 0)
-      memcpy(theArray+pos, cstr, clen);
-    theArray[newlen] = '\0';
-    m_length = newlen;
-  }
+    return operator+=(cstr);
+
+  MakeUnique();
+
+  if (len > slen-pos)
+    len = slen-pos;
+  PINDEX clen = cstr != NULL ? strlen(cstr) : 0;
+  PINDEX newlen = slen-len+clen;
+  if (clen > len)
+    SetMinSize(newlen+1);
+  if (pos+len < slen)
+    memmove(theArray+pos+clen, theArray+pos+len, slen-pos-len+1);
+  if (clen > 0)
+    memcpy(theArray+pos, cstr, clen);
+  theArray[newlen] = '\0';
+  m_length = newlen;
+
+  return *this;
 }
 
 
-bool PString::Split(char delimiter, PString & before, PString & after, bool trim) const
+bool PString::InternalSplit(const PString & delimiter, PString & before, PString & after, SplitOptions_Bits options) const
 {
   PINDEX pos = Find(delimiter);
-  if (pos == P_MAX_INDEX)
-    return false;
 
-  if (trim) {
-    before = Left(pos).Trim();
-    after = Mid(pos+1).Trim();
+  if (pos != P_MAX_INDEX) {
+    before = (options&SplitTrimBefore) ? Left(pos).Trim() : Left(pos);
+    pos += delimiter.GetLength();
+    after = (options&SplitTrimAfter) ? Mid(pos).Trim() : Mid(pos);
   }
   else {
-    before = Left(pos);
-    after = Mid(pos+1);
+    if (!(options&(SplitDefaultToBefore|SplitDefaultToAfter)))
+      return false;
+
+    if (options&SplitDefaultToBefore)
+      before = (options&SplitTrimBefore) ? Trim() : *this;
+
+    if (options&SplitDefaultToAfter)
+      after = (options&SplitTrimBefore) ? Trim() : *this;
   }
 
+  if (before.IsEmpty() && (options&SplitBeforeNonEmpty))
+    return false;
+
+  if (after.IsEmpty() && (options&SplitAfterNonEmpty))
+    return false;
+
   return true;
 }
 
@@ -1819,6 +1884,9 @@ PStringArray PString::Lines() const
 
 PString PString::LeftTrim() const
 {
+  if (IsEmpty())
+    return PString::Empty();
+
   const char * lpos = theArray;
   while (isspace(*lpos & 0xff))
     lpos++;
@@ -1828,6 +1896,9 @@ PString PString::LeftTrim() const
 
 PString PString::RightTrim() const
 {
+  if (IsEmpty())
+    return PString::Empty();
+
   char * rpos = theArray+GetLength()-1;
   if (!isspace(*rpos & 0xff))
     return *this;
@@ -1846,6 +1917,9 @@ PString PString::RightTrim() const
 
 PString PString::Trim() const
 {
+  if (IsEmpty())
+    return PString::Empty();
+
   const char * lpos = theArray;
   while (isspace(*lpos & 0xff))
     lpos++;
@@ -1942,7 +2016,7 @@ PWCharArray PString::AsUCS2() const
   // Note that MB_ERR_INVALID_CHARS is the only dwFlags value supported by Code page 65001 (UTF-8). Windows XP and later.
   PINDEX count = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, theArray, GetLength(), NULL, 0);
   if (count > 0 && ucs2.SetSize(count+1)) { // Allow for trailing NULL
-    MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, theArray, m_length, ucs2.GetPointer(), ucs2.GetSize());
+    MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, theArray, GetLength(), ucs2.GetPointer(), ucs2.GetSize());
     return ucs2;
   }
 
@@ -1958,7 +2032,7 @@ PWCharArray PString::AsUCS2() const
   if (ucs2.SetSize(GetSize())) { // Will be at least this big
     PINDEX count = 0;
     PINDEX i = 0;
-    PINDEX length = GetSize(); // Include the trailing '\0'
+    PINDEX length = GetLength()+1; // Include the trailing '\0'
     while (i < length) {
       int c = theArray[i];
       if ((c&0x80) == 0)
@@ -2167,12 +2241,9 @@ PString pvsprintf(const char * fmt, va_list arg)
 
 PBoolean PString::MakeMinimumSize(PINDEX newLength)
 {
-  if (theArray == NULL) {
-    MakeEmpty();
-    return true;
-  }
-
-  m_length = newLength > 0 ? newLength : strlen(theArray);
+  if (newLength == 0 && theArray != NULL)
+    newLength = strlen(theArray);
+  GetPointerAndSetLength(newLength);
   return SetSize(m_length+1);
 }
 
@@ -2280,11 +2351,11 @@ int PStringStream::Buffer::sync()
   return 0;
 }
 
-streambuf::pos_type PStringStream::Buffer::seekoff(off_type off, ios_base::seekdir dir, ios_base::openmode mode)
+streambuf::pos_type PStringStream::Buffer::seekoff(streamoff off, ios_base::seekdir dir, ios_base::openmode mode)
 {
-  off_type len = (off_type)strlen(string);
-  off_type gpos = (off_type)(gptr() - eback());
-  off_type ppos = (off_type)(pptr() - pbase());
+  streamoff len = (streamoff)strlen(string);
+  streamoff gpos = (streamoff)(gptr() - eback());
+  streamoff ppos = (streamoff)(pptr() - pbase());
   char * newgptr;
   char * newpptr;
   switch (dir) {
@@ -2345,37 +2416,40 @@ PStringStream::Buffer::pos_type PStringStream::Buffer::seekpos(pos_type pos, ios
 
 
 PStringStream::PStringStream()
-  : P_DISABLE_MSVC_WARNINGS(4355, P_IOSTREAM(new PStringStream::Buffer(*this, 0)))
+  : P_DISABLE_MSVC_WARNINGS(4355, std::iostream(new PStringStream::Buffer(*this, 0)))
 {
 }
 
 
 PStringStream::PStringStream(PINDEX fixedBufferSize)
-  : P_DISABLE_MSVC_WARNINGS(4355, P_IOSTREAM(new PStringStream::Buffer(*this, fixedBufferSize)))
+  : P_DISABLE_MSVC_WARNINGS(4355, std::iostream(new PStringStream::Buffer(*this, fixedBufferSize)))
 {
 }
 
 
 PStringStream::PStringStream(const PString & str)
   : PString(str)
-  , P_DISABLE_MSVC_WARNINGS(4355, P_IOSTREAM(new PStringStream::Buffer(*this, 0)))
+  , P_DISABLE_MSVC_WARNINGS(4355, std::iostream(new PStringStream::Buffer(*this, 0)))
 {
 }
 
 
 PStringStream::PStringStream(const char * cstr)
   : PString(cstr)
-  , P_DISABLE_MSVC_WARNINGS(4355, P_IOSTREAM(new PStringStream::Buffer(*this, 0)))
+  , P_DISABLE_MSVC_WARNINGS(4355, std::iostream(new PStringStream::Buffer(*this, 0)))
 {
 }
 
 
 PStringStream::~PStringStream()
 {
-  delete (PStringStream::Buffer *)rdbuf();
-#ifndef _WIN32
+  PStringStream::Buffer * buf = dynamic_cast<PStringStream::Buffer *>(rdbuf());
+#if P_HAS_SET_RDBUF
+  set_rdbuf(NULL);
+#elif !defined(_MSC_VER)
   init(NULL);
 #endif
+  delete buf;
 }
 
 
@@ -2436,7 +2510,14 @@ PStringArray::PStringArray(PINDEX count, char const * const * strarr, PBoolean c
 PStringArray::PStringArray(const PString & str)
 {
   SetSize(1);
-  (*theArray)[0] = new PString(str);
+  (*theArray)[0] = str.CloneAs<PString>();
+}
+
+
+PStringArray::PStringArray(const char * cstr)
+{
+  SetSize(1);
+  (*theArray)[0] = new PString(cstr);
 }
 
 
@@ -2444,8 +2525,8 @@ PStringArray::PStringArray(const PStringList & list)
 {
   SetSize(list.GetSize());
   PINDEX count = 0;
-  for (PStringList::const_iterator i = list.begin(); i != list.end(); i++)
-    (*theArray)[count++] = new PString(*i);
+  for (PStringList::const_iterator it = list.begin(); it != list.end(); ++it)
+    (*theArray)[count++] = it->CloneAs<PString>();
 }
 
 
@@ -2453,7 +2534,16 @@ PStringArray::PStringArray(const PSortedStringList & list)
 {
   SetSize(list.GetSize());
   for (PINDEX i = 0; i < list.GetSize(); i++)
-    (*theArray)[i] = new PString(list[i]);
+    (*theArray)[i] = list[i].CloneAs<PString>();
+}
+
+
+PStringArray::PStringArray(const PStringSet & set)
+{
+  SetSize(set.GetSize());
+  PINDEX count = 0;
+  for (PStringSet::const_iterator it = set.begin(); it != set.end(); ++it)
+    (*theArray)[count++] = it->CloneAs<PString>();
 }
 
 
@@ -2534,6 +2624,19 @@ char ** PStringArray::ToCharArray(PCharArray * storage) const
 }
 
 
+PString PStringArray::ToString(char separator) const
+{
+  PStringStream str;
+
+  for (PINDEX i = 0; i < GetSize(); ++i) {
+    if (i > 0)
+      str << separator;
+    str << (*this)[i];
+  }
+
+  return str;
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -2558,23 +2661,37 @@ PStringList::PStringList(PINDEX count, char const * const * strarr, PBoolean cas
 
 PStringList::PStringList(const PString & str)
 {
-  AppendString(str);
+  Append(str.Clone());
+}
+
+
+PStringList::PStringList(const char * cstr)
+{
+  AppendString(cstr);
 }
 
 
 PStringList::PStringList(const PStringArray & array)
 {
   for (PINDEX i = 0; i < array.GetSize(); i++)
-    AppendString(array[i]);
+    Append(array[i].Clone());
 }
 
 
 PStringList::PStringList(const PSortedStringList & list)
 {
-  for (PINDEX i = 0; i < list.GetSize(); i++)
-    AppendString(list[i]);
+  for (PSortedStringList::const_iterator it = list.begin(); it != list.end(); ++it)
+    Append(it->Clone());
 }
 
+
+PStringList::PStringList(const PStringSet & set)
+{
+  for (PStringSet::const_iterator it = set.begin(); it != set.end(); ++it)
+    Append(it->Clone());
+}
+
+
 PStringList & PStringList::operator += (const PStringList & v)
 {
   for (PStringList::const_iterator i = v.begin(); i != v.end(); i++)
@@ -2619,24 +2736,36 @@ PSortedStringList::PSortedStringList(PINDEX count,
 
 PSortedStringList::PSortedStringList(const PString & str)
 {
-  AppendString(str);
+  Append(str.Clone());
+}
+
+
+PSortedStringList::PSortedStringList(const char * cstr)
+{
+  AppendString(cstr);
 }
 
 
 PSortedStringList::PSortedStringList(const PStringArray & array)
 {
   for (PINDEX i = 0; i < array.GetSize(); i++)
-    AppendString(array[i]);
+    Append(array[i].Clone());
 }
 
 
 PSortedStringList::PSortedStringList(const PStringList & list)
 {
-  for (PStringList::const_iterator i = list.begin(); i != list.end(); i++)
-    AppendString(*i);
+  for (PStringList::const_iterator it = list.begin(); it != list.end(); ++it)
+    Append(it->Clone());
 }
 
 
+PSortedStringList::PSortedStringList(const PStringSet & set)
+{
+  for (PStringSet::const_iterator it = set.begin(); it != set.end(); ++it)
+    Append(it->Clone());
+}
+
 
 void PSortedStringList::ReadFrom(istream & strm)
 {
@@ -2718,6 +2847,13 @@ PStringSet::PStringSet(const PString & str)
 }
 
 
+PStringSet::PStringSet(const char * cstr)
+  : BaseClass(true)
+{
+  Include(cstr);
+}
+
+
 PStringSet::PStringSet(const PStringArray & strArray)
   : BaseClass(true)
 {
@@ -2786,6 +2922,20 @@ PStringToOrdinal::PStringToOrdinal(PINDEX count,
 }
 
 
+PStringToOrdinal::PStringToOrdinal(PINDEX count,
+                                   const POrdinalToString::Initialiser * init,
+                                   PBoolean caseless)
+{
+  while (count-- > 0) {
+    if (caseless)
+      SetAt(PCaselessString(init->value), init->key);
+    else
+      SetAt(init->value, init->key);
+    init++;
+  }
+}
+
+
 void PStringToOrdinal::ReadFrom(istream & strm)
 {
   while (strm.good()) {
@@ -2832,10 +2982,13 @@ void PStringToString::ReadFrom(istream & strm)
       continue;
 
     PString key, value;
-    if (str.Split('=', key, value))
-      SetAt(key, value);
+    str.Split('=', key, value, PString::SplitDefaultToBefore);
+
+    PString * ptr = GetAt(key);
+    if (ptr != NULL)
+      *ptr += '\n' + value;
     else
-      SetAt(str, PString::Empty());
+      SetAt(key, value);
   }
 }
 
@@ -2849,6 +3002,18 @@ void PStringToString::FromString(const PString & str)
 }
 
 
+void PStringToString::Merge(const PStringToString & other, MergeAction action)
+{
+  for (const_iterator it = other.begin(); it != other.end(); ++it) {
+    PString * str = GetAt(it->first);
+    if (str == NULL || action == e_MergeOverwrite)
+      SetAt(it->first, it->second);
+    else if (action == e_MergeAppend)
+      *str += '\n' + it->second;
+  }
+}
+
+
 char ** PStringToString::ToCharArray(bool withEqualSign, PCharArray * storage) const
 {
   const_iterator it;
@@ -3209,7 +3374,7 @@ void PPrintEnum(std::ostream & strm, int value, int begin, int end, char const *
 }
 
 
-int PReadEnum(std::istream & strm, int begin, int end, char const * const * names)
+int PReadEnum(std::istream & strm, int begin, int end, char const * const * names, bool matchCase)
 {
   char name[100]; // If someone has an enumeration longer than this, it deserves to fail!
   strm >> ws;
@@ -3217,15 +3382,25 @@ int PReadEnum(std::istream & strm, int begin, int end, char const * const * name
   if (strm.fail() || strm.bad())
     return end;
 
+  size_t len = strlen(name);
+  int match = end;
   for (int value = begin; value < end; ++value) {
-    if (strcmp(name, names[value-begin]) == 0)
-      return value;
+    const char * cmp = names[value-begin];
+    if ((matchCase ? strncmp(name, cmp, len) : strncasecmp(name, cmp, len)) == 0) {
+      if (match < end) {
+        match = end; // Not unique for the length
+        break;
+      }
+      match = value;
+    }
   }
 
-  size_t i = strlen(name);
+  if (match < end)
+    return match;
+
   do {
-    strm.putback(name[--i]);
-  } while (i > 0);
+    strm.putback(name[--len]);
+  } while (len > 0);
 
   strm.clear();
   strm.setstate(ios::failbit);
@@ -3234,6 +3409,14 @@ int PReadEnum(std::istream & strm, int begin, int end, char const * const * name
 }
 
 
+int PParseEnum(const char * str, int begin, int end, char const * const * names, bool matchCase)
+{
+  std::stringstream strm;
+  strm.str(str);
+  return PReadEnum(strm, begin, end, names, matchCase);
+}
+
+
 void PPrintBitwiseEnum(std::ostream & strm, unsigned bits, char const * const * names)
 {
   if (bits == 0) {
@@ -3241,8 +3424,33 @@ void PPrintBitwiseEnum(std::ostream & strm, unsigned bits, char const * const *
     return;
   }
 
-  bool needSpace = false;
   ++names;
+
+  std::streamsize width = strm.width();
+  if (width > 0) {
+    std::streamsize len = 0;
+    unsigned bit = 1;
+    const char * const * name = names;
+    while (*name != NULL) {
+      if (bits & bit) {
+        if (len > 0)
+          ++len;
+        len += strlen(*name);
+      }
+      bit <<= 1;
+      ++name;
+    }
+    if (width > len)
+      width -= len;
+    else
+      width = 0;
+    strm.width(0);
+  }
+
+  if (width > 0 && (strm.flags()&ios::left) == 0)
+    strm << setw(width) << " ";
+
+  bool needSpace = false;
   for (unsigned bit = 1; *names != NULL; bit <<= 1, ++names) {
     if (bits & bit) {
       if (needSpace)
@@ -3252,13 +3460,16 @@ void PPrintBitwiseEnum(std::ostream & strm, unsigned bits, char const * const *
       strm << *names;
     }
   }
+
+  if (width > 0 && (strm.flags()&ios::right) == 0)
+    strm << setw(width) << " ";
 }
 
 
-unsigned PReadBitwiseEnum(std::istream & strm, char const * const * names)
+unsigned PReadBitwiseEnum(std::istream & strm, char const * const * names, bool continueOnError)
 {
   unsigned bits = 0;
-  while (strm.good()) {
+  while ((continueOnError || strm.good()) && !strm.eof()) {
     char name[100]; // If someone has an enumeration longer than this, it deserves to fail!
     strm >> ws;
     strm.get(name, sizeof(name), ' ');
@@ -3277,6 +3488,9 @@ unsigned PReadBitwiseEnum(std::istream & strm, char const * const * names)
       }
     }
 
+    if (continueOnError)
+      continue;
+
     if (unknown) {
       size_t i = strlen(name);
       do {
