@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28941 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 33765 $
+ * $Author: rjongbloed $
+ * $Date: 2015-07-27 09:43:44 +0100 (Mon, 27 Jul 2015) $
  */
 
 #include <ptlib.h>
@@ -102,20 +102,11 @@ streambuf::int_type PChannelStreamBuffer::underflow()
 
 int PChannelStreamBuffer::sync()
 {
-  if (egptr() > gptr()) {
-    setg(eback(), egptr(), egptr());
-    if (PIsDescendant(channel, PFile))
-      ((PFile *)channel)->SetPosition((off_t)(gptr() - egptr()), PFile::Current);
-  }
-
-  if (pptr() > pbase())
-    return overflow();
-
-  return 0;
+  return pptr() > pbase() ? overflow() : 0;
 }
 
 
-PChannelStreamBuffer::pos_type PChannelStreamBuffer::seekoff(off_type off, ios_base::seekdir dir, ios_base::openmode)
+PChannelStreamBuffer::pos_type PChannelStreamBuffer::seekoff(std::streamoff off, ios_base::seekdir dir, ios_base::openmode)
 {
   sync();
   if (PIsDescendant(channel, PFile)) {
@@ -148,7 +139,7 @@ PChannelStreamBuffer::pos_type PChannelStreamBuffer::seekpos(pos_type pos, ios_b
 
 
 PChannel::PChannel()
-  : P_DISABLE_MSVC_WARNINGS(4355, P_IOSTREAM(new PChannelStreamBuffer(this)))
+  : P_DISABLE_MSVC_WARNINGS(4355, std::iostream(new PChannelStreamBuffer(this)))
   , readTimeout(PMaxTimeInterval)
   , writeTimeout(PMaxTimeInterval)
 {
@@ -162,12 +153,13 @@ PChannel::PChannel()
 
 PChannel::~PChannel()
 {
-  flush();
-  Close();
-  delete (PChannelStreamBuffer *)rdbuf();
-#ifndef _WIN32
+  PChannelStreamBuffer * buf = dynamic_cast<PChannelStreamBuffer *>(rdbuf());
+#if P_HAS_SET_RDBUF
+  set_rdbuf(NULL);
+#elif !defined(_MSC_VER)
   init(NULL);
 #endif
+  delete buf;
 }
 
 PObject::Comparison PChannel::Compare(const PObject & obj) const
@@ -189,6 +181,25 @@ PINDEX PChannel::HashFunction() const
 }
 
 
+bool PChannel::CheckNotOpen()
+{
+  if (IsOpen())
+    return false;
+
+  for (size_t i = 0; i <= NumErrorGroups; ++i) {
+    lastErrorCode[i] = NotOpen;
+    lastErrorNumber[i] = EBADF;
+  }
+  return true;
+}
+
+
+int PChannel::os_errno() const
+{
+  return errno;
+}
+
+
 PBoolean PChannel::IsOpen() const
 {
   return os_handle != -1;
@@ -508,16 +519,6 @@ PString PChannel::GetErrorText(ErrorGroup group) const
 }
 
 
-PBoolean PChannel::ConvertOSError(P_INT_PTR status, ErrorGroup group)
-{
-  Errors lastError;
-  int osError;
-  PBoolean ok = ConvertOSError(status, lastError, osError);
-  SetErrorValues(lastError, osError, group);
-  return ok;
-}
-
-
 PBoolean PChannel::SetErrorValues(Errors errorCode, int errorNum, ErrorGroup group)
 {
   lastErrorCode[NumErrorGroups] = lastErrorCode[group] = errorCode;
@@ -535,7 +536,7 @@ PChannel::AsyncContext::AsyncContext(void * buf, PINDEX len, const AsyncNotifier
   , m_channel(NULL)
   , m_onComplete(NULL)
 {
-  memset(this, 0, sizeof(AsyncContextBase));
+  memset(static_cast<AsyncContextBase*>(this), 0, sizeof(AsyncContextBase));
 }
 
 
@@ -544,11 +545,14 @@ void PChannel::AsyncContext::OnIOComplete(PINDEX length, int errorNumber)
   PTRACE(6, m_channel, "AsyncIO", "OnIOComplete: len=" << length << ", error=" << errorNumber);
 
   m_length = length;
-  m_errorNumber = errorNumber;
-  PChannel::ConvertOSError(-3, m_errorCode, m_errorNumber);
 
   PChannel * channel = m_channel;
   m_channel = NULL;
+
+  channel->SetErrorValues(PChannel::Miscellaneous, errorNumber);
+  channel->ConvertOSError(-3);
+  m_errorCode = channel->GetErrorCode();
+
   (channel->*m_onComplete)(*this);
 }
 
@@ -631,6 +635,20 @@ PString PIndirectChannel::GetName() const
 }
 
 
+P_INT_PTR PIndirectChannel::GetHandle() const
+{
+  PReadWaitAndSignal mutex(channelPointerMutex);
+
+  if (readChannel != NULL)
+    return readChannel->GetHandle();
+
+  if (writeChannel != NULL)
+    return writeChannel->GetHandle();
+
+  return -1;
+}
+
+
 PBoolean PIndirectChannel::Close()
 {
   PBoolean retval = true;
@@ -685,8 +703,6 @@ PBoolean PIndirectChannel::IsOpen() const
 
 PBoolean PIndirectChannel::Read(void * buf, PINDEX len)
 {
-  flush();
-
   PReadWaitAndSignal mutex(channelPointerMutex);
 
   if (readChannel == NULL) {
@@ -706,6 +722,27 @@ PBoolean PIndirectChannel::Read(void * buf, PINDEX len)
 }
 
 
+int PIndirectChannel::ReadChar()
+{
+  PReadWaitAndSignal mutex(channelPointerMutex);
+
+  if (readChannel == NULL) {
+    SetErrorValues(NotOpen, EBADF, LastReadError);
+    return -1;
+  }
+
+  readChannel->SetReadTimeout(readTimeout);
+  int returnValue = readChannel->ReadChar();
+
+  SetErrorValues(readChannel->GetErrorCode(LastReadError),
+                 readChannel->GetErrorNumber(LastReadError),
+                 LastReadError);
+  lastReadCount = readChannel->GetLastReadCount();
+
+  return returnValue;
+}
+
+
 PBoolean PIndirectChannel::Write(const void * buf, PINDEX len)
 {
   flush();
@@ -786,23 +823,8 @@ PBoolean PIndirectChannel::Open(PChannel * readChan,
 
   channelPointerMutex.StartWrite();
 
-  if (readChannel != NULL)
-    readChannel->Close();
-
-  if (readChannel != writeChannel && writeChannel != NULL)
-    writeChannel->Close();
-
-  if (readAutoDelete)
-    delete readChannel;
-
-  if (readChannel != writeChannel && writeAutoDelete)
-    delete writeChannel;
-
-  readChannel = readChan;
-  readAutoDelete = autoDeleteRead;
-
-  writeChannel = writeChan;
-  writeAutoDelete = autoDeleteWrite;
+  SetReadChannel(readChan, autoDeleteRead, true);
+  SetWriteChannel(writeChan, autoDeleteWrite, true);
 
   channelPointerMutex.EndWrite();
 
@@ -816,13 +838,43 @@ PBoolean PIndirectChannel::OnOpen()
 }
 
 
+PChannel * PIndirectChannel::Detach(ShutdownValue option)
+{
+  PWriteWaitAndSignal mutex(channelPointerMutex);
+
+  PChannel * channel;
+  switch (option) {
+    case ShutdownRead :
+      channel = readChannel;
+      readChannel = NULL;
+      break;
+
+    case ShutdownWrite :
+      channel = writeChannel;
+      writeChannel = NULL;
+      break;
+
+    default :
+      if (readChannel != writeChannel)
+        return NULL;
+
+      channel = readChannel;
+      readChannel = writeChannel = NULL;
+  }
+
+  return channel;
+}
+
+
 bool PIndirectChannel::SetReadChannel(PChannel * channel, bool autoDelete, bool closeExisting)
 {
   PWriteWaitAndSignal mutex(channelPointerMutex);
 
   if (closeExisting) {
-    if (readAutoDelete)
+    if (readAutoDelete && readChannel != NULL && readChannel != writeChannel) {
+      readChannel->Close();
       delete readChannel;
+    }
   }
   else {
     if (readChannel != NULL)
@@ -831,6 +883,8 @@ bool PIndirectChannel::SetReadChannel(PChannel * channel, bool autoDelete, bool
 
   readChannel = channel;
   readAutoDelete = autoDelete;
+  if (readChannel != NULL)
+    SetReadTimeout(readChannel->GetReadTimeout());
 
   return channel != NULL && channel->IsOpen();
 }
@@ -841,8 +895,10 @@ bool PIndirectChannel::SetWriteChannel(PChannel * channel, bool autoDelete, bool
   PWriteWaitAndSignal mutex(channelPointerMutex);
 
   if (closeExisting) {
-    if (writeAutoDelete)
+    if (writeAutoDelete && writeChannel != NULL && writeChannel != readChannel) {
+      writeChannel->Close();
       delete writeChannel;
+    }
   }
   else {
     if (writeChannel != NULL)
@@ -851,6 +907,8 @@ bool PIndirectChannel::SetWriteChannel(PChannel * channel, bool autoDelete, bool
 
   writeChannel = channel;
   writeAutoDelete = autoDelete;
+  if (writeChannel != NULL)
+    SetWriteTimeout(writeChannel->GetWriteTimeout());
 
   return channel != NULL && channel->IsOpen();
 }
@@ -882,26 +940,38 @@ PFile::~PFile()
 PObject::Comparison PFile::Compare(const PObject & obj) const
 {
   PAssert(PIsDescendant(&obj, PFile), PInvalidCast);
-  return path.Compare(((const PFile &)obj).path);
+  return m_path.Compare(((const PFile &)obj).m_path);
 }
 
 
-PBoolean PFile::Rename(const PString & newname, PBoolean force)
+bool PFile::Rename(const PString & newname, bool force)
 {
   Close();
 
-  if (!ConvertOSError(Rename(path, newname, force) ? 0 : -1))
+  if (!ConvertOSError(Rename(m_path, newname, force) ? 0 : -1))
     return false;
 
-  path = path.GetDirectory() + newname;
+  m_path = m_path.GetDirectory() + newname;
+  return true;
+}
+
+
+bool PFile::Move(const PFilePath & newname, bool force, bool recurse)
+{
+  Close();
+
+  if (!ConvertOSError(Move(m_path, newname, force, recurse) ? 0 : -1))
+    return false;
+
+  m_path = newname;
   return true;
 }
 
 
 PBoolean PFile::Close()
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
   flush();
 
@@ -913,7 +983,7 @@ PBoolean PFile::Close()
 
   os_handle = -1;
 
-  if (removeOnClose)
+  if (m_removeOnClose)
     Remove();
 
   return ok;
@@ -922,10 +992,9 @@ PBoolean PFile::Close()
 
 PBoolean PFile::Read(void * buffer, PINDEX amount)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
-  flush();
 #ifdef WOT_NO_FILESYSTEM
   lastReadCount = 0;
 #else
@@ -937,8 +1006,8 @@ PBoolean PFile::Read(void * buffer, PINDEX amount)
 
 PBoolean PFile::Write(const void * buffer, PINDEX amount)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
   flush();
 #ifdef WOT_NO_FILESYSTEM
@@ -950,11 +1019,32 @@ PBoolean PFile::Write(const void * buffer, PINDEX amount)
 }
 
 
-PBoolean PFile::Open(const PFilePath & name, OpenMode  mode, OpenOptions opts)
+bool PFile::Open(const PFilePath & name, OpenMode  mode, OpenOptions opts)
 {
   Close();
   SetFilePath(name);
-  return Open(mode, opts);
+  return InternalOpen(mode, opts, PFileInfo::DefaultPerms);
+}
+
+
+bool PFile::Open(const PFilePath & name, OpenMode  mode, OpenOptions opts, PFileInfo::Permissions permissions)
+{
+  Close();
+  SetFilePath(name);
+  return InternalOpen(mode, opts, permissions);
+}
+
+
+bool PFile::Open(OpenMode mode, OpenOptions opts)
+{
+  Close();
+  return InternalOpen(mode, opts, PFileInfo::DefaultPerms);
+}
+
+
+bool PFile::Open(OpenMode mode, OpenOptions opts, PFileInfo::Permissions permissions)
+{
+  return InternalOpen(mode, opts, permissions);
 }
 
 
@@ -974,7 +1064,7 @@ off_t PFile::GetLength() const
 }
 
 
-PBoolean PFile::IsEndOfFile() const
+bool PFile::IsEndOfFile() const
 {
   if (!IsOpen())
     return true;
@@ -989,22 +1079,26 @@ PBoolean PFile::SetPosition(off_t pos, FilePositionOrigin origin)
 #ifdef WOT_NO_FILESYSTEM
   return true;
 #else
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
   return _lseek(GetOSHandleAsInt(), pos, origin) != (off_t)-1;
 #endif
 }
 
 
-PBoolean PFile::Copy(const PFilePath & oldname, const PFilePath & newname, PBoolean force)
+bool PFile::Copy(const PFilePath & oldname, const PFilePath & newname, bool force, bool recurse)
 {
   PFile oldfile(oldname, ReadOnly);
   if (!oldfile.IsOpen())
     return false;
 
-  PFile newfile(newname,
-                   WriteOnly, Create|Truncate|(force ? MustExist : Exclusive));
+  if (recurse && !newname.GetDirectory().Exists()) {
+    if (!newname.GetDirectory().Create(PFileInfo::DefaultDirPerms, true))
+      return false;
+  }
+
+  PFile newfile(newname, WriteOnly, Create|Truncate|(force ? MustExist : Exclusive));
   if (!newfile.IsOpen())
     return false;
 
@@ -1028,6 +1122,50 @@ PBoolean PFile::Copy(const PFilePath & oldname, const PFilePath & newname, PBool
 }
 
 
+bool PFile::Rename(const PFilePath & oldname, const PString & newname, bool force)
+{
+  for (PINDEX i = 0; i< newname.GetLength(); ++i) {
+    if (PDirectory::IsSeparator(newname[i])) {
+#ifdef _WIN32_WCE
+      set_errno(EINVAL);
+#else
+      errno = EINVAL;
+#endif
+      return false;
+    }
+  }
+
+  return Move(oldname, PFilePath(oldname.GetDirectory() + newname), force);
+}
+
+
+bool PFile::Move(const PFilePath & oldname, const PFilePath & newname, bool force, bool recurse)
+{
+  if (rename(oldname, newname) == 0)
+    return true;
+
+  if (errno == ENOENT) {
+    if (!recurse)
+      return false;
+
+    if (!newname.GetDirectory().Create(PFileInfo::DefaultDirPerms, true))
+      return false;
+
+    return rename(oldname, newname) == 0;
+  }
+
+  if (force && Exists(newname)) {
+    if (!Remove(newname, true))
+      return false;
+
+    if (rename(oldname, newname) == 0)
+      return true;
+  }
+
+  return Copy(oldname, newname, force) && Remove(oldname);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PTextFile
 
