@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29337 $
+ * $Revision: 34159 $
  * $Author: rjongbloed $
- * $Date: 2013-03-26 23:04:03 -0500 (Tue, 26 Mar 2013) $
+ * $Date: 2015-10-07 17:13:31 +0100 (Wed, 07 Oct 2015) $
  */
 
 #include <ptlib.h>
@@ -92,18 +92,35 @@ PStringArray PEthSocket::EnumInterfaces(bool detailed)
   pcap_if_t *alldevs;
   char errbuf[PCAP_ERRBUF_SIZE];
   if (pcap_findalldevs(&alldevs, errbuf) == -1) {
-    PTRACE(1, NULL, PTraceModule(), "Could not enumerate interfaces, error: " << errbuf);
+    PTRACE(1, &interfaces, "Could not enumerate interfaces, error: " << errbuf);
   }
   else {
     for (pcap_if_t * dev = alldevs; dev != NULL; dev = dev->next) {
       PStringStream strm;
       strm << dev->name;
-      if (detailed)
+      if (detailed) {
         strm << '\t' << dev->description;
+#if _WIN32
+        static const char DevPrefix[] = "\\Device\\NPF_";
+        if (strncmp(dev->name, DevPrefix, sizeof(DevPrefix)-1) == 0) {
+          PConfig registry("HKEY_LOCAL_MACH64\\"
+                           "SYSTEM\\"
+                           "CurrentControlSet\\"
+                           "Control\\"
+                           "Network\\"
+                           "{4D36E972-E325-11CE-BFC1-08002BE10318}\\" +
+                           PConstString(dev->name+sizeof(DevPrefix)-1) + "\\"
+                           "Connection");
+          PString name = registry.GetString("Name");
+          if (!name.IsEmpty())
+            strm << '\t' << name;
+        }
+#endif
+      }
       interfaces += strm;
     }
     pcap_freealldevs(alldevs);
-    PTRACE(4, NULL, PTraceModule(), "Enumerated interfaces:\n" << setfill('\n') << interfaces);
+    PTRACE(4, &interfaces, "Enumerated interfaces:\n" << setfill('\n') << interfaces);
   }
 
   return interfaces;
@@ -183,8 +200,8 @@ bool PEthSocket::SetFilter(const PString & filter)
 
 PBoolean PEthSocket::Read(void * data, PINDEX length)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF, LastReadError);
+  if (CheckNotOpen())
+    return false;
 
   struct pcap_pkthdr *header;
   const u_char *pkt_data;
@@ -193,6 +210,7 @@ PBoolean PEthSocket::Read(void * data, PINDEX length)
     case 1 :
       memcpy(data, pkt_data, std::min(length, (PINDEX)header->caplen));
       lastReadCount = header->caplen;
+      m_lastPacketTime = PTime(header->ts.tv_sec, header->ts.tv_usec);
       return true;
 
     case 0 :
@@ -211,7 +229,7 @@ PBoolean PEthSocket::Read(void * data, PINDEX length)
 PBoolean PEthSocket::Write(const void * data, PINDEX length)
 {
   if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF, LastWriteError);
+    return false;
 
   if (!ConvertOSError(pcap_sendpacket(m_internal->m_pcap, (u_char *)data, length), LastWriteError)) {
     PTRACE(2, "Write error: " << pcap_geterr(m_internal->m_pcap));
@@ -280,6 +298,7 @@ PEthSocket::PEthSocket(bool promiscuous, unsigned snapLength)
   : m_promiscuous(promiscuous)
   , m_snapLength(snapLength)
   , m_internal(new InternalData)
+  , m_lastPacketTime(0)
 {
   memset(m_internal, 0, sizeof(InternalData));
 }
@@ -324,13 +343,37 @@ PEthSocket::Frame::Frame(PINDEX maxSize)
 }
 
 
-bool PEthSocket::Frame::Read(PChannel & channel, PINDEX packetSize)
+PEthSocket::Frame::Frame(const Frame & frame)
+  : m_rawData(frame.m_rawData, frame.m_rawSize) // Make copy, not reference
+  , m_rawSize(frame.m_rawSize)
+  , m_fragmentated(false)
+  , m_fragmentProto(0)
+  , m_fragmentProcessed(false)
+  , m_timestamp(frame.m_timestamp)
+{
+  m_rawData.SetMinSize(sizeof(PEthFrameHeader));
+}
+
+
+void PEthSocket::Frame::PreRead()
 {
   if (m_fragmentated) {
     m_fragments.SetSize(0);
     m_fragmentated = false;
   }
   m_fragmentProcessed = false;
+}
+
+
+bool PEthSocket::Frame::Write(PChannel & channel) const
+{
+  return channel.Write(m_rawData, m_rawSize);
+}
+
+
+bool PEthSocket::Frame::Read(PChannel & channel, PINDEX packetSize)
+{
+  PreRead();
 
   PINDEX size = std::min(packetSize, m_rawData.GetSize());
   do {
@@ -339,7 +382,11 @@ bool PEthSocket::Frame::Read(PChannel & channel, PINDEX packetSize)
     m_rawSize = channel.GetLastReadCount();
   } while ((size_t)m_rawSize < sizeof(Address)+sizeof(Address)+sizeof(2));
 
-  m_timestamp.SetCurrentTime();
+  const PEthSocket * ethChannel = dynamic_cast<PEthSocket *>(&channel);
+  if (ethChannel != NULL && ethChannel->GetLastPacketTime().IsValid())
+    m_timestamp = ethChannel->GetLastPacketTime();
+  else
+    m_timestamp.SetCurrentTime();
 
   return true;
 }
@@ -369,7 +416,7 @@ int PEthSocket::Frame::GetDataLink(PBYTEArray & payload, Address & src, Address
   // Ethernet II header
   if (len_or_type > 1500) {
     // Subtract off the Ethernet II header
-    payload.Attach(header.ether.payload, m_rawSize - (sizeof(header.dst_addr)+sizeof(header.src_addr)+sizeof(header.snap.length)));
+    payload.Attach(header.ether.payload, m_rawSize - (sizeof(header.dst_addr)+sizeof(header.src_addr)+sizeof(header.ether.type)));
     return len_or_type;
   }
 
@@ -422,6 +469,17 @@ int PEthSocket::Frame::GetDataLink(PBYTEArray & payload, Address & src, Address
 }
 
 
+BYTE * PEthSocket::Frame::CreateDataLink(const Address & src, const Address & dst, unsigned proto, PINDEX length)
+{
+  m_rawSize = length + 14;
+  PEthFrameHeader & header = *(PEthFrameHeader *)m_rawData.GetPointer(sizeof(PEthFrameHeader));
+  header.src_addr = src;
+  header.dst_addr = dst;
+  header.ether.type = htons((u_short)proto);
+  return header.ether.payload;
+}
+
+
 int PEthSocket::Frame::GetIP(PBYTEArray & payload)
 {
   PIPSocket::Address src, dst;
@@ -447,12 +505,19 @@ int PEthSocket::Frame::GetIP(PBYTEArray & payload, PIPSocket::Address & src, PIP
     return -1;
 
   PINDEX totalLength = (ip[2]<<8)|ip[3]; // Total length of packet
-  if (totalLength > ip.GetSize()) {
+  if (totalLength == 0)
+    totalLength = ip.GetSize(); // presume to be part of TCP segmentation offload (TSO), whatever THAT is
+  else if (totalLength > ip.GetSize()) {
     PTRACE(2, "Truncated IP packet, expected " << totalLength << ", got " << ip.GetSize());
     return -1;
   }
 
   PINDEX headerLength = (ip[0]&0xf)*4; // low 4 bits in DWORDS, is this in bytes
+  if (totalLength < headerLength) {
+    PTRACE(2, "Malformed IP header, length " << totalLength << " smaller than header " << headerLength);
+    return -1;
+  }
+
   payload.Attach(&ip[headerLength], totalLength-headerLength);
 
   src = PIPSocket::Address(4, ip+12);
@@ -471,7 +536,12 @@ int PEthSocket::Frame::GetIP(PBYTEArray & payload, PIPSocket::Address & src, PIP
     if (m_fragmentSrcIP != src || m_fragmentDstIP != dst)
       return ip[9]; // Next protocol layer
 
-    if (fragmentsSize != fragmentOffset) {
+    if (fragmentsSize > fragmentOffset) {
+      PTRACE(5, "Repeated IP fragment at " << fragmentOffset << " on " << src << " -> " << dst);
+      return -1;
+    }
+
+    if (fragmentsSize < fragmentOffset) {
       PTRACE(2, "Missing IP fragment, expected " << fragmentsSize << ", got " << fragmentOffset << " on " << src << " -> " << dst);
       m_fragments.SetSize(0);
       return -1;
@@ -500,6 +570,25 @@ int PEthSocket::Frame::GetIP(PBYTEArray & payload, PIPSocket::Address & src, PIP
 }
 
 
+BYTE * PEthSocket::Frame::CreateIP(const PIPSocket::Address & src, const PIPSocket::Address & dst, unsigned proto, PINDEX length)
+{
+  length += 20;
+
+  PBYTEArray dummy;
+  Address srcMac, dstMac;
+  if ((size_t)m_rawSize > sizeof(PEthSocket::Address)*2)
+    GetDataLink(dummy, srcMac, dstMac);
+  BYTE * ip = CreateDataLink(srcMac, dstMac, 0x800, length);
+  memset(ip, 0, 20);
+  ip[0] = 0x45;
+  *(PUInt16b*)(ip+2) = (uint16_t)length;
+  ip[9] = (uint8_t)proto;
+  *(in_addr *)(ip+12) = src;
+  *(in_addr *)(ip+16) = dst;
+  return ip+20;
+}
+
+
 bool PEthSocket::Frame::GetUDP(PBYTEArray & payload, WORD & srcPort, WORD & dstPort)
 {
   PIPSocketAddressAndPort src, dst;
@@ -534,6 +623,18 @@ bool PEthSocket::Frame::GetUDP(PBYTEArray & payload, PIPSocketAddressAndPort & s
 }
 
 
+BYTE * PEthSocket::Frame::CreateUDP(const PIPSocketAddressAndPort & src, const PIPSocketAddressAndPort & dst, PINDEX length)
+{
+  length += 8;
+  BYTE * udp = CreateIP(src.GetAddress(), dst.GetAddress(), 0x11, length);
+  *(PUInt16b*)(udp+0) = src.GetPort();
+  *(PUInt16b*)(udp+2) = dst.GetPort();
+  *(PUInt16b*)(udp+4) = (uint16_t)length;
+  *(uint16_t*)(udp+6) = 0;
+  return udp+8;
+}
+
+
 bool PEthSocket::Frame::GetTCP(PBYTEArray & payload, WORD & srcPort, WORD & dstPort)
 {
   PIPSocketAddressAndPort src, dst;
@@ -569,6 +670,19 @@ bool PEthSocket::Frame::GetTCP(PBYTEArray & payload, PIPSocketAddressAndPort & s
 }
 
 
+BYTE * PEthSocket::Frame::CreateTCP(const PIPSocketAddressAndPort & src, const PIPSocketAddressAndPort & dst, PINDEX length)
+{
+  length += 20;
+
+  BYTE * tcp = CreateIP(src.GetAddress(), dst.GetAddress(), 6, length);
+  memset(tcp, 0, 20);
+  tcp[12] = 0x50;
+  *(PUInt16b*)(tcp+0) = src.GetPort();
+  *(PUInt16b*)(tcp+2) = dst.GetPort();
+  return tcp+20;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PEthSocket::Address::Address()
@@ -673,7 +787,7 @@ PEthSocketThread::PEthSocketThread(const FrameNotifier & notifier)
 }
 
 
-bool PEthSocketThread::Start(const PString & device, const PString & filter)
+bool PEthSocketThread::Start(const PString & device, const PString & filter, PThread::Priority priority)
 {
   Stop();
 
@@ -687,7 +801,7 @@ bool PEthSocketThread::Start(const PString & device, const PString & filter)
 
   if (m_socket->Connect(device) && m_socket->SetFilter(filter)) {
     m_running = true;
-    m_thread = new PThreadObj<PEthSocketThread>(*this, &PEthSocketThread::MainLoop, false, "Sniffer");
+    m_thread = new PThreadObj<PEthSocketThread>(*this, &PEthSocketThread::MainLoop, false, "Sniffer", priority);
     return true;
   }
 
