@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28931 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 06:31:11 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 34255 $
+ * $Author: rjongbloed $
+ * $Date: 2015-10-30 08:38:03 +0000 (Fri, 30 Oct 2015) $
  */
 
 #include <ptlib.h>
@@ -37,7 +37,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 // PTimeInterval
 
-PTimeInterval::PTimeInterval(long millisecs,
+PTimeInterval::PTimeInterval(PInt64 millisecs,
                              long seconds,
                              long minutes,
                              long hours,
@@ -54,23 +54,12 @@ PTimeInterval::PTimeInterval(const PString & str)
 }
 
 
-PInt64 PTimeInterval::GetMilliSeconds() const
-{ 
-  return m_milliseconds; 
-}
-
-void PTimeInterval::SetMilliSeconds(PInt64 msecs)
-{ 
-  m_milliseconds = msecs;
-}
-
 PObject::Comparison PTimeInterval::Compare(const PObject & obj) const
 {
-  PAssert(PIsDescendant(&obj, PTimeInterval), PInvalidCast);
-  const PTimeInterval & other = (const PTimeInterval &)obj;
-
-  return GetMilliSeconds() < other.GetMilliSeconds() ? LessThan :
-         GetMilliSeconds() > other.GetMilliSeconds() ? GreaterThan : EqualTo;}
+  int64_t otherNano = dynamic_cast<const PTimeInterval &>(obj).GetNanoSeconds();
+  int64_t thisNano = GetNanoSeconds();
+  return thisNano < otherNano ? LessThan : thisNano > otherNano ? GreaterThan : EqualTo;
+}
 
 
 void PTimeInterval::PrintOn(ostream & stream) const
@@ -94,9 +83,9 @@ void PTimeInterval::ReadFrom(istream &strm)
   long day = 0;
   long hour = 0;
   long min = 0;
-  float sec;
+  double sec;
   strm >> sec;
-  while (strm.peek() == ':') {
+  while (strm.good() && strm.peek() == ':') {
     day = hour;
     hour = min;
     min = (long)sec;
@@ -104,7 +93,7 @@ void PTimeInterval::ReadFrom(istream &strm)
     strm >> sec;
   }
 
-  SetInterval(((long)(sec*1000))%1000, (int)sec, min, hour, day);
+  InternalSet((int64_t)((((day*24 + hour)*60 + min)*60 + sec)*1000000000));
 }
 
 
@@ -207,17 +196,27 @@ PString PTimeInterval::AsString(int precision, Formats format, int width) const
 }
 
 
-DWORD PTimeInterval::GetInterval() const
-{
-  PInt64 msecs = GetMilliSeconds();
+#ifdef _WIN32
+static const DWORD TimeIntervalLimit = UINT_MAX;
+static const DWORD TimeIntervalInfinite = INFINITE;
+#else
+static const int TimeIntervalLimit = INT_MAX;
+static const int TimeIntervalInfinite = -1;
+#endif
 
+PTimeInterval::IntervalType PTimeInterval::GetInterval() const
+{
+  if (*this == PMaxTimeInterval)
+    return TimeIntervalInfinite;
+
+  PInt64 msecs = GetMilliSeconds();
   if (msecs <= 0)
     return 0;
 
-  if (msecs >= UINT_MAX)
-    return UINT_MAX;
+  if (msecs >= TimeIntervalLimit)
+    return TimeIntervalLimit;
 
-  return (DWORD)msecs;
+  return (IntervalType)msecs;
 }
 
 
@@ -266,7 +265,14 @@ static time_t p_mktime(struct tm * t, int zone)
 }
 
 
+PTime::PTime(time_t tsecs, int64_t usecs)
+  : m_microSecondsSinceEpoch(tsecs*Micro+usecs)
+{
+}
+
+
 PTime::PTime(const PString & str)
+  : m_microSecondsSinceEpoch(0)
 {
   PStringStream s(str);
   ReadFrom(s);
@@ -277,8 +283,6 @@ PTime::PTime(int second, int minute, int hour,
              int day,    int month,  int year,
              int zone)
 {
-  microseconds = 0;
-
   struct tm t;
   PAssert(second >= 0 && second <= 59, PInvalidParameter);
   t.tm_sec = second;
@@ -293,7 +297,7 @@ PTime::PTime(int second, int minute, int hour,
   PAssert(year >= 1970 && year <= 2038, PInvalidParameter);
   t.tm_year   = year-1900;
 
-  theTime = p_mktime(&t, zone);
+  m_microSecondsSinceEpoch.store(p_mktime(&t, zone)*Micro);
 }
 
 
@@ -301,22 +305,105 @@ PObject::Comparison PTime::Compare(const PObject & obj) const
 {
   PAssert(PIsDescendant(&obj, PTime), PInvalidCast);
   const PTime & other = (const PTime &)obj;
-  if (theTime < other.theTime)
+
+  int64_t myMicroSeconds = m_microSecondsSinceEpoch.load();
+  int64_t otherMicroSeconds = other.m_microSecondsSinceEpoch.load();
+  if (myMicroSeconds < otherMicroSeconds)
     return LessThan;
-  if (theTime > other.theTime)
-    return GreaterThan;
-  if (microseconds < other.microseconds)
-    return LessThan;
-  if (microseconds > other.microseconds)
+  if (myMicroSeconds > otherMicroSeconds)
     return GreaterThan;
+
   return EqualTo;
 }
 
 
-void PTime::SetTimestamp(time_t seconds, long usecs)
-{ 
-  microseconds = usecs%1000000;
-  theTime = seconds + usecs/1000000;
+static PUInt64 const SecondsFrom1900to1970 = (70*365+17)*24*60*60U;
+static PUInt64 const MicrosecondsToFraction = 4294;
+
+void PTime::SetNTP(PUInt64 ntp)
+{
+  m_microSecondsSinceEpoch.store(((ntp>>32) - SecondsFrom1900to1970)*Micro + (ntp&0xffffffff)/MicrosecondsToFraction);
+}
+
+
+PUInt64 PTime::GetNTP() const
+{
+  if (!IsValid())
+    return 0;
+  int64_t usecs = m_microSecondsSinceEpoch.load();
+  return ((usecs/Micro+SecondsFrom1900to1970)<<32) | ((usecs%Micro)*MicrosecondsToFraction);
+}
+
+
+bool PTime::InternalLocalTime(struct tm & ts) const
+{
+  time_t t = m_microSecondsSinceEpoch.load()/Micro;
+  return os_localtime(&t, &ts) != NULL;
+}
+
+
+int PTime::GetSecond() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? ts.tm_sec : -1;
+}
+
+
+int PTime::GetMinute() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? ts.tm_min : -1;
+}
+
+
+int PTime::GetHour() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? ts.tm_hour : -1;
+}
+
+
+int PTime::GetDay() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? ts.tm_mday : -1;
+}
+
+
+PTime::Months PTime::GetMonth() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? (Months)(ts.tm_mon+January) : InvalidMonth;
+}
+
+
+int PTime::GetYear() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? ts.tm_year+1900 : -1;
+}
+
+
+PTime::Weekdays PTime::GetDayOfWeek() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? (Weekdays)ts.tm_wday : InvalidWeekday;
+}
+
+int PTime::GetDayOfYear() const
+{
+  struct tm ts;
+  return InternalLocalTime(ts) ? ts.tm_yday : -1;
+}
+
+
+void PTime::PrintOn(ostream & strm) const
+{
+  int fmt = (int)strm.precision();
+  if (fmt >= 0 || fmt <= -NumTimeStrings)
+    strm << AsString();
+  else
+    strm << AsString((NumTimeStrings)-fmt);
 }
 
 
@@ -325,18 +412,41 @@ PString PTime::AsString(TimeFormat format, int zone) const
   if (format >= NumTimeStrings)
     return "Invalid format : " + AsString("yyyy-MM-dd T hh:mm:ss Z");
 
+  if (!IsValid())
+    return "N/A";
+
   switch (format) {
     case RFC1123 :
       return AsString("wwwe, dd MMME yyyy hh:mm:ss z", zone);
+
     case RFC3339 :
       return AsString("yyyy-MM-ddThh:mm:ssZZ", zone);
+
     case ShortISO8601 :
       return AsString("yyyyMMddThhmmssZ", zone);
+
     case LongISO8601 :
       return AsString("yyyy-MM-dd T hh:mm:ss Z", zone);
-    case EpochTime :
-      return psprintf("%u.%06lu", theTime, microseconds);
-    default :
+
+    case EpochTime:
+    {
+      int64_t usecs = m_microSecondsSinceEpoch.load();
+      return psprintf("%u.%06lu", usecs / Micro, usecs % Micro);
+    }
+
+    case TodayFormat:
+    {
+      PTime now;
+      static const PTimeInterval halfDay(0, 0, 0, 12);
+      if (*this > (now - halfDay) && *this < (now + halfDay))
+        return AsString("hh:mm:ss.uuu");
+      // Do next case
+    }
+
+    case LoggingFormat :
+      return AsString("yyyy/MM/dd hh:mm:ss.uuu", zone);
+
+    default:
       break;
   }
 
@@ -443,23 +553,26 @@ PString PTime::AsString(TimeFormat format, int zone) const
 
 PString PTime::AsString(const char * format, int zone) const
 {
-  if (!IsValid())
-    return PString::Empty();
-
   PAssert(format != NULL, PInvalidParameter);
+  PAssert(zone == Local || std::abs(zone) <= 13, PInvalidParameter);
 
-  PBoolean is12hour = strchr(format, 'a') != NULL;
-
-  PStringStream str;
-  str.fill('0');
+  if (!IsValid())
+    return "<invalid>";
 
   // the localtime call automatically adjusts for daylight savings time
   // so take this into account when converting non-local times
   if (zone == Local)
     zone = GetTimeZone();  // includes daylight savings time
-  time_t realTime = theTime + zone*60;     // to correct timezone
+  time_t realTime = m_microSecondsSinceEpoch.load()/Micro + zone*60;     // to correct timezone
   struct tm ts;
   struct tm * t = os_gmtime(&realTime, &ts);
+  if (t == NULL)
+    return "<error>";
+
+  PStringStream str;
+  str.fill('0');
+
+  bool is12hour = strchr(format, 'a') != NULL;
 
   while (*format != '\0') {
     char formatLetter = *format;
@@ -528,12 +641,10 @@ PString PTime::AsString(const char * format, int zone) const
 
       case 'z' :
       case 'Z' :
-        if (repeatCount == 1 && zone == 0) {
-          if (formatLetter == 'Z')
-            str << 'Z';
-          else
-            str << "GMT";
-        }
+        if (zone == 0 && formatLetter == 'Z')
+          str << 'Z';
+        else if (zone == 0 && repeatCount == 1)
+          str << "GMT";
         else {
           str << (zone < 0 ? '-' : '+');
           zone = PABS(zone);
@@ -544,22 +655,25 @@ PString PTime::AsString(const char * format, int zone) const
         }
         break;
 
-      case 'u' :
+      case 'u':
+      {
+        unsigned usecs = m_microSecondsSinceEpoch.load()%Micro;
         switch (repeatCount) {
-          case 1 :
-            str << (microseconds/100000);
+          case 1:
+            str << (usecs / 100000);
             break;
-          case 2 :
-            str << setw(2) << (microseconds/10000);
+          case 2:
+            str << setw(2) << (usecs / 10000);
             break;
-          case 3 :
-            str << setw(3) << (microseconds/1000);
+          case 3:
+            str << setw(3) << (usecs / 1000);
             break;
-          default :
-            str << setw(6) << microseconds;
+          default:
+            str << setw(6) << usecs;
             break;
         }
         break;
+      }
 
       case '\\' :
         // Escaped character, put straight through to output string
@@ -632,9 +746,8 @@ void PTime::ReadFrom(istream & strm)
   time_t now;
   struct tm timeBuf;
   time(&now);
-  microseconds = 0;
   strm >> ws;
-  theTime = PTimeParse(&strm, os_localtime(&now, &timeBuf), GetTimeZone(StandardTime));
+  m_microSecondsSinceEpoch.store(PTimeParse(&strm, os_localtime(&now, &timeBuf), GetTimeZone(StandardTime))*Micro);
 }
 
 
@@ -646,83 +759,38 @@ bool PTime::Parse(const PString & str)
 }
 
 
-PTime PTime::operator+(const PTimeInterval & t) const
-{
-  time_t secs = theTime + t.GetSeconds();
-  long usecs = (long)(microseconds + (t.GetMilliSeconds()%1000)*1000);
-  if (usecs < 0) {
-    usecs += 1000000;
-    secs--;
-  }
-  else if (usecs >= 1000000) {
-    usecs -= 1000000;
-    secs++;
-  }
+//////////////////////////////////////////////////////////////////////////////
+// P_timeval
 
-  return PTime(secs, usecs);
+P_timeval::P_timeval(long secs, long usecs)
+  : m_infinite(false)
+{
+  m_timeval.tv_sec = secs;
+  m_timeval.tv_usec = usecs;
 }
 
 
-PTime & PTime::operator+=(const PTimeInterval & t)
+P_timeval & P_timeval::operator=(const PTimeInterval & time)
 {
-  theTime += t.GetSeconds();
-  microseconds += (long)(t.GetMilliSeconds()%1000)*1000;
-  if (microseconds < 0) {
-    microseconds += 1000000;
-    theTime--;
+  m_infinite = time == PMaxTimeInterval;
+  if (m_infinite)
+    m_timeval.tv_sec = m_timeval.tv_usec = -1;
+  else {
+    m_timeval.tv_sec = time.GetSeconds();
+    m_timeval.tv_usec = (long)(time.GetMicroSeconds() % 1000000);
   }
-  else if (microseconds >= 1000000) {
-    microseconds -= 1000000;
-    theTime++;
-  }
-
   return *this;
 }
 
 
-PTimeInterval PTime::operator-(const PTime & t) const
+P_timeval & P_timeval::operator=(const PTime & time)
 {
-  time_t secs = theTime - t.theTime;
-  long usecs = microseconds - t.microseconds;
-  if (usecs < 0) {
-    usecs += 1000000;
-    secs--;
-  }
-  else if (usecs >= 1000000) {
-    usecs -= 1000000;
-    secs++;
-  }
-  return PTimeInterval(usecs/1000, (long)secs);
-}
-
-
-PTime PTime::operator-(const PTimeInterval & t) const
-{
-  time_t secs = theTime - t.GetSeconds();
-  long usecs = (long)(microseconds - (t.GetMilliSeconds()%1000)*1000);
-  if (usecs < 0) {
-    usecs += 1000000;
-    secs--;
-  }
-  else if (usecs >= 1000000) {
-    usecs -= 1000000;
-    secs++;
-  }
-  return PTime(secs, usecs);
-}
-
-
-PTime & PTime::operator-=(const PTimeInterval & t)
-{
-  theTime -= t.GetSeconds();
-  microseconds -= (long)(t.GetMilliSeconds()%1000)*1000;
-  if (microseconds < 0) {
-    microseconds += 1000000;
-    theTime--;
-  }
-  else if (microseconds >= 1000000) {
-    microseconds -= 1000000;
-    theTime++;
+  m_infinite = !time.IsValid();
+  if (m_infinite)
+    m_timeval.tv_sec = m_timeval.tv_usec = -1;
+  else {
+    m_timeval.tv_sec = (long)time.GetTimeInSeconds();
+    m_timeval.tv_usec = time.GetMicrosecond();
   }
   return *this;
 }
