@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29742 $
+ * $Revision: 34618 $
  * $Author: rjongbloed $
- * $Date: 2013-05-19 22:27:45 -0500 (Sun, 19 May 2013) $
+ * $Date: 2016-02-14 13:52:24 +0000 (Sun, 14 Feb 2016) $
  */
 
 #include <ptlib.h>
-#include <ptbuildopts.h>
 #include <ptlib/sockets.h>
+#include <ptclib/random.h>
 
 #include <ctype.h>
 
@@ -252,7 +252,7 @@ WORD PIPSocket::sockaddr_wrapper::GetPort() const
 }
 
 
-#if (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)) || defined(__NUCLEUS_PLUS__)
+#if (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_LIBC)) || defined(__NUCLEUS_PLUS__)
 #define REENTRANT_BUFFER_LEN 1024
 #endif
 
@@ -585,7 +585,7 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
     host_info = Vx_gethostbyname((char *)name, &hostEnt);
     localErrNo = h_errno;
 
-#elif defined P_LINUX || defined(P_GNU_HURD)
+#elif defined P_LINUX || defined(P_GNU_HURD) || defined(P_ANDROID)
 
     char buffer[REENTRANT_BUFFER_LEN];
     struct hostent hostEnt;
@@ -598,7 +598,7 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
         localErrNo = NETDB_SUCCESS;
     } while (localErrNo == TRY_AGAIN && --retry > 0);
 
-#elif (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)) || defined(__NUCLEUS_PLUS__)
+#elif (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_LIBC)) || defined(__NUCLEUS_PLUS__)
 
     char buffer[REENTRANT_BUFFER_LEN];
     struct hostent hostEnt;
@@ -704,30 +704,36 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
       localErrNo = h_errno;
     } while (localErrNo == TRY_AGAIN && --retry > 0);
 
-#elif defined P_RTEMS || defined P_CYGWIN || defined P_MINGW
+#elif defined P_RTEMS || defined P_CYGWIN || defined P_MINGW || defined P_ANDROID
 
+    // Mutex here is not perfect, but will be 100% provided application only
+    // ever use PTLib do name lookups, no direct calls to gethostbyaddr()
+    mutex.Wait();
     host_info = ::gethostbyaddr(addr.GetPointer(), addr.GetSize(), PF_INET);
     localErrNo = h_errno;
+    mutex.Signal();
 
 #elif defined P_VXWORKS
 
     struct hostent hostEnt;
     host_info = Vx_gethostbyaddr(addr.GetPointer(), &hostEnt);
 
-#elif defined P_LINUX || defined(P_GNU_HURD)
+#elif defined P_LINUX || defined(P_GNU_HURD) || defined(P_FREEBSD)
 
     char buffer[REENTRANT_BUFFER_LEN];
     struct hostent hostEnt;
     do {
-      ::gethostbyaddr_r(addr.GetPointer(), addr.GetSize(),
-                        PF_INET, 
-                        &hostEnt,
-                        buffer, REENTRANT_BUFFER_LEN,
-                        &host_info,
-                        &localErrNo);
+      gethostbyaddr_r(addr.GetPointer(), addr.GetSize(),
+                      PF_INET, 
+                      &hostEnt,
+                      buffer, REENTRANT_BUFFER_LEN,
+                      &host_info,
+                      &localErrNo);
     } while (localErrNo == TRY_AGAIN && --retry > 0);
 
-#elif (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)) || defined(__NUCLEUS_PLUS__)
+#elif (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_LIBC)) || defined(__NUCLEUS_PLUS__) || defined (_WIN32)
+
+#if defined(P_GETHOSTBYNAME_R)
 
     char buffer[REENTRANT_BUFFER_LEN];
     struct hostent hostEnt;
@@ -744,13 +750,9 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
     host_info = ::gethostbyaddr(addr.GetPointer(), addr.GetSize(), PF_INET);
     localErrNo = h_errno;
 
-#if defined(_WIN32) || defined(WINDOWS)  // Kludge to avoid strange 95 bug
-    extern PBoolean P_IsOldWin95();
-    if (P_IsOldWin95() && host_info != NULL && host_info->h_addr_list[0] != NULL)
-      host_info->h_addr_list[1] = NULL;
-#endif
+#endif // P_GETHOSTBYNAME_R
 
-#endif
+#endif // Operating system
 
     mutex.Wait();
 
@@ -766,77 +768,6 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
 }
 
 
-//////////////////////////////////////////////////////////////////////////////
-// P_fd_set
-
-#ifdef _MSC_VER
-#pragma warning(push)
-#pragma warning(disable:4127)
-#endif
-
-P_fd_set::P_fd_set()
-{
-  Construct();
-  Zero();
-}
-
-
-P_fd_set::P_fd_set(SOCKET fd)
-{
-  Construct();
-  Zero();
-  FD_SET(fd, set);
-}
-
-
-P_fd_set & P_fd_set::operator=(SOCKET fd)
-{
-  PAssert(fd < max_fd, PInvalidParameter);
-  Zero();
-  FD_SET(fd, set);
-  return *this;
-}
-
-
-P_fd_set & P_fd_set::operator+=(SOCKET fd)
-{
-  PAssert(fd < max_fd, PInvalidParameter);
-  FD_SET(fd, set);
-  return *this;
-}
-
-
-P_fd_set & P_fd_set::operator-=(SOCKET fd)
-{
-  PAssert(fd < max_fd, PInvalidParameter);
-  FD_CLR(fd, set);
-  return *this;
-}
-
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
-
-
-//////////////////////////////////////////////////////////////////////////////
-// P_timeval
-
-P_timeval::P_timeval()
-{
-  tval.tv_usec = 0;
-  tval.tv_sec = 0;
-  infinite = false;
-}
-
-
-P_timeval & P_timeval::operator=(const PTimeInterval & time)
-{
-  infinite = time == PMaxTimeInterval;
-  tval.tv_usec = (long)(time.GetMilliSeconds()%1000)*1000;
-  tval.tv_sec = time.GetSeconds();
-  return *this;
-}
-
 
 //////////////////////////////////////////////////////////////////////////////
 // PSocket
@@ -1024,10 +955,8 @@ int PSocket::Select(PSocket & sock1,
   read += sock1;
   read += sock2;
 
-  Errors lastError;
-  int osError;
-  if (!PChannel::ConvertOSError(Select(read, dummy1, dummy2, timeout), lastError, osError))
-    return lastError;
+  if (!sock1.ConvertOSError(Select(read, dummy1, dummy2, timeout)))
+    return sock1.GetErrorCode();
 
   switch (read.GetSize()) {
     case 0 :
@@ -1078,6 +1007,27 @@ PChannel::Errors PSocket::Select(SelectList & read,
 }
 
 
+PBoolean PSocket::ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group)
+{
+  if (PChannel::ConvertOSError(libcReturnValue, group))
+    return true;
+
+  switch (lastErrorNumber[group]) {
+    case ECONNRESET :
+    case ECONNREFUSED :
+    case EHOSTUNREACH :
+    case ENETUNREACH :
+      SetErrorValues(Unavailable, lastErrorNumber[group], group);
+      break;
+
+    case EMSGSIZE :
+      return SetErrorValues(BufferTooSmall, lastErrorNumber[group], group);
+  }
+
+  return false;
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PIPSocket
 
@@ -1098,18 +1048,22 @@ void PIPSocket::ClearNameCache()
     ::gethostbyname("www.microsoft.com");
   }
 #endif
-  PTRACE(4, NULL, NULL, "Cleared DNS cache.");
+  PTRACE(4, &pHostByName(), "Cleared DNS cache.");
 }
 
 
 PString PIPSocket::GetName() const
 {
-  PIPSocket::sockaddr_wrapper sa;
-  socklen_t size = sa.GetSize();
-  if (getpeername(os_handle, sa, &size) != 0)
-    return PString::Empty();
+  PStringStream str;
+  str << GetProtocolName() << ':';
 
-  return GetHostName(sa.GetIP()) + psprintf(":%u", sa.GetPort());
+  AddressAndPort ap;
+  if (GetLocalAddress(ap))
+    str << " local=" << ap;
+  if (GetPeerAddress(ap))
+    str << " peer= " << ap;
+
+  return str;
 }
 
 
@@ -1123,6 +1077,34 @@ PString PIPSocket::GetHostName()
 }
 
 
+PBoolean PIPSocket::IsLocalHost(const PString & hostname)
+{
+  if (hostname.IsEmpty())
+    return true;
+
+  if (hostname *= "localhost")
+    return true;
+
+  Address addr(0);
+  if (!GetHostAddress(hostname, addr))
+    return false;
+
+  if (addr.IsLoopback())  // Is 127.0.0.1 or ::1
+    return true;
+
+  InterfaceTable table;
+  if (!GetInterfaceTable(table, true))
+    return false;
+
+  for (PINDEX i = 0; i < table.GetSize(); ++i) {
+    if (table[i].GetAddress() == addr)
+      return true;
+  }
+
+  return false;
+}
+
+
 PString PIPSocket::GetHostName(const PString & hostname)
 {
   // lookup the host address using inet_addr, assuming it is a "." address
@@ -1162,15 +1144,6 @@ PBoolean PIPSocket::GetHostAddress(const PString & hostname, Address & addr)
   if (hostname.IsEmpty())
     return false;
 
-  // Check for special case of "[ipaddr]"
-  if (hostname[0] == '[') {
-    PINDEX end = hostname.Find(']');
-    if (end != P_MAX_INDEX) {
-      if (addr.FromString(hostname(1, end-1)))
-        return true;
-    }
-  }
-
   // Assuming it is a "." address and return if so
   if (addr.FromString(hostname))
     return true;
@@ -1205,16 +1178,14 @@ PStringArray PIPSocket::GetHostAliases(const Address & addr)
 }
 
 
-PString PIPSocket::GetLocalAddress()
+PString PIPSocket::GetLocalAddress() const
 {
-  PIPSocketAddressAndPort addrAndPort;
-  if (!GetLocalAddress(addrAndPort)) 
-    return PString::Empty();
-  return addrAndPort.AsString();
+  PIPSocketAddressAndPort ap;
+  return GetLocalAddress(ap) ? ap.AsString() : PString::Empty();
 }
 
 
-bool PIPSocket::GetLocalAddress(Address & addr)
+bool PIPSocket::GetLocalAddress(Address & addr) const
 {
   PIPSocketAddressAndPort addrAndPort;
   if (!GetLocalAddress(addrAndPort)) 
@@ -1224,7 +1195,7 @@ bool PIPSocket::GetLocalAddress(Address & addr)
 }
 
 
-bool PIPSocket::GetLocalAddress(Address & addr, WORD & portNum)
+bool PIPSocket::GetLocalAddress(Address & addr, WORD & portNum) const
 {
   PIPSocketAddressAndPort addrAndPort;
   if (!GetLocalAddress(addrAndPort)) 
@@ -1235,6 +1206,12 @@ bool PIPSocket::GetLocalAddress(Address & addr, WORD & portNum)
 }
 
 
+bool PIPSocket::GetLocalAddress(AddressAndPort & addr) const
+{
+  return const_cast<PIPSocket *>(this)->InternalGetLocalAddress(addr);
+}
+
+
 bool PIPSocket::InternalGetLocalAddress(PIPSocketAddressAndPort & addrAndPort)
 {
   Address   peerv4;
@@ -1262,16 +1239,14 @@ bool PIPSocket::InternalGetLocalAddress(PIPSocketAddressAndPort & addrAndPort)
 }
 
 
-PString PIPSocket::GetPeerAddress()
+PString PIPSocket::GetPeerAddress() const
 {
-  PIPSocketAddressAndPort addrAndPort;
-  if (GetPeerAddress(addrAndPort)) 
-    return addrAndPort.AsString();
-  return PString::Empty();
+  PIPSocketAddressAndPort ap;
+  return GetPeerAddress(ap) ? ap.AsString() : PString::Empty();
 }
 
 
-bool PIPSocket::GetPeerAddress(Address & addr)
+bool PIPSocket::GetPeerAddress(Address & addr) const
 {
   PIPSocketAddressAndPort addrAndPort;
   if (!GetPeerAddress(addrAndPort)) 
@@ -1281,7 +1256,7 @@ bool PIPSocket::GetPeerAddress(Address & addr)
 }
 
 
-bool PIPSocket::GetPeerAddress(Address & addr, WORD & portNum)
+bool PIPSocket::GetPeerAddress(Address & addr, WORD & portNum) const
 {
   PIPSocketAddressAndPort addrAndPort;
   if (!GetPeerAddress(addrAndPort)) 
@@ -1292,6 +1267,12 @@ bool PIPSocket::GetPeerAddress(Address & addr, WORD & portNum)
 }
 
 
+bool PIPSocket::GetPeerAddress(AddressAndPort & addr) const
+{
+  return const_cast<PIPSocket *>(this)->InternalGetPeerAddress(addr);
+}
+
+
 bool PIPSocket::InternalGetPeerAddress(PIPSocketAddressAndPort & addrAndPort)
 {
   PIPSocket::sockaddr_wrapper sa;
@@ -1333,7 +1314,7 @@ PBoolean PIPSocket::Connect(const PString & host)
   Address ipnum(host);
   if (ipnum.IsValid() || GetHostAddress(host, ipnum))
     return Connect(Address::GetAny(ipnum.GetVersion()), 0, ipnum);
-  return false;
+  return SetErrorValues(BadParameter, EINVAL);
 }
 
 
@@ -1357,6 +1338,11 @@ PBoolean PIPSocket::Connect(const Address & iface, const Address & addr)
 
 PBoolean PIPSocket::Connect(const Address & iface, WORD localPort, const Address & addr)
 {
+  if (!addr.IsValid()) {
+    PTRACE(2, "Cannot connect to invalid address");
+    return SetErrorValues(BadParameter, EINVAL);
+  }
+
   // close the port if it is already open
   if (IsOpen())
     Close();
@@ -1367,18 +1353,22 @@ PBoolean PIPSocket::Connect(const Address & iface, WORD localPort, const Address
   PIPSocket::sockaddr_wrapper sa(addr, port);
 
   // attempt to create a socket with the right family
-  if (!OpenSocket(sa->sa_family))
+  if (!OpenSocket(sa->sa_family)) {
+    PTRACE(2, "Create failure: family=" << sa->sa_family << ", errno=" << GetErrorNumber() << ' ' << GetErrorText());
     return false;
+  }
 
   if (localPort != 0 || (iface.IsValid() && !iface.IsAny())) {
     PIPSocket::sockaddr_wrapper bind_sa(iface, localPort);
 
     if (!SetOption(SO_REUSEADDR, 0)) {
+      PTRACE(2, "setsockopt SO_REUSEADDR failure: errno=" << GetErrorNumber() << ' ' << GetErrorText());
       os_close();
       return false;
     }
-    
+
     if (!ConvertOSError(::bind(os_handle, bind_sa, bind_sa.GetSize()))) {
+      PTRACE(2, "bind failure: errno=" << GetErrorNumber() << ' ' << GetErrorText());
       os_close();
       return false;
     }
@@ -1398,6 +1388,11 @@ bool PIPSocket::InternalListen(const Address & bindAddr,
                                WORD newPort,
                                Reusability reuse)
 {
+  if (!bindAddr.IsValid()) {
+    PTRACE(2, "Cannot listen bound to invalid address");
+    return false;
+  }
+
   // make sure we have a port
   if (newPort != 0)
     port = newPort;
@@ -1409,7 +1404,7 @@ bool PIPSocket::InternalListen(const Address & bindAddr,
 
   // attempt to create a socket
   if (!OpenSocket(sa->sa_family)) {
-    PTRACE(4, "OpenSocket failed");
+    PTRACE(4, "OpenSocket for " << GetProtocolName() << " failed: " << GetErrorText());
     return false;
   }
 
@@ -1769,11 +1764,10 @@ PObject::Comparison PIPSocket::Address::Compare(const PObject & obj) const
   }
 #endif
 
-  if ((DWORD)*this < other)
-    return LessThan;
-  if ((DWORD)*this > other)
-    return GreaterThan;
-  return EqualTo;
+  if (operator==((DWORD)other))
+    return EqualTo;
+
+  return ntohl(*this) < ntohl(other) ? LessThan : GreaterThan;
 }
 
 
@@ -2053,7 +2047,38 @@ bool PIPSocket::Address::IsMulticast() const
 }
 
 
-bool PIPSocket::Address::IsRFC1918() const 
+bool PIPSocket::Address::IsSubNet(const Address & network, const Address & mask) const
+{
+  if (m_version != network.m_version || m_version != mask.m_version)
+    return false;
+
+#if P_HAS_IPV6
+  if (m_version == 6) {
+    for (PINDEX i = 0; i < 16; ++i) {
+      if (((*this)[i] & mask[i]) != network[i])
+        return false;
+    }
+    return true;
+  }
+#endif
+
+  DWORD bitmask;
+  if (!mask.IsAny())
+    bitmask = mask;
+  else if (network.Byte1() < 128)
+    bitmask = inet_addr("255.0.0.0");
+  else if (network.Byte1() < 192)
+    bitmask = inet_addr("255.255.0.0");
+  else if (network.Byte1() < 240)
+    bitmask = inet_addr("255.255.255.0");
+  else
+    bitmask = inet_addr("255.255.255.255");
+
+  return ((DWORD)*this & bitmask) == ((DWORD)network & bitmask);
+}
+
+
+bool PIPSocket::Address::IsRFC1918() const
 {
   PINDEX offset = 0;
 #if P_HAS_IPV6
@@ -2177,6 +2202,22 @@ PIPSocket::Address PIPSocket::GetInterfaceAddress(const PString & ifName, unsign
 }
 
 
+PString PIPSocket::GetInterfaceMACAddress(const char * ifName)
+{
+  PIPSocket::InterfaceTable interfaceTable;
+  if (PIPSocket::GetInterfaceTable(interfaceTable)) {
+    for (PINDEX i = 0; i < interfaceTable.GetSize(); i++) {
+      if ((ifName == NULL || interfaceTable[i].GetName() == ifName)) {
+        PString macAddrStr = interfaceTable[i].GetMACAddress();
+        if (!macAddrStr.IsEmpty() && macAddrStr != "44-45-53-54-00-00") /* not Win32 PPP device */
+          return macAddrStr;
+      }
+    }
+  }
+  return PString::Empty();
+}
+
+
 PIPSocket::Address PIPSocket::GetNetworkInterface(unsigned version)
 {
   PIPSocket::InterfaceTable interfaceTable;
@@ -2260,16 +2301,19 @@ const char * PTCPSocket::GetProtocolName() const
 
 PBoolean PTCPSocket::Write(const void * buf, PINDEX len)
 {
+  if (CheckNotOpen())
+    return false;
+
   flush();
   PINDEX writeCount = 0;
 
-  while (len > 0) {
+  do {
     Slice slice(((char *)buf)+writeCount, len);
     if (!os_vwrite(&slice, 1, 0, NULL, 0))
       return false;
     writeCount += lastWriteCount;
     len -= lastWriteCount;
-  }
+  } while (len > 0);
 
   lastWriteCount = writeCount;
   return true;
@@ -2376,8 +2420,8 @@ bool PIPDatagramSocket::InternalReadFrom(Slice * slices, size_t sliceCount, PIPS
 {
   lastReadCount = 0;
 
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
   PIPSocket::sockaddr_wrapper sa;
   socklen_t size = sa.GetSize();
@@ -2423,11 +2467,13 @@ bool PIPDatagramSocket::InternalWriteTo(const Slice * slices, size_t sliceCount,
 {
   lastWriteCount = 0;
 
+  if (CheckNotOpen())
+    return false;
+
   const PIPSocket::Address & addr = ipAndPort.GetAddress();
   WORD port = ipAndPort.GetPort();
-
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (!addr.IsValid() || port == 0)
+    return SetErrorValues(BadParameter, EINVAL, LastWriteError);
 
   PBoolean broadcast = addr.IsAny() || addr.IsBroadcast();
   if (broadcast) {
@@ -2498,30 +2544,14 @@ bool PIPDatagramSocket::InternalWriteTo(const Slice * slices, size_t sliceCount,
 // PUDPSocket
 
 PUDPSocket::PUDPSocket(WORD newPort, int iAddressFamily)
-#if P_HAS_IPV6
-  : m_sendAddress(iAddressFamily == AF_INET ? loopback4 : loopback6),
-    m_lastReceiveAddress(iAddressFamily == AF_INET ? loopback4 : loopback6)
-#else
-  : m_sendAddress(loopback4),
-    m_lastReceiveAddress(loopback4)
-#endif
 {
-  m_sendPort = 0;
   SetPort(newPort);
   OpenSocket(iAddressFamily);
 }
 
 
 PUDPSocket::PUDPSocket(const PString & service, int iAddressFamily)
-#if P_HAS_IPV6
-  : m_sendAddress(iAddressFamily == AF_INET ? loopback4 : loopback6),
-    m_lastReceiveAddress(iAddressFamily == AF_INET ? loopback4 : loopback6)
-#else
-  : m_sendAddress(loopback4),
-    m_lastReceiveAddress(loopback4)
-#endif
 {
-  m_sendPort = 0;
   SetPort(service);
   OpenSocket(iAddressFamily);
 }
@@ -2608,12 +2638,20 @@ bool PUDPSocket::InternalListen(const Address & bindAddr,
 }
 
 
+bool PUDPSocket::InternalReadFrom(Slice * slices, size_t sliceCount, PIPSocketAddressAndPort & ipAndPort)
+{
+  if (!PIPDatagramSocket::InternalReadFrom(slices, sliceCount, ipAndPort))
+    return false;
+
+  InternalSetLastReceiveAddress(ipAndPort);
+  return true;
+}
+
+
 PBoolean PUDPSocket::Read(void * buf, PINDEX len)
 {
-  PIPSocketAddressAndPort ap;
-  bool stat = PIPDatagramSocket::ReadFrom(buf, len, ap);
-  InternalSetLastReceiveAddress(ap);
-  return stat;
+  PIPSocketAddressAndPort dummy;
+  return ReadFrom(buf, len, dummy);
 }
 
 
@@ -2640,8 +2678,7 @@ void PUDPSocket::SetSendAddress(const PIPSocketAddressAndPort & addressAndPort)
 
 void PUDPSocket::InternalSetSendAddress(const PIPSocketAddressAndPort & addr)
 {
-  m_sendAddress = addr.GetAddress();
-  m_sendPort    = addr.GetPort();
+  m_sendAddressAndPort = addr;
 }
 
 
@@ -2662,13 +2699,13 @@ void PUDPSocket::GetSendAddress(PIPSocketAddressAndPort & addr) const
 
 PString PUDPSocket::GetSendAddress() const
 {
-  return m_sendAddress.AsString(true) + psprintf(":%u", m_sendPort);
+  return m_sendAddressAndPort.AsString();
 }
 
 
 void PUDPSocket::InternalGetSendAddress(PIPSocketAddressAndPort & addr) const
 {
-  addr = PIPSocketAddressAndPort(m_sendAddress, m_sendPort);
+  addr = m_sendAddressAndPort;
 }
 
 
@@ -2689,52 +2726,19 @@ void PUDPSocket::GetLastReceiveAddress(PIPSocketAddressAndPort & ap) const
 
 PString PUDPSocket::GetLastReceiveAddress() const
 {
-  return m_lastReceiveAddress.AsString(true) + psprintf(":%u", m_lastReceivePort);
+  return m_lastReceiveAddressAndPort.AsString();
 }
 
 
 void PUDPSocket::InternalGetLastReceiveAddress(PIPSocketAddressAndPort & ap) const
 {
-  ap = PIPSocketAddressAndPort(m_lastReceiveAddress, m_lastReceivePort);
+  ap = m_lastReceiveAddressAndPort;
 }
 
 
 void PUDPSocket::InternalSetLastReceiveAddress(const PIPSocketAddressAndPort & ap)
 {
-  m_lastReceiveAddress = ap.GetAddress();
-  m_lastReceivePort    = ap.GetPort();
-}
-
-
-PBoolean PUDPSocket::IsAlternateAddress(const Address &, WORD)
-{
-  return false;
-}
-
-PBoolean PUDPSocket::DoPseudoRead(int & /*selectStatus*/)
-{
-   return false;
-}
-
-
-PBoolean PUDPSocket::ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group)
-{
-  if (PIPDatagramSocket::ConvertOSError(libcReturnValue, group))
-    return true;
-
-  switch (lastErrorNumber[group]) {
-    case ECONNRESET :
-    case ECONNREFUSED :
-    case EHOSTUNREACH :
-    case ENETUNREACH :
-      SetErrorValues(Unavailable, lastErrorNumber[group], group);
-      break;
-
-    case EMSGSIZE :
-      return SetErrorValues(BufferTooSmall, lastErrorNumber[group], group);
-  }
-
-  return false;
+  m_lastReceiveAddressAndPort = ap;
 }
 
 
@@ -2811,11 +2815,11 @@ PString PIPSocket::AddressAndPort::AsString(char separator) const
 {
   PString str;
 
-  if (m_address.IsValid())
+  if (m_address.IsValid()) {
     str = m_address.AsString(true, true);
-
-  if (m_port != 0)
-    str.sprintf("%c%u", separator ? separator : m_separator, m_port);
+    if (m_port != 0)
+      str.sprintf("%c%u", separator ? separator : m_separator, m_port);
+  }
 
   return str;
 }
@@ -2829,11 +2833,152 @@ void PIPSocket::AddressAndPort::SetAddress(const PIPSocket::Address & addr, WORD
 }
 
 
+PObject::Comparison PIPSocket::AddressAndPort::Compare(const PObject & obj) const
+{
+  const AddressAndPort & other = dynamic_cast<const AddressAndPort &>(obj);
+  Comparison c = m_address.Compare(other.m_address);
+  if (c == EqualTo)
+    c = Compare2(m_port, other.m_port);
+  return c;
+}
+
+
 bool PIPSocket::AddressAndPort::MatchWildcard(const AddressAndPort & wild) const
 {
-  return (!wild.m_address.IsValid() || wild.m_address == m_address) &&
+  return (!wild.m_address.IsValid() || wild.m_address.IsAny() || wild.m_address == m_address) &&
          ( wild.m_port == 0         || wild.m_port    == m_port);
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+
+PIPSocket::PortRange::PortRange(WORD basePort, WORD maxPort)
+  : m_base(basePort)
+  , m_max(basePort != 0 && maxPort != 0 ? maxPort : basePort)
+{
+}
+
+
+void PIPSocket::PortRange::Set(unsigned newBase, unsigned newMax, unsigned range, unsigned dflt)
+{
+  if (newBase == 0) {
+    newBase = dflt;
+    newMax = dflt;
+    if (dflt > 0)
+      newMax += range;
+  }
+  else {
+    if (newBase < 1024)
+      newBase = 1024;
+    else if (newBase > 65500)
+      newBase = 65500;
+
+    if (newMax <= newBase)
+      newMax = newBase + range;
+    if (newMax > 65530) // Room for something to get 5 consecutive ports.
+      newMax = 65530;
+  }
+
+  m_mutex.Wait();
+
+  m_base = (WORD)newBase;
+  m_max = (WORD)newMax;
+
+  m_mutex.Signal();
+}
+
+
+bool PIPSocket::PortRange::Connect(PIPSocket & socket, const Address & addr, const Address & binding)
+{
+  m_mutex.Wait();
+  WORD basePort = m_base;
+  WORD maxPort = m_max;
+  m_mutex.Signal();
+
+  if (basePort == 0 || basePort == maxPort)
+    return socket.Connect(binding, basePort, addr);
+
+  WORD firstPort = (WORD)PRandom::Number(basePort, maxPort);
+  WORD nextPort = firstPort;
+  do {
+    if (socket.Connect(binding, nextPort, addr))
+      return true;
+    if (socket.GetErrorNumber() != EADDRINUSE && socket.GetErrorNumber() != EADDRNOTAVAIL)
+      return false;
+    PTRACE(5, &socket, PTraceModule(), "Could not connect using local port " << nextPort);
+    if (++nextPort >= maxPort)
+      nextPort = basePort;
+  } while (nextPort != firstPort);
+
+  PTRACE(2, &socket, PTraceModule(), "Connect failed, exhausted all ports from " << basePort << " to " << maxPort);
+  return false;
+}
+
+
+bool PIPSocket::PortRange::Listen(PIPSocket & socket, const Address & binding, unsigned queueSize, Reusability reuse)
+{
+  PIPSocket * sockets = &socket;
+  return Listen(&sockets, 1, binding, queueSize, reuse);
+}
+
+
+bool PIPSocket::PortRange::Listen(PIPSocket ** sockets,
+                                  PINDEX numSockets,
+                                  const Address & binding,
+                                  unsigned queueSize,
+                                  Reusability reuse)
+{
+  if (!PAssert(sockets != NULL && sockets[0] != NULL, PInvalidParameter))
+    return false;
+
+  PWaitAndSignal mutex(m_mutex);
+
+  if (m_base == 0 || m_base == m_max) {
+    if (!sockets[0]->Listen(binding, queueSize, m_base, reuse))
+      return false;
+    WORD nextPort = sockets[0]->GetPort();
+    for (WORD s = 1; s < numSockets; ++s) {
+      if (PAssert(sockets[s] != NULL, PInvalidParameter) && !sockets[s]->Listen(binding, queueSize, nextPort+s, reuse))
+        return false;
+    }
+    return true;
+  }
+
+  if (numSockets >= (m_max - m_base)) {
+    PTRACE(2, NULL, PTraceModule(), "Listen failed, not enough room from "
+           << m_base << " to " << m_max << " for " << numSockets << " sockets");
+    return false;
+  }
+
+  /*
+    Use a random port pair base number in the specified range for
+    the creation of the RTP port pairs (this used to avoid issues with multiple
+    NATed devices opening the same local ports and experiencing issues with
+    the behaviour of the NAT device Refer: draft-jennings-behave-test-results-04 sect 3
+  */
+  WORD firstPort = (WORD)((PRandom::Number(m_base, m_max)/numSockets)*numSockets);
+  WORD nextPort = firstPort;
+  do {
+    WORD s;
+    for (s = 0; s < numSockets; ++s) {
+      if (!PAssert(sockets[s] != NULL, PInvalidParameter))
+        return false;
+      if (!sockets[s]->Listen(binding, queueSize, nextPort+s, reuse)) {
+        PTRACE(5, sockets[0], PTraceModule(), "Could not listen on port " << nextPort);
+        break;
+      }
+    }
+    if (s == numSockets)
+      return true;
+
+    nextPort += (WORD)numSockets;
+    if ((nextPort + numSockets) > m_max)
+      nextPort = m_base;
+  } while (nextPort != firstPort);
+
+  PTRACE(2, sockets[0], PTraceModule(), "Listen failed, exhausted all ports from " << m_base << " to " << m_max);
+  return false;
+}
+
+
 // End Of File ///////////////////////////////////////////////////////////////
