@@ -23,24 +23,27 @@
  *
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
- * $Revision: 28484 $
+ * $Revision: 33858 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2015-08-08 22:17:49 +0100 (Sat, 08 Aug 2015) $
  */
 
 #include <ptlib.h>
 
 #if P_VIDEO
 
-#include <ptlib/vconvert.h>
+#define P_FORCE_STATIC_PLUGIN 1
 
-#include <ptlib/pluginmgr.h>
-#include <ptlib/pprocess.h>
 #include <ptlib/videoio.h>
+#include <ptlib/vconvert.h>
+#include <ptlib/pprocess.h>
 #include <ptclib/delaychan.h>
 #include <ptclib/random.h>
 
 
+#define PTraceModule() "FakeVideo"
+
+
 // Device names for fake output
 
 enum {
@@ -55,14 +58,14 @@ enum {
 };
 
 static const char * const FakeDeviceNames[] = {
-  "Fake/MovingBlocks",
-  "Fake/MovingLine",
-  "Fake/BouncingBoxes",
-  "Fake/SolidColour",
-  "Fake/OriginalMovingBlocks",
-  "Fake/Text",
-  "Fake/NTSCTest",
-  "fake" // Always last
+  P_FAKE_VIDEO_MOVING_BLOCKS,
+  P_FAKE_VIDEO_MOVING_LINE,
+  P_FAKE_VIDEO_BOUNCING_BOXES,
+  P_FAKE_VIDEO_SOLID_COLOUR,
+  P_FAKE_VIDEO_PREFIX"OriginalMovingBlocks",
+  P_FAKE_VIDEO_TEXT,
+  P_FAKE_VIDEO_NTSC,
+  "fake" // Always last, ancient history
 };
 
 /****
@@ -1413,30 +1416,16 @@ class PVideoInputDevice_FakeVideo : public PVideoInputDevice
                   int rectWidth, int rectHeight,
                   int r,         int g,          int b);
 
-    /**Set the video format to be used.
-
-       Default behaviour sets the value of the videoFormat variable and then
-       returns the IsOpen() status.
-    */
-    virtual PBoolean SetVideoFormat(
-      VideoFormat videoFormat   /// New video format
-    );
-
     /**Get the number of video channels available on the device.
-
-       Default behaviour returns 1.
     */
-    virtual int GetNumChannels() ;
+    virtual int GetNumChannels();
 
-    /**Set the video channel to be used on the device.
+    /**Get the names of video channels available on the device.
 
-       Default behaviour sets the value of the channelNumber variable and then
-       returns the IsOpen() status.
+       Default behaviour returns 'A', 'B' up to GetNumChannels().
     */
-    virtual PBoolean SetChannel(
-         int channelNumber  /// New channel number for device.
-    );
-    
+    virtual PStringArray GetChannelNames();
+
     /**Set the colour format to be used.
 
        Default behaviour sets the value of the colourFormat variable and then
@@ -1498,38 +1487,72 @@ class PVideoInputDevice_FakeVideo : public PVideoInputDevice
    PString textLine[PVideoFont::MAX_L_HEIGHT];
 };
 
-PCREATE_VIDINPUT_PLUGIN(FakeVideo);
+PCREATE_VIDINPUT_PLUGIN_EX(FakeVideo,
+  virtual bool ValidateDeviceName(const PString & devName, P_INT_PTR /*userData*/) const
+  {
+    for (PINDEX chan = 0; chan < PARRAYSIZE(FakeDeviceNames) - 1; ++chan) {
+      if (devName.Left(devName.Find('=')) *= FakeDeviceNames[chan])
+        return true;
+    }
+    return false;
+  }
+);
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoInputDevice_FakeVideo
 
-
 PVideoInputDevice_FakeVideo::PVideoInputDevice_FakeVideo()
   : m_open(false)
   , m_Pacing(500)
 {
-  SetColourFormat("RGB24");
-  channelNumber = 3; // Blank screen
+  SetColourFormat(GetColourFormat());
+  channelNumber = eBouncingBoxes;
 
   m_grabCount = 0;
   SetFrameRate(10);
 }
 
 
-
 PBoolean PVideoInputDevice_FakeVideo::Open(const PString & devName, PBoolean /*startImmediate*/)
 {
-  PINDEX i;
-  for (i = 0; i < PARRAYSIZE(FakeDeviceNames)-1; ++i) {
-    if (devName *= FakeDeviceNames[i]) {
-      PVideoDevice::SetChannel(i);
-      break;
+  for (PINDEX chan = 0; chan < PARRAYSIZE(FakeDeviceNames)-1; ++chan) {
+    PINDEX equals = devName.Find('=');
+    if (devName.Left(equals) *= FakeDeviceNames[chan]) {
+      deviceName = FakeDeviceNames[chan];
+      SetChannel(chan);
+
+      if (chan == eText) {
+        PStringStream message;
+        if (equals != P_MAX_INDEX)
+          message << devName.Mid(equals+1);
+        else
+          message << PProcess::Current().GetUserName() <<  " on " <<
+                     PProcess::Current().GetOSName() << ":" <<
+                     PProcess::Current().GetOSHardware();
+
+        for (int scanLine = 0; scanLine < PVideoFont::MAX_L_HEIGHT; ++scanLine)
+          textLine[scanLine].MakeEmpty();
+
+        PINDEX nChars = message.GetLength();
+        for (PINDEX i = 0; i < (nChars + 2); i++) {
+          const PVideoFont::LetterData * ld = PVideoFont::GetLetterData(i >= nChars ? ' ' : message[i]);
+          if (ld != NULL) {
+            for (int scanLine = 0; scanLine < PVideoFont::MAX_L_HEIGHT; ++scanLine) {
+              textLine[scanLine] += ld->line[scanLine];
+              textLine[scanLine] += ' ';
+            }
+          }
+        }
+      }
+
+      m_open = true;
+      return true;
     }
   }
 
-  deviceName = FakeDeviceNames[i];
-  m_open = true;
-  return true;
+  PTRACE(2, "Unknown fake video \"" << devName << '"');
+  return false;
 }
 
 
@@ -1570,24 +1593,18 @@ PStringArray PVideoInputDevice_FakeVideo::GetInputDeviceNames()
 }
 
 
-PBoolean PVideoInputDevice_FakeVideo::SetVideoFormat(VideoFormat newFormat)
-{
-  return PVideoDevice::SetVideoFormat(newFormat);
-}
-
-
 int PVideoInputDevice_FakeVideo::GetNumChannels() 
 {
   return eNumTestPatterns;  
 }
 
 
-PBoolean PVideoInputDevice_FakeVideo::SetChannel(int newChannel)
+PStringArray PVideoInputDevice_FakeVideo::GetChannelNames()
 {
-  if (newChannel < 0 || deviceName != FakeDeviceNames[PARRAYSIZE(FakeDeviceNames)-1])
-    return true; // No change for -1 or explicit channel in device name
-
-  return PVideoDevice::SetChannel(newChannel);
+  PStringArray names(PARRAYSIZE(FakeDeviceNames)-1, FakeDeviceNames);
+  for (PINDEX i = 0; i < names.GetSize(); ++i)
+    names[i].Replace(P_FAKE_VIDEO_PREFIX, PString::Empty());
+  return names;
 }
 
 
@@ -1601,8 +1618,10 @@ PBoolean PVideoInputDevice_FakeVideo::SetColourFormat(const PString & newFormat)
     m_internalColourFormat = eYUV420P;
   else if ((newFormat *= "YUV422") || (newFormat *= "YUY2"))
     m_internalColourFormat = eYUV422;
-  else
+  else {
+    PTRACE(1, "Could not set colour format \"" << newFormat << '"');
     return false;
+  }
 
   if (!PVideoDevice::SetColourFormat(newFormat))
     return false;
@@ -1642,8 +1661,14 @@ PBoolean PVideoInputDevice_FakeVideo::SetFrameSize(unsigned width, unsigned heig
     return false;
 
   m_videoFrameSize = CalculateFrameBytes(frameWidth, frameHeight, colourFormat);
-  m_scanLineWidth = (frameHeight == 0) ? 0 : m_videoFrameSize/frameHeight;
-  return m_videoFrameSize > 0;
+  if (m_videoFrameSize == 0) {
+    PTRACE(1, "Could not calculate video frame size using resolution "
+           << frameWidth << 'x' << frameHeight << " and format \"" << colourFormat << '"');
+    return false;
+  }
+
+  m_scanLineWidth = m_videoFrameSize/frameHeight;
+  return true;
 }
 
 
@@ -1691,16 +1716,18 @@ PBoolean PVideoInputDevice_FakeVideo::GetFrameDataNoDelay(BYTE *destFrame, PINDE
        GrabNTSCTestFrame(destFrame);
        break;
      default :
+       PAssertAlways(PLogicError);
        return false;
   }
 
   if (NULL != converter) {
-    if (!converter->Convert(destFrame, destFrame, bytesReturned))
+    if (!converter->ConvertInPlace(destFrame, bytesReturned))
       return false;
   }
-
-  if (bytesReturned != NULL)
-    *bytesReturned = m_videoFrameSize;
+  else {
+    if (bytesReturned != NULL)
+      *bytesReturned = m_videoFrameSize;
+  }
 
   return true;
 }
@@ -1995,28 +2022,8 @@ void PVideoInputDevice_FakeVideo::GrabTextVideoFrame(BYTE *resFrame)
            200, 200, 200); //a light grey colour.
 
 
-  if (textLine[0].GetLength() < 2) {
-    PStringStream message;
-    message << PProcess::Current().GetUserName() <<  " on " <<
-               PProcess::Current().GetOSName() << ":" <<
-               PProcess::Current().GetOSHardware();
-    PINDEX nChars = message.GetLength();
-    const PVideoFont::LetterData *ld;
-
-    for (j = 0; j < PVideoFont::MAX_L_HEIGHT; j++)
-      textLine[j] = "";
-
-    for (i = 0; i < (nChars + 2); i++){
-      if (i >= nChars)
-        ld = PVideoFont::GetLetterData(' ');
-      else
-        ld = PVideoFont::GetLetterData(message[i]);
-      if (ld == NULL)
-        continue;
-      for (j = 0; j < PVideoFont::MAX_L_HEIGHT; j++)
-        textLine[j] += ld->line[j] + PString(" ");
-    }
-  }
+  if (textLine[0].GetLength() < 2)
+    return;
 
   PINDEX boxSize = (frameHeight / (PVideoFont::MAX_L_HEIGHT * 2) ) & 0xffe;
   int index = (int)((PTime() - startTime).GetMilliSeconds() / 300);
@@ -2060,8 +2067,8 @@ static void FillYUV422Rect(BYTE * frame,
                            int r, int g, int b,
                            PINDEX scanLineWidth)
 {
-  unsigned Y, Cb, Cr;
-  PColourConverter::RGBtoYUV(r, g, b, Y, Cb, Cr);
+  BYTE Y, U, V;
+  PColourConverter::RGBtoYUV(r, g, b, Y, U, V);
 
   x &= 0xfffffffe; // Make sure is even
 
@@ -2069,10 +2076,10 @@ static void FillYUV422Rect(BYTE * frame,
   for (int dy = 0; dy < height; dy++) {
     BYTE * ptr = frame + (y+dy)*scanLineWidth + x*2;
     for (int dx = 0; dx < halfRectWidth; dx++) {
-      *ptr++ = (BYTE)Y;
-      *ptr++ = (BYTE)Cb;
-      *ptr++ = (BYTE)Y;
-      *ptr++ = (BYTE)Cr;
+      *ptr++ = Y;
+      *ptr++ = U;
+      *ptr++ = Y;
+      *ptr++ = V;
     }
   }
 }
@@ -2184,14 +2191,15 @@ PCREATE_VIDOUTPUT_PLUGIN(NULLOutput);
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoOutputDevice_NULLOutput
 
+static PConstString const NullVideoOut(P_NULL_VIDEO_DEVICE);
+
 PVideoOutputDevice_NULLOutput::PVideoOutputDevice_NULLOutput()
 {
-  deviceName = "NULL";
+  deviceName = NullVideoOut;
 }
 
 
-PBoolean PVideoOutputDevice_NULLOutput::Open(const PString & /*deviceName*/,
-                                  PBoolean /*startImmediate*/)
+PBoolean PVideoOutputDevice_NULLOutput::Open(const PString & /*deviceName*/, PBoolean /*startImmediate*/)
 {
   return true;
 }
@@ -2219,7 +2227,7 @@ PBoolean PVideoOutputDevice_NULLOutput::IsOpen()
 
 PStringArray PVideoOutputDevice_NULLOutput::GetOutputDeviceNames()
 {
-  return PString("NULL");
+  return NullVideoOut;
 }
 
 
