@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 29348 $
+ * $Revision: 33855 $
  * $Author: rjongbloed $
- * $Date: 2013-03-27 15:44:35 -0500 (Wed, 27 Mar 2013) $
+ * $Date: 2015-08-08 18:52:41 +0100 (Sat, 08 Aug 2015) $
  */
 
 #ifdef __GNUC__
@@ -41,74 +41,40 @@
 #include <ptlib/vconvert.h>
 
 
-namespace PWLib {
-  PFactory<PDevicePluginAdapterBase>::Worker< PDevicePluginAdapter<PVideoInputDevice> > vidinChannelFactoryAdapter("PVideoInputDevice", true);
-  PFactory<PDevicePluginAdapterBase>::Worker< PDevicePluginAdapter<PVideoOutputDevice> > vidoutChannelFactoryAdapter("PVideoOutputDevice", true);
-};
-
-template <> PVideoInputDevice * PDevicePluginFactory<PVideoInputDevice>::Worker::Create(const PDefaultPFactoryKey & type) const
-{
-  return PVideoInputDevice::CreateDevice(type);
-}
-
-template <> PVideoOutputDevice * PDevicePluginFactory<PVideoOutputDevice>::Worker::Create(const PDefaultPFactoryKey & type) const
-{
-  return PVideoOutputDevice::CreateDevice(type);
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 
-#if PTRACING
-ostream & operator<<(ostream & strm, PVideoDevice::VideoFormat fmt)
-{
-  static const char * const VideoFormatNames[PVideoDevice::NumVideoFormats] = {
-    "PAL",
-    "NTSC",
-    "SECAM",
-    "Auto"
-  };
-
-  if (fmt < PVideoDevice::NumVideoFormats && VideoFormatNames[fmt] != NULL)
-    strm << VideoFormatNames[fmt];
-  else
-    strm << "VideoFormat<" << (unsigned)fmt << '>';
-
-  return strm;
-}
-#endif
-
-
 //Colour format bit per pixel table.
 // These are in rough order of colour gamut size and "popularity"
 static struct {
   const char * colourFormat;
   unsigned     bitsPerPixel;
+  unsigned     alignmentMinus1;
 } ColourFormatBPPTab[] = {
-  { "YUV420P", 12 },
-  { "I420",    12 },
-  { "IYUV",    12 },
-  { "YUV420",  12 },
-  { "RGB32",   32 },
-  { "BGR32",   32 },
-  { "RGB24",   24 },
-  { "BGR24",   24 },
-  { "YUY2",    16 },
-  { "YUV422",  16 },
-  { "YUV422P", 16 },
-  { "YUV411",  12 },
-  { "YUV411P", 12 },
-  { "RGB565",  16 },
-  { "RGB555",  16 },
-  { "RGB16",   16 },
-  { "YUV410",  10 },
-  { "YUV410P", 10 },
-  { "Grey",     8 },
-  { "GreyF",    8 },
-  { "UYVY422", 16 },
-  { "UYV444",  24 },
-  { "SBGGR8",   8 },
-  { "JPEG",    24 },
-  { "MJPEG",   24 }
+  { "YUV420P", 12, 0 },
+  { "I420",    12, 0 },
+  { "IYUV",    12, 0 },
+  { "YUV420",  12, 0 },
+  { "RGB32",   32, 3 },
+  { "BGR32",   32, 3 },
+  { "RGB24",   24, 3 },
+  { "BGR24",   24, 3 },
+  { "YUY2",    16, 0 },
+  { "YUV422",  16, 0 },
+  { "YUV422P", 16, 0 },
+  { "YUV411",  12, 0 },
+  { "YUV411P", 12, 0 },
+  { "RGB565",  16, 3 },
+  { "RGB555",  16, 3 },
+  { "RGB16",   16, 1 },
+  { "YUV410",  10, 0 },
+  { "YUV410P", 10, 0 },
+  { "Grey",     8, 0 },
+  { "GreyF",    8, 0 },
+  { "UYVY422", 16, 1 },
+  { "UYV444",  24, 0 },
+  { "SBGGR8",   8, 0 },
+  { "JPEG",    24, 0 },
+  { "MJPEG",    8, 0 }
 };
 
 
@@ -125,38 +91,11 @@ static VideoDevice * CreateDeviceWithDefaults(PString & adjustedDeviceName,
     adjustedDriverName.MakeEmpty();
 
   if (adjustedDeviceName.IsEmpty()) {
-    if (adjustedDriverName.IsEmpty()) {
-      PStringArray drivers = VideoDevice::GetDriverNames(pluginMgr);
-      if (drivers.IsEmpty())
-        return NULL;
+    PStringArray devices = VideoDevice::GetDriversDeviceNames(adjustedDriverName);
+    if (devices.IsEmpty())
+      return NULL;
 
-      // Give precedence to drivers like camera grabbers, Window
-      static const char * prioritisedDrivers[] = {
-        "Window", "SDL", "DirectShow", "VideoForWindows", "V4L", "V4L2", "1394DC", "1394AVC", "BSDCAPTURE", "FakeVideo", "NULLOutput"
-      };
-      for (PINDEX i = 0; i < PARRAYSIZE(prioritisedDrivers); i++) {
-        PINDEX driverIndex = drivers.GetValuesIndex(PString(prioritisedDrivers[i]));
-        if (driverIndex != P_MAX_INDEX) {
-          PStringArray devices = VideoDevice::GetDriversDeviceNames(drivers[driverIndex]);
-          if (!devices.IsEmpty()) {
-            adjustedDeviceName = devices[0];
-            adjustedDriverName = drivers[driverIndex];
-            break;
-          }
-        }
-      }
-
-      if (adjustedDriverName.IsEmpty())
-        adjustedDriverName = drivers[0];
-    }
-
-    if (adjustedDeviceName.IsEmpty()) {
-      PStringArray devices = VideoDevice::GetDriversDeviceNames(adjustedDriverName);
-      if (devices.IsEmpty())
-        return NULL;
-
-      adjustedDeviceName = devices[0];
-    }
+    adjustedDeviceName = devices[0];
   }
 
   return VideoDevice::CreateDeviceByName(adjustedDeviceName, adjustedDriverName, pluginMgr);
@@ -246,7 +185,7 @@ void PVideoFrameInfo::PrintOn(ostream & strm) const
 
 PBoolean PVideoFrameInfo::SetFrameSize(unsigned width, unsigned height)
 {
-  if (!PAssert(width >= 16 && height >= 16 && width < 65536 && height < 65536, PInvalidParameter))
+  if (!PAssert(width >= 1 && height >= 1 && width < 65536 && height < 65536, PInvalidParameter))
     return false;
 
   frameWidth = width;
@@ -355,7 +294,7 @@ PINDEX PVideoFrameInfo::CalculateFrameBytes(unsigned width, unsigned height,
 {
   for (PINDEX i = 0; i < PARRAYSIZE(ColourFormatBPPTab); i++) {
     if (colourFormat *= ColourFormatBPPTab[i].colourFormat)
-      return width * height * ColourFormatBPPTab[i].bitsPerPixel/8;
+      return  height * ((width * ColourFormatBPPTab[i].bitsPerPixel/8 + ColourFormatBPPTab[i].alignmentMinus1) & (~ColourFormatBPPTab[i].alignmentMinus1));
   }
   return 0;
 }
@@ -440,10 +379,12 @@ static struct {
     { "NTSC",   720,                      480                       },
     { "PAL",    768,                      576                       },
     { "HD480",  PVideoDevice::HD480Width, PVideoDevice::HD480Height },
-    { "HDTVP",  PVideoDevice::HD720Width, PVideoDevice::HD720Height },
     { "HD720",  PVideoDevice::HD720Width, PVideoDevice::HD720Height },
-    { "HDTVI",  PVideoDevice::HD1080Width,PVideoDevice::HD1080Height},
+    { "HDTVP",  PVideoDevice::HD720Width, PVideoDevice::HD720Height },
+    { "720p",   PVideoDevice::HD720Width, PVideoDevice::HD720Height },
     { "HD1080", PVideoDevice::HD1080Width,PVideoDevice::HD1080Height},
+    { "HDTVI",  PVideoDevice::HD1080Width,PVideoDevice::HD1080Height},
+    { "1080p",  PVideoDevice::HD1080Width,PVideoDevice::HD1080Height},
 
     { "CGA",    320,                      240                       },
     { "VGA",    640,                      480                       },
@@ -543,21 +484,24 @@ PVideoDevice::OpenArgs::OpenArgs()
 }
 
 
+PString PVideoDevice::GetDeviceNameFromOpenArgs(const OpenArgs & args) const
+{
+  if (args.deviceName[0] != '#')
+    return args.deviceName;
+  
+  PStringArray devices = GetDeviceNames();
+  PINDEX id = args.deviceName.Mid(1).AsUnsigned();
+  if (id == 0 || id > devices.GetSize())
+    return PString::Empty();
+  
+  return devices[id-1];
+}
+
+
 PBoolean PVideoDevice::OpenFull(const OpenArgs & args, PBoolean startImmediate)
 {
-  if (args.deviceName[0] == '#') {
-    PStringArray devices = GetDeviceNames();
-    PINDEX id = args.deviceName.Mid(1).AsUnsigned();
-    if (id == 0 || id > devices.GetSize())
-      return false;
-
-    if (!Open(devices[id-1], false))
-      return false;
-  }
-  else {
-    if (!Open(args.deviceName, false))
-      return false;
-  }
+  if (!Open(GetDeviceNameFromOpenArgs(args), false))
+    return false;
 
   if (!SetVideoFormat(args.videoFormat))
     return false;
@@ -637,22 +581,39 @@ int PVideoDevice::GetNumChannels()
 }
 
 
-PBoolean PVideoDevice::SetChannel(int channelNum)
+PStringArray PVideoDevice::GetChannelNames()
 {
-  if (channelNum < 0) { // Seek out the first available channel
-    for (int c = 0; c < GetNumChannels(); c++) {
+  int numChannels = GetNumChannels();
+  PStringArray names(numChannels);
+  for (int c = 0; c < numChannels; ++c)
+    names[c] = PString((char)('A'+c));
+  return names;
+}
+
+
+PBoolean PVideoDevice::SetChannel(int newChannelNumber)
+{
+  if (newChannelNumber < 0) { // No change
+    int numChannels = GetNumChannels();
+    if (channelNumber >= 0 && channelNumber < numChannels)
+      return true;
+
+    // Must change
+    for (int c = 0; c < numChannels; c++) {
       if (SetChannel(c))
         return true;
     }
+
+    PTRACE(2, "PVidDev\tCannot set any possible channel number!");
     return false;
   }
 
-  if (channelNum >= GetNumChannels()) {
-    PTRACE(2, "PVidDev\tSetChannel number (" << channelNum << ") too large.");
+  if (newChannelNumber >= GetNumChannels()) {
+    PTRACE(2, "PVidDev\tSetChannel number (" << newChannelNumber << ") too large.");
     return false;
   }
 
-  channelNumber = channelNum;
+  channelNumber = newChannelNumber;
   return true;
 }
 
@@ -671,7 +632,7 @@ bool PVideoDevice::SetFrameInfoConverter(const PVideoFrameInfo & info)
     return false;
   }
 
-  if (!SetFrameSizeConverter(info.GetFrameWidth(), info.GetFrameHeight())) {
+  if (!SetFrameSizeConverter(info.GetFrameWidth(), info.GetFrameHeight(), info.GetResizeMode())) {
     PTRACE(1, "PVidDev\tCould not set frame size in "
            << (CanCaptureVideo() ? "grabber" : "display") << " to " << info);
     return false;
@@ -803,43 +764,23 @@ PBoolean PVideoDevice::GetFrameSizeLimits(unsigned & minWidth,
 
 PBoolean PVideoDevice::SetFrameSizeConverter(unsigned width, unsigned height, ResizeMode resizeMode)
 {
-  bool isMJPEGCapture = (PVideoFrameInfo::GetColourFormat() == "MJPEG") && CanCaptureVideo();
-
-  if (!isMJPEGCapture || (((width | height) & 0xf) == 0)) {
-    if (SetFrameSize(width, height)) {
-      if (nativeVerticalFlip && (converter == NULL))
-        converter = new PSynonymColour(*this, *this);
-      if (converter != NULL) {
-        converter->SetFrameSize(frameWidth, frameHeight);
-        converter->SetVFlipState(nativeVerticalFlip);
-      }
-      return true;
-    }
-  }
-
-  int outW = width;
-  int outH = height;
-
-  if (isMJPEGCapture) {
-    outW = ((width + 15) / 16) * 16;
-    outH = ((height + 15) / 16) * 16;
-    if (resizeMode == eMaxResizeMode)
-      resizeMode = eCropCentre;
-  }
-
   // Try and get the most compatible physical frame size to convert from/to
-  if (!SetNearestFrameSize(outW, outH)) {
+  if (!SetNearestFrameSize(width, height)) {
     PTRACE(1, "PVidDev\tCannot set an apropriate size to scale from.");
     return false;
   }
 
   // Now create the converter ( if not already exist)
   if (converter == NULL) {
+    if (!nativeVerticalFlip && frameWidth == width && frameHeight == height) {
+      PTRACE(3,"PVidDev\tNo converter required for " << width << 'x' << height);
+      return true;
+    }
+
     PVideoFrameInfo src = *this;
     PVideoFrameInfo dst = *this;
-    if (CanCaptureVideo()) {
+    if (CanCaptureVideo())
       dst.SetFrameSize(width, height);
-    }
     else
       src.SetFrameSize(width, height);
     dst.SetResizeMode(resizeMode);
@@ -849,8 +790,7 @@ PBoolean PVideoDevice::SetFrameSizeConverter(unsigned width, unsigned height, Re
       return false;
     }
   }
-  else
-  {
+  else {
     if (CanCaptureVideo())
       converter->SetDstFrameSize(width, height);
     else
@@ -858,6 +798,8 @@ PBoolean PVideoDevice::SetFrameSizeConverter(unsigned width, unsigned height, Re
     converter->SetResizeMode(resizeMode);
   }
 
+  converter->SetVFlipState(nativeVerticalFlip);
+
   PTRACE(3,"PVidDev\tColour converter used from " << converter->GetSrcFrameWidth() << 'x' << converter->GetSrcFrameHeight() << " [" << converter->GetSrcColourFormat() << "]" << " to " << converter->GetDstFrameWidth() << 'x' << converter->GetDstFrameHeight() << " [" << converter->GetDstColourFormat() << "]");
 
   return true;
@@ -962,61 +904,59 @@ PStringArray PVideoDevice::GetDeviceNames() const
   return PStringArray();
 }
 
-////////////////////////////////////////////////////////////////////////////////////////////
-
-PString PVideoInteractionInfo::AsString(const InputInteractType & ctype)
-{
-	switch (ctype) {
-		case InteractKey:
-			return "Remote Key Press";
-		case InteractMouse:
-			return "Remote Mouse Move/Click";
-		case InteractNavigate:
-			return "Remote Navigation";
-		case InteractRTSP:
-			return "Remote RTSP Commands";
-		case InteractOther:
-			return "Custom/Other";
-	}
-	return PString();
-}
 
 /////////////////////////////////////////////////////////////////////////////////////////////
 
-PVideoInputControl::~PVideoInputControl()  
-{ 
-  Reset(); 
-}
-
-
-PBoolean PVideoInputControl::Pan(long /*value*/, bool /*absolute*/)  
+void PVideoControlInfo::PrintOn(ostream & strm) const
 {
-  return false; 
+  strm << m_type << ": min=" << m_minimum << ", max=" << m_maximum << ", step=" << m_step << ", reset=" << m_reset;
 }
 
 
-PBoolean PVideoInputControl::Tilt(long /*value*/, bool /*absolute*/)  
-{ 
-  return false; 
-}
-
-
-PBoolean PVideoInputControl::Zoom(long /*value*/, bool /*absolute*/)  
-{ 
-  return false; 
-}
-
-
-void PVideoInputControl::Reset()
+int PVideoControlInfo::SetCurrent(int current)
 {
-  PTRACE(4,"CC\tResetting camera to default position.");
-
-  Pan (m_control[PVideoControlInfo::Pan ].m_default, true);
-  Tilt(m_control[PVideoControlInfo::Tilt].m_default, true);
-  Zoom(m_control[PVideoControlInfo::Zoom].m_default, true);
+  if (current < m_minimum)
+    m_current = m_minimum;
+  else if (current > m_maximum)
+    m_current = m_maximum;
+  else
+    m_current = current;
+  return m_current;
 }
 
 
+////////////////////////////////////////////////////////////////////////////////////////////
+
+PVideoInteractionInfo::PVideoInteractionInfo()
+  : m_type(InteractOther)
+{
+}
+
+
+void PVideoInteractionInfo::PrintOn(ostream & strm) const
+{
+  strm << AsString(m_type);
+}
+
+
+PString PVideoInteractionInfo::AsString(const Type & ctype)
+{
+  switch (ctype) {
+    case InteractKey:
+      return "Remote Key Press";
+    case InteractMouse:
+      return "Remote Mouse Move/Click";
+    case InteractNavigate:
+      return "Remote Navigation";
+    case InteractRTSP:
+      return "Remote RTSP Commands";
+    case InteractOther:
+      return "Custom/Other";
+    default :
+      return PString::Empty();
+  }
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoOutputDevice
 
@@ -1118,29 +1058,31 @@ PBoolean PVideoOutputDeviceRGB::SetFrameData(unsigned x, unsigned y,
                                          const BYTE * data,
                                          PBoolean endFrame)
 {
-  PWaitAndSignal m(mutex);
+  {
+    PWaitAndSignal m(mutex);
 
-  if (x+width > frameWidth || y+height > frameHeight || PAssertNULL(data) == NULL)
-    return false;
-
-  if (x == 0 && width == frameWidth && y == 0 && height == frameHeight) {
-    if (converter != NULL)
-      converter->Convert(data, frameStore.GetPointer());
-    else
-      memcpy(frameStore.GetPointer(), data, height*scanLineWidth);
-  }
-  else {
-    if (converter != NULL) {
-      PAssertAlways("Converted output of partial RGB frame not supported");
+    if (x+width > frameWidth || y+height > frameHeight || PAssertNULL(data) == NULL)
       return false;
-    }
 
-    if (x == 0 && width == frameWidth)
-      memcpy(frameStore.GetPointer() + y*scanLineWidth, data, height*scanLineWidth);
+    if (x == 0 && width == frameWidth && y == 0 && height == frameHeight) {
+      if (converter != NULL)
+        converter->Convert(data, frameStore.GetPointer());
+      else
+        memcpy(frameStore.GetPointer(), data, height*scanLineWidth);
+    }
     else {
-      for (unsigned dy = 0; dy < height; dy++)
-        memcpy(frameStore.GetPointer() + (y+dy)*scanLineWidth + x*bytesPerPixel,
-               data + dy*width*bytesPerPixel, width*bytesPerPixel);
+      if (converter != NULL) {
+        PAssertAlways("Converted output of partial RGB frame not supported");
+        return false;
+      }
+
+      if (x == 0 && width == frameWidth)
+        memcpy(frameStore.GetPointer() + y*scanLineWidth, data, height*scanLineWidth);
+      else {
+        for (unsigned dy = 0; dy < height; dy++)
+          memcpy(frameStore.GetPointer() + (y+dy)*scanLineWidth + x*bytesPerPixel,
+                 data + dy*width*bytesPerPixel, width*bytesPerPixel);
+      }
     }
   }
 
@@ -1231,68 +1173,122 @@ PBoolean PVideoInputDevice::CanCaptureVideo() const
   return true;
 }
 
-static const char videoInputPluginBaseClass[] = "PVideoInputDevice";
-
 
 PStringArray PVideoInputDevice::GetDriverNames(PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return pluginMgr->GetPluginsProviding(videoInputPluginBaseClass);
+  return PPluginManager::GetPluginsProviding(pluginMgr, PPlugin_PVideoInputDevice::ServiceType(), false);
 }
 
 
 PStringArray PVideoInputDevice::GetDriversDeviceNames(const PString & driverName, PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return pluginMgr->GetPluginsDeviceNames(driverName, videoInputPluginBaseClass);
+  // Give precedence to drivers like camera grabbers, Window
+  static const char * const PrioritisedDrivers[] = {
+#ifdef P_DIRECTSHOW
+    P_DIRECT_SHOW_DRIVER,
+#endif
+#ifdef WIN32
+    P_VIDEO_FOR_WINDOWS_DRIVER,
+#endif
+    "V4L",
+    "V4L2",
+    "1394DC",
+    "1394AVC",
+    "BSDCAPTURE",
+    P_NULL_VIDEO_DRIVER,
+#ifdef P_APPSHARE
+    P_APPLICATION_VIDEO_DRIVER,
+#endif
+#if P_VIDFILE
+    P_VIDEO_FILE_DRIVER,
+#endif
+    NULL
+  };
+  return PPluginManager::GetPluginDeviceNames(pluginMgr, driverName, PPlugin_PVideoInputDevice::ServiceType(), 0, PrioritisedDrivers);
 }
 
 
 PVideoInputDevice * PVideoInputDevice::CreateDevice(const PString &driverName, PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return (PVideoInputDevice *)pluginMgr->CreatePluginsDevice(driverName, videoInputPluginBaseClass);
+  return PPluginManager::CreatePluginAs<PVideoInputDevice>(pluginMgr, driverName, PPlugin_PVideoInputDevice::ServiceType());
 }
 
 
 PVideoInputDevice * PVideoInputDevice::CreateDeviceByName(const PString & deviceName, const PString & driverName, PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return (PVideoInputDevice *)pluginMgr->CreatePluginsDeviceByName(deviceName, videoInputPluginBaseClass,0,driverName);
+  return PPluginManager::CreatePluginAs<PVideoInputDevice>(pluginMgr, driverName.IsEmpty() ? deviceName : driverName, PPlugin_PVideoInputDevice::ServiceType());
 }
 
 
-PBoolean PVideoInputDevice::GetDeviceCapabilities(const PString & deviceName, Capabilities * caps, PPluginManager * pluginMgr)
+bool PVideoInputDevice::GetDeviceCapabilities(Capabilities * capabilities) const
+{
+  return GetDeviceCapabilities(GetDeviceName(), capabilities);
+}
+
+
+bool PVideoInputDevice::GetDeviceCapabilities(const PString & deviceName, Capabilities * caps, PPluginManager * pluginMgr)
 {
   return GetDeviceCapabilities(deviceName, "*", caps, pluginMgr);
 }
 
 
-PBoolean PVideoInputDevice::GetDeviceCapabilities(const PString & deviceName, const PString & driverName, Capabilities * caps, PPluginManager * pluginMgr)
+bool PVideoInputDevice::GetDeviceCapabilities(const PString & deviceName, const PString & driverName, Capabilities * caps, PPluginManager * pluginMgr)
 {
   if (pluginMgr == NULL)
     pluginMgr = &PPluginManager::GetPluginManager();
 
-  return pluginMgr->GetPluginsDeviceCapabilities(videoInputPluginBaseClass,driverName,deviceName, caps);
+  return pluginMgr->GetPluginsDeviceCapabilities(PPlugin_PVideoInputDevice::ServiceType(), driverName, deviceName, caps);
 }
 
-PVideoInputControl * PVideoInputDevice::GetVideoInputControls()
+
+PVideoInputDevice::Capabilities::Capabilities()
+  : m_channels(0)
+  , m_brightness(false)
+  , m_contrast(false)
+  , m_saturation(false)
+  , m_hue(false)
+  , m_gamma(false)
+  , m_exposure(false)
 {
-	return NULL;
+}
+
+
+void PVideoInputDevice::Capabilities::PrintOn(ostream & strm) const
+{
+  std::streamsize indent = strm.precision();
+
+  strm << setw(indent) << ' ';
+  if (framesizes.empty())
+    strm << "No frame sizes.\n";
+  else {
+    strm << "Frame Sizes:\n";
+    for (list<PVideoFrameInfo>::const_iterator it = framesizes.begin(); it != framesizes.end(); ++it)
+      strm << setw(indent+2) << ' ' << *it << '\n';
+  }
+
+  strm << setw(indent) << ' ';
+  if (controls.empty())
+    strm << "No controls.\n";
+  else {
+    strm << "Controls:\n";
+    for (list<PVideoControlInfo>::const_iterator it = controls.begin(); it != controls.end(); ++it)
+      strm << setw(indent+2) << ' ' << *it << '\n';
+  }
+
+  strm << setw(indent) << ' ';
+  if (interactions.empty())
+    strm << "No interactions.\n";
+  else {
+    strm << "Interactions:\n";
+    for (list<PVideoInteractionInfo>::const_iterator it = interactions.begin(); it != interactions.end(); ++it)
+      strm << setw(indent+2) << ' ' << *it << '\n';
+  }
 }
 
 
 PVideoInputDevice * PVideoInputDevice::CreateOpenedDevice(const PString & driverName,
                                                           const PString & deviceName,
-                                                          PBoolean startImmediate,
+                                                          bool startImmediate,
                                                           PPluginManager * pluginMgr)
 {
   PString adjustedDeviceName = deviceName;
@@ -1309,13 +1305,16 @@ PVideoInputDevice * PVideoInputDevice::CreateOpenedDevice(const PString & driver
 
 
 PVideoInputDevice * PVideoInputDevice::CreateOpenedDevice(const OpenArgs & args,
-                                                          PBoolean startImmediate)
+                                                          bool startImmediate)
 {
   OpenArgs adjustedArgs = args;
   PVideoInputDevice * device = CreateDeviceWithDefaults<PVideoInputDevice>(adjustedArgs.deviceName, args.driverName, NULL);
   if (device == NULL)
     return NULL;
 
+  device->colourFormat = args.colourFormat;
+  device->frameWidth = args.width;
+  device->frameHeight = args.height;
   if (device->OpenFull(adjustedArgs, startImmediate))
     return device;
 
@@ -1336,37 +1335,32 @@ PBoolean PVideoInputDevice::SetNearestFrameSize(unsigned width, unsigned height)
 
   std::list<PVideoFrameInfo>::iterator it;
 
-  bool isMJPEG = PVideoFrameInfo::GetColourFormat() == "MJPEG";
+  // First try and pick one with the same width
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    if (it->GetFrameWidth() == width)
+      return SetFrameSize(width, it->GetFrameHeight());
+  }
 
-  if (!isMJPEG) {
+  // Then try for the same height
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    if (it->GetFrameHeight() == height)
+      return SetFrameSize(it->GetFrameWidth(), height);
+  }
 
-    // First try and pick one with the same width
-    for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
-      if (it->GetFrameWidth() == width)
-        return SetFrameSize(width, it->GetFrameHeight());
-    }
+  // Then try for double the size
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    unsigned w, h;
+    it->GetFrameSize(w, h);
+    if (w == width*2 && h == height*2)
+      return SetFrameSize(w, h);
+  }
 
-    // Then try for the same height
-    for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
-      if (it->GetFrameHeight() == height)
-        return SetFrameSize(it->GetFrameWidth(), height);
-    }
-
-    // Then try for double the size
-    for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
-      unsigned w, h;
-      it->GetFrameSize(w, h);
-      if (w == width*2 && h == height*2)
-        return SetFrameSize(w, h);
-    }
-
-    // Then try for half the size
-    for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
-      unsigned w, h;
-      it->GetFrameSize(w, h);
-      if (w == width/2 && h == height/2)
-        return SetFrameSize(w, h);
-    }
+  // Then try for half the size
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    unsigned w, h;
+    it->GetFrameSize(w, h);
+    if (w == width/2 && h == height/2)
+      return SetFrameSize(w, h);
   }
 
   // Now try and pick one that has the nearest number of pixels in total.
@@ -1379,17 +1373,12 @@ PBoolean PVideoInputDevice::SetNearestFrameSize(unsigned width, unsigned height)
     int d = w*h - pixels;
     if (d < 0)
       d = -d;
-    if (diff > d && 
-          // Ensure we don't pick one dimension greater and one 
-          // lower because we can't shrink one and grow the other.
-          ((w < width && h < height) || 
-	       (w > width && h > height))
-    ) {
-      if (!isMJPEG || (((w | h) & 0xf) == 0)) {
-        diff = d;
-        widthToUse = w;
-        heightToUse = h;
-      }
+    // Ensure we don't pick one dimension greater and one 
+    // lower because we can't shrink one and grow the other.
+    if (diff > d && ((w < width && h < height) || (w > width && h > height))) {
+      diff = d;
+      widthToUse = w;
+      heightToUse = h;
     }
   }
 
@@ -1443,6 +1432,12 @@ int PVideoInputDevice::GetCaptureMode() const
 }
 
 
+bool PVideoInputDevice::SetControl(PVideoControlInfo::Types, int, ControlMode)
+{
+  return false;
+}
+
+
 PBoolean PVideoOutputDevice::SetFrameData(
       unsigned x,
       unsigned y,
@@ -1467,7 +1462,7 @@ PBoolean PVideoOutputDevice::SetFrameData(
       const BYTE * data,
       PBoolean endFrame,
       bool & keyFrameNeeded,
-	  const void * /*mark*/
+    const void * /*mark*/
 )
 {
   return SetFrameData(x, y, width, height, data, endFrame, keyFrameNeeded);
@@ -1476,54 +1471,55 @@ PBoolean PVideoOutputDevice::SetFrameData(
 
 PBoolean PVideoOutputDevice::DisableDecode() 
 {
-	return false;
+  return false;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
-static const char videoOutputPluginBaseClass[] = "PVideoOutputDevice";
-
-
 PStringArray PVideoOutputDevice::GetDriverNames(PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return pluginMgr->GetPluginsProviding(videoOutputPluginBaseClass);
+  return PPluginManager::GetPluginsProviding(pluginMgr, PPlugin_PVideoOutputDevice::ServiceType(), false);
 }
 
 
 PStringArray PVideoOutputDevice::GetDriversDeviceNames(const PString & driverName, PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
+  // Give precedence to drivers like camera grabbers, Window
+  static const char * const PrioritisedDrivers[] = {
+#ifdef WIN32
+    P_MSWIN_VIDEO_DRIVER,
+#endif
+#if P_SDL
+    P_SDL_VIDEO_DRIVER,
+#endif
+    P_FAKE_VIDEO_DRIVER,
+    P_NULL_VIDEO_DRIVER,
+#if P_VIDFILE
+    P_VIDEO_FILE_DRIVER,
+#endif
+    NULL
+  };
 
-  return pluginMgr->GetPluginsDeviceNames(driverName, videoOutputPluginBaseClass);
+  return PPluginManager::GetPluginDeviceNames(pluginMgr, driverName, PPlugin_PVideoOutputDevice::ServiceType(), 0, PrioritisedDrivers);
 }
 
 
 PVideoOutputDevice * PVideoOutputDevice::CreateDevice(const PString & driverName, PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return (PVideoOutputDevice *)pluginMgr->CreatePluginsDevice(driverName, videoOutputPluginBaseClass);
+  return PPluginManager::CreatePluginAs<PVideoOutputDevice>(pluginMgr, driverName, PPlugin_PVideoOutputDevice::ServiceType());
 }
 
 
 PVideoOutputDevice * PVideoOutputDevice::CreateDeviceByName(const PString & deviceName, const PString & driverName, PPluginManager * pluginMgr)
 {
-  if (pluginMgr == NULL)
-    pluginMgr = &PPluginManager::GetPluginManager();
-
-  return (PVideoOutputDevice *)pluginMgr->CreatePluginsDeviceByName(deviceName, videoOutputPluginBaseClass, 0, driverName);
+  return PPluginManager::CreatePluginAs<PVideoOutputDevice>(pluginMgr, driverName.IsEmpty() ? deviceName : driverName, PPlugin_PVideoOutputDevice::ServiceType());
 }
 
 
 PVideoOutputDevice * PVideoOutputDevice::CreateOpenedDevice(const PString &driverName,
                                                             const PString &deviceName,
-                                                            PBoolean startImmediate,
+                                                            bool startImmediate,
                                                             PPluginManager * pluginMgr)
 {
   PString adjustedDeviceName = deviceName;
@@ -1540,7 +1536,7 @@ PVideoOutputDevice * PVideoOutputDevice::CreateOpenedDevice(const PString &drive
 
 
 PVideoOutputDevice * PVideoOutputDevice::CreateOpenedDevice(const OpenArgs & args,
-                                                            PBoolean startImmediate)
+                                                            bool startImmediate)
 {
   OpenArgs adjustedArgs = args;
   PVideoOutputDevice * device = CreateDeviceWithDefaults<PVideoOutputDevice>(adjustedArgs.deviceName, args.driverName, NULL);
