@@ -30,9 +30,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29351 $
+ * $Revision: 34236 $
  * $Author: rjongbloed $
- * $Date: 2013-03-27 18:43:51 -0500 (Wed, 27 Mar 2013) $
+ * $Date: 2015-10-24 19:32:05 +0100 (Sat, 24 Oct 2015) $
  */
 
 
@@ -40,26 +40,32 @@
 
 #ifdef P_DIRECTSHOW
 
+#define P_FORCE_STATIC_PLUGIN 1
+
 #include <ptlib/videoio.h>
 #include <ptlib/vconvert.h>
+#include <ptclib/guid.h>
 
 #include <ptlib/msos/ptlib/pt_atl.h>
 
 #include <functional>
 
 
+#define PTraceModule() "DShow"
+
+
 #ifdef _WIN32_WCE
 
   static const GUID MEDIASUBTYPE_IYUV = { 0x56555949, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71 };
   #define CLSID_CaptureGraphBuilder2 CLSID_CaptureGraphBuilder
 
-  #ifdef DEBUG
+  #ifdef _DEBUG
     /* Only the release version is provided as a .lib file, so we need to
        make sure that the compilation does NOT have the extra fields/functions
        that are added when DEBUG version. */
-    #undef DEBUG
+    #undef _DEBUG
     #include <streams.h>
-    #define DEBUG
+    #define _DEBUG
   #else
     #include <streams.h>
   #endif
@@ -100,22 +106,6 @@
   #include <ks.h>
   #include <ksmedia.h>
 
-  class PVideoInputControl_DirectShow : public PVideoInputControl
-  {
-      PCLASSINFO(PVideoInputControl_DirectShow, PVideoInputControl);
-    public:
-      PVideoInputControl_DirectShow(IAMCameraControl * pCC);
-
-      // overrides
-      virtual bool Pan(long value, bool absolute = false);
-      virtual bool Tilt(long value, bool absolute = false);
-      virtual bool Zoom(long value, bool absolute = false);
-
-    protected:
-      bool Control(PVideoControlInfo::Types type, long value, bool absolute);
-      IAMCameraControl * m_pCC;
-  };
-
 
   #ifdef P_DIRECTSHOW_QEDIT_H
 
@@ -169,16 +159,52 @@
     #pragma comment(lib, "quartz.lib")
   #endif
 
+  class PSampleGrabberCB : public ISampleGrabberCB 
+  {
+    private:
+      PBYTEArray m_buffer;
+      bool       m_stopped;
+      PINDEX     m_actualSize;
+      PMutex     m_mutex;
+      PSyncPoint m_frameReady;
+      PINDEX     m_skipInitialGrabs;
+#if PTRACING
+      unsigned     m_totalFrames;
+      PSimpleTimer m_totalTime;
+#endif
+
+    public:
+      PSampleGrabberCB();
+      ~PSampleGrabberCB();
+
+      void Start();
+      void Stop();
+
+      // Fake out any COM ref counting
+      STDMETHODIMP_(ULONG) AddRef() { return 2; }
+      STDMETHODIMP_(ULONG) Release() { return 1; }
+
+      // Fake out any COM QI'ing
+      STDMETHODIMP QueryInterface(REFIID riid, void ** ppv);
+
+      // We don't implement this one
+      STDMETHODIMP SampleCB( double /*SampleTime*/, IMediaSample * /*pSample*/) { return 0; }
+
+      // The sample grabber is calling us back on its deliver thread.
+      STDMETHODIMP BufferCB(double PTRACE_PARAM(dblSampleTime), BYTE * buffer, long size);
+
+      bool GetData(BYTE * data, PINDEX maxSize, PINDEX & actualSize);
+  };
+
 #endif // _WIN32_WCE
 
 #ifdef _MSC_VER
   #pragma comment(lib, "strmiids.lib")
-  #pragma message("Direct Show video support enabled")
 #endif
 
 
 static long const InputControlPropertyCode[PVideoControlInfo::NumTypes] = {
-  CameraControl_Pan, CameraControl_Tilt, CameraControl_Zoom
+  CameraControl_Pan, CameraControl_Tilt, CameraControl_Zoom, CameraControl_Focus
 };
 
 
@@ -199,7 +225,7 @@ class PVideoInputDevice_DirectShow : public PVideoInputDevice
     static PBoolean GetDeviceCapabilities(const PString & deviceName, Capabilities * capabilities);
     virtual bool GetDeviceCapabilities(Capabilities * capabilities) const;
 #ifndef _WIN32_WCE
-    virtual PVideoInputControl * GetVideoInputControls();
+    virtual bool SetControl(PVideoControlInfo::Types type, int value, ControlMode mode);
 #endif
 
     virtual PBoolean Open(const PString & deviceName, PBoolean startImmediate);
@@ -223,10 +249,10 @@ class PVideoInputDevice_DirectShow : public PVideoInputDevice
     bool BindCaptureDevice(const PString & devName);
     bool PlatformOpen();
     PINDEX GetCurrentBufferSize();
-    bool GetCurrentBufferData(BYTE * data);
+    bool GetCurrentBufferData(BYTE * data, PINDEX & bufferSize);
     bool SetPinFormat(unsigned useDefaultColourOrSize = 0);
-    bool SetControlCommon(long control, int newValue);
-    int GetControlCommon(long control);
+    bool SetAttributeCommon(long control, int newValue);
+    int GetAttributeCommon(long control) const;
 
 
     // DirectShow 
@@ -240,13 +266,14 @@ class PVideoInputDevice_DirectShow : public PVideoInputDevice
     PSampleGrabber               * m_pSampleGrabber;
 #else
     CComPtr<ISampleGrabber>        m_pSampleGrabber;
-    CComPtr<ISampleGrabberCB>      m_pSampleGrabberCB;
+    CComPtr<PSampleGrabberCB>      m_pSampleGrabberCB;
     CComPtr<IAMCameraControl>      m_pCameraControls;
 #endif
     CComPtr<IBaseFilter>           m_pNullRenderer;
     CComPtr<IMediaControl>         m_pMediaControl;
 
     PINDEX     m_maxFrameBytes;
+    bool       m_fixedSizeFrames; // Not JPEG
     PBYTEArray m_tempFrame;
     PMutex     m_lastFrameMutex;
 };
@@ -257,17 +284,24 @@ PCREATE_VIDINPUT_PLUGIN(DirectShow);
 
 ////////////////////////////////////////////////////////////////////
 
-static struct {
-    const char * m_colourFormat;
-    GUID         m_guid;
-} const ColourFormat2GUID[] =
+#ifndef __MINGW32__
+static const GUID MEDIASUBTYPE_I420 = { 0x30323449, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71 };
+#endif
+
+static PString GUID2Format(GUID guid)
 {
+  static struct {
+      const char * m_colourFormat;
+      GUID         m_guid;
+  } const ColourFormat2GUID[] =
+  {
     { "Grey",    MEDIASUBTYPE_RGB8   },
     { "BGR32",   MEDIASUBTYPE_RGB32  }, /* Microsoft assumes that we are in little endian */
     { "BGR24",   MEDIASUBTYPE_RGB24  },
     { "RGB565",  MEDIASUBTYPE_RGB565 },
     { "RGB555",  MEDIASUBTYPE_RGB555 },
     { "YUV420P", MEDIASUBTYPE_IYUV   },  // aka I420
+    { "YUV420P", MEDIASUBTYPE_I420   },
     { "YUV420P", MEDIASUBTYPE_YV12   },
     { "YUV411",  MEDIASUBTYPE_Y411   },
     { "YUV411P", MEDIASUBTYPE_Y41P   },
@@ -275,20 +309,14 @@ static struct {
     { "YUY2",    MEDIASUBTYPE_YUY2   },
     { "MJPEG",   MEDIASUBTYPE_MJPG   },
     { "UYVY422", MEDIASUBTYPE_UYVY   }
-};
+  };
 
-static PString GUID2Format(GUID guid)
-{
-   for (int j = 0; j < sizeof(ColourFormat2GUID)/sizeof(ColourFormat2GUID[0]); j++) {
+  for (int j = 0; j < sizeof(ColourFormat2GUID)/sizeof(ColourFormat2GUID[0]); j++) {
     if (guid == ColourFormat2GUID[j].m_guid)
       return ColourFormat2GUID[j].m_colourFormat;
-   }
+  }
 
-   wchar_t guidName[256];
-   if (StringFromGUID2(guid, guidName, sizeof(guidName)) <= 0)
-       return "UNKNOWN"; // Can't use this entry!
-
-   return guidName;
+  return PGloballyUniqueID(guid).AsString();
 }
 
 
@@ -328,10 +356,10 @@ class MediaTypePtr
       CoTaskMemFree(pointer);
     }
 
-    operator AM_MEDIA_TYPE *()        const { return  pointer; }
-    AM_MEDIA_TYPE & operator*()       const { return *pointer; }
-    AM_MEDIA_TYPE** operator&()             { return &pointer; }
-	AM_MEDIA_TYPE* operator->()       const { return  pointer; }
+    operator AM_MEDIA_TYPE *()  const { return  pointer; }
+    AM_MEDIA_TYPE & operator*() const { return *pointer; }
+    AM_MEDIA_TYPE** operator&()       { return &pointer; }
+    AM_MEDIA_TYPE* operator->() const { return  pointer; }
 
   private:
     MediaTypePtr(const MediaTypePtr &) { }
@@ -344,11 +372,12 @@ class MediaTypePtr
 
 PVideoInputDevice_DirectShow::PVideoInputDevice_DirectShow()
   : m_maxFrameBytes(0)
+  , m_fixedSizeFrames(true)
 #ifdef _WIN32_WCE
   , m_pSampleGrabber(NULL)
 #endif
 {
-  PTRACE(4, "DShow\tVideo Device Instance");
+  PTRACE(5, "Constructed " << this);
 
   PThread::Current()->CoInitialise();
 }
@@ -357,6 +386,7 @@ PVideoInputDevice_DirectShow::PVideoInputDevice_DirectShow()
 PVideoInputDevice_DirectShow::~PVideoInputDevice_DirectShow()
 {
   Close();
+  PTRACE(5, "Destroyed " << this);
 }
 
 
@@ -377,6 +407,9 @@ PBoolean PVideoInputDevice_DirectShow::GetDeviceCapabilities(const PString & dev
 
 bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) const
 {
+  if (m_pCameraOutPin == NULL)
+    return false;
+
   CComPtr<IAMStreamConfig> pStreamConfig;
 #ifdef __MINGW32__
   PCOM_RETURN_ON_FAILED(m_pCameraOutPin->QueryInterface,(IID_IAMStreamConfig, (void**)&pStreamConfig));
@@ -390,7 +423,7 @@ bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) co
   /* Sanity check: just to be sure that the Streamcaps is a VIDEOSTREAM and not AUDIOSTREAM */
   VIDEO_STREAM_CONFIG_CAPS scc;
   if (sizeof(scc) != iSize) {
-    PTRACE(1, "DShow\tBad Capapabilities (not a VIDEO_STREAM_CONFIG_CAPS)");
+    PTRACE(1, "Bad Capapabilities (not a VIDEO_STREAM_CONFIG_CAPS)");
     return false;
   }
 
@@ -406,7 +439,8 @@ bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) co
       fsizes.insert(PVideoFrameInfo(scc.MaxOutputSize.cx,
                                     scc.MaxOutputSize.cy,
                                     GUID2Format(pMediaFormat->subtype),
-                                    10000000/(unsigned)scc.MinFrameInterval));
+                                    10000000/(unsigned)scc.MinFrameInterval,
+                                    PVideoFrameInfo::eMaxResizeMode));
   }
 
   if (fsizes.empty())
@@ -414,21 +448,23 @@ bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) co
 
   // Sort so we have unique sizes from largest to smallest
   for (std::set<PVideoFrameInfo, std::greater<PVideoFrameInfo> >::iterator it = fsizes.begin(); it != fsizes.end(); ++it) {
-    PTRACE(5, "DShow\tFormat["<< caps->framesizes.size() << "] = " << *it);
+    PTRACE(5, "Format["<< caps->framesizes.size() << "] = " << *it);
     caps->framesizes.push_back(*it);
   }
 
 #ifndef _WIN32_WCE
-  // Query for camera controls
-  if (m_pCameraControls != NULL) {
-    PVideoInputControl_DirectShow controls(m_pCameraControls);
-    for (PVideoControlInfo::Types type = PVideoControlInfo::BeginTypes; type < PVideoControlInfo::EndTypes; ++type) {
-      if (controls.IsValid(type))
-        caps->controls.push_back(controls.GetControl(type));
-    }
+  for (PVideoControlInfo::Types type = PVideoControlInfo::BeginTypes; type < PVideoControlInfo::EndTypes; ++type) {
+    if (m_controlInfo[type].IsValid())
+      caps->controls.push_back(m_controlInfo[type]);
   }
 #endif
 
+  caps->m_brightness = GetAttributeCommon(VideoProcAmp_Brightness) >= 0;
+  caps->m_contrast   = GetAttributeCommon(VideoProcAmp_Contrast) >= 0;
+  caps->m_saturation = GetAttributeCommon(VideoProcAmp_Saturation) >= 0;
+  caps->m_hue        = GetAttributeCommon(VideoProcAmp_Hue) >= 0;
+  caps->m_gamma      = GetAttributeCommon(VideoProcAmp_Gamma) >= 0;
+
   return true;
 }
 
@@ -436,7 +472,7 @@ bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) co
 bool PVideoInputDevice_DirectShow::SetPinFormat(unsigned useDefaultColourOrSize)
 {
   if (m_pCameraOutPin == NULL) {
-    PTRACE(2, "DShow\tCamera output pin is NULL!");
+    PTRACE(2, "Camera output pin is NULL!");
     return false;
   }
 
@@ -464,43 +500,62 @@ bool PVideoInputDevice_DirectShow::SetPinFormat(unsigned useDefaultColourOrSize)
         (
           useDefaultColourOrSize >= 2 ||
           (
-            scc.MaxOutputSize.cx == (LONG)frameWidth &&
-            scc.MaxOutputSize.cy == (LONG)frameHeight &&
+            scc.MinOutputSize.cx <= (LONG)frameWidth  && (LONG)frameWidth  <= scc.MaxOutputSize.cx &&
+            scc.MinOutputSize.cy <= (LONG)frameHeight && (LONG)frameHeight <= scc.MaxOutputSize.cy &&
             (
               useDefaultColourOrSize >= 1 ||
               GUID2Format(pMediaFormat->subtype) == colourFormat
             )
           )
         )) {
-
-      bool running = IsCapturing();
-      if (running)
-        PCOM_RETURN_ON_FAILED(m_pMediaControl->Stop,());
-
       VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER *)pMediaFormat->pbFormat;
-      pVih->AvgTimePerFrame = 10000000 / frameRate;
-      PCOM_RETURN_ON_FAILED(pStreamConfig->SetFormat,(pMediaFormat));
 
       if (useDefaultColourOrSize >= 1) {
         colourFormat = GUID2Format(pMediaFormat->subtype);
         if (useDefaultColourOrSize >= 2) {
           frameWidth = scc.MaxOutputSize.cx;
           frameHeight = scc.MaxOutputSize.cy;
+          PTRACE(3, "Camera format forced resolution to " << *this);
         }
+        else
+          PTRACE(3, "Camera format forced colour to " << *this);
       }
 
       m_selectedGUID = pMediaFormat->subtype;
       m_maxFrameBytes = CalculateFrameBytes(frameWidth, frameHeight, colourFormat);
+      m_fixedSizeFrames = colourFormat.Find("JPEG") == P_MAX_INDEX;
+
+      if (pVih->bmiHeader.biHeight > 0 && colourFormat.NumCompare("BGR") == EqualTo) {
+        nativeVerticalFlip = true;
+        PTRACE(3, "Image up side down");
+      }
+
+      bool running = IsCapturing();
+      if (running)
+        PCOM_RETURN_ON_FAILED(m_pMediaControl->Stop,());
+
+      pVih->AvgTimePerFrame = 10000000 / frameRate;
+      pVih->bmiHeader.biWidth = frameWidth;
+      pVih->bmiHeader.biHeight = pVih->bmiHeader.biHeight < 0 ? -(LONG)frameHeight : frameHeight;
+      pVih->bmiHeader.biSizeImage = m_maxFrameBytes;
+      PCOM_RETURN_ON_FAILED(pStreamConfig->SetFormat,(pMediaFormat));
 
       if (running)
         PCOM_RETURN_ON_FAILED(m_pMediaControl->Run,());
 
-      PTRACE(4, "DShow\tCamera format set to " << *this);
+      PTRACE(4, "Camera format set to " << *this);
       return true;
     }
+    else {
+      PTRACE(6, "Tested format: "
+             << GUID2Format(pMediaFormat->subtype) << ' '
+             << scc.MinOutputSize.cx << 'x' << scc.MinOutputSize.cy
+             << ".."
+             << scc.MaxOutputSize.cx << 'x' << scc.MaxOutputSize.cy << ' '
+             << scc.MinFrameInterval << ".." << scc.MaxFrameInterval);
+    }
   }
 
-  PTRACE(2, "DShow\tCamera formats available could not be matched to " << *this);
   return false;
 }
 
@@ -511,7 +566,7 @@ PBoolean PVideoInputDevice_DirectShow::Open(const PString & devName,
   Close();
 
   if (devName.IsEmpty()) {
-    PTRACE(2, "DShow\tUnable to Bind to empty device");
+    PTRACE(2, "Unable to Bind to empty device");
     return false;
   }
 
@@ -549,9 +604,12 @@ PBoolean PVideoInputDevice_DirectShow::Open(const PString & devName,
   PCOM_RETURN_ON_FAILED(pEnum->Reset,());
   PCOM_RETURN_ON_FAILED(pEnum->Next,(1, &m_pCameraOutPin, NULL));
 
+  PTRACE(4, "Attempting to use " << *this);
   // Set the format of the Output pin of the camera
-  if (!(SetPinFormat(0) || SetPinFormat(1) || SetPinFormat(2)))
+  if (!(SetPinFormat(0) || SetPinFormat(1) || SetPinFormat(2))) {
+    PTRACE(2, "Camera formats available could not be matched to " << *this);
     return false;
+  }
 
   if (!PlatformOpen())
     return false;
@@ -559,7 +617,7 @@ PBoolean PVideoInputDevice_DirectShow::Open(const PString & devName,
   if (startImmediate) 
     Start();
 
-  PTRACE(4, "DShow\tDevice " << devName << " open.");
+  PTRACE(4, "Device " << devName << " opened as " << this);
   deviceName = devName;
   return true;
 }
@@ -576,11 +634,13 @@ PBoolean PVideoInputDevice_DirectShow::Close()
   if (!IsOpen())
     return false;
 
-  PTRACE(4, "DShow\tTorn Down.");
+  PTRACE(4, "Closing " << this);
 
   // Stop Camera Graph
   Stop();
 
+  m_lastFrameMutex.Wait();
+
   // Release filters
 #ifdef _WIN32_WCE
   if (m_pSampleGrabber != NULL) {
@@ -603,6 +663,8 @@ PBoolean PVideoInputDevice_DirectShow::Close()
   // Relase DirectShow Graph
   m_pGraphBuilder.Release(); 
 
+  m_lastFrameMutex.Signal();
+
   return true;
 }
 
@@ -610,7 +672,7 @@ PBoolean PVideoInputDevice_DirectShow::Close()
 PBoolean PVideoInputDevice_DirectShow::Start()
 {
   if (!IsOpen()) {
-    PTRACE(3, "DShow\tNot open.");
+    PTRACE(3, "Not open.");
     return false;
   }
 
@@ -619,7 +681,9 @@ PBoolean PVideoInputDevice_DirectShow::Start()
 
   PCOM_RETURN_ON_FAILED(m_pMediaControl->Run,());
 
-  PTRACE(4, "DShow\tVideo Started.");
+  m_pSampleGrabberCB->Start();
+
+  PTRACE(4, "Video Started.");
   return true;
 }
 
@@ -627,7 +691,7 @@ PBoolean PVideoInputDevice_DirectShow::Start()
 PBoolean PVideoInputDevice_DirectShow::Stop()
 {
   if (!IsOpen()) {
-    PTRACE(3, "DShow\tNot open.");
+    PTRACE(3, "Not open.");
     return false;
   }
 
@@ -637,7 +701,7 @@ PBoolean PVideoInputDevice_DirectShow::Stop()
   // Use Pause() not Stop() as the latter is to much of a stop and takes too long to restart
   PCOM_RETURN_ON_FAILED(m_pMediaControl->Pause,());
 
-  PTRACE(3, "DShow\tVideo Stopped.");
+  m_pSampleGrabberCB->Stop();
   return true;
 }
 
@@ -646,7 +710,7 @@ PBoolean PVideoInputDevice_DirectShow::IsCapturing()
 {
   OAFilterState state;
   PCOM_RETURN_ON_FAILED(m_pMediaControl->GetState,(0, &state));
-  return state == State_Running;
+  return state != State_Stopped;
 }
 
 
@@ -719,7 +783,7 @@ bool PVideoInputDevice_DirectShow::FlowControl(const void * flowData)
             r =  90000/options[i+1].AsInteger();
     }
 
-    PTRACE(4, "DShow\tAdjusting to new H: " << h << " W: " << w << " R: " << r);
+    PTRACE(4, "Adjusting to new H: " << h << " W: " << w << " R: " << r);
     m_lastFrameMutex.Wait();
     SetFrameSize(w,h);
     SetFrameRate(r);
@@ -742,19 +806,20 @@ PBoolean PVideoInputDevice_DirectShow::GetFrameDataNoDelay(BYTE * destFrame, PIN
 {
   PWaitAndSignal mutex(m_lastFrameMutex);
 
-  PTRACE(6, "DShow\tGrabbing Frame");
-
   PINDEX bufferSize = GetCurrentBufferSize();
+  PTRACE(6, "Grabbing Frame " << frameWidth << 'x' << frameHeight << " (" << bufferSize << ')');
+
   if (converter != NULL) {
-    if (!GetCurrentBufferData(m_tempFrame.GetPointer(bufferSize)))
+    if (!GetCurrentBufferData(m_tempFrame.GetPointer(bufferSize), bufferSize))
       return false;
-    if (!converter->Convert(m_tempFrame, destFrame, bufferSize, bytesReturned))
+    converter->SetSrcFrameBytes(bufferSize);
+    if (!converter->Convert(m_tempFrame, destFrame, bytesReturned))
       return false;
   }
   else {
     if (!PAssert(bufferSize <= m_maxFrameBytes, PLogicError))
       return false;
-    if (!GetCurrentBufferData(destFrame))
+    if (!GetCurrentBufferData(destFrame, bufferSize))
       return false;
     if (bytesReturned != NULL)
       *bytesReturned = bufferSize;
@@ -764,7 +829,7 @@ PBoolean PVideoInputDevice_DirectShow::GetFrameDataNoDelay(BYTE * destFrame, PIN
 }
 
 
-int PVideoInputDevice_DirectShow::GetControlCommon(long control)
+int PVideoInputDevice_DirectShow::GetAttributeCommon(long control) const
 {
   CComPtr<IAMVideoProcAmp> pVideoProcAmp;
   if (PCOM_FAILED(m_pCaptureFilter->QueryInterface,(IID_IAMVideoProcAmp, (void **)&pVideoProcAmp)))
@@ -790,17 +855,17 @@ bool PVideoInputDevice_DirectShow::GetAttributes(Attributes & attrib)
   if (!IsOpen())
     return false;
 
-  attrib.m_brightness = GetControlCommon(VideoProcAmp_Brightness);
-  attrib.m_contrast   = GetControlCommon(VideoProcAmp_Contrast);
-  attrib.m_saturation = GetControlCommon(VideoProcAmp_Saturation);
-  attrib.m_hue        = GetControlCommon(VideoProcAmp_Hue);
-  attrib.m_gamma      = GetControlCommon(VideoProcAmp_Gamma);
+  attrib.m_brightness = GetAttributeCommon(VideoProcAmp_Brightness);
+  attrib.m_contrast   = GetAttributeCommon(VideoProcAmp_Contrast);
+  attrib.m_saturation = GetAttributeCommon(VideoProcAmp_Saturation);
+  attrib.m_hue        = GetAttributeCommon(VideoProcAmp_Hue);
+  attrib.m_gamma      = GetAttributeCommon(VideoProcAmp_Gamma);
 
   return true;
 }
 
 
-PBoolean PVideoInputDevice_DirectShow::SetControlCommon(long control, int newValue)
+PBoolean PVideoInputDevice_DirectShow::SetAttributeCommon(long control, int newValue)
 {
   CComPtr<IAMVideoProcAmp> pVideoProcAmp;
   PCOM_RETURN_ON_FAILED(m_pCaptureFilter->QueryInterface,(IID_IAMVideoProcAmp, (void **)&pVideoProcAmp));
@@ -810,14 +875,14 @@ PBoolean PVideoInputDevice_DirectShow::SetControlCommon(long control, int newVal
 
   if (newValue == -1) {
     if ((flags&VideoProcAmp_Flags_Auto) == 0) {
-      PTRACE(2, "DShow\tAutomatic control for element " << control << " not supported");
+      PTRACE(2, "Automatic control for element " << control << " not supported");
       return false;
     }
     PCOM_RETURN_ON_FAILED(pVideoProcAmp->Set,(control, 0, VideoProcAmp_Flags_Auto));
   }
   else {
     if ((flags&VideoProcAmp_Flags_Manual) == 0) {
-      PTRACE(2, "DShow\tManual control for element " << control << " not supported");
+      PTRACE(2, "Manual control for element " << control << " not supported");
       return false;
     }
 
@@ -825,17 +890,17 @@ PBoolean PVideoInputDevice_DirectShow::SetControlCommon(long control, int newVal
     PCOM_RETURN_ON_FAILED(pVideoProcAmp->Set,(control, scaled, VideoProcAmp_Flags_Manual));
   }
 
-  PTRACE(4, "DShow\tSetControl: element=" << control << ", value=" << newValue);
+  PTRACE(4, "SetControl: element=" << control << ", value=" << newValue);
   return true;
 }
 
 PBoolean PVideoInputDevice_DirectShow::SetAttributes(const Attributes & attrib)
 {
-  return SetControlCommon(VideoProcAmp_Brightness, attrib.m_brightness) &&
-         SetControlCommon(VideoProcAmp_Saturation, attrib.m_saturation) &&
-         SetControlCommon(VideoProcAmp_Contrast, attrib.m_contrast) &&
-         SetControlCommon(VideoProcAmp_Hue, attrib.m_hue) &&
-         SetControlCommon(VideoProcAmp_Gamma, attrib.m_gamma);
+  return SetAttributeCommon(VideoProcAmp_Brightness, attrib.m_brightness) &&
+         SetAttributeCommon(VideoProcAmp_Saturation, attrib.m_saturation) &&
+         SetAttributeCommon(VideoProcAmp_Contrast, attrib.m_contrast) &&
+         SetAttributeCommon(VideoProcAmp_Hue, attrib.m_hue) &&
+         SetAttributeCommon(VideoProcAmp_Gamma, attrib.m_gamma);
 }
 
 
@@ -942,7 +1007,7 @@ PStringArray PVideoInputDevice_DirectShow::GetDeviceNames() const
 
   HANDLE handle = FindFirstDevice(DeviceSearchByGuid, &guidCamera, &devInfo);
   if (handle == NULL) {
-    PTRACE(1, "DShow\tFindFirstDevice failed, error=" << ::GetLastError());
+    PTRACE(1, "FindFirstDevice failed, error=" << ::GetLastError());
     return devices;
   }
 
@@ -950,13 +1015,13 @@ PStringArray PVideoInputDevice_DirectShow::GetDeviceNames() const
     if (devInfo.hDevice != NULL) {
       PString devName(devInfo.szLegacyName);
       devices.AppendString(devName);
-      PTRACE(3, "DShow\tFound capture device \""<< devName <<'"');
+      PTRACE(3, "Found capture device \""<< devName <<'"');
     }
   } while (FindNextDevice(handle, &devInfo));
 
   FindClose(handle);
 
-  PTRACE_IF(2, devices.IsEmpty(), "DShow\tNo video capture devices available.");
+  PTRACE_IF(2, devices.IsEmpty(), "No video capture devices available.");
 
   return devices;
 }
@@ -1096,7 +1161,7 @@ class PComEnumerator
       if (PCOM_FAILED(m_pDevEnum->CreateClassEnumerator,(CLSID_VideoInputDeviceCategory, &m_pClassEnum, 0)))
         return;
 
-      PTRACE_IF(2, m_pClassEnum == NULL, "DShow\tNo video capture device was detected.");
+      PTRACE_IF(2, m_pClassEnum == NULL, "No video capture device was detected.");
     }
 
     bool Next()
@@ -1124,14 +1189,18 @@ class PComEnumerator
           PCOM_FAILED(pPropBag->Read,(L"FriendlyName", &varName, NULL), ERROR_FILE_NOT_FOUND))
         return PString::Empty();
 
-      return varName.AsString();
+      PString name = varName.AsString();
+
+      PINDEX i = name.GetLength();
+      while ((i > 0) && !::isprint(name[i-1]))
+        name = name.Left(--i);
+
+      return name;
     }
 };
 
 PStringArray PVideoInputDevice_DirectShow::GetDeviceNames() const
 {
-  PTRACE(4, "DShow\tEnumerating Device Names");
-
   PStringArray devices;
   unsigned duplicate = 0;
 
@@ -1141,6 +1210,7 @@ PStringArray PVideoInputDevice_DirectShow::GetDeviceNames() const
     if (!name.IsEmpty()) {
       if (devices.GetValuesIndex(name) != P_MAX_INDEX)
         name.sprintf("{%u}", ++duplicate);
+      PTRACE(3, "Found capture device \""<< name <<'"');
       devices.AppendString(name);
     }
   }
@@ -1149,166 +1219,158 @@ PStringArray PVideoInputDevice_DirectShow::GetDeviceNames() const
 }
 
 
-PVideoInputControl_DirectShow::PVideoInputControl_DirectShow(IAMCameraControl * pCC)
-  : m_pCC(pCC)
+bool PVideoInputDevice_DirectShow::SetControl(PVideoControlInfo::Types type, int value, ControlMode mode)
 {
-  PComResult hr;
-
-  for (PVideoControlInfo::Types type = PVideoControlInfo::BeginTypes; type < PVideoControlInfo::EndTypes; ++type) {
-    PVideoControlInfo & info = m_control[type];
-    info.m_type = type;
-    if (hr.Succeeded(pCC->GetRange(InputControlPropertyCode[type],
-                     &info.m_min, &info.m_max, &info.m_step, &info.m_default, &info.m_flags)))
-      info.m_current = info.m_default;
-    else {
-      PTRACE(4, "DShow\tCamera does not support " << type << ": " << hr);
-    }
-  }
-}
-
-
-bool PVideoInputControl_DirectShow::Control(PVideoControlInfo::Types type, long value, bool absolute)
-{
-  PWaitAndSignal mutex(m_mutex);
-
-  if (!IsValid(type))
+  if (m_pCameraControls == NULL) {
+    PTRACE(2, "No controls available to " << type);
     return false;
+  }
 
   long flags = KSPROPERTY_CAMERACONTROL_FLAGS_MANUAL;
-  if (absolute)
-    flags |= KSPROPERTY_CAMERACONTROL_FLAGS_ABSOLUTE;
-  else
-    flags |= KSPROPERTY_CAMERACONTROL_FLAGS_RELATIVE;
+  switch (mode) {
+    case AutomaticControl :
+      flags = KSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
+      break;
 
-  PComResult hr;
-  if (hr.Succeeded(m_pCC->Set(InputControlPropertyCode[type], value, flags)) &&
-      hr.Succeeded(m_pCC->Get(InputControlPropertyCode[type], &m_control[type].m_current, &flags))) {
-    PTRACE(5, "DShow\tSet " << type << " to " << m_control[type].m_current);
-    return true;
+    case AbsoluteControl :
+      value = m_controlInfo[type].SetCurrent(value);
+      break;
+
+    case RelativeControl :
+      value = m_controlInfo[type].SetCurrent(m_controlInfo[type].GetCurrent() + value);
+      break;
+
+    case ResetControl :
+      value = m_controlInfo[type].Reset();
+      break;
   }
 
-  PTRACE(2, "DShow\tFailed to " << type << " to " << value << ": " << hr);
-  return false;
-}
-
-
-bool PVideoInputControl_DirectShow::Pan(long value, bool absolute)
-{
-  return Control(PVideoControlInfo::Pan, value, absolute);
-}
-
-
-bool PVideoInputControl_DirectShow::Tilt(long value, bool absolute)
-{
-  return Control(PVideoControlInfo::Tilt, value, absolute);
-}
-
-
-bool PVideoInputControl_DirectShow::Zoom(long value, bool absolute)
-{
-  // 50 is 100% and 200 is 4x zoom
-  if (absolute && ((value < 50) || (value > 200))) {
-    PTRACE(2, "DShow\tWrong zoom value received: " << value << " must be between 50 (1x) and 200 (4x).");
+  PComResult hr;
+  if (!hr.Succeeded(m_pCameraControls->Set(InputControlPropertyCode[type], value, flags))) {
+    PTRACE(2, "Failed to set " << type << " to " << value << ": " << hr);
     return false;
   }
 
-  if (!absolute)
-    value += m_control[PVideoControlInfo::Zoom].m_current;
-
-  return Control(PVideoControlInfo::Zoom, value, true);
+  PTRACE(4, "Set " << type << " to " << value);
+  return true;
 }
 
 
-// Implementation of CSampleGrabberCB object
-class CSampleGrabberCB : public ISampleGrabberCB 
+// Implementation of PSampleGrabberCB object
+PSampleGrabberCB::PSampleGrabberCB()
+  : m_stopped(true)
+  , m_actualSize(0)
+  , m_skipInitialGrabs(4)
+#if PTRACING
+  , m_totalFrames(0)
+#endif
 {
-public:
-  CSampleGrabberCB()
-  {
-    bufferSize = 0;
-    pBuffer = NULL; 
-    cInd = 0;
+}
+
+
+PSampleGrabberCB::~PSampleGrabberCB()
+{
+  m_frameReady.Signal();
+}
+
+
+void PSampleGrabberCB::Start()
+{
+#if PTRACING
+  m_totalTime = 0;
+#endif
+  m_stopped = false;
+}
+
+
+void PSampleGrabberCB::Stop()
+{
+#if PTRACING
+  static const int Level = 3;
+  if (PTrace::CanTrace(Level)) {
+    int64_t ms = m_totalTime.GetElapsed().GetMilliSeconds();
+    ostream & trace = PTRACE_BEGIN(Level);
+    trace << "Grabber Stopped";
+    if (ms > 0)
+      trace << ", frames/second=" << fixed << setprecision(2) << m_totalFrames*1000.0/ms;
+    trace << PTrace::End;
   }
+#endif
 
-  ~CSampleGrabberCB()
-  {
-    bufferSize = 0;
-    delete pBuffer;
-    frameready.Signal();
-  }
-
-  // Fake out any COM ref counting
-  //
-  STDMETHODIMP_(ULONG) AddRef() { return 2; }
-  STDMETHODIMP_(ULONG) Release() { return 1; }
-
-  // Fake out any COM QI'ing
-  //
-  STDMETHODIMP QueryInterface(REFIID riid, void ** ppv)
-  {
-
-    if( riid == IID_ISampleGrabberCB || riid == IID_IUnknown ) 
-    {
-      *ppv = (void *) static_cast<ISampleGrabberCB*> ( this );
-      return NOERROR;
-    }    
-
-    return E_NOINTERFACE;
-  }
+  m_stopped = true;
+  m_frameReady.Signal();
+}
 
 
-  // We don't implement this one
-  //
-  STDMETHODIMP SampleCB( double /*SampleTime*/, IMediaSample * /*pSample*/)
-  {
-    return 0;
-  }
+// Fake out any COM QI'ing
+//
+STDMETHODIMP PSampleGrabberCB::QueryInterface(REFIID riid, void ** ppv)
+{
+  if( riid == IID_ISampleGrabberCB || riid == IID_IUnknown ) {
+    *ppv = (void *) static_cast<ISampleGrabberCB*> ( this );
+    return NOERROR;
+  }    
 
-  // The sample grabber is calling us back on its deliver thread.
-  // This is NOT the main app thread!
-  //
-  STDMETHODIMP BufferCB( double PTRACE_PARAM(dblSampleTime), BYTE * buffer, long size )
-  {
-    PTRACE(6, "DShow\tBuffer callback: time=" << dblSampleTime
-           << ", buf=" << (void *)buffer << ", size=" << size);
+  return E_NOINTERFACE;
+}
 
-    static int skipFrames = 4;
-    if (cInd < skipFrames) {
-      cInd++;
-      return S_OK;
-    }
 
-    PWaitAndSignal m(mbuf);
+// The sample grabber is calling us back on its deliver thread.
+// This is NOT the main app thread!
+//
+STDMETHODIMP PSampleGrabberCB::BufferCB(double PTRACE_PARAM(dblSampleTime), BYTE * buffer, long size)
+{
+  PTRACE(6, "Buffer callback: time=" << dblSampleTime
+          << ", buf=" << (void *)buffer << ", size=" << size);
 
-    if (size == 0)
-      return S_OK;
-
-    if (bufferSize != size) {
-      bufferSize = size;
-      delete pBuffer;
-      pBuffer = new BYTE[bufferSize];
-      memset(pBuffer,0,bufferSize);
-    }
-
-    if (!buffer)
-      return E_POINTER;
-
-    memcpy(pBuffer,buffer, bufferSize);
-
-    frameready.Signal();
+  if (size == 0 || m_stopped)
+    return S_OK;
 
+  if (m_skipInitialGrabs > 0) {
+    --m_skipInitialGrabs;
     return S_OK;
   }
 
+  PWaitAndSignal mutex(m_mutex);
 
-  long bufferSize;
-  BYTE * pBuffer;
+  if (m_buffer.GetSize() != (PINDEX)size) {
+    if (!m_buffer.SetMinSize(size))
+      return E_POINTER;
+  }
 
-  PMutex mbuf;
-  PSyncPoint frameready;
-  PINDEX cInd;
+  memcpy(m_buffer.GetPointer(), buffer, size);
+  m_actualSize = size;
 
-};
+  m_frameReady.Signal();
+
+#if PTRACING
+  ++m_totalFrames;
+#endif
+  return S_OK;
+}
+
+
+bool PSampleGrabberCB::GetData(BYTE * data, PINDEX maxSize, PINDEX & actualSize)
+{
+  // Live! Cam Optia AF (VC0100) webcam took 3.1 sec.
+  if (!m_frameReady.Wait(5000)) {
+    PTRACE(1, "Timeout awaiting next frame");
+    return false;
+  }
+
+  PWaitAndSignal mutex(m_mutex);
+
+  if (m_stopped) {
+    PTRACE(4, "Stopped");
+    return false;
+  }
+
+  if (m_actualSize <= maxSize)
+    memcpy(data, m_buffer, m_actualSize);
+
+  actualSize = m_actualSize;
+  return true;
+}
 
 
 static HRESULT GetUnconnectedPin(IBaseFilter *pFilter,   // Pointer to the filter.
@@ -1417,7 +1479,7 @@ bool PVideoInputDevice_DirectShow::BindCaptureDevice(const PString & devName)
 bool PVideoInputDevice_DirectShow::PlatformOpen()
 {
   // Buid the Camera Sample Grabber
-  PTRACE(5, "DShow\tBuilding Sample Grabber");
+  PTRACE(5, "Building Sample Grabber");
 
   CComPtr<IBaseFilter> pGrab; 
   PCOM_RETURN_ON_FAILED(CoCreateInstance,(CLSID_SampleGrabber,
@@ -1436,15 +1498,15 @@ bool PVideoInputDevice_DirectShow::PlatformOpen()
 
   PCOM_RETURN_ON_FAILED(m_pSampleGrabber->SetBufferSamples,(true));
 
-  PTRACE(5, "DShow\tSetting Sample Grabber Callback");
-  m_pSampleGrabberCB = new CSampleGrabberCB();
+  PTRACE(5, "Setting Sample Grabber Callback");
+  m_pSampleGrabberCB = new PSampleGrabberCB();
   PCOM_RETURN_ON_FAILED(m_pSampleGrabber->SetCallback,(m_pSampleGrabberCB, 1));
 
-  PTRACE(5, "DShow\tConnect sample grabber to camera");
+  PTRACE(5, "Connect sample grabber to camera");
   PCOM_RETURN_ON_FAILED(ConnectFilters,(m_pGraphBuilder, m_pCameraOutPin, pGrab));
 
   // Set the NULL Renderer
-  PTRACE(5, "DShow\tBuilding NULL output filter");
+  PTRACE(5, "Building NULL output filter");
   PCOM_RETURN_ON_FAILED(CoCreateInstance,(CLSID_NullRenderer,
                                           NULL,
                                           CLSCTX_INPROC_SERVER,
@@ -1453,23 +1515,28 @@ bool PVideoInputDevice_DirectShow::PlatformOpen()
 
   PCOM_RETURN_ON_FAILED(m_pGraphBuilder->AddFilter,(m_pNullRenderer, L"NULL Output"));
 
-  PTRACE(5, "DShow\tConnect Null Render to Grab filter Pin");
+  PTRACE(5, "Connect Null Render to Grab filter Pin");
   PCOM_RETURN_ON_FAILED(ConnectFilters,(m_pGraphBuilder, pGrab, m_pNullRenderer));
 
-  PTRACE(5, "DShow\tChecking image flip");
+  PTRACE(5, "Checking image flip");
   PCOM_RETURN_ON_FAILED(m_pSampleGrabber->GetConnectedMediaType,(&mt));
 
-  VIDEOINFOHEADER * pvih = (VIDEOINFOHEADER *)mt.pbFormat;
-  if (pvih->bmiHeader.biHeight > 0) {
-    nativeVerticalFlip = true;
-    PTRACE(3, "DShow\tImage up side down");
-  }
-
   // Query for camera controls
   if (FAILED(m_pCaptureFilter->QueryInterface(IID_IAMCameraControl, (void **)&m_pCameraControls))) {
-    PTRACE(3, "DShow\tCamera " << deviceName << " does not support Camera Controls.");
+    PTRACE(3, "Camera " << deviceName << " does not support Camera Controls.");
     m_pCameraControls = NULL;
   }
+  else {
+    for (PVideoControlInfo::Types type = PVideoControlInfo::BeginTypes; type < PVideoControlInfo::EndTypes; ++type) {
+      PComResult hr;
+      long minimum, maximum, step, reset, flags;
+      if (hr.Succeeded(m_pCameraControls->GetRange(InputControlPropertyCode[type], &minimum, &maximum, &step, &reset, &flags)))
+        m_controlInfo[type] = PVideoControlInfo(type, minimum, maximum, step, reset);
+      else {
+        PTRACE(4, "Camera does not support " << type << ": " << hr);
+      }
+    }
+  }
 
   return true;
 }
@@ -1481,45 +1548,20 @@ PINDEX PVideoInputDevice_DirectShow::GetCurrentBufferSize()
 }
 
 
-bool PVideoInputDevice_DirectShow::GetCurrentBufferData(BYTE * data)
+bool PVideoInputDevice_DirectShow::GetCurrentBufferData(BYTE * data, PINDEX & bufferSize)
 {
-  CSampleGrabberCB * cb = (CSampleGrabberCB *)&*m_pSampleGrabberCB;
-
-  // Live! Cam Optia AF (VC0100) webcam took 3.1 sec.
-  if (!cb->frameready.Wait(5000)) {
-    PTRACE(1, "DShow\tTimeout awaiting next frame");
+  if (m_pSampleGrabberCB == NULL)
     return false;
+
+  while (IsCapturing() && m_pSampleGrabberCB->GetData(data, m_maxFrameBytes, bufferSize)) {
+    if (m_fixedSizeFrames ? (bufferSize == m_maxFrameBytes) : (bufferSize <= m_maxFrameBytes))
+      return true;
+
+    PTRACE(4, "Camera resolution changed, ignoring received frame at old size.");
   }
 
-  if (PAssertNULL(cb->pBuffer) == NULL)
-    return false;
-
-  if (cb->bufferSize <= m_maxFrameBytes) {
-    cb->mbuf.Wait();
-    memcpy(data, cb->pBuffer, cb->bufferSize);
-    cb->mbuf.Signal();
-  } else
-    PTRACE(1, "DShow\tNot copying, since bufferSize is greater than m_maxFrameBytes (" << cb->bufferSize << " > " << m_maxFrameBytes << ")");
-
-  return true;
+  return false;
 }
 
-
-PVideoInputControl * PVideoInputDevice_DirectShow::GetVideoInputControls()
-{
-  if (m_pCameraControls != NULL)
-    return new PVideoInputControl_DirectShow(m_pCameraControls);
-  return NULL; 
-}
-
-
 #endif  // _WIN32_WCE
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("Direct Show video support DISABLED")
-  #endif
-
 #endif  // P_DIRECTSHOW
