@@ -24,9 +24,9 @@
  *
  * Contributor(s): Ted Szoczei, Nimajin Software Consulting
  *
- * $Revision: 29832 $
+ * $Revision: 34242 $
  * $Author: rjongbloed $
- * $Date: 2013-05-29 22:21:43 -0500 (Wed, 29 May 2013) $
+ * $Date: 2015-10-26 12:01:52 +0000 (Mon, 26 Oct 2015) $
  */
 
 #pragma implementation "directsound.h"
@@ -37,6 +37,8 @@
 
 #define PTraceModule() "dsound"
 
+#define P_FORCE_STATIC_PLUGIN 1
+
 #include <ptlib/pprocess.h>
 #include <algorithm>
 #include <math.h>
@@ -49,11 +51,10 @@
 
 
 #ifdef _MSC_VER
-  #if _MSC_VER >= 1700
+  #if _MSC_VER >= 1700 && !defined(_USING_V110_SDK71_)
     #pragma comment(lib, "dxguid.lib")
   #endif
   #pragma comment(lib, "dsound.lib")
-  #pragma message("Direct Sound support enabled")
 #endif
 
 // It takes a lot of fiddling in configure to find dxerr.h
@@ -236,34 +237,19 @@ static HRESULT GetDefaultDeviceGUID(const PCaselessString & name, PSoundChannelD
 ///////////////////////////////////////////////////////////////////////////////
 // Construction
 
-PSoundChannelDirectSound::PSoundChannelDirectSound ()
+PSoundChannelDirectSound::PSoundChannelDirectSound()
+  : m_isStreaming(true)
+  , m_bufferSectionCount(0)
+  , m_bufferSectionSize(0)
+  , m_bufferSize(0)
+  , m_movePos(0)
+  , m_available(0)
+  , m_dsPos(0)
+  , m_dsMoved(0)
+  , m_moved(0)
+  , m_lost(0)
+  , m_mixer(NULL)
 {
-  Construct();
-}
-
-
-PSoundChannelDirectSound::PSoundChannelDirectSound (const PString &device,
-                                                    Directions dir,
-                                                    unsigned numChannels,
-                                                    unsigned sampleRate,
-                                                    unsigned bitsPerSample)
-{
-  Construct();
-  Open(device, dir, numChannels, sampleRate, bitsPerSample);
-}
-
-
-void PSoundChannelDirectSound::Construct () // private
-{
-  m_captureDevice = NULL;
-  m_captureBuffer = NULL;
-
-  m_playbackDevice = NULL;
-  m_playbackBuffer = NULL;
-
-  m_isStreaming = true;
-  m_bufferSize = 0;
-  m_mixer = NULL;
 
   memset(&m_waveFormat, 0, sizeof(m_waveFormat)); 
 
@@ -274,7 +260,7 @@ void PSoundChannelDirectSound::Construct () // private
 }
 
 
-PSoundChannelDirectSound::~PSoundChannelDirectSound ()
+PSoundChannelDirectSound::~PSoundChannelDirectSound()
 {
   Close();
   if (m_triggerEvent[SOUNDEVENT_SOUND] != NULL)
@@ -287,6 +273,12 @@ PSoundChannelDirectSound::~PSoundChannelDirectSound ()
 }
 
 
+const char * PSoundChannelDirectSound::GetDriverName()
+{
+  return PPlugin_PSoundChannel_DirectSound::ServiceName();
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // Open
 
@@ -298,17 +290,17 @@ PString PSoundChannelDirectSound::GetDefaultDevice (Directions dir) // static
   GUID deviceGUID;
   PComResult result = GetDeviceID((dir == Player)? &DSDEVID_DefaultPlayback  : &DSDEVID_DefaultCapture, &deviceGUID);
   if (result.Failed()) {
-    PTRACE(4, NULL, NULL, "Could not find default device: " << result);
+    PTRACE(4, PTraceModule(), "Could not find default device: " << result);
     return PString();
   }
   PDSoundDeviceInfoVector devices;
   result = GetFilteredDSoundDeviceInfo(deviceGUID, dir, PString::Empty(), -1, devices);
   if (result.Failed()) {
-    PTRACE(4, NULL, NULL, "Open: Could not retrieve device information: " << result);
+    PTRACE(4, PTraceModule(), "Open: Could not retrieve device information: " << result);
     return PString();
   }
   if (devices.size() == 0) {
-    PTRACE(4, NULL, NULL, "Open: Default device not found");
+    PTRACE(4, PTraceModule(), "Open: Default device not found");
     return PString();
   }
   return devices[0].m_Description;
@@ -322,7 +314,7 @@ PStringArray PSoundChannelDirectSound::GetDeviceNames (Directions dir) // static
   PDSoundDeviceInfoVector devices;
   PComResult result = GetFilteredDSoundDeviceInfo(GUID_NULL, dir, PString::Empty(), -1, devices);
   if (result.Failed()) {
-    PTRACE(4, NULL, NULL, "Could not get device list: " << result);
+    PTRACE(4, PTraceModule(), "Could not get device list: " << result);
     return PString();
   }
   PStringArray names;
@@ -343,21 +335,12 @@ PStringArray PSoundChannelDirectSound::GetDeviceNames (Directions dir) // static
 }
 
 
-PBoolean PSoundChannelDirectSound::Open (const PString & device, // public
-                                         Directions dir,
-                                         unsigned numChannels,
-                                         unsigned sampleRate,
-                                         unsigned bitsPerSample)
+bool PSoundChannelDirectSound::Open(const Params & params)
 {
-  PAssert(dir == Player || dir == Recorder, "Invalid device direction parameter");
-
   Close();
 
-  // remove the driver name prefix
-  PINDEX tab = device.Find(PDevicePluginServiceDescriptor::SeparatorChar);
-  m_deviceName = (tab == P_MAX_INDEX)? device : device.Mid(tab+1).Trim();
-
-  activeDirection = dir;
+  m_deviceName = params.m_device;
+  activeDirection = params.m_direction;
   m_available = 0;
   m_dsMoved = 0ui64;
   m_moved = 0ui64;
@@ -365,7 +348,7 @@ PBoolean PSoundChannelDirectSound::Open (const PString & device, // public
 
   // get info for all devices for direction
   PDSoundDeviceInfoVector devices;
-  PComResult result = GetFilteredDSoundDeviceInfo(GUID_NULL, dir, PString::Empty(), -1, devices);
+  PComResult result = GetFilteredDSoundDeviceInfo(GUID_NULL, activeDirection, PString::Empty(), -1, devices);
   if (result.Failed()) {
     SetErrorValues(Miscellaneous, GetErrorNumber());
     PTRACE(4, "Open" << GetDirectionText() << ": Could not get device list: " << result);
@@ -397,7 +380,7 @@ PBoolean PSoundChannelDirectSound::Open (const PString & device, // public
   }
   PTRACE(4, "Opening " << GetDirectionText() << " device \"" << m_deviceName << '"');
 
-  SetFormat(numChannels, sampleRate, bitsPerSample);
+  SetFormat(params.m_channels, params.m_sampleRate, params.m_bitsPerSample);
 
   // open for playback
   if (activeDirection == Player) {
@@ -457,6 +440,12 @@ PBoolean PSoundChannelDirectSound::Close () // public
 }
 
 
+PString PSoundChannelDirectSound::GetName() const
+{
+  return PConstString("DirectSound") + PPluginServiceDescriptor::SeparatorChar + m_deviceName;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // Setup
 // It should be OK to do these even while reading/writing
@@ -495,7 +484,7 @@ PBoolean PSoundChannelDirectSound::SetFormat (unsigned numChannels, // public
     else if (activeDirection == Recorder) {
       if (!OpenCaptureBuffer()) // if this fails, channel is closed
         return false;
-	}
+    }
   }
   else // Closed, no buffers yet
     SetWaveFormat(m_waveFormat, numChannels, sampleRate, bitsPerSample);
@@ -537,7 +526,7 @@ PBoolean PSoundChannelDirectSound::SetBuffers (PINDEX size, PINDEX count) // pub
     else if (activeDirection == Recorder) {
       if (!OpenCaptureBuffer()) // if this fails, channel is closed
         return false;
-	}
+    }
   }
   else
     SetBufferSections(size, count);
@@ -667,7 +656,7 @@ PBoolean PSoundChannelDirectSound::OpenPlayback (LPCGUID deviceId) // private
   if (result != S_OK) {
     SetErrorValues(Miscellaneous, result);
     PTRACE(4, "Open Playback: Could not set cooperative level: " << GetErrorText());
-	ClosePlayback();
+    ClosePlayback();
     return false;
   }
   return OpenPlaybackBuffer();
@@ -699,7 +688,7 @@ PBoolean PSoundChannelDirectSound::OpenPlaybackBuffer () // private
   if (FAILED(result)) { 
     SetErrorValues(Miscellaneous, result);
     PTRACE(4, "OpenPlaybackBuffer: CreateSoundBuffer fail: " << GetErrorText());
-	ClosePlayback();
+    ClosePlayback();
     return false;
   } 
   IDirectSoundNotify * notify; // temporary pointer to the interface
@@ -707,7 +696,7 @@ PBoolean PSoundChannelDirectSound::OpenPlaybackBuffer () // private
   if (FAILED(result)) { 
     SetErrorValues(Miscellaneous, result);
     PTRACE(4, "OpenPlaybackBuffer: notify interface query fail: " << GetErrorText());
-	ClosePlayback();
+    ClosePlayback();
     return false;
   }
   PTRACE(4, "Open Playback: Setting up notification for " << m_bufferSectionCount << " blocks of " << m_bufferSectionSize << " bytes");
@@ -716,7 +705,7 @@ PBoolean PSoundChannelDirectSound::OpenPlaybackBuffer () // private
     SetErrorValues(NoMemory, E_OUTOFMEMORY);
     PTRACE(4, "OpenPlaybackBuffer: notify allocation fail");
     notify->Release();
-	ClosePlayback();
+    ClosePlayback();
     return false;
   }
   DWORD blockOffset = m_bufferSectionSize - 1;
@@ -730,7 +719,7 @@ PBoolean PSoundChannelDirectSound::OpenPlaybackBuffer () // private
   if (FAILED(result)) { 
     SetErrorValues(Miscellaneous, result);
     PTRACE(4, "OpenPlaybackBuffer: Notify interface query fail: " << GetErrorText());
-	ClosePlayback();
+    ClosePlayback();
     return false;
   }
   delete [] position;
@@ -743,32 +732,32 @@ void PSoundChannelDirectSound::ClosePlayback() // private
 {
   if (m_playbackBuffer != NULL) {
     m_playbackBuffer->Stop();
-    int notification;
-    CheckPlayBuffer(notification); // last effort to see how many bytes played
+    CheckPlayBuffer(); // last effort to see how many bytes played
   }
   m_playbackBuffer.Release();
   m_playbackDevice.Release();
 }
 
 
-PBoolean PSoundChannelDirectSound::CheckPlayBuffer (int & notification) // private
+PSoundChannelDirectSound::CheckBufferState PSoundChannelDirectSound::CheckPlayBuffer() // private
 {
-  notification = SOUNDNOTIFY_NOTHING;
   PWaitAndSignal mutex(m_bufferMutex); // make Close & SetBuffers wait
 
   if (!m_playbackBuffer) { // closed
     PTRACE(4, "Playback closed while checking");
-    return false;
-  }               // Write is ahead of Play, DirectSound is playing data from Play to Write - do not put data between them.
+    return SOUNDNOTIFY_ERROR;
+  }
+  
+                  // Write is ahead of Play, DirectSound is playing data from Play to Write - do not put data between them.
                   // we can write ahead of Write, from m_movePos to PlayPos
   DWORD playPos;  // circular buffer byte offset to start of section DirectSound is playing (end of where we can write)
   DWORD writePos; // circular buffer byte offset ahead of which it is safe to write data (start of where we can write)
   HRESULT result = m_playbackBuffer->GetCurrentPosition(&playPos, &writePos);
   if (FAILED(result)) {
     SetErrorValues(Miscellaneous, result, LastWriteError);
-    notification = SOUNDNOTIFY_ERROR;
-    return false;
+    return SOUNDNOTIFY_ERROR;
   }
+
   // Record the driver performance, include check for writer getting way behind player
   // bytes played since last check
   DWORD played = (playPos + m_bufferSize - m_dsPos) % m_bufferSize;
@@ -782,50 +771,49 @@ PBoolean PSoundChannelDirectSound::CheckPlayBuffer (int & notification) // priva
   result = m_playbackBuffer->GetStatus(&status);
   if (FAILED(result)) {
     SetErrorValues(Miscellaneous, result, LastWriteError);
-    notification = SOUNDNOTIFY_ERROR;
-    return false;
+    return SOUNDNOTIFY_ERROR;
   }
+
   if ((status & DSBSTATUS_PLAYING) == 0) { // not started yet, or we let it run empty (not looping)
     m_available = m_bufferSize;
-    return true;
-  }
-  // check for underrun
-  if (m_dsMoved > m_moved + m_lost)
-  { // Write has been delayed so much that card is playing from empty or old space
-    unsigned underruns = (unsigned)(m_dsMoved - (m_moved + m_lost));
-    m_lost += underruns;
-    PTRACE(3, "Playback underrun: wrote " << GetSamplesMoved() << " played " << GetSamplesBuffered() << " replayed " << underruns / m_waveFormat.nBlockAlign << " total loss " << GetSamplesLost() << " samples");
-    notification = SOUNDNOTIFY_UNDERRUN;
-    m_movePos = writePos;
+    return SOUNDNOTIFY_AVAILABLE;
   }
+
   // calculate available space in circular buffer (between our last write position and DirectSound read position)
   m_available = (playPos + m_bufferSize - m_movePos) % m_bufferSize;
-  if (m_available >= (unsigned)m_bufferSectionSize)
-    return true;
+  if (m_available < (unsigned)m_bufferSectionSize) {
+    // some space, but not enough yet
+    PTRACE(6, "Player buffer overrun, waiting for space");
+    return SOUNDNOTIFY_OVERRUN;
+  }
 
-  // some space, but not enough yet
-  PTRACE(6, "Player buffer overrun, waiting for space");
-  notification = SOUNDNOTIFY_OVERRUN;
-  return false;
+  // check for underrun
+  if (m_dsMoved < m_moved + m_lost)
+    return SOUNDNOTIFY_AVAILABLE;
+
+  // Write has been delayed so much that card is playing from empty or old space
+  unsigned underruns = (unsigned)(m_dsMoved - (m_moved + m_lost));
+  m_lost += underruns;
+  PTRACE(3, "Playback underrun: wrote " << GetSamplesMoved() << " played " << GetSamplesBuffered() << " replayed " << underruns / m_waveFormat.nBlockAlign << " total loss " << GetSamplesLost() << " samples");
+  m_movePos = writePos;
+  return SOUNDNOTIFY_UNDERRUN;
 }
 
 
 PBoolean PSoundChannelDirectSound::WaitForPlayBufferFree () // protected
 {
   ResetEvent(m_triggerEvent[SOUNDEVENT_SOUND]);
-  int notification;
   do {
-    notification = SOUNDNOTIFY_NOTHING;
-    PBoolean isSpaceAvailable = CheckPlayBuffer(notification);
+    CheckBufferState notification = CheckPlayBuffer();
     // report errors
-    if (notification != SOUNDNOTIFY_NOTHING && m_notifier.GetObject() != NULL)
+    if (notification != SOUNDNOTIFY_AVAILABLE && m_notifier.GetObject() != NULL)
       m_notifier(*this, notification); // can close here!
 
     if (!m_playbackBuffer) { // closed
       PTRACE(4, "Playback closed while writing");
       return SetErrorValues(NotOpen, EBADF, LastWriteError);
     }
-    if (isSpaceAvailable) // Ok to write
+    if (notification == SOUNDNOTIFY_AVAILABLE || notification == SOUNDNOTIFY_UNDERRUN) // Ok to write
       return true;
   }  // wait for DirectSound to notify us that space is available
   while (WaitForMultipleObjects(2, m_triggerEvent, FALSE, INFINITE) == WAIT_OBJECT_0);
@@ -835,10 +823,10 @@ PBoolean PSoundChannelDirectSound::WaitForPlayBufferFree () // protected
     PTRACE(4, "Playback write abort");
     if (m_playbackBuffer != NULL) { // still open
       m_playbackBuffer->Stop();
-      CheckPlayBuffer(notification);
-	}
+      CheckPlayBuffer();
+    }
   }
-  return SetErrorValues(Interrupted, EINTR, LastWriteError);
+  return SetErrorValues(Interrupted, ECANCELED, LastWriteError);
 }
 
 
@@ -911,19 +899,7 @@ PBoolean PSoundChannelDirectSound::Write (const void *buf, PINDEX len) // public
 
 PBoolean PSoundChannelDirectSound::HasPlayCompleted () // public
 {
-  // only works for non-streaming player
-  PWaitAndSignal mutex(m_bufferMutex); // prevent closing while active
-
-  if (!m_playbackBuffer)
-    return true;
-
-  DWORD status;
-  HRESULT result = m_playbackBuffer->GetStatus(&status);
-  if (FAILED(result)) {
-    SetErrorValues(Miscellaneous, result, LastWriteError);
-    return true; // it's done if we get an error here
-  }
-  return ((status & DSBSTATUS_PLAYING) == 0);
+  return CheckPlayBuffer() != SOUNDNOTIFY_AVAILABLE || m_available == (DWORD)m_bufferSize;
 }
 
 
@@ -956,49 +932,6 @@ PBoolean PSoundChannelDirectSound::PlaySound (const PSound & sound, PBoolean wai
 }
 
 
-PBoolean PSoundChannelDirectSound::PlayFile (const PFilePath & filename, PBoolean wait) // public
-{
-  PAssert(activeDirection == Player, "Invalid device direction");
-
-  PMultiMediaFile mediaFile;
-  PWaveFormat fileFormat;
-  DWORD dataSize;
-  if (!mediaFile.OpenWaveFile(filename, fileFormat, dataSize))
-    return SetErrorValues(NotOpen, mediaFile.GetLastError() | PWIN32ErrorFlag, LastWriteError);
-
-  //Abort();
-  if (!SetFormat(fileFormat->nChannels, fileFormat->nSamplesPerSec, fileFormat->wBitsPerSample))
-    return false;
-
-  int bufferSize = m_waveFormat.nAvgBytesPerSec / 2; // 1/2 second
-  if (!SetBuffers(bufferSize, 4))                    // 2 seconds
-    return false;
-
-  PBYTEArray buffer;
-  m_isStreaming = false;
-
-  while (dataSize) {
-    // Read the waveform data subchunk
-    PINDEX count = PMIN(dataSize,((DWORD)bufferSize));
-    if (!mediaFile.Read(buffer.GetPointer(bufferSize), count)) {
-      SetErrorValues(Miscellaneous, mediaFile.GetLastError() | PWIN32ErrorFlag, LastReadError);
-      PTRACE(4, "PlayFile read error: " << GetErrorText());
-      return false;
-    }
-    if (!Write(buffer, count))
-      break;
-
-    dataSize -= count;
-  }
-  mediaFile.Close();
-
-  if (wait)
-    return WaitForPlayCompletion();
-
-  return true;
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 // Recording
 
@@ -1082,8 +1015,7 @@ void PSoundChannelDirectSound::CloseCapture () // private
 {
   if (m_captureBuffer != NULL) {
     m_captureBuffer->Stop();
-    int notification;
-    CheckCaptureBuffer(notification); // last effort to see how many bytes played
+    CheckCaptureBuffer(); // last effort to see how many bytes played
   }
   m_captureBuffer.Release();
   m_captureDevice.Release();
@@ -1120,22 +1052,24 @@ PBoolean PSoundChannelDirectSound::StartRecording () // public
 }
 
 
-PBoolean PSoundChannelDirectSound::CheckCaptureBuffer (int & notification) // private
+PSoundChannelDirectSound::CheckBufferState PSoundChannelDirectSound::CheckCaptureBuffer() // private
 {
-  notification = SOUNDNOTIFY_NOTHING;
   PWaitAndSignal mutex(m_bufferMutex); // make Abort wait
 
   if (!m_captureBuffer) { // closed
     PTRACE(4, "Recording closed while checking");
-    return false;
-  }                 // Capture is ahead of Read, do not get data from between them. Data for us is at m_movePos behind Read.
+    return SOUNDNOTIFY_ERROR;
+  }
+
+                    // Capture is ahead of Read, do not get data from between them. Data for us is at m_movePos behind Read.
   DWORD readPos;    // circular buffer byte offset to the end of the data that has been fully captured (end of what we can read)
   DWORD capturePos; // circular buffer byte offset to the head of the block that card has locked for new data
   HRESULT result = m_captureBuffer->GetCurrentPosition(&capturePos, &readPos);
   if (FAILED(result)) {
-    notification = SOUNDNOTIFY_ERROR;
-    return SetErrorValues(Miscellaneous, result, LastReadError);
+    SetErrorValues(Miscellaneous, result, LastReadError);
+    return SOUNDNOTIFY_ERROR;
   }
+
   // Record the driver performance, include check for reader getting way behind recorder
   // bytes recorded since last check (this count includes stuff we can't even read yet)
   DWORD captured = (capturePos + m_bufferSize - m_dsPos) % m_bufferSize;
@@ -1146,20 +1080,24 @@ PBoolean PSoundChannelDirectSound::CheckCaptureBuffer (int & notification) // pr
   // count bytes recorded since open
   m_dsMoved += captured;
 
-  // check for overrun
-  if (m_dsMoved > m_moved + m_lost + m_bufferSize)
-  { // Read has been delayed so much that card is writing into space that we have not read yet
-    // overruns are what will never be read
-    unsigned overruns = (unsigned)((m_dsMoved - m_bufferSize) - (m_moved + m_lost));
-    m_lost += overruns;
-    PTRACE(3, "Recorder overrun: captured " << GetSamplesBuffered() << " read " << GetSamplesMoved() << " lost " << overruns / m_waveFormat.nBlockAlign << " total loss " << GetSamplesLost() << " samples");
-    notification = SOUNDNOTIFY_OVERRUN;
-    // Move read position to DirectSound's capture position, where audio has not been overwritten yet
-    m_movePos = m_dsPos;
-  }
   // calculate available data in circular buffer (between our last write position and DirectSound read position)
   m_available = (readPos + m_bufferSize - m_movePos) % m_bufferSize;
-  return m_available >= (unsigned)m_bufferSectionSize;
+
+  if (m_available < (unsigned)m_bufferSectionSize)
+    return SOUNDNOTIFY_UNDERRUN;
+
+  // check for overrun
+  if (m_dsMoved <= m_moved + m_lost + m_bufferSize)
+    return SOUNDNOTIFY_AVAILABLE;
+
+  // Read has been delayed so much that card is writing into space that we have not read yet
+  // overruns are what will never be read
+  unsigned overruns = (unsigned)((m_dsMoved - m_bufferSize) - (m_moved + m_lost));
+  m_lost += overruns;
+  PTRACE(3, "Recorder overrun: captured " << GetSamplesBuffered() << " read " << GetSamplesMoved() << " lost " << overruns / m_waveFormat.nBlockAlign << " total loss " << GetSamplesLost() << " samples");
+  // Move read position to DirectSound's capture position, where audio has not been overwritten yet
+  m_movePos = m_dsPos;
+  return SOUNDNOTIFY_OVERRUN;
 }
 
 
@@ -1168,13 +1106,12 @@ PBoolean PSoundChannelDirectSound::IsRecordBufferFull () // public
   if (!StartRecording()) // Start the first read
     return false;
 
-  int notification = SOUNDNOTIFY_NOTHING;
-  PBoolean isDataAvailable = CheckCaptureBuffer(notification);
+  CheckBufferState notification = CheckCaptureBuffer();
   // report errors
-  if (notification != SOUNDNOTIFY_NOTHING && m_notifier.GetObject() != NULL)
+  if (notification != SOUNDNOTIFY_AVAILABLE && m_notifier.GetObject() != NULL)
     m_notifier(*this, notification); // can close here!
 
-  return m_captureBuffer != NULL && isDataAvailable;
+  return m_captureBuffer != NULL && (notification == SOUNDNOTIFY_AVAILABLE || notification == SOUNDNOTIFY_OVERRUN);
 }
 
 
@@ -1183,33 +1120,22 @@ PBoolean PSoundChannelDirectSound::WaitForRecordBufferFull () // public
   if (!StartRecording()) // Start the first read
     return false;
 
-  int notification;
   ResetEvent(m_triggerEvent[SOUNDEVENT_SOUND]);
   do {
-    notification = SOUNDNOTIFY_NOTHING;
-    PBoolean isDataAvailable = CheckCaptureBuffer(notification);
-    // report errors
-    if (notification != SOUNDNOTIFY_NOTHING && m_notifier.GetObject() != NULL)
-      m_notifier(*this, notification); // can close here!
-
-    if (!m_captureBuffer) { // closed
-      PTRACE(4, "Recording closed while reading");
-      return SetErrorValues(NotOpen, EBADF, LastReadError);
-    }
-    if (isDataAvailable) // Ok to read
+    if (IsRecordBufferFull())
       return true;
   }  // wait for DirectSound to notify us that space is available
-  while (WaitForMultipleObjects(2, m_triggerEvent, FALSE, INFINITE) == WAIT_OBJECT_0);
+  while (WaitForMultipleObjects(2, m_triggerEvent, FALSE, std::max((int)m_bufferSectionSize,1000)) == WAIT_OBJECT_0);
 
   { // Aborted
     PWaitAndSignal mutex(m_bufferMutex);
     PTRACE(4, "Recording read abort");
     if (m_captureBuffer != NULL) { // still open
       m_captureBuffer->Stop();
-      CheckCaptureBuffer(notification);
-	}
+      CheckCaptureBuffer();
+    }
   }
-  return SetErrorValues(Interrupted, EINTR, LastReadError);
+  return SetErrorValues(Interrupted, ECANCELED, LastReadError);
 }
 
 
@@ -1311,14 +1237,14 @@ PBoolean PSoundChannelDirectSound::OpenMixer (UINT waveDeviceId)
   mixerOpen(&m_mixer, waveDeviceId, NULL, NULL, (activeDirection == Player)? MIXER_OBJECTF_WAVEOUT : MIXER_OBJECTF_WAVEIN);
   if (m_mixer == NULL) {
     PTRACE(4, "Open" << GetDirectionText() << ": Failed to open mixer - volume control will not function");
-	return false;
+    return false;
   }
   MIXERLINE line = { sizeof(MIXERLINE) };
   line.dwComponentType = (activeDirection == Player)? MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT : MIXERLINE_COMPONENTTYPE_DST_WAVEIN;
   if (mixerGetLineInfo((HMIXEROBJ)m_mixer, &line, MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE) != MMSYSERR_NOERROR) {
     PTRACE(4, "Open" << GetDirectionText() << ": Failed to access mixer line - volume control will not function");
     CloseMixer();
-	return false;
+    return false;
   }
   m_volumeControl.cbStruct = sizeof(m_volumeControl);
 
@@ -1333,7 +1259,7 @@ PBoolean PSoundChannelDirectSound::OpenMixer (UINT waveDeviceId)
   if (mixerGetLineControls((HMIXEROBJ)m_mixer, &controls, MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR) {
     PTRACE(4, "Open" << GetDirectionText() << ": Failed to configure volume control - volume control will not function");
     CloseMixer();
-	return false;
+    return false;
   }
   return true;
 }
@@ -1407,15 +1333,4 @@ PBoolean PSoundChannelDirectSound::GetVolume (unsigned & devVol) // public
   devVol = 100 * (volume.dwValue - m_volumeControl.Bounds.dwMinimum) / (m_volumeControl.Bounds.dwMaximum - m_volumeControl.Bounds.dwMinimum);
   return true;
 }
-
-
-///////////////////////////////////////////////////////////////////////////////
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("Direct Sound support DISABLED")
-  #endif
-
 #endif // P_DIRECTSOUND
