@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28935 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:12:14 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 34144 $
+ * $Author: rjongbloed $
+ * $Date: 2015-10-05 15:42:35 +0100 (Mon, 05 Oct 2015) $
  */
 
 #include <ptlib.h>
@@ -201,9 +201,8 @@ PUInt64 PString::AsUnsigned64(unsigned base) const
 
 struct tm * PTime::os_localtime(const time_t * clock, struct tm * tb)
 {
-  struct tm * tp = ::localtime(clock);
-  if (tp != NULL)
-    return tp;
+  if (::localtime_s(tb, clock) == 0)
+    return tb;
 
   memset(tb, 0, sizeof(*tb));
   return tb;
@@ -212,9 +211,8 @@ struct tm * PTime::os_localtime(const time_t * clock, struct tm * tb)
 
 struct tm * PTime::os_gmtime(const time_t * clock, struct tm * tb)
 {
-  struct tm * tp = ::gmtime(clock);
-  if (tp != NULL)
-    return tp;
+  if (::gmtime_s(tb, clock) == 0)
+    return tb;
 
   memset(tb, 0, sizeof(*tb));
   return tb;
@@ -248,6 +246,11 @@ PBoolean PChannel::Close()
   return false;
 }
 
+FILE * PChannel::FDOpen(const char * /*mode*/)
+{
+  return NULL;
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // Directories
@@ -261,7 +264,7 @@ PDirectory PDirectory::GetParent() const
 }
 
 
-PBoolean PDirectory::Change(const PString & p)
+bool PDirectory::Change(const PString & p)
 {
   PDirectory d = p;
 
@@ -273,7 +276,7 @@ PBoolean PDirectory::Change(const PString & p)
 }
 
 
-PBoolean PDirectory::Filtered()
+bool PDirectory::InternalEntryCheck()
 {
 #ifdef UNICODE
   PString name(fileinfo.cFileName);
@@ -281,19 +284,21 @@ PBoolean PDirectory::Filtered()
   char * name = fileinfo.cFileName;
 #endif // UNICODE
   if (strcmp(name, ".") == 0)
-    return true;
+    return m_scanMask & PFileInfo::CurrentDirectory;
   if (strcmp(name, "..") == 0)
-    return true;
-  if (scanMask == PFileInfo::AllPermissions)
+    return m_scanMask & PFileInfo::ParentDirectory;
+  if (!PFilePath::IsValid(name))
     return false;
 
-  PFileInfo inf;
-  PAssert(PFile::GetInfo(*this+name, inf), POperatingSystemError);
-  return (inf.type&scanMask) == 0;
+  PFileInfo info;
+  if (!PFile::GetInfo(*this + name, info))
+    return false; // Probably means file disappeared between dir open and here
+
+  return info.type & m_scanMask;
 }
 
 
-PBoolean PDirectory::IsRoot() const
+bool PDirectory::IsRoot() const
 {
   if ((*this)[1] == ':')
     return GetLength() == 3;
@@ -334,20 +339,20 @@ PStringArray PDirectory::GetPath() const
 }
 
 
-PBoolean PDirectory::GetInfo(PFileInfo & info) const
+bool PDirectory::GetInfo(PFileInfo & info) const
 {
   return PFile::GetInfo(*this + GetEntryName(), info);
 }
 
 
-PBoolean PDirectory::Exists(const PString & path)
+bool PDirectory::Exists(const PString & path)
 {
   if (_access(path, 0) != 0)
     return false;
 
   // Could be a file, so actually need to try and open it
   PDirectory dir(path);
-  return dir.Open();
+  return dir.Open(PFileInfo::AllFiles);
 }
 
 
@@ -499,11 +504,11 @@ void PFilePath::SetType(const PCaselessString & type)
 void PFile::SetFilePath(const PString & newName)
 {
   if (!IsOpen())
-    path = newName;
+    m_path = newName;
 }
 
 
-PBoolean PFile::Access(const PFilePath & name, OpenMode mode)
+bool PFile::Access(const PFilePath & name, OpenMode mode)
 {
   int accmode;
 
@@ -542,48 +547,11 @@ PBoolean PFile::Remove(const PString & name, PBoolean force)
 }
 
 
-PBoolean PFile::Rename(const PFilePath & oldname, const PString & newname, PBoolean force)
-{
-  if (newname.FindOneOf(":\\/") != P_MAX_INDEX) {
-#ifdef _WIN32_WCE
-    set_errno(EINVAL);
-#else
-    errno = EINVAL;
-#endif // _WIN32_WCE
-    return false;
-  }
-  PString fullname = oldname.GetDirectory() + newname;
-  if (rename(oldname, fullname) == 0)
-    return true;
-  if (!force || errno == ENOENT || !Exists(fullname))
-    return false;
-  if (!Remove(fullname, true))
-    return false;
-  return rename(oldname, fullname) == 0;
-}
-
-
-PBoolean PFile::Move(const PFilePath & oldname, const PFilePath & newname, PBoolean force)
-{
-  if (rename(oldname, newname) == 0)
-    return true;
-  if (errno == ENOENT)
-    return false;
-  if (force && Exists(newname)) {
-    if (!Remove(newname, true))
-      return false;
-    if (rename(oldname, newname) == 0)
-      return true;
-  }
-  return Copy(oldname, newname, force) && Remove(oldname);
-}
-
-
 #ifdef _WIN32_WCE
 time_t	FileTimeToTime(const FILETIME FileTime);
 time_t	SystemTimeToTime(const LPSYSTEMTIME pSystemTime);
 
-PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
+bool PFile::GetInfo(const PFilePath & name, PFileInfo & info)
 {
   PString fn = name;
   PINDEX pos = fn.GetLength()-1;
@@ -630,18 +598,18 @@ PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
 
 #if defined(_WIN32)
 
-static void TwiddleBits(int newPermissions,
-                        int & oldPermissions,
-                        int permission,
+static void TwiddleBits(PFileInfo::Permissions newPermissions,
+                        PFileInfo::Permissions & oldPermissions,
+                        PFileInfo::Permissions permission,
                         DWORD & mask,
                         DWORD access)
 {
-  if (newPermissions < 0) {
+  if (newPermissions == PFileInfo::NoPermissions) {
     if ((mask&access) == access)
       oldPermissions |= permission;
   }
   else {
-    if ((newPermissions&permission) != 0)
+    if (newPermissions & permission)
       mask |= access;
     else
       mask &= ~access;
@@ -649,7 +617,7 @@ static void TwiddleBits(int newPermissions,
 }
 
 
-static int FileSecurityPermissions(const PFilePath & filename, int newPermissions)
+static PFileInfo::Permissions FileSecurityPermissions(const PFilePath & filename, PFileInfo::Permissions newPermissions)
 {
   // All of the following is to support cygwin style permissions
 
@@ -663,7 +631,7 @@ static int FileSecurityPermissions(const PFilePath & filename, int newPermission
                        storage.GetSize(),
                        &lengthNeeded)) {
     if (GetLastError() != ERROR_INSUFFICIENT_BUFFER || lengthNeeded == 0)
-      return -1;
+      return PFileInfo::NoPermissions;
 
     descriptor = (SECURITY_DESCRIPTOR *)storage.GetPointer(lengthNeeded);
     if (!GetFileSecurity(filename,
@@ -671,23 +639,23 @@ static int FileSecurityPermissions(const PFilePath & filename, int newPermission
                          descriptor,
                          storage.GetSize(),
                          &lengthNeeded))
-      return -1;
+                         return PFileInfo::NoPermissions;
   }
 
   BOOL daclPresent, daclDefaulted;
   PACL dacl;
   if (!GetSecurityDescriptorDacl(descriptor, &daclPresent, &dacl, &daclDefaulted))
-    return -1;
+    return PFileInfo::NoPermissions;
 
   if (!daclPresent || daclDefaulted || dacl == NULL)
-    return -1;
+    return PFileInfo::NoPermissions;
 
 
   ACL_SIZE_INFORMATION aclSize;
   if (!GetAclInformation(dacl, &aclSize, sizeof(aclSize), AclSizeInformation))
-    return -1;
+    return PFileInfo::NoPermissions;
 
-  int oldPermissions = 0;
+  PFileInfo::Permissions oldPermissions;
   int cygwinMask = 0;
 
   for (DWORD aceIndex = 0; aceIndex< aclSize.AceCount; aceIndex++) {
@@ -739,7 +707,7 @@ static int FileSecurityPermissions(const PFilePath & filename, int newPermission
 
   // Only do it if have the three ACE entries as per cygwin
   if (cygwinMask != 7)
-    return -1;
+    return PFileInfo::NoPermissions;
 
   if (newPermissions != -1)
     SetFileSecurity(filename, DACL_SECURITY_INFORMATION, descriptor);
@@ -750,7 +718,7 @@ static int FileSecurityPermissions(const PFilePath & filename, int newPermission
 #endif
 
 
-PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
+bool PFile::GetInfo(const PFilePath & name, PFileInfo & info)
 {
   if (name.IsEmpty())
     return false;
@@ -771,11 +739,10 @@ PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
   info.size = s.st_size;
 
 #if defined(_WIN32)
-  info.permissions = FileSecurityPermissions(name, -1);
-  if (info.permissions < 0)
+  info.permissions = FileSecurityPermissions(name, PFileInfo::NoPermissions);
+  if (info.permissions == PFileInfo::NoPermissions)
 #endif
   {
-    info.permissions = 0;
     if ((s.st_mode&S_IREAD) != 0)
       info.permissions |= PFileInfo::UserRead|PFileInfo::GroupRead|PFileInfo::WorldRead;
     if ((s.st_mode&S_IWRITE) != 0)
@@ -811,7 +778,7 @@ PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
 
 #endif // _WIN32_WCE
 
-PBoolean PFile::SetPermissions(const PFilePath & name, int permissions)
+bool PFile::SetPermissions(const PFilePath & name, PFileInfo::Permissions permissions)
 {
 #if defined(_WIN32) && !defined(_WIN32_WCE)
   FileSecurityPermissions(name, permissions);
@@ -827,13 +794,13 @@ PBoolean PFile::IsTextFile() const
 }
 
 
-PBoolean PFile::Open(OpenMode mode, OpenOptions opts)
+bool PFile::InternalOpen(OpenMode mode, OpenOptions opts, PFileInfo::Permissions permissions)
 {
   Close();
   clear();
 
-  if (path.IsEmpty())
-    path = PFilePath("PTL", NULL);
+  if (m_path.IsEmpty())
+    m_path = PFilePath("PTL", NULL);
 
   int oflags = IsTextFile() ? _O_TEXT : _O_BINARY;
   switch (mode) {
@@ -867,7 +834,7 @@ PBoolean PFile::Open(OpenMode mode, OpenOptions opts)
     oflags |= O_TRUNC;
 
   if (opts & Temporary)
-    removeOnClose = true;
+    m_removeOnClose = true;
 
   int sflags = _SH_DENYNO;
   if (opts & DenySharedRead)
@@ -877,7 +844,7 @@ PBoolean PFile::Open(OpenMode mode, OpenOptions opts)
   else if (opts & (DenySharedRead|DenySharedWrite))
     sflags = _SH_DENYWR;
 
-  os_handle = _sopen(path, oflags, sflags, S_IREAD|S_IWRITE);
+  os_handle = _sopen(m_path, oflags, sflags, permissions.AsBits());
 
   // As ConvertOSError tests for < 0 and some return values _sopen may be
   // negative, only pass -1 through.
@@ -890,6 +857,14 @@ PBoolean PFile::SetLength(off_t len)
   return ConvertOSError(_chsize(GetOSHandleAsInt(), len));
 }
 
+FILE * PFile::FDOpen(const char * mode)
+{
+  FILE * h = _fdopen((int)os_handle, mode);
+  if (h != NULL)
+    os_handle = -1;
+  return h;
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // PTextFile
@@ -904,11 +879,13 @@ PBoolean PTextFile::IsTextFile() const
 // PConsoleChannel
 
 PConsoleChannel::PConsoleChannel()
+  : m_lastMouseState(0)
 {
 }
 
 
 PConsoleChannel::PConsoleChannel(ConsoleType type)
+  : m_lastMouseState(0)
 {
   Open(type);
 }
@@ -944,13 +921,97 @@ PString PConsoleChannel::GetName() const
 }
 
 
+int PConsoleChannel::ReadChar()
+{
+  {
+    DWORD mode;
+    if (!GetConsoleMode(m_hConsole, &mode))
+      return -1;
+    if ((mode & ENABLE_LINE_INPUT) != 0)
+      return PChannel::ReadChar();
+  }
+
+  for (;;) {
+    INPUT_RECORD input;
+    DWORD numRead;
+    if (!ReadConsoleInput(m_hConsole, &input, 1, &numRead))
+      return PChannel::ReadChar();
+
+    if (numRead == 0)
+      continue;
+
+    switch (input.EventType) {
+      case KEY_EVENT :
+        if (input.Event.KeyEvent.bKeyDown)
+          continue;
+
+        switch (input.Event.KeyEvent.wVirtualKeyCode) {
+          case VK_LEFT :
+            return KeyLeft;
+          case VK_RIGHT :
+            return KeyRight;
+          case VK_UP :
+            return KeyUp;
+          case VK_DOWN :
+            return KeyDown;
+          case VK_PRIOR :
+            return KeyPageUp;
+          case VK_NEXT :
+            return KeyPageDown;
+          case VK_HOME :
+            return KeyHome;
+          case VK_END :
+            return KeyEnd;
+          case VK_BACK :
+            return KeyBackSpace;
+          case VK_DELETE :
+            return KeyDelete;
+          case VK_INSERT :
+            return KeyInsert;
+        }
+
+        if (input.Event.KeyEvent.wVirtualKeyCode >= VK_F1 && input.Event.KeyEvent.wVirtualKeyCode <= VK_F24)
+          return input.Event.KeyEvent.wVirtualKeyCode - KeyF1;
+
+        if (input.Event.KeyEvent.uChar.UnicodeChar != 0)
+          return (input.Event.KeyEvent.uChar.UnicodeChar&0xffff);
+
+        break;
+
+      case MOUSE_EVENT :
+        if (input.Event.MouseEvent.dwButtonState != m_lastMouseState) {
+          int code = MouseEvent;
+
+          DWORD buttons = m_lastMouseState|input.Event.MouseEvent.dwButtonState;
+          static const DWORD Mask[] = { FROM_LEFT_1ST_BUTTON_PRESSED, RIGHTMOST_BUTTON_PRESSED, FROM_LEFT_2ND_BUTTON_PRESSED, FROM_LEFT_3RD_BUTTON_PRESSED };
+          for (PINDEX btn = 0; btn < PARRAYSIZE(Mask); ++btn) {
+            if (buttons&Mask[btn]) {
+              code |= MouseButton1 << btn;
+              if (input.Event.MouseEvent.dwButtonState&&Mask[btn])
+                code |= (btn+1) << MouseClickShift;
+            }
+          }
+
+          if (input.Event.MouseEvent.dwEventFlags&DOUBLE_CLICK)
+            code |= MouseDoubleClick;
+
+          code |= input.Event.MouseEvent.dwMousePosition.X << MouseColShift;
+          code |= input.Event.MouseEvent.dwMousePosition.Y << MouseRowShift;
+
+          m_lastMouseState = input.Event.MouseEvent.dwButtonState;
+          return code;
+        }
+        break;
+    }
+  }
+}
+
+
 PBoolean PConsoleChannel::Read(void * buffer, PINDEX length)
 {
   if (!m_hConsole.IsValid())
     return ConvertOSError(-2, LastReadError);
 
-  flush();
-
   DWORD readBytes;
   if (ReadFile(m_hConsole, buffer, length, &readBytes, NULL)) {
     lastReadCount = readBytes;
@@ -989,14 +1050,23 @@ PBoolean PConsoleChannel::Close()
 }
 
 
-#ifdef _WIN32_WCE
-bool PConsoleChannel::SetLocalEcho(bool)
-{
-  return false;
-}
-#else
 bool PConsoleChannel::SetLocalEcho(bool localEcho)
 {
+  return InternalSetConsoleMode(ENABLE_ECHO_INPUT, localEcho);
+}
+
+
+bool PConsoleChannel::SetLineBuffered(bool lineBuffered)
+{
+  return InternalSetConsoleMode(ENABLE_LINE_INPUT, lineBuffered);
+}
+
+
+bool PConsoleChannel::InternalSetConsoleMode(DWORD bit, bool on)
+{
+  if (os_handle != StandardInput)
+    return ConvertOSError(-2, LastReadError);
+
   if (!m_hConsole.IsValid())
     return ConvertOSError(-2, LastReadError);
 
@@ -1004,14 +1074,13 @@ bool PConsoleChannel::SetLocalEcho(bool localEcho)
   if (!GetConsoleMode(m_hConsole, &mode))
     return false;
 
-  if (localEcho)
-    mode |= ENABLE_ECHO_INPUT;
+  if (on)
+    mode |= bit;
   else
-    mode &= ~ENABLE_ECHO_INPUT;
+    mode &= ~bit;
 
   return SetConsoleMode(m_hConsole, mode);
 }
-#endif
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1037,6 +1106,7 @@ void SignalHandler(int sig)
 
 void PProcess::Construct()
 {
+  m_waitOnExitConsoleWindow = true;
   PSetErrorStream(&cerr);
 
 #if !defined(_WIN32) && defined(_MSC_VER) && defined(_WINDOWS)
