@@ -26,15 +26,18 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29106 $
- * $Author: csoutheren $
- * $Date: 2013-02-14 23:08:54 -0600 (Thu, 14 Feb 2013) $
+ * $Revision: 33900 $
+ * $Author: rjongbloed $
+ * $Date: 2015-08-21 23:10:33 +0100 (Fri, 21 Aug 2015) $
  */
 
 #include <ptlib.h>
-#include <ptlib/sound.h>
+#include <ptlib/pprocess.h>
 
-#include <ptlib/plugin.h>
+
+#define P_FORCE_STATIC_PLUGIN 1
+
+#include <ptlib/sound.h>
 #include <ptlib/msos/ptlib/sound_win32.h>
 
 #include <math.h>
@@ -464,12 +467,6 @@ PBoolean PSound::Play(const PString & device)
   return channel.PlaySound(*this, true);
 }
 
-PBoolean PSound::PlayFile(const PFilePath & file, PBoolean wait)
-{
-  PVarString filename = file;
-  return ::PlaySound(filename, NULL, SND_FILENAME|(wait ? SND_SYNC : SND_ASYNC));
-}
-
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -554,40 +551,18 @@ DWORD PWaveBuffer::Release()
 
 ///////////////////////////////////////////////////////////////////////////////
 
-
-
-
-
 PSoundChannelWin32::PSoundChannelWin32()
+  : hWaveIn(NULL)
+  , hWaveOut(NULL)
+  , hMixer(NULL)
+  , hEventDone(CreateEvent(NULL, false, false, NULL))
+  , opened(false)
+  , m_reopened(false)
+  , bufferIndex(0)
+  , bufferByteOffset(P_MAX_INDEX)
 {
-  Construct();
-}
-
-
-PSoundChannelWin32::PSoundChannelWin32(const PString & device,
-                             Directions dir,
-                             unsigned numChannels,
-                             unsigned sampleRate,
-                             unsigned bitsPerSample)
-{
-  Construct();
-  Open(device, dir, numChannels, sampleRate, bitsPerSample);
-}
-
-
-void PSoundChannelWin32::Construct()
-{
-  opened = false;
-  direction = Player;
-  hWaveOut = NULL;
-  hWaveIn = NULL;
-  hEventDone = CreateEvent(NULL, false, false, NULL);
-
   waveFormat.SetFormat(1, 8000, 16);
-
-  bufferByteOffset = P_MAX_INDEX;
-
-  SetBuffers(32768, 3);
+  SetBuffers(32768, 2);
 }
 
 
@@ -600,9 +575,15 @@ PSoundChannelWin32::~PSoundChannelWin32()
 }
 
 
+const char * PSoundChannelWin32::GetDriverName()
+{
+  return PPlugin_PSoundChannel_WindowsMultimedia::ServiceName();
+}
+
+
 PString PSoundChannelWin32::GetName() const
 {
-  return deviceName;
+  return PConstString("WindowsMultimedia") + PPluginServiceDescriptor::SeparatorChar + deviceName;
 }
 
 
@@ -638,6 +619,12 @@ static bool GetWaveInDeviceName(UINT id, PString & name)
 }
 
 
+PString PSoundChannelWin32::GetDefaultDevice(Directions)
+{
+  return "Default";
+}
+
+
 PStringArray PSoundChannelWin32::GetDeviceNames(Directions dir)
 {
   PStringArray devices;
@@ -673,7 +660,7 @@ PStringArray PSoundChannelWin32::GetDeviceNames(Directions dir)
 
 PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir, unsigned& id)
 {
-  PINDEX offset = device.Find(PDevicePluginServiceDescriptor::SeparatorChar);
+  PINDEX offset = device.Find(PPluginServiceDescriptor::SeparatorChar);
   if (offset == P_MAX_INDEX)
     offset = 0;
   else
@@ -732,21 +719,19 @@ PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir,
   return true;
 }
 
-PBoolean PSoundChannelWin32::Open(const PString & device,
-                                  Directions dir,
-                                  unsigned numChannels,
-                                  unsigned sampleRate,
-                                  unsigned bitsPerSample)
+
+bool PSoundChannelWin32::Open(const Params & params)
 {
   Close();
   unsigned id = 0;
 
-  if( !GetDeviceID(device, dir, id) )
+  if( !GetDeviceID(params.m_device, params.m_direction, id) )
     return false;
 
-  waveFormat.SetFormat(numChannels, sampleRate, bitsPerSample);
+  waveFormat.SetFormat(params.m_channels, params.m_sampleRate, params.m_bitsPerSample);
+  SetBuffers(params.m_bufferSize, params.m_bufferCount);
 
-  direction = dir;
+  activeDirection = params.m_direction;
   return OpenDevice(id);
 }
 
@@ -762,7 +747,7 @@ PBoolean PSoundChannelWin32::Open(const PString & device,
 
   waveFormat = format;
 
-  direction = dir;
+  activeDirection = dir;
   return OpenDevice(id);
 }
 
@@ -782,7 +767,7 @@ PBoolean PSoundChannelWin32::OpenDevice(P_INT_PTR id)
   MIXERLINE line;
 
   MMRESULT osError = MMSYSERR_BADDEVICEID;
-  switch (direction) {
+  switch (activeDirection) {
     case Player :
       PTRACE(4, "WinSnd\twaveOutOpen, id=" << id);
       osError = waveOutOpen(&hWaveOut, (UINT)id, format, (DWORD_PTR)hEventDone, 0, CALLBACK_EVENT);
@@ -830,11 +815,7 @@ PBoolean PSoundChannelWin32::OpenDevice(P_INT_PTR id)
     bool haveControl = true;
     MIXERLINECONTROLS controls;
 
-    if ((direction == Recorder)
-#ifndef _WIN32_WCE
-      && ((DWORD)(LOBYTE(LOWORD(GetVersion()))) < 6)
-#endif
-      ) { //5=XP/win2003
+    if ((activeDirection == Recorder) && !PProcess::IsOSVersion(6,0,0)) { //5=XP/win2003
       /* There is no "master" for the recording side, so need to find the
          single selected input or at least individual microphone input
          No need to do all of these on Vista/Win7/Win2008 as there is
@@ -944,7 +925,7 @@ PBoolean PSoundChannelWin32::OpenDevice(P_INT_PTR id)
         if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
                 MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) != MMSYSERR_NOERROR) {
           PTRACE(2, "WinSnd\tFailed to get mixer line mute control ("
-                 << (direction == Recorder ? "Recorder" : "Player") << "), error=" << osError);
+                 << (activeDirection == Recorder ? "Recorder" : "Player") << "), error=" << osError);
         }
         return true;
       }
@@ -1004,8 +985,8 @@ unsigned PSoundChannelWin32::GetSampleSize() const
 
 PBoolean PSoundChannelWin32::Close()
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
   PWaitAndSignal mutex(bufferMutex);
 
@@ -1037,7 +1018,7 @@ PBoolean PSoundChannelWin32::Close()
 PBoolean PSoundChannelWin32::SetBuffers(PINDEX size, PINDEX count)
 {
   if (count == buffers.GetSize() && size == buffers[0].GetSize())
-	  return true;
+      return true;
 
   Abort();
 
@@ -1082,6 +1063,40 @@ PBoolean PSoundChannelWin32::GetBuffers(PINDEX & size, PINDEX & count)
 }
 
 
+int PSoundChannelWin32::WaitEvent(ErrorGroup group)
+{
+  switch (WaitForSingleObject(hEventDone, std::max((int)buffers[0].GetSize(), 1000))) {
+    case WAIT_OBJECT_0 :
+      PTRACE_IF(3, m_reopened, "WinSnd\tRecovered from stuck device");
+      m_reopened = false;
+      return 1;
+
+    case WAIT_TIMEOUT :
+      if (CheckNotOpen())
+        return false;
+
+      if (m_reopened) {
+        PTRACE(2, "WinSnd\tCould not recover from stuck device");
+        return SetErrorValues(Timeout, ETIMEDOUT, group);
+      }
+
+      PTRACE(3, "WinSnd\tTimeout, reopening stuck device");
+      if (!OpenDevice(os_handle))
+        return false;
+
+      // Throw away signal that is executed in Close() (via OpenDevice())
+      if (WaitForSingleObject(hEventDone, 0) == WAIT_OBJECT_0) {
+        m_reopened = true;
+        return -1;
+      }
+
+      PTRACE(2, "WinSnd\tUnexpected error waiting for event, code=" << ::GetLastError());
+  }
+
+  return SetErrorValues(Miscellaneous, ::GetLastError()|PWIN32ErrorFlag, group);
+}
+
+
 PBoolean PSoundChannelWin32::Write(const void * data, PINDEX size)
 {
   lastWriteCount = 0;
@@ -1099,8 +1114,14 @@ PBoolean PSoundChannelWin32::Write(const void * data, PINDEX size)
     while ((buffer.header.dwFlags&WHDR_DONE) == 0) {
       bufferMutex.Signal();
       // No free buffers, so wait for one
-      if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0)
-        return SetErrorValues(Miscellaneous, ::GetLastError()|PWIN32ErrorFlag, LastWriteError);
+      switch (WaitEvent(LastWriteError)) {
+        case 0 :
+          return false;
+        case -1 :
+          return Write(data, size);
+        case 1 :
+          break;
+      }
       bufferMutex.Wait();
     }
 
@@ -1179,76 +1200,6 @@ PBoolean PSoundChannelWin32::PlaySound(const PSound & sound, PBoolean wait)
 }
 
 
-PBoolean PSoundChannelWin32::PlayFile(const PFilePath & filename, PBoolean wait)
-{
-  Abort();
-
-  PMultiMediaFile mmio;
-  PWaveFormat fileFormat;
-  DWORD dataSize;
-  if (!mmio.OpenWaveFile(filename, fileFormat, dataSize))
-    return SetErrorValues(NotOpen, mmio.GetLastError()|PWIN32ErrorFlag, LastWriteError);
-
-  // Save old format and set to one loaded from file.
-  unsigned numChannels = waveFormat->nChannels;
-  unsigned sampleRate = waveFormat->nSamplesPerSec;
-  unsigned bitsPerSample = waveFormat->wBitsPerSample;
-  waveFormat = fileFormat;
-  if (!OpenDevice(os_handle)) {
-    SetFormat(numChannels, sampleRate, bitsPerSample);
-    return false;
-  }
-
-  bufferMutex.Wait();
-
-  DWORD osError = MMSYSERR_NOERROR;
-  while (dataSize > 0) {
-    PWaveBuffer & buffer = buffers[bufferIndex];
-    while ((buffer.header.dwFlags&WHDR_DONE) == 0) {
-      bufferMutex.Signal();
-      // No free buffers, so wait for one
-      if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0) {
-        osError = ::GetLastError();
-        SetFormat(numChannels, sampleRate, bitsPerSample);
-        return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastWriteError);
-      }
-      bufferMutex.Wait();
-    }
-
-    // Can't write more than a buffer full
-    PINDEX count = dataSize;
-    if ((osError = buffer.Prepare(hWaveOut, count)) != MMSYSERR_NOERROR)
-      break;
-
-    // Read the waveform data subchunk
-    if (!mmio.Read(buffer.GetPointer(), count)) {
-      osError = mmio.GetLastError();
-      break;
-    }
-
-    if ((osError = waveOutWrite(hWaveOut, &buffer.header, sizeof(WAVEHDR))) != MMSYSERR_NOERROR)
-      break;
-
-    bufferIndex = (bufferIndex+1)%buffers.GetSize();
-    dataSize -= count;
-  }
-
-  bufferMutex.Signal();
-
-  if (osError != MMSYSERR_NOERROR) {
-    SetFormat(numChannels, sampleRate, bitsPerSample);
-    return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastWriteError);
-  }
-
-  if (dataSize == 0 && wait) {
-    WaitForPlayCompletion();
-    SetFormat(numChannels, sampleRate, bitsPerSample);
-  }
-
-  return true;
-}
-
-
 PBoolean PSoundChannelWin32::HasPlayCompleted()
 {
   PWaitAndSignal mutex(bufferMutex);
@@ -1265,7 +1216,7 @@ PBoolean PSoundChannelWin32::HasPlayCompleted()
 PBoolean PSoundChannelWin32::WaitForPlayCompletion()
 {
   while (!HasPlayCompleted()) {
-    if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0)
+    if (!WaitEvent(LastWriteError))
       return false;
   }
 
@@ -1431,12 +1382,20 @@ PBoolean PSoundChannelWin32::WaitForRecordBufferFull()
     return false;
 
   while (!IsRecordBufferFull()) {
-    if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0)
-      return false;
+    switch (WaitEvent(LastReadError)) {
+      case 0 :
+        return false;
 
-    PWaitAndSignal mutex(bufferMutex);
-    if (bufferByteOffset == P_MAX_INDEX)
-      return false;
+      case -1 :
+        if (StartRecording())  // Restart the read
+          break;
+        return false;
+
+      case  1 :
+        PWaitAndSignal mutex(bufferMutex);
+        if (bufferByteOffset == P_MAX_INDEX)
+          return false;
+    }
   }
 
   return true;
@@ -1449,12 +1408,20 @@ PBoolean PSoundChannelWin32::WaitForAllRecordBuffersFull()
     return false;
 
   while (!AreAllRecordBuffersFull()) {
-    if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0)
-      return false;
+    switch (WaitEvent(LastReadError)) {
+      case 0 :
+        return false;
 
-    PWaitAndSignal mutex(bufferMutex);
-    if (bufferByteOffset == P_MAX_INDEX)
-      return false;
+      case -1 :
+        if (StartRecording())  // Restart the read
+          break;
+        return false;
+
+      case  1 :
+        PWaitAndSignal mutex(bufferMutex);
+        if (bufferByteOffset == P_MAX_INDEX)
+          return false;
+    }
   }
 
   return true;
@@ -1502,7 +1469,7 @@ PString PSoundChannelWin32::GetErrorText(ErrorGroup group) const
     return PChannel::GetErrorText(group);
 
   DWORD osError = lastErrorNumber[group]&~PWIN32ErrorFlag;
-  if (direction == Recorder) {
+  if (activeDirection == Recorder) {
     if (waveInGetErrorText(osError, str, sizeof(str)) != MMSYSERR_NOERROR)
       return PChannel::GetErrorText(group);
   }
@@ -1517,8 +1484,10 @@ PString PSoundChannelWin32::GetErrorText(ErrorGroup group) const
 
 PBoolean PSoundChannelWin32::SetVolume(unsigned newVolume)
 {
-  if (!IsOpen() || hMixer == NULL)
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
+  if (hMixer == NULL)
+    return SetErrorValues(Unavailable, EBADF);
 
   MIXERCONTROLDETAILS_UNSIGNED volume;
   if (newVolume >= MaxVolume)
@@ -1547,8 +1516,10 @@ PBoolean PSoundChannelWin32::SetVolume(unsigned newVolume)
 
 PBoolean PSoundChannelWin32::GetVolume(unsigned & oldVolume)
 {
-  if (!IsOpen() || hMixer == NULL)
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
+  if (hMixer == NULL)
+    return SetErrorValues(Unavailable, EBADF);
 
   MIXERCONTROLDETAILS_UNSIGNED volume;
 
@@ -1571,8 +1542,10 @@ PBoolean PSoundChannelWin32::GetVolume(unsigned & oldVolume)
 
 bool PSoundChannelWin32::SetMute(bool newMute)
 {
-  if (!IsOpen() || hMixer == NULL)
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    false;
+  if (hMixer == NULL)
+    return SetErrorValues(Unavailable, EBADF);
 
   MIXERCONTROLDETAILS_UNSIGNED mute;
   mute.dwValue = newMute;
@@ -1596,8 +1569,10 @@ bool PSoundChannelWin32::SetMute(bool newMute)
 
 bool PSoundChannelWin32::GetMute(bool & oldMute)
 {
-  if (!IsOpen() || hMixer == NULL)
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
+  if (hMixer == NULL)
+    return SetErrorValues(Unavailable, EBADF);
 
   MIXERCONTROLDETAILS_UNSIGNED mute;
 
