@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29525 $
+ * $Revision: 34130 $
  * $Author: rjongbloed $
- * $Date: 2013-04-17 19:04:16 -0500 (Wed, 17 Apr 2013) $
+ * $Date: 2015-10-03 15:52:19 +0100 (Sat, 03 Oct 2015) $
  */
 
 #include <ptlib.h>
@@ -111,6 +111,9 @@ static const char * const ServiceCommandNames[NumSvcCmds] = {
 static const char WindowLogOutput[] = "Window Log Output";
 static const char DebuggerLogOutput[] = "Debugger Log Output";
 
+static char emptyString[] = "";
+static char crlfString[] = "\r\n";
+
 
 class PNotifyIconData : public NOTIFYICONDATA {
   public:
@@ -211,21 +214,6 @@ class PSystemLogToEvent : public PSystemLogTarget
     if (hEventSource == NULL)
       return;
 
-    PString threadName;
-    PThread * thread = PThread::Current();
-    if (thread != NULL)
-      threadName = thread->GetThreadName();
-    else
-      threadName.sprintf(PTHREAD_ID_FMT, GetCurrentThreadId());
-
-    char thrdbuf[16];
-    if (threadName.IsEmpty())
-      sprintf(thrdbuf, "0x%8p", (void *)thread);
-    else {
-      strncpy(thrdbuf, threadName, sizeof(thrdbuf)-1);
-      thrdbuf[sizeof(thrdbuf)-1] = '\0';
-    }
-
     char errbuf[25];
     if (level > PSystemLog::StdError && level < PSystemLog::Info && err != 0)
       ::sprintf(errbuf, "Error code = %lu", err);
@@ -233,7 +221,8 @@ class PSystemLogToEvent : public PSystemLogTarget
       errbuf[0] = '\0';
 
     LPCTSTR strings[4];
-    strings[0] = thrdbuf;
+    PString threadName = PThread::GetCurrentThreadName();
+    strings[0] = threadName.GetPointer();
     strings[1] = msg;
     strings[2] = errbuf;
     strings[3] = level != PSystemLog::Fatal ? "" : " Program aborted.";
@@ -262,7 +251,7 @@ class PSystemLogToEvent : public PSystemLogTarget
 // PServiceProcess
 
 PServiceProcess::PServiceProcess(const char * manuf, const char * name,
-                           WORD major, WORD minor, CodeStatus stat, WORD build)
+                                 unsigned major, unsigned minor, CodeStatus stat, unsigned build)
   : PProcess(manuf, name, major, minor, stat, build, false, true)
   , m_controlWindow(NULL)
   , m_debugWindow(NULL)
@@ -278,6 +267,17 @@ PServiceProcess & PServiceProcess::Current()
 }
 
 
+void PServiceProcess::SetLogLevel(PSystemLog::Level level)
+{
+  PSystemLog::GetTarget().SetThresholdLevel(level);
+
+#if PTRACING
+  if ((PTrace::GetOptions()&PTrace::SystemLogStream) != 0)
+    PTrace::SetLevel(level);
+#endif
+}
+
+
 const char * PServiceProcess::GetServiceDependencies() const
 {
   return "EventLog\0";
@@ -308,27 +308,24 @@ int PServiceProcess::InternalMain(void * arg)
   {
     PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
 
-#if SPECiAL_DEBUG_OUTPUT
-    PSetErrorStream(new ofstream("D:\\temp\\svcprog.log"));
-    PError << "PServiceProcess::InternalMain - " << PTime().AsString("hh:mm:ss.uuuuuu") << endl;
-#else
     PSetErrorStream(new PSystemLog(PSystemLog::StdError));
-#endif
-
 #if PTRACING
     PTrace::SetStream(new PSystemLog(PSystemLog::Debug3));
-    PTrace::ClearOptions(PTrace::FileAndLine);
-    PTrace::SetOptions(PTrace::SystemLogStream);
-    PTrace::SetLevel(4);
+  #if _DEBUG
+    PTrace::ClearOptions(PTrace::Timestamp);
+    PTrace::SetOptions(PTrace::FileAndLine | PTrace::ContextIdentifier | PTrace::SystemLogStream);
+  #else
+    PTrace::ClearOptions(PTrace::Timestamp | PTrace::FileAndLine);
+    PTrace::SetOptions(PTrace::ContextIdentifier | PTrace::SystemLogStream);
+  #endif
+    PTrace::SetLevel(GetLogLevel());
 #endif
   }
 
   hInstance = (HINSTANCE)arg;
 
-  OSVERSIONINFO verinfo;
-  verinfo.dwOSVersionInfoSize = sizeof(verinfo);
-  GetVersionEx(&verinfo);
-  if (verinfo.dwPlatformId != VER_PLATFORM_WIN32_NT) {
+  
+  if (!PProcess::IsOSVersion(5,1,0)) {
     PError << "Unsupported Win32 platform type!" << endl;
     return 1;
   }
@@ -392,14 +389,14 @@ int PServiceProcess::InternalMain(void * arg)
 
   if (m_debugMode) {
     ::SetLastError(0);
-    PError << "Service simulation started for \"" << GetName()
-           << "\" version " << GetVersion(true)
-           << " by " << GetManufacturer()
-           << " on " << GetOSClass() << ' ' << GetOSName()
-           << " (" << GetOSVersion() << '-' << GetOSHardware()
-           << ") with PTLib (v" << GetLibVersion()
-           << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
-           << "\nClose window to terminate.\n" << endl;
+    DebugOutput(PSTRSTRM("Service simulation started for \"" << GetName()
+                << "\" version " << GetVersion(true)
+                << " by " << GetManufacturer()
+                << " on " << GetOSClass() << ' ' << GetOSName()
+                << " (" << GetOSVersion() << '-' << GetOSHardware()
+                << ") with PTLib (v" << GetLibVersion()
+                << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
+                << "\nClose window to terminate.\n"));
   }
 
   m_terminationEvent = CreateEvent(NULL, true, false, GetName());
@@ -436,8 +433,8 @@ int PServiceProcess::InternalMain(void * arg)
         if (!m_debugMode || m_controlWindow == NULL)
           msg.message = WM_QUIT;
         else {
-          PError << "nService simulation stopped for \"" << GetName() << "\".\n\n"
-                    "Close window to terminate.\n" << endl;
+          DebugOutput("\nService simulation stopped.\n\n"
+                      "Close window to terminate.");
           ResetEvent(m_terminationEvent);
         }
     }
@@ -758,7 +755,7 @@ LPARAM PServiceProcess::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
 
         case DeleteMenuID :
           if (m_debugWindow != NULL && m_debugWindow != (HWND)-1)
-            SendMessage(m_debugWindow, WM_CLEAR, 0, 0);
+            SendMessage(m_debugWindow, EM_REPLACESEL, false, (LPARAM)emptyString);
           break;
 
         case SelectAllMenuID :
@@ -823,12 +820,8 @@ LPARAM PServiceProcess::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
           break;
 
         default :
-          if (wParam >= LogLevelBaseMenuID+PSystemLog::Fatal && wParam < LogLevelBaseMenuID+PSystemLog::NumLogLevels) {
+          if (wParam >= LogLevelBaseMenuID+PSystemLog::Fatal && wParam < LogLevelBaseMenuID+PSystemLog::NumLogLevels)
             SetLogLevel((PSystemLog::Level)(wParam-LogLevelBaseMenuID));
-#if PTRACING
-            PTrace::SetLevel((UINT)wParam-LogLevelBaseMenuID-PSystemLog::Warning);
-#endif
-          }
           else if (wParam >= SvcCmdBaseMenuID && wParam < SvcCmdBaseMenuID+NumSvcCmds) {
             const char * cmdname = ServiceCommandNames[wParam-SvcCmdBaseMenuID];
             if (wParam == SvcCmdBaseMenuID+SvcCmdVersion ||
@@ -917,9 +910,6 @@ LPARAM PServiceProcess::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
   return DefWindowProc(hWnd, msg, wParam, lParam);
 }
 
-static char emptyString[] = "";
-static char crlfString[] = "\r\n";
-
 void PServiceProcess::DebugOutput(const char * out)
 {
   if (m_controlWindow == NULL)
@@ -953,18 +943,18 @@ void PServiceProcess::DebugOutput(const char * out)
   }
 
   SendMessage(m_debugWindow, EM_SETSEL, 128000, 128000);
-  char * lf;
-  char * prev = (char *)out;
+  const char * lf;
+  const char * prev = out;
   while ((lf = strchr(prev, '\n')) != NULL) {
-    if (*(lf-1) == '\r')
-      prev = lf+1;
-    else {
-      *lf++ = '\0';
-      SendMessage(m_debugWindow, EM_REPLACESEL, false, (LPARAM)out);
-      SendMessage(m_debugWindow, EM_REPLACESEL, false, (LPARAM)crlfString);
-      out = (const char *)lf;
-      prev = lf;
+    if (lf == out || *(lf-1) != '\r') {
+      size_t len = lf - out;
+      char * line = (char *)alloca(len+3);
+      memcpy(line, out, len);
+      strcpy(line+len, crlfString);
+      SendMessage(m_debugWindow, EM_REPLACESEL, false, (LPARAM)line);
+      out = lf+1;
     }
+    prev = lf+1;
   }
   if (*out != '\0')
     SendMessage(m_debugWindow, EM_REPLACESEL, false, (LPARAM)out);
@@ -1051,6 +1041,10 @@ void PServiceProcess::ThreadEntry()
     SetTerminationValue(0);
     Main();
   }
+  else {
+    if (m_debugMode)
+      DebugOutput("Failed to initialise service.\n");
+  }
 
   ReportStatus(SERVICE_STOP_PENDING, terminationValue, 1, 30000);
   SetEvent(m_terminationEvent);
@@ -1164,15 +1158,15 @@ class NT_ServiceManager
     NT_ServiceManager()  { schSCManager = schService = NULL; error = 0; }
     ~NT_ServiceManager();
 
-    virtual bool Create(PServiceProcess * svc);
-    virtual bool Remove(PServiceProcess * svc);
-    virtual bool Start(PServiceProcess * svc);
-    virtual bool Stop(PServiceProcess * svc)   { return Control(svc, SERVICE_CONTROL_STOP); }
-    virtual bool Pause(PServiceProcess * svc)  { return Control(svc, SERVICE_CONTROL_PAUSE); }
-    virtual bool Resume(PServiceProcess * svc) { return Control(svc, SERVICE_CONTROL_CONTINUE); }
-    virtual bool SetConfig(PServiceProcess * svc, SC_ACTION_TYPE action);
-    virtual bool IsInstalled(PServiceProcess * svc);
-    virtual bool IsRunning(PServiceProcess * svc);
+    bool Create(PServiceProcess * svc);
+    bool Remove(PServiceProcess * svc);
+    bool Start(PServiceProcess * svc);
+    bool Stop(PServiceProcess * svc)   { return Control(svc, SERVICE_CONTROL_STOP); }
+    bool Pause(PServiceProcess * svc)  { return Control(svc, SERVICE_CONTROL_PAUSE); }
+    bool Resume(PServiceProcess * svc) { return Control(svc, SERVICE_CONTROL_CONTINUE); }
+    bool SetConfig(PServiceProcess * svc, SC_ACTION_TYPE action);
+    bool IsInstalled(PServiceProcess * svc);
+    bool IsRunning(PServiceProcess * svc);
 
     DWORD GetError() const { return error; }
 
@@ -1249,6 +1243,16 @@ bool NT_ServiceManager::Create(PServiceProcess * svc)
     return false;
   }
 
+  PString description = svc->GetDescription();
+  if (!description.IsEmpty()) {
+    SERVICE_DESCRIPTION sd;
+    sd.lpDescription = (LPTSTR)description.GetPointer();
+    if (!ChangeServiceConfig2(schService, SERVICE_CONFIG_DESCRIPTION, &sd)) {
+      error = ::GetLastError();
+      PError << "Could not set service description, error=" << error << endl;
+    }
+  }
+
   HKEY key;
   if ((error = RegCreateKey(HKEY_LOCAL_MACHINE,
              "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\" +
@@ -1575,6 +1579,16 @@ bool PServiceProcess::ProcessCommand(const char * cmd)
   return good;
 }
 
+void PServiceProcess::SetDescription(const PString & description)
+{
+  m_description = description;
+}
+
+const PString & PServiceProcess::GetDescription() const
+{
+  return m_description;
+}
+
 #endif // !_WIN32_WCE
 
 // End Of File ///////////////////////////////////////////////////////////////
