@@ -24,20 +24,25 @@
  * Contributor(s): ______________________________________.
  *                 Walter H Whitlock (twohives@nc.rr.com)
  *
- * $Revision: 28935 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:12:14 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 33839 $
+ * $Author: rjongbloed $
+ * $Date: 2015-08-06 20:15:12 +0100 (Thu, 06 Aug 2015) $
  */
 
 #include <ptlib.h>
 
 #if P_VIDEO
 
+#define P_FORCE_STATIC_PLUGIN 1
+
 #include <ptlib/videoio.h>
 #include <ptlib/vconvert.h>
 #include <ptlib/pluginmgr.h>
 #include <ptclib/delaychan.h>
 
+#include <windowsx.h>
+
+
 #if P_VFW_CAPTURE
 
 #ifdef _MSC_VER
@@ -63,6 +68,7 @@ BOOL VFWAPI capGetDriverDescriptionA (WORD wDriverIndex, LPSTR lpszName,
 #endif // __MINGW32
 
 
+#define PTraceModule() "VfW Grab"
 
 /**This class defines a video input device.
  */
@@ -288,7 +294,7 @@ PVideoDeviceBitmap::PVideoDeviceBitmap(HWND hCaptureWindow)
   PINDEX sz = capGetVideoFormatSize(hCaptureWindow);
   SetSize(sz);
   if (!capGetVideoFormat(hCaptureWindow, theArray, sz)) { 
-    PTRACE(1, "PVidInp\tcapGetVideoFormat(hCaptureWindow) failed - " << ::GetLastError());
+    PTRACE(1, "capGetVideoFormat(hCaptureWindow) failed - " << ::GetLastError());
     SetSize(0);
     return;
   }
@@ -299,7 +305,7 @@ PVideoDeviceBitmap::PVideoDeviceBitmap(HWND hCaptureWindow, WORD bpp)
   PINDEX sz = capGetVideoFormatSize(hCaptureWindow);
   SetSize(sz);
   if (!capGetVideoFormat(hCaptureWindow, theArray, sz)) { 
-    PTRACE(1, "PVidInp\tcapGetVideoFormat(hCaptureWindow) failed - " << ::GetLastError());
+    PTRACE(1, "capGetVideoFormat(hCaptureWindow) failed - " << ::GetLastError());
     SetSize(0);
     return;
   }
@@ -332,7 +338,7 @@ PBoolean PVideoDeviceBitmap::ApplyFormat(HWND hWnd, const FormatTableEntry & for
 #endif
 
   if (capSetVideoFormat(hWnd, theArray, GetSize())) {
-    PTRACE(3, "PVidInp\tcapSetVideoFormat succeeded: "
+    PTRACE(3, "capSetVideoFormat succeeded: "
             << PString(formatTableEntry.colourFormat) << ' '
             << bmi.bmiHeader.biWidth << "x" << bmi.bmiHeader.biHeight
             << " sz=" << bmi.bmiHeader.biSizeImage << " time=" << (PTimer::Tick() - startTime));
@@ -342,7 +348,7 @@ PBoolean PVideoDeviceBitmap::ApplyFormat(HWND hWnd, const FormatTableEntry & for
   if (formatTableEntry.negHeight) {
     bmi.bmiHeader.biHeight = height; 
     if (capSetVideoFormat(hWnd, theArray, GetSize())) {
-      PTRACE(3, "PVidInp\tcapSetVideoFormat succeeded: "
+      PTRACE(3, "capSetVideoFormat succeeded: "
               << PString(formatTableEntry.colourFormat) << ' '
               << bmi.bmiHeader.biWidth << "x" << bmi.bmiHeader.biHeight
               << " sz=" << bmi.bmiHeader.biSizeImage << " time=" << (PTimer::Tick() - startTime));
@@ -350,7 +356,7 @@ PBoolean PVideoDeviceBitmap::ApplyFormat(HWND hWnd, const FormatTableEntry & for
     }
   }
 
-  PTRACE(1, "PVidInp\tcapSetVideoFormat failed: "
+  PTRACE(1, "capSetVideoFormat failed: "
           << (formatTableEntry.colourFormat != NULL ? formatTableEntry.colourFormat : "NO-COLOUR-FORMAT") << ' '
           << bmi.bmiHeader.biWidth << "x" << bmi.bmiHeader.biHeight
           << " sz=" << bmi.bmiHeader.biSizeImage << " time=" << (PTimer::Tick() - startTime)
@@ -366,7 +372,7 @@ PCapStatus::PCapStatus(HWND hWnd)
   if (capGetStatus(hWnd, this, sizeof(*this)))
     return;
 
-  PTRACE(1, "PVidInp\tcapGetStatus: failed - " << ::GetLastError());
+  PTRACE(1, "capGetStatus: failed - " << ::GetLastError());
 }
 
 
@@ -412,7 +418,7 @@ bool PVideoInputDevice_VideoForWindows::SetCaptureMode(unsigned mode)
 
   if (!result) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tFailed to set callback on VfW - " << lastError);
+    PTRACE(1, "Failed to set callback on VfW - " << lastError);
     return false;
   }
 
@@ -420,7 +426,7 @@ bool PVideoInputDevice_VideoForWindows::SetCaptureMode(unsigned mode)
   memset(&parms, 0, sizeof(parms));
   if (!capCaptureGetSetup(hCaptureWindow, &parms, sizeof(parms))) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapCaptureGetSetup: failed - " << lastError);
+    PTRACE(1, "capCaptureGetSetup: failed - " << lastError);
     return false;
   }
 
@@ -436,7 +442,7 @@ bool PVideoInputDevice_VideoForWindows::SetCaptureMode(unsigned mode)
 
   if (!capCaptureSetSetup(hCaptureWindow, &parms, sizeof(parms))) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapCaptureSetSetup: failed - " << lastError);
+    PTRACE(1, "capCaptureSetSetup: failed - " << lastError);
     return false;
   }
 
@@ -500,7 +506,7 @@ PBoolean PVideoInputDevice_VideoForWindows::Close()
       // Any of the two ios better than just hanging
       captureThread->Terminate();
       hCaptureWindow = NULL;
-      PTRACE(1, "PVidInp\tCapture thread failed to stop. Terminated");
+      PTRACE(1, "Capture thread failed to stop. Terminated");
   }
 
   delete captureThread;
@@ -561,7 +567,7 @@ PBoolean PVideoInputDevice_VideoForWindows::Start()
   }
 
   lastError = ::GetLastError();
-  PTRACE(1, "PVidInp\tcapCaptureSequenceNoFile: failed - " << lastError);
+  PTRACE(1, "capCaptureSequenceNoFile: failed - " << lastError);
   return false;
 }
 
@@ -583,7 +589,7 @@ PBoolean PVideoInputDevice_VideoForWindows::Stop()
     return true;
 
   lastError = ::GetLastError();
-  PTRACE(1, "PVidInp\tcapCaptureStop: failed - " << lastError);
+  PTRACE(1, "capCaptureStop: failed - " << lastError);
   return false;
 }
 
@@ -664,7 +670,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameRate(unsigned rate)
 
   if (!capCaptureGetSetup(hCaptureWindow, &parms, sizeof(parms))) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapCaptureGetSetup: failed - " << lastError);
+    PTRACE(1, "capCaptureGetSetup: failed - " << lastError);
     return false;
   }
 
@@ -681,7 +687,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameRate(unsigned rate)
 
   if (!capCaptureSetSetup(hCaptureWindow, &parms, sizeof(parms))) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapCaptureSetSetup: failed - " << lastError);
+    PTRACE(1, "capCaptureSetSetup: failed - " << lastError);
     return false;
   }
     
@@ -704,7 +710,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameSize(unsigned width, unsigne
     Stop();
 
   PVideoDeviceBitmap bi(hCaptureWindow); 
-  PTRACE(5, "PVidInp\tChanging frame size from "
+  PTRACE(5, "Changing frame size from "
          << bi->bmiHeader.biWidth << 'x' << bi->bmiHeader.biHeight << " to " << width << 'x' << height);
 
   PINDEX i = 0;
@@ -800,7 +806,7 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameData(BYTE * buffer, PINDEX *
 PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, PINDEX * bytesReturned)
 {
   if (!frameAvailable.Wait(1000)) {
-    PTRACE(1, "PVidInp\tTimeout waiting for frame grab!");
+    PTRACE(1, "Timeout waiting for frame grab!");
     return false;
   }
 
@@ -837,12 +843,9 @@ LRESULT CALLBACK PVideoInputDevice_VideoForWindows::ErrorHandler(HWND hWnd, int
 }
 
 
-LRESULT PVideoInputDevice_VideoForWindows::HandleError(int id, LPCSTR PTRACE_PARAM(err))
+LRESULT PVideoInputDevice_VideoForWindows::HandleError(int PTRACE_PARAM(id), LPCSTR PTRACE_PARAM(err))
 {
-  if (id != 0) {
-    PTRACE(1, "PVidInp\tErrorHandler: [id="<< id << "] " << err);
-  }
-
+  PTRACE_IF(1, id != 0, "ErrorHandler: [id="<< id << "] " << err);
   return true;
 }
 
@@ -900,7 +903,7 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
                                                (HWND)0,
                                                0)) == NULL) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapCreateCaptureWindow failed - " << lastError);
+    PTRACE(1, "capCreateCaptureWindow failed - " << lastError);
     return false;
   }
 
@@ -914,7 +917,7 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
 
   if (!result) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tFailed to set callback on VfW - " << lastError);
+    PTRACE(1, "Failed to set callback on VfW - " << lastError);
     return false;
   }
 
@@ -923,7 +926,7 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
 #if PTRACING
   if (PTrace::CanTrace(4)) { // list available video capture drivers
     ostream & trace = PTRACE_BEGIN(5);
-    trace << "PVidInp\tEnumerating available video capture drivers:\n";
+    trace << "Enumerating available video capture drivers:\n";
     for (devId = 0; devId < 10; devId++) { 
       char name[100];
       char version[200];
@@ -951,7 +954,7 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
   // Use first driver available.
   if (!capDriverConnect(hCaptureWindow, devId)) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapDriverConnect failed - " << lastError);
+    PTRACE(1, "capDriverConnect failed - " << lastError);
     return false;
   }
 
@@ -959,11 +962,11 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
   memset(&driverCaps, 0, sizeof(driverCaps));
   if (!capDriverGetCaps(hCaptureWindow, &driverCaps, sizeof(driverCaps))) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapGetDriverCaps failed - " << lastError);
+    PTRACE(1, "capGetDriverCaps failed - " << lastError);
     return false;
   }
 
-  PTRACE(6, "PVidInp\tEnumerating CAPDRIVERCAPS values:\n"
+  PTRACE(6, "Enumerating CAPDRIVERCAPS values:\n"
             "  driverCaps.wDeviceIndex           = " << driverCaps.wDeviceIndex        << "\n"
             "  driverCaps.fHasOverlay            = " << driverCaps.fHasOverlay         << "\n"
             "  driverCaps.fHasDlgVideoSource     = " << driverCaps.fHasDlgVideoSource  << "\n"
@@ -1014,14 +1017,14 @@ void PVideoInputDevice_VideoForWindows::HandleCapture(PThread &, P_INT_PTR)
       ::DispatchMessage(&msg);
   }
 
-  PTRACE(5, "PVidInp\tDisconnecting driver");
+  PTRACE(5, "Disconnecting driver");
   capDriverDisconnect(hCaptureWindow);
   capSetUserData(hCaptureWindow, NULL);
 
   capSetCallbackOnError(hCaptureWindow, NULL);
   capSetCallbackOnVideoStream(hCaptureWindow, NULL);
 
-  PTRACE(5, "PVidInp\tDestroying VIDCAP window");
+  PTRACE(5, "Destroying VIDCAP window");
   DestroyWindow(hCaptureWindow);
   hCaptureWindow = NULL;
 
@@ -1083,6 +1086,35 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
     virtual PStringArray GetDeviceNames() const
     { return GetOutputDeviceNames(); }
 
+    enum SizeMode {
+      NormalSize,
+      HalfSize,
+      DoubleSize,
+      FullScreen,
+      FixedSize,
+      NumSizeModes
+    };
+
+    /**Get the number of video channels available on the device.
+    */
+    virtual int GetNumChannels() { return NumSizeModes; }
+
+    /**Get the names of video channels available on the device.
+    */
+    virtual PStringArray GetChannelNames();
+
+    /**Set the video channel to be used on the device.
+       The channel number is an integer from 0 to GetNumChannels()-1. The
+       special value of -1 will find the first working channel number.
+    */
+    virtual PBoolean SetChannel(
+      int channelNumber  ///< New channel number for device.
+    );
+
+    /**Get the video channel to be used on the device.
+    */
+    virtual int GetChannel() const { return m_sizeMode; }
+
     /**Set the colour format to be used.
        Note that this function does not do any conversion. If it returns true
        then the video device does the colour format in native mode.
@@ -1150,11 +1182,18 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
 
   protected:
     PDECLARE_NOTIFIER(PThread, PVideoOutputDevice_Window, HandleDisplay);
+    static VOID CALLBACK TimerProc(_In_ HWND, _In_ UINT, _In_ UINT_PTR, _In_ DWORD);
     void CreateDisplayWindow();
-    void SetWindowSize();
     void Draw(HDC hDC);
 
     HWND       m_hWnd;
+    HWND       m_hParent;
+    DWORD      m_dwStyle;
+    DWORD      m_dwExStyle;
+    COLORREF   m_bgColour;
+    int        m_rotation;
+    bool       m_mouseEnabled;
+    bool       m_hidden;
     PThread  * m_thread;
     PMutex     m_openCloseMutex;
     PSyncPoint m_started;
@@ -1162,69 +1201,53 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
     bool       m_flipped;
     POINT      m_lastPosition;
     SIZE       m_fixedSize;
+    SizeMode   m_sizeMode;
+    bool       m_showInfo;
+    unsigned   m_frameCount;
+    unsigned   m_observedFrameRate;
+    PSimpleTimer m_rateTimer;
 };
 
 
 #define DEFAULT_STYLE (WS_POPUP|WS_BORDER|WS_SYSMENU|WS_CAPTION)
 #define DEFAULT_TITLE "Video Output"
 
-static bool ParseWindowDeviceName(const PString & deviceName, DWORD * dwStylePtr = NULL, HWND * hWndParentPtr = NULL)
-{
-  if (deviceName.Find("MSWIN") != 0)
-    return false;
+PCREATE_VIDOUTPUT_PLUGIN_EX(Window,
 
-  PINDEX pos = deviceName.Find("STYLE=");
-  DWORD dwStyle = pos == P_MAX_INDEX ? DEFAULT_STYLE : strtoul(((const char *)deviceName)+pos+6, NULL, 0);
-  if ((dwStyle&(WS_POPUP|WS_CHILD)) == 0) {
-    PTRACE(1, "VidOut\tWindow must be WS_POPUP or WS_CHILD window.");
-    return false;
+  virtual const char * GetFriendlyName() const
+  {
+    return "Microsoft Windows Video Output";
   }
 
-  HWND hWndParent = NULL;
-  pos = deviceName.Find("PARENT=");
-  if (pos != P_MAX_INDEX) {
-    hWndParent = (HWND)strtoul(((const char *)deviceName)+pos+7, NULL, 0);
-    if (!::IsWindow(hWndParent)) {
-      PTRACE(2, "VidOut\tIllegal parent window " << hWndParent << " specified.");
-      hWndParent = NULL;
-    }
+  virtual bool ValidateDeviceName(const PString & deviceName, P_INT_PTR /*userData*/) const
+  {
+    return deviceName.NumCompare(P_MSWIN_VIDEO_PREFIX) == PObject::EqualTo;
   }
-
-  // Have parsed out style & parent, see if legal combination
-  if (hWndParent == NULL && (dwStyle&WS_POPUP) == 0) {
-    PTRACE(1, "VidOut\tWindow must be WS_POPUP if parent window not specified.");
-    return false;
-  }
-
-  if (dwStylePtr != NULL)
-    *dwStylePtr = dwStyle;
-
-  if (hWndParentPtr != NULL)
-    *hWndParentPtr = hWndParent;
-
-  return true;
-}
-
-
-class PVideoOutputDevice_Window_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject *    CreateInstance(int /*userData*/) const { return PNEW PVideoOutputDevice_Window; }
-    virtual PStringArray GetDeviceNames(int /*userData*/) const { return PVideoOutputDevice_Window::GetOutputDeviceNames(); }
-    virtual bool         ValidateDeviceName(const PString & deviceName, int /*userData*/) const { return ParseWindowDeviceName(deviceName); }
-} PVideoOutputDevice_Window_descriptor;
-
-PCREATE_PLUGIN(Window, PVideoOutputDevice, &PVideoOutputDevice_Window_descriptor);
+)
 
 
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoOutputDeviceRGB
 
+#undef PTraceModule
+#define PTraceModule() PPlugin_PVideoOutputDevice_Window::ServiceName()
+
 PVideoOutputDevice_Window::PVideoOutputDevice_Window()
+  : m_hWnd(NULL)
+  , m_hParent(NULL)
+  , m_dwStyle(DEFAULT_STYLE)
+  , m_dwExStyle(0)
+  , m_bgColour(0) // Black
+  , m_rotation(0)
+  , m_mouseEnabled(true)
+  , m_hidden(false)
+  , m_thread(NULL)
+  , m_flipped(false)
+  , m_sizeMode(NormalSize)
+  , m_showInfo(false)
+  , m_frameCount(0)
+  , m_observedFrameRate(0)
 {
-  m_hWnd = NULL;
-  m_thread = NULL;
-  m_flipped = false;
   m_lastPosition.x = 0;
   m_lastPosition.y = 0;
   m_fixedSize.cx = 0;
@@ -1252,35 +1275,110 @@ PVideoOutputDevice_Window::~PVideoOutputDevice_Window()
 
 PStringArray PVideoOutputDevice_Window::GetOutputDeviceNames()
 {
-  return psprintf("MSWIN STYLE=0x%08X TITLE=\"%s\"", DEFAULT_STYLE, DEFAULT_TITLE);
+  return psprintf(P_MSWIN_VIDEO_PREFIX" STYLE=0x%08X TITLE=\"%s\"", DEFAULT_STYLE, DEFAULT_TITLE);
+}
+
+
+VOID CALLBACK PVideoOutputDevice_Window::TimerProc(_In_  HWND,
+                                                   _In_  UINT,
+                                                   _In_  UINT_PTR idEvent,
+                                                   _In_  DWORD)
+{
+  ((PVideoOutputDevice_Window*)idEvent)->CreateDisplayWindow();
+}
+
+
+static int GetTokenValue(const PString & deviceName, const char * token, int defaultValue)
+{
+  PINDEX pos = deviceName.Find(token);
+  if (pos == P_MAX_INDEX)
+    return defaultValue;
+
+  return strtoul(((const char *)deviceName)+pos+strlen(token), NULL, 0);
 }
 
 
 PBoolean PVideoOutputDevice_Window::Open(const PString & name, PBoolean startImmediate)
 {
+  if (name.NumCompare(P_MSWIN_VIDEO_PREFIX) != EqualTo)
+    return false;
+
   Close();
 
   m_openCloseMutex.Wait();
 
   deviceName = name;
 
-  if (deviceName.Find("PARENT") == P_MAX_INDEX) {
-    m_thread = PThread::Create(PCREATE_NOTIFIER(HandleDisplay), "VidOut");
-
-    m_openCloseMutex.Signal();
-    m_started.Wait();     
+  m_hParent = NULL;
+  PINDEX pos = deviceName.Find("PARENT=");
+  if (pos != P_MAX_INDEX) {
+    m_hParent = (HWND)strtoul(((const char *)deviceName)+pos+7, NULL, 0);
+    if (!::IsWindow(m_hParent)) {
+      PTRACE(2, "Illegal parent window " << m_hParent << " specified.");
+      return false;
+    }
   }
-  else {    
-    /* Description: child windows should be created on the same thread as the
-      parent window (well, not necessarily, but it is much less error prone
-      that way) so I am adding another method to handle creating child
-      windows which will be created in the calling thread.
-      Date: 03/15/2012
-      Author: Jonathan M. Henson
-     */
-    CreateDisplayWindow();
+
+  pos = deviceName.Find("STYLE=");
+  m_dwStyle = pos == P_MAX_INDEX ? DEFAULT_STYLE : strtoul(((const char *)deviceName)+pos+6, NULL, 0);
+  if ((m_dwStyle&(WS_POPUP|WS_CHILD)) == 0) {
+    PTRACE(1, "Window must be WS_POPUP or WS_CHILD window.");
+    return false;
+  }
+
+  // Have parsed out style & parent, see if legal combination
+  if (m_hParent == NULL && (m_dwStyle&WS_POPUP) == 0) {
+    PTRACE(1, "Must have parent for WS_CHILD window.");
+    return false;
+  }
+
+  m_lastPosition.x = GetTokenValue(deviceName, "X=", CW_USEDEFAULT);
+  m_lastPosition.y = GetTokenValue(deviceName, "Y=", CW_USEDEFAULT);
+  m_fixedSize.cx   = GetTokenValue(deviceName, "WIDTH=", 0);
+  m_fixedSize.cy   = GetTokenValue(deviceName, "HEIGHT=", 0);
+  m_bgColour       = GetTokenValue(deviceName, "BACKGROUND=", 0);
+  m_rotation       = GetTokenValue(deviceName, "ROTATION=", 0);
+
+  m_mouseEnabled = deviceName.Find("NO-MOUSE") == P_MAX_INDEX;
+  m_hidden = !startImmediate || deviceName.Find("HIDE") != P_MAX_INDEX;
+
+  if (deviceName.Find("FULLSCREEN") != P_MAX_INDEX)
+    m_sizeMode = FullScreen;
+  else if (deviceName.Find("HALF") != P_MAX_INDEX)
+    m_sizeMode = HalfSize;
+  else if (deviceName.Find("DOUBLE") != P_MAX_INDEX)
+    m_sizeMode = DoubleSize;
+  else if (m_fixedSize.cx > 0 && m_fixedSize.cy > 0)
+    m_sizeMode = FixedSize;
+  channelNumber = m_sizeMode;
+
+  if (m_lastPosition.x == CW_USEDEFAULT && m_lastPosition.y == CW_USEDEFAULT) {
+    if (m_hParent != NULL) {
+      RECT rect;
+      GetWindowRect(m_hParent, &rect);
+      m_lastPosition.x = (rect.right + rect.left - frameWidth)/2;
+      m_lastPosition.y = (rect.bottom + rect.top - frameHeight)/2;
+    }
+    else {
+      m_lastPosition.x = (GetSystemMetrics(SM_CXSCREEN) - frameWidth)/2;
+      m_lastPosition.y = (GetSystemMetrics(SM_CYSCREEN) - frameHeight)/2;
+    }
+  }
+  
+  m_showInfo = deviceName.Find("SHOWINFO") != P_MAX_INDEX;
+
+  if (m_hParent != NULL) {
+    // This is a sneaky way to get a callback in the context of the parent windows thread
+    SetTimer(m_hParent, (UINT_PTR)this, USER_TIMER_MINIMUM, TimerProc);
+    m_started.Wait();
+    KillTimer(m_hParent, (UINT_PTR)this);
     m_openCloseMutex.Signal();
   }
+  else {
+    m_thread = PThread::Create(PCREATE_NOTIFIER(HandleDisplay), "VidOut");
+    m_openCloseMutex.Signal();
+    m_started.Wait();
+  }
 
   return startImmediate ? Start() : IsOpen();
 }
@@ -1334,6 +1432,99 @@ PBoolean PVideoOutputDevice_Window::Stop()
 }
 
 
+PStringArray PVideoOutputDevice_Window::GetChannelNames()
+{
+  PStringArray names(NumSizeModes);
+  names[NormalSize] = "Normal size";
+  names[HalfSize] = "Half size";
+  names[DoubleSize] = "Double size";
+  names[FullScreen] = "Full screen";
+  names[FixedSize] = "Fixed size";
+  return names;
+}
+
+
+PBoolean PVideoOutputDevice_Window::SetChannel(int newChannelNumber)
+{
+  if (!PVideoOutputDeviceRGB::SetChannel(newChannelNumber))
+    return false;
+
+  if (m_hWnd != NULL) {
+    UINT flags = m_hidden ? SWP_HIDEWINDOW : SWP_SHOWWINDOW;
+
+    if (m_sizeMode == FullScreen) {
+      if (channelNumber == FullScreen)
+        return true;
+
+      SetWindowLong(m_hWnd, GWL_STYLE, m_dwStyle);
+      SetWindowLong(m_hWnd, GWL_EXSTYLE, m_dwExStyle);
+      flags |= SWP_FRAMECHANGED;
+    }
+
+    RECT rect;
+    rect.right = rect.left = m_lastPosition.x;
+    rect.bottom = rect.top = m_lastPosition.y;
+    bool adjust = true;
+    bool swapForRotation = m_rotation%180 != 0;
+    switch (channelNumber) {
+      case NormalSize :
+        rect.right = frameWidth;
+        rect.bottom = frameHeight;
+        break;
+
+      case HalfSize :
+        rect.right = frameWidth/2;
+        rect.bottom = frameHeight/2;
+        break;
+
+      case DoubleSize :
+        rect.right = frameWidth*2;
+        rect.bottom = frameHeight*2;
+        break;
+
+      case FixedSize :
+        rect.right = m_fixedSize.cx;
+        rect.bottom = m_fixedSize.cy;
+        swapForRotation = false;
+        break;
+
+      case FullScreen :
+        HMONITOR hmon = MonitorFromWindow(m_hWnd, MONITOR_DEFAULTTONEAREST);
+        MONITORINFO mi = { sizeof(mi) };
+        if (GetMonitorInfo(hmon, &mi)) {
+          SetWindowLong(m_hWnd, GWL_STYLE, m_dwStyle & ~(WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|WS_THICKFRAME));
+          SetWindowLong(m_hWnd, GWL_EXSTYLE, m_dwExStyle & ~(WS_EX_DLGMODALFRAME|WS_EX_CLIENTEDGE|WS_EX_STATICEDGE|WS_EX_WINDOWEDGE));
+          rect = mi.rcWork;
+          flags |= SWP_FRAMECHANGED;
+          adjust = false;
+        }
+        else {
+          rect.left = 0;
+          rect.top = 0;
+          rect.right = GetSystemMetrics(SM_CXFULLSCREEN);
+          rect.bottom = GetSystemMetrics(SM_CYFULLSCREEN);
+        }
+    }
+
+    if (adjust) {
+      if (swapForRotation)
+        std::swap(rect.right, rect.bottom);
+      rect.right += rect.left;
+      rect.bottom += rect.top;
+      AdjustWindowRectEx(&rect, m_dwStyle, false, m_dwExStyle);
+    }
+
+    PTRACE(4, "SetWindowPos: chan=" << channelNumber
+           << " pos=" << rect.left << 'x' << rect.top
+           << " size=" << rect.right-rect.left << 'x' << rect.bottom-rect.top);
+    ::SetWindowPos(m_hWnd, HWND_TOP, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, flags);
+  }
+
+  m_sizeMode = (SizeMode)channelNumber;
+  return true;
+}
+
+
 PBoolean PVideoOutputDevice_Window::SetColourFormat(const PString & colourFormat)
 {
   PWaitAndSignal m(mutex);
@@ -1380,29 +1571,17 @@ PBoolean PVideoOutputDevice_Window::SetFrameSize(unsigned width, unsigned height
   }
 
   // Must be outside of mutex
-  SetWindowSize();
+  SetChannel(GetChannel());
 
   return true;
 }
 
 
-void PVideoOutputDevice_Window::SetWindowSize()
-{
-  if (m_hWnd == NULL)
-    return;
-
-  RECT rect;
-  rect.top = 0;
-  rect.left = 0;
-  rect.bottom = m_fixedSize.cy > 0 ? m_fixedSize.cy : frameHeight;
-  rect.right = m_fixedSize.cx > 0 ? m_fixedSize.cx : frameWidth;
-  ::AdjustWindowRectEx(&rect, GetWindowLong(m_hWnd, GWL_STYLE), false, GetWindowLong(m_hWnd, GWL_EXSTYLE));
-  ::SetWindowPos(m_hWnd, HWND_TOP, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_NOMOVE);
-}
-
-
 PBoolean PVideoOutputDevice_Window::FrameComplete()
 {
+  if (m_hidden)
+    m_hidden = !ShowWindow(m_hWnd, SW_SHOW);
+
   PWaitAndSignal m(mutex);
 
   if (m_hWnd == NULL)
@@ -1426,17 +1605,91 @@ PBoolean PVideoOutputDevice_Window::GetPosition(int & x, int & y) const
 
 bool PVideoOutputDevice_Window::SetPosition(int x, int y)
 {
-  if (m_hWnd != NULL) {
-    RECT rect;
-    rect.top = y;
-    rect.left = x;
-    rect.bottom = y;
-    rect.right = x;
-    ::AdjustWindowRectEx(&rect, GetWindowLong(m_hWnd, GWL_STYLE), false, GetWindowLong(m_hWnd, GWL_EXSTYLE));
-    ::SetWindowPos(m_hWnd, HWND_TOP, x+(x-rect.left), y, 0, 0, SWP_NOSIZE);
+  if (m_hWnd == NULL)
+    return false;
+
+  return ::SetWindowPos(m_hWnd, HWND_TOP, x, y, 0, 0, SWP_NOSIZE|SWP_NOACTIVATE);
+}
+
+
+static void Rotate(PBYTEArray & frameStore, int frameWidth, int frameHeight, unsigned bytesPerPixel, unsigned angle)
+{
+  PBYTEArray destination(frameStore.GetSize());
+
+  if (bytesPerPixel == 4) {
+    // Split out the BGR32 version for faster operation
+    const uint32_t * src = (const uint32_t *)frameStore.GetPointer();
+    uint32_t * dst = (uint32_t *)destination.GetPointer();
+    switch (angle) {
+      case -90 :
+        dst += frameWidth*frameHeight;
+        for (int y = frameHeight; y > 0; --y) {
+          uint32_t * tempY = dst - y;
+          for (int x = frameWidth; x > 0; --x) {
+            *tempY = *src++;
+            tempY -= frameHeight;
+          }
+        }
+        break;
+
+      case 90 :
+        for (int y = frameHeight-1; y >= 0; --y) {
+          uint32_t * tempY = dst + y;
+          for (int x = frameWidth; x > 0; --x) {
+            *tempY = *src++;
+            tempY += frameHeight;
+          }
+        }
+        break;
+
+      case 180 :
+        dst += frameWidth*frameHeight;
+        for (int y = frameHeight; y > 0; --y) {
+          for (int x = frameWidth; x > 0; --x)
+            *--dst = *src++;
+        }
+        break;
+    }
+  }
+  else {
+    const BYTE * src = frameStore;
+    BYTE * dst = destination.GetPointer();
+    switch (angle) {
+      case -90 :
+        dst += frameWidth*frameHeight*bytesPerPixel;
+        for (int y = frameHeight; y > 0; --y) {
+          BYTE * tempY = dst - y*bytesPerPixel;
+          for (int x = frameWidth; x > 0; --x) {
+            for (unsigned i = 0; i < bytesPerPixel; ++i)
+              tempY[i] = *src++;
+            tempY -= frameHeight*bytesPerPixel;
+          }
+        }
+        break;
+
+      case 90 :
+        for (int y = frameHeight-1; y >= 0; --y) {
+          BYTE * tempY = dst + y*bytesPerPixel;
+          for (int x = frameWidth; x > 0; --x) {
+            for (unsigned i = 0; i < bytesPerPixel; ++i)
+              tempY[i] = *src++;
+            tempY += frameHeight*bytesPerPixel;
+          }
+        }
+        break;
+
+      case 180 :
+        dst += frameWidth*frameHeight*bytesPerPixel;
+        for (int y = frameHeight; y > 0; --y) {
+          for (int x = frameWidth; x > 0; --x)
+            for (unsigned i = 0; i < bytesPerPixel; ++i)
+              *--dst = *src++;
+        }
+        break;
+    }
   }
 
-  return true;
+  frameStore = destination;
 }
 
 
@@ -1445,27 +1698,114 @@ void PVideoOutputDevice_Window::Draw(HDC hDC)
   RECT rect;
   GetClientRect(m_hWnd, &rect);
 
-  int result;
-  if (frameWidth == (unsigned)rect.right && frameHeight == (unsigned)rect.bottom)
-    result = SetDIBitsToDevice(hDC,
-                               0, 0, frameWidth, frameHeight,
-                               0, 0, 0, frameHeight,
-                               frameStore.GetPointer(), &m_bitmap, DIB_RGB_COLORS);
+  HBRUSH brush = m_bgColour < 0x1000000 ? CreateSolidBrush(m_bgColour) : NULL;
+
+  if (frameStore.IsEmpty()) {
+    if (brush != NULL)
+      FillRect(hDC, &rect, brush);
+  }
   else {
+    int imageWidth = frameWidth;
+    int imageHeight = frameHeight;
+    if (m_rotation != 0) {
+      Rotate(frameStore, frameWidth, frameHeight, bytesPerPixel, m_rotation);
+      if (m_rotation != 180) {
+        imageWidth = frameHeight;
+        imageHeight = frameWidth;
+      }
+    }
+
+    m_bitmap.bmiHeader.biWidth = imageWidth;
+    m_bitmap.bmiHeader.biHeight = m_flipped ? imageHeight : -imageHeight;
+
+    int result;
+    if (imageWidth == rect.right && imageHeight == rect.bottom)
+      result = SetDIBitsToDevice(hDC,
+                                 0, 0, imageWidth, imageHeight,
+                                 0, 0, 0, imageHeight,
+                                 frameStore.GetPointer(), &m_bitmap, DIB_RGB_COLORS);
+    else {
+      int frameAspect = 1000*imageWidth/imageHeight;
+      int windowAspect = 1000*rect.right/rect.bottom;
+      int x,y,w,h;
+      if (frameAspect < windowAspect) {
+        w = imageWidth*rect.bottom/imageHeight;
+        h = rect.bottom;
+        x = (rect.right - w)/2;
+        y = 0;
+        if (brush != NULL) {
+          RECT r;
+          r.left = 0; r.top = 0; r.right = x; r.bottom = rect.bottom;
+          FillRect(hDC, &r, brush);
+          r.left = rect.right-x; r.right = rect.right;
+          FillRect(hDC, &r, brush);
+        }
+      }
+      else if (frameAspect > windowAspect) {
+        w = rect.right;
+        h = imageHeight*rect.right/imageWidth;
+        x = 0;
+        y = (rect.bottom - h)/2;
+        if (brush != NULL) {
+          RECT r;
+          r.left = 0; r.top = 0; r.right = rect.right; r.bottom = y;
+          FillRect(hDC, &r, brush);
+          r.top = rect.bottom - y; r.bottom = rect.bottom;
+          FillRect(hDC, &r, brush);
+        }
+      }
+      else {
+        x = y = 0;
+        w = rect.right;
+        h = rect.bottom;
+      }
+
 #ifdef _WIN32_WCE
-    SetStretchBltMode(hDC, COLORONCOLOR);
+      SetStretchBltMode(hDC, COLORONCOLOR);
 #else
-    SetStretchBltMode(hDC, STRETCH_DELETESCANS);
+      SetStretchBltMode(hDC, STRETCH_DELETESCANS);
 #endif
-    result = StretchDIBits(hDC,
-                           0, 0, rect.right, rect.bottom,
-                           0, 0, frameWidth, frameHeight,
-                           frameStore.GetPointer(), &m_bitmap, DIB_RGB_COLORS, SRCCOPY);
+      result = StretchDIBits(hDC,
+                             x, y, w, h,
+                             0, 0, imageWidth, imageHeight,
+                             frameStore.GetPointer(), &m_bitmap, DIB_RGB_COLORS, SRCCOPY);
+    }
+
+    if (result != imageHeight) {
+      lastError = ::GetLastError();
+      PTRACE(2, "Drawing image failed: resolution=" << rect.right << 'x' << rect.bottom
+             << ", bitmap=" << imageWidth << 'x' << imageHeight
+             << ", size=" << m_bitmap.bmiHeader.biSizeImage << ", result=" << result << ", error=" << lastError);
+    }
   }
 
-  if (result == 0) {
-    lastError = ::GetLastError();
-    PTRACE(2, "VidOut\tDrawing image failed, error=" << lastError);
+  if (brush != NULL)
+    DeleteObject(brush);
+
+  if (m_showInfo) {
+    PStringStream strm;
+    strm << " Video: " << frameWidth << 'x' << frameHeight;
+
+    ++m_frameCount;
+    if (m_rateTimer.HasExpired()) {
+      m_rateTimer.SetInterval(0, 1);
+      m_observedFrameRate = m_frameCount;
+      m_frameCount = 0;
+    }
+
+    if (m_observedFrameRate > 0)
+      strm << " @ " << m_observedFrameRate << "fps";
+    else
+      strm << " (stalled)";
+
+    if (frameWidth != (unsigned)rect.right && frameHeight != (unsigned)rect.bottom)
+      strm << " Window: " << rect.right << 'x' << rect.bottom;
+
+    SetTextColor(hDC, RGB(192,192,192));
+    SetBkMode(hDC, TRANSPARENT);
+    rect.left += 8;
+    rect.right -= 8;
+    DrawText(hDC, strm, strm.GetLength(), &rect, DT_TOP|DT_RIGHT|DT_SINGLELINE|DT_END_ELLIPSIS);
   }
 }
 
@@ -1483,17 +1823,6 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
 }
 
 
-static int GetTokenValue(const PString & deviceName, const char * token, int defaultValue)
-{
-  PINDEX pos = deviceName.Find(token);
-  if (pos == P_MAX_INDEX)
-    return defaultValue;
-
-  pos += strlen(token);
-  return deviceName.Mid(pos).AsInteger();
-}
-
-
 void PVideoOutputDevice_Window::CreateDisplayWindow()
 {
 #ifndef _WIN32_WCE
@@ -1512,45 +1841,26 @@ void PVideoOutputDevice_Window::CreateDisplayWindow()
     wndClass.lpszClassName = wndClassName;
     wndClass.lpszClassName = wndClassName;
     wndClass.lpfnWndProc = ::WndProc;
+    wndClass.hbrBackground = GetStockBrush(HOLLOW_BRUSH);
     wndClass.cbWndExtra = sizeof(this);
     PAssertOS(RegisterClass(&wndClass));
   }
 
-  DWORD dwStyle;
-  HWND hParent;
-
-  if (!ParseWindowDeviceName(deviceName, &dwStyle, &hParent))
-    return;
-
   PVarString title = DEFAULT_TITLE;
   PINDEX pos = deviceName.Find("TITLE=\"");
   if (pos != P_MAX_INDEX)
     title = PString(PString::Literal, deviceName.Mid(pos+6));
 
-  m_lastPosition.x = GetTokenValue(deviceName, "X=", CW_USEDEFAULT);
-  m_lastPosition.y = GetTokenValue(deviceName, "Y=", CW_USEDEFAULT);
-  m_fixedSize.cx   = GetTokenValue(deviceName, "WIDTH=", 0);
-  m_fixedSize.cy   = GetTokenValue(deviceName, "HEIGHT=", 0);
-
-  if (m_lastPosition.x == CW_USEDEFAULT && m_lastPosition.y == CW_USEDEFAULT) {
-    if (hParent != NULL) {
-      RECT rect;
-      GetWindowRect(hParent, &rect);
-      m_lastPosition.x = (rect.right + rect.left - frameWidth)/2;
-      m_lastPosition.y = (rect.bottom + rect.top - frameHeight)/2;
-    }
-    else {
-      m_lastPosition.x = (GetSystemMetrics(SM_CXSCREEN) - frameWidth)/2;
-      m_lastPosition.y = (GetSystemMetrics(SM_CYSCREEN) - frameHeight)/2;
-    }
+  if ((m_hWnd = CreateWindow(wndClassName,
+                             title, 
+                             m_dwStyle,
+                             m_lastPosition.x , m_lastPosition.y, frameWidth, frameHeight,
+                             m_hParent, NULL, GetModuleHandle(NULL), this)) != NULL) {
+    m_dwExStyle = GetWindowLong(m_hWnd, GWL_EXSTYLE);
+    SetChannel(GetChannel());
   }
-  
-  m_hWnd = CreateWindow(wndClassName,
-                        title, 
-                        dwStyle,
-                        m_lastPosition.x , m_lastPosition.y, frameWidth, frameHeight,
-                        hParent, NULL, GetModuleHandle(NULL), this);
-  SetWindowSize();
+
+  m_started.Signal();
 }
 
 
@@ -1558,8 +1868,6 @@ void PVideoOutputDevice_Window::HandleDisplay(PThread &, P_INT_PTR)
 {
   CreateDisplayWindow();
 
-  m_started.Signal();
-
   if (m_hWnd != NULL) {
     MSG msg;
     while (GetMessage(&msg, NULL, 0, 0))
@@ -1583,27 +1891,91 @@ LRESULT PVideoOutputDevice_Window::WndProc(UINT uMsg, WPARAM wParam, LPARAM lPar
       }
 
     case WM_MOVE :
-      if (m_hWnd != NULL) {
+      if (m_hWnd != NULL && m_sizeMode != FullScreen) {
+#if 0
         RECT rect;
-        GetWindowRect(m_hWnd, &rect);
+        rect.left = rect.right = GET_X_LPARAM(lParam);
+        rect.top = rect.bottom = GET_Y_LPARAM(lParam);
+        ::AdjustWindowRectEx(&rect, m_dwStyle, false, m_dwExStyle);
         m_lastPosition.x = rect.left;
         m_lastPosition.y = rect.top;
+#else
+        m_lastPosition.x = GET_X_LPARAM(lParam);
+        m_lastPosition.y = GET_Y_LPARAM(lParam);
+#endif
+        PTRACE(4, "Moved to " << m_lastPosition.x << 'x' << m_lastPosition.y);
       }
       break;
 
+    case WM_LBUTTONDOWN :
+      if (m_mouseEnabled)
+        PostMessage(m_hWnd, WM_NCLBUTTONDOWN, HTCAPTION, lParam);
+      break;
+
     case WM_LBUTTONDBLCLK :
-      if (m_fixedSize.cx < 10000 && m_fixedSize.cy < 10000) {
-        m_fixedSize.cx = 2*(m_fixedSize.cx > 0 ? m_fixedSize.cx : frameWidth);
-        m_fixedSize.cy = 2*(m_fixedSize.cy > 0 ? m_fixedSize.cy : frameHeight);
-        SetWindowSize();
+      if (m_mouseEnabled) {
+        switch (m_sizeMode) {
+          default :
+            SetChannel(NormalSize);
+            break;
+
+          case NormalSize :
+            SetChannel(DoubleSize);
+            break;
+
+          case FixedSize :
+            if (m_fixedSize.cx < 10000 && m_fixedSize.cy < 10000) {
+              m_fixedSize.cx *= 2;
+              m_fixedSize.cy *= 2;
+              SetChannel(FixedSize);
+            }
+        }
       }
       break;
 
     case WM_RBUTTONDBLCLK :
-      if ((m_fixedSize.cx&1) == 0 && (m_fixedSize.cy&1) == 0) {
-        m_fixedSize.cx = (m_fixedSize.cx > 0 ? m_fixedSize.cx : frameWidth)/2;
-        m_fixedSize.cy = (m_fixedSize.cy > 0 ? m_fixedSize.cy : frameHeight)/2;
-        SetWindowSize();
+      if (m_mouseEnabled) {
+        switch (m_sizeMode) {
+          default :
+            SetChannel(NormalSize);
+            break;
+
+          case NormalSize :
+            SetChannel(HalfSize);
+            break;
+
+          case FixedSize :
+            if (m_fixedSize.cx > 64 && m_fixedSize.cy > 36 && (m_fixedSize.cx&1) == 0 && (m_fixedSize.cy&1) == 0) {
+              m_fixedSize.cx /= 2;
+              m_fixedSize.cy /= 2;
+              SetChannel(FixedSize);
+            }
+        }
+      }
+      break;
+
+    case WM_RBUTTONDOWN :
+      if (m_mouseEnabled) {
+        HMENU hMenu = CreatePopupMenu();
+        AppendMenu(hMenu, MF_ENABLED|MF_STRING|(m_sizeMode == NormalSize ? MF_CHECKED : 0), 100+NormalSize, "Normal size");
+        AppendMenu(hMenu, MF_ENABLED|MF_STRING|(m_sizeMode == FullScreen ? MF_CHECKED : 0), 100+FullScreen, "Full screen");
+        AppendMenu(hMenu, MF_ENABLED|MF_STRING|(m_sizeMode == HalfSize   ? MF_CHECKED : 0), 100+HalfSize,   "Half size"  );
+        AppendMenu(hMenu, MF_ENABLED|MF_STRING|(m_sizeMode == DoubleSize ? MF_CHECKED : 0), 100+DoubleSize, "Double size");
+        AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
+        AppendMenu(hMenu, MF_ENABLED|MF_STRING|(m_showInfo ? MF_CHECKED : 0), 10, "Show Info");
+
+        POINT pt;
+        pt.x = GET_X_LPARAM(lParam);
+        pt.y = GET_Y_LPARAM(lParam);
+        ClientToScreen(m_hWnd, &pt);
+        int menu = TrackPopupMenu(hMenu, TPM_RIGHTBUTTON|TPM_RETURNCMD, pt.x, pt.y, 0, m_hWnd, NULL);
+        if (menu >= 100)
+          SetChannel(menu-100);
+        else {
+          m_showInfo = !m_showInfo;
+          InvalidateRect(m_hWnd, NULL, false);
+        }
+        DestroyMenu(hMenu);
       }
       break;
 
@@ -1613,7 +1985,8 @@ LRESULT PVideoOutputDevice_Window::WndProc(UINT uMsg, WPARAM wParam, LPARAM lPar
       break;
 
     case WM_DESTROY:
-      PostThreadMessage(GetCurrentThreadId(), WM_QUIT, 0, 0);
+      if (m_thread != NULL)
+        PostThreadMessage(GetCurrentThreadId(), WM_QUIT, 0, 0);
       break;
   }
   return DefWindowProc(m_hWnd, uMsg, wParam, lParam);
