@@ -19,26 +19,23 @@
  *
  * Contributor(s): Craig Southeren, Post Increment (C) 2008
  *
- * $Revision: 29675 $
+ * $Revision: 33709 $
  * $Author: rjongbloed $
- * $Date: 2013-05-10 01:03:57 -0500 (Fri, 10 May 2013) $
+ * $Date: 2015-07-03 22:07:02 +0100 (Fri, 03 Jul 2015) $
  */
 
 
 #include <ptlib.h>
 
-
 #if P_APPSHARE
 
+#define P_FORCE_STATIC_PLUGIN 1
+
 #include <ptlib/vconvert.h>
 #include <ptclib/delaychan.h>
 #include <ptlib/msos/ptlib/vidinput_app.h>
 
 
-#ifdef _MSC_VER
-  #pragma message("Application sharing video support enabled")
-#endif
-
 
 static const PConstCaselessString DesktopWindowName("Desktop");
 static const PConstCaselessString TopWindowPrefix("TopWindow:");
@@ -80,25 +77,18 @@ static bool GetBitmapColourFormat(BITMAP bitmap, PString & colourFormat) {
 //////////////////////////////////////////////////////////////////////
 // Video Input device
 
-class PVideoInputDevice_Application_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
-{
-  public:
-    virtual PObject *    CreateInstance(int /*userData*/) const { return new PVideoInputDevice_Application; }
+PCREATE_VIDINPUT_PLUGIN_EX(Application,
 
-    virtual PStringArray GetDeviceNames(int /*userData*/) const { return PVideoInputDevice_Application::GetInputDeviceNames(); }
+  virtual const char * GetFriendlyName() const
+  {
+    return "Application Window Video Grabber";
+  }
 
-    virtual bool         GetDeviceCapabilities(const PString & deviceName, void * caps) const
-    {
-      return PVideoInputDevice_Application::GetDeviceCapabilities(deviceName, (PVideoInputDevice::Capabilities *)caps);
-    }
-
-    virtual bool ValidateDeviceName(const PString & deviceName, int /*userData*/) const
-    {
-      return FindTopWindow(deviceName) != NULL;
-    }
-} PVideoInputDevice_Application_descriptor;
-
-PCREATE_PLUGIN(Application, PVideoInputDevice, &PVideoInputDevice_Application_descriptor);
+  virtual bool ValidateDeviceName(const PString & deviceName, P_INT_PTR /*userData*/) const
+  {
+    return FindTopWindow(deviceName) != NULL;
+  }
+);
 
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
@@ -137,9 +127,14 @@ PStringArray PVideoInputDevice_Application::GetDeviceNames() const
 
 static BOOL CALLBACK AddWindowName(HWND hWnd, LPARAM userData)
 {
-  WCHAR wideName[200];
-  if (IsWindowVisible(hWnd) && GetWindowTextW(hWnd, wideName, sizeof(wideName)))
-    reinterpret_cast<PStringArray *>(userData)->AppendString(TopWindowPrefix + wideName);
+  if (IsWindowVisible(hWnd)) {
+    int len = GetWindowTextLengthW(hWnd);
+    if (len > 0) {
+      PWideString wideName;
+      if (GetWindowTextW(hWnd, wideName.GetPointer(len+1), len+1))
+        reinterpret_cast<PStringArray *>(userData)->AppendString(TopWindowPrefix + wideName);
+    }
+  }
   return TRUE;
 }
 
@@ -366,7 +361,8 @@ bool PVideoInputDevice_Application::GetWindowBitmap(BITMAP & bitmap, BYTE * pixe
     BitBlt(hMemDC, 0, 0, width, height, hScrDC, rect.left, rect.top, SRCCOPY);
   else {
     // Get a better quality scale
-    SetStretchBltMode(hMemDC, COLORONCOLOR);
+    SetStretchBltMode(hMemDC, HALFTONE);
+    SetBrushOrgEx(hMemDC, 0, 0, NULL);
     StretchBlt(hMemDC, 0, 0, framewidth, frameheight, hScrDC, rect.left, rect.top, width, height, SRCCOPY);
   }
 
@@ -419,15 +415,7 @@ bool PVideoInputDevice_Application::GetWindowBitmap(BITMAP & bitmap, BYTE * pixe
   // Clean up DCs
   DeleteDC(hScrDC);
   DeleteDC(hMemDC);
-  
+
   return true;
 }
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("Application sharing video support DISABLED")
-  #endif
-
 #endif  // P_APPSHARE
