@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29970 $
+ * $Revision: 34622 $
  * $Author: rjongbloed $
- * $Date: 2013-06-16 20:14:51 -0500 (Sun, 16 Jun 2013) $
+ * $Date: 2016-02-14 15:52:45 +0000 (Sun, 14 Feb 2016) $
  */
 
 #include <ptlib.h>
@@ -39,9 +39,10 @@
 
 #include <process.h>
 #include <errors.h>
-#include <Shlobj.h>
+#include <shlobj.h>
+#include <Psapi.h>
 
-#ifdef WIN32
+#ifdef _MSC_VER
   #ifndef _WIN32_WCE
     #pragma comment(lib, "mpr.lib")
     #pragma comment(lib, "Shell32.lib")
@@ -54,8 +55,13 @@
   #if P_DIRECTSHOW
     #pragma comment(lib, "quartz.lib")
   #endif
+
+  #pragma comment(lib, "Psapi.lib")
 #endif
 
+#if P_VERSION_HELPERS
+  #include <versionhelpers.h>
+#endif
 
 #define new PNEW
 
@@ -85,18 +91,24 @@ PTime::PTime(const FILETIME & timestamp)
 }
 
 
+// Magic constant to convert epoch from 1601 to 1970
+static const ULONGLONG Win332FileTimeDelta = ((PInt64)369*365+(369/4)-3)*24*60*60U*1000000;
+
 void PTime::SetFromFileTime(const FILETIME & timestamp)
 {
-  // Magic constant to convert epoch from 1601 to 1970
-  static const ULONGLONG delta = ((PInt64)369*365+(369/4)-3)*24*60*60U;
-  static const ULONGLONG scale = 10000000;
-
   ULARGE_INTEGER i;
   i.HighPart = timestamp.dwHighDateTime;
   i.LowPart = timestamp.dwLowDateTime;
 
-  theTime = (time_t)(i.QuadPart/scale - delta);
-  microseconds = (long)(i.QuadPart%scale/10);
+  m_microSecondsSinceEpoch.store(i.QuadPart/10 - Win332FileTimeDelta);
+}
+
+void PTime::SetToFileTime(FILETIME & timestamp) const
+{
+    ULARGE_INTEGER i;
+    i.QuadPart = (m_microSecondsSinceEpoch.load()+Win332FileTimeDelta)*10;
+    timestamp.dwHighDateTime = i.HighPart;
+    timestamp.dwLowDateTime = i.LowPart;
 }
 
 #ifdef UNICODE
@@ -123,7 +135,7 @@ PString PTime::GetTimeSeparator()
 }
 
 
-PBoolean PTime::GetTimeAMPM()
+bool PTime::GetTimeAMPM()
 {
   char str[2];
   PWIN32GetLocaleInfo(GetUserDefaultLCID(), LOCALE_ITIME, str, sizeof(str));
@@ -184,7 +196,7 @@ PTime::DateOrder PTime::GetDateOrder()
 }
 
 
-PBoolean PTime::IsDaylightSavings()
+bool PTime::IsDaylightSavings()
 {
   TIME_ZONE_INFORMATION tz;
   DWORD result = GetTimeZoneInformation(&tz);
@@ -232,7 +244,7 @@ PTimeInterval PTimer::Tick()
   
   LARGE_INTEGER count;
   QueryPerformanceCounter(&count);
-  return count.QuadPart/divisor;
+  return PTimeInterval::MicroSeconds(count.QuadPart*1000/divisor);
 }
 
 
@@ -267,26 +279,29 @@ void PDirectory::Construct()
 
 void PDirectory::CopyContents(const PDirectory & dir)
 {
-  scanMask  = dir.scanMask;
+  m_scanMask  = dir.m_scanMask;
   hFindFile = INVALID_HANDLE_VALUE;
   fileinfo  = dir.fileinfo;
 }
 
 
-PBoolean PDirectory::Open(int newScanMask)
+bool PDirectory::Open(PFileInfo::FileTypes newScanMask)
 {
-  scanMask = newScanMask;
+  m_scanMask = newScanMask;
   PVarString wildcard = *this + "*.*";
 
   hFindFile = FindFirstFile(wildcard, &fileinfo);
-  if (hFindFile == INVALID_HANDLE_VALUE)
+  if (hFindFile == INVALID_HANDLE_VALUE) {
+    PTRACE_IF(2, GetLastError() != ERROR_PATH_NOT_FOUND && GetLastError() != ERROR_DIRECTORY,
+              "PTLib", "Could not open directory \"" << *this << "\", error=" << GetLastError());
     return false;
+  }
 
-  return Filtered() ? Next() : true;
+  return InternalEntryCheck() || Next();
 }
 
 
-PBoolean PDirectory::Next()
+bool PDirectory::Next()
 {
   if (hFindFile == INVALID_HANDLE_VALUE)
     return false;
@@ -294,7 +309,7 @@ PBoolean PDirectory::Next()
   do {
     if (!FindNextFile(hFindFile, &fileinfo))
       return false;
-  } while (Filtered());
+  } while (!InternalEntryCheck());
 
   return true;
 }
@@ -306,7 +321,7 @@ PCaselessString PDirectory::GetEntryName() const
 }
 
 
-PBoolean PDirectory::IsSubDir() const
+bool PDirectory::IsSubDir() const
 {
   return (fileinfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) != 0;
 }
@@ -376,7 +391,7 @@ typedef PBoolean (WINAPI *GetDiskFreeSpaceExType)(LPCTSTR lpDirectoryName,
                                               PULARGE_INTEGER lpTotalNumberOfFreeBytes);
 
 
-PBoolean PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSize) const
+bool PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSize) const
 {
   clusterSize = 512;
   total = free = ULONG_MAX;
@@ -513,6 +528,34 @@ bool PFilePath::IsAbsolutePath(const PString & path)
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+// PFile
+
+bool PFile::Touch(const PFilePath & name, const PTime & accessTime, const PTime & modTime)
+{
+  PWin32Handle hFile(::CreateFile(name,FILE_WRITE_ATTRIBUTES,0,NULL,0,0,NULL));
+  if (!hFile.IsValid())
+    return false;
+
+  FILETIME now;
+  GetSystemTimeAsFileTime(&now);
+
+  FILETIME acc;
+  if (accessTime.IsValid())
+    accessTime.SetToFileTime(acc);
+  else
+    acc = now;
+
+  FILETIME mod;
+  if (modTime.IsValid())
+    modTime.SetToFileTime(mod);
+  else
+    mod = now;
+
+  return ::SetFileTime(hFile, NULL, &acc, &mod);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PChannel
 
@@ -558,8 +601,8 @@ bool PChannel::AsyncContext::Initialise(PChannel * channel, CompletionFunction o
 
 bool PChannel::ReadAsync(AsyncContext & context)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
   HANDLE handle = GetAsyncReadHandle();
   if (handle == INVALID_HANDLE_VALUE)
@@ -580,8 +623,8 @@ bool PChannel::ReadAsync(AsyncContext & context)
 
 bool PChannel::WriteAsync(AsyncContext & context)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF);
+  if (CheckNotOpen())
+    return false;
 
   HANDLE handle = GetAsyncWriteHandle();
   if (handle == INVALID_HANDLE_VALUE)
@@ -606,7 +649,7 @@ PString PChannel::GetErrorText(Errors lastError, int osError)
       return PString();
 
     static int const errors[NumNormalisedErrors] = {
-      0, ENOENT, EEXIST, ENOSPC, EACCES, EBUSY, EINVAL, ENOMEM, EBADF, EAGAIN, EINTR,
+      0, ENOENT, EEXIST, ENOSPC, EACCES, EBUSY, EINVAL, ENOMEM, EBADF, EAGAIN, ECANCELED,
       WSAEMSGSIZE|PWIN32ErrorFlag, EIO, 0x1000000|PWIN32ErrorFlag
     };
     osError = errors[lastError];
@@ -657,11 +700,13 @@ PString PChannel::GetErrorText(Errors lastError, int osError)
 }
 
 
-PBoolean PChannel::ConvertOSError(P_INT_PTR status, Errors & lastError, int & osError)
+PBoolean PChannel::ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group)
 {
-  switch (status) {
+  int osError = GetErrorNumber(group);
+
+  switch (libcReturnValue) {
     case -1 :
-      if ((osError = errno) != 0)
+      if ((osError = os_errno()) != 0)
         break;
       // Do next case
 
@@ -669,80 +714,17 @@ PBoolean PChannel::ConvertOSError(P_INT_PTR status, Errors & lastError, int & os
       osError = ::GetLastError();
       switch (osError) {
         case ERROR_INVALID_HANDLE :
-        case WSAEBADF :
           osError = EBADF;
           break;
         case ERROR_INVALID_PARAMETER :
-        case WSAEINVAL :
           osError = EINVAL;
           break;
         case ERROR_ACCESS_DENIED :
-        case WSAEACCES :
           osError = EACCES;
           break;
         case ERROR_NOT_ENOUGH_MEMORY :
           osError = ENOMEM;
           break;
-        case WSAEINTR :
-          osError = EINTR;
-          break;
-        case WSAEINPROGRESS :
-          osError = EINPROGRESS;
-          break;
-        case WSAENOTSOCK :
-          osError = ENOTSOCK;
-          break;
-        case WSAEOPNOTSUPP :
-          osError = EOPNOTSUPP;
-          break;
-        case WSAEAFNOSUPPORT :
-          osError = EAFNOSUPPORT;
-          break;
-        case WSAEADDRINUSE :
-          osError = EADDRINUSE;
-          break;
-        case WSAEADDRNOTAVAIL :
-          osError = EADDRNOTAVAIL;
-          break;
-        case WSAENETDOWN :
-          osError = ENETDOWN;
-          break;
-        case WSAENETUNREACH :
-          osError = ENETUNREACH;
-          break;
-        case WSAENETRESET :
-          osError = ENETRESET;
-          break;
-        case WSAECONNABORTED :
-          osError = ECONNABORTED;
-          break;
-        case WSAECONNRESET :
-          osError = ECONNRESET;
-          break;
-        case WSAENOBUFS :
-          osError = ENOBUFS;
-          break;
-        case WSAEISCONN :
-          osError = EISCONN;
-          break;
-        case WSAENOTCONN :
-          osError = ENOTCONN;
-          break;
-        case WSAECONNREFUSED :
-          osError = ECONNREFUSED;
-          break;
-        case WSAEHOSTUNREACH :
-          osError = EHOSTUNREACH;
-          break;
-        case WSAEMSGSIZE :
-          osError = EMSGSIZE;
-          break;
-        case WSAEWOULDBLOCK :
-          osError = EWOULDBLOCK;
-          break;
-        case WSAETIMEDOUT :
-          osError = ETIMEDOUT;
-          break;
         default :
           osError |= PWIN32ErrorFlag;
       }
@@ -752,15 +734,14 @@ PBoolean PChannel::ConvertOSError(P_INT_PTR status, Errors & lastError, int & os
       break;
 
     default :
-      lastError = NoError;
       osError = 0;
-      return true;
   }
 
+  Errors lastError;
   switch (osError) {
     case 0 :
       lastError = NoError;
-      return true;
+      break;
     case ENOENT :
       lastError = NotFound;
       break;
@@ -780,6 +761,7 @@ PBoolean PChannel::ConvertOSError(P_INT_PTR status, Errors & lastError, int & os
       lastError = BadParameter;
       break;
     case EBADF :
+    case ENOTSOCK :
       lastError = NotOpen;
       break;
     case EAGAIN :
@@ -791,13 +773,14 @@ PBoolean PChannel::ConvertOSError(P_INT_PTR status, Errors & lastError, int & os
       lastError = BufferTooSmall;
       break;
     case EINTR :
+    case ECANCELED :
       lastError = Interrupted;
       break;
     default :
       lastError = Miscellaneous;
   }
 
-  return false;
+  return SetErrorValues(lastError, osError, group);
 }
 
 
@@ -852,6 +835,7 @@ UINT __stdcall PThread::MainFunction(void * threadPtr)
   thread->Main();
   process.OnThreadEnded(*thread);
 
+  process.InternalThreadEnded(thread);
   return 0;
 }
 
@@ -912,6 +896,8 @@ PThread::PThread(PINDEX stackSize,
 
 
 #ifdef P_WIN_COM
+static atomic<bool> s_securityInitialised(false);
+
 bool PThread::CoInitialise()
 {
   if (m_comInitialised)
@@ -919,12 +905,11 @@ bool PThread::CoInitialise()
 
   HRESULT result = ::CoInitializeEx(NULL, COINIT_MULTITHREADED);
   if (FAILED(result)) {
-    PTRACE(1, "PTLib", "Could not initialise COM: error=0x" << hex << result);
+    PTRACE_IF(1, result != RPC_E_CHANGED_MODE, "PTLib", "Could not initialise COM: error=0x" << hex << result);
     return false;
   }
 
-  static PAtomicBoolean securityInitialised;
-  if (!securityInitialised.TestAndSet(true)) {
+  if (!s_securityInitialised.exchange(true)) {
     result = ::CoInitializeSecurity(NULL, 
                                     -1,                          // COM authentication
                                     NULL,                        // Authentication services
@@ -952,11 +937,15 @@ std::ostream & operator<<(std::ostream & strm, const PComResult & result)
                                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                                  msg, sizeof(msg),
                                  NULL);
+  while (dwMsgLen > 0 && isspace(msg[dwMsgLen-1]))
+    msg[--dwMsgLen] = '\0';
   if (dwMsgLen > 0)
     return strm << msg;
 
 #if P_DIRECTSHOW
   dwMsgLen = AMGetErrorText(result.m_result, msg, sizeof(msg));
+  while (dwMsgLen > 0 && isspace(msg[dwMsgLen-1]))
+    msg[--dwMsgLen] = '\0';
   if (dwMsgLen > 0)
     return strm << msg;
 #endif
@@ -971,9 +960,17 @@ bool PComResult::Succeeded(HRESULT result, const char * func, const char * file,
   if (Succeeded(result))
     return true;
 
+  // Don't look at "Facility" code when filterring errors.
+  if (nomsg1 >= 0 && LOWORD(result) == LOWORD(nomsg1))
+    return false;
+
+  if (nomsg2 >= 0 && LOWORD(result) == LOWORD(nomsg2))
+    return false;
+
   static const int Level = 2;
-  if (result != nomsg1 && result != nomsg2 && PTrace::CanTrace(Level))
-    PTrace::Begin(Level, file, line) << "Function \"" << func << "\" failed : " << *this << PTrace::End;
+  if (PTrace::CanTrace(Level))
+    PTrace::Begin(Level, file, line) << "Function \"" << func << "\" failed, "
+           "error=0x" << hex << result << dec << " : " << *this << PTrace::End;
   return false;
 }
 #endif // PTRACING
@@ -1001,29 +998,54 @@ std::ostream & operator<<(std::ostream & strm, const PComVariant & var)
 #endif // P_WIN_COM
 
 
-static ULONGLONG GetMillisecondFromFileTime(const FILETIME & ft)
+__inline static ULONGLONG GetMillisecondFromFileTime(const FILETIME & ft)
 {
-  ULARGE_INTEGER i;
-  i.HighPart = ft.dwHighDateTime;
-  i.LowPart = ft.dwLowDateTime;
-  return (i.QuadPart+9999)/10000;
+  return (reinterpret_cast<const ULARGE_INTEGER *>(&ft)->QuadPart+9999)/10000;
 }
 
 
+struct PWindowsTimes
+{
+  FILETIME m_created;
+  FILETIME m_exit;
+  FILETIME m_kernel;
+  FILETIME m_user;
+  FILETIME m_idle;
+
+  PWindowsTimes()
+  {
+    m_exit.dwHighDateTime = m_exit.dwLowDateTime = m_idle.dwHighDateTime = m_idle.dwLowDateTime = 0;
+  }
+
+  bool FromThread(HANDLE handle)  { return GetThreadTimes(handle, &m_created, &m_exit, &m_kernel, &m_user); }
+  bool FromProcess(HANDLE handle) { return GetProcessTimes(handle, &m_created, &m_exit, &m_kernel, &m_user); }
+  bool FromSystem()               { return GetSystemTimes(&m_idle, &m_kernel, &m_user); }
+
+  void ToTimes(PThread::Times & times)
+  {
+    times.m_kernel.SetInterval(GetMillisecondFromFileTime(m_kernel));
+    times.m_user.SetInterval(GetMillisecondFromFileTime(m_user));
+    if (m_idle.dwHighDateTime != 0 || m_idle.dwLowDateTime != 0)
+      times.m_real.SetInterval(GetMillisecondFromFileTime(m_kernel) + GetMillisecondFromFileTime(m_user) + GetMillisecondFromFileTime(m_idle));
+    else {
+      if (m_exit.dwHighDateTime == 0 && m_exit.dwLowDateTime == 0)
+        GetSystemTimeAsFileTime(&m_exit);
+      times.m_real.SetInterval(GetMillisecondFromFileTime(m_exit) - GetMillisecondFromFileTime(m_created));
+    }
+  }
+};
+
 bool PThread::GetTimes(Times & times)
 {
-  FILETIME created, exit, kernel, user;
-  exit.dwHighDateTime = exit.dwLowDateTime = 0;
-  if (!GetThreadTimes(GetHandle(), &created, &exit, &kernel, &user))
+  // Do not use any PTLib functions in here as they could to a PTRACE, and this deadlock
+  times.m_name = GetThreadName();
+  times.m_uniqueId = times.m_threadId = m_threadId;
+
+  PWindowsTimes wt;
+  if (!wt.FromThread(GetHandle()))
     return false;
 
-  times.m_kernel.SetInterval(GetMillisecondFromFileTime(kernel));
-  times.m_user.SetInterval(GetMillisecondFromFileTime(user));
-  if (exit.dwHighDateTime == 0 && exit.dwLowDateTime == 0)
-    times.m_real = PTime() - PTime(created);
-  else
-    times.m_real = PTime(exit) - PTime(created);
-
+  wt.ToTimes(times);
   return true;
 }
 
@@ -1063,8 +1085,8 @@ void PThread::Restart()
 void PThread::Terminate()
 {
   if (PAssert(m_type != e_IsProcess, "Cannot terminate the process!") &&
-      m_threadHandle.IsValid() &&
-      m_threadHandle.Wait(0)) {
+       m_threadHandle.IsValid() &&
+      !m_threadHandle.Wait(0)) {
     PTRACE(2, "PTLib\tTerminating thread " << *this);
     TerminateThread(m_threadHandle, 1);
   }
@@ -1128,7 +1150,7 @@ void PThread::SetPriority(Priority priorityLevel)
 {
   PAssert(!IsTerminated(), "Operation on terminated thread");
 
-  static int const priorities[NumPriorities] = {
+  static int const priorities[NumPriority] = {
     THREAD_PRIORITY_LOWEST,
     THREAD_PRIORITY_BELOW_NORMAL,
     THREAD_PRIORITY_NORMAL,
@@ -1172,75 +1194,6 @@ void PThread::Yield()
 }
 
 
-///////////////////////////////////////////////////////////////////////////////
-// PProcess::TimerThread
-
-void PProcess::HouseKeeping()
-{
-  while (m_keepingHouse) {
-
-    // collect a list of thread handles to check, and clean up 
-    // handles for threads that disappeared without telling us
-    m_threadMutex.Wait();
-    HANDLE handles[MAXIMUM_WAIT_OBJECTS];
-    DWORD numHandles = 1;
-    handles[0] = m_signalHouseKeeper.GetHandle();
-    for (ThreadList::iterator thread = m_autoDeleteThreads.begin(); thread != m_autoDeleteThreads.end(); ++thread) {
-      if (thread->IsTerminated())
-        continue;
-
-      handles[numHandles] = thread->GetHandle();
-
-      // make sure we don't put invalid handles into the list
-#ifndef _WIN32_WCE
-      DWORD dwFlags;
-      if (GetHandleInformation(handles[numHandles], &dwFlags) == 0) {
-        PTRACE(2, "PTLib\tRefused to put invalid handle into wait list");
-      }
-      else
-#endif
-      // don't put the handle for the current process in the list
-      if (handles[numHandles] != GetHandle()) {
-        numHandles++;
-        if (numHandles >= MAXIMUM_WAIT_OBJECTS)
-          break;
-      }
-    }
-    m_threadMutex.Signal();
-
-    PTimeInterval nextTimer = m_timers.Process();
-    DWORD delay;
-    if (nextTimer == PMaxTimeInterval)
-      delay = INFINITE;
-    else if (nextTimer > 1000)
-      delay = 1000;
-    else
-      delay = nextTimer.GetInterval();
-
-    DWORD result;
-    int retries = 100;
-
-    while ((result = WaitForMultipleObjects(numHandles, handles, false, delay)) == WAIT_FAILED) {
-
-      // if we get an invalid handle error, than assume this is because a thread ended between
-      // creating the handle list and testing it. So, cleanup the list before calling 
-      // WaitForMultipleObjects again
-      if (::GetLastError() == ERROR_INVALID_HANDLE)
-        break;
-
-      // sometimes WaitForMultipleObjects fails. No idea why, so allow some retries
-      else {
-        retries--;
-        if (retries <= 0)
-          break;
-      }
-    }
-
-    InternalCleanAutoDeleteThreads();
-  }
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 // PProcess
 
@@ -1248,11 +1201,7 @@ PProcess::~PProcess()
 {
   // do whatever needs to shutdown
   PreShutdown();
-
-#if _DEBUG
   WaitOnExitConsoleWindow();
-#endif
-
   PostShutdown();
 }
 
@@ -1265,6 +1214,28 @@ PString PProcess::GetOSClass()
 
 PString PProcess::GetOSName()
 {
+#if P_VERSION_HELPERS
+  if (IsWindows8Point1OrGreater())
+    return "8.1";
+  if (IsWindows8OrGreater())
+    return "8";
+  if (IsWindows7SP1OrGreater())
+    return "7 sp1";
+  if (IsWindows7OrGreater())
+    return "7";
+  if (IsWindowsVistaSP2OrGreater())
+    return "Vista sp2";
+  if (IsWindowsVistaSP1OrGreater())
+    return "Vista sp1";
+  if (IsWindowsVistaOrGreater())
+    return "Vista";
+  if (IsWindowsServer())
+    return "Server 2003";
+  if (IsWindowsXPSP3OrGreater())
+    return "XP sp3";
+  if (IsWindowsXPOrGreater())
+    return "XP";
+#else
   OSVERSIONINFO info;
   info.dwOSVersionInfoSize = sizeof(info);
   GetVersionEx(&info);
@@ -1303,11 +1274,12 @@ PString PProcess::GetOSName()
               return "Vista";
             case 1 :
               return "7";
-			case 2 :
-              return "8";
+            case 2 :
+              return info.dwBuildNumber < 9200 ? "8" : "8.1";
           }
       }
   }
+#endif // P_VERSION_HELPERS
   return "?";
 }
 
@@ -1349,17 +1321,56 @@ PString PProcess::GetOSHardware()
 
 PString PProcess::GetOSVersion()
 {
+#if P_VERSION_HELPERS
+  if (IsWindows8Point1OrGreater())
+    return "v6.3";
+  if (IsWindows8OrGreater())
+    return "v6.2";
+  if (IsWindows7SP1OrGreater())
+    return "v6.1.1";
+  if (IsWindows7OrGreater())
+    return "v6.1";
+  if (IsWindowsVistaSP2OrGreater())
+    return "v6.0.2";
+  if (IsWindowsVistaSP1OrGreater())
+    return "v6.0.1";
+  if (IsWindowsVistaOrGreater())
+    return "v6.0";
+  if (IsWindowsServer())
+    return "v5.2";
+  if (IsWindowsXPSP3OrGreater())
+    return "v5.1.3";
+  if (IsWindowsXPOrGreater())
+    return "v5.1";
+  return "?";
+#else
   OSVERSIONINFO info;
   info.dwOSVersionInfoSize = sizeof(info);
   GetVersionEx(&info);
   WORD wBuildNumber = (WORD)info.dwBuildNumber;
   return psprintf(wBuildNumber > 0 ? "v%u.%u.%u" : "v%u.%u",
                   info.dwMajorVersion, info.dwMinorVersion, wBuildNumber);
+#endif // P_VERSION_HELPERS
 }
 
 
 bool PProcess::IsOSVersion(unsigned major, unsigned minor, unsigned build)
 {
+#if P_VERSION_HELPERS
+  OSVERSIONINFOEXW osvi = { sizeof(osvi), 0, 0, 0, 0, { 0 }, 0, 0 };
+  DWORDLONG        const dwlConditionMask = VerSetConditionMask(
+    VerSetConditionMask(
+    VerSetConditionMask(
+    0, VER_MAJORVERSION, VER_GREATER_EQUAL),
+    VER_MINORVERSION, VER_GREATER_EQUAL),
+    VER_SERVICEPACKMAJOR, VER_GREATER_EQUAL);
+
+  osvi.dwMajorVersion = major;
+  osvi.dwMinorVersion = minor;
+  osvi.wServicePackMajor = (WORD)build;
+
+  return VerifyVersionInfoW(&osvi, VER_MAJORVERSION | VER_MINORVERSION | VER_SERVICEPACKMAJOR, dwlConditionMask) != FALSE;
+#else
   OSVERSIONINFO info;
   info.dwOSVersionInfoSize = sizeof(info);
   GetVersionEx(&info);
@@ -1375,6 +1386,7 @@ bool PProcess::IsOSVersion(unsigned major, unsigned minor, unsigned build)
     return true;
 
   return info.dwBuildNumber >= build;
+#endif // P_VERSION_HELPERS
 }
 
 
@@ -1383,17 +1395,8 @@ PDirectory PProcess::GetOSConfigDir()
 #ifdef _WIN32_WCE
   return PString("\\Windows");
 #else
-  OSVERSIONINFO info;
-  info.dwOSVersionInfoSize = sizeof(info);
-  GetVersionEx(&info);
-
   char dir[_MAX_PATH];
 
-  if (info.dwPlatformId != VER_PLATFORM_WIN32_NT) {
-    PAssertOS(GetWindowsDirectory(dir, sizeof(dir)) != 0);
-    return dir;
-  }
-
   PAssertOS(GetSystemDirectory(dir, sizeof(dir)) != 0);
   PDirectory sysdir = dir;
   return sysdir;  //+ "drivers\\etc";
@@ -1487,6 +1490,51 @@ PBoolean PProcess::IsServiceProcess() const
 }
 
 
+void PProcess::GetMemoryUsage(MemoryUsage & usage)
+{
+  PROCESS_MEMORY_COUNTERS info;
+  info.cb = sizeof(info);
+  if (GetProcessMemoryInfo(GetCurrentProcess(), &info, sizeof(info))) {
+    usage.m_virtual = info.PeakWorkingSetSize;
+    usage.m_resident = info.WorkingSetSize;
+  }
+}
+
+
+bool PProcess::GetProcessTimes(Times & times) const
+{
+  times.m_name = GetName();
+
+  PWindowsTimes wt;
+  if (!wt.FromProcess(GetCurrentProcess()))
+    return false;
+
+  wt.ToTimes(times);
+  return true;
+}
+
+
+bool PProcess::GetSystemTimes(Times & times)
+{
+  times.m_name = "SYSTEM";
+
+  PWindowsTimes wt;
+  if (!wt.FromSystem())
+    return false;
+
+  wt.ToTimes(times);
+  return true;
+}
+
+
+unsigned PThread::GetNumProcessors()
+{
+  SYSTEM_INFO info;
+  GetSystemInfo(&info);
+  return info.dwNumberOfProcessors;
+}
+
+
 #ifdef _WIN32_WCE
 
 PBoolean PProcess::IsGUIProcess() const
@@ -1497,6 +1545,7 @@ PBoolean PProcess::IsGUIProcess() const
 #else
 
 static bool s_IsGUIProcess = true;
+static atomic<bool> s_checkGUIProcess(false);
 
 static BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM thisProcess)
 {
@@ -1516,8 +1565,7 @@ static BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM thisProcess)
 
 PBoolean PProcess::IsGUIProcess() const
 {
-  static PAtomicBoolean CheckGUIProcess;
-  if (!CheckGUIProcess.TestAndSet(true))
+  if (!s_checkGUIProcess.exchange(true))
     EnumWindows(EnumWindowsProc, GetCurrentProcessId());
   return s_IsGUIProcess;
 }
@@ -1656,21 +1704,16 @@ bool PProcess::HostSystemURLHandlerInfo::Register()
 ///////////////////////////////////////////////////////////////////////////////
 // PSemaphore
 
-PSemaphore::PSemaphore(unsigned initial, unsigned maxCount)
-  : m_maxCountVal(maxCount)
-  , m_initialVal(std::min(initial, maxCount))
-  , m_handle(CreateSemaphore(NULL, m_initialVal, m_maxCountVal, NULL))
+void PSemaphore::Reset(unsigned initial, unsigned maximum)
 {
+  m_maximum = std::min(maximum, (unsigned)INT_MAX);
+  m_initial = std::min(initial, m_maximum);
+
+  m_handle.Close();
+  m_handle = CreateSemaphore(NULL, m_initial, m_maximum, m_name.IsEmpty() ? NULL : m_name.GetPointer());
   PAssertOS(m_handle.IsValid());
 }
 
-PSemaphore::PSemaphore(const PSemaphore & sem)
-  : m_maxCountVal(sem.m_maxCountVal)
-  , m_initialVal(sem.m_initialVal)
-  , m_handle(CreateSemaphore(NULL, m_initialVal, m_maxCountVal, NULL))
-{
-  PAssertOS(m_handle.IsValid());
-}
 
 PSemaphore::~PSemaphore()
 {
@@ -1691,24 +1734,35 @@ PBoolean PSemaphore::Wait(const PTimeInterval & timeout)
 
 void PSemaphore::Signal()
 {
-  if (!ReleaseSemaphore(m_handle, 1, NULL))
-    PAssertOS(::GetLastError() != ERROR_INVALID_HANDLE);
-  SetLastError(ERROR_SUCCESS);
+  if (PAssertOS(ReleaseSemaphore(m_handle, 1, NULL) || ::GetLastError() == ERROR_INVALID_HANDLE))
+    SetLastError(ERROR_SUCCESS);
 }
 
 
 ///////////////////////////////////////////////////////////////////////////////
 // PTimedMutex
 
-PTimedMutex::PTimedMutex()
+PTimedMutex::PTimedMutex(const char * name, unsigned line)
   : m_lockerId(PNullThreadIdentifier)
+  , m_lastLockerId(PNullThreadIdentifier)
+  , m_lastUniqueId(0)
+  , m_lockCount(0)
+  , m_excessiveLockTime(false)
+  , m_fileOrName(name)
+  , m_line(line)
   , m_handle(::CreateMutex(NULL, FALSE, NULL))
 {
 }
 
 
-PTimedMutex::PTimedMutex(const PTimedMutex &)
+PTimedMutex::PTimedMutex(const PTimedMutex & other)
   : m_lockerId(PNullThreadIdentifier)
+  , m_lastLockerId(PNullThreadIdentifier)
+  , m_lastUniqueId(0)
+  , m_lockCount(0)
+  , m_excessiveLockTime(false)
+  , m_fileOrName(other.m_fileOrName)
+  , m_line(other.m_line)
   , m_handle(::CreateMutex(NULL, FALSE, NULL))
 {
 }
@@ -1716,19 +1770,14 @@ PTimedMutex::PTimedMutex(const PTimedMutex &)
 
 void PTimedMutex::Wait()
 {
-#if PTRACING
-  if (!m_handle.Wait(15000)) {
-    PTRACE(1, "PTLib", "Possible deadlock in mutex " << this << ", owner id="
-           << m_lockerId << " (0x" << std::hex << m_lockerId << std::dec << ')');
+  if (!m_handle.Wait(ExcessiveLockWaitTime)) {
+    ExcessiveLockWait();
     m_handle.Wait(INFINITE);
-    PTRACE(1, "PTLib", "Phantom deadlock in mutex " << this);
+    PTRACE(0, "PTLib", "Phantom deadlock in mutex " << *this);
   }
-#else
-  m_handle.Wait(INFINITE);
-#endif
 
   if (m_lockCount++ == 0)
-    m_lockerId = ::GetCurrentThreadId();
+    m_lastUniqueId = m_lastLockerId = m_lockerId = ::GetCurrentThreadId();
 }
 
 
@@ -1738,7 +1787,7 @@ PBoolean PTimedMutex::Wait(const PTimeInterval & timeout)
     return false;
 
   if (m_lockCount++ == 0)
-    m_lockerId = ::GetCurrentThreadId();
+    m_lastUniqueId = m_lastLockerId = m_lockerId = ::GetCurrentThreadId();
   return true;
 }
 
@@ -1746,26 +1795,12 @@ PBoolean PTimedMutex::Wait(const PTimeInterval & timeout)
 void PTimedMutex::Signal()
 {
   if (--m_lockCount == 0)
-    m_lockerId = PNullThreadIdentifier;
+    CommonSignal();
 
   PAssertOS(::ReleaseMutex(m_handle));
 }
 
 
-///////////////////////////////////////////////////////////////////////////////
-
-PBoolean PCriticalSection::Wait(const PTimeInterval & timeout)
-{
-  PSimpleTimer timer(timeout);
-  do {
-    if (Try())
-      return true;
-    PThread::Sleep(100);
-  } while (timer.IsRunning());
-  return false;
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
 // PSyncPoint
 
@@ -1893,18 +1928,21 @@ PString PDynaLink::GetExtension()
 }
 
 
-PBoolean PDynaLink::Open(const PString & name)
+PBoolean PDynaLink::Open(const PString & names)
 {
   m_lastError.MakeEmpty();
 
-  PVarString filename = name;
+  PStringArray filenames = names.Lines();
+  for (PINDEX i = 0; i < filenames.GetSize(); ++i) {
+    PVarString filename = filenames[i];
 #ifndef _WIN32_WCE
-  m_hDLL = LoadLibraryEx(filename, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
+    m_hDLL = LoadLibraryEx(filename, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
 #else
-  m_hDLL = LoadLibrary(filename);
+    m_hDLL = LoadLibrary(filename);
 #endif
-  if (m_hDLL != NULL)
-    return true;
+    if (m_hDLL != NULL)
+      return true;
+  }
 
   m_lastError.sprintf("0x%x", ::GetLastError());
   PTRACE(1, "DLL\tError loading DLL: " << m_lastError);
@@ -1945,9 +1983,10 @@ PString PDynaLink::GetName(PBoolean full) const
 }
 
 
-PBoolean PDynaLink::GetFunction(PINDEX index, Function & func)
+PBoolean PDynaLink::GetFunction(PINDEX index, Function & func, bool compulsory)
 {
   m_lastError.MakeEmpty();
+  func = NULL;
 
   if (m_hDLL == NULL)
     return false;
@@ -1957,14 +1996,18 @@ PBoolean PDynaLink::GetFunction(PINDEX index, Function & func)
   if (func != NULL)
     return true;
 
+  if (compulsory)
+    Close();
+
   m_lastError.sprintf("0x%x", ::GetLastError());
   return false;
 }
 
 
-PBoolean PDynaLink::GetFunction(const PString & name, Function & func)
+PBoolean PDynaLink::GetFunction(const PString & name, Function & func, bool compulsory)
 {
   m_lastError.MakeEmpty();
+  func = NULL;
 
   if (m_hDLL == NULL)
     return false;
@@ -1974,6 +2017,9 @@ PBoolean PDynaLink::GetFunction(const PString & name, Function & func)
   if (func != NULL)
     return true;
 
+  if (compulsory)
+    Close();
+
   m_lastError.sprintf("0x%x", ::GetLastError());
   return false;
 }
