@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28931 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 06:31:11 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 34166 $
+ * $Author: rjongbloed $
+ * $Date: 2015-10-08 11:13:12 +0100 (Thu, 08 Oct 2015) $
  */
 
 #include <ptlib.h>
@@ -40,9 +40,10 @@
 #define new PNEW
 
 
-const char LocalMachineStr[] = "HKEY_LOCAL_MACHINE\\";
-const char CurrentUserStr[]  = "HKEY_CURRENT_USER\\";
-const char ClassesRootStr[]  = "HKEY_CLASSES_ROOT\\";
+static PConstString const LocalMachineStr("HKEY_LOCAL_MACHINE\\");
+static PConstString const LocalMach64Str ("HKEY_LOCAL_MACH64\\");
+static PConstString const CurrentUserStr ("HKEY_CURRENT_USER\\");
+static PConstString const ClassesRootStr ("HKEY_CLASSES_ROOT\\");
 
 ///////////////////////////////////////////////////////////////////////////////
 // Configuration files
@@ -207,16 +208,21 @@ RegistryKey::RegistryKey(const PString & subkeyname, OpenMode mode)
 
   PVarString subkey;
   HKEY basekey;
-  if (subkeyname.Find(LocalMachineStr) == 0) {
-    subkey = subkeyname.Mid(19);
+  if (subkeyname.NumCompare(LocalMachineStr) == PObject::EqualTo) {
+    subkey = subkeyname.Mid(LocalMachineStr.GetLength());
     basekey = HKEY_LOCAL_MACHINE;
   }
-  else if (subkeyname.Find(CurrentUserStr) == 0) {
-    subkey = subkeyname.Mid(18);
+  else if (subkeyname.NumCompare(LocalMach64Str) == PObject::EqualTo) {
+    subkey = subkeyname.Mid(LocalMach64Str.GetLength());
+    basekey = HKEY_LOCAL_MACHINE;
+    access |= KEY_WOW64_64KEY;
+  }
+  else if (subkeyname.NumCompare(CurrentUserStr) == PObject::EqualTo) {
+    subkey = subkeyname.Mid(CurrentUserStr.GetLength());
     basekey = HKEY_CURRENT_USER;
   }
-  else if (subkeyname.Find(ClassesRootStr) == 0) {
-    subkey = subkeyname.Mid(18);
+  else if (subkeyname.NumCompare(ClassesRootStr) == PObject::EqualTo) {
+    subkey = subkeyname.Mid(ClassesRootStr.GetLength());
     basekey = HKEY_CLASSES_ROOT;
   }
   else {
@@ -271,7 +277,8 @@ RegistryKey::RegistryKey(const PString & subkeyname, OpenMode mode)
   }
 
 #ifndef _WIN32_WCE
-  error = SecureCreateKey(basekey, subkey, key);
+  if (basekey != HKEY_CURRENT_USER)
+    error = SecureCreateKey(basekey, subkey, key);
   if (error != ERROR_SUCCESS)
 #endif
   {
@@ -351,7 +358,8 @@ BOOL RegistryKey::QueryValue(const PString & valueName, PString & str)
   PVarString varValueName = valueName;
 
   DWORD type, size, num;
-  if (RegQueryValueEx(key, varValueName, NULL, &type, NULL, &size) != ERROR_SUCCESS)
+  DWORD error = RegQueryValueEx(key, varValueName, NULL, &type, NULL, &size);
+  if (error != ERROR_SUCCESS)
     return false;
 
   if (type == REG_DWORD) {
@@ -449,10 +457,40 @@ BOOL RegistryKey::SetValue(const PString & value, DWORD num)
 
 #if P_CONFIG_FILE
 
-static PBoolean IsRegistryPath(const PString & path)
+static PBoolean IsRegistryPath(PString & path)
 {
-  return (path.Find(LocalMachineStr) == 0 && path != LocalMachineStr) ||
-         (path.Find(CurrentUserStr) == 0 && path != CurrentUserStr);
+  if (path.NumCompare(LocalMachineStr) != PObject::EqualTo &&
+      path.NumCompare(LocalMach64Str ) != PObject::EqualTo &&
+      path.NumCompare(CurrentUserStr ) != PObject::EqualTo)
+    return false;
+
+  if (path[path.GetLength() - 1] != '\\')
+    path += '\\';
+
+  return true;
+}
+
+
+static PString GetDefaultRegistryPath(const PString & manuf, const PString & appname)
+{
+  PStringStream regPath;
+
+  regPath << CurrentUserStr << "SOFTWARE\\";
+
+  if (!manuf)
+    regPath << manuf;
+
+  if (!manuf && !appname)
+    regPath << '\\';
+
+  if (!appname)
+    regPath << appname;
+  else if (manuf.IsEmpty())
+    regPath << "PTLib";
+
+  regPath << "\\CurrentVersion\\";
+
+  return regPath;
 }
 
 
@@ -463,7 +501,7 @@ PString PProcess::GetConfigurationFile()
   // No paths set, use defaults
   if (configurationPaths.IsEmpty()) {
     configurationPaths.AppendString(executableFile.GetVolume() + executableFile.GetPath());
-    configurationPaths.AppendString(CurrentUserStr);
+    configurationPaths.AppendString(GetDefaultRegistryPath(GetManufacturer(), GetName()));
   }
 
   // See if explicit filename
@@ -493,9 +531,8 @@ void PConfig::Construct(Source src, const PString & appname, const PString & man
 
   switch (src) {
     case System :
-      if (IsRegistryPath(appname))
-        location = appname;
-      else {
+      location = appname;
+      if (!IsRegistryPath(location)) {
         TCHAR dir[_MAX_PATH];
         GetWindowsDirectory(dir, sizeof(dir));
         Construct(PDirectory(PString(dir))+"WIN.INI");
@@ -507,27 +544,17 @@ void PConfig::Construct(Source src, const PString & appname, const PString & man
         location = PString();
       else {
         PProcess & proc = PProcess::Current();
-        PString cfgPath = proc.GetConfigurationFile();
-        if (IsRegistryPath(cfgPath))
-          location = cfgPath;
-        else if (!cfgPath) {
-          source = NumSources; // Make a file based config
-          location = cfgPath;
-        }
+        if (!manuf || !appname)
+          location = GetDefaultRegistryPath(manuf, appname);
         else {
-          location = "SOFTWARE\\";
-          if (!manuf)
-            location += manuf;
-          else if (!proc.GetManufacturer())
-            location += proc.GetManufacturer();
-          else
-            location += "PWLib";
-          location += PDIR_SEPARATOR;
-          if (appname.IsEmpty())
-            location += proc.GetName();
-          else
-            location += appname;
-          location += "\\CurrentVersion\\";
+          PString cfgPath = proc.GetConfigurationFile();
+          if (cfgPath.IsEmpty())
+            location = GetDefaultRegistryPath(manuf, appname);
+          else {
+            if (!IsRegistryPath(cfgPath))
+              source = NumSources; // Make a file based config
+            location = cfgPath;
+          }
         }
       }
       break;
