@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29945 $
+ * $Revision: 34311 $
  * $Author: rjongbloed $
- * $Date: 2013-06-13 04:43:07 -0500 (Thu, 13 Jun 2013) $
+ * $Date: 2015-11-17 15:57:53 +0000 (Tue, 17 Nov 2015) $
  */
 
 #include <ptlib.h>
@@ -37,25 +37,6 @@
 #include <svcguid.h>
 #include <iphlpapi.h>
 
-#if P_HAS_IPV6
-  #pragma message("IPv6 support enabled")
-#else
-  #pragma message("IPv6 support DISABLED")
-#endif
-
-#if P_GQOS
-  #pragma message("GQOS support enabled")
-#else
-  #pragma message("GQOS support DISABLED")
-#endif
-
-#if P_QWAVE
-  #pragma message("qWAVE support enabled")
-#else
-  #pragma message("qWAVE support DISABLED")
-#endif
-
-
 #ifndef _WIN32_WCE
   #include <nspapi.h>
   #include <wsipx.h>
@@ -85,6 +66,10 @@
 
 #endif // !_WIN32_WCE
 
+#if P_GQOS
+  #include <qossp.h>
+#endif
+
 #if P_QWAVE
   #include <qos2.h>
 #endif
@@ -97,6 +82,9 @@
 #endif
 
 
+#define PTraceModule() "WinSock"
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PWinSock
 
@@ -119,21 +107,21 @@ struct PWinSock : public PProcessStartup
 
 #if P_GQOS
     // Use for NT (5.1) and Win2003 (5.2) but not Win2000 (5.0) or Vista and later (6.x)
-    OSVERSIONINFO info;
-    info.dwOSVersionInfoSize = sizeof(info);
-    GetVersionEx(&info);
-    m_useGQOS = info.dwMajorVersion == 5 && info.dwMinorVersion >= 1;
+    m_useGQOS = PProcess::IsOSVersion(5,1,0) && !PProcess::IsOSVersion(6,0,0);
 #endif // P_GQOS
 
 #if P_QWAVE
-    // Not always installed so test if there
-    HMODULE hTest = LoadLibrary("QWAVE.DLL");
-    if (hTest != NULL) {
-      QOS_VERSION qosVersion;
-      qosVersion.MajorVersion = 1;
-      qosVersion.MinorVersion = 0;
-      PAssert(QOSCreateHandle(&qosVersion, &m_hQoS), POperatingSystemError);
-      FreeLibrary(hTest);
+    // Seems to crash on Server 2003
+    if (PProcess::IsOSVersion(6, 0, 0)) {
+      // Not always installed so test if there
+      HMODULE hTest = LoadLibrary("QWAVE.DLL");
+      if (hTest != NULL) {
+        QOS_VERSION qosVersion;
+        qosVersion.MajorVersion = 1;
+        qosVersion.MinorVersion = 0;
+        QOSCreateHandle(&qosVersion, &m_hQoS);
+        FreeLibrary(hTest);
+      }
     }
 #endif // P_QWAVE
   }
@@ -142,7 +130,7 @@ struct PWinSock : public PProcessStartup
   {
 #if P_QWAVE
     if (m_hQoS != NULL)
-      PAssert(QOSCloseHandle(m_hQoS), POperatingSystemError);
+      QOSCloseHandle(m_hQoS);
 #endif // P_QWAVE
 
     WSACleanup();
@@ -155,9 +143,15 @@ PFACTORY_CREATE_SINGLETON(PProcessStartupFactory, PWinSock);
 //////////////////////////////////////////////////////////////////////////////
 // P_fd_set
 
+P_fd_set::P_fd_set()
+{
+  Construct();
+  Zero();
+}
+
 void P_fd_set::Construct()
 {
-  max_fd = UINT_MAX;
+  max_fd = INT_MAX;
   set = (fd_set *)malloc(sizeof(fd_set));
 }
 
@@ -168,6 +162,52 @@ void P_fd_set::Zero()
     FD_ZERO(set);
 }
 
+PBoolean P_fd_set::IsPresent(intptr_t fd) const
+{
+  return FD_ISSET(fd, set);
+}
+
+
+#ifdef _MSC_VER
+  #pragma warning(disable:4127)
+#endif
+
+P_fd_set::P_fd_set(intptr_t fd)
+{
+  Construct();
+  Zero();
+  FD_SET((SOCKET)fd, set);
+}
+
+
+P_fd_set & P_fd_set::operator=(intptr_t fd)
+{
+  PAssert(fd < max_fd, PInvalidParameter);
+  Zero();
+  FD_SET((SOCKET)fd, set);
+  return *this;
+}
+
+
+P_fd_set & P_fd_set::operator+=(intptr_t fd)
+{
+  PAssert(fd < max_fd, PInvalidParameter);
+  FD_SET((SOCKET)fd, set);
+  return *this;
+}
+
+
+P_fd_set & P_fd_set::operator-=(intptr_t fd)
+{
+  PAssert(fd < max_fd, PInvalidParameter);
+  FD_CLR((SOCKET)fd, set);
+  return *this;
+}
+
+#ifdef _MSC_VER
+  #pragma warning(default:4127)
+#endif
+
 
 //////////////////////////////////////////////////////////////////////////////
 // PSocket
@@ -192,9 +232,11 @@ HANDLE PSocket::GetAsyncWriteHandle() const
 
 PBoolean PSocket::Read(void * buf, PINDEX len)
 {
-  flush();
   lastReadCount = 0;
 
+  if (CheckNotOpen())
+    return false;
+
   if (len == 0)
     return SetErrorValues(BadParameter, EINVAL, LastReadError);
 
@@ -206,9 +248,11 @@ PBoolean PSocket::Read(void * buf, PINDEX len)
 
 PBoolean PSocket::Read(Slice * slices, size_t sliceCount)
 {
-  flush();
   lastReadCount = 0;
 
+  if (CheckNotOpen())
+    return false;
+
   os_vread(slices, sliceCount, 0, NULL, NULL);
   return lastReadCount > 0;
 }
@@ -216,6 +260,11 @@ PBoolean PSocket::Read(Slice * slices, size_t sliceCount)
 
 PBoolean PSocket::Write(const void * buf, PINDEX len)
 {
+  lastWriteCount = 0;
+
+  if (CheckNotOpen())
+    return false;
+
   flush();
   Slice slice(buf, len);
   return os_vwrite(&slice, 1, 0, NULL, 0) && lastWriteCount >= len;
@@ -224,6 +273,11 @@ PBoolean PSocket::Write(const void * buf, PINDEX len)
 
 PBoolean PSocket::Write(const Slice * slices, size_t sliceCount)
 {
+  lastWriteCount = 0;
+
+  if (CheckNotOpen())
+    return false;
+
   flush();
   return os_vwrite(slices, sliceCount, 0, NULL, 0) && lastWriteCount >= 0;
 }
@@ -231,8 +285,9 @@ PBoolean PSocket::Write(const Slice * slices, size_t sliceCount)
 
 PBoolean PSocket::Close()
 {
-  if (!IsOpen())
+  if (CheckNotOpen())
     return false;
+
   flush();
   return ConvertOSError(os_close());
 }
@@ -349,7 +404,7 @@ PBoolean PSocket::os_connect(struct sockaddr * addr, socklen_t size)
 
   fionbio = 0;
   if (::ioctlsocket(os_handle, FIONBIO, &fionbio) < 0) {
-    PTRACE(1, "Socket\tCOuld nto reset FIONBIO mode, error=" << GetLastError());
+    PTRACE(1, "Could not reset FIONBIO mode, error=" << GetLastError());
   }
 
   SetLastError(err);
@@ -418,7 +473,7 @@ bool PSocket::os_vwrite(const Slice * slices,
 {
   lastWriteCount = 0;
 
-  if (!IsOpen())
+  if (CheckNotOpen())
     return false;
 
   P_fd_set writefds = os_handle;
@@ -453,12 +508,16 @@ PChannel::Errors PSocket::Select(SelectList & read,
                                  SelectList & except,
                                  const PTimeInterval & timeout)
 {
+  PSocket * firstSocket = NULL;
+
   SelectList::iterator sock;
   P_fd_set readfds;
   for (sock = read.begin(); sock != read.end(); ++sock) {
     if (!sock->IsOpen())
       return NotOpen;
     readfds += sock->GetHandle();
+    if (firstSocket == NULL)
+      firstSocket = &*sock;
   }
 
   P_fd_set writefds;
@@ -466,6 +525,8 @@ PChannel::Errors PSocket::Select(SelectList & read,
     if (!sock->IsOpen())
       return NotOpen;
     writefds += sock->GetHandle();
+    if (firstSocket == NULL)
+      firstSocket = &*sock;
   }
 
   P_fd_set exceptfds;
@@ -473,15 +534,18 @@ PChannel::Errors PSocket::Select(SelectList & read,
     if (!sock->IsOpen())
       return NotOpen;
     exceptfds += sock->GetHandle();
+    if (firstSocket == NULL)
+      firstSocket = &*sock;
   }
 
+  if (firstSocket == NULL)
+    return BadParameter;
+
   P_timeval tval = timeout;
   int retval = select(INT_MAX, readfds, writefds, exceptfds, tval);
 
-  Errors lastError;
-  int osError;
-  if (!ConvertOSError(retval, lastError, osError))
-    return lastError;
+  if (!firstSocket->ConvertOSError(retval))
+    return firstSocket->GetErrorCode();
 
   if (retval > 0) {
     sock = read.begin();
@@ -525,20 +589,61 @@ PChannel::Errors PSocket::Select(SelectList & read,
 }
 
 
-PBoolean PSocket::ConvertOSError(P_INT_PTR status, ErrorGroup group)
+int PSocket::os_errno() const
 {
-  Errors lastError;
-  int osError;
-  PBoolean ok = ConvertOSError(status, lastError, osError);
-  SetErrorValues(lastError, osError, group);
-  return ok;
-}
+  DWORD err = WSAGetLastError();
+  SetLastError(err);
 
+  switch (err) {
+    case WSAEBADF:
+      return EBADF;
+    case WSAEINVAL:
+      return EINVAL;
+    case WSAEACCES:
+      return EACCES;
+    case WSAEINTR:
+      return EINTR;
+    case WSAEINPROGRESS:
+      return EINPROGRESS;
+    case WSAENOTSOCK:
+      return ENOTSOCK;
+    case WSAEOPNOTSUPP:
+      return EOPNOTSUPP;
+    case WSAEAFNOSUPPORT:
+      return EAFNOSUPPORT;
+    case WSAEADDRINUSE:
+      return EADDRINUSE;
+    case WSAEADDRNOTAVAIL:
+      return EADDRNOTAVAIL;
+    case WSAENETDOWN:
+      return ENETDOWN;
+    case WSAENETUNREACH:
+      return ENETUNREACH;
+    case WSAENETRESET:
+      return ENETRESET;
+    case WSAECONNABORTED:
+      return ECONNABORTED;
+    case WSAECONNRESET:
+      return ECONNRESET;
+    case WSAENOBUFS:
+      return ENOBUFS;
+    case WSAEISCONN:
+      return EISCONN;
+    case WSAENOTCONN:
+      return ENOTCONN;
+    case WSAECONNREFUSED:
+      return ECONNREFUSED;
+    case WSAEHOSTUNREACH:
+      return EHOSTUNREACH;
+    case WSAEMSGSIZE:
+      return EMSGSIZE;
+    case WSAEWOULDBLOCK:
+      return EWOULDBLOCK;
+    case WSAETIMEDOUT:
+      return ETIMEDOUT;
+  }
 
-PBoolean PSocket::ConvertOSError(P_INT_PTR status, Errors & lastError, int & osError)
-{
-  SetLastError(WSAGetLastError());
-  return PChannel::ConvertOSError(status, lastError, osError);
+  return err|PWIN32ErrorFlag;
 }
 
 
@@ -599,65 +704,6 @@ BYTE PIPSocket::Address::Byte4() const
 }
 
 
-//////////////////////////////////////////////////////////////////////////////
-// PIPSocket
-
-PBoolean P_IsOldWin95()
-{
-  static int state = -1;
-  if (state < 0) {
-    state = 1;
-    OSVERSIONINFO info;
-    info.dwOSVersionInfoSize = sizeof(info);
-    if (GetVersionEx(&info)) {
-      state = 0;
-      if (info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS && info.dwBuildNumber < 1000)
-        state = 1;
-    }
-  }
-  return state != 0;
-}
-
-
-PBoolean PIPSocket::IsLocalHost(const PString & hostname)
-{
-  if (hostname.IsEmpty())
-    return true;
-
-  if (hostname *= "localhost")
-    return true;
-
-  // lookup the host address using inet_addr, assuming it is a "." address
-  Address addr(hostname);
-  if (addr.IsLoopback())  // Is 127.0.0.1 or ::1
-    return true;
-
-  if (addr == 0) {
-    if (!GetHostAddress(hostname, addr))
-      return false;
-  }
-
-  // Seb: Should check that it's really IPv4 aware.
-  struct hostent * host_info = ::gethostbyname(GetHostName());
-
-  if (P_IsOldWin95())
-    return addr == *(struct in_addr *)host_info->h_addr_list[0];
-
-  for (PINDEX i = 0; host_info->h_addr_list[i] != NULL; i++) {
-#if P_HAS_IPV6
-    if (host_info->h_length == 16) {
-      if (addr == *(struct in6_addr *)host_info->h_addr_list[i])
-        return true;
-    }
-    else
-#endif
-    if (addr == *(struct in_addr *)host_info->h_addr_list[i])
-      return true;
-  }
-  return false;
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 
 class PIPRouteTable
@@ -680,6 +726,24 @@ public:
   const MIB_IPFORWARDTABLE & operator *() const { return *Ptr(); }
   operator const MIB_IPFORWARDTABLE *  () const { return  Ptr(); }
 
+  bool ValidateAddress(DWORD ifIndex, LPSOCKADDR lpSockAddr)
+  {
+    int numEntries = Ptr()->dwNumEntries;
+    if (numEntries == 0)
+      return true;
+
+    if (lpSockAddr == NULL)
+      return false;
+
+    const MIB_IPFORWARDROW * row = Ptr()->table;
+    for (int i = 0; i < numEntries; i++, row++) {
+      if (row->dwForwardIfIndex == ifIndex && (u_long)row->dwForwardDest == ((sockaddr_in *)lpSockAddr)->sin_addr.S_un.S_addr)
+              return true;
+    }
+
+    return false;
+  }
+
   private:
     PBYTEArray m_buffer;
 };
@@ -710,7 +774,7 @@ public:
 };
 
 
-#if P_HAS_IPV6
+#if (P_HAS_IPV6 || _WIN32_WINNT_WINXP)
 class PIPAdaptersAddressTable
 {
 public:
@@ -739,8 +803,9 @@ public:
 private:
   PBYTEArray buffer;
 };
+#endif
 
-
+#if P_HAS_IPV6
 typedef NETIO_STATUS (NETIOAPI_API_ *GETIPFORWARDTABLE2)(ADDRESS_FAMILY, PMIB_IPFORWARD_TABLE2 *);
 typedef VOID (NETIOAPI_API_ *FREEMIBTABLE)(PVOID);
 
@@ -970,9 +1035,6 @@ bool PIPSocket::SetQoS(const QoS & qos)
 {
   m_qos = qos;
 
-  if (!IsOpen())
-    return false;
-
   int new_tos = qos.m_dscp >= 0 || qos.m_dscp < 64 ? (qos.m_dscp<<2) : -1;
 
 #if P_GQOS
@@ -1013,70 +1075,71 @@ bool PIPSocket::SetQoS(const QoS & qos)
         return true;
     }
     else {
-      PTRACE(2, "Socket\tCould not SIO_SET_QOS: " << GetErrorText());
+      PTRACE(2, "Could not SIO_SET_QOS: " << GetErrorText());
     }
   }
 #endif // P_GQOS
 
 #if P_QWAVE
-    PIPSocketAddressAndPort peer = qos.m_remote;
-    if (!peer.IsValid())
-      GetPeerAddress(peer);
+  if (PWinSock::GetInstance().m_hQoS != NULL) {
+    static QOS_TRAFFIC_TYPE const TrafficType[NumQoSType] = {
+      QOSTrafficTypeBackground,      // BackgroundQoS
+      QOSTrafficTypeBestEffort,      // BestEffortQoS
+      QOSTrafficTypeExcellentEffort, // ExcellentEffortQoS
+      QOSTrafficTypeControl,         // CriticalQoS
+      QOSTrafficTypeAudioVideo,      // VideoQoS
+      QOSTrafficTypeVoice,           // VoiceQoS
+      QOSTrafficTypeControl          // ControlQoS
+    };
 
-    if (PWinSock::GetInstance().m_hQoS != NULL && peer.IsValid()) {
-      static QOS_TRAFFIC_TYPE const TrafficType[NumQoSType] = {
-        QOSTrafficTypeBackground,      // BackgroundQoS
-        QOSTrafficTypeBestEffort,      // BestEffortQoS
-        QOSTrafficTypeExcellentEffort, // ExcellentEffortQoS
-        QOSTrafficTypeControl,         // CriticalQoS
-        QOSTrafficTypeAudioVideo,      // VideoQoS
-        QOSTrafficTypeVoice,           // VoiceQoS
-        QOSTrafficTypeControl          // ControlQoS
-      };
-
-      bool ok = false;
-      if (m_qosFlowId == 0) {
-        if (qos.m_type != BestEffortQoS || new_tos >= 0 || qos.m_transmit.m_maxBandwidth > 0) {
-          ok = QOSAddSocketToFlow(PWinSock::GetInstance().m_hQoS, os_handle,
-                                  peer.IsValid() ? (PSOCKADDR)sockaddr_wrapper(peer) : (PSOCKADDR)NULL,
-                                  TrafficType[qos.m_type], QOS_NON_ADAPTIVE_FLOW, &m_qosFlowId);
-          PTRACE_IF(1, !ok, "WinSock", "Could not add socket to QoS flow, error=" << ::GetLastError());
-        }
-      }
-      else {
-        ok = QOSSetFlow(PWinSock::GetInstance().m_hQoS, m_qosFlowId, QOSSetTrafficType, sizeof(QOS_TRAFFIC_TYPE), (PVOID)&TrafficType[qos.m_type], 0, NULL);
-        PTRACE_IF(1, !ok, "WinSock", "Could not set QoS flow, error=" << ::GetLastError());
+    if (m_qosFlowId == 0) {
+      PIPSocketAddressAndPort peer = qos.m_remote;
+      if (!peer.IsValid() && !GetPeerAddress(peer)) {
+        PTRACE(3, "Cannot set QoS, no peer address yet.");
+        return false;
       }
 
-      if (ok && qos.m_transmit.m_maxBandwidth > 0) {
-        QOS_FLOWRATE_OUTGOING out;
-        out.Bandwidth = qos.m_transmit.m_maxBandwidth;
-        out.ShapingBehavior = QOSUseNonConformantMarkings;
-        out.Reason = QOSFlowRateNotApplicable;
-        if (!QOSSetFlow(PWinSock::GetInstance().m_hQoS, m_qosFlowId, QOSSetOutgoingRate, sizeof(out), &out, 0, NULL)) {
-          PTRACE(1, "WinSock", "Could not set QoS rates, error=" << ::GetLastError());
-        }
+      if (!QOSAddSocketToFlow(PWinSock::GetInstance().m_hQoS, os_handle,
+                              sockaddr_wrapper(peer), TrafficType[qos.m_type], QOS_NON_ADAPTIVE_FLOW, &m_qosFlowId)) {
+        PTRACE(1, "Could not add socket to QoS flow, error=" << ::GetLastError());
+        return false;
       }
-
-      if (ok && new_tos >= 0) {
-#if P_QWAVE_DSCP
-        DWORD dscp = qos.m_dscp;
-        if (!QOSSetFlow(PWinSock::GetInstance().m_hQoS, m_qosFlowId, QOSSetOutgoingDSCPValue, sizeof(dscp), &dscp, 0, NULL)) {
-          PTRACE(1, "WinSock", "Could not set DSCP, error=" << ::GetLastError());
-        }
-#else
-        ok = false;
-#endif // P_QWAVE_DSCP
-      }
-
-      if (ok)
-        return true;
     }
+    else {
+      if (!QOSSetFlow(PWinSock::GetInstance().m_hQoS, m_qosFlowId, QOSSetTrafficType,
+                      sizeof(QOS_TRAFFIC_TYPE), (PVOID)&TrafficType[qos.m_type], 0, NULL)) {
+        PTRACE(1, "Could not set QoS flow, error=" << ::GetLastError());
+        return false;
+      }
+    }
+
+    if (qos.m_transmit.m_maxBandwidth > 0) {
+      QOS_FLOWRATE_OUTGOING out;
+      out.Bandwidth = qos.m_transmit.m_maxBandwidth;
+      out.ShapingBehavior = QOSUseNonConformantMarkings;
+      out.Reason = QOSFlowRateNotApplicable;
+      if (!QOSSetFlow(PWinSock::GetInstance().m_hQoS, m_qosFlowId, QOSSetOutgoingRate, sizeof(out), &out, 0, NULL)) {
+        PTRACE(2, "Could not set QoS rates, error=" << ::GetLastError());
+      }
+    }
+
+#if P_QWAVE_DSCP
+    if (new_tos < 0)
+      return true;
+
+    DWORD dscp = qos.m_dscp;
+    if (QOSSetFlow(PWinSock::GetInstance().m_hQoS, m_qosFlowId, QOSSetOutgoingDSCPValue, sizeof(dscp), &dscp, 0, NULL))
+      return true;
+
+    PTRACE(2, "Could not set DSCP, error=" << ::GetLastError());
+    return false;
+#endif // P_QWAVE_DSCP
+  }
 #endif // P_QWAVE
 
   // Not explicitly set, so make a value up.
   if (new_tos < 0) {
-    static int const DSCP[NumQoSType] = {
+    static int const DSCP[PIPSocket::NumQoSType] = {
       0,     // BackgroundQoS
       0,     // BestEffortQoS
       8<<2,  // ExcellentEffortQoS
@@ -1088,23 +1151,27 @@ bool PIPSocket::SetQoS(const QoS & qos)
     new_tos = DSCP[qos.m_type];
   }
 
-  // On Win7 and later this succeeds, does not actually do anything. Useless!
+  // On Win7 and later IP_TOS succeeds, but does not actually do anything. Useless!
+  if (PProcess::IsOSVersion(6, 1, 0)) {
+    PTRACE(3, "Setting TOS field of IP header does not work in Windows 7 and later.");
+    return false;
+  }
 
   if (!SetOption(IP_TOS, new_tos, IPPROTO_IP)) {
-    PTRACE(2, "Socket\tCould not set TOS field in IP header: " << GetErrorText());
+    PTRACE(2, "Could not set TOS field in IP header: " << GetErrorText());
     return false;
   }
 
   int actual_tos;
   if (!GetOption(IP_TOS, actual_tos, IPPROTO_IP)) {
-    PTRACE(1, "Socket\tCould not get TOS field in IP header: " << GetErrorText());
+    PTRACE(1, "Could not get TOS field in IP header: " << GetErrorText());
     return false;
   }
 
   if (new_tos == actual_tos)
     return true;
 
-  PTRACE(2, "Socket\tSetting TOS field of IP header appeared successful, but was not really set.");
+  PTRACE(2, "Setting TOS field of IP header appeared successful, but was not really set.");
   return false;
 }
 
@@ -1193,7 +1260,7 @@ class Win32RouteTableDetector : public PIPSocket::RouteTableDetector
         memset(&overlap, 0, sizeof(overlap));
         DWORD error = NotifyAddrChange(&hNotify, &overlap);
         if (error != ERROR_IO_PENDING) {
-          PTRACE(1, "PTlib\tCould not get network interface change notification: error=" << error);
+          PTRACE(1, "Could not get network interface change notification: error=" << error);
           hNotify = NULL;
         }
       }
@@ -1235,33 +1302,72 @@ PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
 }
 
 
-PIPSocket::Address PIPSocket::GetRouteInterfaceAddress(PIPSocket::Address remoteAddress)
+PIPSocket::Address PIPSocket::GetRouteInterfaceAddress(const PIPSocket::Address & remoteAddress)
 {
-  DWORD best;
-  if (GetBestInterface(remoteAddress, &best) == NO_ERROR) {
-    PIPInterfaceAddressTable interfaces;
-    for (unsigned j = 0; j < interfaces->dwNumEntries; ++j) {
-      if (interfaces->table[j].dwIndex == best)
-        return interfaces->table[j].dwAddr;
+#if P_HAS_IPV6
+  if (remoteAddress.GetVersion() == 6) {
+    sockaddr_in6 addr;
+    memset(&addr, 0, sizeof(addr));
+    addr.sin6_family = AF_INET6;
+    addr.sin6_addr = remoteAddress;
+    DWORD best;
+    if (GetBestInterfaceEx((sockaddr*)&addr, &best) == NO_ERROR) {
+      PIPAdaptersAddressTable interfaces;
+      PIPRouteTableIPv6 routes;
+      for (const IP_ADAPTER_ADDRESSES * adapter = &*interfaces; adapter != NULL; adapter = adapter->Next) {
+        if (adapter->IfIndex != best)
+          continue;
+
+        for (PIP_ADAPTER_UNICAST_ADDRESS unicast = adapter->FirstUnicastAddress; unicast != NULL; unicast = unicast->Next) {
+          if (unicast->Address.lpSockaddr->sa_family == AF_INET6 &&
+              routes.ValidateAddress(adapter->IfIndex, unicast->Address.lpSockaddr))
+            return ((sockaddr_in6 *)unicast->Address.lpSockaddr)->sin6_addr;
+        }
+      }
     }
   }
+  else
+#endif
+  {
+    // For some variants of Windows GetBestInterface will return 127.0.0.1
+    // when we are trying to talk to one of our own interfaces.
+    PIPInterfaceAddressTable interfaces;
+    for (DWORD i = 0; i < interfaces->dwNumEntries; ++i) {
+      if (remoteAddress == interfaces->table[i].dwAddr)
+        return remoteAddress;
+    }
+
+    DWORD best;
+    if (GetBestInterface(remoteAddress, &best) == NO_ERROR) {
+      for (DWORD i = 0; i < interfaces->dwNumEntries; ++i) {
+        if (interfaces->table[i].dwIndex == best)
+          return interfaces->table[i].dwAddr;
+      }
+    }
+  }
+
   return GetInvalidAddress();
 }
 
 
 PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & table, PBoolean includeDown)
 {
-#if P_HAS_IPV6
-  // Adding IPv6 addresses
-  PIPRouteTableIPv6 routes;
-  PIPAdaptersAddressTable interfaces;
   PIPInterfaceAddressTable byAddress;
 
-  PINDEX count = 0; // address count
-
   if (!table.SetSize(0))
     return false;
 
+  PINDEX count = 0; // address count
+
+#if P_HAS_IPV6 || _WIN32_WINNT_WINXP
+  PIPAdaptersAddressTable interfaces;
+
+#if P_HAS_IPV6
+  PIPRouteTableIPv6 routes;
+#else _WIN32_WINNT_WINXP
+  PIPRouteTable routes;
+#endif
+
   for (const IP_ADAPTER_ADDRESSES * adapter = &*interfaces; adapter != NULL; adapter = adapter->Next) {
     if (!includeDown && (adapter->OperStatus != IfOperStatusUp))
       continue;
@@ -1276,50 +1382,47 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & table, PBoolean includeDo
         macAddr << setw(2) << (unsigned)adapter->PhysicalAddress[b];
 
       if (unicast->Address.lpSockaddr->sa_family == AF_INET) {
-        PIPSocket::Address ip(((sockaddr_in *)unicast->Address.lpSockaddr)->sin_addr);
+        PIPSocket::Address addr(((sockaddr_in *)unicast->Address.lpSockaddr)->sin_addr);
+        if (addr.IsAny() || addr.IsBroadcast())
+          addr = GetInvalidAddress();
 
         // Find out address index in byAddress table for the mask
-        DWORD dwMask = 0L;
+        Address mask = Address::GetBroadcast(addr.GetVersion());
         for (unsigned i = 0; i < byAddress->dwNumEntries; ++i) {
           if (adapter->IfIndex == byAddress->table[i].dwIndex) {
-            dwMask = byAddress->table[i].dwMask;
+            mask = byAddress->table[i].dwMask;
             break; 
           }
         } // find mask for the address
 
-        table.SetAt(count++, new InterfaceEntry(adapter->Description, ip, dwMask, macAddr));
-
+        table.SetAt(count++, new InterfaceEntry(adapter->Description, addr, mask, macAddr));
       } // ipv4
+#if P_HAS_IPV6
       else if (unicast->Address.lpSockaddr->sa_family == AF_INET6) {
         sockaddr_in6 * sock6 = (sockaddr_in6 *)unicast->Address.lpSockaddr;
-        PIPSocket::Address ip(sock6->sin6_addr, sock6->sin6_scope_id);
-        table.SetAt(count++, new InterfaceEntry(adapter->Description, ip, 0L, macAddr));
-      } // ipv6
+        PIPSocket::Address addr(sock6->sin6_addr, sock6->sin6_scope_id);
+        if (addr.IsAny() || addr.IsBroadcast())
+          addr = GetInvalidAddress();
+        table.SetAt(count++, new InterfaceEntry(adapter->Description, addr, 0L, macAddr));
+      }
+#endif
     }
   }
-
 #else
 
-  PIPInterfaceAddressTable byAddress;
-
-  if (!table.SetSize(byAddress->dwNumEntries))
-    return false;
-
-  if (table.IsEmpty())
-    return false;
-
-  PINDEX count = 0;
   for (unsigned i = 0; i < byAddress->dwNumEntries; ++i) {
-    Address addr = byAddress->table[i].dwAddr;
-
     MIB_IFROW info;
     info.dwIndex = byAddress->table[i].dwIndex;
-    if (GetIfEntry(&info) == NO_ERROR && (includeDown || (addr.IsValid() && info.dwAdminStatus))) {
+    if (GetIfEntry(&info) == NO_ERROR && (includeDown || info.dwAdminStatus != 0)) {
       PStringStream macAddr;
       macAddr << ::hex << ::setfill('0');
       for (unsigned b = 0; b < info.dwPhysAddrLen; ++b)
         macAddr << setw(2) << (unsigned)info.bPhysAddr[b];
 
+      Address addr = byAddress->table[i].dwAddr;
+      if (addr.IsAny() || addr.IsBroadcast())
+        addr = GetInvalidAddress();
+
       table.SetAt(count++, new InterfaceEntry(PString((const char *)info.bDescr, info.dwDescrLen),
                                               addr,
                                               byAddress->table[i].dwMask,
@@ -1327,11 +1430,9 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & table, PBoolean includeDo
     }
   }
 
-  table.SetSize(count); // May shrink due to "down" interfaces.
-
 #endif
 
-  return true;
+  return !table.IsEmpty();
 }
 
 
