@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28942 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 08:20:56 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 34242 $
+ * $Author: rjongbloed $
+ * $Date: 2015-10-26 12:01:52 +0000 (Mon, 26 Oct 2015) $
  */
 
 #pragma implementation "channel.h"
@@ -37,6 +37,9 @@
 #include <ptlib.h>
 #include <sys/ioctl.h>
 
+#if defined(P_SOLARIS)
+  #include <sys/filio.h>
+#endif
 
 #include "../common/pchannel.cxx"
 
@@ -105,9 +108,10 @@ PBoolean PChannel::PXSetIOBlock(PXBlockType type, const PTimeInterval & timeout)
 
       case PXReadBlock :
         if (px_readThread != NULL && px_lastBlockType == PXReadBlock)
-          PAssertAlways(psprintf("Attempt to do simultaneous reads from multiple threads:"
-                                 " os_handle=%i, thread ID=" PTHREAD_ID_FMT,
-                                 os_handle, px_readThread->GetThreadId()));
+          PAssertAlways(PSTRSTRM("Attempt to do simultaneous reads from multiple threads:"
+                                 " os_handle=" << os_handle << ","
+                                 " this-thread=" << *blockedThread << ","
+                                 " other-thread=" << *px_readThread));
         // Fall into default case
 
       default :
@@ -143,6 +147,14 @@ PBoolean PChannel::PXSetIOBlock(PXBlockType type, const PTimeInterval & timeout)
   return SetErrorValues(Timeout, ETIMEDOUT, group);
 }
 
+FILE * PChannel::FDOpen(const char * mode)
+{
+  FILE * h = fdopen(os_handle, mode);
+  if (h != NULL)
+    os_handle = -1;
+  return h;
+}
+
 
 PBoolean PChannel::Read(void * buf, PINDEX len)
 {
@@ -151,7 +163,13 @@ PBoolean PChannel::Read(void * buf, PINDEX len)
   if (os_handle < 0)
     return SetErrorValues(NotOpen, EBADF, LastReadError);
 
-  while ((lastReadCount = ::read(os_handle, buf, len)) < 0) {
+  for (;;) {
+    PPROFILE_SYSTEM(
+      int result = ::read(os_handle, buf, len);
+    );
+    if (result >= 0)
+      return ConvertOSError(lastReadCount = result, LastReadError);
+
     switch (errno) {
       case EINTR :
         break;
@@ -165,11 +183,9 @@ PBoolean PChannel::Read(void * buf, PINDEX len)
         // Next case
 
       default :
-        return ConvertOSError(-1);
+        return ConvertOSError(-1, LastReadError);
     }
   }
-
-  return lastReadCount > 0;
 }
 
 
@@ -211,10 +227,6 @@ PBoolean PChannel::Write(const void * buf, PINDEX len)
     len -= result;
   }
 
-#if !defined(P_PTHREADS) && !defined(P_MAC_MPTHREADS)
-  PThread::Yield(); // Starvation prevention
-#endif
-
   // Reset all the errors.
   return ConvertOSError(0, LastWriteError);
 }
@@ -337,21 +349,10 @@ int PChannel::PXClose()
   os_handle = -1;
   IOSTREAM_MUTEX_SIGNAL();
 
-#if !defined(P_PTHREADS) && !defined(BE_THREADS) && !defined(P_MAC_MPTHREADS) && !defined(VX_TASKS)
-  // abort any I/O block using this os_handle
-  PProcess::Current().PXAbortIOBlock(handle);
-
-#ifndef BE_BONELESS
-  DWORD cmd = 0;
-  ::ioctl(handle, FIONBIO, &cmd);
-#endif
-#endif
-
   AbortIO(px_readThread, px_threadMutex);
   AbortIO(px_writeThread, px_threadMutex);
-  AbortIO(px_selectThread[0], px_threadMutex);
-  AbortIO(px_selectThread[1], px_threadMutex);
-  AbortIO(px_selectThread[2], px_threadMutex);
+  for (PINDEX i = 0; i < 3; ++i)
+    AbortIO(px_selectThread[i], px_threadMutex);
 
   int stat;
   do {
@@ -385,15 +386,15 @@ PString PChannel::GetErrorText(Errors normalisedError, int osError /* =0 */)
 }
 
 
-PBoolean PChannel::ConvertOSError(P_INT_PTR err, Errors & lastError, int & osError)
-
+PBoolean PChannel::ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group)
 {
-  osError = (err >= 0) ? 0 : errno;
+  int osError = (libcReturnValue >= 0) ? 0 : os_errno();
+  Errors lastError;
 
   switch (osError) {
     case 0 :
       lastError = NoError;
-      return true;
+      break;
 
     case EMSGSIZE:
       lastError = BufferTooSmall;
@@ -401,6 +402,7 @@ PBoolean PChannel::ConvertOSError(P_INT_PTR err, Errors & lastError, int & osErr
 
     case EBADF:  // will get EBADF if a read/write occurs after closing. This must return Interrupted
     case EINTR:
+    case ECANCELED :
       lastError = Interrupted;
       break;
 
@@ -452,7 +454,8 @@ PBoolean PChannel::ConvertOSError(P_INT_PTR err, Errors & lastError, int & osErr
       lastError = Miscellaneous;
       break;
   }
-  return false;
+
+  return SetErrorValues(lastError, osError, group);
 }
 
 
