@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 34800 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2016-05-01 18:46:57 +0100 (Sun, 01 May 2016) $
  */
 
 #define _OSUTIL_CXX
@@ -69,18 +69,28 @@
 #endif
 #include <ctype.h>
 
+#if P_CURSES==1
+  #include <ncurses.h>
+#endif
+
 #if defined(P_LINUX) || defined(P_GNU_HURD)
 
 #include <mntent.h>
 #include <sys/vfs.h>
 
-#if (__GNUC_MINOR__ < 7 && __GNUC__ < 3)
+#if __GNUC__ < 3 || (__GNUC__ == 3 && __GNUC_MINOR__ < 7)
 #include <localeinfo.h>
 #else
 #define P_USE_LANGINFO
 #endif
 
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS)
+#elif defined(P_ANDROID)
+
+#include <mntent.h>
+#include <sys/vfs.h>
+#define P_USE_STRFTIME
+
+#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_IOS)
 #define P_USE_STRFTIME
 
 #include <sys/param.h>
@@ -140,6 +150,11 @@
 #include <langinfo.h>
 #endif
 
+#if defined(P_MACOSX) || defined(P_IOS)
+  #include <mach/mach_time.h>
+#endif
+
+
 #define  LINE_SIZE_STEP  100
 
 #define  DEFAULT_FILE_MODE  (S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP)
@@ -168,6 +183,68 @@ static char *tzname[2] = { "STD", "DST" };
 
 #define new PNEW
 
+#if defined(__arm__) && !defined(__llvm__) && (__GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 6))
+
+// ARM build doesn't link before GCC v4.6
+// This code mostly, and shamelessly, stolen from 4.6 source
+
+typedef int (*__kernel_cmpxchg64_t)(const uint64_t * oldval,
+                                    const uint64_t * newval,
+                                    volatile void * ptr);
+
+static int dummy_cmpxchg64(const uint64_t *,
+                           const uint64_t * newval,
+                           volatile void * ptr)
+{
+  // Do it without memory barrier
+  *(uint64_t *)ptr = *newval;
+  return 0;
+}
+
+static __kernel_cmpxchg64_t get_kernel_cmpxchg64()
+{
+  if (*(unsigned int *)0xffff0ffc >= 5) // kernel_helper_version
+    return (__kernel_cmpxchg64_t)0xffff0f60;
+
+  static const char err[] = "WARNING: __kernel_cmpxchg64 helper not in kernel, no 64 bit atomic operations.\n";
+  if (write(2, err, sizeof(err)-1) != sizeof(err) - 1)
+    abort();
+  return dummy_cmpxchg64;
+}
+
+extern "C" {
+  uint64_t __sync_lock_test_and_set_8(volatile void * ptr, uint64_t val)
+  {
+    static __kernel_cmpxchg64_t kernel_cmpxchg64 = get_kernel_cmpxchg64();
+
+    int failure;
+    uint64_t oldval;
+
+    do {
+      oldval = *(uint64_t *)ptr;
+      failure = kernel_cmpxchg64(&oldval, &val, ptr);
+    } while (failure != 0);
+
+    return oldval;
+  }
+
+  uint64_t __sync_add_and_fetch_8(volatile void * ptr, uint64_t val)
+  {
+    static __kernel_cmpxchg64_t kernel_cmpxchg64 = get_kernel_cmpxchg64();
+
+    int failure;
+    uint64_t tmp1,tmp2;
+
+    do {
+      tmp1 = *(uint64_t *)ptr;
+      tmp2 = tmp1 + val;
+      failure = kernel_cmpxchg64(&tmp1, &tmp2, ptr);
+    } while (failure != 0);
+
+    return tmp2;
+  }
+};
+#endif
 
 static PMutex waterMarkMutex;
 static int lowWaterMark = INT_MAX;
@@ -175,7 +252,7 @@ static int highWaterMark = 0;
   
 int PX_NewHandle(const char * clsName, int fd)
 {
-  if (fd < 0)
+  if (fd < 0 || !PProcess::IsInitialised())
     return fd;
 
   PWaitAndSignal m(waterMarkMutex);
@@ -186,14 +263,14 @@ int PX_NewHandle(const char * clsName, int fd)
 
     int maxHandles = PProcess::Current().GetMaxHandles();
     if (fd < (maxHandles-maxHandles/20))
-      PTRACE(4, "PTLib\tFile handle high water mark set: " << fd << ' ' << clsName);
+      PTRACE(2, "PTLib", "File handle high water mark set: " << fd << ' ' << clsName);
     else
-      PTRACE(1, "PTLib\tFile handle high water mark within 5% of maximum: " << fd << ' ' << clsName);
+      PTRACE(1, "PTLib", "File handle high water mark within 5% of maximum: " << fd << ' ' << clsName);
   }
 
   if (fd < lowWaterMark) {
     lowWaterMark = fd;
-    PTRACE(4, "PTLib\tFile handle low water mark set: " << fd << ' ' << clsName);
+    PTRACE(4, "PTLib", "File handle low water mark set: " << fd << ' ' << clsName);
   }
 
   return fd;
@@ -274,25 +351,13 @@ static PString CanonicaliseFilename(const PString & filename)
 PInt64 PString::AsInt64(unsigned base) const
 {
   char * dummy;
-#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined(P_IRIX) || defined (P_QNX)
-  return strtoll(theArray, &dummy, base);
-#elif defined(P_VXWORKS) || defined(P_RTEMS)
-  return strtol(theArray, &dummy, base);
-#else
-  return strtoq(theArray, &dummy, base);
-#endif
+  return P_STRTOQ(theArray, &dummy, base);
 }
 
 PUInt64 PString::AsUnsigned64(unsigned base) const
 {
   char * dummy;
-#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined (P_IRIX) || defined (P_QNX)
-  return strtoull(theArray, &dummy, base);
-#elif defined(P_VXWORKS) || defined(P_RTEMS)
-  return strtoul(theArray, &dummy, base);
-#else
-  return strtouq(theArray, &dummy, base);
-#endif
+  return P_STRTOUQ(theArray, &dummy, base);
 }
 
 
@@ -303,15 +368,22 @@ PUInt64 PString::AsUnsigned64(unsigned base) const
 
 PTimeInterval PTimer::Tick()
 {
-#if defined(_POSIX_MONOTONIC_CLOCK) && !defined(P_MACOSX)
+#if defined(_POSIX_TIMERS) && _POSIX_TIMERS > 0
   struct timespec ts;
   clock_gettime(CLOCK_MONOTONIC, &ts);
-  return ts.tv_sec*1000LL + ts.tv_nsec/1000000LL;
+  return PTimeInterval::NanoSeconds(ts.tv_nsec, ts.tv_sec);
+#elif defined(P_MACOSX) || defined(P_IOS)
+  static mach_timebase_info_data_t timebaseInfo;
+  if (timebaseInfo.denom == 0) {
+    mach_timebase_info(&timebaseInfo);
+    timebaseInfo.denom *= 1000000; // Want milliseconds, not nanoseconds
+  }
+  return mach_absolute_time() * timebaseInfo.numer / timebaseInfo.denom;
 #else
   #warning System does not have clock_gettime with CLOCK_MONOTONIC, using gettimeofday
   struct timeval tv;
   gettimeofday(&tv, NULL);
-  return tv.tv_sec*1000LL + tv.tv_usec/1000LL;
+  return PTimeInterval::MicroSeconds(tv.tv_usec, tv.tv_sec);
 #endif // P_VXWORKS
 }
 
@@ -360,13 +432,13 @@ void PDirectory::Construct ()
   PString::AssignContents(CanonicaliseDirectory(*this));
 }
 
-PBoolean PDirectory::Open(int ScanMask)
+bool PDirectory::Open(PFileInfo::FileTypes ScanMask)
 
 {
   if (directory != NULL)
     Close();
 
-  scanMask = ScanMask;
+  m_scanMask = ScanMask;
 
   if ((directory = opendir(theArray)) == NULL)
     return false;
@@ -382,7 +454,7 @@ PBoolean PDirectory::Open(int ScanMask)
 }
 
 
-PBoolean PDirectory::Next()
+bool PDirectory::Next()
 {
   if (directory == NULL)
     return false;
@@ -411,16 +483,13 @@ PBoolean PDirectory::Next()
     /* Ignore this file if we can't get info about it */
     if (PFile::GetInfo(*this+entryBuffer->d_name, *entryInfo) == 0)
       continue;
-    
-    if (scanMask == PFileInfo::AllPermissions)
-      return true;
-  } while ((entryInfo->type & scanMask) == 0);
+  } while ((entryInfo->type & m_scanMask) == 0);
 
   return true;
 }
 
 
-PBoolean PDirectory::IsSubDir() const
+bool PDirectory::IsSubDir() const
 {
   if (entryInfo == NULL)
     return false;
@@ -428,9 +497,9 @@ PBoolean PDirectory::IsSubDir() const
   return entryInfo->type == PFileInfo::SubDirectory;
 }
 
-PBoolean PDirectory::Restart(int newScanMask)
+bool PDirectory::Restart(PFileInfo::FileTypes newScanMask)
 {
-  scanMask = newScanMask;
+  m_scanMask = newScanMask;
   if (directory != NULL)
     rewinddir(directory);
   return true;
@@ -445,7 +514,7 @@ PString PDirectory::GetEntryName() const
 }
 
 
-PBoolean PDirectory::GetInfo(PFileInfo & info) const
+bool PDirectory::GetInfo(PFileInfo & info) const
 {
   if (entryInfo == NULL)
     return false;
@@ -455,7 +524,7 @@ PBoolean PDirectory::GetInfo(PFileInfo & info) const
 }
 
 
-PBoolean PDirectory::Exists(const PString & p)
+bool PDirectory::Exists(const PString & p)
 {
   struct stat sbuf;
   if (stat((const char *)p, &sbuf) != 0)
@@ -465,21 +534,7 @@ PBoolean PDirectory::Exists(const PString & p)
 }
 
 
-PBoolean PDirectory::Create(const PString & p, int perm)
-{
-  PAssert(!p.IsEmpty(), "attempt to create dir with empty name");
-  PINDEX last = p.GetLength()-1;
-  PString str = p;
-  if (p[last] == '/')
-    str = p.Left(last);
-#ifdef P_VXWORKS
-  return mkdir(str) == 0;
-#else    
-  return mkdir(str, perm) == 0;
-#endif
-}
-
-PBoolean PDirectory::Remove(const PString & p)
+bool PDirectory::Remove(const PString & p)
 {
   PAssert(!p.IsEmpty(), "attempt to remove dir with empty name");
   PString str = p.Left(p.GetLength()-1);
@@ -506,9 +561,13 @@ PString PDirectory::GetVolume() const
   if (stat(operator+("."), &status) != -1) {
     dev_t my_dev = status.st_dev;
 
-#if defined(P_LINUX) || defined(P_IRIX) || defined(P_GNU_HURD)
+#if defined(P_LINUX) || defined(P_IRIX) || defined(P_GNU_HURD) || defined(P_ANDROID)
 
+  #if defined(P_ANDROID)
+    FILE * fp = fopen("/etc/mtab", "r");
+  #else
     FILE * fp = setmntent(MOUNTED, "r");
+  #endif
     if (fp != NULL) {
       struct mntent * mnt;
       while ((mnt = getmntent(fp)) != NULL) {
@@ -518,7 +577,11 @@ PString PDirectory::GetVolume() const
         }
       }
     }
+  #if defined(P_ANDROID)
+    fclose(fp);
+  #else
     endmntent(fp);
+  #endif
 
 #elif defined(P_SOLARIS)
 
@@ -534,7 +597,7 @@ PString PDirectory::GetVolume() const
     }
     fclose(fp);
 
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS)
+#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_IOS)
 
     struct statfs * mnt;
     int count = getmntinfo(&mnt, MNT_NOWAIT);
@@ -572,9 +635,9 @@ PString PDirectory::GetVolume() const
   return volume;
 }
 
-PBoolean PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSize) const
+bool PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSize) const
 {
-#if defined(P_LINUX) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(P_GNU_HURD)
+#if defined(P_LINUX) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_IOS) || defined(P_GNU_HURD) || defined(P_ANDROID)
 
   struct statfs fs;
 
@@ -682,22 +745,22 @@ void PFile::SetFilePath(const PString & newName)
   PINDEX p;
 
   if ((p = newName.FindLast('/')) == P_MAX_INDEX) 
-    path = CanonicaliseDirectory("") + newName;
+    m_path = CanonicaliseDirectory("") + newName;
   else
-    path = CanonicaliseDirectory(newName(0,p)) + newName(p+1, P_MAX_INDEX);
+    m_path = CanonicaliseDirectory(newName(0, p)) + newName(p + 1, P_MAX_INDEX);
 }
 
 
-PBoolean PFile::Open(OpenMode mode, OpenOptions opt)
+bool PFile::InternalOpen(OpenMode mode, OpenOptions opt, PFileInfo::Permissions permissions)
 
 {
   Close();
   clear();
 
   if (opt > 0)
-    removeOnClose = (opt & Temporary) != 0;
+    m_removeOnClose = (opt & Temporary) != 0;
 
-  if (path.IsEmpty()) {
+  if (m_path.IsEmpty()) {
     char templateStr[3+6+1];
     strcpy(templateStr, "PWLXXXXXX");
 #ifndef P_VXWORKS
@@ -710,7 +773,7 @@ PBoolean PFile::Open(OpenMode mode, OpenOptions opt)
 #endif // P_RTEMS
     if (!ConvertOSError(os_handle))
       return false;
-    path = templateStr;
+    m_path = templateStr;
   } else {
 #else
     static int number = 0;
@@ -747,8 +810,11 @@ PBoolean PFile::Open(OpenMode mode, OpenOptions opt)
     if (opt & Truncate)
       oflags |= O_TRUNC;
 
-
-    if (!ConvertOSError(os_handle = PX_NewHandle(GetClass(), ::open(path, oflags, DEFAULT_FILE_MODE))))
+    // We really want the permissions we specify
+    mode_t oldMask = umask(0);
+    int h = ::open(m_path, oflags, permissions.AsBits());
+    umask(oldMask);
+    if (!ConvertOSError(os_handle = PX_NewHandle(GetClass(), h)))
       return false;
   }
 
@@ -766,47 +832,7 @@ PBoolean PFile::SetLength(off_t len)
 }
 
 
-PBoolean PFile::Rename(const PFilePath & oldname, const PString & newname, PBoolean force)
-{
-  if (newname.Find('/') != P_MAX_INDEX) {
-    errno = EINVAL;
-    return false;
-  }
-
-  if (rename(oldname, oldname.GetPath() + newname) == 0)
-    return true;
-
-  if (!force || errno == ENOENT || !Exists(newname))
-    return false;
-
-  if (!Remove(newname, true))
-    return false;
-
-  return rename(oldname, oldname.GetPath() + newname) == 0;
-}
-
-
-PBoolean PFile::Move(const PFilePath & oldname, const PFilePath & newname, PBoolean force)
-{
-  PFilePath from = oldname.GetDirectory() + oldname.GetFileName();
-  PFilePath to = newname.GetDirectory() + newname.GetFileName();
-
-  if (rename(from, to) == 0)
-    return true;
-
-  if (errno == EXDEV)
-    return Copy(from, to, force) && Remove(from);
-
-  if (force && errno == EEXIST)
-    if (Remove(to, true))
-      if (rename(from, to) == 0)
-  return true;
-
-  return false;
-}
-
-
-PBoolean PFile::Exists(const PFilePath & name)
+bool PFile::Exists(const PFilePath & name)
 { 
 #ifdef P_VXWORKS
   // access function not defined for VxWorks
@@ -823,7 +849,7 @@ PBoolean PFile::Exists(const PFilePath & name)
 }
 
 
-PBoolean PFile::Access(const PFilePath & name, OpenMode mode)
+bool PFile::Access(const PFilePath & name, OpenMode mode)
 {
 #ifdef P_VXWORKS
   // access function not defined for VxWorks
@@ -855,7 +881,17 @@ PBoolean PFile::Access(const PFilePath & name, OpenMode mode)
 }
 
 
-PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & status)
+bool PFile::Touch(const PFilePath & name, const PTime & accessTime, const PTime & modTime)
+{
+  PTime now;
+  P_timeval acc(accessTime.IsValid() ? accessTime : now);
+  P_timeval mod(modTime.IsValid() ? modTime : now);
+  timeval times[2] = { *acc, *mod };
+  return utimes(name, times) == 0;
+}
+
+
+bool PFile::GetInfo(const PFilePath & name, PFileInfo & status)
 {
   status.type = PFileInfo::UnknownFileType;
 
@@ -900,13 +936,13 @@ PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & status)
   status.modified    = s.st_mtime;
   status.accessed    = s.st_atime;
   status.size        = s.st_size;
-  status.permissions = s.st_mode & PFileInfo::AllPermissions;
+  status.permissions = PFileInfo::Permissions::FromBits(s.st_mode);
 
   return true;
 }
 
 
-PBoolean PFile::SetPermissions(const PFilePath & name, int permissions)
+bool PFile::SetPermissions(const PFilePath & name, PFileInfo::Permissions permissions)
 
 {
   mode_t mode = 0;
@@ -1133,12 +1169,58 @@ PBoolean PConsoleChannel::Close()
 }
 
 
+int PConsoleChannel::ReadChar()
+{
+#if P_CURSES==1
+  int ch = getch();
+  if (ch >= 0) {
+    switch (ch) {
+      case KEY_LEFT :
+        return KeyLeft;
+      case KEY_RIGHT :
+        return KeyRight;
+      case KEY_UP :
+        return KeyUp;
+      case KEY_DOWN :
+        return KeyDown;
+      case KEY_PPAGE :
+        return KeyPageUp;
+      case KEY_NPAGE :
+        return KeyPageDown;
+      case KEY_HOME :
+        return KeyHome;
+      case KEY_END :
+        return KeyEnd;
+      case KEY_BACKSPACE :
+        return KeyBackSpace;
+      case KEY_DC :
+        return KeyDelete;
+      case KEY_EIC :
+        return KeyInsert;
+    }
+
+    if (ch >= KEY_F(0) && ch <= KEY_F(63))
+      return ch - KEY_F0 + KeyF1;
+
+    return ch;
+  }
+#endif
+
+  return PChannel::ReadChar();
+}
+
+
 bool PConsoleChannel::SetLocalEcho(bool localEcho)
 {
-  if (!IsOpen())
-    return ConvertOSError(-2, LastReadError);
+  if (CheckNotOpen())
+    return false;
 
-#ifdef P_VXWORKS
+#if P_CURSES==1
+  if (localEcho)
+    return echo() == OK;
+  else
+    return noecho() == OK;
+#elif defined(P_VXWORKS)
   PAssertAlways("PConsoleChannel::GetName - Not implemented for VxWorks");
   return PString("Not Implemented");
 #else
@@ -1155,6 +1237,33 @@ bool PConsoleChannel::SetLocalEcho(bool localEcho)
 }
 
 
+bool PConsoleChannel::SetLineBuffered(bool lineBuffered)
+{
+  if (CheckNotOpen())
+    return false;
+
+#if P_CURSES==1
+  if (lineBuffered)
+    return nocbreak() == OK;
+  else
+    return cbreak() == OK;
+#elif defined(P_VXWORKS)
+  PAssertAlways("PConsoleChannel::GetName - Not implemented for VxWorks");
+  return PString("Not Implemented");
+#else
+  struct termios ios;
+  if (!ConvertOSError(tcgetattr(os_handle, &ios)))
+    return false;
+
+  if (lineBuffered)
+    ios.c_lflag |= ICANON;
+  else
+    ios.c_lflag &= ~ICANON;
+  return ConvertOSError(tcsetattr(os_handle, TCSANOW, &ios));
+#endif
+}
+
+
 //////////////////////////////////////////////////////
 //
 //  PTime
@@ -1165,13 +1274,11 @@ void PTime::SetCurrentTime()
 #ifdef P_VXWORKS
   struct timespec ts;
   clock_gettime(0,&ts);
-  theTime = ts.tv_sec;
-  microseconds = ts.tv_sec*10000 + ts.tv_nsec/100000L;
+  m_microSecondsSinceEpoch.store(ts.tv_sec*Micro + ts.tv_nsec/1000);
 #else
   struct timeval tv;
   gettimeofday(&tv, NULL);
-  theTime = tv.tv_sec;
-  microseconds = tv.tv_usec;
+  m_microSecondsSinceEpoch.store(tv.tv_sec*Micro + tv.tv_usec);
 #endif // P_VXWORKS
 }
 
@@ -1428,7 +1535,7 @@ int PTime::GetTimeZone(PTime::TimeZoneType type)
     return tz;
   else
     return tz + ::daylight*60;
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(__BEOS__) || defined(P_QNX) || defined(P_GNU_HURD)
+#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_IOS) || defined(__BEOS__) || defined(P_QNX) || defined(P_GNU_HURD) || defined(P_ANDROID)
   time_t t;
   time(&t);
   struct tm ts;
