@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29365 $
+ * $Revision: 32634 $
  * $Author: rjongbloed $
- * $Date: 2013-03-28 05:14:40 -0500 (Thu, 28 Mar 2013) $
+ * $Date: 2014-08-30 06:26:37 +0100 (Sat, 30 Aug 2014) $
  */
 
 #pragma implementation "pipechan.h"
@@ -47,9 +47,13 @@
 #include <termio.h>
 #endif
 
+#if defined(P_SOLARIS)
+  #include <sys/filio.h>
+#endif
+
 #include "../common/pipechan.cxx"
 
-#if defined(P_MACOSX) && !defined(P_IPHONEOS)
+#if defined(P_MACOSX) && !defined(P_IOS)
 #  include <crt_externs.h>
 #  define environ (*_NSGetEnviron())
 #else
@@ -329,8 +333,8 @@ PBoolean PPipeChannel::Close()
 
 PBoolean PPipeChannel::Read(void * buffer, PINDEX len)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF, LastReadError);;
+  if (CheckNotOpen())
+    return false;
 
   if (!PAssert(m_fromChildPipe[0] != -1, "Attempt to read from write-only pipe"))
     return false;
@@ -342,8 +346,8 @@ PBoolean PPipeChannel::Read(void * buffer, PINDEX len)
 
 PBoolean PPipeChannel::Write(const void * buffer, PINDEX len)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF, LastWriteError);;
+  if (CheckNotOpen())
+    return false;
 
   if (!PAssert(m_toChildPipe[1] != -1, "Attempt to write to read-only pipe"))
     return false;
@@ -396,7 +400,7 @@ int PPipeChannel::WaitForTermination(const PTimeInterval & timeout)
   if (m_childPID < 0)
     return m_returnCode;
 
-#if defined(P_PTHREADS) || defined(P_MAC_MPTHREADS)
+#if defined(P_PTHREADS)
   PAssert(timeout == 0 || timeout == PMaxTimeInterval, PUnimplementedFunction);
   int result, status;
   while ((result = waitpid(m_childPID, &status, timeout == 0 ? WNOHANG : 0)) != m_childPID) {
@@ -445,8 +449,8 @@ PBoolean PPipeChannel::CanReadAndWrite()
 
 PBoolean PPipeChannel::ReadStandardError(PString & errors, PBoolean wait)
 {
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF, LastReadError);;
+  if (CheckNotOpen())
+    return false;
 
   if (!PAssert(m_stderrChildPipe[0] != -1, "Attempt to read from write-only pipe"))
     return false;
