@@ -26,35 +26,40 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28484 $
+ * $Revision: 29859 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2013-06-02 13:32:45 +0100 (Sun, 02 Jun 2013) $
  */
 
 #pragma implementation "remconn.h"
 
+#include <ptlib_config.h>
+
+#if HAVE_IOCTL_H
+  #include <ioctl.h>
+#elif HAVE_SYS_IOCTL_H
+  #include <sys/ioctl.h>
+#endif
+
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
+#ifdef HAVE_NET_IF_H
+  #include <net/if.h>
+#endif
+
+#ifdef HAVE_SYS_SOCKIO_H
+  #include <sys/sockio.h>
+#endif
+
+#include "uerror.h"
+
 #include <ptlib.h>
 #include <ptlib/pipechan.h>
 #include <ptlib/remconn.h>
 #include <ptlib/pprocess.h>
 
-#include <stdio.h>
-#include <sys/ioctl.h>
-#include <sys/socket.h>
-#ifdef P_VXWORKS
-#include <socklib.h>
-#endif // P_VXWORKS
-
-#ifdef P_SOLARIS
-#include <signal.h>
-#endif
-
-#ifdef P_BEOS
-#include <signal.h>
-#endif
-
-#include "uerror.h"
-
 static const PString RasStr      = "ras";
 static const PString NumberStr   = "Number";
 static const PCaselessString UsernameStr = "$USERID";
