@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29750 $
+ * $Revision: 34801 $
  * $Author: rjongbloed $
- * $Date: 2013-05-20 01:17:04 -0500 (Mon, 20 May 2013) $
+ * $Date: 2016-05-02 09:05:54 +0100 (Mon, 02 May 2016) $
  */
 
 #pragma implementation "sockets.h"
@@ -40,14 +40,69 @@
 #pragma implementation "ethsock.h"
 #pragma implementation "qos.h"
 
+#include <ptlib_config.h>
+
+#if HAVE_IOCTL_H
+  #include <ioctl.h>
+#elif HAVE_SYS_IOCTL_H
+  #include <sys/ioctl.h>
+#endif
+
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
+#ifdef HAVE_NET_IF_H
+  #include <net/if.h>
+#endif
+
+#ifdef HAVE_IFADDRS_H
+  #include <ifaddrs.h>
+#endif
+
+#ifdef HAVE_SYS_SYSCTL_H
+  #include <sys/sysctl.h>
+#endif
+
+#ifdef P_RTEMS
+  #include <bsp.h>
+#endif
+
+#ifdef HAVE_LINUX_ERRQUEUE_H
+  #include <asm/types.h>
+  #include <linux/errqueue.h>
+#endif
+
+
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_SOLARIS) || defined(P_MACOSX) || defined(P_IOS) || defined(P_IRIX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
+  #define ifr_netmask ifr_addr
+#endif
+
+#if defined(P_SOLARIS)
+  #include <sys/filio.h>
+  #include <sys/sockio.h>
+#endif
+
+#ifdef HAVE_NET_IF_DL_H
+  #include <net/if_dl.h>
+#endif
+
+#ifdef HAVE_NET_IF_TYPES_H
+  #include <net/if_types.h>
+#endif
+
+#ifdef HAVE_NETINET_IF_ETHER_H
+  #include <netinet/if_ether.h>
+#endif
+
+#if defined(P_HAS_RT_MSGHDR)
+  #include <net/route.h>
+#endif
+
 #include <ptlib.h>
 
-#include <ptbuildopts.h>
 #include <ptlib/sockets.h>
 
-#include <map>
-#include <ptlib/pstring.h>
-
 #if defined(SIOCGENADDR)
 #define SIO_Get_MAC_Address SIOCGENADDR
 #define  ifr_macaddr         ifr_ifru.ifru_enaddr
@@ -56,56 +111,6 @@
 #define  ifr_macaddr         ifr_hwaddr.sa_data
 #endif
 
-#if defined(P_LINUX) && P_HAS_RECVMSG && P_HAS_RECVMSG_MSG_ERRQUEUE
-  #include "asm/types.h"
-  #include "linux/errqueue.h"
-#endif
-
-#if defined(P_LINUX)
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <ifaddrs.h>
-#endif
-
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_SOLARIS) || defined(P_MACOSX) || defined(P_MACOS) || defined(P_IRIX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
-#define ifr_netmask ifr_addr
-
-#include <net/if_dl.h>
-
-#if !defined(P_IPHONEOS)
-#include <net/if_types.h>
-#include <net/route.h>
-#endif
-
-#include <netinet/in.h>
-#if !defined(P_QNX)  && !defined(P_IPHONEOS)
-#include <netinet/if_ether.h>
-#endif
-
-#if defined(P_FREEBSD) || defined(P_NETBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_SOLARIS)
-#include <ifaddrs.h>
-#endif
-
-#define ROUNDUP(a) \
-        ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
-
-#endif
-
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(P_QNX)
-#include <sys/sysctl.h>
-#endif
-
-#ifdef P_RTEMS
-#include <bsp.h>
-#endif
-
-#ifdef P_BEOS
-#include <posix/sys/ioctl.h> // for FIONBIO
-#include <be/bone/net/if.h> // for ifconf
-#include <be/bone/sys/sockio.h> // for SIOCGI*
-#endif
-
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
 // Define _SIZEOF_IFREQ for platforms (eg OpenBSD) which do not have it.
 #ifndef _SIZEOF_ADDR_IFREQ
 #define _SIZEOF_ADDR_IFREQ(ifr) \
@@ -113,7 +118,7 @@
   (sizeof(struct ifreq) - sizeof(struct sockaddr) + \
   (ifr).ifr_addr.sa_len) : sizeof(struct ifreq))
 #endif
-#endif
+
 
 int PX_NewHandle(const char *, int);
 
@@ -124,21 +129,10 @@ int PX_NewHandle(const char *, int);
 extern "C" void inet_ntoa_b(struct in_addr inetAddress, char *pString);
 #endif // P_VXWORKS
 
-//////////////////////////////////////////////////////////////////////////////
-// P_fd_set
 
-void P_fd_set::Construct()
-{
-  max_fd = PProcess::Current().GetMaxHandles();
-  set = (fd_set *)malloc((max_fd+7)>>3);
-}
+#define ROUNDUP(a) ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
 
-
-void P_fd_set::Zero()
-{
-  if (PAssertNULL(set) != NULL)
-    memset(set, 0, (max_fd+7)>>3);
-}
+#define PTraceModule() "Socket"
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -197,26 +191,31 @@ int PSocket::os_socket(int af, int type, int protocol)
 
 PBoolean PSocket::os_connect(struct sockaddr * addr, socklen_t size)
 {
-  int val;
+  int result;
   do {
-    val = ::connect(os_handle, addr, size);
-  } while (val != 0 && errno == EINTR);
-  if (val == 0 || errno != EINPROGRESS)
-    return ConvertOSError(val);
+    do {
+      result = ::connect(os_handle, addr, size);
+    } while (result != 0 && errno == EINTR);
 
-  if (!PXSetIOBlock(PXConnectBlock, readTimeout))
-    return false;
+    if (result == 0 || errno != EINPROGRESS)
+      return ConvertOSError(result);
 
-  // A successful select() call does not necessarily mean the socket connected OK.
-  int optval = -1;
-  socklen_t optlen = sizeof(optval);
-  getsockopt(os_handle, SOL_SOCKET, SO_ERROR, (char *)&optval, &optlen);
-  if (optval != 0) {
-    errno = optval;
-    return ConvertOSError(-1);
-  }
+    if (!PXSetIOBlock(PXConnectBlock, readTimeout))
+      return false;
 
-  return true;
+    // A successful select() call does not necessarily mean the socket connected OK.
+    result = -1;
+    socklen_t optlen = sizeof(result);
+    if (!ConvertOSError(getsockopt(os_handle, SOL_SOCKET, SO_ERROR, (char *)&result, &optlen))) {
+      PTRACE(2, "getsockopt SO_ERROR failure: errno=" << GetErrorNumber() << ' ' << GetErrorText());
+      return false;
+    }
+  } while (result == EINTR);
+
+  if (result == 0)
+    return true;
+  errno = result;
+  return ConvertOSError(-1);
 }
 
 
@@ -230,7 +229,7 @@ PBoolean PSocket::os_accept(PSocket & listener, struct sockaddr * addr, socklen_
 
 #if defined(E_PROTO)
       case EPROTO :
-        PTRACE(3, "PTLib\tAccept on " << listener << " failed with EPROTO - retrying");
+        PTRACE(3, "Accept on " << listener << " failed with EPROTO - retrying");
         break;
 #endif
 
@@ -251,83 +250,111 @@ PBoolean PSocket::os_accept(PSocket & listener, struct sockaddr * addr, socklen_
 }
 
 
-#if !defined(P_PTHREADS) && !defined(P_MAC_MPTHREADS) && !defined(P_BEOS)
-
-PChannel::Errors PSocket::Select(SelectList & read,
-                                 SelectList & write,
-                                 SelectList & except,
-      const PTimeInterval & timeout)
-{
-  PINDEX i, j;
-  PINDEX nextfd = 0;
-  int maxfds = 0;
-  Errors lastError = NoError;
-  PThread * unblockThread = PThread::Current();
-  
-  P_fd_set fds[3];
-  SelectList * list[3] = { &read, &write, &except };
-
-  for (i = 0; i < 3; i++) {
-    for (j = 0; j < list[i]->GetSize(); j++) {
-      PSocket & socket = (*list[i])[j];
-      if (!socket.IsOpen())
-        lastError = NotOpen;
-      else {
-        int h = socket.GetHandle();
-        fds[i] += h;
-        if (h > maxfds)
-          maxfds = h;
-      }
-      socket.px_selectMutex[i].Wait();
-      socket.px_selectThread[i] = unblockThread;
-    }
-  }
-
-  if (lastError == NoError) {
-    P_timeval tval = timeout;
-    int result = ::select(maxfds+1, 
-                          (fd_set *)fds[0], 
-                          (fd_set *)fds[1], 
-                          (fd_set *)fds[2], 
-                          tval);
-
-    int osError;
-    (void)ConvertOSError(result, lastError, osError);
-  }
-
-  for (i = 0; i < 3; i++) {
-    for (j = 0; j < list[i]->GetSize(); j++) {
-      PSocket & socket = (*list[i])[j];
-      socket.px_selectThread[i] = NULL;
-      socket.px_selectMutex[i].Signal();
-      if (lastError == NoError) {
-        int h = socket.GetHandle();
-        if (h < 0)
-          lastError = Interrupted;
-        else if (!fds[i].IsPresent(h))
-          list[i]->RemoveAt(j--);
-      }
-    }
-  }
-
-  return lastError;
-}
-                     
-#else
-
 PChannel::Errors PSocket::Select(SelectList & read,
                                  SelectList & write,
                                  SelectList & except,
                                  const PTimeInterval & timeout)
 {
   PINDEX i, j;
-  int maxfds = 0;
+  int result = -1;
   Errors lastError = NoError;
+#if P_PTHREADS
   PThread * unblockThread = PThread::Current();
   int unblockPipe = unblockThread->unblockPipe[0];
-
-  P_fd_set fds[3];
+#endif
   SelectList * list[3] = { &read, &write, &except };
+  PSocket * firstSocket = NULL;
+
+#if P_HAS_POLL
+
+  size_t pfdSize = sizeof(::pollfd)*(read.GetSize() + write.GetSize() + except.GetSize() + 1);
+  ::pollfd * pfd = (::pollfd *)alloca(pfdSize);
+  memset(pfd, 0, pfdSize);
+
+#if P_PTHREADS
+  PINDEX count = 1;
+  pfd[0].fd = unblockPipe;
+  pfd[0].events = POLLIN;
+#else
+  PIDNEX count = 0;
+#endif
+
+  for (i = 0; i < 3; i++) {
+    for (SelectList::iterator it = list[i]->begin(); it != list[i]->end(); it++) {
+      if (firstSocket == NULL)
+        firstSocket = &*it;
+
+      int h = it->GetHandle();
+      for (j = 0; j < count; ++j) {
+        if (pfd[j].fd == h)
+          break;
+      }
+      if (j == count)
+        pfd[count++].fd = h;
+
+      static int const EventBit[3] = { POLLIN | POLLNVAL, POLLOUT | POLLNVAL, POLLERR | POLLNVAL };
+      pfd[j].events |= EventBit[i];
+
+#if P_PTHREADS
+      it->px_selectMutex[i].Wait();
+      it->px_selectThread[i] = unblockThread;
+#endif
+    }
+  }
+
+  do {
+    PPROFILE_SYSTEM(
+      result = ::poll(pfd, count, timeout.GetInterval());
+    );
+  } while (result < 0 && errno == EINTR);
+
+  if (firstSocket->ConvertOSError(result)) {
+#if P_PTHREADS
+    if (pfd[0].revents != 0) {
+      PTRACE2(6, NULL, "Select unblocked fd=" << unblockPipe);
+      char ch;
+      PPROFILE_SYSTEM(
+        firstSocket->ConvertOSError(::read(unblockPipe, &ch, 1));
+      );
+      lastError = Interrupted;
+    }
+#endif
+  }
+  else
+    lastError = firstSocket->GetErrorCode();
+
+  for (i = 0; i < 3; i++) {
+    SelectList::iterator it = list[i]->begin();
+    while (it != list[i]->end()) {
+      PSocket & socket = *it;
+      socket.px_selectThread[i] = NULL;
+      socket.px_selectMutex[i].Signal();
+      if (lastError != NoError)
+        ++it;
+      else {
+        int h = socket.GetHandle();
+        if (h < 0) {
+          lastError = Interrupted;
+          ++it;
+        }
+        else {
+          for (j = 0; j < count; ++j) {
+            if (pfd[j].fd == h)
+              break;
+          }
+          if (j < count && pfd[j].revents != 0)
+            ++it;
+          else
+            list[i]->erase(it++);
+        }
+      }
+    }
+  }
+
+#else // P_HAS_POLL
+
+  int maxfds = 0;
+  P_fd_set fds[3];
 
   for (i = 0; i < 3; i++) {
     for (j = 0; j < list[i]->GetSize(); j++) {
@@ -339,32 +366,44 @@ PChannel::Errors PSocket::Select(SelectList & read,
         fds[i] += h;
         if (h > maxfds)
           maxfds = h;
+        if (firstSocket == NULL)
+          firstSocket = &socket;
       }
+#if P_PTHREADS
       socket.px_selectMutex[i].Wait();
       socket.px_selectThread[i] = unblockThread;
+#endif
     }
   }
 
-  int result = -1;
   if (lastError == NoError) {
+#if P_PTHREADS
     fds[0] += unblockPipe;
     if (unblockPipe > maxfds)
       maxfds = unblockPipe;
+#endif
 
     P_timeval tval = timeout;
     do {
-      result = ::select(maxfds+1, (fd_set *)fds[0], (fd_set *)fds[1], (fd_set *)fds[2], tval);
+      PPROFILE_SYSTEM(
+        result = ::select(maxfds+1, (fd_set *)fds[0], (fd_set *)fds[1], (fd_set *)fds[2], tval);
+      );
     } while (result < 0 && errno == EINTR);
 
-    int osError;
-    if (PChannel::ConvertOSError(result, lastError, osError)) {
+    if (firstSocket->ConvertOSError(result)) {
+#if P_PTHREADS
       if (fds[0].IsPresent(unblockPipe)) {
-        PTRACE2(6, NULL, "PTLib\tSelect unblocked fd=" << unblockPipe);
-        BYTE ch;
-        if (PChannel::ConvertOSError(::read(unblockPipe, &ch, 1), lastError, osError))
-          lastError = Interrupted;
+        PTRACE2(6, NULL, "Select unblocked fd=" << unblockPipe);
+        char ch;
+        PPROFILE_SYSTEM(
+          firstSocket->ConvertOSError(::read(unblockPipe, &ch, 1));
+        );
+        lastError = Interrupted;
       }
+#endif
     }
+    else
+      lastError = firstSocket->GetErrorCode();
   }
 
   for (i = 0; i < 3; i++) {
@@ -389,39 +428,45 @@ PChannel::Errors PSocket::Select(SelectList & read,
     }
   }
 
+#endif // P_HAS_POLL
+
   return lastError;
 }
 
-#endif
-
 
 #if P_HAS_RECVMSG
 
 #if P_HAS_RECVMSG_MSG_ERRQUEUE
   #include "linux/errqueue.h"
-  PBoolean PSocket::ConvertOSError(P_INT_PTR libcReturnValue, ErrorGroup group)
+  int PSocket::os_errno() const
   {
-    if (libcReturnValue < 0) {
-      msghdr errorData;
-      memset(&errorData, 0, sizeof(errorData));
+    int err = errno;
 
-      char control_data[50];
-      errorData.msg_control    = control_data;
-      errorData.msg_controllen = sizeof(control_data);
+    msghdr errorData;
+    memset(&errorData, 0, sizeof(errorData));
 
-      if (::recvmsg(os_handle, &errorData, MSG_ERRQUEUE) >= 0) {
-        for (struct cmsghdr *cmsg = CMSG_FIRSTHDR(&errorData); cmsg != NULL; cmsg = CMSG_NXTHDR(&errorData, cmsg)) {
-          if (cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_RECVERR) {
-            struct sock_extended_err * sock_error = (struct sock_extended_err *)CMSG_DATA(cmsg);
-            PTRACE_IF(4, sock_error->ee_origin == SO_EE_ORIGIN_ICMP,
-                      "PTLib\tICMP error from " << PIPSocketAddressAndPort(SO_EE_OFFENDER(sock_error), sizeof(sockaddr)));
-            errno = sock_error->ee_errno;
-          }
+    char control_data[50];
+    errorData.msg_control    = control_data;
+    errorData.msg_controllen = sizeof(control_data);
+
+    if (::recvmsg(os_handle, &errorData, MSG_ERRQUEUE) >= 0) {
+      for (struct cmsghdr *cmsg = CMSG_FIRSTHDR(&errorData); cmsg != NULL; cmsg = CMSG_NXTHDR(&errorData, cmsg)) {
+        if (cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_RECVERR) {
+          struct sock_extended_err * sock_error = (struct sock_extended_err *)CMSG_DATA(cmsg);
+          PTRACE_IF(4, sock_error->ee_origin == SO_EE_ORIGIN_ICMP,
+                    "ICMP error from " << PIPSocketAddressAndPort(SO_EE_OFFENDER(sock_error), sizeof(sockaddr)));
+          return errno = sock_error->ee_errno;
         }
       }
     }
 
-    return PChannel::ConvertOSError(libcReturnValue, group);
+    // Could not get any better error information
+    return errno = err;
+  }
+#else
+  int PSocket::os_errno() const
+  {
+    return errno;
   }
 #endif
 
@@ -440,13 +485,15 @@ bool PSocket::os_vread(Slice * slices, size_t sliceCount, int flags, struct sock
     readData.msg_iovlen     = sliceCount;
 
     // read a packet 
-    int result = ::recvmsg(os_handle, &readData, flags);
+    PPROFILE_SYSTEM(
+      int result = ::recvmsg(os_handle, &readData, flags);
+    );
     if (ConvertOSError(result, LastReadError)) {
       lastReadCount = result;
       if ((readData.msg_flags&MSG_TRUNC) == 0)
         return lastReadCount > 0;
 
-      PTRACE(4, "PTlib\tTruncated packet read, returning EMSGSIZE");
+      PTRACE(4, "Truncated packet read, returning EMSGSIZE");
       SetErrorValues(BufferTooSmall, EMSGSIZE, LastReadError);
       return false;
     }
@@ -460,8 +507,8 @@ bool PSocket::os_vwrite(const Slice * slices, size_t sliceCount, int flags, stru
 {
   lastWriteCount = 0;
 
-  if (!IsOpen())
-    return SetErrorValues(NotOpen, EBADF, LastWriteError);
+  if (CheckNotOpen())
+    return false;
 
   do {
     msghdr writeData;
@@ -474,7 +521,9 @@ bool PSocket::os_vwrite(const Slice * slices, size_t sliceCount, int flags, stru
     writeData.msg_iovlen  = sliceCount;
 
     // write the packet 
-    int result = ::sendmsg(os_handle, &writeData, flags);
+    PPROFILE_SYSTEM(
+      int result = ::sendmsg(os_handle, &writeData, flags);
+    );
     if (ConvertOSError(result, LastWriteError)) {
       lastWriteCount = result;
       return true;
@@ -549,115 +598,6 @@ PIPSocket::Address::Address(BYTE b1, BYTE b2, BYTE b3, BYTE b4)
   p[3] = b4;
 }
 
-PBoolean PIPSocket::IsLocalHost(const PString & hostname)
-{
-  if (hostname.IsEmpty())
-    return true;
-
-  if (hostname *= "localhost")
-    return true;
-
-  // lookup the host address using inet_addr, assuming it is a "." address
-  Address addr(hostname);
-  if (addr.IsLoopback())  // Is 127.0.0.1
-    return true;
-  if (!addr.IsValid())
-    return false;
-
-  if (!GetHostAddress(hostname, addr))
-    return false;
-
-#if P_HAS_IPV6
-  {
-    FILE * file;
-    int dummy;
-    int addr6[16];
-    char ifaceName[255];
-    PBoolean found = false;
-    if ((file = fopen("/proc/net/if_inet6", "r")) != NULL) {
-      while (!found && (fscanf(file, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x %x %x %x %x %255s\n",
-              &addr6[0],  &addr6[1],  &addr6[2],  &addr6[3], 
-              &addr6[4],  &addr6[5],  &addr6[6],  &addr6[7], 
-              &addr6[8],  &addr6[9],  &addr6[10], &addr6[11], 
-              &addr6[12], &addr6[13], &addr6[14], &addr6[15], 
-             &dummy, &dummy, &dummy, &dummy, ifaceName) != EOF)) {
-        Address ip6addr(
-          psprintf("%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x",
-              addr6[0],  addr6[1],  addr6[2],  addr6[3], 
-              addr6[4],  addr6[5],  addr6[6],  addr6[7], 
-              addr6[8],  addr6[9],  addr6[10], addr6[11], 
-              addr6[12], addr6[13], addr6[14], addr6[15]
-          )
-        );
-        found = (ip6addr *= addr);
-      }
-      fclose(file);
-    }
-    if (found)
-      return true;
-  }
-#endif
-
-  // check IPV4 addresses
-  PUDPSocket sock;
-  
-  PBYTEArray buffer;
-  struct ifconf ifConf;
-
-#if defined(P_NETBSD)
-  struct ifaddrs *ifap, *ifa;
-
-  PAssert(getifaddrs(&ifap) == 0, "getifaddrs failed");
-  for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
-#else
-#ifdef SIOCGIFNUM
-  int ifNum;
-  PAssert(::ioctl(sock.GetHandle(), SIOCGIFNUM, &ifNum) >= 0, "could not do ioctl for ifNum");
-  ifConf.ifc_len = ifNum * sizeof(ifreq);
-#else
-  ifConf.ifc_len = 100 * sizeof(ifreq); // That's a LOT of interfaces!
-#endif
-
-  ifConf.ifc_req = (struct ifreq *)buffer.GetPointer(ifConf.ifc_len);
-  
-  if (ioctl(sock.GetHandle(), SIOCGIFCONF, &ifConf) >= 0) {
-    void * ifEndList = (char *)ifConf.ifc_req + ifConf.ifc_len;
-    ifreq * ifName = ifConf.ifc_req;
-
-    while (ifName < ifEndList) {
-#endif
-      struct ifreq ifReq;
-#if !defined(P_NETBSD)
-      memcpy(&ifReq, ifName, sizeof(ifreq));
-#else
-      memset(&ifReq, 0, sizeof(ifReq));
-      strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
-#endif
-      
-      if (ioctl(sock.GetHandle(), SIOCGIFFLAGS, &ifReq) >= 0) {
-        int flags = ifReq.ifr_flags;
-        if ((flags & IFF_UP) && ioctl(sock.GetHandle(), SIOCGIFADDR, &ifReq) >= 0) {
-          sockaddr_in * sin = (sockaddr_in *)&ifReq.ifr_addr;
-          PIPSocket::Address address = sin->sin_addr;
-          if (addr *= address)
-            return true;
-        }
-      }
-      
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
-      // move the ifName pointer along to the next ifreq entry
-      ifName = (struct ifreq *)((char *)ifName + _SIZEOF_ADDR_IFREQ(*ifName));
-#elif !defined(P_NETBSD)
-      ifName++;
-#endif
-    }
-#if !defined(P_NETBSD)
-  }
-#endif
-  
-  return false;
-}
-
 
 ////////////////////////////////////////////////////////////////
 //
@@ -680,7 +620,9 @@ PBoolean PTCPSocket::Read(void * buf, PINDEX maxLen)
     OnOutOfBand(buffer, ooblen);
 
   // attempt to read non-out of band data
-  int r = ::recv(os_handle, (char *)buf, maxLen, 0);
+  PPROFILE_SYSTEM(
+    int r = ::recv(os_handle, (char *)buf, maxLen, 0);
+  );
   if (!ConvertOSError(r, LastReadError))
     return false;
 
@@ -697,8 +639,9 @@ PBoolean PSocket::Read(void * buf, PINDEX len)
   if (!PXSetIOBlock(PXReadBlock, readTimeout)) 
     return false;
 
-  int lastReadCount = ::recv(os_handle, (char *)buf, len, 0);
-    return lastReadCount > 0;
+  PPROFILE_SYSTEM(
+    int lastReadCount = ::recv(os_handle, (char *)buf, len, 0);
+  );
   if (ConvertOSError(lastReadCount))
     return lastReadCount > 0;
 
@@ -713,9 +656,12 @@ bool PSocket::Write(const void * buf, PINDEX len)
 
 PBoolean PSocket::Read(Slice * slices, size_t sliceCount)
 {
-  flush();
   lastReadCount = 0;
 
+  if (CheckNotOpen())
+    return false;
+
+  flush();
   if (sliceCount == 0)
     return SetErrorValues(BadParameter, EINVAL, LastReadError);
 
@@ -726,6 +672,11 @@ PBoolean PSocket::Read(Slice * slices, size_t sliceCount)
 
 PBoolean PSocket::Write(const Slice * slices, size_t sliceCount)
 {
+  lastWriteCount = 0;
+
+  if (CheckNotOpen())
+    return false;
+
   flush();
   return os_vwrite(slices, sliceCount, 0, NULL, 0) && lastWriteCount >= 0;
 }
@@ -754,7 +705,7 @@ bool PIPSocket::SetQoS(const QoS & qos)
   if (SetOption(IP_TOS, qos.m_dscp < 0 || qos.m_dscp > 63 ? DSCP[qos.m_type] : qos.m_dscp, IPPROTO_IP))
     return true;
 
-  PTRACE(1, "Socket\tCould not set TOS field in IP header: " << GetErrorText());
+  PTRACE(1, "Could not set TOS field in IP header: " << GetErrorText());
   return false;
 }
 
@@ -776,13 +727,13 @@ PIPSocket::Address NetmaskV6WithPrefix(unsigned prefixbits, unsigned masklen = 0
     memset(&fullmask, 0, sizeof(fullmask));
     memcpy(&fullmask, mask, std::min((size_t)masklen, sizeof(fullmask)));
   }
-  for(unsigned i=128; i >= prefixbits; --i) {
+  for(unsigned i=128; i > prefixbits; --i) {
     ResetBit(i, fullmask);
   }
   return PIPSocket::Address(16, (BYTE*)&fullmask);
 }
 
-#if defined(P_LINUX) || defined (P_AIX)
+#if defined(P_LINUX) || defined(P_ANDROID) || defined (P_AIX)
 
 PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 {
@@ -837,7 +788,7 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
         entry->interfaceName = tokens[9];
         entry->metric = tokens[5].AsUnsigned(16);
 		BYTE net_mask[16];
-		bzero(net_mask, sizeof(net_mask));
+		memset(net_mask, 0, sizeof(net_mask));
 		for(size_t i = 0; i < tokens[1].AsUnsigned(16) / 4; ++i)
 			net_mask[i/2] = (i % 2 == 0) ? 0xf0 : 0xff;
         entry->net_mask = Address(sizeof(net_mask), net_mask);
@@ -850,7 +801,7 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
   return !table.IsEmpty();
 }
 
-#elif (defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_QNX)) && !defined(P_IPHONEOS)
+#elif defined(P_HAS_RT_MSGHDR)
 
 PBoolean process_rtentry(struct rt_msghdr *rtm, char *ptr, PIPSocket::Address & net_addr,
                      PIPSocket::Address & net_mask, PIPSocket::Address & dest_addr, int & metric);
@@ -1355,10 +1306,9 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 #ifdef P_HAS_NETLINK
 
 #include <asm/types.h>
-#include <sys/socket.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
-#include <linux/genetlink.h>
+//#include <linux/genetlink.h>
 
 #include <memory.h>
 #include <errno.h>
@@ -1385,7 +1335,7 @@ class NetLinkRouteTableDetector : public PIPSocket::RouteTableDetector
       if (pipe(m_fdCancel) == -1)
         m_fdCancel[0] = m_fdCancel[1] = -1;
 
-      PTRACE(3, "PTLIB\tOpened NetLink socket");
+      PTRACE(3, "Opened NetLink socket");
     }
 
     ~NetLinkRouteTableDetector()
@@ -1409,17 +1359,13 @@ class NetLinkRouteTableDetector : public PIPSocket::RouteTableDetector
         FD_ZERO(&fds);
         FD_SET(m_fdCancel[0], &fds);
 
-        struct timeval tval;
-        struct timeval * ptval = NULL;
+        P_timeval tv(PMaxTimeInterval);
         if (m_fdLink != -1) {
-          tval.tv_sec  = timeout.GetMilliSeconds() / 1000;
-          tval.tv_usec = (timeout.GetMilliSeconds() % 1000) * 1000;
-          ptval = &tval;
-
+          tv = timeout;
           FD_SET(m_fdLink, &fds);
         }
 
-        int result = select(std::max(m_fdLink, m_fdCancel[0])+1, &fds, NULL, NULL, ptval);
+        int result = select(std::max(m_fdLink, m_fdCancel[0])+1, &fds, NULL, NULL, tv);
         if (result < 0)
           return false;
         if (result == 0)
@@ -1433,7 +1379,9 @@ class NetLinkRouteTableDetector : public PIPSocket::RouteTableDetector
         struct iovec iov = { buf, sizeof buf };
         struct msghdr msg = { (void*)&snl, sizeof snl, &iov, 1, NULL, 0, 0};
 
-        int status = recvmsg(m_fdLink, &msg, 0);
+        PPROFILE_SYSTEM(
+          int status = recvmsg(m_fdLink, &msg, 0);
+        );
         if (status < 0)
           return false;
 
@@ -1446,7 +1394,7 @@ class NetLinkRouteTableDetector : public PIPSocket::RouteTableDetector
           switch (nlmsg->nlmsg_type) {
             case RTM_NEWADDR :
             case RTM_DELADDR :
-              PTRACE(3, "PTLIB\tInterface table change detected via NetLink");
+              PTRACE(3, "Interface table change detected via NetLink");
               return true;
           }
         }
@@ -1465,13 +1413,7 @@ PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
   return new NetLinkRouteTableDetector();
 }
 
-#elif defined(P_IPHONEOS)
-
-#include <netdb.h>
-#include <sys/time.h>
-#include <net/if.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#elif defined(P_IOS)
 
 #include <SystemConfiguration/SystemConfiguration.h>
 #include <SystemConfiguration/SCNetworkReachability.h>
@@ -1506,11 +1448,11 @@ class ReachabilityRouteTableDetector : public PIPSocket::RouteTableDetector
 		struct sockaddr_in6		sin6;
 		SCNetworkReachabilityRef	target	= NULL;
 
-		bzero(&sin, sizeof(sin));
+		memset(&sin, 0, sizeof(sin));
 		sin.sin_len    = sizeof(sin);
 		sin.sin_family = AF_INET;
 
-		bzero(&sin6, sizeof(sin6));
+		memset(&sin6, 0, sizeof(sin6));
 		sin6.sin6_len    = sizeof(sin6);
 		sin6.sin6_family = AF_INET6;
 
@@ -1534,7 +1476,7 @@ class ReachabilityRouteTableDetector : public PIPSocket::RouteTableDetector
 			
 		target = SCNetworkReachabilityCreateWithName(NULL, anchor);
 				
-#if	!TARGET_OS_IPHONE
+#if !defined(P_IOS)
 		if (CFDictionaryGetCount(options) > 0) {
 
 			target = SCNetworkReachabilityCreateWithOptions(NULL, options);
@@ -1553,19 +1495,8 @@ class ReachabilityRouteTableDetector : public PIPSocket::RouteTableDetector
 	  
 	static void callout(SCNetworkReachabilityRef target, SCNetworkReachabilityFlags flags, void *info)
 	{
-		struct tm	tm_now;
-		struct timeval	tv_now;
-
-		(void)gettimeofday(&tv_now, NULL);
-		(void)localtime_r(&tv_now.tv_sec, &tm_now);
-
-		PTRACE(1, psprintf("Reachability changed at: %2d:%02d:%02d.%03d, now it is %sreachable",
-			tm_now.tm_hour,
-			tm_now.tm_min,
-			tm_now.tm_sec,
-			tv_now.tv_usec / 1000,
-			flags & kSCNetworkReachabilityFlagsReachable? "" : "not ")
-			);
+		PTRACE(3, "Reachability changed to "
+                  << ((flags & kSCNetworkReachabilityFlagsReachable) != 0 ? "" : "un") << "reachable");
 
 		ReachabilityRouteTableDetector* d = (ReachabilityRouteTableDetector*) info;	
 		d->Cancel();
@@ -1579,19 +1510,19 @@ class ReachabilityRouteTableDetector : public PIPSocket::RouteTableDetector
 		
 		target_async = _setupReachability(&context);
 		if (target_async == NULL) {
-			PTRACE(1, psprintf("  Could not determine status: %s\n", SCErrorString(SCError())));
+			PTRACE(1, "Could not determine status: " << SCErrorString(SCError()));
 			return;
 		}
 		
 		context.info = (void*) this;
 		
 		if (!SCNetworkReachabilitySetCallback(target_async, ReachabilityRouteTableDetector::callout, &context)) {
-			PTRACE(1, psprintf("SCNetworkReachabilitySetCallback() failed: %s\n", SCErrorString(SCError())));
+			PTRACE(1, "SCNetworkReachabilitySetCallback() failed: " << SCErrorString(SCError()));
 			return;
 		}
 
 		if (!SCNetworkReachabilityScheduleWithRunLoop(target_async, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode)) {
-			PTRACE(1, psprintf("SCNetworkReachabilityScheduleWithRunLoop() failed: %s\n", SCErrorString(SCError()) ) );
+			PTRACE(1, "SCNetworkReachabilityScheduleWithRunLoop() failed: " << SCErrorString(SCError()));
 			return;
 		}
 	}
@@ -1622,7 +1553,7 @@ PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
 	return new ReachabilityRouteTableDetector();
 }
 
-#else // P_HAS_NETLINK, elif defined(P_IPHONEOS)
+#else // P_HAS_NETLINK, elif defined(P_IOS)
 
 class DummyRouteTableDetector : public PIPSocket::RouteTableDetector
 {
@@ -1647,12 +1578,12 @@ PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
   return new DummyRouteTableDetector();
 }
 
-#endif // P_HAS_NETLINK, elif defined(P_IPHONEOS)
+#endif // P_HAS_NETLINK, elif defined(P_IOS)
 
 
 PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDown)
 {
-#if defined(P_LINUX) || defined(P_FREEBSD) || defined (P_NETBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_SOLARIS)
+#if defined(P_LINUX) || defined(P_FREEBSD) || defined (P_NETBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_IOS) || defined(P_SOLARIS)
   // tested on Linux 2.6.x, FreeBSD 8.2, NetBSD 5.1, OpenBSD 5.0, MacOS X 10.5.6 and Solaris 11
   struct ifaddrs *interfaces, *ifa;
 
@@ -1668,21 +1599,28 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
       ifReq.ifr_addr.sa_family = ifa->ifa_addr->sa_family;
       strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
       if (ioctl(ifsock.GetHandle(), SIO_Get_MAC_Address, &ifReq) == 0) {
-        macAddr = PEthSocket::Address((BYTE *)ifReq.ifr_macaddr);
+        PEthSocket::Address eth((BYTE *)ifReq.ifr_macaddr);
+        if (eth != PEthSocket::Address(NULL))
+          macAddr = eth;
       }
 #endif
-      if (ifa->ifa_addr->sa_family == AF_INET)
-        list.Append(PNEW InterfaceEntry(ifa->ifa_name,
-                                        Address(AF_INET, sizeof(sockaddr_in), ifa->ifa_addr),
-                                        Address(AF_INET, sizeof(sockaddr_in), ifa->ifa_netmask),
-                                        macAddr));
+      PIPSocket::Address addr = GetInvalidAddress(), mask = GetInvalidAddress();
+      if (ifa->ifa_addr->sa_family == AF_INET) {
+        addr = Address(AF_INET, sizeof(sockaddr_in), ifa->ifa_addr);
+        mask = Address(AF_INET, sizeof(sockaddr_in), ifa->ifa_netmask);
+      }
 #ifdef P_HAS_IPV6
-      else if (ifa->ifa_addr->sa_family == AF_INET6)
-        list.Append(PNEW InterfaceEntry(ifa->ifa_name,
-                                        Address(AF_INET6, sizeof(sockaddr_in6), ifa->ifa_addr),
-                                        Address(AF_INET6, sizeof(sockaddr_in6), ifa->ifa_netmask),
-                                        macAddr));
+      else if (ifa->ifa_addr->sa_family == AF_INET6) {
+        addr = Address(AF_INET6, sizeof(sockaddr_in6), ifa->ifa_addr);
+        mask = Address(AF_INET6, sizeof(sockaddr_in6), ifa->ifa_netmask);
+      }
 #endif
+      else
+        continue;
+
+      if (addr.IsAny() || addr.IsBroadcast())
+        addr = GetInvalidAddress();
+      list.Append(PNEW InterfaceEntry(ifa->ifa_name, addr, mask, macAddr));
     }
     freeifaddrs(interfaces);
   }
@@ -1781,7 +1719,7 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
         }
       }
 
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_IOS) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
       // move the ifName pointer along to the next ifreq entry
       ifName = (struct ifreq *)((char *)ifName + _SIZEOF_ADDR_IFREQ(*ifName));
 #elif !defined(P_NETBSD)
@@ -1856,7 +1794,9 @@ PIPSocket::Address PIPSocket::GetGatewayAddress(unsigned version)
   RouteTable table;
   if (GetRouteTable(table)) {
     for (PINDEX i = 0; i < table.GetSize(); i++) {
-      if (table[i].GetNetwork().IsAny() && table[i].GetDestination().GetVersion() == version)
+      if (table[i].GetNetwork().IsAny() &&
+          table[i].GetDestination().GetVersion() == version &&
+         !table[i].GetDestination().IsAny())
         return table[i].GetDestination();
     }
   }
@@ -1884,53 +1824,78 @@ PIPSocket::Address PIPSocket::GetGatewayInterfaceAddress(unsigned version)
 }
 
 
-PIPSocket::Address PIPSocket::GetRouteInterfaceAddress(Address remoteAddress)
+static unsigned CountMaskBits(const PIPSocket::Address & mask)
 {
-  InterfaceTable hostInterfaceTable;
-  GetInterfaceTable(hostInterfaceTable);
+  unsigned count = 0;
+  DWORD m = (DWORD) mask;
 
-  RouteTable hostRouteTable;
-  GetRouteTable(hostRouteTable);
+  switch (mask.GetVersion()) {
+    case 4:
+      while (m > 0) {
+        count++;
+        m = m & (m - 1);
+      }
+      break;
 
-  if (hostInterfaceTable.IsEmpty())
-    return GetDefaultIpAny();
+    case 6 :
+      while (count < 128 && (mask[count/8] & (1 << (7 - count%8))) != 0)
+        ++count;
+  }
 
-  for (PINDEX IfaceIdx = 0; IfaceIdx < hostInterfaceTable.GetSize(); IfaceIdx++) {
-    if (remoteAddress == hostInterfaceTable[IfaceIdx].GetAddress()) {
-      PTRACE(5, NULL, NULL, "Route packet for " << remoteAddress
-              << " over interface " << hostInterfaceTable[IfaceIdx].GetName()
-              << "[" << hostInterfaceTable[IfaceIdx].GetAddress() << "]");
-      return hostInterfaceTable[IfaceIdx].GetAddress();
+  return count;
+}
+
+
+PIPSocket::Address PIPSocket::GetRouteInterfaceAddress(const Address & remoteAddress)
+{
+  InterfaceTable interfaceTable;
+  if (!GetInterfaceTable(interfaceTable)) {
+    PTRACE(4, NULL, PTraceModule(), "No interface table available.");
+    return GetInvalidAddress();
+  }
+
+  // Check for remote being local host
+  for (PINDEX i = 0; i < interfaceTable.GetSize(); i++) {
+    Address addr = interfaceTable[i].GetAddress();
+    if (remoteAddress == addr) {
+      PTRACE(5, NULL, PTraceModule(), "Route for " << remoteAddress
+             << " is over interface " << interfaceTable[i].GetName()
+             << "[" << addr << "]");
+      return addr;
     }
   }
 
+  RouteTable routeTable;
+  if (!GetRouteTable(routeTable)) {
+    PTRACE(4, NULL, PTraceModule(), "No route table available.");
+    return GetInvalidAddress();
+  }
+
   RouteEntry * route = NULL;
-  for (PINDEX routeIdx = 0; routeIdx < hostRouteTable.GetSize(); routeIdx++) {
-    RouteEntry & routeEntry = hostRouteTable[routeIdx];
-
-    DWORD network = (DWORD) routeEntry.GetNetwork();
-    DWORD mask = (DWORD) routeEntry.GetNetMask();
-
-    if (((DWORD)remoteAddress & mask) == network) {
+  for (PINDEX i = 0; i < routeTable.GetSize(); i++) {
+    RouteEntry & routeEntry = routeTable[i];
+    if (remoteAddress.IsSubNet(routeEntry.GetNetwork(), routeEntry.GetNetMask())) {
       if (route == NULL)
         route = &routeEntry;
-      else if ((DWORD)routeEntry.GetNetMask() > (DWORD)route->GetNetMask())
+      else if (CountMaskBits(routeEntry.GetNetMask()) > CountMaskBits(route->GetNetMask()))
         route = &routeEntry;
     }
   }
 
   if (route != NULL) {
-    for (PINDEX IfaceIdx = 0; IfaceIdx < hostInterfaceTable.GetSize(); IfaceIdx++) {
-      if (route->GetInterface() == hostInterfaceTable[IfaceIdx].GetName()) {
-        PTRACE(5, NULL, NULL, "Route packet for " << remoteAddress
-                << " over interface " << hostInterfaceTable[IfaceIdx].GetName()
-                << "[" << hostInterfaceTable[IfaceIdx].GetAddress() << "]");
-        return hostInterfaceTable[IfaceIdx].GetAddress();
+    for (PINDEX i = 0; i < interfaceTable.GetSize(); i++) {
+      PString name = interfaceTable[i].GetName();
+      Address addr = interfaceTable[i].GetAddress();
+      if (addr.IsValid() && (name == route->GetInterface()) && (remoteAddress.GetVersion() == addr.GetVersion())) {
+        PTRACE(5, NULL, PTraceModule(), "Route for " << remoteAddress
+               << " is over interface " << name << "[" << addr << "]");
+        return addr;
       }
     }
   }
 
-  return GetDefaultIpAny();
+  PTRACE(5, NULL, PTraceModule(), "Could not find route for " << remoteAddress);
+  return GetInvalidAddress();
 }
 
 
@@ -1983,6 +1948,7 @@ struct hostent * Vx_gethostbyaddr(char *name, struct hostent *hp)
 #endif // P_VXWORKS
 
 
+#undef PTraceModule
 #include "../common/pethsock.cxx"
 
 ///////////////////////////////////////////////////////////////////////////////
