@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29390 $
+ * $Revision: 34747 $
  * $Author: rjongbloed $
- * $Date: 2013-03-29 22:38:40 -0500 (Fri, 29 Mar 2013) $
+ * $Date: 2016-04-02 10:22:45 +0100 (Sat, 02 Apr 2016) $
  */
 
 #include <ptlib.h>
@@ -88,10 +88,10 @@ static const unsigned TraceDownSignal = SIGUSR2;
 //
 PServiceProcess::PServiceProcess(const char * manuf,
                                  const char * name,
-                                         WORD majorVersion,
-                                         WORD minorVersion,
+                                     unsigned majorVersion,
+                                     unsigned minorVersion,
                                    CodeStatus status,
-                                         WORD buildNumber)
+                                     unsigned buildNumber)
   : PProcess(manuf, name, majorVersion, minorVersion, status, buildNumber)
 {
   isTerminating = false;
@@ -113,6 +113,17 @@ PServiceProcess & PServiceProcess::Current()
 }
 
 
+void PServiceProcess::SetLogLevel(PSystemLog::Level level)
+{
+  PSystemLog::GetTarget().SetThresholdLevel(level);
+
+#if PTRACING
+  if ((PTrace::GetOptions()&PTrace::SystemLogStream) != 0)
+    PTrace::SetLevel(level);
+#endif
+}
+
+
 #ifndef P_VXWORKS
 static int KillProcess(int pid, unsigned timeout, int sig)
 {
@@ -180,18 +191,22 @@ static PString ExpandOptionSet(const char * options)
 int PServiceProcess::InitialiseService()
 {
 #ifndef P_VXWORKS
-#if PMEMORY_CHECK
-  PMemoryHeap::SetIgnoreAllocations(true);
-#endif
-  PSetErrorStream(new PSystemLog(PSystemLog::StdError));
+  {
+    PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
+
 #if PTRACING
-  PTrace::SetStream(new PSystemLog(PSystemLog::Debug3));
-  PTrace::SetOptions(PTrace::SystemLogStream);
-  PTrace::SetLevel(GetLogLevel());
+    PTrace::SetStream(new PSystemLog(PSystemLog::Debug3));
+#if _DEBUG
+    PTrace::ClearOptions(PTrace::Timestamp);
+    PTrace::SetOptions(PTrace::FileAndLine | PTrace::ContextIdentifier | PTrace::SystemLogStream);
+#else
+    PTrace::ClearOptions(PTrace::Timestamp | PTrace::FileAndLine);
+    PTrace::SetOptions(PTrace::ContextIdentifier | PTrace::SystemLogStream);
 #endif
-#if PMEMORY_CHECK
-  PMemoryHeap::SetIgnoreAllocations(false);
+    PTrace::SetLevel(GetLogLevel());
 #endif
+  }
+
   m_debugMode = false;
 
   // parse arguments so we can grab what we want
@@ -279,9 +294,12 @@ int PServiceProcess::InitialiseService()
     pid_t pid;
 
     {
-      ifstream pidfile(pidfilename);
+      ifstream pidfile((const char *)pidfilename);
       if (!pidfile.is_open()) {
-        cout << "Could not open pid file: \"" << pidfilename << "\""
+        if (args.HasOption('s'))
+          cout << "Process has not been started." << endl;
+        else
+          cout << "Could not open pid file: \"" << pidfilename << "\""
                 " - " << strerror(errno) << endl;
         return 1;
       }
@@ -340,6 +358,9 @@ int PServiceProcess::InitialiseService()
 
   // set flag for console messages
   if (args.HasOption('c')) {
+#if PTRACING
+    PSetErrorStream(PTrace::GetStream());
+#endif
     PSystemLog::SetTarget(new PSystemLogToStderr());
     m_debugMode = true;
   }
@@ -384,9 +405,9 @@ int PServiceProcess::InitialiseService()
 
   if (args.HasOption('H')) {
     int uid = geteuid();
-    seteuid(getuid()); // Switch back to starting uid for next call
+    (void)seteuid(getuid()); // Switch back to starting uid for next call
     SetMaxHandles(args.GetOptionString('H').AsInteger());
-    seteuid(uid);
+    (void)seteuid(uid);
   }
 
 #ifdef P_LINUX
@@ -397,7 +418,7 @@ int PServiceProcess::InitialiseService()
       cout << "Could not get current core file size : error = " << errno << endl;
     else {
       int uid = geteuid();
-      seteuid(getuid()); // Switch back to starting uid for next call
+      (void)seteuid(getuid()); // Switch back to starting uid for next call
       int v = args.GetOptionString('C').AsInteger();
       rlim.rlim_cur = v;
       if (setrlimit(RLIMIT_CORE, &rlim) != 0) 
@@ -406,23 +427,35 @@ int PServiceProcess::InitialiseService()
         getrlimit(RLIMIT_CORE, &rlim);
         cout << "Core file size set to " << rlim.rlim_cur << "/" << rlim.rlim_max << endl;
       }
-      seteuid(uid);
+      (void)seteuid(uid);
     }
   }
 #endif
 
-  // We are a service, don't want to get blocked on input from stdin during asserts
+  bool daemon = args.HasOption('d');
+
+  // Remove the service arguments
+  args.SetArgs(args.GetParameters());
+
+ // We are a service, don't want to get blocked on input from stdin during asserts
   if (!m_debugMode)
     ::close(STDIN_FILENO);
 
-#if !defined(BE_THREADS) && !defined(P_RTEMS)
-  if (!args.HasOption('d'))
+  // We intercept the core dumping signals so get a message in the log file.
+  signal(SIGSEGV, PXSignalHandler);
+  signal(SIGFPE, PXSignalHandler);
+  signal(SIGBUS, PXSignalHandler);
+
+  if (!daemon)
     return -1;
 
-  // Run as a daemon, ie fork
+  PSetErrorStream(new PSystemLog(PSystemLog::StdError));
 
+#if !defined(BE_THREADS) && !defined(P_RTEMS)
+
+  // Run as a daemon, ie fork
   if (!pidfilename) {
-    ifstream pidfile(pidfilename);
+    ifstream pidfile((const char *)pidfilename);
     if (pidfile.is_open()) {
       pid_t pid;
       pidfile >> pid;
@@ -450,7 +483,7 @@ int PServiceProcess::InitialiseService()
       cout << "Daemon started with pid " << pid << endl;
       if (!pidfilename) {
         // Write out the child pid to magic file in /var/run (at least for linux)
-        ofstream pidfile(pidfilename);
+        ofstream pidfile((const char *)pidfilename);
         if (pidfile.is_open())
           pidfile << pid;
         else
@@ -470,12 +503,6 @@ int PServiceProcess::InitialiseService()
 
   pidFileToRemove = pidfilename;
 
-  // Only if we are running in the background as a daemon, we intercept
-  // the core dumping signals so get a message in the log file.
-  signal(SIGSEGV, PXSignalHandler);
-  signal(SIGFPE, PXSignalHandler);
-  signal(SIGBUS, PXSignalHandler);
-
 #endif // !BE_THREADS && !P_RTEMS
 #endif // !P_VXWORKS
   return -1;
@@ -551,6 +578,8 @@ void PServiceProcess::Terminate()
   _exit(terminationValue);
 }
 
+static atomic<bool> InSignalHandler(false);
+
 void PServiceProcess::PXOnAsyncSignal(int sig)
 {
   const char * sigmsg;
@@ -581,34 +610,23 @@ void PServiceProcess::PXOnAsyncSignal(int sig)
   signal(SIGFPE, SIG_DFL);
   signal(SIGBUS, SIG_DFL);
 
-  static PBoolean inHandler = false;
-  if (inHandler) {
-    raise(SIGQUIT); // Dump core
-    _exit(-1); // Fail safe if raise() didn't dump core and exit
-  }
-
-  inHandler = true;
-
-  {
+  if (!InSignalHandler.exchange(true)) {
     PThreadIdentifier tid = GetCurrentThreadId();
-    ThreadMap::iterator thread = m_activeThreads.find(tid);
+    PUniqueThreadIdentifier uid = PThread::GetCurrentUniqueIdentifier();
 
     PSystemLog log(PSystemLog::Fatal);
-    log << "\nCaught " << sigmsg << ", thread_id=" << tid;
+    log << "Caught " << sigmsg << ","
+           " thread-id=" << PThread::GetIdentifiersAsString(tid, uid) << ","
+           " name=\"" << PThread::GetThreadName(tid) << '"';
 
-    if (thread != m_activeThreads.end()) {
-      PString thread_name = thread->second->GetThreadName();
-      if (thread_name.IsEmpty())
-        log << " obj_ptr=" << thread->second;
-      else
-        log << " name=" << thread_name;
-    }
-
-    log << ", aborting.\n";
+#if PTRACING
+    log << ", stack:";
+    PTrace::WalkStack(log);
+#endif
   }
 
-  raise(SIGQUIT); // Dump core
-  _exit(-1); // Fail safe if raise() didn't dump core and exit
+  abort(); // Dump core
+  _exit(sig); // Fail safe if raise() didn't dump core and exit
 }
 
 
